#!/usr/bin/env python

# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Appointments Server"""

__version__ = '$Revision: 1.24 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import time

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.AppointmentsCommon import *
import glasnost.common.faults as faults
from glasnost.common.tools import sendMail, splitId, getConfig

from glasnost.server.ObjectsServer import ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.things import register
from glasnost.server.tools import *

from glasnost.proxy.GroupsProxy import setContains, getSetContainedIds


applicationName = 'AppointmentsServer'
applicationRole = 'appointments'
applicationTokens = None
dispatcher = None


class AdminAppointments(AdminServerMixin, AdminAppointmentsCommon):
    pass
register(AdminAppointments)


class Appointment(ObjectServerMixin, AppointmentCommon):
    pass
register(Appointment)


class AppointmentsServer(ObjectsServer):
    adminClassName = 'AdminAppointments'
    useAdminWritersSet = 1
    version = __version__

    def isAskedResource(self, appointment, resourceIds):
        people = getSetContainedIds(resourceIds)
        print 'isAskedResource', resourceIds, people
        for personId in people:
            if setContains(appointment.resourcesSet, personId):
                return 1
        return 0

    def getAppointments(self, virtualServerId, clientToken, userToken,
                        beginningDate = None, endingDate = None,
                        resourceIds = None, category = None):
        """
        @param beginningDate: get all appointments after this date
        @type beginningDate: float
        @param endingDate: get all appointments before this date
        @type endingDate: float
        @param resourceIds: get all appointments for the specified user
        @type resourceIds: list of id
        return all appointments for the user between beginningDate
        and endingDate
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            isAdmin = self.isAdmin(virtualServerId, clientToken, userToken)
##            if not isAdmin:
##                raise faults.UserAccessDenied()
            virtualServer = self.getVirtualServer(virtualServerId)
            if not resourceIds or resourceIds == ['']:
                resourceIds = []
                
            result = []
            for objectId, object in virtualServer.objects.items():
                print repr((resourceIds, object.resourcesSet, beginningDate, endingDate))
                if not (isAdmin or not object.readersSet or
                        getProxyForServerRole('authentication').setContainsUser(object.readersSet)
                        or (object.resourcesSet
                            and getProxyForServerRole('authentication').setContainsUser(
                    object.resourcesSet))):
                    print 'getAppointments condition 1'
                    continue
                elif category and object.category and \
                     object.category.lower().find(category.lower()) < 0:
                    print 'getAppointments condition 2 %s %s' % (object.category, category)
                    continue
                elif resourceIds and not self.isAskedResource(object, resourceIds):
                    print 'getAppointments condition 3 %s not in %s' % (resourceIds, object.resourcesSet)
                    continue
                elif object.dateIn(beginningDate, endingDate):
                    print 'getAppointments ajout %s' % object.getLabel()
                    result.append(object)
            return result
        finally:
            self.pullContext()
    getAppointments.useAutoDeserialize = 1


    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            result = ObjectsServer.addObjectXmlRpc(self, virtualServerId,
                                          clientToken, userToken, objectImport)

            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)
            object.id = result

            ## send notification mail.
            ## ...
            subject = _('New appointment')
            body = _("""
A new appointement has been set for you on:
http://%(hostName)s/%(id)s

Subject: %(title)s
Message: %(body)s
Beginning: %(start)s

""") % {
                'hostName': getProxyForServerRole('virtualHosts').getHostName(virtualServerId),
                'id' : '/'.join(splitId(result)[1:]),
                'title': object.title,
                'body': object.body,
                'start': time.strftime('%Y-%m-%d %H:%M:%S',
                                       time.localtime(object.beginningDate))
                }
            
            self.sendNotification(virtualServerId, object, subject, body)
            return result
        finally:
            self.pullContext()
           
    def modifyObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                           objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            result = ObjectsServer.modifyObjectXmlRpc(self, virtualServerId,
                                          clientToken, userToken, objectImport)

            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)

            ## send notification mail.
            ## ...
            subject = '[Glasnost] Modification de rendez-vous'
            body = """
Un rendez-vous a t modifi pour vous sur le site
http://%(hostName)s/%(id)s

Sujet: %(title)s
Message: %(body)s
Debut du rendez-vous: %(start)s

""" % {
                'hostName': getProxyForServerRole('virtualHosts').getHostName(virtualServerId),
                'id' : '/'.join(splitId(object.id)[1:]),
                'title': object.title,
                'body': object.body,
                'start': time.strftime('%Y-%m-%d %H:%M:%S',
                                       time.localtime(object.beginningDate))
                }
            
            self.sendNotification(virtualServerId, object, subject, body) 

            return result
        finally:
            self.pullContext()

    def searchAndSendNotification(self, virtualServerId, clientToken,
                                  userToken, interval = 3600):
        """
        @param interval: interval between to call of this method
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            currentTime = time.time()

            appointments = self.getAppointments(virtualServerId, clientToken,
                                                userToken, currentTime)

            number = 0
            for appointment in appointments:
                end = currentTime + appointment.rememberTime + interval

                print 'before isNotified', appointment.isNotified
                ## if notification is obsolete remove it
                if appointment.isNotified:
                    if appointment.repetition != 'none':
                        if appointment.repetition == 'daily' and \
                               appointment.isNotified + ONE_DAY < currentTime:
                            appointment.isNotified = 0
                        elif appointment.repetition == 'weekly' and \
                               appointment.isNotified + ONE_WEEK < currentTime:
                            appointment.isNotified = 0
                        elif appointment.repetition == 'monthly' and \
                               appointment.isNotified + ONE_MONTH < currentTime:
                            appointment.isNotified = 0
                        elif appointment.repetition == 'yearly' and \
                               appointment.isNotified + ONE_YEAR < currentTime:
                            appointment.isNotified = 0
                print 'after isNotified', appointment.isNotified
                
                if not appointment.isNotified and \
                       appointment.rememberTime and \
                       appointment.dateFor(currentTime, end):
                    print 'send email for %s' % appointment.getLabel()
                    ## indicate that this appointment is already notified.
                    appointment.isNotified = currentTime
                    subject = '[Glasnost] Notification de rendez-vous'
                    body = """
Vous avez un rendez-vous sur le site
http://%(hostName)s/%(id)s

Sujet: %(title)s
Message: %(body)s
Debut du rendez-vous: %(start)s
""" % {
                        'hostName': getProxyForServerRole('virtualHosts').getHostName(virtualServerId),
                        'id' : '/'.join(splitId(appointment.id)[1:]),
                        'title': appointment.title,
                        'body': appointment.body,
                        'start': time.strftime('%Y-%m-%d %H:%M:%S',
                                     time.localtime(appointment.beginningDate))
                        }
                    self.sendNotification(virtualServerId, appointment,
                                          subject, body)
                    number = number + 1
            self.saveVirtualServer(virtualServer)
            return number
        finally:
            self.pullContext()
    searchAndSendNotification.useAutoDeserialize = 1

    def sendNotification(self, virtualServerId, appointment, subject, body):
        mailFrom = self.getAdminEmailAddress(virtualServerId)
        people = getSetContainedIds(appointment.resourcesSet)
        for personId in people:
            person = getProxyForServerRole('people').getObject(personId)
            if person.email:
                mailTo = person.email
                try:
                    sendMail(mailFrom, mailTo, mailSubject = subject,
                             mailMessage = body,
                             mailPerson = person, mailPersonId = personId)
                except faults.SmtpError:
                    pass
           
    def getAdminEmailAddress(self, virtualServerId):
        return getConfig('Mail', 'Admin', default='admin@entrouvert.org')

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getAppointments')
        self.registerXmlRpcMethod('searchAndSendNotification')

appointmentsServer = AppointmentsServer()


if __name__ == "__main__":
    appointmentsServer.launch(applicationName, applicationRole)
