#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost AuthenticationLdap Server"""

__version__ = '$Revision: 1.27 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import crypt
import traceback

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults
import glasnost.common.tools as commonToolsConfig

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *

from glasnost.proxy.GroupsProxy import setContains

try:
    import ldap
except ImportError:
    #import sys
    #sys.stderr.write('Python-LDAP is not installed.  Not launching AuthenticationLdapServer.\n')
    sys.exit(1)

applicationName = 'AuthenticationLdapServer'
applicationRole = 'authenticationldap'
applicationTokens = None
dispatcher = None


class AuthenticationLdapVirtualServer(VirtualServer):
    pass

class AuthenticationLdapServer(Server):
    VirtualServer = AuthenticationLdapVirtualServer
    useDataFile = 0
    version = __version__
    ldap_host = 'localhost'
    ldap_port = 389
    suffix = 'o=codelutin,c=fr'
    ldap_login_field = 'uid'
    ldap_passorwd_field = 'userPassword'
    dn = ldap_login_field+'=%s,' + suffix
    admin_dn = dn % 'admin'
    admin_password = 'xxxxxxxx'
    objectClass = ['top', 'person']

    def __init__(self):
        Server.__init__(self)
        self.ldap_host = commonToolsConfig.getConfig('AuthenticationLdapServer',
                                            'ldap_server', self.ldap_host)
        ldap_port = commonToolsConfig.getConfig('AuthenticationLdapServer',
                                          'ldap_port', self.ldap_port)
        if ldap_port:
            self.ldap_port = int(ldap_port)
            
        self.suffix = commonToolsConfig.getConfig(
                        'AuthenticationLdapServer', 'suffix')
        if not self.suffix:
            sys.exit(1)
        self.ldap_login_field = commonToolsConfig.getConfig(
                        'AuthenticationLdapServer', 'ldap_login_field',
                        default = self.ldap_login_field)
        self.ldap_passorwd_field = commonToolsConfig.getConfig(
                        'AuthenticationLdapServer', 'ldap_passorwd_field',
                        default = self.ldap_passorwd_field)
        self.dn = commonToolsConfig.getConfig(
                        'AuthenticationLdapServer', 'dn', 
                        default = self.ldap_login_field + '=%s,' + self.suffix)
        self.admin_dn = commonToolsConfig.getConfig(
                        'AuthenticationLdapServer', 'admin_dn',
                        default = self.dn % 'admin')
        self.admin_password = commonToolsConfig.getConfig(
                        'AuthenticationLdapServer', 'admin_password')
        objectClass = commonToolsConfig.getConfig(
                        'AuthenticationLdapServer', 'objectClass')
        if objectClass:
            self.objectClass = objectClass.split(',')

#        print self.ldap_host, ldap_port, self.suffix, self.ldap_login_field, self.ldap_passorwd_field, self.dn,  self.admin_dn, self.admin_password, self.objectClass

    def addUser(self, virtualServerId, clientToken, userToken, userName, login,
                password):
        try:
            cryptMethod, password = self.cryptPassword(password)

            con = ldap.open(self.ldap_host, self.ldap_port)
            con.simple_bind_s(self.admin_dn, self.admin_password)
            con.add_s(
                self.dn % login,
                [('objectClass', self.objectClass),
                 ('sn', userName),
                 ('cn', userName),
                 (self.ldap_login_field, login),
                 (self.ldap_passorwd_field, cryptMethod + password)])
            con.unbind_s()
        except:
            traceback.print_exc()
            print "ERROR: can't add user %s in LDAP" % userName
        return 1
    addUser.useAutoDeserialize = 1

    def canEmailPassword(self, virtualServerId, clientToken, userToken,
                         userName):
        return 0
    canEmailPassword.useAutoDeserialize = 1

    def changePassword(self, virtualServerId, clientToken, userToken, userName,
                       oldPassword, newPassword):
        try:
            cryptMethod, oldPassword = self.cryptPassword(oldPassword)
            cryptMethod, newPassword = self.cryptPassword(newPassword)

            con = ldap.open(self.ldap_host, self.ldap_port)
            con.simple_bind_s(self.admin_dn, self.admin_password)
            con.modify_s(self.dn % userName,
                 [(ldap.MOD_REPLACE, 'userPassword', cryptMethod+newPassword)])
            con.unbind_s()
        except:
            traceback.print_exc()
            print "ERROR: can't change password for %s in LDAP" % userName
        return ''
    changePassword.useAutoDeserialize = 1

    def checkAuthentication(self, virtualServerId, clientToken, userToken,
                            userName, userPassword):
        try:
            con = ldap.open(self.ldap_host, self.ldap_port)
            con.simple_bind_s(self.dn % userName, userPassword)
            con.unbind_s()        
            return 1
        except:
            traceback.print_exc()
            print "LOG: Bad Authentication for %s" % (self.dn % userName)
        return 0
    checkAuthentication.useAutoDeserialize = 1

    def cryptPassword(self, password):
        """
        Return crypt method and encrypted password. ('{crypt}', 'mlfezhfmflkj')
        """
        return '{crypt}', crypt.crypt(password, password[:2])

    def deleteUser(self, virtualServerId, clientToken, userToken, userName):
        try:
            con = ldap.open(self.ldap_host, self.ldap_port)
            con.simple_bind_s(self.admin_dn, self.admin_password)
            con.delete_s(self.dn % userName)
            con.unbind_s()
        except:
            traceback.print_exc()
            print "ERROR: can't delete %s in LDAP" % userName
        return ''
    deleteUser.useAutoDeserialize = 1

    def getPassword(self, virtualServerId, clientToken, userToken, login):
        return ''
    getPassword.useAutoDeserialize = 1

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('addUser')
        self.registerXmlRpcMethod('canEmailPassword')
        self.registerXmlRpcMethod('changePassword')
        self.registerXmlRpcMethod('checkAuthentication')
        self.registerXmlRpcMethod('deleteUser')
        self.registerXmlRpcMethod('getPassword')


authenticationLdapServer = AuthenticationLdapServer()


if __name__ == "__main__":
    authenticationLdapServer.launch(applicationName, applicationRole)
