#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Authentication PAM Server"""

__version__ = '$Revision: 1.18 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import whrandom

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *

from glasnost.proxy.GroupsProxy import setContains

try:
    import PAM
except ImportError:
    #import sys
    #sys.stderr.write('Python-PAM is not installed.  Not launching AuthenticationPamServer.\n')
    sys.exit(1)


applicationName = 'AuthenticationPamServer'
applicationRole = 'authenticationpam'
applicationTokens = None
dispatcher = None


def pam_conv(auth, query_list, userData):
    resp = []
    for i in range(len(query_list)):
        query, type = query_list[i]
        if userData and userData.has_key(query):
            val = userData[query]
            resp.append((val, 0))
        else:
            return None
    return resp


class AuthenticationPamVirtualServer(VirtualServer):
    pass


class AuthenticationPamServer(Server):
    VirtualServer = AuthenticationPamVirtualServer
    useDataFile = 0
    version = __version__

    def addUser(self, virtualServerId, clientToken, userToken, userName, login,
                password):
        return 0
    addUser.useAutoDeserialize = 1

    def canEmailPassword(self, virtualServerId, clientToken, userToken,
                         userName):
        return 0
    canEmailPassword.useAutoDeserialize = 1

    def changePassword(self, virtualServerId, clientToken, userToken, userName,
                       oldPassword, newPassword):
        auth = PAM.pam()
        auth.start('passwd', userName, pam_conv)
        auth.setUserData({'login: ':userName,
                          'Username: ':userName,
                          'Password: ':oldPassword,
                          '(current) UNIX password: ':oldPassword,
                          'Enter new UNIX password: ':newPassword,
                          'Retype new UNIX password: ':newPassword})
        try:
##            auth.authenticate()
##            auth.acct_mrgmt()
            auth.chauthtok(PAM.PAM_SILENT); 
        except PAM.error, resp:
            return 0
        except:
            ## internal error then refused
            return 0
        return 1               
    changePassword.useAutoDeserialize = 1

    def checkAuthentication(self, virtualServerId, clientToken, userToken,
                            userName, userPassword):
        auth = PAM.pam()
        auth.start('passwd', userName, pam_conv)
        auth.setUserData({'login: ':userName, 'Password: ':userPassword})
        try:
            auth.authenticate()
            auth.acct_mgmt()
        except PAM.error, resp:
            return 0
        except:
            ## internal error refused
            return 0
        return 1
    checkAuthentication.useAutoDeserialize = 1

    def deleteUser(self, virtualServerId, clientToken, userToken, userName):
        return ''
    deleteUser.useAutoDeserialize = 1

    def getPassword(self, virtualServerId, clientToken, userToken, login):
        return ''
    getPassword.useAutoDeserialize = 1

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('addUser')
        self.registerXmlRpcMethod('canEmailPassword')
        self.registerXmlRpcMethod('changePassword')
        self.registerXmlRpcMethod('checkAuthentication')
        self.registerXmlRpcMethod('deleteUser')
        self.registerXmlRpcMethod('getPassword')


authenticationPamServer = AuthenticationPamServer()


if __name__ == "__main__":
    authenticationPamServer.launch(applicationName, applicationRole)
