#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Authentication Server"""

__version__ = '$Revision: 1.21 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import time
import whrandom

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *

from glasnost.proxy.GroupsProxy import setContains


applicationName = 'AuthenticationServer'
applicationRole = 'authentication'
applicationTokens = None
dispatcher = None

# Don't forget to also change the value in SessionsServer.py.
expirationTime = int(getConfig('Misc', 'ExpirationTime', '120')) * 60
    # Default is 120 minutes.


class AuthenticationVirtualServer(VirtualServer):
    userIds = None
    userTokens = None
    tokenExpirations = None

    def init(self, server, virtualServerId):
        VirtualServer.init(self, server, virtualServerId)
        self.userIds = {}
        self.userTokens = {}
        self.tokenExpirations = {}

    def testExpiration(self, userToken):
        if not self.userIds.has_key(userToken):
            return
        if time.time() > self.tokenExpirations[userToken]:
            print 'userToken %s expired' % userToken
            userId = self.userIds[userToken]
            del self.userIds[userToken]
            del self.userTokens[userId]
            del self.tokenExpirations[userToken]
        else:
            self.tokenExpirations[userToken] = time.time() + expirationTime


class AuthenticationServer(Server):
    VirtualServer = AuthenticationVirtualServer
    useDataFile = 0
    version = __version__

    def canLogin(self, virtualServerId, clientToken):
        dispatcherId = commonTools.extractDispatcherId(virtualServerId)
        return dispatcherId != 'glasnost://system'

    def computeVirtualServerIdFromUserToken(self, userToken):
        return self.computeVirtualServerId(userToken)

    def getUserId(self, clientToken, userToken):
        virtualServerId = self.computeVirtualServerIdFromUserToken(userToken)
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            virtualServer.testExpiration(userToken)
            if not userToken:
                return ''
            if not virtualServer.userIds.has_key(userToken):
                raise faults.UnknownUserToken(userToken)
            userId = virtualServer.userIds[userToken]
            return userId
        finally:
            self.pullContext()

    def getUserIdAndTokenXmlRpc(
        self, virtualServerId, clientToken, userName, userPassword):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userName = iso8859_15(userName)
            userPassword = iso8859_15(userPassword)
            userId = getProxyForServerRole('people').checkObjectAuthentication(
                userName, userPassword, serverId = virtualServerId)
            virtualServer.lock.acquire()
            if virtualServer.userTokens.has_key(userId):
                userToken = virtualServer.userTokens[userId]
                virtualServer.testExpiration(userToken)
            if virtualServer.userTokens.has_key(userId):
                userToken = virtualServer.userTokens[userId]
                virtualServer.lock.release()
            else:
                while 1:
                    userLocalToken = str(
                        self.randomGenerator.uniform(0.1, 1))[2:]
                    userToken = '%s/%s' % (virtualServerId, userLocalToken)
                    if not virtualServer.userIds.has_key(userToken):
                        break
                virtualServer.userTokens[userId] = userToken
                virtualServer.userIds[userToken] = userId
                virtualServer.tokenExpirations[userToken] = \
                    time.time() + expirationTime
                virtualServer.lock.release()
                self.saveVirtualServer(virtualServer)
            return [userId, userToken]
        finally:
            self.pullContext()

    def init(self):
        self.randomGenerator = whrandom.whrandom()
        Server.init(self)

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('canLogin')
        self.registerXmlRpcMethod('getUserId')
        self.registerXmlRpcMethod('getUserIdAndToken',
                                  self.getUserIdAndTokenXmlRpc)
        self.registerXmlRpcMethod('setContainsUser')

    def setContainsUser(self, virtualServerId, clientToken, userToken, set):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            if not userToken:
                return 0
            userId = getProxyForServerRole('authentication').getUserId()
            return setContains(set, userId)
        finally:
            self.pullContext()


authenticationServer = AuthenticationServer()


if __name__ == "__main__":
    authenticationServer.launch(applicationName, applicationRole)
