#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Ballots Server"""

__version__ = '$Revision: 1.17 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import whrandom

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *


applicationName = 'BallotsServer'
applicationRole = 'ballots'
applicationTokens = None
dispatcher = None


class BallotsVirtualServer(VirtualServer):
    electionIds = None
    voteIds = None
    voterIds = None

    def convertIds(self, sourceDispatcherId, destinationDispatcherId):
        VirtualServer.convertIds(
            self, sourceDispatcherId, destinationDispatcherId)
        for token, id in self.electionIds.items():
            newId = id.replace(sourceDispatcherId, destinationDispatcherId)
            if newId != id:
                self.electionIds[token] = newId
        for token, id in self.voteIds.items():
            newId = id.replace(sourceDispatcherId, destinationDispatcherId)
            if newId != id:
                self.voteIds[token] = newId
        for token, id in self.voterIds.items():
            newId = id.replace(sourceDispatcherId, destinationDispatcherId)
            if newId != id:
                self.voterIds[token] = newId
        
    def init(self, server, virtualServerId):
        VirtualServer.init(self, server, virtualServerId)
        self.electionIds = {}
        self.voteIds = {}
        self.voterIds = {}

    def initFromOldData(self, server, virtualServerId, data):
        VirtualServer.initFromOldData(self, server, virtualServerId, data)
        (self.electionIds,
         self.voteIds,
         self.voterIds,
         ) = data

    def removeIds(self, rolesToKeep):
        VirtualServer.removeIds(self, rolesToKeep)
        for token, id in self.electionIds.items():
            if not commonTools.extractRole(id) in rolesToKeep:
                del self.electionIds[token]
        for token, id in self.voteIds.items():
            if not commonTools.extractRole(id) in rolesToKeep:
                del self.voteIds[token]
        for token, id in self.voterIds.items():
            if not commonTools.extractRole(id) in rolesToKeep:
                del self.voterIds[token]


class BallotsServer(Server):
    VirtualServer = BallotsVirtualServer
    version = __version__

    def abstainForVote(self, virtualServerId, clientToken, userToken,
                       electionId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            user = getProxyForServerRole('people').getObject(userId)
            if user.voteTokens is not None \
                   and user.voteTokens.has_key(electionId):
                previousVoteToken = user.voteTokens[electionId]
            else:
                previousVoteToken = ''
            if previousVoteToken \
               and virtualServer.voteIds.has_key(previousVoteToken):
                voteId = virtualServer.voteIds[previousVoteToken]
                previousVote = getProxyForServerRole('votes').getObject(voteId)
                del virtualServer.electionIds[previousVote.electionToken]
                del virtualServer.voterIds[previousVote.voterToken]
                del virtualServer.voteIds[previousVote.token]
                getProxyForServerRole('people').abstainForVote(electionId)
                getProxyForServerRole('elections').abstainForVote(electionId, previousVoteToken)
                getProxyForServerRole('votes').deleteObject(voteId)

            self.saveVirtualServer(virtualServer)
            return 0 # We have to return something.
        finally:
            self.pullContext()

    def canGetVoteVoterId(self, virtualServerId, clientToken, userToken,
                          voteId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            vote = getProxyForServerRole('votes').getObject(voteId)
            if not virtualServer.voterIds.has_key(vote.voterToken):
                return 0
            if not virtualServer.electionIds.has_key(vote.electionToken):
                return 0
            voterId = virtualServer.voterIds[vote.voterToken]
            electionId = virtualServer.electionIds[vote.electionToken]
            if voterId == userId:
                voteIsSecret = 0
            else:
                electionBallotKind = getProxyForServerRole('elections').getBallotKind(
                    electionId)
                voteIsSecret = electionBallotKind == 'secret' \
                               or electionBallotKind == 'voterChoice' \
                               and vote.ballotKind == 'secret'
            return not voteIsSecret
        finally:
            self.pullContext()

    def getElectionIdFromToken(self, virtualServerId, clientToken, userToken,
                               electionToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.electionIds.has_key(electionToken):
                raise faults.UnknownElectionToken(electionToken)
            return virtualServer.electionIds[electionToken]
        finally:
            self.pullContext()

    def getElectionVotes(self, virtualServerId, clientToken, voteTokens,
                         electionVoterIds):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            votesExport = []
            for voteToken in voteTokens:
                if virtualServer.voteIds.has_key(voteToken):
                    voteId = virtualServer.voteIds[voteToken]
                    if getProxyForServerRole('votes').hasObject(voteId):
                        vote = getProxyForServerRole('votes').getObject(voteId)
                        if virtualServer.voterIds.has_key(vote.voterToken):
                            voterId = virtualServer.voterIds[vote.voterToken]
                            if voterId in electionVoterIds:
                                votesExport.append(vote.exportToXmlRpc())
            return votesExport
        finally:
            self.pullContext()

    def getElectionWeightings(self, virtualServerId, clientToken, voteTokens,
                              weightingsById):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            electionVoterIds = weightingsById.keys()
            weightings = {}
            for voteToken in voteTokens:
                if virtualServer.voteIds.has_key(voteToken):
                    voteId = virtualServer.voteIds[voteToken]
                    if getProxyForServerRole('votes').hasObject(voteId):
                        vote = getProxyForServerRole('votes').getObject(voteId)
                        if virtualServer.voterIds.has_key(vote.voterToken):
                            voterId = virtualServer.voterIds[vote.voterToken]
                            if voterId in electionVoterIds:
                                weightings[vote.voterToken] = weightingsById[
                                    voterId]
            return weightings
        finally:
            self.pullContext()

    def getVote(self, virtualServerId, clientToken, userToken, voteId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            vote = getProxyForServerRole('votes').getObject(voteId)
            if not virtualServer.voterIds.has_key(vote.voterToken):
                raise faults.UnknownVoterToken(vote.voterToken)
            if not virtualServer.electionIds.has_key(vote.electionToken):
                raise faults.UnknownElectionToken(vote.electionToken)
##         voterId = virtualServer.voterIds[vote.voterToken]
##         electionId = virtualServer.electionIds[vote.electionToken]
##         if voterId == userId:
##             voteIsSecret = 0
##         else:
##             electionBallotKind = getProxyForServerRole('elections').getBallotKind(
##                 electionId)
##             voteIsSecret = electionBallotKind == 'secret' \
##                            or electionBallotKind == 'voterChoice' \
##                            and vote.ballotKind == 'secret'
##         if voteIsSecret:
##             voteExport = 'secret'
##         else:
##             voteExport = vote.exportToXmlRpc()
            voteExport = vote.exportToXmlRpc()
            return voteExport
        finally:
            self.pullContext()

    def getVoteFromToken(self, virtualServerId, clientToken, userToken,
                         voteToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if voteToken == 'secret':
                return 'secret'
            userId = getProxyForServerRole('authentication').getUserId()
            if not virtualServer.voteIds.has_key(voteToken):
                raise faults.UnknownVoteToken(voteToken)
            voteId = virtualServer.voteIds[voteToken]
            vote = getProxyForServerRole('votes').getObject(voteId)
            if not virtualServer.voterIds.has_key(vote.voterToken):
                raise faults.UnknownVoterToken(vote.voterToken)
            voterId = virtualServer.voterIds[vote.voterToken]
            if not virtualServer.electionIds.has_key(vote.electionToken):
                raise faults.UnknownElectionToken(vote.electionToken)
            electionId = virtualServer.electionIds[vote.electionToken]
            if voterId == userId:
                voteIsSecret = 0
            else:
                electionBallotKind = getProxyForServerRole('elections').getBallotKind(
                    electionId)
                voteIsSecret = electionBallotKind == 'secret' \
                               or electionBallotKind == 'voterChoice' \
                               and vote.ballotKind == 'secret'
            if voteIsSecret:
                voteExport = 'secret'
            else:
                voteExport = vote.exportToXmlRpc()
            return voteExport
        finally:
            self.pullContext()

    def getVotesFromTokens(self, virtualServerId, clientToken, userToken,
                           voteTokens):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            votesExport = {}
            for electionId, voteToken in voteTokens.items():
                if voteToken == 'secret':
                    votesExport[electionId] = 'secret'
                    continue
                if not getProxyForServerRole('elections').hasObject(electionId):
                    continue
                if not virtualServer.voteIds.has_key(voteToken):
                    continue
                voteId = virtualServer.voteIds[voteToken]
                if not getProxyForServerRole('votes').hasObject(voteId):
                    continue
                vote = getProxyForServerRole('votes').getObject(voteId)
                if not virtualServer.voterIds.has_key(vote.voterToken):
                    continue
                voterId = virtualServer.voterIds[vote.voterToken]
                if voterId == userId:
                    voteIsSecret = 0
                else:
                    electionBallotKind = getProxyForServerRole('elections').getBallotKind(
                        electionId)
                    voteIsSecret = electionBallotKind == 'secret' \
                                   or electionBallotKind == 'voterChoice' \
                                   and vote.ballotKind == 'secret'
                if voteIsSecret:
                    votesExport[electionId] = 'secret'
                else:
                    votesExport[electionId] = vote.exportToXmlRpc()
            return votesExport
        finally:
            self.pullContext()

    def getVoteVoterId(self, virtualServerId, clientToken, userToken, voteId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            vote = getProxyForServerRole('votes').getObject(voteId)
            if not virtualServer.voterIds.has_key(vote.voterToken):
                raise faults.UnknownVoterToken(vote.voterToken)
            if not virtualServer.electionIds.has_key(vote.electionToken):
                raise faults.UnknownElectionToken(vote.electionToken)
            voterId = virtualServer.voterIds[vote.voterToken]
            electionId = virtualServer.electionIds[vote.electionToken]
            if voterId == userId:
                voteIsSecret = 0
            else:
                electionBallotKind = getProxyForServerRole('elections').getBallotKind(
                    electionId)
                voteIsSecret = electionBallotKind == 'secret' \
                               or electionBallotKind == 'voterChoice' \
                               and vote.ballotKind == 'secret'
            if voteIsSecret:
                raise faults.UserAccessDenied()
            return voterId
        finally:
            self.pullContext()

    def init(self):
        self.randomGenerator = whrandom.whrandom()
        Server.init(self)

    def isVoteSecret(self, virtualServerId, clientToken, userToken, voteToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if voteToken == 'secret':
                return 'secret'
            userId = getProxyForServerRole('authentication').getUserId()
            if not virtualServer.voteIds.has_key(voteToken):
                raise faults.UnknownVoteToken(voteToken)
            voteId = virtualServer.voteIds[voteToken]
            vote = getProxyForServerRole('votes').getObject(voteId)
            if not virtualServer.voterIds.has_key(vote.voterToken):
                raise faults.UnknownVoterToken(vote.voterToken)
            voterId = virtualServer.voterIds[vote.voterToken]
            if not virtualServer.electionIds.has_key(vote.electionToken):
                raise faults.UnknownElectionToken(vote.electionToken)
            electionId = virtualServer.electionIds[vote.electionToken]
            if voterId == userId:
                voteIsSecret = 0
            else:
                electionBallotKind = getProxyForServerRole('elections').getBallotKind(
                    electionId)
                voteIsSecret = electionBallotKind == 'secret' \
                               or electionBallotKind == 'voterChoice' \
                               and vote.ballotKind == 'secret'
            return voteIsSecret
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('abstainForVote')
        self.registerXmlRpcMethod('canGetVoteVoterId')
        self.registerXmlRpcMethod('getElectionIdFromToken')
        self.registerXmlRpcMethod('getElectionVotes')
        self.registerXmlRpcMethod('getElectionWeightings')
        self.registerXmlRpcMethod('getVote')
        self.registerXmlRpcMethod('getVoteFromToken')
        self.registerXmlRpcMethod('getVotesFromTokens')
        self.registerXmlRpcMethod('getVoteVoterId')
        self.registerXmlRpcMethod('isVoteSecret')
        self.registerXmlRpcMethod('vote')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 4000:
            for token, id in virtualServer.electionIds.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    virtualServer.electionIds[token] = newId
            for token, id in virtualServer.voteIds.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    virtualServer.voteIds[token] = newId
            for token, id in virtualServer.voterIds.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    virtualServer.voterIds[token] = newId
        if version < 1007000:
            for token, id in virtualServer.voteIds.items():
                if not token.startswith('glasnost://'):
                    changed = 1
                    del virtualServer.voteIds[token]
                    token = '%s/votes/%s' % (
                        commonTools.extractDispatcherId(
                            virtualServer.virtualServerId), token)
                    virtualServer.voteIds[token] = id
        if changed:
            self.saveVirtualServer(virtualServer)

    def vote(self, virtualServerId, clientToken, userToken, electionId,
             voteImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            user = getProxyForServerRole('people').getObject(userId)
            if user.voteTokens is not None \
                   and user.voteTokens.has_key(electionId):
                previousVoteToken = user.voteTokens[electionId]
            else:
                previousVoteToken = ''

            if previousVoteToken \
               and virtualServer.voteIds.has_key(previousVoteToken):
                voteId = virtualServer.voteIds[previousVoteToken]
            else:
                voteId = ''

            vote = getProxyForServerRole('votes').newThing('object', voteImport['__thingName__'])
            vote.importFromXmlRpc(voteImport)
            if voteId:
                vote.id = voteId

            while 1:
                electionToken = str(self.randomGenerator.uniform(0.1, 1))[2:]
                if not virtualServer.electionIds.has_key(electionToken):
                    break
            vote.electionToken = electionToken

            while 1:
                voterToken = str(self.randomGenerator.uniform(0.1, 1))[2:]
                if not virtualServer.voterIds.has_key(voterToken):
                    break
            vote.voterToken = voterToken

            while 1:
                voteLocalToken = str(self.randomGenerator.uniform(0.1, 1))[2:]
                voteToken = '%s/votes/%s' % (
                    commonTools.extractDispatcherId(virtualServerId),
                    voteLocalToken)
                if not virtualServer.voteIds.has_key(voteToken):
                    break
            vote.token = voteToken

            if voteId:
                previousVote = getProxyForServerRole('votes').getObject(voteId)
                getProxyForServerRole('votes').modifyObject(vote)
                del virtualServer.electionIds[previousVote.electionToken]
                del virtualServer.voterIds[previousVote.voterToken]
                del virtualServer.voteIds[previousVote.token]
            else:
                voteId = getProxyForServerRole('votes').addObject(
                    vote, serverId = extractServerId(electionId))

            getProxyForServerRole('people').vote(electionId, voteToken)
            getProxyForServerRole('elections').vote(electionId, voteToken, previousVoteToken)

            virtualServer.electionIds[electionToken] = electionId
            virtualServer.voterIds[voterToken] = userId
            virtualServer.voteIds[voteToken] = voteId

            self.saveVirtualServer(virtualServer)
            return voteId
        finally:
            self.pullContext()


ballotsServer = BallotsServer()


if __name__ == "__main__":
    ballotsServer.launch(applicationName, applicationRole)
