#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Books Server"""

__version__ = '$Revision: 1.33 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import fcntl
import os

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.BooksCommon import *
import glasnost.common.faults as faults
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.DispatcherProxy import getApplicationId
from glasnost.proxy.GroupsProxy import getSetContainedIds


applicationName = 'BooksServer'
applicationRole = 'books'
applicationTokens = None
dispatcher = None


class AdminBooks(AdminServerMixin, AdminBooksCommon):
    pass
register(AdminBooks)


class Book(ObjectServerMixin, BookCommon):
    def acquireNonCore(self, objectDirectoryPath = None,
                       dataDirectoryPath = None, parentSlot = None):
        ObjectServerMixin.acquireNonCore(
            self, objectDirectoryPath = objectDirectoryPath,
            dataDirectoryPath = dataDirectoryPath, parentSlot = parentSlot)
        self.loadBody()

    def loadBody(self):
        virtualServerId = self.getServer().computeVirtualServerId(self.id)
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        booksDirectoryPath = os.path.join(
            virtualServer.dataDirectoryPath, self.getServer().applicationRole)
        serverHostNameAndPort, serverRole, localId = splitId(self.id)
        bodyFilePath = os.path.join(booksDirectoryPath, localId)
        try:
            bodyFile = open(bodyFilePath, 'rb')
        except IOError:
            if self.__dict__.has_key('body'):
                del self.body
        else:
            fcntl.lockf(bodyFile, fcntl.LOCK_SH)
            self.body = bodyFile.read()
            fcntl.lockf(bodyFile, fcntl.LOCK_UN)
            bodyFile.close()

    def makeBody(self):
        ### FIXME: makeBody does nothing, because TAL doesn't work when called
        ### from a server (ImportError: No module named driver).
        return
##         if self.nonItemIds is None:
##             nonItemIds = []
##         else:
##             nonItemIds = self.nonItemIds
##         allItemIds = []
##         if self.chapterIds is not None:
##             for itemId in self.chapterIds:
##                 if itemId not in allItemIds and itemId not in nonItemIds:
##                     allItemIds.append(itemId)
##         if self.appendixIds is not None:
##             for itemId in self.appendixIds:
##                 if itemId not in allItemIds and itemId not in nonItemIds:
##                     allItemIds.append(itemId)
##         if self.itemIds is not None:
##             for itemId in self.itemIds:
##                 if itemId not in allItemIds and itemId not in nonItemIds:
##                     allItemIds.append(itemId)
        from glasnost.common.parsers import FormatterDocBook, FormatterLatex
        from glasnost.proxy.tools import getProxy
        virtualServerId = self.getServer().computeVirtualServerId(self.id)
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        booksDirectoryPath = os.path.join(
            virtualServer.dataDirectoryPath, self.getServer().applicationRole)
        serverHostNameAndPort, serverRole, localId = splitId(self.id)
        illustrationsDirectoryPath = os.path.join(
            booksDirectoryPath, localId + '-illustrations')
        if not os.access(illustrationsDirectoryPath, os.F_OK):
            os.mkdir(illustrationsDirectoryPath)
            os.chmod(illustrationsDirectoryPath, 0750)
        else:
            fileNames = os.listdir(illustrationsDirectoryPath)
            map(os.remove, map(
                lambda fileName,
                    illustrationsDirectoryPath = illustrationsDirectoryPath:
                os.path.join(illustrationsDirectoryPath, fileName),
                fileNames))
        allItemIds = []
##         formatter = FormatterDocBook()
        formatter = FormatterLatex()
##         text = formatter.open(docType = 'book', title = self.title)
        text = formatter.open(docType = 'report', title = self.title)
        if self.chapterIds is not None:
            for itemId in self.chapterIds:
                allItemIds.append(itemId)
##                 chapterText = getProxy(itemId).getObjectDocBookChapter(
##                     itemId)
                chapterText = getProxy(itemId).getObjectLatexChapter(itemId)
                text += chapterText
        text += formatter.close(docType = 'book')
        # FIXME: Replace Req by XhtmlGenerator.Context and uriScriptPath by
        # scriptPath.
        class Req:
##             uriScriptPath = '/'
            uriScriptPath = 'http://localhost/'
        req = Req()
        from glasnost.web.tools import replaceSpecialTags
        text = replaceSpecialTags(req, text)

        import re
        import whrandom
        randomGenerator = whrandom.whrandom()

        def includegraphicsReplacer(
            matchObject,
            illustrationsDirectoryPath = illustrationsDirectoryPath,
            randomGenerator = randomGenerator):
            url = matchObject.group('url')
            width = matchObject.group('width')
            height = matchObject.group('height')

            def newIllustrationFilePath(
                fileNameSuffix = '',
                directoryPath = illustrationsDirectoryPath,
                randomGenerator = randomGenerator):
                fileName = str(randomGenerator.uniform(0.1, 1))[2:] \
                           + fileNameSuffix
                filePath = os.path.join(directoryPath, fileName)
                return filePath

            import urllib
            imagePath = newIllustrationFilePath()
            try:
                imagePath, notUsed = urllib.urlretrieve(
                    url, imagePath)
            except IOError:
                # The server is not available.
                return ''
            import Image
            image = Image.open(imagePath)
            fileExtensions = {
                'GIF': '.gif',
                'JPEG': '.jpg',
                'PNG': '.png',
                }
            if image.format in fileExtensions.keys():
                fileExtension = fileExtensions[image.format]
                # Latex is not always able to read some jpef files. So, we
                # rewrite the file using PIL instead of just renaming it.
                # os.rename(imagePath, imagePath + fileExtension)
                image.save(imagePath + fileExtension, image.format)
                os.remove(imagePath)
                imagePath += fileExtension
            keys = 'keepaspectratio'
            if width:
                keys += ',width=%spt' % width
            if height:
                keys += ',height=%spt' % height
            return '\\includegraphics[%(keys)s]{%(imagePath)s}' % {
                'imagePath': imagePath,
                'keys': keys,
                }

        text = re.sub('\[\{glasnost\:latex\:includegraphics\:(?P<url>.*?)\:width=(?P<width>.*?)\:height=(?P<height>.*?)\}\]',
                      includegraphicsReplacer, text)

        self.body = text

    def releaseNonCore(self, parentSlot = None):
        if self.__dict__.has_key('body'):
            del self.body
        ObjectServerMixin.releaseNonCore(self, parentSlot = parentSlot)

    def removeBodyFile(self):
        virtualServerId = self.getServer().computeVirtualServerId(self.id)
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        booksDirectoryPath = os.path.join(
            virtualServer.dataDirectoryPath, self.getServer().applicationRole)
        serverHostNameAndPort, serverRole, localId = splitId(self.id)
        bodyFilePath = os.path.join(booksDirectoryPath, localId)
        try:
            os.remove(bodyFilePath)
        except OSError, error:
            # Ignore 'No such file or directory' error.
            if error.errno != 2:
                raise

    def removeNonCore(self, objectDirectoryPath = None,
                      dataDirectoryPath = None, parentSlot = None):
        ObjectServerMixin.removeNonCore(
            self, objectDirectoryPath = objectDirectoryPath,
            dataDirectoryPath = dataDirectoryPath, parentSlot = parentSlot)
        self.removeBodyFile()

    def saveBody(self):
        if self.body is None:
            self.removeBodyFile()
            return
        virtualServerId = self.getServer().computeVirtualServerId(self.id)
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        booksDirectoryPath = os.path.join(
            virtualServer.dataDirectoryPath, self.getServer().applicationRole)
        if not os.access(virtualServer.dataDirectoryPath, os.F_OK):
            os.mkdir(virtualServer.dataDirectoryPath)
            os.chmod(virtualServer.dataDirectoryPath, 0750)
        if not os.access(booksDirectoryPath, os.F_OK):
            os.mkdir(booksDirectoryPath)
            os.chmod(booksDirectoryPath, 0750)
        serverHostNameAndPort, serverRole, localId = splitId(self.id)
        bodyFilePath = os.path.join(booksDirectoryPath, localId)
        bodyFile = open(bodyFilePath, 'wb')
        os.chmod(bodyFilePath, 0640)
        fcntl.lockf(bodyFile, fcntl.LOCK_EX)
        bodyFile.write(self.body)
        fcntl.lockf(bodyFile, fcntl.LOCK_UN)
        bodyFile.close()

    def saveNonCore(self, objectDirectoryPath = None, dataDirectoryPath = None,
                    parentSlot = None):
        ObjectServerMixin.saveNonCore(
            self, objectDirectoryPath = objectDirectoryPath,
            dataDirectoryPath = dataDirectoryPath, parentSlot = parentSlot)
        self.saveBody()
register(Book)
    

class BooksServer(ObjectsServer):
    adminClassName = 'AdminBooks'
    objectClassName = 'Book'
    version = __version__

    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)
            if not self.isAdmin(virtualServerId, clientToken, userToken) \
               and not (getProxyForServerRole('authentication').setContainsUser(
                self.getAdminCore().writersSet)
                        and getProxyForServerRole('authentication').setContainsUser(
                object.writersSet)):
                clientId = getApplicationId(clientToken)
                if not object.canBeCreatedByClient(clientId):
                    raise faults.UserAccessDenied()
            object.checkAddIsPossible()
            object.setAutomaticalSlots()
            object.id = self.getNextId()
            virtualServer.objects[object.id] = object
            object.saveNonCore()
            object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            return object.id
        finally:
            self.pullContext()

    def convertVirtualServersIds(self, sourceDispatcherId,
                                 destinationDispatcherId):
        exitCode = ObjectsServer.convertVirtualServersIds(
            self, sourceDispatcherId, destinationDispatcherId)
        if exitCode is not None:
            return exitCode
        destinationVirtualServerId = '%s/%s' % (
            destinationDispatcherId, self.applicationRole)
        virtualServer = self.virtualServers[destinationVirtualServerId]
        sourceHostName = extractApplicationHostName(sourceDispatcherId)
        destinationHostName = extractApplicationHostName(
            destinationDispatcherId)
        sourceVirtualServerDataDirectoryPath = os.path.join(
            self.dataDirectoryPath, sourceHostName)
        destinationVirtualServerDataDirectoryPath = os.path.join(
            self.dataDirectoryPath, destinationHostName)
        sourceObjectsDirectoryPath = os.path.join(
            sourceVirtualServerDataDirectoryPath, self.applicationRole)
        destinationObjectsDirectoryPath = os.path.join(
            destinationVirtualServerDataDirectoryPath, self.applicationRole)
        for id in virtualServer.objects.keys():
            serverHostNameAndPortNotUsed, serverRoleNotUsed, localId = \
                                          splitId(id)
            sourceBodyFilePath = os.path.join(
                sourceObjectsDirectoryPath, localId)
            destinationBodyFilePath = os.path.join(
                destinationObjectsDirectoryPath, localId)
            if not os.access(sourceBodyFilePath, os.F_OK):
                continue
            if not os.access(
                destinationVirtualServerDataDirectoryPath, os.F_OK):
                os.mkdir(destinationVirtualServerDataDirectoryPath)
                os.chmod(destinationVirtualServerDataDirectoryPath, 0750)
            if not os.access(destinationObjectsDirectoryPath, os.F_OK):
                os.mkdir(destinationObjectsDirectoryPath)
                os.chmod(destinationObjectsDirectoryPath, 0750)
            sourceBodyFile = open(sourceBodyFilePath, 'rb')
            destinationBodyFile = open(destinationBodyFilePath, 'wb')
            os.chmod(destinationBodyFilePath, 0640)
            destinationBodyFile.write(sourceBodyFile.read())
            sourceBodyFile.close()
            destinationBodyFile.close()
        return None

    def deleteObject(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            if not self.isAdmin(virtualServerId, clientToken, userToken) \
               and not getProxyForServerRole('authentication').setContainsUser(object.writersSet):
                raise faults.UserAccessDenied()
            del virtualServer.objects[objectId]
            object.removeNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(objectId)
            return objectId # We have to return something.
        finally:
            self.pullContext()

    def exportVirtualServer(self, virtualServerId, exportDirectoryPath):
        exitCode = ObjectsServer.exportVirtualServer(
            self, virtualServerId, exportDirectoryPath)
        if exitCode is not None:
            return exitCode
        virtualServer = self.virtualServers[virtualServerId]
        hostName = extractApplicationHostName(virtualServerId)
        virtualServerDataDirectoryPath = os.path.join(
            self.dataDirectoryPath, hostName)
        objectsDirectoryPath = os.path.join(
            virtualServerDataDirectoryPath, self.applicationRole)
        exportObjectsDirectoryPath = os.path.join(
            exportDirectoryPath, self.applicationRole)
        for id in virtualServer.objects.keys():
            serverHostNameAndPortNotUsed, serverRoleNotUsed, localId = \
                                          splitId(id)
            sourceBodyFilePath = os.path.join(
                objectsDirectoryPath, localId)
            exportBodyFilePath = os.path.join(
                exportObjectsDirectoryPath, localId)
            if not os.access(sourceBodyFilePath, os.F_OK):
                continue
            if not os.access(exportDirectoryPath, os.F_OK):
                os.mkdir(exportDirectoryPath)
                os.chmod(exportDirectoryPath, 0750)
            if not os.access(exportObjectsDirectoryPath, os.F_OK):
                os.mkdir(exportObjectsDirectoryPath)
                os.chmod(exportObjectsDirectoryPath, 0750)
            sourceBodyFile = open(sourceBodyFilePath, 'rb')
            exportBodyFile = open(exportBodyFilePath, 'wb')
            os.chmod(exportBodyFilePath, 0640)
            exportBodyFile.write(sourceBodyFile.read())
            sourceBodyFile.close()
            exportBodyFile.close()
        return None

    def getLastObjectIds(self, virtualServerId, clientToken, userToken,
                         objectsCount, possibleAuthorsSet, possibleReadersSet,
                         possibleWritersSet):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            isAdmin = self.isAdmin(virtualServerId, clientToken, userToken)
            possibleAuthorIds = getSetContainedIds(
                possibleAuthorsSet, ['people'], serverId = virtualServerId)
            possibleReaderIds = getSetContainedIds(
                possibleReadersSet, ['people'], serverId = virtualServerId)
            possibleWriterIds = getSetContainedIds(
                possibleWritersSet, ['people'], serverId = virtualServerId)
            objectIds = virtualServer.objects.keys()
            def modificationTimeSorter(xId, yId,
                                       virtualServer = virtualServer):
                return cmp(virtualServer.objects[yId].modificationTime,
                           virtualServer.objects[xId].modificationTime)
            objectIds.sort(modificationTimeSorter)
            result = []
            for objectId in objectIds:
                object = virtualServer.objects[objectId]
                if not isAdmin and object.readersSet \
                   and not getProxyForServerRole('authentication').setContainsUser(
                    object.readersSet):
                    continue
                if not possibleAuthorIds:
                    authorFound = 1
                else:
                    authorFound = 0
                    objectAuthorIds = getSetContainedIds(
                        object.authorsSet, ['people'],
                        serverId = virtualServerId)
                    if objectAuthorIds:
                        for possibleAuthorId in possibleAuthorIds:
                            if possibleAuthorId in objectAuthorIds:
                                authorFound = 1
                                break
                if not possibleReaderIds or not object.readersSet:
                    readerFound = 1
                else:
                    readerFound = 0
                    objectReaderIds = getSetContainedIds(
                        object.readersSet, ['people'],
                        serverId = virtualServerId)
                    if objectReaderIds:
                        for possibleReaderId in possibleReaderIds:
                            if possibleReaderId in objectReaderIds:
                                readerFound = 1
                                break
                if not possibleWriterIds:
                    writerFound = 1
                else:
                    writerFound = 0
                    objectWriterIds = getSetContainedIds(
                        object.writersSet, ['people'],
                        serverId = virtualServerId)
                    if objectWriterIds:
                        for possibleWriterId in possibleWriterIds:
                            if possibleWriterId in objectWriterIds:
                                writerFound = 1
                                break
                if authorFound and readerFound and writerFound:
                    result.append(objectId)
                    if len(result) >= objectsCount:
                        break
            return result
        finally:
            self.pullContext()

    def importVirtualServer(self, virtualServerId, importDirectoryPath):
        virtualServer = ObjectsServer.importVirtualServer(
            self, virtualServerId, importDirectoryPath)
        if virtualServer is None:
            return None
        hostName = extractApplicationHostName(virtualServerId)
        virtualServerDataDirectoryPath = os.path.join(
            self.dataDirectoryPath, hostName)
        importObjectsDirectoryPath = os.path.join(
            importDirectoryPath, self.applicationRole)
        objectsDirectoryPath = os.path.join(
            virtualServerDataDirectoryPath, self.applicationRole)
        for id in virtualServer.objects.keys():
            serverHostNameAndPortNotUsed, serverRoleNotUsed, localId = \
                                          splitId(id)
            importBodyFilePath = os.path.join(
                importObjectsDirectoryPath, localId)
            destinationBodyFilePath = os.path.join(
                objectsDirectoryPath, localId)
            if not os.access(importBodyFilePath, os.F_OK):
                continue
            if not os.access(virtualServerDataDirectoryPath, os.F_OK):
                os.mkdir(virtualServerDataDirectoryPath)
                os.chmod(virtualServerDataDirectoryPath, 0750)
            if not os.access(objectsDirectoryPath, os.F_OK):
                os.mkdir(objectsDirectoryPath)
                os.chmod(objectsDirectoryPath, 0750)
            importBodyFile = open(importBodyFilePath, 'rb')
            destinationBodyFile = open(destinationBodyFilePath, 'wb')
            os.chmod(destinationBodyFilePath, 0640)
            destinationBodyFile.write(importBodyFile.read())
            importBodyFile.close()
            destinationBodyFile.close()
        return virtualServer

    def makeObjectBody(self, virtualServerId, clientToken, userToken,
                       objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            object.acquireNonCore()
            try:
                if not object.canBeModified():
                    raise faults.ReadOnlyObject()
                if not self.isAdmin(virtualServerId, clientToken, userToken) \
                   and not getProxyForServerRole('authentication').setContainsUser(
                    object.writersSet):
                    raise faults.UserAccessDenied()
                oldBody = object.body
                object.makeBody()
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            return object.id # We have to return something.
        finally:
            self.pullContext()

    def modifyObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                           objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            objectChanges = self.newThing(
                'object', objectImport['__thingName__'])
            objectChanges.importFromXmlRpc(objectImport)
            virtualServerId = self.computeVirtualServerId(objectChanges.id)
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectChanges.id):
                raise faults.MissingItem(objectChanges.id)
            object = virtualServer.objects[objectChanges.id]
            object.acquireNonCore()
            try:
                if not object.canBeModified():
                    raise faults.ReadOnlyObject()
                if not self.isAdmin(virtualServerId, clientToken, userToken):
                    if not getProxyForServerRole('authentication').setContainsUser(
                        object.writersSet) \
                       or not getProxyForServerRole('authentication').setContainsUser(
                        objectChanges.writersSet):
                        clientId = getApplicationId(clientToken)
                        if not object.canBeModifiedByClient(clientId):
                            raise faults.UserAccessDenied()
                object.checkModifyIsPossible(objectChanges)
                oldBody = object.body
                object.modify(objectChanges)
                object.makeBody()
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            return object.version
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('makeObjectBody')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 4000:
            changed = virtualServer.admin.repair(4000) or changed
            for id, object in virtualServer.objects.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    del virtualServer.objects[id]
                    virtualServer.objects[newId] = object
                changed = object.repair(4000) or changed
                if not object.__dict__.has_key('language'):
                    changed = 1
                    object.language = 'fr'
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1017000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


booksServer = BooksServer()


if __name__ == "__main__":
    booksServer.launch(applicationName, applicationRole)
