#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Breves Server"""

__version__ = '$Revision: 1.21 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.BrevesCommon import *
import glasnost.common.faults as faults
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.GroupsProxy import getSetContainedIds


applicationName = 'BrevesServer'
applicationRole = 'breves'
applicationTokens = None
dispatcher = None


class AdminBreves(AdminServerMixin, AdminBrevesCommon):
    pass
register(AdminBreves)


class BreveMixin(ObjectServerMixin):
    pass


class Breve(BreveMixin, BreveCommon):
    pass
register(Breve)


class BreveElectionMixin(BreveMixin):
    def canBeCreatedByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return ObjectServerMixin.canBeCreatedByClient(self, clientId) \
               or clientRole == 'elections'

    def canBeModified(self):
        return 0


class BreveElectionCandidateAdded(BreveElectionMixin,
                                  BreveElectionCandidateAddedCommon):
    pass
register(BreveElectionCandidateAdded)


class BreveElectionClosed(BreveElectionMixin,
                          BreveElectionClosedCommon):
    pass
register(BreveElectionClosed)


class BreveElectionRunning(BreveElectionMixin,
                           BreveElectionRunningCommon):
    pass
register(BreveElectionRunning)


class BrevesServer(ObjectsServer):
    adminClassName = 'AdminBreves'
    objectClassName = 'Breve'
    version = __version__

    def getLastObjectIds(self, virtualServerId, clientToken, userToken,
                         objectsCount, possibleAuthorsSet, possibleReadersSet,
                         possibleWritersSet):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            isAdmin = self.isAdmin(virtualServerId, clientToken, userToken)
            possibleAuthorIds = getSetContainedIds(
                possibleAuthorsSet, ['people'], serverId = virtualServerId)
            possibleReaderIds = getSetContainedIds(
                possibleReadersSet, ['people'], serverId = virtualServerId)
            possibleWriterIds = getSetContainedIds(
                possibleWritersSet, ['people'], serverId = virtualServerId)
            objectIds = virtualServer.objects.keys()
            def modificationTimeSorter(xId, yId,
                                       virtualServer = virtualServer):
                return cmp(virtualServer.objects[yId].modificationTime,
                           virtualServer.objects[xId].modificationTime)
            objectIds.sort(modificationTimeSorter)
            result = []
            for objectId in objectIds:
                object = virtualServer.objects[objectId]
                if not isAdmin and object.readersSet \
                   and not getProxyForServerRole('authentication').setContainsUser(
                    object.readersSet):
                    continue
                if not possibleAuthorIds:
                    authorFound = 1
                else:
                    authorFound = 0
                    objectAuthorIds = getSetContainedIds(
                        object.authorsSet, ['people'],
                        serverId = virtualServerId)
                    if objectAuthorIds:
                        for possibleAuthorId in possibleAuthorIds:
                            if possibleAuthorId in objectAuthorIds:
                                authorFound = 1
                                break
                if not possibleReaderIds or not object.readersSet:
                    readerFound = 1
                else:
                    readerFound = 0
                    objectReaderIds = getSetContainedIds(
                        object.readersSet, ['people'],
                        serverId = virtualServerId)
                    if objectReaderIds:
                        for possibleReaderId in possibleReaderIds:
                            if possibleReaderId in objectReaderIds:
                                readerFound = 1
                                break
                if not possibleWriterIds:
                    writerFound = 1
                else:
                    writerFound = 0
                    objectWriterIds = getSetContainedIds(
                        object.writersSet, ['people'],
                        serverId = virtualServerId)
                    if objectWriterIds:
                        for possibleWriterId in possibleWriterIds:
                            if possibleWriterId in objectWriterIds:
                                writerFound = 1
                                break
                if authorFound and readerFound and writerFound:
                    result.append(objectId)
                    if len(result) >= objectsCount:
                        break
            return result
        finally:
            self.pullContext()

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 3006:
            for object in virtualServer.objects.values():
                if object.__dict__.has_key('state'):
                    changed = 1
                    object.writersSet = object.authorsSet
                    if object.writersSet is not None:
                        object.writersSet = object.writersSet[:]
                    if object.state != 'published':
                        object.readersSet = object.authorsSet
                        if object.readersSet is not None:
                            object.readersSet = object.readersSet[:]
                    del object.state
        if version < 4000:
            changed = virtualServer.admin.repair(4000) or changed
            for id, object in virtualServer.objects.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    del virtualServer.objects[id]
                    virtualServer.objects[newId] = object
                changed = object.repair(4000) or changed
                if not object.__dict__.has_key('language'):
                    changed = 1
                    object.language = 'fr'
        if version < 4004:
            for id, object in virtualServer.objects.items():
                if not object.__dict__.has_key('language'):
                    changed = 1
                    object.language = 'fr'
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1008000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


brevesServer = BrevesServer()


if __name__ == "__main__":
    brevesServer.launch(applicationName, applicationRole)
