#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Cache"""

__version__ = '$Revision: 1.9 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import time

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *


applicationName = 'Cache'
applicationRole = 'cache' # no!
applicationTokens = None
dispatcher = None


class CacheVirtualServer(VirtualServer):
    def init(self, server, virtualServerId):
        VirtualServer.init(self, server, virtualServerId)


class Cache(Server):
    VirtualServer = CacheVirtualServer
    useDataFile = 0
    version = __version__
    virtualServer = None
    invalidatedValues = {}

    def load(self):
        self.virtualServer = self.loadVirtualServer(None)

    def loadVirtualServer(self, virtualServerIdNotUsed):
        return Server.loadVirtualServer(self, None)

    def invalidateValue(self, clientToken, objectId):
        self.invalidatedValues[objectId] = time.time()
        return objectId

    def isValueStillValid(self, clientToken, objectId, timeCached):
        if not self.invalidatedValues.has_key(objectId):
            return ''
        if self.invalidatedValues[objectId] < timeCached:
            return ''
        return objectId

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('invalidateValue')
        self.registerXmlRpcMethod('isValueStillValid')


cache = Cache()


if __name__ == "__main__":
    cache.launch(applicationName, applicationRole)
