#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Cards Server"""

__version__ = '$Revision: 1.22 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.CardsCommon import *
import glasnost.common.faults as faults

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
import glasnost.server.properties as properties
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue


applicationName = 'CardsServer'
applicationRole = 'cards'
applicationTokens = None
dispatcher = None


class AdminCards(AdminServerMixin, AdminCardsCommon):
    pass
register(AdminCards)


class Card(ObjectServerMixin, properties.PropertiesContainerMixin, CardCommon):
    def initPartial(self, partial, givenSlotNames = None):
        if givenSlotNames and not 'prototypeIds' in givenSlotNames:
            partial.prototypeIds = self.prototypeIds
        if givenSlotNames and not 'properties' in givenSlotNames:
            partial.properties = self.properties

    def getModifySlotNames(self, parentSlot = None):
        names = properties.PropertiesContainerMixin.getModifySlotNames(
            self, parentSlot = parentSlot)
        names = names[:]
        if 'prototypeIds' in names:
            # The slot 'prototypeIds' is already mofified by the method
            # modify.
            names.remove('prototypeIds')
        return names

    def modify(self, changes, givenSlotNames = None):
        # The slot 'prototypeIds' (and 'properties') must be modified first,
        # because it is needed to compute the slotNames list.
        for slotName in ['prototypeIds']:
            slot = self.getSlot(slotName)
            kind = slot.getKind()
            if kind.isAutomaticalyModified:
                kind.setAutomaticalValue(slot)
                continue
            if not kind.hasToModify:
                continue
            if givenSlotNames and not slotName in givenSlotNames:
                continue
            changesSlot = changes.getSlot(slotName)
            slot.setValue(changesSlot.getValue())
        properties.PropertiesContainerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
register(Card)


class CardsServer(ObjectsServer):
    adminClassName = 'AdminCards'
    objectClassName = 'Card'
    version = __version__

    def getC3PrototypeLinearization(self, virtualServerId, clientToken,
                                    userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            return object.getC3PrototypeLinearization()
        finally:
            self.pullContext()

    def getImplementationPropertyNames(self, virtualServerId, clientToken,
                                       userToken, objectId, parentPath):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            parentSlot = object.getSlotByPath(parentPath)
            if parentSlot is None:
                implementation = object
            else:
                implementation = parentSlot.getValue()
            return implementation.getImplementationPropertyNames(
                parentSlot = parentSlot)
        finally:
            self.pullContext()

    def getImplementationPropertyValueKindXmlRpc(
        self, virtualServerId, clientToken, userToken, objectId, propertyName,
        parentPath):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            parentSlot = object.getSlotByPath(parentPath)
            if parentSlot is None:
                implementation = object
            else:
                implementation = parentSlot.getValue()

            propertyKind = implementation.getImplementationPropertyValueKind(
                propertyName, parentSlot = parentSlot)
            if propertyKind is None:
                raise faults.NoneValue()

            propertySlot = implementation.getPropertySlot(
                propertyName, parentSlot = parentSlot)
            property = propertySlot.getValue()
            propertyKindSlot = property.getSlot(
                'kind', parentSlot = propertySlot)
            propertyKindSlot.value = propertyKind
            propertyKindExport = propertyKind.exportToXmlRpc(
                parentSlot = propertyKindSlot)
            return propertyKindExport
        finally:
            self.pullContext()

    def getPropertyValueXmlRpc(
        self, virtualServerId, clientToken, userToken, objectId,
        propertyName, propertyKindImport, parentPath):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            parentSlot = object.getSlotByPath(parentPath)
            if parentSlot is None:
                implementation = object
            else:
                implementation = parentSlot.getValue()

            propertySlot = implementation.getPropertySlot(
                propertyName, parentSlot = parentSlot)
            property = propertySlot.getValue()
            propertyKindSlot = property.getSlot(
                'kind', parentSlot = propertySlot)
            propertyKind = self.newThing(
                'kind', propertyKindImport['__thingName__'])
            propertyKindSlot.value = propertyKind
            propertyKind.importFromXmlRpc(
                propertyKindImport, parentSlot = propertyKindSlot)

            value = implementation.getPropertyValue(
                propertyName, propertyKind, parentSlot = parentSlot)
            if value is None:
                raise faults.NoneValue()
            slot = implementation.getPropertyValueSlot(
                propertyName, parentSlot = parentSlot)
            return propertyKind.exportValueToXmlRpc(slot, value)
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getC3PrototypeLinearization')
        self.registerXmlRpcMethod('getImplementationPropertyNames')
        self.registerXmlRpcMethod(
            'getImplementationPropertyValueKind',
            self.getImplementationPropertyValueKindXmlRpc)
        self.registerXmlRpcMethod('getPropertyValue',
                                  self.getPropertyValueXmlRpc)
        self.registerXmlRpcMethod(
            'setImplementationPropertyValueKind',
            self.setImplementationPropertyValueKindXmlRpc)

    def setImplementationPropertyValueKindXmlRpc(
        self, virtualServerId, clientToken, userToken, objectId,
        propertyName, propertyKindImport, parentPath):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            parentSlot = object.getSlotByPath(parentPath)
            if parentSlot is None:
                implementation = object
            else:
                implementation = parentSlot.getValue()

            propertySlot = implementation.getPropertySlot(
                propertyName, parentSlot = parentSlot)
            property = propertySlot.getValue()
            propertyKindSlot = property.getSlot(
                'kind', parentSlot = propertySlot)
            propertyKind = self.newThing(
                'kind', propertyKindImport['__thingName__'])
            propertyKindSlot.value = propertyKind
            propertyKind.importFromXmlRpc(
                propertyKindImport, parentSlot = propertyKindSlot)

            object.acquireNonCore()
            try:
                if not object.canBeModified():
                    raise faults.ReadOnlyObject()
                if not self.canModifyObject(
                    virtualServerId, clientToken, userToken, object.id):
                    clientId = getApplicationId(clientToken)
                    if not object.canBeModifiedByClient(clientId):
                        raise faults.UserAccessDenied()
                implementation.setImplementationPropertyValueKind(
                    propertyName, propertyKind, parentSlot = parentSlot)
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            # FIXME: Maybe we need to increment the version number and return
            # it.
            return 0 # We have to return something.
        finally:
            self.pullContext()


cardsServer = CardsServer()


if __name__ == "__main__":
    cardsServer.launch(applicationName, applicationRole)
