#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Comments Server"""

__version__ = '$Revision: 1.14 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.CommentsCommon import *
import glasnost.common.faults as faults

from glasnost.server.ObjectsServer import ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.things import register
from glasnost.server.tools import *


applicationName = 'CommentsServer'
applicationRole = 'comments'
applicationTokens = None
dispatcher = None


class AdminComments(AdminServerMixin, AdminCommentsCommon):
    pass
register(AdminComments)


class Comment(ObjectServerMixin, CommentCommon):
    pass
register(Comment)


class CommentsServer(ObjectsServer):
    adminClassName = 'AdminComments'
    useAdminWritersSet = 1
    version = __version__
    
    def canAddObject(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            admin = self.getAdminCore()
            if admin.allowAnonymousComments:
                return 1
            return ObjectsServer.canAddObject(self, virtualServerId, clientToken, userToken)
        finally:
            self.pullContext()
    
    def getComments(self, virtualServerId, clientToken, userToken, sourceId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            objects = virtualServer.objects.values()
            result = [o.id for o in objects if o.sourceId == sourceId]
            result = [self.getObjectXmlRpc(virtualServerId, clientToken, userToken, x) \
                        for x in result]
            return result
        finally:
            self.pullContext()

    def getEveryCommentsUnder(self, virtualServerId, clientToken, userToken, sourceId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            objects = virtualServer.objects.values()
            result = [o.id for o in objects if o.sourceId == sourceId]
            doneIds = [sourceId]
            i = 0
            while i < len(result):
                id = result[i]
                i += 1
                if id in doneIds:
                    continue
                doneIds.append(id)
                result += [o.id for o in objects if o.sourceId == id]
            result = [self.getObjectXmlRpc(virtualServerId, clientToken, userToken, x) \
                        for x in result]
            return result
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getComments')


commentsServer = CommentsServer()


if __name__ == "__main__":
    commentsServer.launch(applicationName, applicationRole)
