#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Directories Server"""

__version__ = '$Revision: 1.32 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import base64
import time

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.DirectoriesCommon import *
import glasnost.common.faults as faults
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *
import glasnost.server.things as things

import os
from stat import *

applicationName = 'DirectoriesServer'
applicationRole = 'directories'
applicationTokens = None
dispatcher = None


class AdminDirectories(AdminServerMixin, AdminDirectoriesCommon):
    pass
register(AdminDirectories)


class Directory(ObjectServerMixin, DirectoryCommon):
    pass
register(Directory)


class DirectoryFile(things.ThingMixin, DirectoryFileCommon):
    def __init__(self):
        pass

register(DirectoryFile)


class DirectoriesServer(ObjectsServer):
    adminClassName = 'AdminDirectories'
    objectClassName = 'Directory'
    useAdminWritersSet = 1
    version = __version__

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('addObject')
        self.registerXmlRpcMethod('deletePath')
        self.registerXmlRpcMethod('getObject')
        self.registerXmlRpcMethod('modifyObject')
        self.registerXmlRpcMethod('renamePath')
        self.registerXmlRpcMethod('search')

    def __getAdmin(self, virtualServerId, clientToken, userToken,
                   objectId, path):
        """
        Give the list of Allowed Administrator for the spefied objectId and
        path.
        @param path: Path relatively to the rootPath in object
        @param objectId: Id of the object
        """
        adminLocal = None
        if path:
            realpath = self.__getRealPath(clientToken,userToken,objectId,path)
            gfile = getGlasnostFile(realpath)
            
            minLen = len(realpath) - len(path)        
        
            while len(realpath) >= minLen:
                if os.path.exists(gfile):
                    config = ConfigParser.ConfigParser()
                    config.readfp(open(gfile))
                    if config.has_option('MAIN', 'adminsSet'):
                        adminLocal = config.get('MAIN', 'adminsSet').split(',')
                        break
                realpath = os.path.dirname(realpath)
                gfile = getGlasnostFile(realpath)

        virtualServer = self.getVirtualServer(virtualServerId)
        if not virtualServer.objects.has_key(objectId):
            raise faults.MissingItem(objectId)
        object = virtualServer.objects[objectId]
        admin = object.adminsSet

        if adminLocal and admin:
            admin = admin + adminLocal
        elif adminLocal:
            admin = adminLocal

        return admin

    def __getWriter(self, virtualServerId, clientToken, userToken,
                   objectId, path):
        """
        Give the list of Writers for the spefied objectId and
        path.
        @param path: Path relatively to the rootPath in object
        @param objectId: Id of the object
        """
        writerLocal = None
        if path:
            realpath = self.__getRealPath(clientToken,userToken,objectId,path)
            gfile = getGlasnostFile(realpath)
            
            minLen = len(realpath) - len(path)        
        
            while len(realpath) > minLen:
                if os.path.exists(gfile):
                    config = ConfigParser.ConfigParser()
                    config.readfp(open(gfile))
                    if config.has_option('MAIN', 'writersSet'):
                        writerLocal=config.get('MAIN','writersSet').split(',')
                        break
                realpath = os.path.dirname(realpath)
                gfile = getGlasnostFile(realpath)

        virtualServer = self.getVirtualServer(virtualServerId)
        if not virtualServer.objects.has_key(objectId):
            raise faults.MissingItem(objectId)
        object = virtualServer.objects[objectId]
        writer = object.writersSet

        if writerLocal and writer:
            writer = writer + writerLocal
        elif writerLocal:
            writer = writerLocal

        return writer

    def __getReader(self, virtualServerId, clientToken, userToken,
                   objectId, path):
        """
        Give the list of Readers for the spefied objectId and
        path.
        @param path: Path relatively to the rootPath in object
        @param objectId: Id of the object
        """
        readerLocal = None
        if path:
            realpath = self.__getRealPath(clientToken,userToken,objectId,path)
            gfile = getGlasnostFile(realpath)

            minLen = len(realpath) - len(path)        
        
            while len(realpath) > minLen:
                if os.path.exists(gfile):
                    config = ConfigParser.ConfigParser()
                    config.readfp(open(gfile))
                    if config.has_option('MAIN', 'readersSet'):
                        readerLocal=config.get('MAIN','readersSet').split(',')
                        break
                realpath = os.path.dirname(realpath)
                gfile = getGlasnostFile(realpath)

        virtualServer = self.getVirtualServer(virtualServerId)
        if not virtualServer.objects.has_key(objectId):
            raise faults.MissingItem(objectId)
        object = virtualServer.objects[objectId]
        reader = object.readersSet

        if readerLocal and reader:
            reader = reader + readerLocal
        elif readerLocal:
            reader = readerLocal

        return reader

    def __isAdmin(self, virtualServerId, clientToken, userToken,
                  objectId, path):
        admin = self.__getAdmin(virtualServerId, clientToken, userToken,
                                objectId, path)
        if not admin:
            return 1
        return getProxyForServerRole('authentication').setContainsUser(admin)

    def __isWriter(self, virtualServerId, clientToken, userToken,
                  objectId, path):
        writer = self.__getWriter(virtualServerId, clientToken, userToken,
                                objectId, path)
        if not writer:
            return 1
        return getProxyForServerRole('authentication').setContainsUser(writer)

    def __isReader(self, virtualServerId, clientToken, userToken,
                  objectId, path):
        reader = self.__getReader(virtualServerId, clientToken, userToken,
                                  objectId, path)
        if not reader:
            return 1
        return getProxyForServerRole('authentication').setContainsUser(reader)


    def __idToObject(self, objectId):
        virtualServerId = self.computeVirtualServerId(objectId)
        virtualServer = self.getVirtualServer(virtualServerId)
        if not virtualServer.objects.has_key(objectId):
            raise faults.MissingItem(objectId)
        object = virtualServer.objects[objectId]
        return object

    def __getRealPath(self, clientToken, userToken, objectId, path):
        object = self.__idToObject(objectId)
 ##       object = ProxyTools.getObject(objectId)
        print objectId, object, object.rootPath, file_separator, path
        return os.path.normpath(object.rootPath + file_separator + path)

    def __addRepertory(self, object, realPath):
        """
        add repertory
        """
        path = realPath
        if os.path.exists(path):
            raise "All ready exist"
        os.makedirs(path)
        writeGlasnostFile(path, object)
        return object.getId() # We have to return something.

    def __addFile(self, object, realPath):
        """
        add file
        """
        path = realPath
        if os.path.exists(path):
            raise "All ready exist"
        file = open(path, "w")
        file.write(object.data)
        file.close()
        writeGlasnostFile(path, object)
        return object.getId() # We have to return something.

    def __getObject(self, virtualServerId, clientToken, userToken,
                    objectId, path, withData = 0):
        # check if the user can do that        
        if not self.canGetObject(virtualServerId, clientToken, userToken,
                                 objectId, path):
            raise faults.UserAccessDenied()
        result = DirectoryFile().init(objectId, path, withData)
        return result

    def __modifyRepertory(self, object, realPath):
        """
        modify repertory
        """
        path = realPath
        if os.path.exists(path):
            if not stat.S_ISDIR(os.stat(path)[stat.ST_MODE]):
                raise "All ready exist but is not repertory"
        else:
            raise "Repertory doesn't exist"
        writeGlasnostFile(path, object)
        return object.getId() # We have to return something.

    def __modifyFile(self, object, realPath):
        """
        modify file
        """
        path = realPath
        if os.path.exists(path):
            if not stat.S_ISREG(os.stat(path)[stat.ST_MODE]):
                raise "All ready exist but is not file"
        else:
            raise "File doesn't exist"
        if object.data:
            file = open(path, "w")
            file.write(object.data)
            file.close()
        writeGlasnostFile(path, object)
        return object.getId() # We have to return something.

    def deletePath(self, virtualServerId, clientToken, userToken,
                   objectId, path):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            # check if the user can do that
            if not self.canDeleteObject(
                virtualServerId, clientToken, userToken, objectId, path):
                raise faults.UserAccessDenied()

            path = self.__getRealPath(clientToken, userToken, objectId, path)
            pathGlasnostFile = getGlasnostFile(path)

            if not os.path.exists(path):
                return ''

            ## create .trash.glasnost if don't exist
            trash = self.__getRealPath(
                clientToken, userToken, objectId, file_separator + trashFile)
            if not os.path.exists(trash):
                os.makedirs(trash)

            filestat = os.stat(path)
            mode = filestat[stat.ST_MODE]
            if stat.S_ISDIR(mode):  # It's a directory
                trashedFile = trash + file_separator \
                              + path.split(file_separator)[-1]
                ## destroy it in the trash if exist
                if os.path.exists(trashedFile):
                    os.system("rm -fr -- '"+trashedFile+"'")

            ## don't delete move it in the trash
            os.system("mv --force --backup -- '"+path+"' '"+trash+"'")

            ## move glasnostFile too
            if os.path.exists(pathGlasnostFile):
                os.system("mv --force --backup -- '" + pathGlasnostFile
                          + "' '" + trash + "'")

            return '' # We have to return something.
        finally:
            self.pullContext()
    deletePath.useAutoDeserialize = 1
    
    def getObject(self, virtualServerId, clientToken, userToken,
                  objectId, path = '', withData = 0):
        """
        @param objectId:
        @type objectId: string

        @param path: path to object, if empty this method return
        Directory object if is not empty return DirectoryFile object
        @type path: string

        @param withData: return object with data if true
        @type withData: boolean

        @return: returned object is DirectoryRepertory or DirectoryFile
           case path is directory or file
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            if not path:
                result = self.getObjectCore(objectId)
            else:
                result = self.__getObject(
                    virtualServerId, clientToken, userToken, objectId, path,
                    withData)
            return result
        finally:
            self.pullContext()
    getObject.useAutoDeserialize = 1

    def renamePath(self, virtualServerId, clientToken, userToken,
                   objectId, oldPath, newPath):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            # check if the user can do that
            if not self.canModifyObject(virtualServerId, clientToken,
                                        userToken, objectId, path):
                raise faults.UserAccessDenied()

            oldPath = self.__getRealPath(
                clientToken, userToken, objectId, oldPath)
            newPath = self.__getRealPath(
                clientToken, userToken, objectId, newPath)
            os.rename(oldPath, newPath)
            return objectId # We have to return something.
        finally:
            self.pullContext()
    renamePath.useAutoDeserialize = 1

    def addObject(self, virtualServerId, clientToken, userToken, object):
        """
        add file or repertory in directory object
        @param objectImport: object to add
        @type objectImport: DirectoryFile
        """
        if isinstance(object, DirectoryCommon):
            return ObjectsServer.addObjectXmlRpc(self, virtualServerId,
                               clientToken, userToken, object.exportToXmlRpc())
        
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            # check if the user can do that
            if not self.canAddObject(virtualServerId, clientToken, userToken,
                                     object.getId(), object.getParentPath()):
                raise faults.UserAccessDenied()

            # write access is ok
            object.creationTime = object.modificationTime = time.time()
            realPath = self.__getRealPath(
                clientToken, userToken, object.getId(), object.path)
            if object.isFile():
                self.__addFile(object, realPath)
            else:
                self.__addRepertory(object, realPath)

            return object.getId()
        finally:
            self.pullContext()
    addObject.useAutoDeserialize = 1

    def modifyObject(self, virtualServerId, clientToken, userToken, object):
        """
        add file or repertory in directory object
        @param objectImport: object to add
        @type objectImport: DirectoryFile
        """
        if isinstance(object, DirectoryCommon):
            return ObjectsServer.modifyObjectXmlRpc(self, virtualServerId,
                               clientToken, userToken, object.exportToXmlRpc())
            
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)

            # check if the user can do that
            if not self.canModifyObject(
                virtualServerId, clientToken, userToken, object.getId(),
                object.path):
                raise faults.UserAccessDenied()
            # write access is ok
            object.modificationTime = time.time()
            realPath = self.__getRealPath(
                clientToken, userToken, object.getId(), object.path)
            if object.isFile():
                self.__modifyFile(object, realPath)
            else:
                self.__modifyRepertory(object, realPath)
            return object.getId()
        finally:
            self.pullContext()
    modifyObject.useAutoDeserialize = 1

    ## redefined methods
    def canAddObject(self, virtualServerId, clientToken, userToken,
                     objectId = None, path = file_separator):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            if not objectId:
                return ObjectsServer.canAddObject(self, virtualServerId,
                                                  clientToken, userToken)
            else:
                return self.__isAdmin(virtualServerId, clientToken, userToken,
                                      objectId, path) or \
                       self.__isWriter(virtualServerId, clientToken, userToken,
                                       objectId, path)
        finally:
            self.pullContext()
    canAddObject.useAutoDeserialize = 1

    def canDeleteObject(self, virtualServerId, clientToken, userToken,
                        objectId, path = file_separator):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            return self.__isAdmin(virtualServerId, clientToken, userToken,
                                  objectId, path)
        finally:
            self.pullContext()
    canDeleteObject.useAutoDeserialize = 1

    def canGetObject(self, virtualServerId, clientToken, userToken,
                     objectId, path = file_separator):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            return self.__isAdmin(virtualServerId, clientToken, userToken,
                                  objectId, path) or \
                   self.__isWriter(virtualServerId, clientToken, userToken,
                                   objectId, path) or \
                   self.__isReader(virtualServerId, clientToken, userToken,
                                   objectId, path)
        finally:
            self.pullContext()
    canGetObject.useAutoDeserialize = 1
    
    def canModifyObject(self, virtualServerId, clientToken, userToken,
                        objectId, path = file_separator):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            return self.__isAdmin(
                virtualServerId, clientToken, userToken, objectId, path) \
                or self.__isWriter(
                virtualServerId, clientToken, userToken, objectId, path)
        finally:
            self.pullContext()
    canModifyObject.useAutoDeserialize = 1
    
    def hasObject(self, virtualServerId, clientToken, userToken,
                  objectId, path = file_separator):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            if not path or path == file_separator:
                return ObjectsServer.hasObject(
                    self, virtualServerId, clientToken, userToken, objectId)
            else:
                path = self.__getRealPath(
                    clientToken, userToken, objectId, path)
                return os.path.exists(path)
        finally:
            self.pullContext()
    hasObject.useAutoDeserialize = 1

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version <= 1017000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)

    def search(self, virtualServerId, clientToken, userToken,
               objectId, name, description):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            root = self.getObjectCore(objectId)

            resultNames = os.popen(
                "find %s -name '*%s*'" % (root.rootPath, name))
            result = []
            for resultName in resultNames.readlines():
                resultName = resultName[:-1] # remove \n char
                # if is glasnost file or file in trash, don't take it
                if not isGlasnostSpecialFile(
                    resultName[len(root.rootPath)+1:]):
                    # if there is glasnost file and description is given,
                    # look for match
                    try:
                        if matchDescription(resultName, description):
                            result.append(self.__getObject(
                                virtualServerId, clientToken, userToken,
                                objectId, resultName[len(root.rootPath):])
                                          )
                    except: ## faults.UserAccessDenied or OSError
                        pass
            return result
        finally:
            self.pullContext()
    search.useAutoDeserialize = 1
             
directoriesServer = DirectoriesServer()


if __name__ == "__main__":
    directoriesServer.launch(applicationName, applicationRole)
