#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Dispatcher"""

__version__ = '$Revision: 1.11 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import whrandom

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *


applicationName = 'Dispatcher'
applicationRole = None


class DispatcherVirtualServer(VirtualServer):
    virtualServerIds = None
    applicationTokens = None
    serverAccessors = None

    def init(self, server, virtualServerId):
        VirtualServer.init(self, server, virtualServerId)
        self.virtualServerIds = {}
        self.applicationTokens = {}
        self.serverAccessors = {}


class Dispatcher(Server):
    VirtualServer = DispatcherVirtualServer
    randomGenerator = None
    useDataFile = 0
    version = __version__
    virtualServer = None

    def getApplicationId(self, clientToken, applicationToken):
        virtualServer = self.getVirtualServer(None)
        if not virtualServer.virtualServerIds.has_key(applicationToken):
            raise faults.UnknownApplicationToken(applicationToken)
        return virtualServer.virtualServerIds[applicationToken]

    def getApplicationToken(self, clientId):
        virtualServer = self.getVirtualServer(None)
        virtualServer.lock.acquire()
        if virtualServer.applicationTokens.has_key(clientId):
            clientToken = virtualServer.applicationTokens[clientId]
            virtualServer.lock.release()
        else:
            while 1:
                clientToken = str(self.randomGenerator.uniform(0.1, 1))[2:]
                if not virtualServer.virtualServerIds.has_key(clientToken):
                    break
            virtualServer.applicationTokens[clientId] = clientToken
            virtualServer.virtualServerIds[clientToken] = clientId
            virtualServer.lock.release()
            self.saveVirtualServer(virtualServer)
        return clientToken

    def getRegisteredRoles(self, forDispatcherId):
        virtualServer = self.getVirtualServer(None)
        return [x[x.rindex('/') + 1:]
                for x in virtualServer.serverAccessors.keys()
                if x.startswith(forDispatcherId) \
                    and len(x) > len(forDispatcherId)]

    def getServerAccessor(self, serverId):
        virtualServer = self.getVirtualServer(None)
        if not virtualServer.serverAccessors.has_key(serverId):
            raise faults.UnknownServerId(serverId)
        return virtualServer.serverAccessors[serverId]

    def getVirtualServer(self, virtualServerIdNotUsed):
        return self.virtualServer

    def init(self):
        self.randomGenerator = whrandom.whrandom()
        Server.init(self)

    def load(self):
        self.virtualServer = self.loadVirtualServer(None)

    def loadVirtualServer(self, virtualServerIdNotUsed):
        return Server.loadVirtualServer(self, None)

    def registerToDispatcher(self):
        virtualServer = self.getVirtualServer(None)
        for dispatcherId in self.dispatcherIds:
            virtualServer.serverAccessors[dispatcherId] = {
                'serverHostName': self.hostName,
                'serverPort': self.port
                }
        self.saveVirtualServer(virtualServer)

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getApplicationId')
        self.registerXmlRpcMethod('getApplicationToken')
        self.registerXmlRpcMethod('getRegisteredRoles')
        self.registerXmlRpcMethod('getServerAccessor')
        self.registerXmlRpcMethod('registerServer')
        self.registerXmlRpcMethod('unregisterServer')

    def registerServer(self, clientToken, serverHostName, serverPort):
        virtualServer = self.getVirtualServer(None)
        if not virtualServer.virtualServerIds.has_key(clientToken):
            raise faults.UnknownApplicationToken(clientToken)
        clientId = virtualServer.virtualServerIds[clientToken]
        if virtualServer.serverAccessors.has_key(clientId):
            # client previously registered, we check the hostname and
            # serverPort did not change
            serverAccessor = virtualServer.serverAccessors[clientId]
            if serverAccessor['serverHostName'] != serverHostName \
               or serverAccessor['serverPort'] != serverPort:
                raise faults.DuplicateServer(serverHostName, serverPort)

        autorizedHostNames = getConfig(
            'Security', 'RestrictRegisterTo', default = '')
        autorizedHostNames = autorizedHostNames.replace(',', ' ')
        autorizedHostNames = autorizedHostNames.strip().split()
        if autorizedHostNames and not serverHostName in autorizedHostNames:
            raise faults.ServerRegistrationDenied(serverHostName, serverPort)
            
        virtualServer.serverAccessors[clientId] = {
            'serverHostName': serverHostName,
            'serverPort': serverPort,
            }
        print 'Registering %s (for %s:%s)' % (
            clientId, serverHostName, serverPort)
        self.saveVirtualServer(virtualServer)
        return 0 # We have to return something.

    def unregisterServer(self, clientToken):
        virtualServer = self.getVirtualServer(None)
        if not virtualServer.virtualServerIds.has_key(clientToken):
            raise faults.UnknownApplicationToken(clientToken)
        clientId = virtualServer.virtualServerIds[clientToken]
        # TODO: raise something if unknown clientId
        del virtualServer.serverAccessors[clientId]
        self.saveVirtualServer(virtualServer)
        return 0 # We have to return something.

dispatcher = Dispatcher()


if __name__ == "__main__":
    dispatcher.launch(applicationName, applicationRole)
