#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


from __future__ import nested_scopes
    # lambdas are so much fun with nested scopes...


__doc__ = """Glasnost Events Server"""

__version__ = '$Revision: 1.40 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.EventsCommon import *
import glasnost.common.faults as faults
import glasnost.common.context as context
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.tools import getProxyForServerRole

applicationName = 'EventsServer'
applicationRole = 'events'
applicationTokens = None
dispatcher = None


class AdminEvents(AdminServerMixin, AdminEventsCommon):
    pass
register(AdminEvents)


class Event(ObjectServerMixin, EventCommon):
    def acquireNonCore(self, objectDirectoryPath = None,
                       dataDirectoryPath = None, parentSlot = None):
        ObjectServerMixin.acquireNonCore(
            self, objectDirectoryPath = objectDirectoryPath,
            dataDirectoryPath = dataDirectoryPath, parentSlot = parentSlot)
        if self.getServer().port is None:
            # The server is not running, so assume that Glasnost
            # is not running either (it happens for example during
            # ids conversion). Don't call the other servers. Do nothing.
            return        
        if self.operationId is None:
            self.getSlot('currency').setValue(None)
            self.getSlot('partnerId').setValue(None)
        else:
            try:
                operation = getProxyForServerRole(
                    'operations').getObject(self.operationId)
            except faults.MissingItem:
                self.getSlot('currency').setValue(None)
                self.getSlot('partnerId').setValue(None)
            else:
                self.getSlot('currency').setValue(
                    operation.getSlot('currency').getValue())
                self.getSlot('partnerId').setValue(
                    operation.getSlot('partnerId').getValue())

    def balanced(self):
        members = self.getSlot('membersSet').getValue()
        balance = reduce(lambda x,y:x[3]+y[3], members)
        if balance == 0:
            return 1
        return 0
    
    def canBeGottenByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return ObjectServerMixin.canBeGottenByClient(self, clientId) \
               or clientRole in ['records', 'workflows']

    def canBeModifiedByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return clientRole in ['records', 'workflows']

    def compute(self):
        members = self.getSlot('membersSet').getValue()
        balance = reduce(lambda x,y:x[3]+y[3], members)
        if balance == 0:
            isBalanced = 1
        else:
            isBalanced = 0
        self.getSlot('isBalanced').setValue(isBalanced)
        membersProxy = getProxyForServerRole('records')
        for memberId in members:
            membersProxy.computeObject(memberId)
        
    def modify(self, changes = None, givenSlotNames = None):
        ObjectServerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
        self.compute()
register(Event)


class EventsServer(ObjectsServer):
    adminClassName = 'AdminEvents'
    objectClassName = 'Event'
    version = __version__

    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)
            object.getSlot('isBalanced').setValue(object.balanced())
            # User must be an admin or (a writer and a writer of this object))
            if not self.isAdmin(virtualServerId, clientToken, userToken) \
                   and not (self.useAdminWritersSet
                and authenticationProxy.setContainsUser(
                self.getAdminCore().writersSet)
                and authenticationProxy.setContainsUser(object.writersSet)):
                clientId = getApplicationId(clientToken)
                if not object.canBeCreatedByClient(clientId):
                    raise faults.UserAccessDenied()
            object.checkAddIsPossible()
            object.setAutomaticalSlots()
            object.id = self.getNextId()
            virtualServer.objects[object.id] = object
            object.saveNonCore()
            object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            return object.id
        finally:
            self.pullContext()
    addObjectXmlRpc.allowSubscriptionAfter = 1

    def computeObject(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            object.compute()
            self.saveVirtualServer(virtualServer)
            return objectId # We have to return something.
        finally:
            self.pullContext()
                    
    def deleteObject(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            if not self.isAdmin(virtualServerId, clientToken, userToken) \
               and not authenticationProxy.setContainsUser(object.writersSet):
                raise faults.UserAccessDenied()
            memberIds = object.getSlot('membersSet').getValue()
            if memberIds:
                for memberId in memberIds:
                    try:
                        getProxyForServerRole('records').deleteObject(memberId)
                    except:
                        raise "Unable to delete object %s" % memberId
            del virtualServer.objects[objectId]
            self.saveVirtualServer(virtualServer)
            invalidateValue(objectId)
            return objectId # We have to return something.
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('computeObject')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 6000:
            for id, object in virtualServer.objects.items():
                year, month, day = date[:4], date[4:6], date[6:8]
                date = time.mktime((year, month, day, 0, 0, 0, 0, 0, 0))
                setattr(object, 'date', date)
            changed = 1
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1016000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


eventsServer = EventsServer()


if __name__ == "__main__":
    eventsServer.launch(applicationName, applicationRole)
