#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Function Calls Server"""

__version__ = '$Revision: 1.22 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.context as context
import glasnost.common.faults as faults
from glasnost.common.FunctionCallsCommon import *
import glasnost.common.tools_new as commonTools

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.DispatcherProxy import getApplicationId


applicationName = 'FunctionCallsServer'
applicationRole = 'functioncalls'
applicationTokens = None
dispatcher = None


class AdminFunctionCalls(AdminServerMixin, AdminFunctionCallsCommon):
    pass
register(AdminFunctionCalls)


class FunctionCall(ObjectServerMixin, FunctionCallCommon):
    def eval(self):
        functionDeclaration = self.getFunctionDeclaration()
        if functionDeclaration is None:
            result = None
        else:
            from glasnost.proxy.DispatcherProxy import callServer, \
                 getApplicationToken
            serverRole = commonTools.extractRole(self.serverId)
            userToken = context.getVar('userToken', default = '')
            arguments = []
            parameters  = functionDeclaration.parameters[:]
            parameters = parameters[:]
            for name in ('virtualServerId', 'clientToken', 'userToken'):
                if len(parameters) > 0 and parameters[0].name == name:
                    if name == 'virtualServerId':
                        argument = self.serverId
                    elif name == 'clientToken':
                        argument = getApplicationToken()
                    else: # name == 'userToken'
                        argument = userToken
                    arguments.append(argument)
                    del parameters[0]
            if self.arguments is not None:
                arguments += self.arguments
            try:
                result = callServer(
                    self.serverId, self.functionName, arguments)
            except:
                result = None
        if result is None:
            if self.result is not None:
                del self.result
        else:
            self.result = result
        return self.result

    def modify(self, changes, givenSlotNames = None):
##         if self.serverId != changes.serverId \
##            or self.functionName != changes.functionName:
        if self.functionDeclaration is not None:
            del self.functionDeclaration
        if self.arguments is not None:
            del self.arguments
        if self.result is not None:
            del self.result
        ObjectServerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
register(FunctionCall)


class FunctionCallsServer(ObjectsServer):
    adminClassName = 'AdminFunctionCalls'
    objectClassName = 'FunctionCall'
    version = __version__

    def evaluateObject(self, virtualServerId, clientToken, userToken,
                       objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            object.acquireNonCore()
            try:
                if not object.canBeModified():
                    raise faults.ReadOnlyObject()
                if not self.canGetObject(
                    virtualServerId, clientToken, userToken, objectId):
                    raise faults.UserAccessDenied()
                object.eval()
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            return object.version
        finally:
            self.pullContext()

    def getObjectEvaluation(self, virtualServerId, clientToken, userToken,
                            objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            object.acquireNonCore()
            try:
                if not object.canBeModified():
                    raise faults.ReadOnlyObject()
                if not self.canGetObject(
                    virtualServerId, clientToken, userToken, objectId):
                    raise faults.UserAccessDenied()
                result = object.eval()
                resultSlot = object.getSlot('result')
                resultKind = resultSlot.getKind()
                resultKindExport = resultKind.exportToXmlRpc()
                resultExport = resultKind.exportValueToXmlRpc(
                    resultSlot, result)
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            return [resultKindExport, resultExport]
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('evaluateObject')
        self.registerXmlRpcMethod('getObjectEvaluation')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version <= 1007000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


functionCallsServer = FunctionCallsServer()


if __name__ == "__main__":
    functionCallsServer.launch(applicationName, applicationRole)
