#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Authentication Server"""

__version__ = '$Revision: 1.13 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import getopt
import os
import signal
import sys
import traceback

try:
    from M2Crypto import SSL
except ImportError:
    SSL = None

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.proxy.DispatcherProxy import callServer

from glasnost.server.tools import *
from glasnost.server.xmlrpcServer import xmlrpcServer


dispatcher = None


def init_context(certfile, cafile, verify_depth=10):
        if not SSL:
            return None
        ctx = SSL.Context('sslv2')
        ctx.load_cert(certfile)
        ctx.load_client_ca(cafile)
        ctx.load_verify_info(cafile)
        ctx.set_verify(SSL.verify_none, verify_depth)
        ctx.set_allow_unknown_ca(1)
        ctx.set_session_id_ctx('xmlrpc')
        ctx.set_info_callback()
        return ctx


class Gateway:
    applicationName = 'Gateway'
    hostName = None
    port = None
    version = __version__
    xmlRpcServer = None

    def callGateway(self, serverId, functionName, arguments):
        return callServer(serverId, functionName, arguments)

    def getServerAccessor(self, serverId):
        return {
            'isGateway': 1,
            'serverHostName': self.hostName,
            'serverPort': self.port,
            }
        
    def handleSignal(self, signalNumber, stackFrame):
        if signalNumber == signal.SIGINT:
            raise KeyboardInterrupt
        if signalNumber == signal.SIGHUP:
            self.setLogFile()
            return
        if signalNumber == signal.SIGTERM:
            print 'Received signal SIGTERM: exiting.'
            os._exit(0)

    def launch(self):
        try:
            opts, args = getopt.getopt(
                sys.argv[1:],
                'd:hi:s:x:y:',
                ['destination=', 'export=', 'help', 'import=', 'source=',
                 'system='])
        except getopt.GetoptError:
            self.usage()
            sys.exit(2)
        if len(args) != 0:
            self.usage()
            sys.exit(2)
        destinationDispatcherId = None
        doExport = 0
        doImport = 0
        doSystem = 0
        exportDirectoryPath = None
        importDirectoryPath = None
        sourceDispatcherId = None
        for o, a in opts:
            if o in ('-d', '--destination'):
                destinationDispatcherId = a
                if destinationDispatcherId[-1] == '/':
                    destinationDispatcherId = destinationDispatcherId[:-1]
            if o in ('-h', '--help'):
                usage()
                sys.exit()
            if o in ('-i', '--import'):
                importDirectoryPath = a
                doImport = 1
            if o in ('-s', '--source'):
                sourceDispatcherId = a
                if sourceDispatcherId[-1] == '/':
                    sourceDispatcherId = sourceDispatcherId[:-1]
            if o in ('-x', '--export'):
                exportDirectoryPath = a
                doExport = 1
            if o in ('-y', '--system'):
                doSystem = 1
        if doImport or doExport or doSystem:
            sys.exit()
        if sourceDispatcherId and destinationDispatcherId:
            sys.exit()
        goDaemon()
        self.setLogFile()
        self.startXmlRpcServer()
        self.run()

    def registerXmlRpcMethod(self, methodName, method = None):
        if method is None:
            method = getattr(self, methodName)
        self.xmlRpcServer.add_method(methodName, method)

    def registerXmlRpcMethods(self):
        self.registerXmlRpcMethod('callGateway')
        self.registerXmlRpcMethod('getServerAccessor')

    def run(self):
        signal.signal(signal.SIGINT, self.handleSignal)
        signal.signal(signal.SIGHUP, self.handleSignal)
        signal.signal(signal.SIGTERM, self.handleSignal)
        while 1:
            try:
                self.xmlRpcServer.serve_forever()
            except KeyboardInterrupt:
                break
            except:
                traceback.print_exc()

    def setLogFile(self):
        logFilePath = '%s/%s.log' % (
            getConfig('Misc', 'LogFilesDir', default='/var/log/glasnost'),
            self.applicationName)
        logFile = open(logFilePath, 'a', 1)
        sys.stdout = sys.__stdout__ = logFile
        sys.stderr = sys.__stderr__ = logFile

    def startXmlRpcServer(self):
        self.hostName = getConfig(self.applicationName, 'ServerHostName')
        self.port = eval(getConfig(self.applicationName, 'ServerPort'), {})
        
        ctx = init_context(
            '/etc/%s/server.pem' % glasnost.applicationName,
            '/etc/%s/ca.pem' % glasnost.applicationName)
        self.xmlRpcServer = xmlrpcServer(('', self.port), sslContext = ctx)
        self.registerXmlRpcMethods()
        # Don't call registerServer, because the gateway can only be called
        # from the outside, not from the other Glasnost servers.

    def usage(self):
        print 'Usage: %s ### FIXME: To Do' % self.applicationName


gateway = Gateway()


if __name__ == "__main__":
    gateway.launch()
