#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Groups Server"""

__version__ = '$Revision: 1.33 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import time

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.context as context
from glasnost.common.GroupsCommon import *
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.DispatcherProxy import getApplicationId
from glasnost.proxy.tools import getProxy


applicationName = 'GroupsServer'
applicationRole = 'groups'
applicationTokens = None
dispatcher = None


### FIXME: To remove ASAP.
mainModule = sys.modules['__main__']


class AdminGroups(AdminServerMixin, AdminGroupsCommon):
    pass
register(AdminGroups)


class Group(ObjectServerMixin, GroupCommon):
    def contains(self, objectId):
        """Contain 'objectId' as member and do not contain it as non-member
        """
        if self.containsAsNonMember(objectId, [], []):
            return 0
        return self.containsAsMember(objectId, [], [])

    def containsAsMember(self, objectId, encounteredForMember,
                         encounteredForNonMember):
        if self.id in encounteredForMember:
            return 0
        encounteredForMember.append(self.id)
        if self.id == objectId:
            return 1
        if self.membersSet is not None:
            for memberId in self.membersSet:
                if memberId == objectId:
                    return 1
            for memberId in self.membersSet:
                if not isGroupId(memberId):
                    continue
                proxy = getProxy(memberId)
                if proxy.hasObject(memberId) and proxy.objectContainsAsMember(
                    memberId, objectId, encounteredForMember,
                    encounteredForNonMember):
                    return 1
        if self.nonMembersSet is not None:
            for nonMemberId in self.nonMembersSet:
                if not isGroupId(nonMemberId):
                    continue
                proxy = getProxy(nonMemberId)
                if proxy.hasObject(nonMemberId) \
                       and proxy.objectContainsAsNonMember(
                    nonMemberId, objectId, encounteredForMember,
                    encounteredForNonMember):
                    return 1
        return 0

    def containsAsNonMember(self, objectId, encounteredForMember,
                            encounteredForNonMember):
        if self.id in encounteredForNonMember:
            return 0
        encounteredForNonMember.append(self.id)
        if self.id == objectId:
            return 1
        if self.membersSet is not None:
            for memberId in self.membersSet:
                if not isGroupId(memberId):
                    continue
                proxy = getProxy(memberId)
                if proxy.hasObject(memberId) \
                       and proxy.objectContainsAsNonMember(
                    memberId, objectId, encounteredForMember,
                    encounteredForNonMember):
                    return 1
        if self.nonMembersSet is not None:
            for nonMemberId in self.nonMembersSet:
                if nonMemberId == objectId:
                    return 1
            for nonMemberId in self.nonMembersSet:
                if not isGroupId(nonMemberId):
                    continue
                proxy = getProxy(nonMemberId)
                if proxy.hasObject(nonMemberId) \
                       and proxy.objectContainsAsMember(
                    nonMemberId, objectId, encounteredForMember,
                    encounteredForNonMember):
                    return 1
        return 0

    def getContainedIds(self, serverRoles):
        """Get contained ids in current (group) object
        """
        containedMemberIds = self.getContainedMemberIds(serverRoles, [], [])
        containedNonMemberIds = self.getContainedNonMemberIds(
            serverRoles, [], [])
        containedIds = []
        for containedMemberId in containedMemberIds:
            if not containedMemberId in containedNonMemberIds:
                containedIds.append(containedMemberId)
        return containedIds

    def getContainedMemberIds(self, serverRoles, encounteredForMember,
                              encounteredForNonMember):
        if self.id in encounteredForMember:
            return []
        encounteredForMember.append(self.id)
        containedMemberIds = []
        if self.membersSet is not None:
            for memberId in self.membersSet:
                proxy = getProxy(memberId)
                if isGroupId(memberId) and proxy.hasObject(memberId):
                    newMemberIds = proxy.getObjectContainedMemberIds(
                        memberId, serverRoles, encounteredForMember,
                        encounteredForNonMember)
                else:
                    memberServerRole = commonTools.extractRole(memberId)
                    if not serverRoles or memberServerRole in serverRoles:
                        newMemberIds = [memberId]
                    else:
                        newMemberIds = []
                for newMemberId in newMemberIds:
                    if newMemberId not in containedMemberIds:
                        containedMemberIds.append(newMemberId)
        if self.nonMembersSet is not None:
            for nonMemberId in self.nonMembersSet:
                proxy = getProxy(nonMemberId)
                if isGroupId(nonMemberId) and proxy.hasObject(nonMemberId):
                    newMemberIds = proxy.getObjectContainedNonMemberIds(
                        nonMemberId, serverRoles, encounteredForMember,
                        encounteredForNonMember)
                    for newMemberId in newMemberIds:
                        if newMemberId not in containedMemberIds:
                            containedMemberIds.append(newMemberId)
        return containedMemberIds

    def getContainedNonMemberIds(self, serverRoles, encounteredForMember,
                                 encounteredForNonMember):
        if self.id in encounteredForNonMember:
            return []
        encounteredForNonMember.append(self.id)
        containedNonMemberIds = []
        if self.membersSet is not None:
            for memberId in self.membersSet:
                proxy = getProxy(memberId)
                if isGroupId(memberId) and proxy.hasObject(memberId):
                    newNonMemberIds = proxy.getObjectContainedNonMemberIds(
                        memberId, serverRoles, encounteredForMember,
                        encounteredForNonMember)
                    for newNonMemberId in newNonMemberIds:
                        if newNonMemberId not in containedNonMemberIds:
                            containedNonMemberIds.append(newNonMemberId)
        if self.nonMembersSet is not None:
            for nonMemberId in self.nonMembersSet:
                proxy = getProxy(nonMemberId)
                if isGroupId(nonMemberId) and proxy.hasObject(nonMemberId):
                    newNonMemberIds = proxy.getObjectContainedMemberIds(
                        nonMemberId, serverRoles, encounteredForMember,
                        encounteredForNonMember)
                else:
                    nonMemberServerRole = commonTools.extractRole(nonMemberId)
                    if not serverRoles or nonMemberServerRole in serverRoles:
                        newNonMemberIds = [nonMemberId]
                    else:
                        newNonMemberIds = []
                for newNonMemberId in newNonMemberIds:
                    if newNonMemberId not in containedNonMemberIds:
                        containedNonMemberIds.append(newNonMemberId)
        return containedNonMemberIds
register(Group)


### FIXME: To remove later.
GroupDelta = Group


class GroupsServer(ObjectsServer):
    adminClassName = 'AdminGroups'
    objectClassName = 'Group'
    version = __version__

    def addObjectMember(self, virtualServerId, clientToken, userToken,
                        objectId, memberId):
        """ Add member 'memberId' to group 'objectId'
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServerId = self.computeVirtualServerId(objectId)
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            if not self.isAdmin(virtualServerId, clientToken, userToken):
                clientId = getApplicationId(clientToken)
                clientNameAndPort, clientRole = splitApplicationId(clientId)
                if clientRole != 'people':
                    raise faults.UserAccessDenied()
            object = virtualServer.objects[objectId]
            object.acquireNonCore()
            try:
                if not object.canBeModified():
                    raise faults.ReadOnlyObject()
                object.version += 1
                if object.membersSet is None:
                    object.membersSet = []
                object.membersSet.append(memberId)
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(objectId)
            return object.version
        finally:
            self.pullContext()

    def getObjectContainedIds(self, virtualServerId, clientToken, userToken,
                              objectId, serverRoles):
        """ Return items contained in group 'objectId'
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            return object.getContainedIds(serverRoles)
        finally:
            self.pullContext()

    def getObjectContainedMemberIds(self, virtualServerId, clientToken,
                                    userToken, objectId, serverRoles,
                                    encounteredForMember,
                                    encounteredForNonMember):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            return object.getContainedMemberIds(
                serverRoles, encounteredForMember, encounteredForNonMember)
        finally:
            self.pullContext()

    def getObjectContainedNonMemberIds(self, virtualServerId, clientToken,
                                       userToken, objectId, serverRoles,
                                       encounteredForMember,
                                       encounteredForNonMember):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            return object.getContainedNonMemberIds(
                serverRoles, encounteredForMember, encounteredForNonMember)
        finally:
            self.pullContext()

    def getSetContainedIds(self, virtualServerId, clientToken, userToken, set,
                           serverRoles):
        """ Return ids of items contained in at least one of the groups in
            'set' 
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            result = []
            if set is not None:
                for itemId in set:
                    proxy = getProxy(itemId)
                    if isGroupId(itemId) and proxy.hasObject(itemId):
                        containedIds = proxy.getObjectContainedIds(
                            itemId, serverRoles)
                    else:
                        itemServerRole = commonTools.extractRole(itemId)
                        if not serverRoles or itemServerRole in serverRoles:
                            containedIds = [itemId]
                        else:
                            containedIds = []
                    for containedId in containedIds:
                        if not containedId in result:
                            result.append(containedId)
            return result
        finally:
            self.pullContext()

    def getSetDeltaContainedIds(self, virtualServerId, clientToken, userToken,
                                membersSet, nonMembersSet, serverRoles):
        """ Return ids of items contained
        in at least one of the groups in 'membersSet'
        but in none of the groups in 'nonMembersSet'
        
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            object = Group()
            object.membersSet = membersSet
            object.nonMembersSet = nonMembersSet
            result = object.getContainedIds(serverRoles)
            return result
        finally:
            self.pullContext()

    def objectContains(self, virtualServerId, clientToken, userToken, objectId,
                       containedId):
        """ This group 'objectId' contains item 'containedId'
        """ 
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            result = object.contains(containedId)
            return result
        finally:
            self.pullContext()

    def objectContainsAsMember(self, virtualServerId, clientToken, userToken,
                               objectId, containedId, encounteredForMember,
                               encounteredForNonMember):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            result = object.containsAsMember(
                containedId, encounteredForMember, encounteredForNonMember)
            return result
        finally:
            self.pullContext()

    def objectContainsAsNonMember(self, virtualServerId, clientToken,
                                  userToken, objectId, containedId,
                                  encounteredForMember,
                                  encounteredForNonMember):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            result = object.containsAsNonMember(
                containedId, encounteredForMember, encounteredForNonMember)
            return result
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('addObjectMember')
        self.registerXmlRpcMethod('getObjectContainedIds')
        self.registerXmlRpcMethod('getObjectContainedMemberIds')
        self.registerXmlRpcMethod('getObjectContainedNonMemberIds')
        self.registerXmlRpcMethod('getSetContainedIds')
        self.registerXmlRpcMethod('getSetDeltaContainedIds')
        self.registerXmlRpcMethod('objectContains')
        self.registerXmlRpcMethod('objectContainsAsMember')
        self.registerXmlRpcMethod('objectContainsAsNonMember')
        self.registerXmlRpcMethod('setContains')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 4000:
            changed = virtualServer.admin.repair(4000) or changed
            for id, object in virtualServer.objects.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    del virtualServer.objects[id]
                    virtualServer.objects[newId] = object
                changed = object.repair(4000) or changed
                if not object.__dict__.has_key('language'):
                    changed = 1
                    object.language = 'fr'
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1013000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)

    def setContains(self, virtualServerId, clientToken, userToken, set,
                    objectId):
        """ Item 'objectId' is contained by at least one of the groups in 'set'
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            if set is not None:
                for itemId in set:
                    if itemId == objectId:
                        return 1
                    if isGroupId(itemId):
                        proxy = getProxy(itemId)
                        if proxy.hasObject(itemId) \
                               and proxy.objectContains(itemId, objectId):
                            return 1
            return 0
        finally:
            self.pullContext()


groupsServer = GroupsServer()


def isGroupId(id):
    return context.getVar('serverRole') == commonTools.extractRole(
        id)


if __name__ == "__main__":
    groupsServer.launch(applicationName, applicationRole)
