#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Ledgers Server"""

__version__ = '$Revision: 1.21 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.LedgersCommon import *

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.DispatcherProxy import getApplicationId


applicationName = 'LedgersServer'
applicationRole = 'ledgers'
applicationTokens = None
dispatcher = None


class AdminLedgers(AdminServerMixin, AdminLedgersCommon):
    pass
register(AdminLedgers)


class Ledger(ObjectServerMixin, LedgerCommon): 
    def canBeGottenByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return ObjectServerMixin.canBeGottenByClient(self, clientId) \
               or clientRole == 'operations'

    def modify(self, changes, givenSlotNames = None):
        ObjectServerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
        userId = getProxyForServerRole('authentication').getUserId()
        self.lastEditorId = userId
register(Ledger)


class LedgersServer(ObjectsServer):
    adminClassName = 'AdminLedgers'
    objectClassName = 'Ledger'
    version = __version__

    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            userId = getProxyForServerRole('authentication').getUserId()
            objectImport['lastEditorId'] = userId
            return ObjectsServer.addObjectXmlRpc(
                self, virtualServerId, clientToken, userToken, objectImport)
        finally:
            self.pullContext()

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1009000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


ledgersServer = LedgersServer()


if __name__ == "__main__":
    ledgersServer.launch(applicationName, applicationRole)
