#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Monitorings Server"""

__version__ = '$Revision: 1.12 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *


applicationName = 'MonitoringsServer'
applicationRole = 'monitorings'
applicationTokens = None
dispatcher = None


class MonitoringsVirtualServer(VirtualServer):
    pass


class MonitoringsServer(Server):
    VirtualServer = MonitoringsVirtualServer
    version = __version__

    functionDeclarations = {}

    def getFunctionDeclarationsXmlRpc(self, virtualServerId, clientToken,
                                      userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            exportedDeclarations = {}
            for name, declaration in self.functionDeclarations.items():
                exportedDeclarations[name] = declaration.exportToXmlRpc()
            return exportedDeclarations
        finally:
            self.pullContext()
            
    def getProcessorNameXmlRpc(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            result = utf8('Pentium ///')
            return result
        finally:
            self.pullContext()

    def getSomethingStupid(self, virtualServerId, clientToken, userToken,
                           boolean, id, integer):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            result = boolean
            return result
        finally:
            self.pullContext()

    def getTemperature(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            result = 19.3
            return result
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getFunctionDeclarations',
                                  self.getFunctionDeclarationsXmlRpc)
        self.registerXmlRpcMethod('getProcessorName',
                                  self.getProcessorNameXmlRpc)
        self.registerXmlRpcMethod('getSomethingStupid')
        self.registerXmlRpcMethod('getTemperature')

        import glasnost.server.functions as functions
        import glasnost.server.kinds as kinds

        # Declare function getSomethingStupid.
        functionDeclaration = functions.FunctionDeclaration()
        self.functionDeclarations['getSomethingStupid'] = functionDeclaration
        functionDeclaration.name = 'getSomethingStupid'

        parameters = []
        functionDeclaration.parameters = parameters

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'virtualServerId'
        parameter.kind = kinds.ServerId()
        parameter.kind.serverRoles = [self.applicationRole]
        parameter.kind.isPublicForXmlRpc = 1

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'clientToken'
        parameter.kind = kinds.ApplicationToken()
        parameter.kind.isPublicForXmlRpc = 1

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'userToken'
        parameter.kind = kinds.UserToken()
        parameter.kind.isPublicForXmlRpc = 1

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'boolean'
        parameter.kind = kinds.Boolean()
        parameter.kind.isPublicForXmlRpc = 1

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'id'
        parameter.kind = kinds.Id()
        parameter.kind.isPublicForXmlRpc = 1

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'integer'
        parameter.kind = kinds.Integer()
        parameter.kind.isPublicForXmlRpc = 1

        result = functions.Parameter()
        functionDeclaration.result = result
        result.kind = kinds.Boolean()
        result.kind.isPublicForXmlRpc = 1

        # Declare function getTemperature.
        functionDeclaration = functions.FunctionDeclaration()
        self.functionDeclarations['getTemperature'] = functionDeclaration
        functionDeclaration.name = 'getTemperature'

        parameters = []
        functionDeclaration.parameters = parameters

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'virtualServerId'
        parameter.kind = kinds.ServerId()
        parameter.kind.serverRoles = [self.applicationRole]
        parameter.kind.isPublicForXmlRpc = 1

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'clientToken'
        parameter.kind = kinds.ApplicationToken()
        parameter.kind.isPublicForXmlRpc = 1

        parameter = functions.Parameter()
        parameters.append(parameter)
        parameter.name = 'userToken'
        parameter.kind = kinds.UserToken()
        parameter.kind.isPublicForXmlRpc = 1

        result = functions.Parameter()
        functionDeclaration.result = result
        result.kind = kinds.Float()
        result.kind.isPublicForXmlRpc = 1


monitoringsServer = MonitoringsServer()


if __name__ == "__main__":
    monitoringsServer.launch(applicationName, applicationRole)
