#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Operations Server"""

__version__ = '$Revision: 1.30 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.OperationsCommon import *
import glasnost.common.context as context

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.DispatcherProxy import getApplicationId
from glasnost.proxy.tools import getProxy

applicationName = 'OperationsServer'
applicationRole = 'operations'
applicationTokens = None
dispatcher = None


class AdminOperations(AdminServerMixin, AdminOperationsCommon):
    pass
register(AdminOperations)


class Operation(ObjectServerMixin, OperationCommon):
    def acquireNonCore(self, objectDirectoryPath = None,
                       dataDirectoryPath = None, parentSlot = None):
        ObjectServerMixin.acquireNonCore(
            self, objectDirectoryPath = objectDirectoryPath,
            dataDirectoryPath = dataDirectoryPath, parentSlot = parentSlot)
        if self.getServer().port is None:
            # The server is not running, so assume that Glasnost
            # is not running either (it happens for example during
            # ids conversion). Don't call the other servers. Do nothing.
            return        
        if self.ledgerId is None:
            self.getSlot('currency').setValue(None)
        else:
            try:
                ledger = getProxyForServerRole('ledgers').getObject(
                    self.ledgerId)
            except faults.MissingItem:
                self.getSlot('currency').setValue(None)
            else:
                self.getSlot('currency').setValue(
                    ledger.getSlot('currency').getValue())

    def balanced(self):
        eventsProxy = getProxyForServerRole('events')
        events = [eventsProxy.getObject(eventId)
                  for eventId in self.membersSet]
        result = 1
        for e in events:
            if not e.balanced():
                result = 0
                break
        return result

    def canBeGottenByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return ObjectServerMixin.canBeGottenByClient(self, clientId) \
               or clientRole == 'events'

    def canBeModifiedByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return clientRole in ['events', 'workflows']

    def modify(self, changes, givenSlotNames = None):
        ObjectServerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
        userId = getProxyForServerRole('authentication').getUserId()
        self.lastEditorId = userId
register(Operation)


class OperationsServer(ObjectsServer):
    adminClassName = 'AdminOperations'
    objectClassName = 'Operation'
    version = __version__

    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            userId = getProxyForServerRole('authentication').getUserId()
            objectImport['lastEditorId'] = userId
            return ObjectsServer.addObjectXmlRpc(
                self, virtualServerId, clientToken, userToken, objectImport)
        finally:
            self.pullContext()

    def deleteObject(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            clientId = getApplicationId(clientToken)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            if not self.canDeleteObject(
                virtualServerId, clientToken, userToken, objectId):
                if not object.canBeDeletedByClient(clientId):
                    raise faults.UserAccessDenied()
            memberIds = object.getSlot('membersSet').getValue()
            if memberIds:
                for memberId in memberIds:
                    try:
                        getProxyForServerRole('events').deleteObject(memberId)
                    except:
                        raise "Unable to delete object %s" % memberId
            del virtualServer.objects[objectId]
            object.removeNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(objectId)
            return objectId # We have to return something.
        finally:
            self.pullContext()

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1013000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)

    
operationsServer = OperationsServer()


if __name__ == "__main__":
    operationsServer.launch(applicationName, applicationRole)
