#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost PageNames Server"""

__version__ = '$Revision: 1.23 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults
from glasnost.common.PageNamesCommon import *
import glasnost.common.context as context

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *


applicationName = 'PageNamesServer'
applicationRole = 'pagenames'
applicationTokens = None
dispatcher = None


class AdminPageNames(AdminServerMixin, AdminPageNamesCommon):
    pass
register(AdminPageNames)


class PageName(ObjectServerMixin, PageNameCommon):
    def checkAddIsPossible(self):
        ObjectServerMixin.checkAddIsPossible(self)
        virtualServerId = context.getVar('applicationId')
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        if self.name is not None:
            for pageName in virtualServer.objects.values():
                if pageName.name == self.name:
                    raise faults.DuplicateValue('name', self.name)

    def checkModifyIsPossible(self, changes, givenSlotNames = None):
        ObjectServerMixin.checkModifyIsPossible(
            self, changes, givenSlotNames = givenSlotNames)
        virtualServerId = context.getVar('applicationId')
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        if (not givenSlotNames or 'name' in givenSlotNames) \
               and changes.name != self.name and changes.name is not None:
            for pageName in virtualServer.objects.values():
                if pageName.name == changes.name:
                    raise faults.DuplicateValue('name', changes.name)
register(PageName)


class PageNamesServer(ObjectsServer):
    adminClassName = 'AdminPageNames'
    objectClassName = 'PageName'
    version = __version__

    def getIdByName(self, virtualServerId, clientToken, userToken, name):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            for objectId, object in virtualServer.objects.items():
                if object.name == name:
                    return object.mappedId
            # Id not found. Look for it into the system virtual server.
            virtualServerId = 'glasnost://system/%s' % self.applicationRole
            virtualServer = self.getVirtualServer(virtualServerId)
            for objectId, object in virtualServer.objects.items():
                if object.name == name:
                    return object.mappedId
            return '' # or should it raise sth ?
        finally:
            self.pullContext()

    def getNameByMappedId(self, virtualServerId, clientToken, userToken,
                          mappedId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            for objectId, object in virtualServer.objects.items():
                if object.mappedId == mappedId:
                    return object.name
            # Name not found. Look for it into the system virtual server.
            virtualServerId = 'glasnost://system/%s' % self.applicationRole
            virtualServer = self.getVirtualServer(virtualServerId)
            for objectId, object in virtualServer.objects.items():
                if object.mappedId == mappedId:
                    return object.name
            return '' # or should it raise sth ?
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getIdByName')
        self.registerXmlRpcMethod('getNameByMappedId')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version <= 1008000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


pageNamesServer = PageNamesServer()


if __name__ == "__main__":
    pageNamesServer.launch(applicationName, applicationRole)
