#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost People Server"""

__version__ = '$Revision: 1.65 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import copy
import os
import time

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.context as context
from glasnost.common.PeopleCommon import *
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer, ObjectsVirtualServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.DispatcherProxy import getApplicationId
from glasnost.proxy.GroupsProxy import getSetContainedIds
from glasnost.proxy.tools import getProxy


applicationName = 'PeopleServer'
applicationRole = 'people'
applicationTokens = None
dispatcher = None


### FIXME: To remove ASAP.
mainModule = sys.modules['__main__']


class AdminPeople(AdminServerMixin, AdminPeopleCommon):
    pass
register(AdminPeople)


class Person(ObjectServerMixin, PersonCommon):
    def abstainForVote(self, electionId):
        if self.voteTokens is not None and self.voteTokens.has_key(electionId):
            del self.voteTokens[electionId]
            if len(self.voteTokens) == 0:
                del self.voteTokens

    def checkAddIsPossible(self):
        ObjectServerMixin.checkAddIsPossible(self)
        virtualServerId = context.getVar('applicationId')
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        if self.login is not None:
            for person in virtualServer.objects.values():
                if person.login == self.login:
                    raise faults.DuplicateLogin(self.login)
        fullName = self.getFullName()
        if fullName:
            for person in virtualServer.objects.values():
                if person.getFullName() == fullName:
                    raise faults.DuplicateFullName(fullName)
        if self.email:
            for person in virtualServer.objects.values():
                if person.email == self.email:
                    raise faults.DuplicateEmail(self.email)
        if self.fingerprint:
            for person in virtualServer.objects.values():
                if person.fingerprint == self.fingerprint:
                    raise faults.DuplicateFingerprint(self.fingerprint)

    def checkModifyIsPossible(self, changes, givenSlotNames = None):
        ObjectServerMixin.checkModifyIsPossible(
            self, changes, givenSlotNames = givenSlotNames)
        virtualServerId = context.getVar('applicationId')
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        if (not givenSlotNames or 'login' in givenSlotNames) \
               and changes.login != self.login and changes.login is not None:
            if virtualServer.objectsByLogin.has_key(changes.login) \
               and changes.id != virtualServer.objectsByLogin[
                changes.login].id:
                raise faults.DuplicateLogin(changes.login)
        changesFullName = changes.getFullName()
        if (not givenSlotNames or 'firstName' in givenSlotNames
            and 'lastName' in givenSlotNames
            and 'nickname' in givenSlotNames) \
            and changesFullName != self.getFullName() and changesFullName:
            for person in virtualServer.objects.values():
                if person.getFullName() == changesFullName \
                   and person.id != changes.id:
                    raise faults.DuplicateFullName(changesFullName)
        if (not givenSlotNames or 'email' in givenSlotNames) \
               and changes.email != self.email and changes.email is not None:
            for person in virtualServer.objects.values():
                if person.email == changes.email \
                   and person.id != changes.id:
                    raise faults.DuplicateEmail(changes.email)
        if (not givenSlotNames or 'fingerprint' in givenSlotNames) \
               and changes.fingerprint != self.fingerprint \
               and changes.fingerprint is not None:
            for person in virtualServer.objects.values():
                if person.fingerprint == changes.fingerprint \
                   and person.id != changes.id:
                    raise faults.DuplicateFingerprint(changes.fingerprint)

    def exportToXmlRpc(self, requiredSlotNames = None, parentSlot = None):
        personExport = PersonCommon.exportToXmlRpc(
            self, requiredSlotNames = requiredSlotNames,
            parentSlot = parentSlot)
        userId = getProxyForServerRole('authentication').getUserId()
        if self.id != userId:
            if personExport.has_key('email') and not userId:
                del personExport['email']
            if personExport.has_key('password'):
                del personExport['password']
            if personExport.has_key('voteTokens'):
                if not userId:
                    del personExport['voteTokens']
                else:
                    voteTokens = personExport['voteTokens'].copy()
                    for electionId, voteToken in voteTokens.items():
                        if not getProxyForServerRole('elections').hasObject(electionId):
                            del voteTokens[electionId]
                        else:
                            electionBallotKind = getProxyForServerRole('elections').getBallotKind(
                                electionId)
                            if electionBallotKind == 'secret' \
                               or electionBallotKind == 'voterChoice' \
                               and getProxyForServerRole('ballots').isVoteSecret(voteToken):
                                voteTokens[electionId] = 'secret'
                    if len(voteTokens) == 0:
                        del personExport['voteTokens']
                    else:
                        personExport['voteTokens'] = voteTokens
        return personExport

    def modify(self, changes, givenSlotNames = None):
        virtualServerId = context.getVar('applicationId')
        virtualServer = self.getServer().getVirtualServer(virtualServerId)
        login = self.login
        if not virtualServer.admin.userCanChoosePassword:
            self.password_kind = copy.copy(self.password_kind)
            self.password_kind.hasToModify = 0
        ObjectServerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
        if not virtualServer.admin.userCanChoosePassword:
            del self.password_kind
        if self.login != login:
            if login is not None:
                del virtualServer.objectsByLogin[login]
            if self.login is not None:
                virtualServer.objectsByLogin[self.login] = self

    def vote(self, electionId, voteToken):
        if self.voteTokens is None:
            self.voteTokens = {}
        self.voteTokens[electionId] = voteToken
register(Person)


class PeopleVirtualServer(ObjectsVirtualServer):
    objectsByLogin = None
        
    def init(self, server, virtualServerId):
        ObjectsVirtualServer.init(self, server, virtualServerId)
        self.objectsByLogin = {}

    def initFromOldData(self, server, virtualServerId, data):
        ObjectsVirtualServer.initFromOldData(
            self, server, virtualServerId, data)
        self.objectsByLogin = {}
        for object in self.objects.values():
            if object.login is not None:
                self.objectsByLogin[object.login] = object


class PeopleServer(ObjectsServer):
    adminClassName = 'AdminPeople'
    objectClassName = 'Person'
    VirtualServer = PeopleVirtualServer
    version = __version__

    def abstainForVote(self, virtualServerId, clientToken, userToken,
                       electionId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            voterId = getProxyForServerRole('authentication').getUserId()
            if not virtualServer.objects.has_key(voterId):
                raise faults.MissingItem(voterId)
            object = virtualServer.objects[voterId]
            object.abstainForVote(electionId)
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            return 0 # We have to return something.
        finally:
            self.pullContext()

    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)
            # The following behaviour is different from the other servers,
            # because the people server allows unlogged people to create a user
            # account.
            if not self.isAdmin(virtualServerId, clientToken, userToken) \
               and virtualServer.admin.writersSet \
               and not getProxyForServerRole('authentication').setContainsUser(
                virtualServer.admin.writersSet):
                raise faults.UserAccessDenied()
            object.checkAddIsPossible()
            object.setAutomaticalSlots()
            object.id = self.getNextId()
            virtualServer.objects[object.id] = object
            virtualServer.objectsByLogin[object.login] = object
            object.password = commonTools.makepassword()
            object.saveNonCore()
            object.releaseNonCore()
            self.saveVirtualServer(virtualServer)

            ## Add object in specific authentication if needed
            if virtualServer.admin.authenticationServerId is not None:
                authProxy = getProxy(
                        virtualServer.admin.authenticationServerId)
                authProxy.addUser(
                    object.login, object.login, object.password,
                    serverId = virtualServer.admin.authenticationServerId)

            if virtualServer.admin.defaultGroupId is not None:
                getProxyForServerRole('groups').addObjectMember(
                    virtualServer.admin.defaultGroupId, object.id)
            try:
                gpg = Gpg(
                    email = object.email, fingerprint = object.fingerprint)
                gpg.addKey()
            except: # raise Exception('GnuPG not configured for the user')
                # it should be a real fault that could be catched specifically
                pass
            if object.login and object.email:
                pushedFault = None
                try:
                    self.emailPassword(
                        virtualServerId, clientToken, userToken, object.id)
                except faults.Fault, fault:
                    pushedFault = fault
                try:
                    self.emailAdminsForNewUser(virtualServerId, object.id)
                except faults.Fault, fault:
                    if pushedFault is not None:
                        pushedFault = fault
                if pushedFault is not None:
                    raise pushedFault
            self.createAliasFile(virtualServer)
            return object.id
        finally:
            self.pullContext()

    def canAddObject(self, virtualServerId, clientToken, userToken):
        # The following behaviour is different from the other servers, because
        # the people server allows unlogged people to create a user account.
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            dispatcherId = commonTools.extractDispatcherId(virtualServerId)
            if dispatcherId == 'glasnost://system':
                return 0
            virtualServer = self.getVirtualServer(virtualServerId)
            return self.isAdmin(virtualServerId, clientToken, userToken) \
                   or not virtualServer.admin.writersSet \
                   or getProxyForServerRole('authentication').setContainsUser(
                virtualServer.admin.writersSet)
        finally:
            self.pullContext()

    def canDeleteObject(self, virtualServerId, clientToken, userToken,
                        objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                return 0
            object = virtualServer.objects[objectId]
            return self.isAdmin(virtualServerId, clientToken, userToken) \
                   or getProxyForServerRole('authentication').setContainsUser([object.id])
        finally:
            self.pullContext()

    def canEmailPassword(self, virtualServerId, clientToken, userToken,
                         objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                return 0
            object = virtualServer.objects[objectId]
            if object.login and object.email:
                return 1

        ## for futur change, when we drop glasnost people information
##        if virtualServer.admin.authenticationServerId is not None:
##            object = virtualServer.objects[objectId]
##            if object.login:
##                authProxy = getProxy(
##                    virtualServer.admin.authenticationServerId)
##                return authProxy.canEmailPassword(
##                    object.login,
##                    serverId = virtualServer.admin.authenticationServerId)

            return 0
        finally:
            self.pullContext()

    def canModifyObject(self, virtualServerId, clientToken, userToken,
                        objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                return 0
            object = virtualServer.objects[objectId]
            return self.isAdmin(virtualServerId, clientToken, userToken) \
                   or getProxyForServerRole('authentication').setContainsUser([object.id])
        finally:
            self.pullContext()

    def changePasswordXmlRpc(self, virtualServerId, clientToken, userToken,
                             objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            if not getProxyForServerRole('authentication').setContainsUser([object.id]):
                raise faults.UserAccessDenied()
            newPassword = commonTools.makepassword()

            ## change in futur when glasnost don't conserve user information
            object.password = newPassword
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)

            if virtualServer.admin.authenticationServerId is not None:
                authProxy = getProxy(
                    virtualServer.admin.authenticationServerId)
                authProxy.changePassword(
                    object.login, object.password, newPassword,
                    serverId = virtualServer.admin.authenticationServerId)

            fromAddress = self.getAdminEmailAddress(virtualServerId)
            if object.password is None:
                password = ''
            else:
                password = object.password
            if not object.login or not object.email:
                return utf8(password)
            toAddress = object.email
            message = """\
Here is the new password you have requested.

Web Site:   http://%(hostName)s
First Name: %(firstName)s
Last Name:  %(lastName)s
Nickname:   %(nickname)s
Login:      %(login)s
Password:   %(password)s

The Glasnost administrator - %(fromAddress)s
""" % {
                'firstName': object.firstName,
                'fromAddress': fromAddress,
                'hostName': getProxyForServerRole('virtualHosts').getHostName(virtualServerId),
                'login': object.login,
                'lastName': object.lastName,
                'nickname': object.nickname,
                'password': password,
                }
            sendMail(
                mailFrom = fromAddress,
                mailTo = toAddress,
                mailSubject = "[Glasnost] login & password",
                mailMessage = message,
                mailPerson = object,
                mailPersonId = objectId,
                )
            return utf8(password)
        finally:
            self.pullContext()

    def checkObjectAuthenticationXmlRpc(
            self, virtualServerId, clientId, objectLogin, objectPassword):
        self.pushContext(virtualServerId, None, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            objectLogin = iso8859_15(objectLogin)
            objectPassword = iso8859_15(objectPassword)
            if not virtualServer.objectsByLogin.has_key(objectLogin):
                raise faults.WrongLogin(objectLogin)
            object = virtualServer.objectsByLogin[objectLogin]

            ## we check authentication with server defined by user if possible
            if virtualServer.admin.authenticationServerId is None:
                if object.password and objectPassword != object.password:
                    raise faults.WrongPassword(objectPassword)
            else:
                authProxy = getProxy(
                    virtualServer.admin.authenticationServerId)
                if not authProxy.checkAuthentication(
                    objectLogin, objectPassword,
                    serverId = virtualServer.admin.authenticationServerId):
                    raise faults.WrongPassword(objectPassword)
            return object.id
        finally:
            self.pullContext()

    def createAliasFile(self, virtualServer):
        aliasesDirectoryPath = os.path.join(
                        virtualServer.dataDirectoryPath, applicationRole, 'files')
        if not os.path.exists(aliasesDirectoryPath):
            os.mkdir(aliasesDirectoryPath)
            os.chmod(aliasesDirectoryPath, 0750)
        aliases = {}
        for objectId, objectCore in virtualServer.objects.items():
            if not hasattr(objectCore, 'email') or not objectCore.email:
                continue
            aliases[objectCore.login] = objectCore.email
        file = open('%s/%s' % (aliasesDirectoryPath, 'alias'), 'w')
        file.write(''.join( ['%s: %s\n' % x for x in aliases.items()] ) )
        file.close()

    def deleteObject(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            if not self.isAdmin(virtualServerId, clientToken, userToken) \
               and not getProxyForServerRole('authentication').setContainsUser([objectId]):
                raise faults.UserAccessDenied()
            object = virtualServer.objects[objectId]
            try:
                gpg = Gpg(object.email)
                gpg.deleteKey()
            except:
                pass
            if virtualServer.objectsByLogin.has_key(object.login):
                del virtualServer.objectsByLogin[object.login]
            del virtualServer.objects[objectId]
            object.removeNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(objectId)

            if virtualServer.admin.authenticationServerId is not None:
                authProxy = getProxy(
                    virtualServer.admin.authenticationServerId)
                authProxy.deleteUser(
                    object.login,
                    serverId = virtualServer.admin.authenticationServerId)

            self.createAliasFile(virtualServer)

            return objectId # We have to return something.
        finally:
            self.pullContext()

    def emailAdminsForNewUser(self, virtualServerId, objectId):
        virtualServer = self.getVirtualServer(virtualServerId)
        if not virtualServer.objects.has_key(objectId):
            raise faults.MissingItem(objectId)
        object = virtualServer.objects[objectId]
        if not object.login or not object.email:
            raise faults.BadEmailAddress(object.email)
        adminIds = getSetContainedIds(
            virtualServer.admin.adminsSet, ['people'],
            serverId = virtualServerId)
        toAddresses = []
        for adminId in adminIds:
            if not virtualServer.objects.has_key(adminId):
                continue
            administrator = virtualServer.objects[adminId]
            if administrator.email:
                toAddresses.append(administrator.email)
        if not toAddresses:
            return
        for adminId in adminIds:
            if not virtualServer.objects.has_key(adminId):
                continue
            administrator = virtualServer.objects[adminId]
            if administrator.email:
                fromAddress = administrator.email
                break
        else:
            fromAddress = self.getAdminEmailAddress(virtualServerId)
        message = """\
Dear Glasnost administrators,

A new user has been registered.

Web Site:   http://%(hostName)s
First Name: %(firstName)s
Last Name:  %(lastName)s
Nickname:   %(nickname)s
Login:      %(login)s
Email:      %(email)s
GnuPG:      %(fingerprint)s
""" % {
            'email': object.email,
            'fingerprint': object.fingerprint,
            'firstName': object.firstName,
            'hostName': getProxyForServerRole('virtualHosts').getHostName(virtualServerId),
            'lastName': object.lastName,
            'login': object.login,
            'nickname': object.nickname,
            }
        moreHeaders = None
        if object.email:
            moreHeaders = { 'Reply-To': object.email }
        sendMail(
            mailFrom = fromAddress,
            mailTo = toAddresses,
            mailSubject = "[Glasnost admin] New user",
            mailMessage = message,
            moreHeaders = moreHeaders,
            )

    def emailPassword(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            if not object.login or not object.email:
                raise faults.BadEmailAddress(object.email)
            toAddresses = [
                object.email,
                ]
            adminIds = getSetContainedIds(
                virtualServer.admin.adminsSet, ['people'],
                serverId = virtualServerId)
            for adminId in adminIds:
                if not virtualServer.objects.has_key(adminId):
                    continue
                administrator = virtualServer.objects[adminId]
                if administrator.email:
                    fromAddress = administrator.email
                    break
            else:
                fromAddress = self.getAdminEmailAddress(virtualServerId)
            if object.password is None:
                password = ''
            else:
                password = object.password
            message = """\
Here are the Glasnost login & password you have requested.

Web Site:   http://%(hostName)s
First Name: %(firstName)s
Last Name:  %(lastName)s
Nickname:   %(nickname)s
Login:      %(login)s
Password:   %(password)s

The Glasnost administrator - %(fromAddress)s
""" % {
                'firstName': object.firstName,
                'fromAddress': fromAddress,
                'hostName': getProxyForServerRole('virtualHosts').getHostName(virtualServerId),
                'login': object.login,
                'lastName': object.lastName,
                'nickname': object.nickname,
                'password': password,
                }
            sendMail(
                mailFrom = fromAddress,
                mailTo = toAddresses,
                mailSubject = "[Glasnost] login & password",
                mailMessage = message,
                mailPerson = object,
                mailPersonId = objectId,
                )
            return objectId # We have to return something.
        finally:
            self.pullContext()

    def findObjectIdsXmlRpc(self, virtualServerId, clientToken, userToken,
                            text):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            text = iso8859_15(text)
            foundIds = []
            for objectId, object in virtualServer.objects.items():
                if object.containsText(text):
                    foundIds.append(objectId)
            return foundIds
        finally:
            self.pullContext()

    def getAdminEmailAddress(self, virtualServerId):
        return getConfig('Mail', 'Admin', default='admin@entrouvert.org')

    def getObjectByLoginXmlRpc(self, virtualServerId, clientToken, userToken,
                               objectLogin):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            object = self.getObjectCoreByLogin(virtualServerId, objectLogin)
            object.acquireNonCore()
            try:
                result = object.exportToXmlRpc()
            finally:
                object.releaseNonCore()
            return result
        finally:
            self.pullContext()

    # The real XML-RPC function is getObjectXmlRpc.
    def getObjectCore(self, objectId):
        virtualServerId = self.computeVirtualServerId(objectId)
        virtualServer = self.getVirtualServer(virtualServerId)
        if not virtualServer.objects.has_key(objectId):
            raise faults.MissingItem(objectId)
        object = virtualServer.objects[objectId]
        return object

    # The real XML-RPC function is getObjectByLoginXmlRpc.
    def getObjectCoreByLogin(self, virtualServerId, objectLogin):
        virtualServer = self.getVirtualServer(virtualServerId)
        objectLogin = iso8859_15(objectLogin)
        if not virtualServer.objectsByLogin.has_key(objectLogin):
            raise faults.MissingItem(objectLogin)
        object = virtualServer.objectsByLogin[objectLogin]
        return object

    def getObjectStringFromDigestXmlRpc(
        self, virtualServerId, clientToken, userToken, objectId, path, digest):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            object.acquireNonCore()
            try:
                # The objectLabel is accessible from everybody.
                if path == 'self.getLabel()':
                    result = getStringFromDigest(object.getLabel(), digest)
                else:
                    if not self.isAdmin(
                        virtualServerId, clientToken, userToken) \
                       and not getProxyForServerRole('authentication').setContainsUser([objectId]):
                        raise faults.UserAccessDenied()
                    result = getStringFromDigest(
                        eval(path, {'self': object}), digest)
            finally:
                object.releaseNonCore()
            if result is None:
                raise faults.UnknownStringDigest(digest)
            return utf8(result)
        finally:
            self.pullContext()

    def getObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            object = self.getObjectCore(objectId)
            object.acquireNonCore()
            try:
                result = object.exportToXmlRpc()
            finally:
                object.releaseNonCore()
            return result
        finally:
            self.pullContext()

    def getPartialObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                               objectId, requiredSlotNames):
        if not requiredSlotNames:
            return self.getObjectXmlRpc(
                virtualServerId, clientToken, userToken, objectId)
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            object = self.getObjectCore(objectId)
            useNonCore = 0
            if not 'id' in requiredSlotNames \
               and object.isSlotName('id') \
               and object.getSlot('id').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('id')
            if not 'language' in requiredSlotNames \
               and object.isSlotName('language') \
               and object.getSlot('language').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('language')
            if not 'readersSet' in requiredSlotNames \
               and object.isSlotName('readersSet') \
               and object.getSlot('readersSet').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('readersSet')
            for slotName in object.getSlotNames():
                if not slotName in requiredSlotNames:
                    continue
                slot = object.getSlot(slotName)
                kind = slot.getKind()
                if not kind.isInCore(slot):
                    useNonCore = 1
                    break
            if useNonCore:
                object.acquireNonCore()
            try:
                result = object.exportToXmlRpc(
                    requiredSlotNames = requiredSlotNames)
            finally:
                if useNonCore:
                    object.releaseNonCore()
            return result
        finally:
            self.pullContext()

    def hasObjectByLoginXmlRpc(
        self, virtualServerId, clientToken, userToken, objectLogin):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            objectLogin = iso8859_15(objectLogin)
            return virtualServer.objectsByLogin.has_key(objectLogin)
        finally:
            self.pullContext()

    def modifyObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                           objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            objectChanges = self.newThing(
                'object', objectImport['__thingName__'])
            objectChanges.importFromXmlRpc(objectImport)
            if not virtualServer.objects.has_key(objectChanges.id):
                raise faults.MissingItem(objectChanges.id)
            object = virtualServer.objects[objectChanges.id]

            ## if there is authenticationServer, we can't change login
            if virtualServer.admin.authenticationServerId is not None \
                   and objectChanges.login != object.login:
                raise faults.WrongLogin(objectChanges.login)

            object.acquireNonCore()
            try:
                if not object.canBeModified():
                    raise faults.ReadOnlyObject()
                if not self.isAdmin(virtualServerId, clientToken, userToken) \
                   and not getProxyForServerRole('authentication').setContainsUser([object.id]):
                    clientId = getApplicationId(clientToken)
                    if not object.canBeModifiedByClient(clientId):
                        raise faults.UserAccessDenied()
                object.checkModifyIsPossible(objectChanges)
                if object.fingerprint:
                    try:
                        gpg = Gpg(objectChanges.email,
                                  fingerprint = object.fingerprint)
                        gpg.updateKey()
                    except:
                        pass
                object.modify(objectChanges)
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)

            # Only password can be changed in other authentication, for the
            # moment.
            if virtualServer.admin.authenticationServerId is not None:
                authProxy = getProxy(
                    virtualServer.admin.authenticationServerId)
                authProxy.changePassword(
                    object.login, object.password, objectChanges.password,
                    serverId = virtualServer.admin.authenticationServerId)

            self.createAliasFile(virtualServer)

            return object.version
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('abstainForVote')
        self.registerXmlRpcMethod('canEmailPassword')
        self.registerXmlRpcMethod('changePassword', self.changePasswordXmlRpc)
        self.registerXmlRpcMethod('checkObjectAuthentication',
                                  self.checkObjectAuthenticationXmlRpc)
        self.registerXmlRpcMethod('emailPassword')
        self.registerXmlRpcMethod('findObjectIds', self.findObjectIdsXmlRpc)
        self.registerXmlRpcMethod('getObjectByLogin',
                                  self.getObjectByLoginXmlRpc)
        self.registerXmlRpcMethod('hasObjectByLogin',
                                  self.hasObjectByLoginXmlRpc)
        self.registerXmlRpcMethod('setLoginTime')
        self.registerXmlRpcMethod('vote')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 3004:
            import time
            for object in virtualServer.objects.values():
                if object.creationTime is None:
                    changed = 1
                    object.modificationTime = object.creationTime = time.time()
        if version < 4000:
            changed = virtualServer.admin.repair(4000) or changed
            for id, object in virtualServer.objects.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    del virtualServer.objects[id]
                    virtualServer.objects[newId] = object
                changed = object.repair(4000) or changed
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1005000:
            changed = 1
            virtualServer.objectsByLogin = {}
            for id, object in virtualServer.objects.items():
                virtualServer.objectsByLogin[object.login] = object
        if version <= 1029000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if version <= 1041000:
            for object in virtualServer.objects.values():
                if object.voteTokens is None:
                    continue
                for electionId, voteToken in object.voteTokens.items():
                    if not voteToken.startswith('glasnost://'):
                        changed = 1
                        object.voteTokens[electionId] = '%s/votes/%s' % (
                            commonTools.extractDispatcherId(
                            virtualServer.virtualServerId),
                            voteToken)
        if changed:
            self.saveVirtualServer(virtualServer)

    def setLoginTime(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            try:
                userId = getProxyForServerRole('authentication').getUserId()
            except faults.UnknownUserToken:
                print 'setLoginTime on unknown userToken: strange.'
                return 0
            if not virtualServer.objects.has_key(userId):
                raise faults.MissingItem(userId)
            object = virtualServer.objects[userId]
            object.loginTime = time.time()
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            return 0 # We have to return something.
        finally:
            self.pullContext()

    def vote(self, virtualServerId, clientToken, userToken, electionId,
             voteToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            voterId = getProxyForServerRole('authentication').getUserId()
            if not virtualServer.objects.has_key(voterId):
                raise faults.MissingItem(voterId)
            object = virtualServer.objects[voterId]
            object.vote(electionId, voteToken)
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            return 0 # We have to return something.
        finally:
            self.pullContext()


peopleServer = PeopleServer()


if __name__ == "__main__":
    peopleServer.launch(applicationName, applicationRole)
