#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Preferences Server"""

__version__ = '$Revision: 1.19 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults
from glasnost.common.PreferencesCommon import PreferenceCommon
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *


applicationName = 'PreferencesServer'
applicationRole = 'preferences'
applicationTokens = None
dispatcher = None


class PreferencesVirtualServer(VirtualServer):
    objects = None

    def convertIds(self, sourceDispatcherId, destinationDispatcherId):
        VirtualServer.convertIds(
            self, sourceDispatcherId, destinationDispatcherId)
        for id, struct in self.objects.items():
            object = PreferenceCommon()
            object.__dict__ = struct
            object.convertIds(sourceDispatcherId, destinationDispatcherId)

    def init(self, server, virtualServerId):
        VirtualServer.init(self, server, virtualServerId)
        self.objects = {}

    def initFromOldData(self, server, virtualServerId, data):
        VirtualServer.initFromOldData(self, server, virtualServerId, data)
        self.objects = data

    def removeIds(self, rolesToKeep):
        VirtualServer.removeIds(self, rolesToKeep)
        for struct in self.objects.values():
            object = PreferenceCommon()
            object.__dict__ = struct
            object.removeIds(rolesToKeep)


class PreferencesServer(Server):
    VirtualServer = PreferencesVirtualServer
    version = __version__

    def deleteObject(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            virtualServer.lock.acquire()
            if virtualServer.objects.has_key(userId):
                del virtualServer.objects[userId]
            virtualServer.lock.release()
            self.saveVirtualServer(virtualServer)
            #invalidateValue(no id!)
            return 0 # We have to return something.
        finally:
            self.pullContext()

    def getObject(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            userId = getProxyForServerRole('authentication').getUserId()
            return self.getObjectByUserId(virtualServerId, clientToken, userId)
        finally:
            self.pullContext()

    def getObjectByUserId(self, virtualServerId, clientToken, userId):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(userId):
                preference = {
                    '__thingCategory__': 'object',
                    '__thingName__': 'Preference',
                    'version': 0,
                    }
            else:
                preference = virtualServer.objects[userId]
                if not preference.has_key('version'):
                    preference['version'] = 0
            return preference
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('deleteObject')
        self.registerXmlRpcMethod('getObject')
        self.registerXmlRpcMethod('getObjectByUserId')
        self.registerXmlRpcMethod('rememberId')
        self.registerXmlRpcMethod('setObject')

    def rememberId(self, virtualServerId, clientToken, userToken, id):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            if virtualServer.objects.has_key(userId):
                preference = virtualServer.objects[userId]
            else:
                preference = {
                    '__thingCategory__': 'object',
                    '__thingName__': 'Preference',
                    'version': 0,
                    }
            serverHostNameAndPort, serverRole, localId = splitId(id)
            if preference.has_key('objectsMemory'):
                objectsMemory = preference['objectsMemory']
            else:
                objectsMemory = None
            if objectsMemory is None:
                objectsMemory = preference['objectsMemory'] = {}
            if objectsMemory.has_key(serverRole):
                serverMemory = objectsMemory[serverRole]
            else:
                serverMemory = None
            if serverMemory is None:
                serverMemory = objectsMemory[serverRole] = []
            if id in serverMemory:
                serverMemory.remove(id)
            serverMemory.insert(0, id)
            if len(serverMemory) > 10:
                serverMemory[10:] = []
            virtualServer.objects[userId] = preference
            self.saveVirtualServer(virtualServer)
            #invalidateValue(no id!)
            return 0 # We have to return something.
        finally:
            self.pullContext()

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 2006:
            for object in virtualServer.objects.values():
                if object.has_key('isBallotSecret'):
                    changed = 1
                    del object['isBallotSecret']
        if version < 4000:
            for id, object in virtualServer.objects.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    del virtualServer.objects[id]
                    virtualServer.objects[newId] = object
                if object.has_key('objectsMemory'):
                    objectsMemory = object['objectsMemory']
                    for serverId, serverMemory in objectsMemory.items():
                        if serverId.startswith('ee') \
                           and serverId.endswith('Server'):
                            changed = 1
                            serverRole = serverId[2:-6].lower()
                            del objectsMemory[serverId]
                            objectsMemory[serverRole] = serverMemory
                        for i in range(len(serverMemory)):
                            itemId = serverMemory[i]
                            newItemId = repairId(itemId)
                            if newItemId:
                                changed = 1
                                serverMemory[i] = newItemId
        if version <= 1007000:
            for object in virtualServer.objects.values():
                if object.has_key('type'):
                    changed = 1
                    object['__thingName__'] = object['type']
                    del object['type']
        if version <= 1010000:
            for object in virtualServer.objects.values():
                if not object.has_key('__thingCategory__'):
                    changed = 1
                    object['__thingCategory__'] = 'object'
        if changed:
            self.saveVirtualServer(virtualServer)

    def setObject(self, virtualServerId, clientToken, userToken,
                  objectChanges):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            version = 0
            if virtualServer.objects.has_key(userId):
                object = virtualServer.objects[userId]
                if object.has_key('version'):
                    version = object['version']
            changesVersion = 0
            if objectChanges.has_key('version'):
                changesVersion = objectChanges['version']
            if version != changesVersion:
                raise faults.WrongVersion()
            if object.has_key('objectsMemory'):
                objectChanges['objectsMemory'] = object['objectsMemory']
            object = objectChanges
            version += 1
            object['version'] = version
            virtualServer.objects[userId] = object
            self.saveVirtualServer(virtualServer)
            #invalidateValue(no id!)
            return version
        finally:
            self.pullContext()


preferencesServer = PreferencesServer()


if __name__ == "__main__":
    preferencesServer.launch(applicationName, applicationRole)
