#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Records Server"""

__version__ = '$Revision: 1.31 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults
import glasnost.common.context as context
from glasnost.common.RecordsCommon import *

from glasnost.server.ObjectsServer \
     import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.DispatcherProxy import getApplicationId
from glasnost.proxy.tools import getProxyForServerRole


applicationName = 'RecordsServer'
applicationRole = 'records'
applicationTokens = None
dispatcher = None


class AdminRecords(AdminServerMixin, AdminRecordsCommon):
    pass
register(AdminRecords)


class Record(ObjectServerMixin, RecordCommon):
    def acquireEventData(self):
        if self.eventId is None:
            self.getSlot('certainty').setValue(None)
            self.getSlot('currency').setValue(None)
            self.getSlot('date').setValue(None)
            self.getSlot('label').setValue(None)
            self.getSlot('partnerId').setValue(None)
            self.getSlot('readersSet').setValue(None)
            self.getSlot('voucherId').setValue(None)
            self.getSlot('writersSet').setValue(None)
        else:
            try:
                event = getProxyForServerRole('events').getObject(self.eventId)
            except faults.MissingItem:
                self.getSlot('certainty').setValue(None)
                self.getSlot('currency').setValue(None)
                self.getSlot('date').setValue(None)
                self.getSlot('label').setValue(None)
                self.getSlot('partnerId').setValue(None)
                self.getSlot('readersSet').setValue(None)
                self.getSlot('voucherId').setValue(None)
                self.getSlot('writersSet').setValue(None)
            else:
                self.getSlot('certainty').setValue(
                    event.getSlot('certainty').getValue())
                self.getSlot('currency').setValue(
                    event.getSlot('currency').getValue())
                self.getSlot('date').setValue(
                    event.getSlot('date').getValue())
                self.getSlot('label').setValue(
                    event.getSlot('label').getValue())
                self.getSlot('partnerId').setValue(
                    event.getSlot('partnerId').getValue())
                self.getSlot('readersSet').setValue(
                    event.getSlot('readersSet').getValue())
                self.getSlot('voucherId').setValue(
                    event.getSlot('voucherId').getValue())
                self.getSlot('writersSet').setValue(
                    event.getSlot('writersSet').getValue())
 
    def acquireNonCore(self, objectDirectoryPath = None,
                       dataDirectoryPath = None, parentSlot = None):
        ObjectServerMixin.acquireNonCore(
            self, objectDirectoryPath = objectDirectoryPath,
            dataDirectoryPath = dataDirectoryPath, parentSlot = parentSlot)
        if self.getServer().port is None:
            # The server is not running, so assume that Glasnost
            # is not running either (it happens for example during
            # ids conversion). Don't call the other servers. Do nothing.
            return        
        self.acquireAccountData()
        self.acquireEventData()

    def acquireReadersAndWriters(self):
        if self.eventId is None:
            self.getSlot('readersSet').setValue(None)
            self.getSlot('writersSet').setValue(None)
        else:
            try:
                event = getProxyForServerRole('events').getObject(self.eventId)
            except faults.MissingItem:
                self.getSlot('readersSet').setValue(None)
                self.getSlot('writersSet').setValue(None)
            else:
                self.getSlot('readersSet').setValue(
                    event.getSlot('readersSet').getValue())
                self.getSlot('writersSet').setValue(
                    event.getSlot('writersSet').getValue())

    def canBeCreatedByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return clientRole in ['events', 'workflows']

    def canBeDeletedByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return clientRole == 'events'

    def canBeModifiedByClient(self, clientId):
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        return clientRole in ['events', 'workflows']

    def compute(self):
        accountId = self.getSlot('accountId').getValue()
        getProxyForServerRole('accounts').computeObject(accountId)
        
    def modify(self, changes = None, givenSlotNames = None):
        ObjectServerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
        self.compute()

    def releaseNonCore(self, parentSlot = None):
        self.getSlot('accountLabel').setValue(None)
        self.getSlot('accountNumber').setValue(None)
        self.getSlot('certainty').setValue(None)
        self.getSlot('currency').setValue(None)
        self.getSlot('date').setValue(None)
        self.getSlot('certainty').setValue(None)
        self.getSlot('label').setValue(None)
        self.getSlot('readersSet').setValue(None)
        self.getSlot('voucherId').setValue(None)
        self.getSlot('writersSet').setValue(None)
        ObjectServerMixin.releaseNonCore(self, parentSlot = parentSlot)

    def releaseReadersAndWriters(self):
        self.getSlot('readersSet').setValue(None)
        self.getSlot('writersSet').setValue(None)
register(Record)


class RecordsServer(ObjectsServer):
    adminClassName = 'AdminRecords'
    objectClassName = 'Record'
    version = __version__

    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        try:
            accountId = objectImport['accountId']
        except:
            raise ValueError
        objectId = ObjectsServer.addObject(
            self, virtualServerId, clientToken, userToken, objectImport)
        getProxyForServerRole('accounts').computeObject(accountId)
        return objectId

    def computeObject(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            object.compute()
            self.saveVirtualServer(virtualServer)
            return objectId # We have to return something.
        finally:
            self.pullContext()
                    
    def deleteObject(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            clientId = getApplicationId(clientToken)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            object = virtualServer.objects[objectId]
            accountId = object.getSlot('accountId').getValue()
            if not self.canDeleteObject(
                virtualServerId, clientToken, userToken, objectId):
                if not object.canBeDeletedByClient(clientId):
                    raise faults.UserAccessDenied()
            del virtualServer.objects[objectId]
            getProxyForServerRole('accounts').computeObject(accountId)
            self.saveVirtualServer(virtualServer)
            invalidateValue(objectId)
            return objectId # We have to return something.
        finally:
            self.pullContext()

    def getCustomObjectIds(self, virtualServerId, clientToken, userToken,
                           accountsSet, startDate, endDate):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            print accountsSet, startDate, endDate
            virtualServer = self.getVirtualServer(applicationId)
            isAdmin = self.isAdmin(virtualServerId, clientToken, userToken)
            clientId = getApplicationId(clientToken)
            result = []
            for objectId, object in virtualServer.objects.items():
                print objectId
                print object.accountId
                object.acquireReadersAndWriters()
                try:
                    if not isAdmin and object.readersSet \
                           and not getProxyForServerRole(
                        'authentication').setContainsUser(object.readersSet) \
                        and not object.canBeGottenByClient(clientId):
                        print 'no access'
                        continue
                    if object.accountId in accountsSet \
                       and (startDate == '' or startDate <= object.date) \
                       and (endDate == '' or object.date <= endDate):
                        result.append(objectId)
                except:
                    raise
                object.releaseReadersAndWriters()
            print result    
            return result
        finally:
            self.pullContext()

    def getObjectIds(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            isAdmin = self.isAdmin(virtualServerId, clientToken, userToken)
            clientId = getApplicationId(clientToken)
            result = []
            for objectId, object in virtualServer.objects.items():
                object.acquireReadersAndWriters()
                try:
                    if isAdmin or not object.readersSet \
                           or getProxyForServerRole(
                        'authentication').setContainsUser(object.readersSet) \
                        or object.canBeGottenByClient(clientId):
                        result.append(objectId)
                finally:
                    object.releaseReadersAndWriters()
            return result
        finally:
            self.pullContext()

    def getPartialObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                               objectId, requiredSlotNames):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            if not requiredSlotNames:
                return self.getObjectXmlRpc(clientToken, userToken, objectId)
            easySlotNames = ['amount', 'side']
            easy = 1
            for name in requiredSlotNames:
                if not name in easySlotNames:
                    easy = 0
                    break
            if easy:
                virtualServerId = self.computeVirtualServerId(objectId)
                virtualServer = self.getVirtualServer(virtualServerId)
                if not virtualServer.objects.has_key(objectId):
                    raise faults.MissingItem(objectId)
                object = virtualServer.objects[objectId]
                try:
                    result = object.exportToXmlRpc(
                        requiredSlotNames = requiredSlotNames)
                except:
                    raise
                return result
            object = self.getObjectCore(objectId)
            useAccount = 0
            useEvent = 0
            if not 'id' in requiredSlotNames \
               and object.isSlotName('id') \
               and object.getSlot('id').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('id')
            if not 'language' in requiredSlotNames \
               and object.isSlotName('language') \
               and object.getSlot('language').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('language')
            if not 'readersSet' in requiredSlotNames \
               and object.isSlotName('readersSet') \
               and object.getSlot('readersSet').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('readersSet')
            for slotName in object.getSlotNames():
                if not slotName in requiredSlotNames:
                    continue
                if slotName in ['accountNumber', 'accountLabel']:
                    useAccount = 1
                elif slotName in ['certainty', 'currency', 'date', 'label',
                                  'partnerId', 'readersSet', 'voucherId',
                                  'writersSet']:
                    useEvent = 1
            if useAccount:
                object.acquireAccountData()
            if useEvent:
                object.acquireEventData()
            try:
                result = object.exportToXmlRpc(
                    requiredSlotNames = requiredSlotNames)
            finally:
                if useAccount or useEvent:
                    object.releaseNonCore()
            return result
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('computeObject')
        self.registerXmlRpcMethod('getCustomObjectIds')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 6000:
            for id, object in virtualServer.objects.items():
                amount = object.amount
                side = object.side
                if side == 'debit':
                    amount = - amount
                setattr(object, 'amount', amount)
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1011000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


recordsServer = RecordsServer()


if __name__ == "__main__":
    recordsServer.launch(applicationName, applicationRole)



