#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Sessions Server"""

__version__ = '$Revision: 1.18 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import time
import whrandom

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.faults as faults

from glasnost.server.ObjectsServer import Server, VirtualServer
from glasnost.server.tools import *


applicationName = 'SessionsServer'
applicationRole = 'sessions'
applicationTokens = None
dispatcher = None

# Don't forget to also change the value in AuthenticationServer.py.
expirationTime = int(getConfig('Misc', 'ExpirationTime', '120')) * 60
    # Default is 120 minutes after last action.


class SessionsVirtualServer(VirtualServer):
    objects = None

    def init(self, server, virtualServerId):
        VirtualServer.init(self, server, virtualServerId)
        self.objects = {}

    def initFromOldData(self, server, virtualServerId, data):
        VirtualServer.initFromOldData(self, server, virtualServerId, data)
        self.objects = data


class SessionsServer(Server):
    VirtualServer = SessionsVirtualServer
    useDataFile = 0
    version = __version__

    def deleteObject(self, virtualServerId, clientToken, sessionToken):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            virtualServer.lock.acquire()
            if not virtualServer.objects.has_key(sessionToken):
                virtualServer.lock.release()
                raise faults.UnknownSessionToken(sessionToken)
            del virtualServer.objects[sessionToken]
            virtualServer.lock.release()
            self.saveVirtualServer(virtualServer)
            #invalidateValue(no id!)
            return 0 # We have to return something.
        finally:
            self.pullContext()

    def getObject(self, virtualServerId, clientToken, sessionToken):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(sessionToken):
                raise faults.UnknownSessionToken(sessionToken)
            session = virtualServer.objects[sessionToken]
            if time.time() > session['expirationTime']:
                raise faults.UnknownSessionToken(sessionToken)
            return session
        finally:
            self.pullContext()

    def init(self):
        self.randomGenerator = whrandom.whrandom()
        Server.init(self)

    def newObject(self, virtualServerId, clientToken):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            virtualServer.lock.acquire()
            while 1:
                sessionToken = str(self.randomGenerator.uniform(0.1, 1))[2:]
                if not virtualServer.objects.has_key(sessionToken):
                    break
            session = {
                'sessionToken': sessionToken,
                'version': 0,
                'expirationTime': time.time() + expirationTime
                }
            virtualServer.objects[sessionToken] = session
            virtualServer.lock.release()
            self.saveVirtualServer(virtualServer)
            return session
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        Server.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('deleteObject')
        self.registerXmlRpcMethod('getObject')
        self.registerXmlRpcMethod('newObject')
        self.registerXmlRpcMethod('setObject')

    def setObject(self, virtualServerId, clientToken, sessionChanges):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not sessionChanges.has_key('sessionToken'):
                raise faults.UnknownSessionToken(None)
            sessionToken = sessionChanges['sessionToken']
            virtualServer.lock.acquire()
            if not virtualServer.objects.has_key(sessionToken):
                virtualServer.lock.release()
                raise faults.UnknownSessionToken(sessionToken)
            session = virtualServer.objects[sessionToken]
            version = session['version']
            if version != sessionChanges['version']:
                virtualServer.lock.release()
                raise faults.WrongVersion()
            session = sessionChanges
            version += 1
            session['version'] = version
            virtualServer.objects[sessionToken] = session
            virtualServer.lock.release()
            self.saveVirtualServer(virtualServer)
            #invalidateValue(no id!)
            return version
        finally:
            self.pullContext()


sessionsServer = SessionsServer()


if __name__ == "__main__":
    sessionsServer.launch(applicationName, applicationRole)
