#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Tasks Server"""

__version__ = '$Revision: 1.7 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.TasksCommon import *
import glasnost.common.faults as faults
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.GroupsProxy import setContains, getSetContainedIds
import glasnost.proxy.tools as proxyTools

applicationName = 'TasksServer'
applicationRole = 'tasks'
applicationTokens = None
dispatcher = None


class AdminTasks(AdminServerMixin, AdminTasksCommon):
    pass
register(AdminTasks)


class Task(ObjectServerMixin, TaskCommon):
    pass
register(Task)


class TasksServer(ObjectsServer):
    adminClassName = 'AdminTasks'
    objectClassName = 'Task'
    useAdminWritersSet = 0
    version = __version__

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getTask')

    def isAskedTask(self, task, ownerIds):
        """
        @param task:
        @type task: Task
        @param ownerIds: list of id those in ownersSet
        @param ownerIds: list
        """
        people = getSetContainedIds(ownerIds)
        print ownerIds, people
        for personId in people:
            if setContains(task.ownersSet, personId):
                return 1
        return 0

    def getTask(self, virtualServerId, clientToken, userToken,
                ownersId = None, sortedOrder = None, showFinished = 0):
        """
        @param owner: owner that we want the task list
        @type owner: list
        @return: dictionnary of list of object
        """
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            result = {}
            for objectId, object in virtualServer.objects.items():
                if self.canGetObject(
                    virtualServerId, clientToken, userToken, objectId) and \
                    (showFinished or object.state != 'finish'):
                    if ownersId:
                        for ownerId in ownersId:
                            if self.isAskedTask(object, [ownerId]):
                                if not result.has_key(ownerId):
                                    result[ownerId] = []
                                result[ownerId].append(object)
                    else:
                        for ownerId in object.ownersSet:
                            if not result.has_key(ownerId):
                                result[ownerId] = []
                            result[ownerId].append(object)
            ## result contains list of object by owner
            if sortedOrder and sortedOrder != 'None':
                for key, objects in result.items():
                    objects.sort(
                        lambda ob1, ob2, attr = sortedOrder:
                        cmp(getattr(ob1, attr), getattr(ob2, attr)))

            return result                    
        finally:
            self.pullContext()
    getTask.useAutoDeserialize = 1


tasksServer = TasksServer()


if __name__ == "__main__":
    tasksServer.launch(applicationName, applicationRole)
