#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Virtual Hosts Server"""

__version__ = '$Revision: 1.29 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

from fnmatch import fnmatch

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.VirtualHostsCommon import *
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer, ObjectsVirtualServer
from glasnost.server.tools import *


applicationName = 'VirtualHostsServer'
applicationRole = 'virtualhosts'
applicationTokens = None
dispatcher = None


class AdminVirtualHosts(AdminServerMixin, AdminVirtualHostsCommon):
    pass
register(AdminVirtualHosts)


class VirtualHost(ObjectServerMixin, VirtualHostCommon):
    strings = None
    strings_keyType = 'string'
    strings_type = 'mapping'
    strings_valueType = 'string'

    def checkModifyIsPossible(self, changes, givenSlotNames = None):
        ObjectServerMixin.checkModifyIsPossible(
            self, changes, givenSlotNames = givenSlotNames)
        objectsByHostName = self.getServer().virtualServer.objectsByHostName
        if (not givenSlotNames or 'hostName' in givenSlotNames) \
               and changes.hostName != self.hostName \
               and changes.hostName is not None:
            if objectsByHostName.has_key(changes.hostName) \
               and changes.id != objectsByHostName[changes.hostName].id:
                raise faults.DuplicateHostName(changes.hostName)

    def importFromXmlRpc(self, virtualHostImport, parentSlot = None):
        hostName = self.hostName
        VirtualHostCommon.importFromXmlRpc(
            self, virtualHostImport, parentSlot = parentSlot)
        objectsByHostName = self.getServer().virtualServer.objectsByHostName
        if self.hostName != hostName:
            if hostName is not None:
                del objectsByHostName[hostName]
            if self.hostName is not None:
                if objectsByHostName.has_key(self.hostName) \
                   and self.id != objectsByHostName[self.hostName].id:
                    raise faults.DuplicateHostName(self.hostName)
                objectsByHostName[self.hostName] = self

    def modify(self, changes, givenSlotNames = None):
        objectsByHostName = self.getServer().virtualServer.objectsByHostName
        hostName = self.hostName
        ObjectServerMixin.modify(
            self, changes, givenSlotNames = givenSlotNames)
        if self.hostName != hostName:
            if hostName is not None:
                del objectsByHostName[hostName]
            if self.hostName is not None:
                objectsByHostName[self.hostName] = self
register(VirtualHost)


class VirtualHostsVirtualServer(ObjectsVirtualServer):
    objectsByHostName = None
        
    def init(self, server, virtualServerId):
        ObjectsVirtualServer.init(self, server, virtualServerId)
        self.objectsByHostName = {}

    def initFromOldData(self, server, virtualServerId, data):
        ObjectsVirtualServer.initFromOldData(
            self, server, virtualServerId, data)
        self.objectsByHostName = {}
        for object in self.objects.values():
            if object.hostName is not None:
                self.objectsByHostName[object.hostName] = object


class VirtualHostsServer(ObjectsServer):
    adminClassName = 'AdminVirtualHosts'
    objectClassName = 'VirtualHost'
    VirtualServer = VirtualHostsVirtualServer
    version = __version__
    virtualServer = None

    def addObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                        objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            result = ObjectsServer.addObjectXmlRpc(self, virtualServerId,
                                    clientToken, userToken, objectImport)
            virtualServer = self.getVirtualServer(virtualServerId)
            object = virtualServer.objects[result]
            self.updateApacheVHost(object)
            return result
        finally:
            self.pullContext()

    def convertVirtualServersIds(
        self, sourceDispatcherId, destinationDispatcherId):
        self.virtualServer.convertIds(
            sourceDispatcherId, destinationDispatcherId)
        self.saveVirtualServer(self.virtualServer)
        return None

    def exportVirtualServer(self, virtualServerId, exportDirectoryPath):
        return None

    def getHostNameXmlRpc(self, virtualServerId, clientToken, userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            dispatcherId = commonTools.extractDispatcherId(
                virtualServerId).lower()
            for object in virtualServer.objects.values():
                if dispatcherId == commonTools.extractDispatcherId(
                    object.defaultDispatcherId).lower():
                    return utf8(object.hostName)
            url = getConfig(
                'Mail', 'Website', default='http://localhost').lower()
            if url.startswith('http://'):
                url = url[len('http://'):]
            return utf8(url)
        finally:
            self.pullContext()

    def getObjectByDispatcherIdXmlRpc(self, virtualServerId, clientToken,
                                  userToken, dispatcherName):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            dispatcherName = iso8859_15(dispatcherName)
            for object in virtualServer.objects.values():
                if object.defaultDispatcherId == dispatcherName:
                    return object.exportToXmlRpc()
            raise faults.MissingItem(dispatcherName)
        finally:
            self.pullContext()

    def getObjectByHostNameXmlRpc(self, virtualServerId, clientToken,
                                  userToken, hostName):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            hostName = iso8859_15(hostName)
            if not virtualServer.objectsByHostName.has_key(hostName):
                for k in virtualServer.objectsByHostName.keys():
                    if fnmatch(hostName, k):
                        hostName = k
                        break
                else:
                    raise faults.MissingItem(hostName)
            object = virtualServer.objectsByHostName[hostName]
            return object.exportToXmlRpc()
        finally:
            self.pullContext()

    def getVirtualServer(self, virtualServerIdNotUsed):
        return self.virtualServer

    def importVirtualServer(self, virtualServerId, importDirectoryPath):
        return None

    def load(self):
        self.virtualServer = self.loadVirtualServer(None)

    def loadVirtualServer(self, virtualServerIdNotUsed):
        return ObjectsServer.loadVirtualServer(self, None)

    def modifyObjectXmlRpc(self, virtualServerId, clientToken, userToken,
                           objectImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            result = ObjectsServer.modifyObjectXmlRpc(self, virtualServerId,
                                            clientToken, userToken, objectImport)
            id = objectImport['id']
            virtualServer = self.getVirtualServer(virtualServerId)
            object = virtualServer.objects[id]
            self.updateApacheVHost(object)
            return result
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('getHostName', self.getHostNameXmlRpc)
        self.registerXmlRpcMethod('getObjectByDispatcherId',
                                  self.getObjectByDispatcherIdXmlRpc)
        self.registerXmlRpcMethod('getObjectByHostName',
                                  self.getObjectByHostNameXmlRpc)

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 5001:
            changed = 1
            virtualServer.objectsByHostName = {}
            for object in virtualServer.objects.values():
                if object.hostName is not None:
                    virtualServer.objectsByHostName[object.hostName] = object
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1009000:
            admin = virtualServer.admin
            adminId = '%s/%s/__admin__' % (
                self.dispatcherIds[0], self.applicationRole)
            if admin.id != adminId:
                changed = 1
                admin.id = adminId
        if changed:
            self.saveVirtualServer(virtualServer)

    def updateApacheVHost(self, object):
        hostName = object.hostName
        templateFileName = '%s/%s/apache-template.conf' % (self.dataDirectoryPath, applicationRole)
        if not os.path.exists(templateFileName):
            return
        if not os.path.exists('%s/%s/apache' % (self.dataDirectoryPath, applicationRole)):
            try:
                os.mkdir('%s/%s/apache' % (self.dataDirectoryPath, applicationRole))
            except OSError:
                return
            
        apacheConfFileName = '%s/%s/apache/vhost-%s.conf' % (self.dataDirectoryPath, applicationRole, hostName)
        try:
            open(apacheConfFileName, 'w').write(
                open(templateFileName).read() % {'hostName': hostName})
        except IOError:
            pass

virtualHostsServer = VirtualHostsServer()


if __name__ == "__main__":
    virtualHostsServer.launch(applicationName, applicationRole)
