#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Votes Server"""

__version__ = '$Revision: 1.32 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.VotesCommon import *
import glasnost.common.faults as faults
import glasnost.common.xhtmlgenerator as X

from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerWithoutWritersMixin, ObjectsServer
from glasnost.server.tools import *

from glasnost.proxy.CacheProxy import invalidateValue
from glasnost.proxy.DispatcherProxy import getApplicationId


applicationName = 'VotesServer'
applicationRole = 'votes'
applicationTokens = None
dispatcher = None


### FIXME: To remove ASAP.
mainModule = sys.modules['__main__']


class AdminVotes(AdminServerWithoutWritersMixin, AdminVotesCommon):
    pass
register(AdminVotes)


class VoteMixin(ObjectServerMixin):
    pass


class AbstractVote(VoteMixin, AbstractVoteCommon):
    pass
register(AbstractVote)


class VoteApproval(VoteMixin, VoteApprovalCommon):
    pass
register(VoteApproval)


class VoteDistribution(VoteMixin, VoteDistributionCommon):
    pass
register(VoteDistribution)


class VoteRanking(VoteMixin, VoteRankingCommon):
    pass
register(VoteRanking)


class VoteRating(VoteMixin, VoteRatingCommon):
    pass
register(VoteRating)


class VotesServer(ObjectsServer):
    adminClassName = 'AdminVotes'
    objectClassName = 'AbstractVote'
    useAdminReadersSet = 0
    useAdminWritersSet = 0
    version = __version__

    def addObjectXmlRpc(self, virtualServerId, clientToken, objectImport):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            clientId = getApplicationId(clientToken)
            clientNameAndPort, clientRole = splitApplicationId(clientId)
            if clientRole != 'ballots':
                raise faults.ApplicationAccessDenied(clientId)
            virtualServer = self.getVirtualServer(virtualServerId)
            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)
            object.checkAddIsPossible()
            object.setAutomaticalSlots()
            object.id = self.getNextId()
            virtualServer.objects[object.id] = object
            object.saveNonCore()
            object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            return object.id
        finally:
            self.pullContext()

    def deleteObject(self, virtualServerId, clientToken, objectId):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            clientId = getApplicationId(clientToken)
            clientNameAndPort, clientRole = splitApplicationId(clientId)
            if clientRole != 'ballots':
                raise faults.ApplicationAccessDenied(clientId)
            virtualServerId = self.computeVirtualServerId(objectId)
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            del virtualServer.objects[objectId]
            #object.removeNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(objectId)
            return objectId # We have to return something.
        finally:
            self.pullContext()

    # The real XML-RPC function is getObjectXmlRpc.
    def getObjectCore(self, objectId):
        clientId = getApplicationId(
            context.getVar('clientToken', default = ''))
        clientNameAndPort, clientRole = splitApplicationId(clientId)
        if clientRole != 'ballots':
            raise faults.ApplicationAccessDenied(clientId)
        virtualServerId = self.computeVirtualServerId(objectId)
        virtualServer = self.getVirtualServer(virtualServerId)
        if not virtualServer.objects.has_key(objectId):
            raise faults.MissingItem(objectId)
        object = virtualServer.objects[objectId]
        return object

    def getObjectXmlRpc(self, virtualServerId, clientToken, objectId):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            object = self.getObjectCore(objectId)
            object.acquireNonCore()
            try:
                result = object.exportToXmlRpc()
            finally:
                object.releaseNonCore()
            return result
        finally:
            self.pullContext()

    def getPartialObjectXmlRpc(self, virtualServerId, clientToken, objectId,
                               requiredSlotNames):
        if not requiredSlotNames:
            return self.getObjectXmlRpc(virtualServerId, clientToken, objectId)
        self.pushContext(virtualServerId, clientToken, None)
        try:
            object = self.getObjectCore(objectId)
            useNonCore = 0
            if not 'id' in requiredSlotNames \
               and object.isSlotName('id') \
               and object.getSlot('id').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('id')
            if not 'language' in requiredSlotNames \
               and object.isSlotName('language') \
               and object.getSlot('language').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('language')
            if not 'readersSet' in requiredSlotNames \
               and object.isSlotName('readersSet') \
               and object.getSlot('readersSet').getKind().isPublicForXmlRpc:
                requiredSlotNames.append('readersSet')
            for slotName in object.getSlotNames():
                if not slotName in requiredSlotNames:
                    continue
                slot = object.getSlot(slotName)
                kind = slot.getKind()
                if not kind.isInCore(slot):
                    useNonCore = 1
                    break
            if useNonCore:
                object.acquireNonCore()
            try:
                result = object.exportToXmlRpc(
                    requiredSlotNames = requiredSlotNames)
            finally:
                if useNonCore:
                    object.releaseNonCore()
            return result
        finally:
            self.pullContext()

    def hasObject(self, virtualServerId, clientToken, objectId):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            clientId = getApplicationId(clientToken)
            clientNameAndPort, clientRole = splitApplicationId(clientId)
            if clientRole != 'ballots':
                raise faults.ApplicationAccessDenied(clientId)
            virtualServerId = self.computeVirtualServerId(objectId)
            virtualServer = self.getVirtualServer(virtualServerId)
            return virtualServer.objects.has_key(objectId)
        finally:
            self.pullContext()

    def modifyObjectXmlRpc(self, virtualServerId, clientToken, objectImport):
        self.pushContext(virtualServerId, clientToken, None)
        try:
            clientId = getApplicationId(clientToken)
            clientNameAndPort, clientRole = splitApplicationId(clientId)
            if clientRole != 'ballots':
                raise faults.ApplicationAccessDenied(clientId)
            objectChanges = self.newThing(
                'object', objectImport['__thingName__'])
            objectChanges.importFromXmlRpc(objectImport)
            virtualServerId = self.computeVirtualServerId(objectChanges.id)
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectChanges.id):
                raise faults.MissingItem(objectChanges.id)
            object = virtualServer.objects[objectChanges.id]
            object.acquireNonCore()
            try:
                object.checkModifyIsPossible(objectChanges)
                object.modify(objectChanges)
                object.saveNonCore()
            finally:
                object.releaseNonCore()
            self.saveVirtualServer(virtualServer)
            invalidateValue(object.id)
            return object.version
        finally:
            self.pullContext()

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version < 4000:
            changed = virtualServer.admin.repair(4000) or changed
            for id, object in virtualServer.objects.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    del virtualServer.objects[id]
                    virtualServer.objects[newId] = object
                changed = object.repair(4000) or changed
                if not object.__dict__.has_key('language'):
                    changed = 1
                    object.language = 'fr'
        if version < 5004:
            changed = virtualServer.admin.repair(5004) or changed
            for id, object in virtualServer.objects.items():
                changed = object.repair(5004) or changed
        if version <= 1015000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if version <= 1019000:
            for object in virtualServer.objects.values():
                if object.token is not None \
                       and not object.token.startswith('glasnost://'):
                    changed = 1
                    object.token = '%s/%s' % (
                        virtualServer.virtualServerId, object.token)
        if changed:
            self.saveVirtualServer(virtualServer)


votesServer = VotesServer()


if __name__ == "__main__":
    votesServer.launch(applicationName, applicationRole)
