#!/usr/bin/env python
#
# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Workflows Server"""

__version__ = '$Revision: 1.22 $'[11:-2]


import __builtin__
__builtin__.__dict__['_'] = lambda x: x
__builtin__.__dict__['N_'] = lambda x: x

import md5
import copy

import glasnost as glasnost # Do not edit or remove this line!

from glasnost.common.WorkflowsCommon import *
import glasnost.common.faults as faults
import glasnost.common.xhtmlgenerator as X

import actions

import glasnost.server.kinds
from glasnost.server.ObjectsServer import register, ObjectServerMixin, AdminServerMixin, ObjectsServer
from glasnost.server.tools import *
from glasnost.proxy.DispatcherProxy import getApplicationId

## from glasnost.proxy.DataflowsProxy import dataflowsProxy
from glasnost.proxy.tools import getProxy, getProxyForServerRole


virtualServerId = None
applicationName = 'WorkflowsServer'
applicationRole = 'workflows'
applicationTokens = None
dispatcher = None

class AdminWorkflows(AdminServerMixin, AdminWorkflowsCommon):
    pass
register(AdminWorkflows)


class Workflow(ObjectServerMixin, WorkflowCommon):
    pass
register(Workflow)

register(actions.WorkflowAction)
register(actions.WorkflowActionAddObject)
register(actions.WorkflowActionBranch)
register(actions.WorkflowActionEditObject)
register(actions.WorkflowActionEnd)
register(actions.WorkflowActionGoto)
register(actions.WorkflowActionMail)
register(actions.WorkflowActionShowMessage)
register(actions.WorkflowActionUserChoice)
register(actions.WorkflowActionUserData)

RWState = actions.RWState


class RunningWorkflow(ObjectServerMixin, RunningWorkflowCommon):
    workflow = None
    userId = None

    def init(self, workflow, virtualServerId, data=None):
        self.state = 'initData'
        # self.state will advance to [0]->[n], [i,0]->[i,m], [i,j,0]->[i,j,m] etc.
        self.userId = getProxyForServerRole('authentication').getUserId()
        self.virtualServerId = virtualServerId
        self.workflow = workflow
        self.workflowName = workflow.getLabel() # <- this is not wise for i18n
        self.data = {}
        if data:
            self.feedData(data)
        self.state = [ RWState(self.workflow.actions, self.workflow.actions[0]) ]

    def feedData(self, data):
        if type(data) is type('') and data.startswith('glasnost://'):
            #data = GlasnostObject(data)
            data = getProxy(data).getObject(data)
        
        if type(self.state) is type(''):
            key = self.state
        elif self.getAction().actionName:
            key = self.getAction().actionName
        elif type(self.state) is type([]):
            key = 's'+ '_'.join([str(x) for x in self.state])
        else:
            key = repr(self.state)
        self.data[key] = data

    def getAction(self):
        action = copy.copy(self.state[-1].action)
        for slotName in action.getSlotNames():
            slot = action.getSlot(slotName)
            kind = slot.getKind()
            if not kind.isPublicForXmlRpc:
                continue
            if not kind.__class__ in [glasnost.server.kinds.String,
                                      glasnost.server.kinds.Id]:
                continue
            value = slot.getValue()
            if not type(value) is type(''):
                continue
            parts = re.findall('<<.*?>>', value)
            for p in parts:
                v = eval(p[2:-2], self.data)
                print p, v
                value = value.replace(p, v, 1)
            slot.setValue(value)
        return action

    def isFinished(self):
        return 0
        dataflow = dataflowsProxy.getObject(self.workflow.dataflowId)
        return self.state[0] == len(dataflow.actions)-1

    def advance(self, **keywords):
        currentAc = self.state[-1].action
        if currentAc and hasattr(currentAc, 'action'):
            currentAc.action(self)
        action = None
        while self.state:
            currentWf = self.state[-1].workflowActions
            currentAc = self.state[-1].action
            try:
                indexAct = currentWf.index(currentAc)
            except:
                print 'starting branch'
                indexAct = -1
            indexAct += 1
            if indexAct == len(currentWf):
                # end of branch
                self.state = self.state[:-1]
            else:
                try:
                    action = currentWf[indexAct]
                except IndexError:
                    raise
                self.state[-1].action = action
                action = self.getAction()
                if not action.noUserInteraction:
                    break
                try:
                    action.action(self)
                except:
                    print 'action (%s) failed' % action
                    # if this fails we might be in a bad situation
                    # but this is not a good reason to continue silently...
                    pass
        
        if not action:
            raise Exception('Aie Aie')
            
        return
        

class WorkflowsServer(ObjectsServer):
    adminClassName = 'AdminWorkflows'
    objectClassName = 'Workflow'
    version = __version__

    def canRunWorkflow(self, virtualServerId, clientToken, userToken,
                       objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                return 0
            object = virtualServer.objects[objectId]
            if self.isAdmin(virtualServerId, clientToken, userToken):
                return 1
            if not object.runnersSet or \
                getProxyForServerRole('authentication').setContainsUser(object.runnersSet):
                return 1
            return 0
        finally:
            self.pullContext()

    def initWorkflow(self, virtualServerId, clientToken, userToken, objectId,
                     dataObjectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            workflow = virtualServer.objects[objectId]
            if not self.canRunWorkflow(virtualServerId, clientToken, userToken,
                                       objectId):
                raise faults.UserAccessDenied()
            userId = getProxyForServerRole('authentication').getUserId()

            key = '%s:%s' % (userId, objectId)
                # this won't allow a user to concurrently run several workflow
                # of the same kind. Will fix later.
            if not hasattr(virtualServer, 'runningWorkflow'):
                virtualServer.runningWorkflow = {}

            if not dataObjectId:
                dataObjectId = None

            # for now we overwrite the possibly existing previous run
            rW = RunningWorkflow()
            rW.init(workflow, virtualServerId, dataObjectId)
            virtualServer.runningWorkflow[key] = rW

            return '' # we have to return sth
        finally:
            self.pullContext()

    def getRunningWorkflow(self, virtualServerId, clientToken, userToken,
                           objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            workflow = virtualServer.objects[objectId]
            userId = getProxyForServerRole('authentication').getUserId()

            key = '%s:%s' % (userId, objectId)
            try:
                rW = virtualServer.runningWorkflow[key]
            except:
                raise faults.MissingItem(objectId)
            result = rW.getAction().exportToXmlRpc()
            return result
        finally:
            self.pullContext()

    def getRunningWorkflowsIdsForUser(self, virtualServerId, clientToken,
                                      userToken):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            userId = getProxyForServerRole('authentication').getUserId()
            if not hasattr(virtualServer, 'runningWorkflow'):
                virtualServer.runningWorkflow = {}

            result = []
            for k,v in virtualServer.runningWorkflow.items():
                if k.split(':')[0] == userId:
                    result.append(k)
            return result
        finally:
            self.pullContext()

    def advanceRunningWorkflow(self, virtualServerId, clientToken, userToken,
                               objectId, dataImport):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            workflow = virtualServer.objects[objectId]
            userId = getProxyForServerRole('authentication').getUserId()
            key = '%s:%s' % (userId, objectId)
            try:
                rW = virtualServer.runningWorkflow[key]
            except:
                raise faults.MissingItem(objectId)

            if rW.isFinished():
                del virtualServer.runningWorkflow[key]
                raise faults.Fault(0, 'WorkflowFinished')

            if not dataImport.has_key('isPublicForXmlRpc'):
                t = dataImport['__thingName__'].lower() + 's'
                    # ugly and won't work everywhere
                proxy = getProxyForServerRole(t)
                data = proxy.newThing('object', dataImport['__thingName__'])
                data.importFromXmlRpc(dataImport)
            else:
                data = self.newThing('kind', dataImport['__thingName__'])
                data.value_kind = data
                data.isPublicForXmlRpc = 1
                data.importFromXmlRpc(dataImport)
                data = data.value
            rW.feedData(data)
            rW.advance()
            return '' # we have to return something
        finally:
            self.pullContext()

    def isFinished(self, virtualServerId, clientToken, userToken, objectId):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            if not virtualServer.objects.has_key(objectId):
                raise faults.MissingItem(objectId)
            workflow = virtualServer.objects[objectId]
            userId = getProxyForServerRole('authentication').getUserId()
            key = '%s:%s' % (userId, objectId)
            try:
                rW = virtualServer.runningWorkflow[key]
            except:
                raise faults.MissingItem(objectId)

            result = rW.isFinished()
            return result
        finally:
            self.pullContext()

    def getWorkflowsIdsForServerRole(self, virtualServerId, clientToken,
                                     userToken, serverRole):
        self.pushContext(virtualServerId, clientToken, userToken)
        try:
            virtualServer = self.getVirtualServer(virtualServerId)
            isAdmin = self.isAdmin(virtualServerId, clientToken, userToken)
            clientId = getApplicationId(clientToken)
            result = []
            for objectId, objectCore in virtualServer.objects.items():
                if objectCore.startupObjectType == serverRole \
                       and (self.canGetObject(
                    virtualServerId, clientToken, userToken, objectId) \
                            or objectCore.canBeGottenByClient(clientId)):
                    result.append(objectId)
            return result
        finally:
            self.pullContext()

    def registerXmlRpcMethods(self):
        ObjectsServer.registerXmlRpcMethods(self)
        self.registerXmlRpcMethod('canRunWorkflow')
        self.registerXmlRpcMethod('initWorkflow')
        self.registerXmlRpcMethod('isFinished')
        self.registerXmlRpcMethod('getRunningWorkflow')
        self.registerXmlRpcMethod('getRunningWorkflowsIdsForUser')
        self.registerXmlRpcMethod('advanceRunningWorkflow')
        self.registerXmlRpcMethod('getWorkflowsIdsForServerRole')

    def repairVirtualServer(self, virtualServer, version):
        changed = 0
        if version <= 1007000:
            admin = virtualServer.admin
            if admin.id is None:
                changed = 1
                admin.id = '%s/__admin__' % virtualServer.virtualServerId
        if changed:
            self.saveVirtualServer(virtualServer)


workflowsServer = WorkflowsServer()

if __name__ == "__main__":
    workflowsServer.launch(applicationName, applicationRole)

