# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Workflows Actions for Workflows Server"""

__version__ = '$Revision: 1.10 $'[11:-2]


from glasnost.server.ObjectsServer import ObjectServerMixin
from glasnost.server.tools import *

from glasnost.proxy.tools import getProxy, getProxyForServerRole
from glasnost.common.WorkflowsCommon import *

from glasnost.proxy.GroupsProxy import getSetContainedIds

class RWState:
    action = None
    workflow = None
    def __init__(self, workflowActions, action):
        self.action = action
        self.workflowActions = workflowActions


class WorkflowAction(ObjectServerMixin, WorkflowActionCommon):
    noUserInteraction = 0
    def action(self, rW):
        pass


class WorkflowActionAddObject(WorkflowAction, WorkflowActionAddObjectCommon):
    def action(self, rW):
        data = rW.data[self.actionName]
        t = data.__class__.__name__.lower() + 's' # ugly and won't work everywhere
        proxy = getProxyForServerRole(t)
        data.id = proxy.addObject(data, serverId = rW.virtualServerId)

class WorkflowActionBranch(WorkflowAction, WorkflowActionBranchCommon):
    noUserInteraction = 1
    def action(self, rW):
        res = eval(self.condition, {})
        if res:
            actions = self.actionsTrue
        else:
            actions = self.actionsFalse
        rW.state.append( RWState(actions, None) )

class WorkflowActionEditObject(WorkflowAction, WorkflowActionEditObjectCommon):
    noUserInteraction = 1
    def action(self, rW):
        objectProxy = getProxy(self.objectId)
        object = objectProxy.getObject(self.objectId)
        slot = object.getSlot(self.fieldToChange)
        fieldValue = slot.getValue()
        if self.operation == 'append':
            if fieldValue is None:
                fieldValue = []
            fieldValue.append(self.value)
        elif self.operation == 'prepend':
            if fieldValue is None:
                fieldValue = []
            fieldValue.insert(0, self.value)
        elif self.operation == 'set':
            fieldValue = self.value
        else:
            print 'Unknown operation:', self.operation
        slot.setValue(fieldValue)
        objectProxy.modifyObject(object)

class WorkflowActionEnd(WorkflowAction, WorkflowActionEndCommon):
    pass

class WorkflowActionGoto(WorkflowAction, WorkflowActionGotoCommon):
    noUserInteraction = 1
    def action(self, rW):
        i = len(rW.state)-1
        action = None
        while i+1:
            actions = rW.state[i].workflowActions
            t = [ x for x in actions if x.actionName == self.gotoActionName ]
            if len(t) == 0:
                i -= 1
                continue
            action = t[0]
            break
        if not action:
            raise Exception('Aie Aie, going nowhere')
        
        indexAct = actions.index(action)
        if indexAct == 0:
            action = None
        else:
            action = actions[indexAct-1]
        rW.state = [ RWState(actions, action) ]


class WorkflowActionMail(WorkflowAction, WorkflowActionMailCommon):
    noUserInteraction = 1
    def action(self, rW):
        peopleProxy = getProxyForServerRole('people')
        sendersIds = getSetContainedIds(
                    self.recipientsSet, ['people'],
                    serverId = rW.virtualServerId)
        toAddresses = []
        for senderId in sendersIds:
            sender = peopleProxy.getObject(senderId, serverId = rW.virtualServerId)
            if sender.email:
                toAddresses.append(sender.email)
        
        if not toAddresses:
            return

        sendMail(
            mailFrom = self.sender,
            mailTo = toAddresses,
            mailSubject = self.subject,
            mailMessage = self.message)

class WorkflowActionShowMessage(WorkflowAction, WorkflowActionShowMessageCommon):
    pass

class WorkflowActionUserChoice(WorkflowAction, WorkflowActionUserChoiceCommon):
    pass

class WorkflowActionUserData(WorkflowAction, WorkflowActionUserDataCommon):
    pass

