# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Accounts Common Models"""

__version__ = '$Revision: 1.15 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon
import accounting


class AdminAccountsCommon(AdminCommon):
    pass


class AccountCommon(ObjectCommon):
    balance = None
    balance_kind_isPublicForXmlRpc = 1
    balance_kindName = 'Amount'
    
    currency = None
    currency_kind_useCustomStorage = 1    
    currency_kind_isPublicForXmlRpc = 1
    currency_kind_isRequired = 1
    currency_kind_values = accounting.currencyValues    
    currency_kindName = 'Choice'

    label = None
    label_kind_isPublicForXmlRpc = 1
    label_kind_isRequired = 1
    label_kindName = 'String'

    language_kindName = None

    ledgerId = None
    ledgerId_kind_isPublicForXmlRpc = 1
    ledgerId_kind_isRequired = 1
    ledgerId_kind_serverRoles = ['ledgers']
    ledgerId_kindName = 'Id'

    members = None
    members_kind_hasToModify = 0    
    members_kind_isPublicForXmlRpc = 1    
    members_kind_useCustomStorage = 1    
    members_kind_requiredCount = 0
    # localId, date, certainty, amount
    members_kind_itemKind_value_itemKind_valueName = 'Integer'
    members_kind_itemKind_value_requiredCount = 0    
    members_kind_itemKind_valueName = 'Sequence'
    members_kindName = 'Sequence'
    
    number = None
    number_kind_isPublicForXmlRpc = 1
    number_kind_isRequired = 1
    number_kind_isTranslatable = 0
    number_kindName = 'String'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    startDate = None
    startDate_kind_useCustomStorage = 1    
    startDate_kind_isPublicForXmlRpc = 1
    startDate_kindName = 'Date'
    
    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def canCache(self):
        return 0

    def getLabel(self):
        label = self.label
        if label is None:
            label = ''
        number = self.number
        if number is None:
            number = ''
        if not label and not number:
            return ''
        return number + ". " + label 

    def getLanguage(self):
        language = self.language
        if language is None:
            return ''
        return language
    
    def getNumber(self):
        number = self.number
        if number is None:
            number = '00'
        return number
            
    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'ledgerId', 'startDate', 'number', 'label', 'currency',
            'balance', 'writersSet', 'readersSet']
        return slotNames


class AccountsCommon:
    adminClassName = 'AdminAccounts'
    newObjectNameCapitalized = N_('New Account')
    objectClassName = 'Account'
    objectName = N_('account')
    objectNameCapitalized = N_('Account')
    objectsName = N_('accounts')
    objectsNameCapitalized = N_('Accounts')
    serverRole = 'accounts'

