# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Appointments Common Models"""

__version__ = '$Revision: 1.20 $'[11:-2]


import time
import calendar

from ObjectsCommon import AdminCommon, ObjectCommon

ONE_MINUTE = 60
ONE_HOUR = ONE_MINUTE * 60
ONE_DAY = ONE_HOUR * 24
ONE_WEEK = ONE_DAY * 7
ONE_MONTH = ONE_DAY * 28
ONE_YEAR = ONE_DAY * 365

MONTHS = ['', 'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
DAYS = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']

class AdminAppointmentsCommon(AdminCommon):
    pass


class AppointmentCommon(ObjectCommon):
    isNotified = 0
    
    authorsSet = None
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    beginningDate = None
    beginningDate_kind_isPublicForXmlRpc = 1
    beginningDate_kind_isRequired = 1
    beginningDate_kindName = 'Time'

    body = None
    body_kind_isPublicForXmlRpc = 1
    body_kindName = 'String'

    category = None
    category_kind_isPublicForXmlRpc = 1
    category_kind_isRequired = 1
    category_kindName = 'String'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    ## duration in seconds
    duration = 0
    duration_kind_requiredParts = ['hour', 'minute']
    duration_kind_isPublicForXmlRpc = 1
    duration_kind_isRequired = 1
    duration_kindName = 'Duration'
    
    endingDate = None
    endingDate_kind_isPublicForXmlRpc = 1
    endingDate_kindName = 'Time'

    lastEditorId = None
    lastEditorId_kind_hasToModify = 0
    lastEditorId_kind_isPublicForXmlRpc = 1
    lastEditorId_kind_serverRoles = ['people']
    lastEditorId_kindName = 'Id'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    rememberTime = 0
    rememberTime_kind_requiredParts = ['day', 'hour', 'minute']
    rememberTime_kind_isPublicForXmlRpc = 1
    rememberTime_kind_isRequired = 1
    rememberTime_kindName = 'Duration'

    repetition = 'none'
    repetition_kind_isPublicForXmlRpc = 1
    repetition_kind_isRequired = 1
    repetition_kind_values = [
        'none',
        'daily',
        'weekly',
        'monthly',
        'yearly'
        ]
    repetition_kindName = 'Choice'

    repetitionDay = None
    repetitionDay_kind_isPublicForXmlRpc = 1
    repetitionDay_kind_itemKind_valueName = 'Choice'
    repetitionDay_kind_itemKind_value_values = DAYS
    repetitionDay_kindName = 'Sequence'

    resourcesSet = None
    resourcesSet_kind_isPublicForXmlRpc = 1
    resourcesSet_kind_itemKind_valueName = 'Id'
    resourcesSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    resourcesSet_kind_requiredCount = 1
    resourcesSet_kindName = 'Sequence'

    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def canCache(self):
        return 1

    def getCategory(self):
        result = self.category
        if result is None:
            result = ''
        return result

    def getLabel(self):
        label = self.getTitle()
        if label is None:
            return ''
        hourTime = self.getBeginningHourAndMinute()
        if label and hourTime:
            return '%s - %s' % (hourTime, label)
        return label

    def getTitle(self):
        return self.title

    def dateFor(self, beginningDate, endingDate):
        """
        Verify appointment is valid for this date.
        @param date: date 
        """
        if self.dateIn(beginningDate, endingDate):
            ##verify for repetition appointment, day, month, year, ...
            if self.repetition == 'none' or not self.endingDate:
                return 1
            elif self.repetition == 'daily':
                ## verify the hour
                result = self.hourIn(beginningDate, endingDate)
                return result
            elif self.repetition == 'weekly':
                result = self.dayIn(beginningDate, endingDate)
                result = result and self.hourIn(beginningDate, endingDate)
                return result
            elif self.repetition == 'monthly':
                result = self.dayNumberIn(beginningDate, endingDate)
                result = result and self.hourIn(beginningDate, endingDate)
                return result
            elif self.repetition == 'yearly':
                result = self.dayMonthIn(beginningDate, endingDate)
                result = result and self.hourIn(beginningDate, endingDate)
                return result
        return 0

    def dateIn(self, beginningDate, endingDate=None):
        start = self.beginningDate
        if self.endingDate and self.repetition != 'none':
            end = self.endingDate
        else:
            end = self.beginningDate + self.duration
            
        return ((not beginningDate or beginningDate <= start)and \
                (not endingDate or start <= endingDate)) or \
                ((not beginningDate or beginningDate <= end) and \
                 (not endingDate or end <= endingDate)) or \
                 ((not beginningDate or start <= beginningDate) and \
                  (not endingDate or endingDate <= end))
        
    def hourIn(self, beginningDate, endingDate):
        startHourEvent, startMinuteEvent = time.localtime(self.beginningDate)[3:5]
        startEvent = (startHourEvent * 60 + startMinuteEvent) * 60
        endEvent = startEvent + self.duration
        
        startHour, startMinute = time.localtime(beginningDate)[3:5]
        start =  (startHour * 60 + startMinute) * 60
        end = start + endingDate - beginningDate
        
        result = ((start <= startEvent) and (startEvent < end)) or \
                 ((start < endEvent) and (endEvent < end)) or \
                 ((startEvent <= start) and (end < endEvent))
###        open("/tmp/repet","a").write(repr(('hourIn', self.getLabel(), beginningDate, endingDate, self.beginningDate, self.endingDate, startHourEvent, startMinuteEvent, startEvent, endEvent, startHour, startMinute, start, end, result))+'\n')
        return result

    def dayIn(self, beginningDate, endingDate):
        startDay = time.localtime(beginningDate)[6]
        endDay =  time.localtime(endingDate)[6]
        currentDay = startDay
        while DAYS[currentDay] not in self.repetitionDay and \
                  currentDay != endDay:
            currentDay = (currentDay + 1) % 7
        result = DAYS[currentDay] in self.repetitionDay
###        open("/tmp/repet","a").write(repr(('dayIn', self.getLabel(), startDay, endDay, DAYS[startDay], DAYS[endDay], self.repetitionDay, result))+'\n')
        return result

    def dayNumberIn(self, beginningDate, endingDate):
        dayNumberEvent = time.localtime(self.beginningDate)[2]
        dayNumberList = []
        while beginningDate <= endingDate:
            dayNumber = time.localtime(beginningDate)[2]
            dayNumberList.append(dayNumber)
            beginningDate = beginningDate + 24 * 60 * 60 ## add one day
        result = dayNumberEvent in dayNumberList
###        open("/tmp/repet","a").write(repr(('dayNumberIn', self.getLabel(), dayNumberEvent, dayNumberList, result))+'\n')
        return result

    def dayMonthIn(self, beginningDate, endingDate):
        monthEvent, dayEvent = time.localtime(self.beginningDate)[1:3]
        validDay = []
        while beginningDate <= endingDate:
            month, day = time.localtime(beginningDate)[1:3]
            validDay.append((month, day))
            beginningDate = beginningDate + 24 * 60 * 60 ## add one day
        result = (monthEvent, dayEvent) in validDay
###        open("/tmp/repet","a").write(repr(('dayMonthIn', self.getLabel(), monthEvent, dayEvent, validDay, result))+'\n')
        return result
       
    def getBeginningHourAndMinute(self):
        """Returns a string with the start time of the appointment (or None
           if no time set)"""
        if not self.duration:
            return None
        return '%02d:%02d' % time.localtime(self.beginningDate)[3:5]

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'category',
            'title',
            'creationTime',
            'modificationTime',
            'beginningDate',
            'duration',
            'endingDate',
            'repetition',
            'repetitionDay',
            'body',
            'rememberTime',
            'resourcesSet',
            'authorsSet',
            'lastEditorId',
            'writersSet',
            'readersSet',
            ]
        return slotNames

class AppointmentsCommon:
    adminClassName = 'AdminAppointments'
    objectClassName = 'Appointment'
    newObjectNameCapitalized = N_('New Appointment')
    objectName = N_('appointment')
    objectNameCapitalized = N_('Appointment')
    objectsName = N_('appointments')
    objectsNameCapitalized = N_('Appointments')
    serverRole = 'appointments'

