# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Articles Common Models"""

__version__ = '$Revision: 1.18 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminArticlesCommon(AdminCommon):
    pass


class ArticleCommon(ObjectCommon):
    authorsSet = None
    authorsSet_kind_balloonHelp = N_('Select the author(s) of the document.')
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    body = None
    body_kind_balloonHelp = N_('Enter the text of the document.')
    body_kind_isPublicForXmlRpc = 1
    body_kind_isRequired = 1
    body_kind_useCustomStorage = 1
    body_kindName = 'String'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    editionTime = None
    editionTime_kind_hasToModify = 0
    editionTime_kind_isPublicForXmlRpc = 1
    editionTime_kindName = 'Time'

    format = 'spip'
    format_kind_balloonHelp = N_('Select which format you want to type the contents of this document in.  If you are unsure of which format to use, just select SPIP and type the document as you usually do.')
    format_kind_isPublicForXmlRpc = 1
    format_kind_isRequired = 1
    format_kind_values = [
        'docbook',
        'html',
        'spip',
        'text',
        'rst',
        ]
    format_kindName = 'Choice'

    lastEditorId = None
    lastEditorId_kind_hasToModify = 0
    lastEditorId_kind_isPublicForXmlRpc = 1
    lastEditorId_kind_serverRoles = ['people']
    lastEditorId_kindName = 'Id'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    title = None
    title_kind_balloonHelp = N_('Enter the title of the document.')
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def getBody(self):
        return self.body

    def getLabel(self):
        label = self.getTitle()
        if label is None:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'format', 'body', 'creationTime', 'lastEditorId',
            'editionTime', 'modificationTime', 'authorsSet', 'writersSet',
            'readersSet']
        return slotNames

    def getTitle(self):
        return self.title


class ArticlesCommon:
    adminClassName = 'AdminArticles'
    newObjectNameCapitalized = N_('New Article')
    objectClassName = 'Article'
    objectName = N_('article')
    objectNameCapitalized = N_('Article')
    objectsName = N_('articles')
    objectsNameCapitalized = N_('Articles')
    serverRole = 'articles'

