# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Books Common Models"""

__version__ = '$Revision: 1.15 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminBooksCommon(AdminCommon):
    pass


class BookCommon(ObjectCommon):
    appendixIds = None
    appendixIds_kind_isPublicForXmlRpc = 1
    appendixIds_kind_itemKind_valueName = 'Id'
    appendixIds_kind_itemKind_value_serverRoles = ['articles']
    appendixIds_kindName = 'Sequence'

    authorsSet = None
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    body = None
    body_kind_hasToModify = 0
    body_kind_useCustomStorage = 1
    body_kind_isPublicForXmlRpc = 1
    body_kindName = 'String'

    chapterIds = None
    chapterIds_kind_isPublicForXmlRpc = 1
    chapterIds_kind_itemKind_valueName = 'Id'
    chapterIds_kind_itemKind_value_serverRoles = ['articles']
    chapterIds_kindName = 'Sequence'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    itemIds = None
    itemIds_kind_isPublicForXmlRpc = 1
    itemIds_kind_itemKind_valueName = 'Id'
    itemIds_kind_itemKind_value_serverRoles = ['articles']
    itemIds_kindName = 'Sequence'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    nonItemIds = None
    nonItemIds_kind_isPublicForXmlRpc = 1
    nonItemIds_kind_itemKind_valueName = 'Id'
    nonItemIds_kind_itemKind_value_serverRoles = ['articles']
    nonItemIds_kindName = 'Sequence'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def getBody(self):
        return self.body

    def getLabel(self):
        label = self.getTitle()
        if label is None:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'body', 'title', 'chapterIds', 'appendixIds', 'itemIds',
            'nonItemIds', 'creationTime', 'modificationTime', 'authorsSet',
            'writersSet', 'readersSet']
        return slotNames

    def getTitle(self):
        return self.title

class BooksCommon:
    adminClassName = 'AdminBooks'
    newObjectNameCapitalized = N_('New Book')
    objectClassName = 'Book'
    objectName = N_('book')
    objectNameCapitalized = N_('Book')
    objectsName = N_('books')
    objectsNameCapitalized = N_('Books')
    serverRole = 'books'

