# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Breves Common Models"""

__version__ = '$Revision: 1.15 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminBrevesCommon(AdminCommon):
    pass


class BreveCommon(ObjectCommon):
    authorsSet = None
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    body = None
    body_kind_isPublicForXmlRpc = 1
    body_kind_isRequired = 1
    body_kindName = 'String'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    links = None
    links_kind_isPublicForXmlRpc = 1
    links_kind_itemKind_valueName = 'Link'
    links_kindName = 'Sequence'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def bodyGetter(self):
        return self.getBody()

    def getBody(self):
        return self.body

    def getLabel(self):
        label = self.getTitle()
        if label is None:
            return ''
        return label

    def getLinks(self):
        return self.links

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'body', 'links', 'creationTime', 'modificationTime',
            'authorsSet', 'writersSet', 'readersSet']
        return slotNames

    def getTitle(self):
        return self.title

    def linksGetter(self):
        return self.getLinks()

    def titleGetter(self):
        return self.getTitle()


class BreveElectionCommon(BreveCommon):
    body_kind_hasToModify = 0
    body_kind_isPublicForXmlRpc = 0

    electionId = None
    electionId_kind_isPublicForXmlRpc = 1
    electionId_kind_isRequired = 1
    electionId_kind_serverRoles = ['elections']
    electionId_kindName = 'Id'

    electionSubject = None
    electionSubject_kind_isPublicForXmlRpc = 1
    electionSubject_kind_isRequired = 1
    electionSubject_kindName = 'String'

    electionTitle = None
    electionTitle_kind_isPublicForXmlRpc = 1
    electionTitle_kind_isRequired = 1
    electionTitle_kindName = 'String'

    language_kindName = None

    links_kind_hasToModify = 0
    links_kind_isPublicForXmlRpc = 0

    title_kind_hasToModify = 0
    title_kind_isPublicForXmlRpc = 0

    def getLabelLanguage(self):
        return ''

    def getLinks(self):
        return [
            (_('The election "%s"') % self.electionTitle,
             self.electionId),
            ]


class BreveElectionCandidateAddedCommon(BreveElectionCommon):
    candidateName_kind_isPublicForXmlRpc = 1
    candidateName_kind_isRequired = 1
    candidateName_kindName = 'String'

    def getBody(self):
        return _("""\
The candidate "%(candidate)s" has been added to the running election whose subject is:

%(subject)s
""") % {
            'candidate': self.candidateName,
            'subject': self.electionSubject,
            }

    def getTitle(self):
        return _('A candidate has been added to the election "%s".') \
               % self.electionTitle


class BreveElectionClosedCommon(BreveElectionCommon):
    def getBody(self):
        return _("""\
An election is closed.

{The Subject:}

%s
""") % self.electionSubject

    def getTitle(self):
        return _('The election "%s" is closed.') % self.electionTitle


class BreveElectionRunningCommon(BreveElectionCommon):
    def getBody(self):
        return _("""\
An election is running.

{The Subject:}

%s
""") % self.electionSubject

    def getTitle(self):
        return _('The election "%s" is running.') % self.electionTitle

class BrevesCommon:
    adminClassName = 'AdminBreves'
    newObjectNameCapitalized = N_('New Short Article')
    objectClassName = 'Breve'
    objectName = N_('short article')
    objectNameCapitalized = N_('Short Article')
    objectsName = N_('short news')
    objectsNameCapitalized = N_('Short News')
    serverRole = 'breves'

