# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Cards Common Models"""

__version__ = '$Revision: 1.29 $'[11:-2]


import faults
from ObjectsCommon import AdminCommon, ObjectCommon
import properties
import slots


class AdminCardsCommon(AdminCommon):
    pass


class CardCommon(properties.PropertiesContainerMixin, ObjectCommon):
    language_kindName = None

    prototypeIds = None
    prototypeIds_kind_isPublicForXmlRpc = 1
    prototypeIds_kind_itemKind_value_serverRoles = ['cards']
    prototypeIds_kind_itemKind_valueName = 'Id'
    prototypeIds_kindName = 'Sequence'

    views = None
##     views_kind_isPublicForXmlRpc = 1
##     views_kind_itemKind_value_valueThingCategory = 'widget'
##     views_kind_itemKind_value_valueThingName = 'View'
##     views_kind_itemKind_valueName = 'Thing'
##     views_kindName = 'Sequence'

    def getC3PrototypeLinearization(self):
        # An implementation of the C3 class hierarchy Linearization for the
        # cards prototypes.
        # Cf. http://www.webcom.com/haahr/dylan/linearization-oopsla96.html
        linearization = [self.id]
        if self.prototypeIds:
            prototypeIdsByPrecedenceOrder = self.prototypeIds[:]
            prototypeIdsByPrecedenceOrder.reverse()
            remainingInputs = [
                self.getProxyForId(prototypeId).getC3PrototypeLinearization(
                    prototypeId)
                for prototypeId in self.prototypeIds] \
                + [prototypeIdsByPrecedenceOrder]
            while remainingInputs:
                # The first class which is not in the tail of each remaining
                # input is the next class to append to the linearization.
                for remainingInput in remainingInputs:
                    candidate = remainingInput[0]
                    for remainingInput2 in remainingInputs:
                        if candidate in remainingInput2[1:]:
                            break
                    else:
                        break
                else:
                    raise faults.InconsistentPrototypeHierarchy(self)
                # A good candidate has been found.
                # Add it  to the partial linearization and remove it from the
                # remaining inputs.
                linearization.append(candidate)
                newRemainingInputs = []
                for remainingInput in remainingInputs:
                    if remainingInput[0] == candidate:
                        del remainingInput[0]
                        if not remainingInput:
                            continue
                    newRemainingInputs.append(remainingInput)
                remainingInputs = newRemainingInputs
        return linearization

    def getImportSlotNames(self, parentSlot = None):
        names = properties.PropertiesContainerMixin.getImportSlotNames(
            self, parentSlot = parentSlot)
        names = names[:]
        if 'prototypeIds' in names:
            # The slot 'prototypeIds' is already imported by the method
            # importFromXmlRpc.
            names.remove('prototypeIds')
        return names

    def getLabel(self):
        if self.isSlotName('title'):
            label = self.getSlot('title').getValue()
            if label:
                return label
        from tools import extractLocalId
        return 'Card number %s' % extractLocalId(self.id)

    def getLabelLanguage(self):
        if hasattr(self, 'language') and self.language:
            return self.language
        return ''

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['prototypeIds', 'properties']
        return slotNames

    def importFromXmlRpc(self, dataImport, parentSlot = None):
        # The slot 'prototypeIds' (and 'properties') must be imported first,
        # because it is needed to compute the slotNames list.
        if dataImport.has_key('__noneSlotNames__'):
            noneSlotNames = dataImport['__noneSlotNames__']
        else:
            noneSlotNames = []
        for slotName in ['prototypeIds']:
            if dataImport.has_key(slotName):
                exportedValue = dataImport[slotName]
            elif slotName in noneSlotNames:
                exportedValue = None
            else:
                continue
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if not kind.isPublicForXmlRpc:
                continue
            slot.setValue(kind.importValueFromXmlRpc(slot, exportedValue))
        return properties.PropertiesContainerMixin.importFromXmlRpc(
            self, dataImport, parentSlot = parentSlot)


class CardsCommon:
    adminClassName = 'AdminCards'
    newObjectNameCapitalized = N_('New Card')
    objectClassName = 'Card'
    objectName = N_('card')
    objectNameCapitalized = N_('Card')
    objectsName = N_('cards')
    objectsNameCapitalized = N_('Cards')
    serverRole = 'cards'

