# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002, 2003 Odile Bnassy, Code Lutin, Thierry Dulieu,
#     Easter-eggs, Entr'ouvert, Frdric Pters, Benjamin Poussin,
#     Emmanuel Raviart, Emmanuel Saracco & Thridion
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Contact Infos Common Models"""

__version__ = '$Revision: 1.7 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon
import geography


class AdminContactInfosCommon(AdminCommon):
    pass


class ContactInfoCommon(ObjectCommon):
    address = None
    address_kind_isPublicForXmlRpc = 1
    address_kindName = 'String'
    
    city = None
    city_kind_isPublicForXmlRpc = 1
    city_kind_isRequired = 1
    city_kindName = 'String'
    
    comment = None
    comment_kind_balloonHelp = N_('Enter any extra information here.')
    comment_kind_isPublicForXmlRpc = 1
    comment_kindName = 'String'
    
    country = None
    country_kind_isPublicForXmlRpc = 1
    country_kindName = 'Choice'
    country_kind_isRequired = 1
    country_kind_values = geography.countryCodes
    
    courtesy = None
    courtesy_kind_balloonHelp = N_('Enter contact courtesy title, if any.')
    courtesy_kind_isPublicForXmlRpc = 1
    courtesy_kindName = 'XChoice'
    courtesy_kind_values = [
        'Dr',
        'Drs',
        'Fr',
        'Herr',
        'Hr',
        'Hra',
        'Ing',
        'M',
        'Miss',
        'Mlle',
        'Mme',
        'Mr',
        'Mrs',
        'Ms',
        'Mstr',
        'Prof',
        'Sir',
        'Sr',
        'Sra',
        ]
    
    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    email = None
    email_kind_balloonHelp = N_(
        'Enter contact email address, in the form identifier@server.name.')
    email_kind_isPublicForXmlRpc = 1
    email_kindName = 'Email'

    first = None
    first_kind_balloonHelp = N_('Enter contact first name.')
    first_kind_isPublicForXmlRpc = 1
    first_kind_isRequired = 1
    first_kindName = 'String'

    last = None
    last_kind_balloonHelp = N_('Enter contact last name.')
    last_kind_isPublicForXmlRpc = 1
    last_kind_isRequired = 1
    last_kindName = 'String'

    middle = None
    middle_kind_balloonHelp = N_(
        'Enter contact middle name, or nickname, if any.')
    middle_kind_isPublicForXmlRpc = 1
    middle_kindName = 'String'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    organizationIds = None
    organizationIds_kind_balloonHelp = N_(
        'Select organizations to which the contact belongs.')
    organizationIds_kind_isPublicForXmlRpc = 1
    organizationIds_kind_itemKind_valueName = 'Id'
    organizationIds_kind_itemKind_value_serverRoles = ['organizations']
    organizationIds_kindName = 'Sequence'

    phone = None
    phone_kind_isPublicForXmlRpc = 1
    phone_kind_isRequired = 1
    phone_kindName = 'String'

    readersSet = None
    readersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to read this item.')
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    writersSet = None
    writersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to edit this item.')
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    zip = None
    zip_kind_isPublicForXmlRpc = 1
    zip_kind_isRequired = 1
    zip_kindName = 'String'
    
    def getLabel(self):
        label = getFullName(self.first, self.last, self.middle)
        if label is None:
            return ''
        return label

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        return ''

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'courtesy', 'first', 'middle', 'last', 'email', 'phone',
            'address', 'zip', 'city', 'country',
            'organizationIds', 'comment',  
            'creationTime', 'modificationTime', 'writersSet', 'readersSet']
        return slotNames


def getFullName(firstName, lastName, middle = None):
    fullName = ''
    if firstName:
        fullName += firstName
    if middle:
        if not fullName:
            fullName = middle
        else:
            fullName = fullName + ' ' + middle
    if lastName:
        if not fullName:
            fullName = lastName
        else:
            fullName = fullName + ' ' + lastName
    return fullName

class ContactInfosCommon:
    adminClassName = 'AdminContactInfos'
    newObjectNameCapitalized = N_('New Contact Info')
    objectClassName = 'ContactInfo'
    objectName = N_('contact info')
    objectNameCapitalized = N_('Contact Info')
    objectsName = N_('contact infos')
    objectsNameCapitalized = N_('Contact Infos')
    serverRole = 'contactinfos'

