# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Contacts Common Models"""

__version__ = '$Revision: 1.12 $'[11:-2]

from tools import iso8859_15, utf8
from ObjectsCommon import AdminCommon, ObjectCommon
import geography

class AdminContactsCommon(AdminCommon):
    ldap_host = 'localhost'
    ldap_host_kind_isPublicForXmlRpc = 1
    ldap_host_kind_isRequired = 1
    ldap_host_kind_isTranslatable = 0
    ldap_host_kindName = 'String'
    
    ldap_port = 389
    ldap_port_kind_isPublicForXmlRpc = 1
    ldap_port_kind_isRequired = 1
    ldap_port_kindName = 'Integer'
    
    suffix = 'o=codelutin,c=fr'
    suffix_kind_isPublicForXmlRpc = 1
    suffix_kind_isRequired = 1
    suffix_kind_isTranslatable = 0
    suffix_kindName = 'String'
    
    ldap_idField = 'uid'
    ldap_idField_kind_isPublicForXmlRpc = 1
    ldap_idField_kind_isRequired = 1
    ldap_idField_kind_isTranslatable = 0
    ldap_idField_kindName = 'String'
    
    dn = ldap_idField + '=%s,' + suffix
    dn_kind_isPublicForXmlRpc = 1
    dn_kind_isRequired = 1
    dn_kind_isTranslatable = 0
    dn_kindName = 'String'
    
    admin_dn = dn % 'admin'
    admin_dn_kind_isPublicForXmlRpc = 1
    admin_dn_kind_isRequired = 1
    admin_dn_kind_isTranslatable = 0
    admin_dn_kindName = 'String'
    
    admin_password = ''
    admin_password_kind_isPublicForXmlRpc = 1
    admin_password_kind_isRequired = 1
    admin_password_kindName = 'Password'
    
    objectClassSet = None
    objectClassSet_kind_isPublicForXmlRpc = 1
    objectClassSet_kind_isRequired = 1
    objectClassSet_kindName = 'Sequence'
    objectClassSet_kind_itemKind_valueName = 'String'
    objectClassSet_kind_itemKind_value_isTranslatable = 0

class ContactCommon(ObjectCommon):
    address = None
    address_kind_isPublicForXmlRpc = 1
    address_kindName = 'String'

    city = None
    city_kind_isPublicForXmlRpc = 1
    city_kindName = 'String'

    comment = None
    comment_kind_isPublicForXmlRpc = 1
    comment_kindName = 'String'
    
    company = None
    company_kind_isPublicForXmlRpc = 1
    company_kindName = 'String'

    completName = None
    completName_kind_isPublicForXmlRpc = 1
    completName_kind_isRequired = 1
    completName_kindName = 'String'

    country = None
    country_kind_isPublicForXmlRpc = 1
    country_kindName = 'Choice'
    country_kind_values = geography.countryCodes

    departmentNumber = None
    departmentNumber_kind_isPublicForXmlRpc = 1
    departmentNumber_kindName = 'String'

    email = None
    email_kind_isPublicForXmlRpc = 1
    email_kindName = 'Email'

    fax = None
    fax_kind_isPublicForXmlRpc = 1
    fax_kindName = 'String'

    fonction = None
    fonction_kind_isPublicForXmlRpc = 1
    fonction_kindName = 'String'

##    list = None
##    list_kind_isPublicForXmlRpc = 1
##    list_kindName = 'String'

    name = None
    name_kind_isPublicForXmlRpc = 1
    name_kind_isRequired = 1
    name_kindName = 'String'

##    password = None
##    password_kind_isPublicForXmlRpc = 1
##    password_kindName = 'Password'

    phone = None
    phone_kind_isPublicForXmlRpc = 1
    phone_kindName = 'String'

    photo = None
    photo_kind_isPublicForXmlRpc = 1
    photo_kindName = 'String'

    postOfficeBox = None
    postOfficeBox_kind_isPublicForXmlRpc = 1
    postOfficeBox_kindName = 'String'
    
    mobile = None
    mobile_kind_isPublicForXmlRpc = 1
    mobile_kindName = 'String'

    service = None
    service_kind_isPublicForXmlRpc = 1
    service_kindName = 'String'

    type = None
    type_kind_isPublicForXmlRpc = 1
    type_kind_values = [
        'groupe',
        'client',
        'fournisseur',
        ]
    type_kindName = 'XChoice'

    username = None
    username_kind_isPublicForXmlRpc = 1
    username_kind_isRequired = 1
    username_kindName = 'String'

    webSite = None
    webSite_kind_isPublicForXmlRpc = 1
    webSite_kindName = 'String'

    zip = None
    zip_kind_isPublicForXmlRpc = 1
    zip_kind_isRequired = 0
    zip_kindName = 'String'

    def canCache(self):
        return 0

    def getFullName(self):
        result = self.completName
        if not result:
            result = self.name
        if not result:
            result = self.username
        if not result:
            result = ''
        return result

    def getLabel(self):
        label = self.getFullName()
        if label is None:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'name',
            'completName',
            'username',
#            'password',
            'service',
            'fonction',
            'company',
            'departmentNumber',
            'address',
            'postOfficeBox',
            'zip',
            'city',
            'country',
            'phone',
            'mobile',
            'fax',
            'email',
            'webSite',
            'type',
            'comment',
#            'list',
            'photo',
            ]
        return slotNames

    def getSortString(self):
        if self.name:
            sortString = self.name
        else:
            sortString = ''
        if self.completName:
            sortString += ' ' + self.completName
        return sortString
        
    def updateId(self, serverId):
        """
        Allow to set the Id after creation.
        """
        if not serverId.endswith('/'):
            serverId = serverId + '/'
        self.id = serverId + self.username

    # next methods are used by LdapPersistence
    mapping = [
        ('address', 'street'),
        ('city', 'postalAddress'),
        ('comment', 'description'),
        ('company', 'o'),
        ('completName', 'cn'),
        ('country', 'st'),
        ('departmentNumber', 'departmentNumber'),
        ('email', 'mail'),
        ('fax', 'facsimileTelephoneNumber'),
        ('fonction', 'title'),
#           ('list', ),
        ('mobile', 'mobile'),
        ('name', 'sn'),
#           ('password', ),
        ('phone', 'telephoneNumber'),
        ('photo', 'photo'),
        ('postOfficeBox', 'postOfficeBox'),
        ('service', 'ou'),
        ('type', 'businessCategory'),
        ('username', 'uid'),
        ('webSite', 'labeledURI'),
        ('zip', 'postalCode'),
        ]

    def load(self, data):
       """
       data is dictionnary of value    
       """
       for fieldName, ldapName in self.mapping:
           if data.has_key(ldapName):
               setattr(self, fieldName, iso8859_15(data[ldapName][0]))

    def storeList(self):
        """
        return list of attributes.
        """
        result = [(ldapName, utf8(getattr(self, fieldName))) for fieldName, ldapName in self.mapping if getattr(self, fieldName)]
        return result


class ContactsCommon:
    adminClassName = 'AdminContacts'
    newObjectNameCapitalized = N_('New Contact')
    objectClassName = 'Contact'
    objectName = N_('contact')
    objectNameCapitalized = N_('Contact')
    objectsName = N_('contacts')
    objectsNameCapitalized = N_('Contacts')
    serverRole = 'contacts'

