# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Directories Common Models"""

__version__ = '$Revision: 1.23 $'[11:-2]

import os
import stat
import ConfigParser

import glasnost.common.context as context
import glasnost.proxy.tools as ProxyTools

from ObjectsCommon import AdminCommon, ObjectCommon
import things

file_separator = '/'
glasnostFile = '.glasnost'
trashFile = ".trash"+glasnostFile

def getGlasnostFile(filename):
    """
    give the glasnost file information name
    
    @param filename: the real file 
    @type filename: string
    """
    filestat = os.stat(filename)
    mode = filestat[stat.ST_MODE]
    if stat.S_ISDIR(mode):  # It's a directory
        return filename + file_separator + glasnostFile
    else:
        return filename + glasnostFile


def writeGlasnostFile(filename, object):
    filename = getGlasnostFile(filename)
    config = ConfigParser.ConfigParser()    
    if os.path.exists(filename):
        file = open(filename, 'r+')
        config.readfp(file)
    else:
        file = open(filename, "w")
    if not config.has_section('MAIN'):
        config.add_section('MAIN')
    config.set('MAIN', 'description', object.description)
    if object.creationTime:
        config.set('MAIN', 'creationTime', object.creationTime)
    else:
        config.remove_option('MAIN', 'creationTime')
    if object.modificationTime:
        config.set('MAIN', 'modificationTime', object.modificationTime)
    else:
        config.remove_option('MAIN', 'modificationTime')
    if object.owner:
        config.set('MAIN', 'owner', object.owner)
    else:
        config.remove_option('MAIN', 'owner')
    if object.adminsSet:
        config.set('MAIN', 'adminsSet', ','.join(object.adminsSet))
    else:
        config.remove_option('MAIN', 'adminsSet')
    if object.readersSet:
        config.set('MAIN', 'readersSet', ','.join(object.readersSet))
    else:
        config.remove_option('MAIN', 'readersSet')
    if object.writersSet:
        config.set('MAIN', 'writersSet', ','.join(object.writersSet))
    else:
        config.remove_option('MAIN', 'writersSet')
    config.write(file)



def isGlasnostSpecialFile(filename):
    """
    @param filename: file name without root directories prefix
    @type filename: String
    """
    return filename.endswith(glasnostFile) or \
           filename.startswith(trashFile)


def matchDescription(filename, description):
    """
    @return : true if glansost file doesn't exist or
    if glansost file with description exist and
    match the description argument, all word in description is in the
    glasnost file
    """
    filename = getGlasnostFile(filename)
    if os.path.exists(filename):
        config = ConfigParser.ConfigParser()
        config.readfp(open(filename))
        if config.has_option('MAIN', 'description'):
            descriptionInFile = config.get('MAIN', 'description')
            for word in description.split(' '):
                if not descriptionInFile.count(word):
                    return 0
                
    return 1
        

class AdminDirectoriesCommon(AdminCommon):
    pass


class DirectoryCommon(ObjectCommon):
    adminsSet = None
    adminsSet_kind_isPublicForXmlRpc = 1
    adminsSet_kind_itemKind_valueName = 'Id'
    adminsSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    adminsSet_kindName = 'Sequence'

    name = ''
    name_kind_isPublicForXmlRpc = 1
    name_kind_isRequired = 1
    name_kindName = 'String'

    rootPath = None
    rootPath_kind_isPublicForXmlRpc = 1
    rootPath_kind_isRequired = 1
    rootPath_kindName = 'Path'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kind_requiredCount = 0
    writersSet_kindName = 'WritersSet'

    def canCache(self):
        return 1

    def getLabel(self):
        label = self.name
        if not label:
            label = self.rootPath
        if not label:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'name', 'rootPath', 'adminsSet', 'writersSet', 'readersSet']
        return slotNames


class DirectoryFileCommon(ObjectCommon):
    adminsSet = None
    adminsSet_kind_isPublicForXmlRpc = 1
    adminsSet_kind_itemKind_valueName = 'Id'
    adminsSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    adminsSet_kind_requiredCount = 0
    adminsSet_kindName = 'Sequence'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'
    creationTime_kind_isRequired = 0

    data = None
    data_kind_hasToModify = 1
    data_kind_isPublicForXmlRpc = 1
    data_kindName = 'Data'
    data_kind_isRequired = 0

    description = None
    description_kind_hasToModify = 1
    description_kind_isPublicForXmlRpc = 1
    description_kind_isRequired = 0
    description_kindName = 'String'

    directory = None
    directory_kind_hasToModify = 0
    directory_kind_isPublicForXmlRpc = 1
    directory_kind_isRequired = 1
    directory_kindName = 'Id'

    fileChildren = []
    fileChildren_kind_hasToModify = 0
    fileChildren_kind_isPublicForXmlRpc = 1
    fileChildren_kindName = 'Sequence'    
    fileChildren_kind_itemKind_valueName = 'Path'

    fileType = None
    fileType_kind_hasToModify = 1
    fileType_kind_isPublicForXmlRpc = 1
    fileType_kind_isRequired = 0
    fileType_kindName = 'String'

    mimeType = None
    mimeType_kind_hasToModify = 1
    mimeType_kind_isPublicForXmlRpc = 1
    mimeType_kind_isRequired = 0
    mimeType_kindName = 'String'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'
    modificationTime_kind_isRequired = 0

    owner = None
    owner_kind_isPublicForXmlRpc = 1
    owner_kindName = 'Id'
    owner_kind_serverRoles = ['people']

    path = None
    path_kind_hasToModify = 0
    path_kind_isPublicForXmlRpc = 1
    path_kind_isRequired = 1
    path_kindName = 'Path'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    repertoryChildren = []
    repertoryChildren_kind_isPublicForXmlRpc = 1
    repertoryChildren_kindName = 'Sequence'    
    repertoryChildren_kind_itemKind_valueName = 'Path'

    size = 0
    size_kind_hasToModify = 0
    size_kind_isPublicForXmlRpc = 1
    size_kindName = 'Integer'    

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kind_requiredCount = 0
    writersSet_kindName = 'WritersSet'

    def __init__(self):
        pass

    def init(self, directoryId, path, withData = 0):
        self.setPath(path)
        self.directory = directoryId

        directory = ProxyTools.getObject(directoryId)

        print repr(('iiiii', dir(self), hasattr(self, 'id'),  directory, dir(directory), hasattr(directory, 'id'), 'iiiii'))
        
        self.id = directory.id
        self.language = directory.language

        file = directory.rootPath + file_separator + path

        filestat = os.stat(file)
        mode = filestat[stat.ST_MODE]
        if stat.S_ISDIR(mode):  # It's a directory
            self.fileType = 'repertory'
            self.mimeType = 'repertory'
            self.readGlasnostFile(file)
            self.repertoryChildren, self.fileChildren = self.getRepertoriesAndFiles(directory.getProxy(), directoryId, directory.rootPath, file)
            self.size = len(self.repertoryChildren) +len(self.fileChildren)
        elif stat.S_ISREG(mode): # It's a file
            self.fileType = 'file'
            self.mimeType = os.popen('file -b --mime "%s" 2>/dev/null' % file).readline()[:-1]
            self.readGlasnostFile(file)
            self.size = filestat[stat.ST_SIZE]
            if withData:
                self.data = open(file).read()
        else:
            # Unknown file type
            raise Exception('Unsupported file %s' % path)

        return self

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'path', 'description', 'fileType', 'creationTime',
            'modificationTime', 'data', 
            'owner', 'adminsSet', 'writersSet', 'readersSet']
        return slotNames

    def getRepertoriesAndFiles(self, proxy, directoryId,
                               rootPath, path):
        resultRepertory = []
        resultFile = []

        files = os.listdir(path)
        for file in files:    
            if file[-len(glasnostFile):] != glasnostFile:
                realPath = path + file_separator + file
                mode = os.stat(realPath)[stat.ST_MODE]
                if stat.S_ISDIR(mode):  # It's a directory
                    if proxy.canGetObject(directoryId,
                                          realPath[len(rootPath):]):
                        resultRepertory.append(file)
                elif stat.S_ISREG(mode): # It's a file
                    if proxy.canGetObject(directoryId,
                                          realPath[len(rootPath):]):
                        resultFile.append(file)

        return (resultRepertory, resultFile)

    def readGlasnostFile(self, filename):
        filename = getGlasnostFile(filename)
        if os.path.exists(filename):
            config = ConfigParser.ConfigParser()
            config.readfp(open(filename))
            if config.has_option('MAIN', 'description'):
                self.description = config.get('MAIN', 'description')
            if config.has_option('MAIN', 'creationTime'):
                self.creationTime = config.getfloat('MAIN', 'creationTime')
            if config.has_option('MAIN', 'modificationTime'):
                self.modificationTime = config.getfloat('MAIN', 'modificationTime')
            if config.has_option('MAIN', 'owner'):
                self.owner = config.get('MAIN', 'owner')
            if config.has_option('MAIN', 'adminsSet'):
                self.adminsSet = config.get('MAIN', 'adminsSet').split(',')
            if config.has_option('MAIN', 'readersSet'):
                self.readersSet = config.get('MAIN', 'readersSet').split(',')
            if config.has_option('MAIN', 'writersSet'):
                self.writersSet = config.get('MAIN', 'writersSet').split(',')
        
    def canCache(self):
        return 1

    def setPath(self, path):
        self.path = path
        if self.path[:2] == '//':
            self.path = self.path[1:]
        elif self.path == '':
            self.path = '/'
    
    def getLabel(self):
        label = self.path
        if label is None:
            return ''
        return label

    def getName(self):
        name = self.path.split(file_separator)
        return name[-1]

    def isRepertory(self):
        return self.fileType == "repertory"
    
    def isFile(self):
        return self.fileType == "file"

    def getParentPath(self):
        result = file_separator.join(self.path.split(file_separator)[:-1])
        if not result:
            result = file_separator
        return result

class DirectoriesCommon:
    adminClassName = 'AdminDirectories'
    newObjectNameCapitalized = N_('New Directory')
    objectClassName = 'Directory'
    objectName = N_('directory')
    objectNameCapitalized = N_('Directory')
    objectsName = N_('directories')
    objectsNameCapitalized = N_('Directories')
    serverRole = 'directories'

