# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Elections Common Models"""

__version__ = '$Revision: 1.16 $'[11:-2]


import time

from ObjectsCommon import AdminCommon, ObjectCommon


class AdminElectionsCommon(AdminCommon):
    pass


class AbstractElectionCommon(ObjectCommon):
    authorsSet = None
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    ballotKind = 'voterChoice'
    ballotKind_kind_balloonHelp = N_('Select the type of the ballots.')
    ballotKind_kind_isPublicForXmlRpc = 1
    ballotKind_kind_isRequired = 1
    ballotKind_kind_values = [
        'public',
        'voterChoice',
        'secret',
        ]
    ballotKind_kindName = 'Choice'

    candidateIds = None
    candidateIds_kind_isPublicForXmlRpc = 1
    candidateIds_kind_itemKind_valueName = 'Id'
    candidateIds_kindName = 'Sequence'

    candidatesSet = None
    candidatesSet_kind_balloonHelp = N_('Choose the candidates for this election.')
    candidatesSet_kind_isPublicForXmlRpc = 1
    candidatesSet_kind_itemKind_valueName = 'Id'
    candidatesSet_kind_requiredCount = 1
    candidatesSet_kindName = 'Sequence'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    endTime = None
    endTime_kind_balloonHelp = N_('Enter the date the votes should be counted (leave blank if you want them counted from the start).')
    endTime_kind_isPublicForXmlRpc = 1
    endTime_kindName = 'Time'

    isAlwaysRunning = 0
    isAlwaysRunning_kind_balloonHelp = N_('Select whether the election should never stop.')
    isAlwaysRunning_kind_isPublicForXmlRpc = 1
    isAlwaysRunning_kind_isRequired = 1
    isAlwaysRunning_kindName = 'Boolean'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    state = 'draft'
    state_kind_isPublicForXmlRpc = 1
    state_kind_isRequired = 1
    state_kind_values = [
        'draft',
        'proposed',
        'running',
        'closed',
        ]
    state_kindName = 'Choice'

    subject = None
    subject_kind_balloonHelp = N_('Enter the subject of this election.')
    subject_kind_isPublicForXmlRpc = 1
    subject_kind_isRequired = 1
    subject_kindName = 'String'

    title = None
    title_kind_balloonHelp = N_('Enter the title of this election.')
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    voteKind = 'VoteRanking'
    voteKind_kind_isPublicForXmlRpc = 1
    voteKind_kind_isRequired = 1
    voteKind_kind_values = [
        'VoteApproval',
        'VoteDistribution',
        'VoteRanking',
        'VoteRating',
        ]
    voteKind_kindName = 'Choice'

    votersSet = None
    votersSet_kind_balloonHelp = N_('Choose the persons or groups who will vote for this election.')
    votersSet_kind_isPublicForXmlRpc = 1
    votersSet_kind_itemKind_valueName = 'Id'
    votersSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    votersSet_kind_requiredCount = 1
    votersSet_kindName = 'Sequence'

    voteTokens = None
    voteTokens_kind_hasToModify = 0
    voteTokens_kind_isPublicForXmlRpc = 1
    voteTokens_kind_isRequired = 1
    voteTokens_kind_itemKind_valueName = 'Token'
    voteTokens_kindName = 'Sequence'

    weightingsGradeId = None
    weightingsGradeId_kind_isPublicForXmlRpc = 1
    weightingsGradeId_kind_serverRoles = ['grades']
    weightingsGradeId_kindName = 'Id'

    winnerIds = None
    winnerIds_kind_hasToModify = 0
    winnerIds_kind_isPublicForXmlRpc = 1
    winnerIds_kind_itemKind_valueName = 'Id'
    winnerIds_kindName = 'Sequence'

    winnersCount = 1
    winnersCount_kind_balloonHelp = N_('Enter the number of winners this election should have.')
    winnersCount_kind_isPublicForXmlRpc = 1
    winnersCount_kind_isRequired = 1
    winnersCount_kindName = 'Integer'

    winnersGroupId = None
    winnersGroupId_kind_isPublicForXmlRpc = 1
    winnersGroupId_kind_serverRoles = ['groups']
    winnersGroupId_kindName = 'Id'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def canCache(self):
        return 0

    def canCompute(self):
        return self.state == 'closed' \
               or self.state == 'running' and self.isAlwaysRunning \
                   and (self.endTime is None or self.endTime <= time.time())

    def getLabel(self):
        label = self.getTitle()
        if label is None:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'subject', 'creationTime', 'modificationTime', 'endTime',
            'method', 'voteKind', 'isAlwaysRunning', 'ballotKind', 'state',
            'authorsSet', 'writersSet', 'readersSet', 'votersSet',
            'weightingsGradeId', 'candidatesSet', 'winnersCount',
            'winnersGroupId', 'winnerIds']
        return slotNames

    def getTitle(self):
        return self.title

    def getVoterIds(self, virtualServerId = None):
        from glasnost.proxy.GroupsProxy import getSetContainedIds
        return getSetContainedIds(
            self.votersSet, ['people'], serverId = virtualServerId)

    def newVote(self):
        from glasnost.proxy.VotesProxy import votesProxy
        vote = getattr(votesProxy, self.voteKind)()
        return vote


class ElectionAverageCommon(AbstractElectionCommon):
    ratings = None
    ratings_kind_hasToModify = 0
    ratings_kind_isPublicForXmlRpc = 1
    ratings_kind_keyKind_valueName = 'Id'
    ratings_kind_valueKind_valueName = 'Float'
    ratings_kindName = 'Mapping'

    voteKind = 'VoteRating'

    winnersGradeId = None
    winnersGradeId_kind_isPublicForXmlRpc = 1
    winnersGradeId_kind_serverRoles = ['grades']
    winnersGradeId_kindName = 'Id'


class ElectionCondorcetCommon(AbstractElectionCommon):
    pairwiseMatrix = None
    pairwiseMatrix_kind_hasToModify = 0
    pairwiseMatrix_kind_isPublicForXmlRpc = 1
    pairwiseMatrix_kindName = 'PairwiseMatrix'

    ratings = None
    ratings_kind_hasToModify = 0
    ratings_kind_isPublicForXmlRpc = 1
    ratings_kind_keyKind_valueName = 'Id'
    ratings_kind_valueKind_valueName = 'Rating'
    ratings_kindName = 'Mapping'

    voteKind = 'VoteRanking'


class ElectionsCommon:
    adminClassName = 'AdminElections'
    newObjectNameCapitalized = N_('New Election')
    objectClassName = 'AbstractElection'
    objectName = N_('election')
    objectNameCapitalized = N_('Election')
    objectsName = N_('elections')
    objectsNameCapitalized = N_('Elections')
    serverRole = 'elections'

