# Glasnost 
# By: Odile Bnassy <obenassy@entrouvert.org>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@easter-eggs.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Events Common Models"""

__version__ = '$Revision: 1.19 $'[11:-2]


from tools import extractLocalId
import accounting
from ObjectsCommon import AdminCommon, ObjectCommon


class AdminEventsCommon(AdminCommon):
    pass


class EventCommon(ObjectCommon):
    certainty = None
    certainty_kind_isPublicForXmlRpc = 1
    certainty_kind_isRequired = 1
    certainty_kind_values = accounting.certaintyValues
    certainty_kindName = 'IntegerChoice'
   
    comment = None
    comment_kind_isPublicForXmlRpc = 1
    comment_kindName = 'String'

    currency = None
    currency_kind_hasToModify = 0
    currency_kind_useCustomStorage = 1
    currency_kind_isPublicForXmlRpc = 1
    currency_kind_isRequired = 1
    currency_kind_values = accounting.currencyValues    
    currency_kindName = 'Choice'

    date = None
    date_kind_isPublicForXmlRpc = 1
    date_kind_isRequired = 1
    date_kindName = 'Date'
    
    isBalanced = None
    isBalanced_kind_hasToModify = 0
    isBalanced_kind_isPublicForXmlRpc = 1
    isBalanced_kindName = 'Boolean'

    label = None
    label_kind_isPublicForXmlRpc = 1
    label_kind_isRequired = 1
    label_kindName = 'String'

    language_kindName = None

    membersSet = None
    membersSet_kind_isPublicForXmlRpc = 1    
    membersSet_kind_itemKind_valueName = 'Id'
    membersSet_kind_itemKind_value_serverRoles = ['records']
    membersSet_kindName = 'Sequence'
    
    operationId = None
    operationId_kind_isPublicForXmlRpc = 1
    operationId_kind_isRequired = 1
    operationId_kind_serverRoles = ['operations']
    operationId_kindName = 'Id'

    partnerId = None
    partnerId_kind_useCustomStorage = 1
    partnerId_kind_isPublicForXmlRpc = 1
    partnerId_kind_serverRoles = ['groups', 'people']
    partnerId_kindName = 'Id'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    voucherId = None
    voucherId_kind_isPublicForXmlRpc = 1
    voucherId_kind_serverRoles = ['uploadfiles']
    voucherId_kindName = 'Id'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def canCache(self):
        return 0

    def getLabel(self):
        return self.getResume()

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        language = self.language
        if language == None:
            return ''
        return language

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'operationId', 'partnerId', 'date', 'label', 'currency', 
            'voucherId', 'comment', 'certainty', 'membersSet',
            'isBalanced', 'writersSet', 'readersSet']
        return slotNames

    def getResume(self):
        localId = extractLocalId(self.id)
        dateDisplay = accounting.dateDisplay(self.date)
        labelDisplay = accounting.labelDisplay(self.label)
        resume = "(%s) %s - %s" % (localId, dateDisplay, labelDisplay)
        if resume is None:
            resume = ''
        return resume


class EventsCommon:
    adminClassName = 'AdminEvents'
    newObjectNameCapitalized = N_('New Event')
    objectClassName = 'Event'
    objectName = N_('event')
    objectNameCapitalized = N_('Event')
    objectsName = N_('events')
    objectsNameCapitalized = N_('Events')
    serverRole = 'events'

