# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost FIInitiatives Common Models"""

__version__ = '$Revision: 1.6 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminFIInitiativesCommon(AdminCommon):
    keywordValues = None
    keywordValues_kind_balloonHelp = N_(
        'Enter list of keywords for the object.')
    keywordValues_kind_isPublicForXmlRpc = 1
    keywordValues_kind_itemKind_valueName = 'String'
    keywordValues_kindName = 'Sequence'    

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = AdminCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames = slotNames[:]
        if 'views' in slotNames:
            i = slotNames.index('views')
        else:
            i = len(slotNames)
        slotNames[i:i] = ['keywordsValues']
        return slotNames
    

class FIInitiativeCommon(ObjectCommon):
    authorsSet = None
    authorsSet_kind_balloonHelp = N_('Select the author(s) of this item.')
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    description = None
    description_kind_isPublicForXmlRpc = 1
    description_kind_isRequired = 1
    description_kindName = 'String'

    endDate = None
    endDate_kind_isPublicForXmlRpc = 1
    endDate_kind_isRequired = 1
    endDate_kindName = 'Time'

    evaluation = None
    evaluation_kind_isPublicForXmlRpc = 1
    evaluation_kind_isRequired = 1
    evaluation_kindName = 'String'

    goals = None
    goals_kind_isPublicForXmlRpc = 1
    goals_kind_isRequired = 1
    goals_kindName = 'String'

    keywords = None
    keywords_kind_isPublicForXmlRpc = 1
    keywords_kind_itemKind_value_values = []
    keywords_kind_itemKind_value_valuesGetterName = 'getKeywordValues'
    keywords_kind_itemKind_valueName = 'XChoice'
    keywords_kind_requiredCount = 1
    keywords_kindName = 'Sequence'

    language_kindName = None

    lastEditorId = None
    lastEditorId_kind_hasToModify = 0
    lastEditorId_kind_isPublicForXmlRpc = 1
    lastEditorId_kind_serverRoles = ['people']
    lastEditorId_kindName = 'Id'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    owner = None
    owner_kind_isPublicForXmlRpc = 1
    owner_kind_isRequired = 1
    owner_kindName = 'String'

    partners = None
    partners_kind_isPublicForXmlRpc = 1
    partners_kind_isRequired = 1
    partners_kindName = 'String'

    readersSet = None
    readersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to read this item.')
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    startDate = None
    startDate_kind_isPublicForXmlRpc = 1
    startDate_kind_isRequired = 1
    startDate_kindName = 'Time'

    themes = None
    themes_kind_isPublicForXmlRpc = 1
    themes_kind_isRequired = 1
    themes_kindName = 'String'

    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    writersSet = None
    writersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to edit this item.')
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def getKeywordValues(self, keywordSlot, fields):
        raise NotImplementedError
        
    def getLabel(self):
        label = self.title
        if label is None:
            return ''
        return label

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        language = self.language
        if language == None :
            return ''
        return language 

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'themes', 'keywords', 'owner', 'goals', 'description',
            'partners', 'startDate', 'endDate', 'evaluation',
            'creationTime', 'lastEditorId', 'modificationTime', 'authorsSet',
            'writersSet', 'readersSet'
            ]
        return slotNames

class FIInitiativesCommon:
    adminClassName = 'AdminFIInitiatives'
    newObjectNameCapitalized = 'Nouvelle Initiative Fete de l\'Internet'
    objectClassName = 'FIInitiative'
    objectName = 'initiative FI'
    objectNameCapitalized = 'Initiative Fete de l\'Internet'
    objectsName = 'initiatives Fete de l\'Internet'
    objectsNameCapitalized = 'Initiatives Fete de l\'Internet'
    serverRole = 'fiinitiatives'

