# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Function Calls Common Models"""

__version__ = '$Revision: 1.18 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon
import tools_new as commonTools


class AdminFunctionCallsCommon(AdminCommon):
    pass


class FunctionCallCommon(ObjectCommon):
    arguments = None
    arguments_kind_isPublicForXmlRpc = 1
    arguments_kindName = 'Structure'
    # Use arguments_kindGetter

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    functionDeclaration = None

    functionName = None
    functionName_kind_isPublicForXmlRpc = 1
    functionName_kind_isRequired = 1
    functionName_kind_serverIdSlotName = 'serverId'
    functionName_kindName = 'FunctionName'
    
    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    result = None
    result_kindName ='__getter__'

    serverId = None
    serverId_kind_isPublicForXmlRpc = 1
    serverId_kind_isRequired = 1
    serverId_kindName = 'ServerId'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def arguments_kindGetter(self):
        functionDeclaration = self.getFunctionDeclaration()
        if functionDeclaration is not None:
            parameters  = functionDeclaration.parameters[:]
            for name in ('virtualServerId', 'clientToken', 'userToken'):
                if len(parameters) > 0 and parameters[0].name == name:
                    del parameters[0]
            self.arguments_kind.parameters = parameters
        return self.arguments_kind

    def getFunctionDeclaration(self):
        if self.functionDeclaration is not None:
            return self.functionDeclaration
        if not self.serverId:
            return None
        if not self.functionName:
            return None
        import context
        try:
            serverRole = commonTools.extractRole(self.serverId)
        except:
            return None
        from glasnost.proxy.tools import getProxyForServerRole
        try:
            proxy = getProxyForServerRole(serverRole)
        except:
            return None
        if proxy is None:
            return None
        try:
            functionDeclarations = proxy.getFunctionDeclarations() 
        except:
            return None
        if not functionDeclarations.has_key(self.functionName):
            return None
        self.functionDeclaration = functionDeclarations[self.functionName]
        return self.functionDeclaration

    def getImportSlotNames(self, parentSlot = None):
        names = ObjectCommon.getImportSlotNames(self, parentSlot = parentSlot)
        names = names[:]
        for slotName in ['arguments', 'result']:
            if slotName in names:
                names.remove(slotName)
        return names

    def getLabel(self):
        label = ''
        if self.serverId:
            if label:
                label += ' '
            label += self.serverId
        if self.functionName:
            if label:
                label += ' '
            label += self.functionName
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'serverId', 'functionName', 'arguments', 'result', 'creationTime',
            'modificationTime', 'writersSet', 'readersSet']
        return slotNames

    def importFromXmlRpc(self, dataImport, parentSlot = None):
        if dataImport.has_key('__noneSlotNames__'):
            noneSlotNames = dataImport['__noneSlotNames__']
        else:
            noneSlotNames = []
        if dataImport.has_key('arguments'):
            argumentsImport = dataImport['arguments']
        elif 'arguments' in noneSlotNames:
            argumentsImport = None
        if dataImport.has_key('result'):
            resultImport = dataImport['result']
        elif 'result' in noneSlotNames:
            resultImport = None

        ObjectCommon.importFromXmlRpc(
            self, dataImport, parentSlot = parentSlot)

        if dataImport.has_key('arguments') or 'arguments' in noneSlotNames:
            argumentsSlot = self.getSlot('arguments')
            argumentsKind = argumentsSlot.getKind()
            if argumentsKind.isPublicForXmlRpc:
                argumentsSlot.setValue(argumentsKind.importValueFromXmlRpc(
                    argumentsSlot, argumentsImport))
        if dataImport.has_key('result') or 'result' in noneSlotNames:
            resultSlot = self.getSlot('result')
            resultKind = resultSlot.getKind()
            if resultKind.isPublicForXmlRpc:
                resultSlot.setValue(resultKind.importValueFromXmlRpc(
                    resultSlot, resultImport))

    def result_kindGetter(self):
        functionDeclaration = self.getFunctionDeclaration()
        if functionDeclaration is not None:
            self.result_kind = functionDeclaration.result.kind
            self.result_kind.isConstant = 1
            self.result_kind.isPublicForXmlRpc = 1
            self.result_kind.widget_fieldLabel = N_('Result')
        else:
            self.result_kind = self.newThing('kind', 'String') # Dummy
            self.result_kind.isConstant = 1
            self.result_kind.isPublicForXmlRpc = 0
        return self.result_kind

class FunctionCallsCommon:
    adminClassName = 'AdminFunctionCalls'
    newObjectNameCapitalized = N_('New Function Call')
    objectClassName = 'FunctionCall'
    objectName = N_('function call')
    objectNameCapitalized = N_('Function Call')
    objectsName = N_('function calls')
    objectsNameCapitalized = N_('Function Calls')
    serverRole = 'functioncalls'

