# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Grades Common Models"""

__version__ = '$Revision: 1.10 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminGradesCommon(AdminCommon):
    pass


class GradeCommon(ObjectCommon):
    marks = None
    marks_kind_isPublicForXmlRpc = 1
    marks_kind_isRequired = 1
    marks_kindName = 'Marks'

    membersSet = None
    membersSet_kind_isPublicForXmlRpc = 1
    membersSet_kind_itemKind_valueName = 'Id'
    membersSet_kind_requiredCount = 1
    membersSet_kindName = 'Sequence'

    name = None
    name_kind_isPublicForXmlRpc = 1
    name_kind_isRequired = 1
    name_kindName = 'String'

    def canCache(self):
        return 1

    def getLabel(self):
        label = self.name
        if label is None:
            return ''
        return label

    def getMarks(self, objectIds):
        return self.repairMarks(self.marks, objectIds)

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['name', 'membersSet', 'marks']
        return slotNames

    def repairMarks(self, marks, objectIds):
        if not marks:
            return {}
        distribution = {}
        marksSum = 0
        for objectId, mark in marks.items():
            if not objectId in objectIds:
                continue
            marksSum += mark
        if marksSum == 0:
            score = 1 / float(len(objectIds))
            for objectId in objectIds:
                distribution[objectId] = score
        else:
            coef = 1 / float(marksSum)
            for objectId in objectIds:
                if marks.has_key(objectId):
                    value = marks[objectId] * coef
                else:
                    value = 0
                distribution[objectId] = value
        return distribution

class GradesCommon:
    adminClassName = 'AdminGrades'
    newObjectNameCapitalized = N_('New Grade')
    objectClassName = 'Grade'
    objectName = N_('grading')
    objectNameCapitalized = N_('Grading')
    objectsName = N_('gradings')
    objectsNameCapitalized = N_('Gradings')
    serverRole = 'grades'

