# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Initiatives Common Models"""

__version__ = '$Revision: 1.20 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon
import geography


class AdminInitiativesCommon(AdminCommon):
    pass


class InitiativeCommon(ObjectCommon):
    # Resume en 2-3 lignes de l'initiative.
    abstract = None
    abstract_kind_balloonHelp = N_('Enter short description here.')
    abstract_kind_isPublicForXmlRpc = 1
    abstract_kind_isRequired = 1
    abstract_kindName = 'String'

    authorsSet = None
    authorsSet_kind_balloonHelp = N_('Select the author(s) of this item.')
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    # En quoi cette initiative participe t elle a un objectif d'interet general
    # ou a une dynamique de transformation sociale, culturelle, politique ?
    benefits = None
    benefits_kind_balloonHelp = N_(
        'How does this initiative contribute to public interest goals or '
        'social/cultural/political transformation?')
    benefits_kind_isPublicForXmlRpc = 1
    benefits_kindName = 'String'

    # Contacts
    contactIds = None
    contactIds_kind_balloonHelp = N_(
        'Select contact people for this initiative.')
    contactIds_kind_isPublicForXmlRpc = 1
    contactIds_kind_itemKind_valueName = 'Id'
    contactIds_kind_itemKind_value_serverRoles = ['contactinfos']
    contactIds_kind_requiredCount = 1
    contactIds_kindName = 'Sequence'

    # Partenaires du projet
    creationPartnerIds = None
    creationPartnerIds_kind_balloonHelp = N_(
        'With whom (people, organizations...) did you build this '
        'initiative ?.')
    creationPartnerIds_kind_isPublicForXmlRpc = 1
    creationPartnerIds_kind_itemKind_valueName = 'Id'
    creationPartnerIds_kind_itemKind_value_serverRoles = [
        'contactinfos', 'organizations']
    creationPartnerIds_kindName = 'Sequence'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    # Description de l'initiative.
    description = None
    description_kind_balloonHelp = N_('Enter free description here.')
    description_kind_isPublicForXmlRpc = 1
    description_kind_isRequired = 1
    description_kindName = 'String'

    # Quelles sont les difficultes rencontrees ?
    difficulties = None
    difficulties_kind_balloonHelp = N_('What difficulties did you meet with?')
    difficulties_kind_isPublicForXmlRpc = 1
    difficulties_kindName = 'String'

    # Avez vous mis en place une evaluation, si oui laquelle ?
    evaluation = None
    evaluation_kind_balloonHelp = N_(
        'Did you set up any assessment process? How?')
    evaluation_kind_isPublicForXmlRpc = 1
    evaluation_kindName = 'String'

    # Financement de l'initiative (comment avez vous assure le financement du
    # projet ?)
    financing = None
    financing_kind_balloonHelp = N_('How did you finance this initiative?.')
    financing_kind_isPublicForXmlRpc = 1
    financing_kindName = 'String'
    
    # Objectifs de l'initiative (quels objectifs poursuivez-vous ?)
    goals = None
    goals_kind_balloonHelp = N_(
        'What are your main goals? (examples: reinforcing some already acting '
        'project; make contact with new groups of people; address '
        'dissatisfied social needs...')
    goals_kind_isPublicForXmlRpc = 1
    goals_kindName = 'String'

    # Date de creation de l'initiative (quand avez vous initie le projet ?)
    initiativeCreationDate = None
    initiativeCreationDate_kind_balloonHelp = N_(
        'When did you initiate this project?')
    initiativeCreationDate_kind_isPublicForXmlRpc = 1
    initiativeCreationDate_kind_requiredParts = ['month', 'year']
    initiativeCreationDate_kindName = 'Date'

    language_kindName = None

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    # Reseaux avec lesquels vous cooperez autour de cette initiative (echanges
    # d'informations, rencontres, ...)
    networkIds = None
    networkIds_kind_balloonHelp = N_(
        'In the course of this project, with which groups do you cooperate?')
    networkIds_kind_isPublicForXmlRpc = 1
    networkIds_kind_itemKind_value_serverRoles = [
        'contactinfos', 'organizations']
    networkIds_kind_itemKind_valueName = 'Id'
    networkIds_kindName = 'Sequence'

    # Porteur du projet : nom de l'association, la personne ou l'organisation
    # (quelle association ou organisation porte cette initiative ?)
    ownerId = None
    ownerId_kind_balloonHelp = N_(
        'Who carries this initiative (organization/person/group of people)?')
    ownerId_kind_isPublicForXmlRpc = 1
    ownerId_kindName = 'Id'
    ownerId_kind_serverRoles = ['organizations', 'people']

    # Projets envisages (en 2-3 lignes pour permettre des mises en relation
    # mais aussi ne pas decrire un futur possible au detriment du reel
    # realise).
    projectsUnderConsideration = None
    projectsUnderConsideration_kind_isPublicForXmlRpc = 1
    projectsUnderConsideration_kindName = 'String'

    readersSet = None
    readersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to read this item.')
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    # Quels sont les usagers reels ?
    realUsers = None
    realUsers_kind_balloonHelp = N_('Who are the initiative real users?')
    realUsers_kind_isPublicForXmlRpc = 1
    realUsers_kindName = 'String'

    # Source : pour les fiches prises sur d'autres sites.
    source = None
    source_kind_balloonHelp = N_(
        'If this information comes from another web site, please provide here'
        ' source and Internet address.')
    source_kind_isPublicForXmlRpc = 1
    source_kindName = 'String'

    # Quels sont selon vous les elements positifs de cette initiative ?
    strongPoints = None
    strongPoints_kind_balloonHelp = N_(
        'According to you, what are the strong points for this initiative? ')
    strongPoints_kind_isPublicForXmlRpc = 1
    strongPoints_kindName = 'String'

    # Territoire concerne par l'initiative (quel est ou sont les territoires
    # couverts par cette initiative?) (liste predefinie avec possibilite de
    # cocher un champs autre et de le renseigner : liste des regions de
    # France/autre pays (liste)/international)
    territory = None
    territory_kind_balloonHelp = N_(
        'Which territories does this initiative cover?')
    territory_kind_isPublicForXmlRpc = 1
    territory_kind_itemKind_valueName = 'XChoice'
    territory_kind_itemKind_value_groupedValues = {
        'French Regions': geography.frenchRegions,
        'European Countries': geography.europeanCountries,
        'International':['fra'],
        }
    territory_kind_itemKind_value_groupNames = [
        N_('French Regions'),
        N_('European Countries'),
        N_('International'),
        ]
    territory_kind_requiredCount = 1
    territory_kind_itemKind_value_values = geography.frenchRegions + \
                                           geography.europeanCountries
    territory_kindName = 'Sequence'

    # Comment les utilisateurs s'approprient-ils les TIC ?
    ticAppropriation = None
    ticAppropriation_kind_balloonHelp = N_(
        'How did the users react to the project (easy - difficult - '
        'apprehensive - turning away')
    ticAppropriation_kind_isPublicForXmlRpc = 1
    ticAppropriation_kindName = 'String'

    # Role des Tic dans l'initiative (titre de la rubrique).
    # (ex : objet central de l'initiative, levier, accelerateur, obstacle,
    # changement d'organisation, neutre...) ?
    ticRole = None
    ticRole_kind_balloonHelp = N_(
        'Describe what role new technologies played in this initiative ?')
    ticRole_kind_isPublicForXmlRpc = 1
    ticRole_kind_isRequired = 1
    ticRole_kind_values = [
        'central',
        'lever',
        'accelerator',
        'hinder',
        'change',
        'neutral',
        ]
    ticRole_kindName = 'XChoice'

    # A quoi servent les TIC ?
    ticUsages = None
    ticUsages_kind_balloonHelp = N_(
        'To what extent are new technologies useful to this project?')
    ticUsages_kind_isPublicForXmlRpc = 1
    ticUsages_kindName = 'String'

    # Quelles TIC utilisez vous ?
    ticUsed = None
    ticUsed_kind_balloonHelp = N_(
        'Which tools do you use?')
    ticUsed_kind_isPublicForXmlRpc = 1
    ticUsed_kindName = 'String'

    # Qui utilise les TIC ?
    ticUsers = None
    ticUsers_kind_balloonHelp = N_(
        'Who uses Internet tools?')
    ticUsers_kind_isPublicForXmlRpc = 1
    ticUsers_kindName = 'String'

    title = None
    title_kind_balloonHelp = N_('Enter initiative title.')
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    # Declencheur de l'initiative (qu'est ce qui vous a amene a prendre cette
    # initiative ?)
    trigger = None
    trigger_kind_balloonHelp = N_('Enter the initiative triggering elements.')
    trigger_kind_isPublicForXmlRpc = 1
    trigger_kindName = 'String'

    # Site web de l'initiative.
    webSite = None
    webSite_kind_balloonHelp = N_('Enter initiative web site.')
    webSite_kind_isPublicForXmlRpc = 1
    webSite_kind_isRequired = 1
    webSite_kindName = 'String'

    writersSet = None
    writersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to edit this item.')
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def getLabel(self):
        label = self.title
        if label is None:
            return ''
        return label

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        language = self.language
        if language == None :
            return ''
        return language 

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'creationTime', 'modificationTime', 'abstract',
            'webSite', 'contactIds', 'authorsSet', 'source', 
            'description', 'ticUsed', 'ticUsers', 'ticUsages',
            'ticAppropriation',
            'initiativeCreationDate', 'trigger', 'goals', 'benefits',
            'projectsUnderConsideration',
            'ownerId', 'creationPartnerIds', 'networkIds', 'financing',
            'strongPoints', 'difficulties', 'ticRole', 'realUsers',
            'evaluation', 'territory', #'networkIds',
            'writersSet']#, 'readersSet']
        return slotNames

class InitiativesCommon:
    adminClassName = 'AdminInitiatives'
    newObjectNameCapitalized = N_('New Initiative')
    objectClassName = 'Initiative'
    objectName = N_('initiative')
    objectNameCapitalized = N_('Initiative')
    objectsName = N_('initiatives')
    objectsNameCapitalized = N_('Initiatives')
    serverRole = 'initiatives'

