# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Ledgers Common Models"""

__version__ = '$Revision: 1.17 $'[11:-2]


import accounting
from ObjectsCommon import AdminCommon, ObjectCommon


class AdminLedgersCommon(AdminCommon):
    pass


class LedgerCommon(ObjectCommon):
    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    currency = None
    currency_kind_isPublicForXmlRpc = 1
    currency_kind_isRequired = 1
    currency_kind_values = accounting.currencyValues    
    currency_kindName = 'Choice'

    lastEditorId = None
    lastEditorId_kind_isPublicForXmlRpc = 1
    lastEditorId_kind_serverRoles = ['people']
    lastEditorId_kindName = 'Id'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    startDate = None
    startDate_kind_isPublicForXmlRpc = 1
    startDate_kindName = 'Date'
    
    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'
        
    def getLabel(self):
        label = self.title
        if label == None :
            return ''
        return label 

    def getLanguage(self):
        language = self.language
        if language == None :
            return ''
        return language 

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'currency', 'startDate', 'membersSet', 'creationTime',
            'lastEditorId', 'modificationTime', 'writersSet', 'readersSet']
        return slotNames


class LedgersCommon:
    adminClassName = 'AdminLedgers'
    newObjectNameCapitalized = N_('New Ledger')
    objectClassName = 'Ledger'
    objectName = N_('ledger')
    objectNameCapitalized = N_('Ledger')
    objectsName = N_('ledgers')
    objectsNameCapitalized = N_('Ledgers')
    serverRole = 'ledgers'

