# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Objects Common Models"""

__version__ = '$Revision: 1.20 $'[11:-2]


import types

import kinds # Do not remove!
import things
from tools import *


register = things.register


class ObjectCommon(things.BaseThing):
    id = None
    id_kind_hasToModify = 0
    id_kind_isPublicForXmlRpc = 1
    id_kindName = 'Id'

    language = None
    language_kind_balloonHelp = N_('Select which language you will type the contents of this object in.')
    language_kind_isPublicForXmlRpc = 1
    language_kind_isRequired = 1
    language_kindName = 'LanguageChoice'

    thingCategory = 'object'

    version = 0
    version_kind_hasToModify = 0
    version_kind_isPublicForXmlRpc = 1
    version_kind_isRequired = 1
    version_kindName = 'Integer'

    def canCache(self):
        if hasattr(self, 'readersSet_kind') \
           and self.readersSet_kind is not None \
           and not self.readersSet:
            return 1
        return 0

    def getId(self):
        return self.id

    def getLabel(self):
        raise NotImplementedError

    def getLabelLanguage(self):
        return self.getLanguage()

    def getLanguage(self):
        return self.language

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = things.BaseThing.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['id', 'version', 'language']
        return slotNames


class AdminWithoutWritersCommon(ObjectCommon):
    adminsSet = None
    adminsSet_kind_balloonHelp = N_('Select the people and groups who have administrative powers on this kind of items.')
    adminsSet_kind_isPublicForXmlRpc = 1
    adminsSet_kind_itemKind_valueName = 'Id'
    adminsSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    adminsSet_kindName = 'Sequence'

    language_kindName = None

    views = None
##     views_kind_isPublicForXmlRpc = 1
##     views_kind_itemKind_value_valueThingCategory = 'widget'
##     views_kind_itemKind_value_valueThingName = 'View'
##     views_kind_itemKind_valueName = 'Thing'
##     views_kindName = 'Sequence'

    def getView(self, name, default = None):
        if self.views:
            for view in self.views:
                if view.name == name:
                    return view
        return default

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['adminsSet', 'views']
        return slotNames

    def hasView(self, name):
        if self.views is None:
            return 0
        for view in self.views:
            if view.name == name:
                return 1
        return 0

    def newDefaultObject(self):
        raise NotImplementedError


class AdminCommon(AdminWithoutWritersCommon):
    writersSet = None
    writersSet_kind_balloonHelp = N_('Select the people and groups who are allowed to create items of this kind.')
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kind_requiredCount = 0
    writersSet_kindName = 'WritersSet'

    readersSet = None
    readersSet_kind_balloonHelp = N_('Select the people and groups who are allowed to read items of this kind.')
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = AdminWithoutWritersCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames = slotNames[:]
        if 'views' in slotNames:
            i = slotNames.index('views')
        else:
            i = len(slotNames)
        slotNames[i:i] = ['writersSet', 'readersSet']
        return slotNames
