# Glasnost 
# By: Odile Bnassy <obenassy@entrouvert.org>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@easter-eggs.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002, 2003 Odile Bnassy, Thierry Dulieu, Easter-eggs, 
#                    entr'ouvert, Frdric Pters,
#                    Emmanuel Raviart, Emmanuel Saracco & Thridion
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Operations Common Models"""

__version__ = '$Revision: 1.16 $'[11:-2]


import accounting
from ObjectsCommon import AdminCommon, ObjectCommon


class AdminOperationsCommon(AdminCommon):
    pass


class OperationCommon(ObjectCommon):
    comment = None
    comment_kind_isPublicForXmlRpc = 1
    comment_kindName = 'String'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    currency = None
    currency_kind_hasToModify = 0
    currency_kind_useCustomStorage = 1
    currency_kind_isPublicForXmlRpc = 1
    currency_kind_isRequired = 1
    currency_kind_values = accounting.currencyValues    
    currency_kindName = 'Choice'

    language_kindName = None

    lastEditorId = None
    lastEditorId_kind_isPublicForXmlRpc = 1
    lastEditorId_kind_serverRoles = ['people']
    lastEditorId_kindName = 'Id'

    ledgerId = None
    ledgerId_kind_isPublicForXmlRpc = 1
    ledgerId_kind_isRequired = 1
    ledgerId_kind_serverRoles = ['ledgers']
    ledgerId_kindName = 'Id'

    membersSet = None
    membersSet_kind_isPublicForXmlRpc = 1    
    membersSet_kind_itemKind_valueName = 'Id'
    membersSet_kind_itemKind_value_serverRoles = ['events']
    membersSet_kindName = 'Sequence'
    
    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    partnerId = None
    partnerId_kind_isPublicForXmlRpc = 1
    partnerId_kind_isRequired = 1
    partnerId_kind_serverRoles = ['groups', 'people']
    partnerId_kindName = 'Id'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    #type = None
    #type_kind_hasToModify = 0
    #type_kind_isPublicForXmlRpc = 1
    #type_kind_isRequired = 1
    #type_kind_values = accounting.typeValues    
    #type_kindName = 'Choice'
    
    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def canCache(self):
        return 0

    def getTitle(self):
        return self.title

    def getLabel(self):
        return self.getTitle()

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        return ''

    def getLedgerId(self):
        ledgerId = self.ledgerId
        if ledgerId is None:
            ledgerId = ''
        return ledgerId

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'ledgerId', 'title', 'currency', 'partnerId', 'comment', 
            'membersSet', 'creationTime', 'modificationTime', 'lastEditorId',
            'writersSet', 'readersSet']
        return slotNames

    def getReadersAndWriters(self):
        readers = self.readersSet
        if readers == None :
            readers = ''    
        writers = self.writersSet    
        if writers == None :
            writers = ''    
        return readers, writers


class OperationsCommon:
    adminClassName = 'AdminOperations'
    newObjectNameCapitalized = N_('New Operation')
    objectClassName = 'Operation'
    objectName = N_('operation')
    objectNameCapitalized = N_('Operation')
    objectsName = N_('operations')
    objectsNameCapitalized = N_('Operations')
    serverRole = 'operations'

