# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Organizations Common Models"""

__version__ = '$Revision: 1.12 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon
import geography


class AdminOrganizationsCommon(AdminCommon):
    pass


class OrganizationCommon(ObjectCommon):
    address = None
    address_kind_isPublicForXmlRpc = 1
    address_kindName = 'String'
    
    city = None
    city_kind_isPublicForXmlRpc = 1
    city_kind_isRequired = 1
    city_kindName = 'String'
    
    # Contacts
    contactIds = None
    contactIds_kind_balloonHelp = N_(
        'Select contact people for this organization.')
    contactIds_kind_isPublicForXmlRpc = 1
    contactIds_kind_itemKind_valueName = 'Id'
    contactIds_kind_itemKind_value_serverRoles = ['contactinfos', 'people']
    contactIds_kind_requiredCount = 1
    contactIds_kindName = 'Sequence'

    country = None
    country_kind_isPublicForXmlRpc = 1
    country_kindName = 'Choice'
    country_kind_isRequired = 1
    country_kind_values = geography.countryCodes
    
    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    # Description de l'organisation.
    description = None
    description_kind_balloonHelp = N_('Enter free description here.')
    description_kind_isPublicForXmlRpc = 1
    description_kind_isRequired = 1
    description_kindName = 'String'

    email = None
    email_kind_balloonHelp = N_(
        'If organization has an email address, enter it here in the form '
        'identifier@server.name.')
    email_kind_isPublicForXmlRpc = 1
    email_kindName = 'Email'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    name = None
    name_kind_balloonHelp = N_('Enter organization name.')
    name_kind_isPublicForXmlRpc = 1
    name_kind_isRequired = 1
    name_kindName = 'String'

    # Statut juridique de cette organisation : association loi 1901,
    # cooperative, mutuelle, entreprise, autre - precisez.
    official = None
    official_kind_balloonHelp = N_('Enter organization legal status.')
    official_kind_isPublicForXmlRpc = 1
    official_kind_isRequired = 1
    official_kind_values = [
        'nonprofit',
        'cooperative',
        'mutual',
        'local',
        'company',
        ]
    official_kindName = 'XChoice'

    phone = None
    phone_kind_isPublicForXmlRpc = 1
    phone_kind_isRequired = 1
    phone_kindName = 'String'

    readersSet = None
    readersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to read this item.')
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    # Site web de l'organization.
    webSite = None
    webSite_kind_balloonHelp = N_('Enter organization web site.')
    webSite_kind_isPublicForXmlRpc = 1
    webSite_kind_isRequired = 1
    webSite_kindName = 'String'

    writersSet = None
    writersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to edit this item.')
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    zip = None
    zip_kind_isPublicForXmlRpc = 1
    zip_kind_isRequired = 1
    zip_kindName = 'String'
    
    def getLabel(self):
        label = self.name
        if label is None:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'name', 'official', 'description', 'webSite', 'email', 'phone',
            'address', 'zip', 'city', 'country', 'contactIds' 
            'creationTime', 'modificationTime', 'writersSet', 'readersSet']
        return slotNames

class OrganizationsCommon:
    adminClassName = 'AdminOrganizations'
    newObjectNameCapitalized = N_('New Organization')
    objectClassName = 'Organization'
    objectName = N_('organization')
    objectNameCapitalized = N_('Organization')
    objectsName = N_('organizations')
    objectsNameCapitalized = N_('Organizations')
    serverRole = 'organizations'

