# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost PELActions Common Models"""

__version__ = '$Revision: 1.7 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminPELActionsCommon(AdminCommon):
    pass


class PELActionCommon(ObjectCommon):
    abstract = None
    abstract_kind_isPublicForXmlRpc = 1
    abstract_kind_isRequired = 1
    abstract_kindName = 'String'

    actionStatus = None
    actionStatus_kind_isPublicForXmlRpc = 1
    actionStatus_kind_isRequired = 1
    actionStatus_kind_values = [
        'new',
        'renewed',
        'continued',
        ]
    actionStatus_kindName = 'Choice'

    ages = None
    ages_kind_isPublicForXmlRpc = 1
    ages_kind_itemKind_value_values = [
        '0-2',
        '3-5',
        '6-11',
        '12-15', 
        '16+',
        ]
    ages_kind_itemKind_valueName = 'Choice'
    ages_kind_requiredCount = 1
    ages_kindName = 'Sequence'

    area = None
    area_kind_isPublicForXmlRpc = 1
    area_kind_itemKind_valueName = 'Choice'
    area_kind_requiredCount = 1
    area_kind_itemKind_value_values = [
        'insidearea',
        'outsidearea',
        ]
    area_kindName = 'Sequence'

    authorsSet = None
    authorsSet_kind_balloonHelp = N_('Select the author(s) of the item.')
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    contactIds = None
    contactIds_kind_balloonHelp = N_('Select contact people for this action.')
    contactIds_kind_isPublicForXmlRpc = 1
    contactIds_kind_itemKind_valueName = 'Id'
    contactIds_kind_itemKind_value_serverRoles = ['contactinfos', 'people']
    contactIds_kind_requiredCount = 1
    contactIds_kindName = 'Sequence'

    contributions = None
    contributions_kind_isPublicForXmlRpc = 1
    contributions_kind_keyKind_valueName = 'String'
    contributions_kind_valueKind_valueName = 'Integer'
    contributions_kindName = 'Mapping'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    description = None
    description_kind_isPublicForXmlRpc = 1
    description_kind_isRequired = 1
    description_kindName = 'String'

    designPartners = None
    designPartners_kind_isPublicForXmlRpc = 1
    designPartners_kind_isRequired = 1
    designPartners_kindName = 'String'

    duration = None
    duration_kind_isPublicForXmlRpc = 1
    duration_kind_requiredParts = ['hour', 'minute']
    duration_kindName = 'Duration'

    endDate = None
    endDate_kind_isPublicForXmlRpc = 1
    endDate_kind_isRequired = 1
    endDate_kindName = 'Time'

    evaluationPartners = None
    evaluationPartners_kind_isPublicForXmlRpc = 1
    evaluationPartners_kind_isRequired = 1
    evaluationPartners_kindName = 'String'

    expectedResults = None
    expectedResults_kind_isPublicForXmlRpc = 1
    expectedResults_kind_isRequired = 1
    expectedResults_kindName = 'String'

    goals = None
    goals_kind_isPublicForXmlRpc = 1
    goals_kind_itemKind_valueName = 'XChoice'
    goals_kind_requiredCount = 1
    goals_kind_itemKind_value_values = [
        'freetime',
        'autonomy',
        'remedial',
        'socialisation',
        'appropriation',
        'physical',
        'effort',
        'localenv',
        'remoteenv',
        'citizenship',
        'violence',
        'parents',
        'generationmix',
        'expression',
        'selfrespect',
        'locallife',
        'localbonds',
        'none',
        ]
    goals_kindName = 'Sequence'

    implementationPartners = None
    implementationPartners_kind_isPublicForXmlRpc = 1
    implementationPartners_kind_isRequired = 1
    implementationPartners_kindName = 'String'

    language_kindName = None

    means = None
    means_kind_isPublicForXmlRpc = 1
    means_kind_isRequired = 1
    means_kindName = 'String'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    name = None
    name_kind_isPublicForXmlRpc = 1
    name_kind_isRequired = 1
    name_kindName = 'String'

    ordinaryFundings = None
    ordinaryFundings_kind_isPublicForXmlRpc = 1
    ordinaryFundings_kind_keyKind_valueName = 'String'
    ordinaryFundings_kind_valueKind_valueName = 'Integer'
    ordinaryFundings_kindName = 'Mapping'

    organizationCosts = None
    organizationCosts_kind_isPublicForXmlRpc = 1
    organizationCosts_kindName = 'Integer'

    otherFundings = None
    otherFundings_kind_deleteNullValues = 1
    otherFundings_kind_isPublicForXmlRpc = 1
    otherFundings_kind_keyKind_valueName = 'String'
    otherFundings_kind_valueKind_valueName = 'Integer'
    otherFundings_kindName = 'Mapping'

    owner = None
    owner_kind_isPublicForXmlRpc = 1
    owner_kind_isRequired = 1
    owner_kindName = 'String'

    participantCount = None
    participantCount_kind_isPublicForXmlRpc = 1
    participantCount_kind_isRequired = 1
    participantCount_kindName = 'Integer'

    periodicity = None
    periodicity_kind_isPublicForXmlRpc = 1
    periodicity_kind_isRequired = 1
    periodicity_kind_values = [
        'lunch',
        'wednesday',
        'saturday',
        'schooldays',
        'holiday',
        'vacation',
        ]
    periodicity_kindName = 'Choice'

    presentationDate = None
    presentationDate_kind_isPublicForXmlRpc = 1
    presentationDate_kind_isRequired = 1
    presentationDate_kindName = 'Time'

    purchases = None
    purchases_kind_isPublicForXmlRpc = 1
    purchases_kind_keyKind_valueName = 'String'
    purchases_kind_valueKind_valueName = 'Integer'
    purchases_kindName = 'Mapping'

    readersSet = None
    readersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to read this item.')
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    requiredSubsidy = None
    requiredSubsidy_kind_isPublicForXmlRpc = 1
    requiredSubsidy_kindName = 'Integer'

    salaries = None
    salaries_kind_isPublicForXmlRpc = 1
    salaries_kind_keyKind_valueName = 'String'
    salaries_kind_valueKind_valueName = 'Integer'
    salaries_kindName = 'Mapping'

    startDate = None
    startDate_kind_isPublicForXmlRpc = 1
    startDate_kind_isRequired = 1
    startDate_kindName = 'Time'

    targetAudience = None
    targetAudience_kind_isPublicForXmlRpc = 1
    targetAudience_kind_isRequired = 1
    targetAudience_kindName = 'String'

    territory = None
    territory_kind_isPublicForXmlRpc = 1
    territory_kind_isRequired = 1
    territory_kind_values = [
        'basrivedroite',
        'hautrivedroite',
        'bellevue',
        'saintmarc',
        'centreville',
        'europe',
        'lambezellec',
        ]
    territory_kindName = 'Choice'

    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    totalExpenses = None 
    #totalExpenses_kind_hasToModify = 0
    totalExpenses_kind_isPublicForXmlRpc = 1
    totalExpenses_kind_isReadOnly = 1
    totalExpenses_kindName = 'Integer'
    
    totalResources = None
    #totalResources_kind_hasToModify = 0
    totalResources_kind_isPublicForXmlRpc = 1 
    totalResources_kind_isReadOnly = 1 
    totalResources_kindName = 'Integer'

    writersSet = None 
    writersSet_kind_balloonHelp = N_( 
        'Select persons or groups who will be able to edit this item.')
    writersSet_kind_isPublicForXmlRpc = 1 
    writersSet_kindName = 'WritersSet' 


    def getLabel(self):
        label = self.title
        if label is None:
            return ''
        return label

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        language = self.language
        if language == None :
            return ''
        return language 

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'territory', 'abstract', 'name', 'owner', 'contactIds',
            'actionStatus', 'startDate', 'endDate', 'periodicity', 'duration',
            'targetAudience', 'ages', 'participantCount', 'area',
            'description', 'designPartners', 'implementationPartners',
            'evaluationPartners', 'goals', 'means', 'expectedResults',
            'presentationDate',
            ]
        slotNames.append('purchases')
        slotNames += [
            'creationTime', 'modificationTime', 'authorsSet',
            'writersSet', #'readersSet',
            ]
        return slotNames


class PELActionsCommon:
    adminClassName = 'AdminPELActions'
    newObjectNameCapitalized = 'Nouvelle Action Plan Educatif Local'
    objectClassName = 'PELAction'
    objectName = 'action Plan Educatif Local'
    objectNameCapitalized = 'Action Plan Educatif Local'
    objectsName = 'actions Plan Educatif Local'
    objectsNameCapitalized = 'Actions Plan Educatif Local'
    serverRole = 'pelactions'

