# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost People Common Models"""

__version__ = '$Revision: 1.16 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminPeopleCommon(AdminCommon):
    authenticationServerId = None
    authenticationServerId_kind_isPublicForXmlRpc = 1
    authenticationServerId_serverRoles = ['authenticationldap']
    authenticationServerId_kindName = 'ServerId'

    defaultGroupId = None
    defaultGroupId_kind_balloonHelp = N_('Select the group new users will be automatically added to.')
    defaultGroupId_kind_isPublicForXmlRpc = 1
    defaultGroupId_kind_serverRoles = ['groups']
    defaultGroupId_kindName = 'Id'

    userCanChoosePassword = 0
    userCanChoosePassword_kind_balloonHelp = N_('Specifies whether users are allowed to choose their passwords (instead of autogenerating one for them).')
    userCanChoosePassword_kind_isPublicForXmlRpc = 1
    userCanChoosePassword_kind_isRequired = 1
    userCanChoosePassword_kindName = 'Boolean'

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = AdminCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['defaultGroupId', 'authenticationServerId',
                      'userCanChoosePassword']
        return slotNames


class PersonCommon(ObjectCommon):
    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    email = None
    email_kind_balloonHelp = N_('Enter the e-mail address.')
    email_kind_isPublicForXmlRpc = 1
    email_kind_isRequired = 1
    email_kindName = 'Email'

    fingerprint = None
    fingerprint_kind_balloonHelp = N_('Enter the GnuPG public key fingerprint; mandatory to receive encrypted e-mails.')
    fingerprint_kind_isPublicForXmlRpc = 1
    fingerprint_kindName = 'Fingerprint'

    firstName = None
    firstName_kind_balloonHelp = N_('Enter the first name.')
    firstName_kind_isPublicForXmlRpc = 1
    firstName_kind_isRequired = 1
    firstName_kind_isTranslatable = 0
    firstName_kindName = 'String'

    language_kindName = None

    lastName = None
    lastName_kind_balloonHelp = N_('Enter the last name.')
    lastName_kind_isPublicForXmlRpc = 1
    lastName_kind_isRequired = 1
    lastName_kind_isTranslatable = 0
    lastName_kindName = 'String'

    login = None
    login_kind_balloonHelp = N_('Enter the login.')
    login_kind_isPublicForXmlRpc = 1
    login_kind_isRequired = 1
    login_kind_isTranslatable = 0
    login_kindName = 'String'

    loginTime = None
    loginTime_kind_isPublicForXmlRpc = 1
    loginTime_kindName = 'Time'

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    nickname = None
    nickname_kind_balloonHelp = N_('Enter an eventual nickname here.')
    nickname_kind_isPublicForXmlRpc = 1
    nickname_kind_isTranslatable = 0
    nickname_kindName = 'String'

    password = None
    password_kind_isPublicForXmlRpc = 1
    password_kind_isRequired = 1
    password_kindName = 'Password'

    voteTokens = None
    voteTokens_kind_hasToModify = 0
    voteTokens_kind_isPublicForXmlRpc = 1
    voteTokens_kind_keyKind_valueName = 'Id'
    voteTokens_kind_keyKind_value_serverRoles = ['elections']
    voteTokens_kind_valueKind_valueName = 'Token'
    voteTokens_kindName = 'Mapping'

    def canCache(self):
        return 0

    def containsText(self, text):
        text = text.lower()
        for attributeName in [
            'firstName',
            'lastName',
            'login',
            'nickname',
            ]:
            attribute = getattr(self, attributeName)
            if type(attribute) is type('') \
               and attribute.lower().find(text) >= 0:
                return 1
        return 0

    def getFullName(self):
        return getFullName(self.firstName, self.lastName, self.nickname)

    def getLabel(self):
        label = self.getFullName()
        if label is None:
            return ''
        return label

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        return ''

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'firstName', 'lastName', 'nickname', 'login', 'password', 'email',
            'fingerprint', 'creationTime', 'modificationTime', 'loginTime']
        return slotNames

    def getSortString(self):
        if self.lastName:
            sortString = self.lastName
        else:
            sortString = ''
        if self.firstName:
            sortString += ' ' + self.firstName
        return sortString


def getFullName(firstName, lastName, nickname = None):
    fullName = ''
    if firstName:
        if not fullName:
            fullName = firstName
        else:
            fullName = fullName + ' ' + firstName
    if nickname:
        if not fullName:
            fullName = '"' + nickname + '"'
        else:
            fullName = fullName + ' ' + '"' + nickname + '"'
    if lastName:
        if not fullName:
            fullName = lastName
        else:
            fullName = fullName + ' ' + lastName
    return fullName

class PeopleCommon:
    adminClassName = 'AdminPeople'
    newObjectNameCapitalized = N_('New Person')
    objectClassName = 'Person'
    objectName = N_('person')
    objectNameCapitalized = N_('Person')
    objectsName = N_('people')
    objectsNameCapitalized = N_('People')
    serverRole = 'people'

