# Glasnost 
# By: Odile Bnassy <obenassy@entrouvert.org>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@easter-eggs.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Records Common Models"""

__version__ = '$Revision: 1.23 $'[11:-2]


from tools import extractLocalId
import accounting
import faults
from ObjectsCommon import AdminCommon, ObjectCommon

from glasnost.proxy.tools import getProxyForServerRole


class AdminRecordsCommon(AdminCommon):
    pass


class RecordCommon(ObjectCommon):
    accountId = None
    accountId_kind_isPublicForXmlRpc = 1
    accountId_kind_isRequired = 1
    accountId_kind_serverRoles = ['accounts']
    accountId_kindName = 'Id'

    accountLabel = None
    accountLabel_kind_hasToModify = 0
    accountLabel_kind_isPublicForXmlRpc = 1
    accountLabel_kind_useCustomStorage = 1
    accountLabel_kindName = 'String'
    
    accountNumber = None
    accountNumber_kind_hasToModify = 0
    accountNumber_kind_isPublicForXmlRpc = 1
    accountNumber_kind_useCustomStorage = 1
    accountNumber_kindName = 'String'
    
    amount = None
    amount_kind_isPublicForXmlRpc = 1
    amount_kind_isRequired = 1
    amount_kindName = 'Amount'

    certainty = None
    certainty_kind_isPublicForXmlRpc = 1
    certainty_kind_useCustomStorage = 1
    certainty_kind_values = accounting.certaintyValues
    certainty_kindName = 'IntegerChoice'
   
    currency = None
    currency_kind_hasToModify = 0
    currency_kind_useCustomStorage = 1
    currency_kind_isPublicForXmlRpc = 1
    currency_kind_isRequired = 1
    currency_kind_values = accounting.currencyValues    
    currency_kindName = 'Choice'

    date = None
    date_kind_hasToModify = 0
    date_kind_useCustomStorage = 1
    date_kind_isPublicForXmlRpc = 1
    date_kindName = 'Date'
    
    eventId = None
    eventId_kind_isPublicForXmlRpc = 1
    eventId_kind_isRequired = 1
    eventId_kind_serverRoles = ['events']
    eventId_kindName = 'Id'

    label = None
    label_kind_hasToModify = 0
    label_kind_isPublicForXmlRpc = 1
    label_kind_useCustomStorage = 1
    label_kindName = 'String'

    language_kindName = None

    partnerId = None
    partnerId_kind_useCustomStorage = 1
    partnerId_kind_hasToModify = 0
    partnerId_kind_isPublicForXmlRpc = 1
    partnerId_kind_serverRoles = ['groups', 'people']
    partnerId_kindName = 'Id'

    readersSet = None
    readersSet_kind_hasToModify = 0
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kind_useCustomStorage = 1
    readersSet_kindName = 'ReadersSet'

    voucherId = None
    voucherId_kind_useCustomStorage = 1
    voucherId_kind_hasToModify = 0
    voucherId_kind_isPublicForXmlRpc = 1
    voucherId_kind_serverRoles = ['uploadfiles']
    voucherId_kindName = 'Id'

    writersSet = None
    writersSet_kind_hasToModify = 0
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kind_useCustomStorage = 1
    writersSet_kindName = 'WritersSet'

    def acquireAccountData(self):
        if self.accountId is None:
            self.getSlot('accountNumber').setValue(None)
            self.getSlot('accountLabel').setValue(None)
        else:
            try:
                account = getProxyForServerRole('accounts').getPartialObject(
                    self.accountId, ['label', 'number'])
            except faults.MissingItem:
                self.getSlot('accountLabel').setValue(None)
                self.getSlot('accountNumber').setValue(None)
            else:
                self.getSlot('accountLabel').setValue(
                    account.getSlot('label').getValue())
                self.getSlot('accountNumber').setValue(
                    account.getSlot('number').getValue())

    def canCache(self):
        return 0

    def getLabel(self):
        if self.accountNumber is None:
            self.acquireAccountData()
        localId = extractLocalId(self.id)
        return '%s - %s -- %s' % (
            localId, self.accountNumber,
            accounting.amountDisplay(self.amount)
            )

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        language = self.language
        if language == None :
            return ''
        return language 

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'eventId', 'partnerId', 'date', 'label', 'voucherId', 'accountId',
            'currency', 'amount', 'side', 'certainty', 'writersSet',
            'readersSet']
        return slotNames
        

class RecordsCommon:
    adminClassName = 'AdminRecords'
    newObjectNameCapitalized = N_('New Record')
    objectClassName = 'Record'
    objectName = N_('record')
    objectNameCapitalized = N_('Record')
    objectsName = N_('records')
    objectsNameCapitalized = N_('Records')
    serverRole = 'records'
