# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Skill Demands Common Models"""

__version__ = '$Revision: 1.2 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminSkillDemandsCommon(AdminCommon):
    pass


class SkillDemandCommon(ObjectCommon):
    authorsSet = None
    authorsSet_kind_balloonHelp = N_('Select the author(s) of this item.')
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    contactIds = None
    contactIds_kind_balloonHelp = N_(
        'Select contact people for this demand.')
    contactIds_kind_isPublicForXmlRpc = 1
    contactIds_kind_itemKind_valueName = 'Id'
    contactIds_kind_itemKind_value_serverRoles = ['contactinfos']
    contactIds_kind_requiredCount = 1
    contactIds_kindName = 'Sequence'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    deal = None
    deal_kind_balloonHelp = N_('What do you expect as compensation?')
    deal_kind_isPublicForXmlRpc = 1
    deal_kind_isRequired = 1
    deal_kind_values = [
        'nocharge',
        'exchanging',
        'paying',
        ]
    deal_kindName = 'XChoice'
    
    language_kindName = None

    modificationTime = None
    modificationTime_kind_isPublicForXmlRpc = 1
    modificationTime_kindName = 'ModificationTime'

    demandDescription = None
    demandDescription_kind_balloonHelp = N_('Describe your demand.')
    demandDescription_kind_isPublicForXmlRpc = 1
    demandDescription_kind_isRequired = 1
    demandDescription_kindName = 'String'

    demandKeywords = None
    demandKeywords_kind_balloonHelp = N_('Keyword(s) matching your demand.')
    demandKeywords_kind_isPublicForXmlRpc = 1
    demandKeywords_kind_itemKind_value_values = []
    demandKeywords_kind_itemKind_valueName = 'Choice'
    #demandKeywords_kind_requiredCount = 1
    demandKeywords_kind_requiredCount = 0
    demandKeywords_kindName = 'Sequence'

    # (de quelles personnes ou organisations provient cette offre ?)
    ownerIds = None
    ownerIds_kind_balloonHelp = N_(
        'Who carries this skill demand (organization/person/group of people)?')
    ownerIds_kind_isPublicForXmlRpc = 1
    ownerIds_kind_itemKind_serverRoles = [
        'organizations', 'people', 'contactinfos', 'groups']
    ownerIds_kind_itemKind_valueName = 'Id'
    ownerIds_kind_requiredCount = 1
    ownerIds_kindName = 'Sequence'

    readersSet = None
    readersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to read this item.')
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    skillDescription = None
    skillDescription_kind_balloonHelp = N_('Describe your skills.')
    skillDescription_kind_isPublicForXmlRpc = 1
    skillDescription_kind_isRequired = 1
    skillDescription_kindName = 'String'

    skillList = None
    skillList_kind_isPublicForXmlRpc = 1
    #skillList_kind_requiredCount = 1
    skillList_kind_requiredCount = 0
    skillList_kind_itemKind_valueName = 'XChoice'
    skillList_kind_itemKind_values = []
    skillList_kindName = 'Sequence'

    title = None
    title_kind_balloonHelp = N_('Enter skill demand title.')
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    # Site web de l'offre (?).
    webSite = None
    webSite_kind_balloonHelp = N_(
        'More info about this demand on this web page...')
    webSite_kind_isPublicForXmlRpc = 1
    webSite_kindName = 'String'

    writersSet = None
    writersSet_kind_balloonHelp = N_(
        'Select persons or groups who will be able to edit this item.')
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def getLabel(self):
        label = self.title
        if label is None:
            return ''
        return label

    def getLabelLanguage(self):
        return ''

    def getLanguage(self):
        language = self.language
        if language == None :
            return ''
        return language 

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'title', 'ownerIds', 'contactIds', 'deal', 
            'skillList', 'skillDescription', 'demandKeywords',
            'demandDescription',
            'webSite', 'creationTime', 'modificationTime', 'authorsSet',
            'writersSet',
            ]
        return slotNames


class SkillDemandsCommon:
    adminClassName = 'AdminSkillDemands'
    newObjectNameCapitalized = N_('New Skill Demand')
    objectClassName = 'SkillDemand'
    objectName = N_('skill demand')
    objectNameCapitalized = N_('Skill Demand')
    objectsName = N_('skill demands')
    objectsNameCapitalized = N_('Skill Demands')
    serverRole = 'skilldemands'

