# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Tasks Common Models"""

__version__ = '$Revision: 1.7 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon


class AdminTasksCommon(AdminCommon):
    pass


class TaskCommon(ObjectCommon):
    authorsSet = None
    authorsSet_kind_isPublicForXmlRpc = 1
    authorsSet_kindName = 'AuthorsSet'

    category = None
    category_kind_isPublicForXmlRpc = 1
    category_kind_isRequired = 1
    category_kindName = 'String'

    creationTime = None
    creationTime_kind_isPublicForXmlRpc = 1
    creationTime_kindName = 'CreationTime'

    description = None
    description_kind_isPublicForXmlRpc = 1
    description_kindName = 'String'

    endingTime = None
    endingTime_kind_isPublicForXmlRpc = 1
    endingTime_kindName = 'Time'

    ownersSet = None
    ownersSet_kind_isPublicForXmlRpc = 1
    ownersSet_kind_itemKind_valueName = 'Id'
    ownersSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    ownersSet_kind_requiredCount = 1
    ownersSet_kindName = 'Sequence'

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'
    
    title = None
    title_kind_isPublicForXmlRpc = 1
    title_kind_isRequired = 1
    title_kindName = 'String'

    priority = 10
    priority_kind_isPublicForXmlRpc = 1
    priority_kind_isRequired = 1
    priority_kindName = 'Integer'
    
    state = 'not started'
    state_kind_isPublicForXmlRpc = 1
    state_kind_isRequired = 1
    state_kind_values = [
        'not started',
        'in progress',
        'finish'
        ]
    state_kindName = 'Choice'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'
    
    def getLabel(self):
        label = self.title
        if label is None:
            return ''
        return label

    def isAssigned(self):
        return ownerSet is not None

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'category',
            'title',
            'description',
            'creationTime',
            'endingTime',
            'ownersSet',
            'priority',
            'state',
            'authorsSet',
            'writersSet',
            'readersSet',
            ]
        return slotNames


class TasksCommon:
    adminClassName = 'AdminTasks'
    newObjectNameCapitalized = N_('New Task')
    objectClassName = 'Task'
    objectName = N_('task')
    objectNameCapitalized = N_('Task')
    objectsName = N_('tasks')
    objectsNameCapitalized = N_('Tasks')
    serverRole = 'tasks'

