# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Translations Common Models"""

__version__ = '$Revision: 1.11 $'[11:-2]


from ObjectsCommon import AdminWithoutWritersCommon, ObjectCommon
import translation


class AdminTranslationsCommon(AdminWithoutWritersCommon):
    translatorsSets = None
    translatorsSets_kind_isPublicForXmlRpc = 1
    translatorsSets_kind_isRequired = 1
    translatorsSets_kindName = 'TranslatorsSets'


class LocalizationCommon(ObjectCommon):
    destinationLanguage = None
    destinationLanguage_kind_isPublicForXmlRpc = 1
    destinationLanguage_kind_isRequired = 1
    destinationLanguage_kind_values = translation.languageKeys
    destinationLanguage_kindName = 'Choice'

    destinationString = None
    destinationString_kind_isPublicForXmlRpc = 1
    destinationString_kind_isRequired = 1
    destinationString_kindName = 'String'
    
    id_kindName = None

    isFuzzy = 0
    isFuzzy_kind_isPublicForXmlRpc = 1
    isFuzzy_kind_isRequired = 1
    isFuzzy_kindName = 'Boolean'

    isTranslatable = 0
    isTranslatable_kind_isPublicForXmlRpc = 1
    isTranslatable_kind_isRequired = 1
    isTranslatable_kindName = 'Boolean'

    language_kindName = None

    similarString = None
    similarString_kind_isPublicForXmlRpc = 1
    similarString_kindName = 'String'

    sourceIds = None
    sourceIds_kind_isPublicForXmlRpc = 1
    sourceIds_kind_itemKind_valueName = 'Id'
    sourceIds_kindName = 'Sequence'

    sourceLanguage = None
    sourceLanguage_kind_isPublicForXmlRpc = 1
    sourceLanguage_kind_isRequired = 1
    sourceLanguage_kind_values = translation.languageKeys
    sourceLanguage_kindName = 'Choice'

    sourceString = None
    sourceString_kind_isPublicForXmlRpc = 1
    sourceString_kind_isRequired = 1
    sourceString_kindName = 'String'

    sourceStringDigest = None
    sourceStringDigest_kind_isPublicForXmlRpc = 1
    sourceStringDigest_kindName = 'String'

    def getKey(self):
        return self.sourceLanguage + self.destinationLanguage

    def getLabel(self):
        label = self.getTitle()
        if label is None:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'sourceLanguage', 'sourceString', 'sourceIds', 'isTranslatable',
            'destinationLanguage', 'similarString', 'destinationString',
            'isFuzzy']
        return slotNames

    def getTitle(self):
        title = self.sourceString
        if len(title) > 80:
            title = title[:60] + '...'
        return title


class TranslationsCommon:
    adminClassName = 'AdminTranslations'
    newObjectNameCapitalized = N_('New Translation')
    objectName = N_('translation')
    objectNameCapitalized = N_('Translation')
    objectsName = N_('translations')
    objectsNameCapitalized = N_('Translations')
    serverRole = 'translations'

