# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Workflows Common Models"""

__version__ = '$Revision: 1.13 $'[11:-2]


from ObjectsCommon import AdminCommon, ObjectCommon
## from kinds import BaseKind

#class WorkflowAction(BaseKind):
#    thingCategory = 'kind'

class WorkflowActionCommon(ObjectCommon):
    actionName = None
    actionName_kind_isPublicForXmlRpc = 1
    actionName_kindName = 'String'
    # isTranslatable = 0

    message = None
    message_kind_isPublicForXmlRpc = 1
    message_kindName = 'String'

class WorkflowActionEndCommon(WorkflowActionCommon):
    pass

class WorkflowActionShowMessageCommon(WorkflowActionCommon):
    message_kind_isRequired = 1

class WorkflowActionUserChoiceCommon(WorkflowActionCommon):
    message = None
    message_kind_isPublicForXmlRpc = 1
    message_kind_isRequired = 1
    message_kindName = 'String'

    choices = None
    choices_kind_isPublicForXmlRpc = 1
    choices_kind_keyKind_valueName = 'String'
    choices_kind_valueKind_value_itemKind_valueName = 'String' #'WorkflowAction' # (will this work?)
    choices_kind_valueKind_valueName = 'Sequence'
    choices_kindName = 'Mapping'


class WorkflowActionAddObjectCommon(WorkflowActionCommon):
    objectType = None
    objectType_kind_isPublicForXmlRpc = 1
    objectType_kind_isRequired = 1
    objectType_kindName = 'String' # should be serverRole

    ignoreFields = None
    ignoreFields_kind_isPublicForXmlRpc = 1
    ignoreFields_kind_itemKind_valueName = 'String'
    ignoreFields_kindName = 'Sequence'

class WorkflowActionEditObjectCommon(WorkflowActionCommon):
    objectId = None
    objectId_kind_isPublicForXmlRpc = 1
    objectId_kindName = 'Id'

    fieldToChange = None
    fieldToChange_kind_isPublicForXmlRpc = 1
    fieldToChange_kindName = 'String'

    operation = None
    operation_kind_isPublicForXmlRpc = 1
    operation_kindName = 'String'

    value = None
    value_kind_isPublicForXmlRpc = 1
    value_kindName = 'String'

class WorkflowActionMailCommon(WorkflowActionCommon):
    subject = None
    subject_kind_isPublicForXmlRpc = 1
    subject_kind_isRequired = 1
    subject_kindName = 'String'

    message = None
    message_kind_isPublicForXmlRpc = 1
    message_kind_isRequired = 1
    message_kindName = 'String'

    sender = None
    sender_kind_isPublicForXmlRpc = 1
    sender_kind_isRequired = 1
    sender_kindName = 'String'

    recipientsSet = None
    recipientsSet_kind_isPublicForXmlRpc = 1
    recipientsSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    recipientsSet_kind_itemKind_valueName = 'Id'
    recipientsSet_kind_requiredCount = 1
    recipientsSet_kindName = 'Sequence'

class WorkflowActionUserDataCommon(WorkflowActionCommon):
    dataKind = None
    dataKind_kind_isPublicForXmlRpc = 1
    dataKind_kind_isRequired = 1
    dataKind_kindName = 'Kind'

class WorkflowActionBranchCommon(WorkflowActionCommon):
    condition = None
    condition_kind_isPublicForXmlRpc = 1
    condition_kind_isRequired = 1
    condition_kindName = 'String'
    
    actionsTrue = None
    actionsTrue_kind_isPublicForXmlRpc = 1
    actionsTrue_kind_itemKind_value_valueThingCategory = 'object'
    actionsTrue_kind_itemKind_value_valueThingName = 'WorkflowAction'
    actionsTrue_kind_itemKind_valueName = 'Thing'
    actionsTrue_kindName = 'Sequence'

    actionsFalse = None
    actionsFalse_kind_isPublicForXmlRpc = 1
    actionsFalse_kind_itemKind_value_valueThingCategory = 'object'
    actionsFalse_kind_itemKind_value_valueThingName = 'WorkflowAction'
    actionsFalse_kind_itemKind_valueName = 'Thing'
    actionsFalse_kindName = 'Sequence'

class WorkflowActionGotoCommon(WorkflowActionCommon):
    gotoActionName = None
    gotoActionName_kind_isPublicForXmlRpc = 1
    gotoActionName_kind_isRequired = 1
    gotoActionName_kindName = 'String'



class AdminWorkflowsCommon(AdminCommon):
    pass


class WorkflowCommon(ObjectCommon):
    name = None
    name_kind_isPublicForXmlRpc = 1
    name_kind_isRequired = 1
    name_kindName = 'String'

    actions = None
    actions_kind_isPublicForXmlRpc = 1
    actions_kind_isRequired = 1
    actions_kind_itemKind_value_valueThingCategory = 'object'
    actions_kind_itemKind_value_valueThingName = 'WorkflowAction'
    actions_kind_itemKind_valueName = 'Thing'
    actions_kindName = 'Sequence'

    startupObjectType = None
    startupObjectType_kind_isPublicForXmlRpc = 1
    startupObjectType_kindName = 'String'

    startupEvents = None
    startupEvents_kind_isPublicForXmlRpc = 1
    startupEvents_kind_isRequired = 0
    startupEvents_kindName = 'Sequence'
    startupEvents_kind_itemKind_valueName = 'String' # For now

    readersSet = None
    readersSet_kind_isPublicForXmlRpc = 1
    readersSet_kindName = 'ReadersSet'

    runnersSet = None
    runnersSet_kind_isPublicForXmlRpc = 1
    runnersSet_kind_itemKind_valueName = 'Id'
    runnersSet_kind_itemKind_value_serverRoles = ['groups', 'people']
    runnersSet_kindName = 'Sequence'

    writersSet = None
    writersSet_kind_isPublicForXmlRpc = 1
    writersSet_kindName = 'WritersSet'

    def getLabel(self):
        label = self.name
        if label is None:
            return ''
        return label

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = ObjectCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'name', 'startupEvents', 'startupObjectType', 'actions',
            'writersSet', 'readersSet']
        return slotNames


class RunningWorkflowCommon(ObjectCommon):
    action = None
    action_kind_isPublicForXmlRpc = 1
    action_kindName = 'WorkflowAction'


class WorkflowsCommon:
    adminClassName = 'AdminWorkflows'
    newObjectNameCapitalized = N_('New Workflow')
    objectClassName = 'Workflow'
    objectName = N_('workflow')
    objectNameCapitalized = N_('Workflow')
    objectsName = N_('workflows')
    objectsNameCapitalized = N_('Workflows')
    serverRole = 'workflows'

