# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# inspired from xml-rpc librairies

__doc__ = """XmlRpc tools and converter"""

__version__ = '$Revision: 1.13 $'[11:-2]


import types

from glasnost.common import things
from glasnost.common import tools

import glasnost.proxy.tools_new as proxyTools


def convertFromXmlRpc(one):
    result = None
    if isGlasnostObjectSerialized(one):
        result = convertGlasnostObjectFromXmlRpc(one)
    elif type(one) == types.UnicodeType:
        result = convertStringFromXmlRpc(one)
    elif type(one) == types.StringType:
        result = convertStringFromXmlRpc(one)
    elif type(one) in [types.ListType, types.TupleType]:
        result = []
        for listItem in one:            
            result.append(convertFromXmlRpc(listItem))
    elif type(one) == types.DictionaryType:
        result = {}
        for k, v in one.items():            
            result[convertFromXmlRpc(k)] = convertFromXmlRpc(v)
    else:
        result = one
    return result


def convertGlasnostObjectFromXmlRpc(objectImport):
    object = newThing(
        objectImport['__thingCategory__'], objectImport['__thingName__'])
    object.importFromXmlRpc(objectImport)
    return object 


def convertGlasnostObjectToXmlRpc(obj):
    return obj.exportToXmlRpc()


def convertStringFromXmlRpc(str):
    return tools.iso8859_15(str)


def convertStringToXmlRpc(str):
    return tools.utf8(str)

    
def convertToXmlRpc(one):
    result = None
    if type(one) == types.UnicodeType:
        result = convertStringToXmlRpc(one)
    elif type(one) == types.StringType:
        result = convertStringToXmlRpc(one)
    elif type(one) in [types.ListType, types.TupleType]:
        result = []
        for listItem in one:            
            result.append(convertToXmlRpc(listItem))
    elif type(one) == types.DictionaryType:
        result = {}
        for k, v in one.items():            
            result[convertToXmlRpc(k)] = convertToXmlRpc(v)
    elif isGlasnostObject(one):
        result = convertGlasnostObjectToXmlRpc(one)
    else:
        result = one
    return result


def isGlasnostObject(obj):
    try:
        return isinstance(obj, things.BaseThing)
    except: # FIXME: tighten except
        pass
    return 0


def isGlasnostObjectSerialized(obj):
    try:
        return type(obj) == types.DictionaryType \
               and obj.has_key('__thingCategory__') \
               and obj.has_key('__thingName__')
    except: # FIXME: tighten except
        pass
    return 0


def newThing(thingCategory, thingName, *arguments, **keywords):
    ## to force import of all proxies
    proxyTools.getAllBaseProxies()
    import glasnost.proxy.things as proxyThings
    try:
        thingClass = proxyThings.allThingClasses[thingCategory][thingName]
    except KeyError:
        raise Exception(
            'Unknown category (= %s) or name (= %s) for new thing.'
            ' Known categories = %s' % (
            thingCategory, thingName, proxyThings.allThingClasses))
    return thingClass(*arguments, **keywords)
