# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Context Handling Functions"""

__version__ = '$Revision: 1.1 $'[11:-2]


import threading


class _Context:
    next = None
    vars = None

    def __init__(self, vars):
        self.vars = vars


def clear():
    threading.currentThread()._glasnostContext = None


def initThread(thread):
    context = _Context({})
    context.next = threading.currentThread()._glasnostContext
    thread._glasnostContext = context
    

def delVar(name):
    context = threading.currentThread()._glasnostContext
    if context.vars.has_key(name):
        del context.vars[name]


def getVar(name, default = None):
    context = threading.currentThread()._glasnostContext
    while context is not None:
        if context.vars.has_key(name):
            return context.vars[name]
        context = context.next
    return default


def getVarNames():
    varNames = []
    context = threading.currentThread()._glasnostContext
    while context is not None:
        for varName in context.vars.keys():
            if not varName in varNames:
                varNames.append(varName)
        context = context.next
    return varNames


def isClear():
    thread = threading.currentThread()
    return hasattr(thread, '_glasnostContext') \
           and thread._glasnostContext is None


def pull():
    thread = threading.currentThread()
    thread._glasnostContext = thread._glasnostContext.next


def push(**contextVars):
    thread = threading.currentThread()
    context = _Context(contextVars)
    context.next = thread._glasnostContext
    thread._glasnostContext = context


def setVar(name, value):
    threading.currentThread()._glasnostContext.vars[name] = value
