# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost XMLRPC Faults"""

__version__ = '$Revision: 1.13 $'[11:-2]


import types
import xmlrpclib


faultCodeUnknownServerException = 0
faultCodeUnknownApplicationToken = 1
faultCodeUnknownServerId = 2
#faultCodeWrongPersonAuthentication = 3
faultCodeUnknownUserToken = 4
faultCodeUserAccessDenied = 5
faultCodeMissingItem = 6
faultCodeUnknownSessionToken = 7
faultCodeApplicationAccessDenied = 8
faultCodeReadOnlyObject = 9
faultCodeMissingMainRubric = 10
faultCodeBadEmailAddress = 11
faultCodeSmtpError = 12
faultCodeNonExistentFilePath = 13
faultCodeNotAFile = 14
faultCodeFileAccessDenied = 15
faultCodeWrongLogin = 16
faultCodeWrongPassword = 17
faultCodeUnknownStringDigest = 18
faultCodeWrongVersion = 19
faultCodeDuplicateServer = 20
faultCodeServerRegistrationDenied = 21
faultCodeNoneValue = 22
faultCodeMissingId = 23
faultCodeDuplicateValue = 24
faultCodeInconsistentPrototypeHierarchy = 25
faultCodeNonExistentSlotPath = 26

faultCodeUnknownVoteToken = 1000
faultCodeUnknownVoterToken = 1001
faultCodeUnknownElectionToken = 1002

faultCodeDuplicateLogin = 2000
faultCodeDuplicateFullName = 2001
faultCodeDuplicateEmail = 2002
faultCodeDuplicateFingerprint = 2003

faultCodeDuplicateHostName = 3000

faultCodeMissingParent = 4000

Fault = xmlrpclib.Fault


class BaseFault(xmlrpclib.Fault):
    def __init__(self, *arguments):
        xmlrpclib.Fault.__init__(self, self.faultCode,
                                 self.makeFaultString(*arguments))

    def makeFaultString(self, *arguments):
        return 'Unknown fault (arguments = %s)' % str(arguments)


class UnknownServerException(BaseFault):
    faultCode = faultCodeUnknownServerException

    def makeFaultString(self):
        return 'Unknown server exception'


class UnknownApplicationToken(BaseFault):
    faultCode = faultCodeUnknownApplicationToken

    def makeFaultString(self, applicationToken):
        return 'Unknown application token = %s' % applicationToken


class UnknownServerId(BaseFault):
    faultCode = faultCodeUnknownServerId

    def makeFaultString(self, serverId):
        return 'Unknown server id = %s' % serverId
 

## class WrongPersonAuthentication(BaseFault):
##     faultCode = faultCodeWrongPersonAuthentication
## 
##     def makeFaultString(self, login, password):
##         return 'Unknown person authentication login = "%s",' \
##             ' password = "%s"' % (login, password)


class UnknownUserToken(BaseFault):
    faultCode = faultCodeUnknownUserToken

    def makeFaultString(self, userToken):
        return 'Unknown user token = %s' % userToken


class UserAccessDenied(BaseFault):
    faultCode = faultCodeUserAccessDenied

    def makeFaultString(self):
        return 'Access denied to user'


class MissingItem(BaseFault):
    faultCode = faultCodeMissingItem

    def makeFaultString(self, itemId):
        return 'Missing item = %s' % itemId


class MissingParent(BaseFault):
    faultCode = faultCodeMissingParent

    def makeFaultString(self, objectId):
        return 'Missing parent for %s' % objectId


class UnknownSessionToken(BaseFault):
    faultCode = faultCodeUnknownSessionToken

    def makeFaultString(self, sessionToken):
        return 'Unknown session token = %s' % sessionToken


class ApplicationAccessDenied(BaseFault):
    faultCode = faultCodeApplicationAccessDenied

    def makeFaultString(self, virtualServerId):
        return 'Access denied to application = %s' % virtualServerId


class ReadOnlyObject(BaseFault):
    faultCode = faultCodeReadOnlyObject

    def makeFaultString(self):
        return 'Object is read only'


class MissingMainRubric(BaseFault):
    faultCode = faultCodeMissingMainRubric

    def makeFaultString(self):
        return 'Main rubric is missing'


class BadEmailAddress(BaseFault):
    faultCode = faultCodeBadEmailAddress

    def makeFaultString(self, email):
        return 'Bad email address = %s' % email


class SmtpError(BaseFault):
    faultCode = faultCodeSmtpError

    def makeFaultString(self):
        return 'A SMTP error occurred'


class NonExistentFilePath(BaseFault):
    faultCode = faultCodeNonExistentFilePath

    def makeFaultString(self, filePath):
        return 'Non existent file path = %s' % filePath


class NotAFile(BaseFault):
    faultCode = faultCodeNotAFile

    def makeFaultString(self, filePath):
        return 'Not a file = %s' % filePath


class FileAccessDenied(BaseFault):
    faultCode = faultCodeFileAccessDenied

    def makeFaultString(self, filePath):
        return 'File access denied = %s' % filePath


class WrongLogin(BaseFault):
    faultCode = faultCodeWrongLogin

    def makeFaultString(self, login):
        return 'Unknown login = "%s"' % login


class WrongPassword(BaseFault):
    faultCode = faultCodeWrongPassword

    def makeFaultString(self, password):
        return 'Unknown password = "%s"' % password


class UnknownStringDigest(BaseFault):
    faultCode = faultCodeUnknownStringDigest

    def makeFaultString(self, digest):
        return 'Unknown string digest = "%s"' % digest


class WrongVersion(BaseFault):
    faultCode = faultCodeWrongVersion

    def makeFaultString(self):
        return 'Wrong version'


class DuplicateServer(BaseFault):
    faultCode = faultCodeDuplicateServer

    def makeFaultString(self, hostName, port):
        return 'Duplicate server: host name = %s, port = %s' % (hostName, port)


class ServerRegistrationDenied(BaseFault):
    faultCode = faultCodeServerRegistrationDenied

    def makeFaultString(self, hostName, port):
        return 'Registration denied to host name = %s, port = %s' % (
            hostName, port)


class NoneValue(BaseFault):
    faultCode = faultCodeNoneValue

    def makeFaultString(self):
        return 'Wrong value: None'


class MissingId(BaseFault):
    faultCode = faultCodeMissingId

    def makeFaultString(self):
        return 'Missing ID'


class DuplicateValue(BaseFault):
    faultCode = faultCodeDuplicateValue

    def makeFaultString(self, name, value):
        return 'Duplicate value for %s = %s' % (name, value)


class InconsistentPrototypeHierarchy(BaseFault):
    faultCode = faultCodeInconsistentPrototypeHierarchy

    def makeFaultString(self, implementation):
        return 'Inconsistent prototype hierarchy for implementation %s' \
               % implementation


class NonExistentSlotPath(BaseFault):
    faultCode = faultCodeNonExistentSlotPath

    def makeFaultString(self, slotPath):
        return 'Non existent slot path = %s' % slotPath


class UnknownVoteToken(BaseFault):
    faultCode = faultCodeUnknownVoteToken

    def makeFaultString(self, voteToken):
        return 'Unknown vote token = %s' % voteToken


class UnknownVoterToken(BaseFault):
    faultCode = faultCodeUnknownVoterToken

    def makeFaultString(self, voterToken):
        return 'Unknown voter token = %s' % voterToken


class UnknownElectionToken(BaseFault):
    faultCode = faultCodeUnknownElectionToken

    def makeFaultString(self, electionToken):
        return 'Unknown election token = %s' % electionToken


class DuplicateLogin(BaseFault):
    faultCode = faultCodeDuplicateLogin

    def makeFaultString(self, login):
        return 'Duplicate login = %s' % login


class DuplicateFullName(BaseFault):
    faultCode = faultCodeDuplicateFullName

    def makeFaultString(self, fullName):
        return 'Duplicate fullName = %s' % fullName


class DuplicateEmail(BaseFault):
    faultCode = faultCodeDuplicateEmail

    def makeFaultString(self, email):
        return 'Duplicate email address = %s' % email


class DuplicateFingerprint(BaseFault):
    faultCode = faultCodeDuplicateFingerprint

    def makeFaultString(self, fingerprint):
        return 'Duplicate fingerprint = %s' % fingerprint


class DuplicateHostName(BaseFault):
    faultCode = faultCodeDuplicateHostName

    def makeFaultString(self, hostName):
        return 'Duplicate host name = %s' % hostName


faults = {}
for value in locals().values():
    if type(value) == types.ClassType and not value in (Fault, BaseFault):
        faults[value.faultCode] = value
