# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Common Kinds"""

__version__ = '$Revision: 1.84 $'[11:-2]


import base64
import locale
import time
import types

import functions
import slots
import translation
import things
from tools import *
import tools_new as commonTools

import functions # Do not remove!
import properties # Do not remove!
import uploads # Do not remove!
import views # Do not remove!
import widgets # Do not remove!


register = things.register


class EmptyKind(things.BaseThing):
    category = None
    category_kind_isProperty = 0
    category_kind_isPublicForXmlRpc = 0
    category_kind_label = N_('Category')
    category_kindName = 'String'

    name = None
    name_kind_isProperty = 0
    name_kind_isPublicForXmlRpc = 0
    name_kind_label = N_('Name')
    name_kindName = 'String'

    thingCategory = 'kind'

    def __init__(self, **attributes):
        things.BaseThing.__init__(self)
        for name, value in attributes.items():
            # Ensure that each argument has been declared as an attribute.
            if not hasattr(self.__class__, name):
                raise Exception('Undeclared attribute %s for %s' % (
                    name, self.__class__.__name__))
            if getattr(self, name) == value:
                continue
            setattr(self, name, value)

    def __str__(self):
        return self.getName()

    def buildOptions(self, options):
        for name, value in options.items():
            # Ensure that each option has been declared as an attribute.
            nameBase = name.split('_')[0]
            if not hasattr(self.__class__, nameBase):
                raise Exception('Undeclared attribute %s for %s' % (
                    name, self.__class__.__name__))
            if hasattr(self, name) and getattr(self, name) == value:
                continue
            setattr(self, name, value)

    def getName(cls):
        if type(cls) == types.InstanceType:
            cls = cls.__class__
        if cls.name:
            name = cls.name
        else:
            name = cls.getThingName.im_func(cls)
        return name

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = things.BaseThing.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['category', 'name']
        return slotNames


class BaseKind(EmptyKind):
    balloonHelp = None
    balloonHelp_kind_isPublicForXmlRpc = 1
    balloonHelp_kind_label = N_('Balloon Help')
    balloonHelp_kind_widgetName = 'TextArea'
    balloonHelp_kindName = 'String'

    defaultValue = None
    defaultValue_kind_isProperty = 0
    defaultValue_kind_isPublicForXmlRpc = 1
    defaultValue_kind_label = N_('Default Value')
    defaultValue_kindName = None # To override

    hasToModify = 1
    hasToModify_kind_isProperty = 0
    hasToModify_kind_isPublicForXmlRpc = 1
    hasToModify_kindName = 'Boolean'

    helpAlias = None
    helpAlias_kind_isPublicForXmlRpc = 1
    helpAlias_kind_label = N_('Help Alias')
    helpAlias_kindName = 'Alias'

    isAutomaticalyModified = 0
    isAutomaticalyModified_kind_isProperty = 0
    isAutomaticalyModified_kind_isPublicForXmlRpc = 1
    isAutomaticalyModified_kindName = 'Boolean'

    isProperty = 1
    isProperty_kind_isProperty = 0
    isProperty_kind_isPublicForXmlRpc = 1
    isProperty_kindName = 'Boolean'

    isPublicForXmlRpc = 0
    isPublicForXmlRpc_kind_isProperty = 0
    isPublicForXmlRpc_kind_isPublicForXmlRpc = 1
    isPublicForXmlRpc_kindName = 'Boolean'

    isReadOnly = 0
    isReadOnly_kind_isPublicForXmlRpc = 1
    isReadOnly_kind_label = N_('Read-Only')
    isReadOnly_kindName = 'Boolean'

    isRequired = 0
    isRequired_kind_isPublicForXmlRpc = 1
    isRequired_kind_label = N_('Mandatory')
##     isRequired_kind_labels = {
##         '0': N_('Optional'),
##         '1': N_('Required'),
##         }
    isRequired_kindName = 'Boolean'

    isTranslatable = 0
    isTranslatable_kind_isProperty = 0
    isTranslatable_kind_isPublicForXmlRpc = 1
    isTranslatable_kind_label = N_('Translatable')
##     isTranslatable_kind_labels = {
##         '0': N_('Invariant'),
##         '1': N_('Translatable'),
##         }
    isTranslatable_kindName = 'Boolean'

    label = None
    label_kind_isPublicForXmlRpc = 1
    label_kind_label = N_('Label')
    label_kindName = 'String'

    labelPlural = None
    labelPlural_kind_isProperty = 0
    labelPlural_kind_isPublicForXmlRpc = 1
    labelPlural_kind_label = N_('Label (plural form)')
    labelPlural_kindName = 'String'

    possibleWidgetNames = None
    possibleWidgetNames_kind_isProperty = 0
    possibleWidgetNames_kind_isPublicForXmlRpc = 1
    possibleWidgetNames_kind_itemKind_valueName = 'WidgetName'
    possibleWidgetNames_kindName = 'Sequence'

    useCustomStorage = 0
    useCustomStorage_kind_isProperty = 0
    useCustomStorage_kind_isPublicForXmlRpc = 1
    useCustomStorage_kindName = 'Boolean'

    useFileStorage = 0
    useFileStorage_kind_isPublicForXmlRpc = 1
    useFileStorage_kind_label = N_('Storage')
    useFileStorage_kind_labels = {
        '0': N_('Internal'),
        '1': N_('In File'),
         }
    useFileStorage_kindName = 'Boolean'

    widget = None
    widget_kind_isProperty = 0
    widget_kind_isPublicForXmlRpc = 1
    widget_kind_valueThingNameSlotName = 'widgetName'
    widget_kindName = 'Widget'

    widgetName = 'InputText'
    widgetName_kind_isPublicForXmlRpc = 1
    widgetName_kind_label = N_('Graphical Component')
    widgetName_kind_valuesGetterName = 'getWidgetNameValues'
    widgetName_kindName = 'WidgetName'

    def acquireNonCoreValue(self, slot, objectDirectoryPath):
        pass

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        return value

    def exportValueToXmlRpc(self, slot, value):
        if type(value) in (types.StringType, types.UnicodeType):
            value = utf8(value)
        return value

    def getBalloonHelp(self):
        return self.balloonHelp
    
    def getCmpFunction(self):
        return cmp

    def getImplementationPropertyNames(self, parentSlot = None):
        allNames = [
            slotName
            for slotName in self.getSlotNames(parentSlot = parentSlot)
            if self.getSlot(
                slotName, parentSlot = parentSlot).getKind().isProperty]
        orderedSlotNames = self.getOrderedLayoutSlotNames(
            parentSlot = parentSlot)
        names = []
        for name in orderedSlotNames:
            if name in allNames and name not in names:
                names.append(name)
        for name in allNames:
            if name not in names:
                names.append(name)
        return names

    def getImplementationPropertyValueKind(self, propertyName,
                                           parentSlot = None):
        slot = self.getSlot(propertyName, parentSlot = parentSlot)
        return slot.getKind()

    def getModelLabel(self, slot):
        if self.label:
            return self.label
        return slot.getLabel()

    def getModelLabelPlural(self, slot):
        if self.labelPlural:
            return self.labelPlural
        return self.getModelLabel(slot)

    def getModelSlotByPath(self, slot, path):
        if path.startswith('self'):
            path = path[len('self'):]
        if not path:
            return slot
        raise Exception('Path "%s" not found' % path)

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = EmptyKind.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += [
            'label', 'labelPlural', 'widgetName', 'widget','isRequired',
            'isReadOnly', 'defaultValue', 'balloonHelp', 'helpAlias',
            'useFileStorage', 'useCommonStorage']
        return slotNames

    def getPossibleWidgetNames(self):
        if self.possibleWidgetNames is not None:
            return self.possibleWidgetNames
        return [self.widgetName]

    def getPropertySlot(self, propertyName, parentSlot = None):
        propertiesSlot = self.getSlot('properties', parentSlot = parentSlot)
        return slots.Property(propertyName, parent = propertiesSlot)

    def getPropertyValue(self, propertyName, propertyKind, parentSlot = None):
        try:
            return getattr(self, propertyName)
        except AttributeError:
            return None

    def getPropertyValueSlot(self, propertyName, parentSlot = None):
        return self.getSlot(propertyName, parentSlot = parentSlot)

    def getWidgetNameValues(self, widgetNameSlot, fields):
        kindSlot = widgetNameSlot.parent
        return kindSlot.getValue().getPossibleWidgetNames()

    def importValueFromXmlRpc(self, slot, value):
        if type(value) in (types.StringType, types.UnicodeType):
            value = iso8859_15(value)
            value = value.replace('\r\n', '\n') # MS-DOS return character
            value = value.replace('\r', '\n') # MacOS return character
        return value

    def isInCore(self, slot):
        return not self.useCustomStorage and not self.useFileStorage

    def releaseNonCoreValue(self, slot):
        pass

    def removeValueIds(self, slot, value, rolesToKeep):
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 5004:
            if type(value) in (types.StringType, types.UnicodeType):
                repairedValue = value.replace('\r\n', '\n')
                repairedValue = repairedValue.replace('\r', '\n')
                if repairedValue != value:
                    return repairedValue
        return None

    def saveNonCoreValue(self, slot, objectDirectoryPath):
        pass
register(BaseKind)


class AbstractSequence(BaseKind):
    widgetName = 'Multi'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        for i in range(len(value)):
            itemSlot = self.getItemSlot(slot, i)
            newItem = itemSlot.getKind().convertValueIds(
                itemSlot, value[i], sourceDispatcherId,
                destinationDispatcherId)
            if newItem != value[i]:
                value[i] = newItem
        return value

    def exportValueToXmlRpc(self, slot, value):
        if value is None:
            return []
        exportedValue = []
        for i in range(len(value)):
            itemSlot = self.getItemSlot(slot, i)
            exportedValue.append(itemSlot.getKind().exportValueToXmlRpc(
                itemSlot, value[i]))
        return exportedValue

    def getItemKind(self, index):
        raise NotImplementedError

    def getItemSlot(self, slot, index):
        return slots.Item(index, parent = slot)

    def getModelSlotByPath(self, slot, path):
        if path.startswith('self'):
            path = path[len('self'):]
        if not path:
            return slot
        if path[0] != '[' or not ']' in path:
            raise Exception('Path "%s" not found' % path)
        index = int(path[1:path.index(']')])
        return self.getItemKind(index).getModelSlotByPath(
            self.getItemSlot(slot, index), path[path.index(']') + 1:])

    def importValueFromXmlRpc(self, slot, importedValue):
        if importedValue is None:
            return []
        value = []
        slot.value = value
        for i in range(len(importedValue)):
            itemSlot = self.getItemSlot(slot, i)
            value.append(itemSlot.getKind().importValueFromXmlRpc(
                itemSlot, importedValue[i]))
        del slot.value
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        indexesToDelete = []
        for i in range(len(value)):
            itemSlot = self.getItemSlot(slot, i)
            newItem = itemSlot.getKind().removeValueIds(
                itemSlot, value[i], rolesToKeep)
            if newItem != value[i]:
                indexesToDelete.append(i)
        indexesToDelete.reverse()
        for i in indexesToDelete:
            del value[i]
        if not value:
            return None
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            changed = 0
            for i in range(len(value)):
                itemSlot = self.getItemSlot(slot, i)
                newItem = itemSlot.getKind().repairValue(
                    itemSlot, value[i], toVersion)
                if newItem:
                    changed = 1
                    value[i] = newItem
            if changed:
                return value
        return None


class Alias(BaseKind):
    category = N_('Glasnost')

    defaultValue_kindName = 'Alias'

    name = N_('Alias')

register(Alias)


class Amount(BaseKind):
##     category = N_('Numbers')

    defaultValue_kindName = 'Amount'

    name = N_('Amount')
register(Amount)


class Boolean(BaseKind):
    category = N_('Numbers')

    defaultValue_kindName = 'Boolean'

    labels = {
        '0': N_('False'),
        '1': N_('True'),
        }
    labels_kind_isPublicForXmlRpc = 1
    labels_kind_keyKind_value_isReadOnly = 1
    labels_kind_keyKind_valueName = 'String'
    labels_kind_label = N_('Labels')
    labels_kind_valueKind_valueName = 'String'
    labels_kindName = 'Mapping'

    name = N_('Boolean')

    possibleWidgetNames = ['InputCheckBox', 'Select']

    titles = None
    titles_kind_isPublicForXmlRpc = 1
    titles_kind_keyKind_value_isReadOnly = 1
    titles_kind_keyKind_valueName = 'String'
    titles_kind_label = N_('Titles')
    titles_kind_valueKind_valueName = 'String'
    titles_kindName = 'Mapping'

    values = [0, 1]
    values_kind_itemKind_valueName = 'Boolean'
    values_kindName = 'Sequence'

    widgetName = 'InputCheckBox'

    def getGroupedValues(self, slot, fields):
        return (None, None) # groupNames, groupedValues

    def getLabels(self, slot, fields):
        return self.labels

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = BaseKind.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['values', 'labels', 'titles']
        return slotNames
register(Boolean)


class Choice(BaseKind):
    category = N_('Choice')

    defaultValue_kindName = 'Choice'

    labels = None
    labels_kind_isPublicForXmlRpc = 1
    labels_kind_keyKind_valueName = 'String'
    labels_kind_label = N_('Labels')
    labels_kind_valueKind_valueName = 'String'
    labels_kindName = 'Mapping'

    name = N_('Choice')

    titles = None
    titles_kind_isPublicForXmlRpc = 1
    titles_kind_keyKind_value_isReadOnly = 1
    titles_kind_keyKind_valueName = 'String'
    titles_kind_label = N_('Titles')
    titles_kind_valueKind_valueName = 'String'
    titles_kindName = 'Mapping'

    values = None
    values_kind_itemKind_valueName = 'String'
    values_kindName = 'Sequence'

    valuesGetterName = None
    valuesGetterName_kind_isPublicForXmlRpc = 1
    valuesGetterName_kindName = 'PythonIdentifier'

    widgetName = 'Select'

    def getGroupedValues(self, slot, fields):
        return (None, None) # groupNames, groupedValues

    def getLabels(self, slot, fields):
        if self.labels:
            return self.labels
        labels = {}
        values = self.getValues(slot, fields)
        if values is not None:
            for value in values:
                labels[str(value)] = str(value)
        return labels

    def getValues(self, slot, fields):
        valuesGetterName = self.valuesGetterName
        if valuesGetterName is not None:
            currentSlot = slot
            while currentSlot is not None:
                container = currentSlot.getContainer()
                try:
                    valuesGetter = getattr(container, valuesGetterName)
                except AttributeError:
                    currentSlot = currentSlot.parent
                    continue
                return valuesGetter(slot, fields)
            return []
        if self.values:
            return self.values
        else:
            return []
register(Choice)


class Data(BaseKind):
    category = N_('Compound')

    defaultValue_kindName = 'Data'

    def exportValueToXmlRpc(self, slot, value):
        if value:
            value = base64.encodestring(value)
        return value

    def importValueFromXmlRpc(self, slot, value):
        if value:
            value = base64.decodestring(value)
        return value
register(Data)


class Date(BaseKind):
##     category = N_('Date & Time')

    defaultValue_kindName = 'Date'

    requiredParts = ['year', 'month', 'day']
    requiredParts_kind_isPublicForXmlRpc = 1
    requiredParts_kind_itemKind_valueName = 'String'
    requiredParts_kindName = 'Sequence'

    name = N_('Date')
register(Date)


class Duration(BaseKind):
    category = N_('Time')

    defaultValue_kindName = 'Duration'

    requiredParts = ['day', 'hour', 'minute', 'second']
    requiredParts_kind_isPublicForXmlRpc = 1
    requiredParts_kind_itemKind_valueName = 'String'
    requiredParts_kindName = 'Sequence'

    name = N_('Duration')
    widgetName = 'Duration'
register(Duration)


class Email(BaseKind):
    category = N_('Text')

    defaultValue_kindName = 'Email'

    name = N_('Email Address')
    widgetName = 'Email'
register(Email)


class Path(BaseKind):
    category = N_('Text')

    checkFilePath = 1
    checkFilePath_kind_isPublicForXmlRpc = 1
    checkFilePath_kindName = 'Boolean'

    defaultValue_kindName = 'Path'

    name = N_('Path')
register(Path)


class FilePath(Path):
    category = N_('Text')

    defaultValue_kindName = 'FilePath'

    name = N_('File Path')
register(FilePath)


class Fingerprint(BaseKind):
    category = N_('Text')

    defaultValue_kindName = 'Fingerprint'
register(Fingerprint)


class Float(BaseKind):
    category = N_('Numbers')

    defaultValue_kindName = 'Float'

    name = N_('Float Number')
register(Float)


class FunctionName(Choice):
    category = N_('Glasnost')

    defaultValue_kindName = 'FunctionName'

    name = N_('Function Name')

    serverIdSlotName = None
    serverIdSlotName_kind_isPublicForXmlRpc = 1
    serverIdSlotName_kindName = 'String'

    def getValues(self, slot, fields):
        if not self.serverIdSlotName:
            return []
        serverId = slot.getContainer().getSlot(
            self.serverIdSlotName, parentSlot = slot.parent).getField(fields)
        if not serverId:
            return []
        try:
            serverRole = commonTools.extractRole(serverId)
        except:
            return []
        from glasnost.proxy.tools import getProxyForServerRole
        try:
            proxy = getProxyForServerRole(serverRole)
        except:
            return []
        if proxy is None:
            return []
        try:
            functionDeclarations = proxy.getFunctionDeclarations() 
        except:
            return []
        return functionDeclarations.keys()
register(FunctionName)


class Id(BaseKind):
    category = N_('Glasnost')

    defaultValue_kindName = 'Id'

    name = N_('Object ID')

    serverRoles = None
    serverRoles_kind_isPublicForXmlRpc = 1
    serverRoles_kind_itemKind_valueName = 'ServerRole'
    serverRoles_kindName = 'Sequence'

    widgetName = 'SelectId'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        return value.replace(sourceDispatcherId, destinationDispatcherId)

    def getServerRoles(self):
        return self.serverRoles

    def removeValueIds(self, slot, value, rolesToKeep):
        if not commonTools.extractRole(value) in rolesToKeep:
            return None
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            return repairId(value)
        return None
register(Id)


class Integer(BaseKind):
    category = N_('Numbers')

    defaultValue_kindName = 'Integer'

    max = None
    max_kind_isPublicForXmlRpc = 1
    max_kind_label = N_('Maximum')
    max_kindName = 'Integer'

    min = None
    min_kind_isPublicForXmlRpc = 1
    min_kind_label = N_('Minimum')
    min_kindName = 'Integer'

    name = N_('Integer Number')

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = BaseKind.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames = slotNames[:]
        i = slotNames.index('defaultValue')
        slotNames[i:i] = ['min', 'max']
        return slotNames
register(Integer)


class IntegerChoice(Choice):
    defaultValue_kindName = 'IntegerChoice'

    name = N_('Integer Choice')

    titles_kind_keyKind_valueName = 'Integer'

    values_kind_itemKind_valueName = 'Integer'

    def getLabels(self, slot, fields):
        if self.labels:
            return self.labels
        labels = {}
        values = self.getValues(slot, fields)
        if values is not None:
            for value in values:
                labels[str(value)] = str(value)
        return labels
register(IntegerChoice)


class LanguageChoice(Choice):
    category = N_('Choice')

    defaultValue_kindName = 'LanguageChoice'

    name = N_('Language')

    values = translation.languageKeys
register(LanguageChoice)


class Link(BaseKind):
##     category = N_('Compound')

    defaultValue_kindName = 'Link'

    name = N_('Link')

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        name, url = value
        if url.startswith('id:'):
            url = url.replace(sourceDispatcherId, destinationDispatcherId)
        return (name, url)

    def removeValueIds(self, slot, value, rolesToKeep):
        name, url = value
        if url.startswith('id:') \
               and not commonTools.extractRole(url[3:]) in rolesToKeep:
            return None
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            name, url = value
            if url.startswith('id:'):
                newId = repairId(url[3:])
                if newId:
                    return (name, 'id:' + newId)
        return None
register(Link)


class Mapping(BaseKind):
    category = N_('Compound')

    defaultValue_kindName = 'Mapping'

    deleteNullValues = 0
    deleteNullValues_kind_isPublicForXmlRpc = 1
    deleteNullValues_kindName = 'Boolean'
    
    keyKind = None
    keyKind_kind_isPublicForXmlRpc = 1
    keyKind_kind_isRequired = 1
    keyKind_kindName = 'Kind'

    name = N_('Mapping')

    requiredCount = 0
    requiredCount_kind_isPublicForXmlRpc = 1
    requiredCount_kindName = 'Integer'

    valueKind = None
    valueKind_kind_isPublicForXmlRpc = 1
    valueKind_kind_isRequired = 1
    valueKind_kindName = 'Kind'

    widgetName = 'Mapping'

    def buildKinds(self):
        # Should be converted to a class method, when porting Glasnost to
        # Python 2.2.
        BaseKind.buildKinds(self)

        if self.keyKind_valueName is not None:
            baseClasses = self.getC3ClassLinearization()
            keyKindOptionNames = [key
                                  for key in self.__dict__.keys()
                                  if key.startswith('keyKind_value_')]
            for baseClass in baseClasses:
                for key in baseClass.__dict__.keys():
                    if key.startswith('keyKind_value_') \
                       and not key in keyKindOptionNames:
                        keyKindOptionNames.append(key)
            keyKindOptions = {}
            for keyKindOptionName in keyKindOptionNames:
                keyKindOptions[keyKindOptionName[len('keyKind_value_'):]
                    ] = getattr(self, keyKindOptionName)
            keyKind = self.newThing('kind', self.keyKind_valueName)
            keyKind.buildOptions(keyKindOptions)
            self.__class__.keyKind = keyKind

        if self.valueKind_valueName is not None:
            baseClasses = self.getC3ClassLinearization()
            valueKindOptionNames = [key
                                    for key in self.__dict__.keys()
                                    if key.startswith('valueKind_value_')]
            for baseClass in baseClasses:
                for key in baseClass.__dict__.keys():
                    if key.startswith('valueKind_value_') \
                       and not key in valueKindOptionNames:
                        valueKindOptionNames.append(key)
            valueKindOptions = {}
            for valueKindOptionName in valueKindOptionNames:
                valueKindOptions[valueKindOptionName[len('valueKind_value_'):]
                    ] = getattr(self, valueKindOptionName)
            valueKind = self.newThing('kind', self.valueKind_valueName)
            valueKind.buildOptions(valueKindOptions)
            self.__class__.valueKind = valueKind

    def buildOptions(self, options):
        baseClasses = self.getC3ClassLinearization()
        keyKindOptionNames = [key
                              for key in self.__dict__.keys()
                              if key.startswith('keyKind_value_')]
        for baseClass in baseClasses:
            for key in baseClass.__dict__.keys():
                if key.startswith('keyKind_value_') \
                   and not key in keyKindOptionNames:
                    keyKindOptionNames.append(key)
        keyKindOptions = {}
        for keyKindOptionName in keyKindOptionNames:
            keyKindOptions[keyKindOptionName[len('keyKind_value_'):]
                ] = getattr(self, keyKindOptionName)
        keyKindOptionNames = [key
                              for key in options.keys()
                              if key.startswith('keyKind_value_')]
        for keyKindOptionName in keyKindOptionNames:
            keyKindOptions[keyKindOptionName[len('keyKind_value_'):]
                ] = options[keyKindOptionName]
            del options[keyKindOptionName]
        if options.has_key('keyKind_valueName'):
            keyKindName = options['keyKind_valueName']
            del options['keyKind_valueName']
        else:
            keyKindName = self.keyKind_valueName

        valueKindOptionNames = [key
                                for key in self.__dict__.keys()
                                if key.startswith('valueKind_value_')]
        for baseClass in baseClasses:
            for key in baseClass.__dict__.keys():
                if key.startswith('valueKind_value_') \
                   and not key in valueKindOptionNames:
                    valueKindOptionNames.append(key)
        valueKindOptions = {}
        for valueKindOptionName in valueKindOptionNames:
            valueKindOptions[valueKindOptionName[len('valueKind_value_'):]
                ] = getattr(self, valueKindOptionName)
        valueKindOptionNames = [key
                                for key in options.keys()
                                if key.startswith('valueKind_value_')]
        for valueKindOptionName in valueKindOptionNames:
            valueKindOptions[valueKindOptionName[len('valueKind_value_'):]
                ] = options[valueKindOptionName]
            del options[valueKindOptionName]
        if options.has_key('valueKind_valueName'):
            valueKindName = options['valueKind_valueName']
            del options['valueKind_valueName']
        else:
            valueKindName = self.valueKind_valueName

        BaseKind.buildOptions(self, options)

        if keyKindName is not None:
            keyKind = self.newThing('kind', keyKindName)
            keyKind.buildOptions(keyKindOptions)
            self.keyKind = keyKind

        if valueKindName is not None:
            valueKind = self.newThing('kind', valueKindName)
            valueKind.buildOptions(valueKindOptions)
            self.valueKind = valueKind

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        keyIndex = 0
        for key, val in value.items():
            keySlot = self.getItemKeySlot(slot, keyIndex)
            newKey = keySlot.getKind().convertValueIds(
                keySlot, key, sourceDispatcherId, destinationDispatcherId)
            if newKey != key:
                del value[key]
                value[newKey] = val
            keyIndex += 1
        for key, val in value.items():
            valueSlot = self.getItemValueSlot(slot, key)
            newVal = valueSlot.getKind().convertValueIds(
                valueSlot, val, sourceDispatcherId, destinationDispatcherId)
            if newVal != val:
                value[key] = newVal
        return value

    def exportValueToXmlRpc(self, slot, value):
        if value is None:
            return {}
        exportedValue = {}
        keyIndex = 0
        for key, itemValue in value.items():
            keySlot = self.getItemKeySlot(slot, keyIndex)
            exportedKey = keySlot.getKind().exportValueToXmlRpc(keySlot, key)
            valueSlot = self.getItemValueSlot(slot, key)
            exportedItemValue = valueSlot.getKind().exportValueToXmlRpc(
                valueSlot, itemValue)
            exportedValue[exportedKey] = exportedItemValue
            keyIndex += 1
        return exportedValue

    def getItemKeyKind(self, index):
        return self.keyKind

    def getItemKeySlot(self, slot, index):
        return slots.ItemKey(index, parent = slot)

    def getItemKeyValueKind(self, key):
        return self.valueKind

    def getItemKeyValueSlot(self, slot):
        return slots.ItemKeyValue(parent = slot)

    def getItemValueKind(self, key):
        return self.valueKind

    def getItemValueSlot(self, slot, key):
        return slots.ItemValue(key, parent = slot)

    def getModelSlotByPath(self, slot, path):
        if path.startswith('self'):
            path = path[len('self'):]
        if not path:
            return slot
        if path.startswith('.keys()[') and ']' in path:
            index = int(path[len('.keys()['):path.index(']')])
            return self.getItemKeyKind(index).getModelSlotByPath(
                self.getItemKeySlot(slot, index), path[path.index(']') + 1:])
        if path[0] != '[' or not ']' in path:
            raise Exception('Path "%s" not found' % path)
        key = eval(path[1:path.index(']')], {})
        return self.getItemValueKind(key).getModelSlotByPath(
            self.getItemValueSlot(slot, key), path[path.index(']') + 1:])

    def importValueFromXmlRpc(self, slot, importedValue):
        if importedValue is None:
            return {}
        value = {}
        slot.value = value
        keyIndex = 0
        for importedKey, importedItemValue in importedValue.items():
            keySlot = self.getItemKeySlot(slot, keyIndex)
            key = keySlot.getKind().importValueFromXmlRpc(keySlot, importedKey)
            valueSlot = self.getItemValueSlot(slot, key)
            itemValue = valueSlot.getKind().importValueFromXmlRpc(
                valueSlot, importedItemValue)
            value[key] = itemValue
            keyIndex += 1
        del slot.value
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        keyIndex = 0
        for key, val in value.items():
            keySlot = self.getItemKeySlot(slot, keyIndex)
            newKey = keySlot.getKind().removeValueIds(
                keySlot, key, rolesToKeep)
            if newKey != key:
                del value[key]
            keyIndex += 1
        for key, val in value.items():
            valueSlot = self.getItemValueSlot(slot, key)
            newVal = valueSlot.getKind().removeValueIds(
                valueSlot, val, rolesToKeep)
            if newVal != val:
                del value[key]
        if not value:
            return None
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            changed = 0
            keyIndex = 0
            for key, val in value.items():
                keySlot = self.getItemKeySlot(slot, keyIndex)
                newKey = keySlot.getKind().repairValue(keySlot, key, toVersion)
                if newKey:
                    changed = 1
                    del value[key]
                    value[newKey] = val
                keyIndex += 1
            for key, val in value.items():
                valueSlot = self.getItemValueSlot(slot, key)
                newVal = valueSlot.getKind().repairValue(
                    valueSlot, val, toVersion)
                if newVal:
                    changed = 1
                    value[key] = newVal
            if changed:
                return value
        return None
register(Mapping)


class Marks(BaseKind):
##     category = N_('Other')

    defaultValue_kindName = 'Marks'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        for id, mark in value.items():
            newId = id.replace(sourceDispatcherId, destinationDispatcherId)
            if newId != id:
                del value[id]
                value[newId] = mark
        return value

    def exportValueToXmlRpc(self, slot, value):
        exportedValue = {}
        if value is not None:
            for objectId, mark in value.items():
                if mark is None:
                    exportedMark = -1
                else:
                    exportedMark = mark
                exportedValue[objectId] = exportedMark
        return exportedValue

    def importValueFromXmlRpc(self, slot, importedValue):
        value = {}
        slot.value = value
        for objectId, importedMark in importedValue.items():
            if importedMark == -1:
                mark = None
            else:
                mark = importedMark
            value[objectId] = mark
        if not value:
            value = None
        del slot.value
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        for id in value.keys():
            if not commonTools.extractRole(id) in rolesToKeep:
                del value[id]
        if not value:
            return None
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            changed = 0
            for id, mark in value.items():
                newId = repairId(id)
                if newId:
                    changed = 1
                    del value[id]
                    value[newId] = mark
            if changed:
                return value
        return None
register(Marks)


class Memory(BaseKind):
##     category = N_('Other')

    defaultValue_kindName = 'Memory'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        for serverId, serverMemory in value.items():
            newServerId = serverId.replace(
                sourceDispatcherId, destinationDispatcherId)
            if newServerId != serverId:
                del value[serverId]
                value[newServerId] = serverMemory
            for i in range(len(serverMemory)):
                itemId = serverMemory[i]
                newItemId = itemId.replace(
                    sourceDispatcherId, destinationDispatcherId)
                if newItemId != itemId:
                    serverMemory[i] = newItemId
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        for serverId, serverMemory in value.items():
            if not commonTools.extractRole(serverId) in rolesToKeep:
                del value[serverId]
                continue
            indexesToDelete = []
            for i in range(len(serverMemory)):
                itemId = serverMemory[i]
                if not commonTools.extractRole(itemId) in rolesToKeep:
                    indexesToDelete.append(i)
            indexesToDelete.reverse()
            for i in indexesToDelete:
                del serverMemory[i]
            if not serverMemory:
                del value[serverId]
        if not value:
            return None
        return value
register(Memory)


class Mutation(properties.PropertiesContainerMixin, EmptyKind):
    category = N_('Glasnost')

##     defaultValue_kindName = 'Mutation'

    isEffective = 0

    name = N_('Mutation')
register(Mutation)


class PairwiseMatrix(BaseKind):
##     category = N_('Other')

    defaultValue_kindName = 'PairwiseMatrix'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        for id1, row in value.items():
            newId1 = id1.replace(sourceDispatcherId, destinationDispatcherId)
            if newId1 != id1:
                del value[id1]
                value[newId1] = row
            for id2, score in row.items():
                newId2 = id2.replace(
                    sourceDispatcherId, destinationDispatcherId)
                if newId2 != id2:
                    del row[id2]
                    row[newId2] = score
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        for id1, row in value.items():
            if not commonTools.extractRole(id1) in rolesToKeep:
                del value[id1]
                continue
            for id2, score in row.items():
                if not commonTools.extractRole(id2) in rolesToKeep:
                    del row[id2]
            if not row:
                del value[id1]
        if not value:
            return None
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            changed = 0
            for id1, row in value.items():
                newId1 = repairId(id1)
                if newId1:
                    changed = 1
                    del value[id1]
                    value[newId1] = row
                for id2, score in row.items():
                    newId2 = repairId(id2)
                    if newId2:
                        changed = 1
                        del row[id2]
                        row[newId2] = score
            if changed:
                return value
        return None
register(PairwiseMatrix)


class Password(BaseKind):
    category = N_('Text')

    defaultValue_kindName = 'Password'

    name = N_('Password')
register(Password)


class PythonIdentifier(BaseKind):
    category = N_('Text')

    defaultValue_kindName = 'PythonIdentifier'

    name = N_('Python Identifier')
register(PythonIdentifier)


class Rating(BaseKind):
##     category = N_('Other')

    defaultValue_kindName = 'Rating'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        for k in ['wins', 'losses', 'ties']:
            v = value[k]
            for i in range(len(v)):
                id = v[i]
                newId = id.replace(sourceDispatcherId, destinationDispatcherId)
                if newId != id:
                    v[i] = newId
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        for k in ['wins', 'losses', 'ties']:
            v = value[k]
            r = range(len(v))
            r.reverse()
            for i in r:
                id = v[i]
                if not commonTools.extractRole(id) in rolesToKeep:
                    del v[i]
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            changed = 0
            for k in ['wins', 'losses', 'ties']:
                v = value[k]
                for i in range(len(v)):
                    id = v[i]
                    newId = repairId(id)
                    if newId:
                        changed = 1
                        v[i] = newId
            if changed:
                return value
        return None
register(Rating)


class Sequence(AbstractSequence):
    category = N_('Compound')

    defaultValue_kindName = 'Sequence'

    itemKind = None
    itemKind_kind_isPublicForXmlRpc = 1
    itemKind_kind_isRequired = 1
    itemKind_kindName = 'Kind'
    itemKind_valueName = None

    name = N_('Sequence')

    requiredCount = 0
    requiredCount_kind_isPublicForXmlRpc = 1
    requiredCount_kindName = 'Integer'

    def buildKinds(self):
        # Should be converted to a class method, when porting Glasnost to
        # Python 2.2.
        AbstractSequence.buildKinds(self)

        if self.itemKind_valueName is not None:
            baseClasses = self.getC3ClassLinearization()
            itemKindOptionNames = [key
                                   for key in self.__dict__.keys()
                                   if key.startswith('itemKind_value_')]
            for baseClass in baseClasses:
                for key in baseClass.__dict__.keys():
                    if key.startswith('itemKind_value_') \
                       and not key in itemKindOptionNames:
                        itemKindOptionNames.append(key)
            itemKindOptions = {}
            for itemKindOptionName in itemKindOptionNames:
                itemKindOptions[itemKindOptionName[len('itemKind_value_'):]
                    ] = getattr(self, itemKindOptionName)
            itemKind = self.newThing('kind', self.itemKind_valueName)
            itemKind.buildOptions(itemKindOptions)
            self.__class__.itemKind = itemKind

    def buildOptions(self, options):
        baseClasses = self.getC3ClassLinearization()
        itemKindOptionNames = [key
                               for key in self.__dict__.keys()
                               if key.startswith('itemKind_value_')]
        for baseClass in baseClasses:
            for key in baseClass.__dict__.keys():
                if key.startswith('itemKind_value_') \
                   and not key in itemKindOptionNames:
                    itemKindOptionNames.append(key)
        itemKindOptions = {}
        for itemKindOptionName in itemKindOptionNames:
            itemKindOptions[itemKindOptionName[len('itemKind_value_'):]
                ] = getattr(self, itemKindOptionName)
        itemKindOptionNames = [key
                               for key in options.keys()
                               if key.startswith('itemKind_value_')]
        for itemKindOptionName in itemKindOptionNames:
            itemKindOptions[itemKindOptionName[len('itemKind_value_'):]
                ] = options[itemKindOptionName]
            del options[itemKindOptionName]
        if options.has_key('itemKind_valueName'):
            itemKindName = options['itemKind_valueName']
            del options['itemKind_valueName']
        else:
            itemKindName = self.itemKind_valueName

        AbstractSequence.buildOptions(self, options)

        if itemKindName is not None:
            itemKind = self.newThing('kind', itemKindName)
            itemKind.buildOptions(itemKindOptions)
            self.itemKind = itemKind

    def getItemKind(self, index):
        return self.itemKind

    def getModelLabel(self, slot):
        if self.label:
            return self.label
        return self.itemKind.getModelLabelPlural(self.getItemSlot(slot, 0))

    def getModelLabelPlural(self, slot):
        if self.labelPlural:
            return self.labelPlural
        return self.itemKind.getModelLabelPlural(self.getItemSlot(slot, 0))
register(Sequence)


class AuthorsSet(Sequence):
    balloonHelp = N_('Choose the author(s) for this object.')
    defaultValue_kindName = 'AuthorsSet'

    itemKind_value_serverRoles = ['groups', 'people']
    itemKind_valueName = 'Id'

    label = N_('Authors')

    name = N_('Authors')

    requiredCount = 1
register(AuthorsSet)


class Properties(Sequence):
    itemKind_value_valueThingCategory = 'other'
    itemKind_value_valueThingName = 'Property'
    itemKind_valueName = 'Thing'

    label = N_('Properties')

    name = N_('Properties')

    def exportValueToXmlRpc(self, slot, value):
        if value is None:
            return []
        exportedValue = []
        for i in range(len(value)):
            property = value[i]
            propertySlot = self.getPropertySlot(slot, property.name)
            exportedProperty = propertySlot.getKind().exportValueToXmlRpc(
                propertySlot, property)
            exportedValue.append(exportedProperty)
        return exportedValue

    def getItemSlot(self, slot, index):
        return slots.PropertiesItem(index, parent = slot)

    def getModelSlotByPath(self, slot, path):
        if path.startswith('self'):
            path = path[len('self'):]
        if not path:
            return slot
        if path[0] != '[' or not ']' in path:
            raise Exception('Path "%s" not found' % path)
        key = eval(path[1:path.index(']')], {})
        if type(key) in (types.StringType, types.UnicodeType):
            # Key is a property name.
            # FIXME: We should raise Exception('Path "%s" not found' % path),
            # when there is no property named key.
            kind = slot.getContainer().getImplementationPropertyValueKind(
                key, parentSlot = slot.parent)
            if kind is None:
                kind = self.newThing('kind', 'Mutation')
            property = self.newThing('other', 'Property')
            property.kind = kind
            property.kindName = kind.getThingName()
            property.name = key
            return property.getSlotByPath(
                path[path.index(']') + 1:],
                parentSlot = slot.getContainer().getPropertySlot(
                    key, parentSlot = slot.parent))
        else:
            # Key is an integer index.
            return self.getItemKind(key).getModelSlotByPath(
                self.getItemSlot(slot, key), path[path.index(']') + 1:])

    def getPropertySlot(self, slot, name):
        return slots.Property(name, parent = slot)

    def importValueFromXmlRpc(self, slot, importedValue):
        if importedValue is None:
            return []
        value = []
        slot.value = value
        for i in range(len(importedValue)):
            importedProperty = importedValue[i]
            propertyName = importedProperty['name']
            propertySlot = self.getPropertySlot(slot, propertyName)
            value.append(propertySlot.getKind().importValueFromXmlRpc(
                propertySlot, importedProperty))
        del slot.value
        return value
register(Properties)


class ReadersSet(Sequence):
    balloonHelp = N_(
        'Select the people and groups who are allowed to read the item.')

    defaultValue_kindName = 'ReadersSet'

    itemKind_value_serverRoles = ['groups', 'people']
    itemKind_valueName = 'Id'

    label = N_('Readers')

    name = N_('Readers')

    requiredCount = 0
register(ReadersSet)


class ServerId(Id):
    category = N_('Glasnost')

    defaultValue_kindName = 'ServerId'

    name = N_('Server ID')

##     widgetName = 'SelectServerId'
    widget_size = 40
    widgetName = 'InputText'
register(ServerId)


class ServerRole(Choice):
    category = N_('Glasnost')

    defaultValue_kindName = 'ServerRole'

    name = N_('Server Role')

    def getValues(self, slot, fields):
        import glasnost.proxy.tools_new as proxyTools
        return proxyTools.getAllServerRoles()
register(ServerRole)


class SlotName(Choice):
    category = N_('Glasnost')

    defaultValue_kindName = 'SlotName'

    name = N_('Slot Name')
register(SlotName)


class Source(BaseKind):
    category = N_('Glasnost')

    defaultValue_kindName = 'Source'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        return value.replace(sourceDispatcherId, destinationDispatcherId)

    def removeValueIds(self, slot, value, rolesToKeep):
        if not commonTools.extractRole(value) in rolesToKeep:
            return None
        return value
register(Source)


class String(BaseKind):
    category = N_('Text')

    defaultValue_kindName = 'String'

    # pour les cls des Mapping
    defaultValues = None
    defaultValues_kind_isPublicForXmlRpc = 0
    defaultValues_kind_itemKind_value_kindName = 'String'
    defaultValues_kind_label = N_('Default Values')
    defaultValues_kindName = 'Sequence'

    isTranslatable = 1

    name = N_('String')

    possibleWidgetNames = ['InputText', 'TextArea']

    def getCmpFunction(self):
        return locale.strcoll
register(String)


class Structure(AbstractSequence):
    category = N_('Compound')

    defaultValue_kindName = 'Structure'

    name = N_('Structure')

    parameters = None
    parameters_kind_isPublicForXmlRpc = 1
    parameters_kind_itemKind_value_valueThingCategory = 'other'
    parameters_kind_itemKind_value_valueThingName = 'Parameter'
    parameters_kind_itemKind_valueName = 'Thing'
    parameters_kindName = 'Sequence'

    def getItemKind(self, index):
        if self.parameters is None or index >= len(self.parameters):
            return None
        return self.parameters[index].kind
register(Structure)


class Thing(BaseKind):
    category = N_('Compound')

    defaultValue_kindName = 'Thing'

    name = N_('Thing')

    valueThingCategory = None
    valueThingCategory_kind_isPublicForXmlRpc = 1
    valueThingCategory_kindName = 'String'

    valueThingName = None
    valueThingName_kind_isPublicForXmlRpc = 1
    valueThingName_kindName = 'String'

    valueThingNameSlotName = None
    valueThingNameSlotName_kind_isPublicForXmlRpc = 1
    valueThingNameSlotName_kindName = 'PythonIdentifier'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        value.convertIds(
            sourceDispatcherId, destinationDispatcherId, parentSlot = slot)
        return value

    def exportValueToXmlRpc(self, slot, value):
        if value is None:
            return None
        return value.exportToXmlRpc(parentSlot = slot)

    def getModelSlotByPath(self, slot, path):
        value = slot.getValue()
        if value is None:
            value = self.newThing(
                self.valueThingCategory, self.getValueThingName(slot))
            slot = slot.detach(value)
        return value.getSlotByPath(path, parentSlot = slot)

    def getValueThingName(self, slot):
        if self.valueThingNameSlotName is not None:
            container = slot.getContainer()
            valueThingName = container.getSlot(
                self.valueThingNameSlotName, parentSlot = slot.parent
                ).getValue()
            if valueThingName is not None:
                return valueThingName
        return self.valueThingName

    def getValueThingNameFromFields(self, slot, fields):
        if self.valueThingNameSlotName is not None:
            container = slot.getContainer()
            valueThingName = container.getSlot(
                self.valueThingNameSlotName, parentSlot = slot.parent
                ).getField(fields)
            if valueThingName is not None:
                return valueThingName
        return self.valueThingName

    def importValueFromXmlRpc(self, slot, importedValue):
        if importedValue is None:
            return None
        value = self.newThing(
            importedValue['__thingCategory__'], importedValue['__thingName__'])
        slot.value = value
        value.importFromXmlRpc(importedValue, parentSlot = slot)
        del slot.value
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        return value.removeIds(rolesToKeep, parentSlot = slot)

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            changed = value.repair(toVersion, parentSlot = slot)
            if changed:
                return value
        return None
register(Thing)


class Time(BaseKind):
    category = N_('Date & Time')

    defaultValue_kindName = 'Time'
    name = N_('Time')
    widgetName = 'Time'
register(Time)


class CreationTime(Time):
##     defaultValue = 'now'
    defaultValue_kindName = 'CreationTime'

    hasToModify = 0

    isAutomaticalyModified = 1

    isReadOnly = 1

    label = N_('Creation Time')

    name = N_('Creation Time')

    def setAutomaticalValue(self, slot, init = 0):
        if init:
            slot.setValue(time.time())
register(CreationTime)


class ModificationTime(Time):
##     defaultValue = 'now'
    defaultValue_kindName = 'ModificationTime'

    hasToModify = 0

    isAutomaticalyModified = 1

    isReadOnly = 1

    label = N_('Modification Time')

    name = N_('Modification Time')

    def setAutomaticalValue(self, slot, init = 0):
        slot.setValue(time.time())
register(ModificationTime)


class Token(BaseKind):
    category = N_('Text')

    defaultValue_kindName = 'Token'

    name = N_('Token')
register(Token)


class ApplicationToken(Token):
    defaultValue_kindName = 'ApplicationToken'

    name = N_('Application Token')
register(ApplicationToken)


class UserToken(Token):
    defaultValue_kindName = 'UserToken'

    name = N_('User Token')
register(UserToken)


class TranslationToAdd(BaseKind):
    category = None

    defaultValue_kindName = 'TranslationToAdd'
register(TranslationToAdd)


class TranslatorsSets(BaseKind):
    defaultValue_kindName = 'TranslatorsSets'

    def convertValueIds(
        self, slot, value, sourceDispatcherId, destinationDispatcherId):
        for translationName, translatorsSet in value.items():
            if translatorsSet:
                for i in range(len(translatorsSet)):
                    id = translatorsSet[i]
                    newId = id.replace(
                        sourceDispatcherId, destinationDispatcherId)
                    if newId != id:
                        translatorsSet[i] = newId
        return value

    def removeValueIds(self, slot, value, rolesToKeep):
        for translationName, translatorsSet in value.items():
            if translatorsSet:
                r = range(len(translatorsSet))
                r.reverse()
                for i in r:
                    id = translatorsSet[i]
                    if not commonTools.extractRole(id) in rolesToKeep:
                        del translatorsSet[i]
                if not translatorsSet:
                    del value[translationName]
        if not value:
            return None
        return value

    def repairValue(self, slot, value, toVersion):
        if toVersion == 4000:
            changed = 0
            for translationName, translatorsSet in value.items():
                if translatorsSet:
                    for i in range(len(translatorsSet)):
                        id = translatorsSet[i]
                        newId = repairId(id)
                        if newId:
                            changed = 1
                            translatorsSet[i] = newId
            if changed:
                return value
        return None
register(TranslatorsSets)


class WritersSet(Sequence):
    balloonHelp = N_(
        'Select the people and groups who are allowed to modify the item.')

    defaultValue_kindName = 'WritersSet'

    itemKind_value_serverRoles = ['groups', 'people']
    itemKind_valueName = 'Id'

    label = N_('Writers')

    name = N_('Writers')

    requiredCount = 1
register(WritersSet)


class XChoice(Choice):
    category = N_('Choice')

    defaultValue_kindName = 'XChoice'

    groupedValues = None
    groupedValues_kind_keyKind_valueName = 'String'
    groupedValues_kind_valueKind_valueName = 'String'
    groupedValues_kindName = 'Mapping'    
    
    groupNames = None
    groupNames_kind_itemKind_valueName = 'String'
    groupNames_kindName = 'Sequence'    
    
    name = N_('Extended Choice')

    widgetName = 'XSelect'

    def getGroupedValues(self, slot, fields):
        return (self.groupNames, self.groupedValues)

register(XChoice)


class Kind(Thing):
    defaultValue_kindName = 'Kind'

    name = N_('Kind')

    valueThingCategory = 'kind'

    valueThingName = 'BaseKind'
register(Kind)


class KindName(Choice):
    defaultValue_kindName = 'KindName'

    name = N_('Kind Name')

##     def __init__(self, **attributes):
##         Choice.__init__(self, **attributes)
##         self.values = [
##             kindClass.getThingName.im_func(kindClass)
##             for kindClass in self.getAllThingClasses()['kind'].values()]

    def getGroupedValues(self, slot, fields):
        categories = []
        groupedValues = {}
        for kindClass in self.getAllThingClasses()['kind'].values():
            if kindClass.category is not None:
                if not kindClass.category in categories:
                    categories.append(kindClass.category)
                    groupedValues[kindClass.category] = []
                groupedValues[kindClass.category].append(
                    kindClass.getThingName.im_func(kindClass))
        return (categories, groupedValues) # groupNames, groupedValues

    def getValues(self, slot, fields):
        return [
            kindClass.getThingName.im_func(kindClass)
            for kindClass in self.getAllThingClasses()['kind'].values()
            if kindClass.category is not None]
register(KindName)


class Upload(Thing):
    category = N_('Compound')

    defaultValue_kindName = 'Upload'

    name = N_('Upload')

    valueThingCategory = 'other'

    valueThingName = 'Upload'
register(Upload)


class Widget(Thing):
    defaultValue_kindName = 'Widget'

    name = N_('Widget')

    valueThingCategory = 'widget'

    valueThingName = 'BaseWidget'
register(Widget)


class WidgetName(Choice):
    defaultValue_kindName = 'WidgetName'

    name = N_('Widget Name')

    def getValues(self, slot, fields):
        values = Choice.getValues(self, slot, fields)
        if values:
            return values
        else:
            return ['InputText']
register(WidgetName)
