# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Common Properties"""

__version__ = '$Revision: 1.19 $'[11:-2]


import faults
import slots
import things
import tools_new as commonTools


class PropertiesContainerMixin:
    properties = None
    properties_kind_isPublicForXmlRpc = 1
    properties_kindName = 'Properties'

    values = None

    def getDefaultPropertyValue(self, propertyName, propertyKind,
                                parentSlot = None):
        object = self.getSlotObject(parentSlot = parentSlot)
        if object is None:
            return None
        # FIXME: We should use a Python iterator for computing the C3
        # linearization, because here we use only the second prototype. 
        prototypeIds = object.getC3PrototypeLinearization()
        if len(prototypeIds) <= 1:
            return None
        prototypeId = prototypeIds[1]
        return self.getProxyForId(prototypeId).getPropertyValue(
            prototypeId, propertyName, propertyKind, self,
            parentSlot = parentSlot)

# This method has ben commented, because it doesn't work when a property name
# has changed: self.values may contain an item whose name doesn't correspond to
# a property anymore.
##     def getExportSlotNames(self, parentSlot = None):
##         slotNames = self.inherited(
##             PropertiesContainerMixin, 'getExportSlotNames')(
##             self, parentSlot = parentSlot)
##         if self.values is None:
##             return slotNames
##         # FIXME: It is not sufficient to export only the items in values:
##         # For a Mutation, the default value is not None.
##         return slotNames + self.values.keys()

    def getImplementationPropertyNames(self, parentSlot = None):
        if self.properties is None:
            return []
        return [property.name for property in self.properties]

    def getImplementationPropertyValue(self, propertyName, propertyKind,
                                       parentSlot = None):
        if self.values and self.values.has_key(propertyName):
            return self.values[propertyName]
        return None

    def getImplementationPropertyValueKind(self, propertyName,
                                           parentSlot = None):
        if self.properties:
            for property in self.properties:
                if property.name == propertyName:
                    return property.kind
        return None

    def getImportSlotNames(self, parentSlot = None):
        names = self.inherited(PropertiesContainerMixin, 'getImportSlotNames')(
            self, parentSlot = parentSlot)
        names = names[:]
        if 'properties' in names:
            # The slot 'properties' is already imported by the method
            # importFromXmlRpc.
            names.remove('properties')
        return names

    def getPropertyNames(self, parentSlot = None):
        propertyNames = []
        object = self.getSlotObject(parentSlot = parentSlot)
        if object is None:
            prototypeIds = []
        else:
            prototypeIds = object.getC3PrototypeLinearization()
            prototypeIds = prototypeIds[1:]
        prototypeIds.reverse()
        for prototypeId in prototypeIds:
            prototypePropertyNames = self.getProxyForId(
                prototypeId).getImplementationPropertyNames(
                    prototypeId, parentSlot = parentSlot)
            for propertyName in prototypePropertyNames:
                if propertyName not in propertyNames:
                    propertyNames.append(propertyName)
        propertyNames += self.getImplementationPropertyNames(
            parentSlot = parentSlot)
        return propertyNames

    def getPropertySlot(self, propertyName, parentSlot = None):
        propertiesSlot = self.getSlot('properties', parentSlot = parentSlot)
        return slots.Property(propertyName, parent = propertiesSlot)

    def getPropertyValue(self, propertyName, propertyKind, parentSlot = None):
        if self.values and self.values.has_key(propertyName):
            return self.getImplementationPropertyValue(
                propertyName, propertyKind, parentSlot = parentSlot)
        return self.getDefaultPropertyValue(
            propertyName, propertyKind, parentSlot = parentSlot)

    def getPropertyValueEffectiveKind(self, propertyName, parentSlot = None):
        kind = None
        if self.properties:
            for property in self.properties:
                if property.name == propertyName:
                    kind = property.kind
                    break
        mutations = []
        if kind is not None:
            if kind.isEffective:
                return kind
            mutations.append(kind)
        object = self.getSlotObject(parentSlot = parentSlot)
        if object is not None:
            prototypeIds = object.getC3PrototypeLinearization()
            for prototypeId in prototypeIds[1:]:
                kind = self.getProxyForId(
                    prototypeId).getImplementationPropertyValueKind(
                        prototypeId, propertyName, self,
                        parentSlot = parentSlot)
                if kind is None:
                    continue
                if kind.isEffective:
                    return kind.buildEffective(mutations)
                mutations.append(kind)
        raise Exception('Property "%s" of %s has no effective kind' % (
            propertyName, commonTools.getSlotPath(parentSlot)))

##     def getPropertyValueEffectiveKind(self, propertyName, parentSlot = None):
##         kind = None
##         if self.properties:
##             for property in self.properties:
##                 if property.name == propertyName:
##                     kind = property.kind
##                     break
##         if kind is None:
##             kind = self.newThing('kind', 'Mutation')
##         if kind.isEffective:
##             return kind
##         if kind.values is not None:
##             kindValues = kind.values.copy()
##         else:
##             kindValues = {}
##         effectiveWidget = None
##         widgetValues = {}
##         if kindValues.has_key('widget'):
##             widget = kindValues['widget']
##             del kindValues['widget']
##             if widget is not None:
##                 if widget.isEffective:
##                     effectiveWidget = widget
##                 elif widget.values is not None:
##                     widgetValues = widget.values.copy()
##         object = self.getSlotObject(parentSlot = parentSlot)
##         if object is not None:
##             prototypeIds = object.getC3PrototypeLinearization()
##             for prototypeId in prototypeIds[1:]:
##                 kind = self.getProxyForId(
##                     prototypeId).getImplementationPropertyValueKind(
##                         prototypeId, propertyName, self,
##                         parentSlot = parentSlot)
##                 if kind is None:
##                     continue
##                 if kind.isEffective:
##                     if effectiveWidget is None:
##                         if widgetValues:
##                             notUsed = kind.getModelWidget(
##                                 self.getSlot(
##                                     propertyName, parentSlot = parentSlot))
##                             assert kind.widget.isEffective
##                             for name, value in widgetValues.items():
##                                 setattr(kind.widget, name, value)
##                     for name, value in kindValues.items():
##                         setattr(kind, name, value)
##                     return kind
##                 elif kind.values is not None:
##                     for name, value in kind.values.items():
##                         if name == 'widget':
##                             if effectiveWidget is not None:
##                                 continue
##                             widget = value
##                             if widget is None:
##                                 continue
##                             if widget.isEffective:
##                                 effectiveWidget = widget
##                                 for name2, value2 in widgetValues.items():
##                                     setattr(widget, name2, value2)
##                                 continue
##                             if widget.values is None:
##                                 continue
##                             for name2, value2 in widget.values.items():
##                                 if widgetValues.has_key(name2):
##                                     continue
##                                 widgetValues[name2] = value2
##                             continue
##                         if kindValues.has_key(name):
##                             continue
##                         kindValues[name] = value
##         raise Exception('Property "%s" of %s has no effective kind' % (
##             propertyName, commonTools.getSlotPath(parentSlot)))

    def getPropertyValueSlot(self, propertyName, parentSlot = None):
        if parentSlot is None:
            container = self
        else:
            container = None
        return slots.PropertyValue(
            propertyName, container = container, parent = parentSlot)

    def getProxyForId(self, id):
        from glasnost.proxy.tools import getProxy
        return getProxy(id)

    def getSlot(self, attributeName, parentSlot = None):
        if attributeName in self.inherited(
            PropertiesContainerMixin, 'getSlotNames')(
                self, parentSlot = parentSlot):
            return self.inherited(PropertiesContainerMixin, 'getSlot')(
                self, attributeName, parentSlot = parentSlot)
        return self.getPropertyValueSlot(
            attributeName, parentSlot = parentSlot)

    def getSlotNames(self, parentSlot = None):
        slotNames = self.inherited(PropertiesContainerMixin, 'getSlotNames')(
            self, parentSlot = parentSlot)
        return slotNames + self.getPropertyNames(parentSlot = parentSlot)

    def importFromXmlRpc(self, dataImport, parentSlot = None):
        # The slot 'properties' must be imported first, because it is needed
        # to compute the slotNames list.
        if dataImport.has_key('__noneSlotNames__'):
            noneSlotNames = dataImport['__noneSlotNames__']
        else:
            noneSlotNames = []
        for slotName in ['properties']:
            if dataImport.has_key(slotName):
                exportedValue = dataImport[slotName]
            elif slotName in noneSlotNames:
                exportedValue = None
            else:
                continue
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if not kind.isPublicForXmlRpc:
                continue
            slot.setValue(kind.importValueFromXmlRpc(slot, exportedValue))
        return self.inherited(PropertiesContainerMixin, 'importFromXmlRpc')(
            self, dataImport, parentSlot = parentSlot)

    def setImplementationPropertyValue(self, propertyName, value,
                                       parentSlot = None):
        if self.values is None:
            self.values = {}
        self.values[propertyName] = value

    def setImplementationPropertyValueKind(self, propertyName, propertyKind,
                                           parentSlot = None):
        propertyKindIsEmpty = propertyKind is None or (
            not propertyKind.isEffective
            and not propertyKind.properties
            and not propertyKind.values)
        if self.properties is None:
            self.properties = []
        else:
            for i in range(len(self.properties)):
                if self.properties[i].name == propertyName:
                    if propertyKindIsEmpty:
                        del self.properties[i]
                        return
                    else:
                        property = self.properties[i]
                        property.kind = propertyKind
                        property.kindName = propertyKind.getThingName()
                        return
        if not propertyKindIsEmpty:
            property = self.newThing('other', 'Property')
            property.kind = propertyKind
            property.kindName = propertyKind.getThingName()
            property.name = propertyName
            self.properties.append(property)

    def setPropertyValue(self, propertyName, propertyKind, value,
                         parentSlot = None):
        if self.values is None:
            self.values = {}
        if value == self.getDefaultPropertyValue(
            propertyName, propertyKind, parentSlot = parentSlot):
            if self.values.has_key(propertyName):
                del self.values[propertyName]
                if not self.values:
                    del self.values
        else:
            self.values[propertyName] = value


class Property(things.BaseThing):
    kind = None
    kind_kind_isPublicForXmlRpc = 1
    kind_kind_valueThingNameSlotName = 'kindName'
    kind_kindName = 'Kind'

    kindName = None
    kindName_kind_isPublicForXmlRpc = 1
    kindName_kind_isRequired = 1
    kindName_kindName = 'KindName'

    name = None
    name_kind_isPublicForXmlRpc = 1
    name_kind_isRequired = 1
    name_kindName = 'PythonIdentifier'

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = things.BaseThing.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames += ['label','name', 'kindName', 'kind']
        return slotNames
things.register(Property)
