# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Common Things"""

__version__ = '$Revision: 1.36 $'[11:-2]


import copy
import types

import faults
import tools


allThingClasses = {}


def register(thingClass):
    thingName = thingClass.getThingName.im_func(thingClass)
    if not allThingClasses.has_key(thingClass.thingCategory):
        allThingClasses[thingClass.thingCategory] = {}
    assert not allThingClasses[thingClass.thingCategory].has_key(thingName)
    allThingClasses[thingClass.thingCategory][thingName] = thingClass


class BaseThing:
    isEffective = 1
    isEffective_kind_isProperty = 0
    isEffective_kind_isPublicForXmlRpc = 0
    isEffective_kindName = 'Boolean'

    thingCategory = 'other'
    thingCategory_kind_isProperty = 0
    thingCategory_kindName = 'String'

    thingName = None
    thingName_kind_isProperty = 0
    thingName_kindName = 'String'

    def __init__(self):
        pass

    def buildEffective(self, mutations):
        names = []
        for mutation in mutations:
            if mutation.values is not None:
                for name in mutation.values.keys():
                    if not name in names:
                        names.append(name)
        if not names:
            return self
        effective = copy.copy(self)
        for name in names:
            valueMutations = []
            for mutation in mutations:
                if mutation.values is not None \
                       and mutation.values.has_key(name):
                    value = mutation.values[name]
                    if not isinstance(value, BaseThing):
                        break
                    if value.isEffective:
                        if valueMutations:
                            value = value.buildEffective(valueMutations)
                        break
                    else:
                        valueMutations.append(value)
            setattr(effective, name, value)
        return effective

    def buildKinds(self):
        # Should be converted to a class method, when porting Glasnost to
        # Python 2.2.

        baseClasses = self.getC3ClassLinearization()
        slotNames = [slotName
                     for slotName in self.__dict__.keys()
                     if slotName.endswith('_kindName')]
        for baseClass in baseClasses:
            for slotName in baseClass.__dict__.keys():
                if slotName.endswith('_kindName') \
                   and not slotName in slotNames:
                    slotNames.append(slotName)
        slotNames = [slotName[:-len('_kindName')]
                     for slotName in slotNames
                     if getattr(self, slotName) is not None]
        slotNames = [slotName
                     for slotName in slotNames
                     if not '_' in slotName \
                        or not slotName.split('_', 1)[0] in slotNames]
        for slotName in slotNames:
            slotOptionHeader = slotName + '_kind_'
            slotOptionHeaderLen = len(slotOptionHeader)
            slotOptionNames = [name
                               for name in self.__dict__.keys()
                               if name.startswith(slotOptionHeader)]
            for baseClass in baseClasses:
                for name in baseClass.__dict__.keys():
                    if name.startswith(slotOptionHeader) \
                       and name not in slotOptionNames:
                        slotOptionNames.append(name)
            slotOptions = {}
            for slotOptionName in slotOptionNames:
                slotOptions[slotOptionName[slotOptionHeaderLen:]] = getattr(
                    self, slotOptionName)
            kindName = getattr(self, slotName + '_kindName')
            if kindName is None or kindName == '__getter__':
                kind = None
            else:
                kind = self.newThing('kind', kindName)
                kind.buildOptions(slotOptions)
            setattr(self.__class__, slotName + '_kind', kind)

    def convertIds(self, sourceDispatcherId, destinationDispatcherId,
                   parentSlot = None):
        slotNames = self.getSlotNames(parentSlot = parentSlot)
        for slotName, value in self.__dict__.items():
            if not slotName in slotNames:
                continue
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            newValue = slot.getKind().convertValueIds(
                slot, value, sourceDispatcherId,
                destinationDispatcherId)
            if newValue != value:
                slot.setValue(newValue)

    def exportToXmlRpc(self, requiredSlotNames = None, parentSlot = None):
        result = {
            '__thingCategory__': self.thingCategory,
            '__thingName__': self.getExportClassName(),
            }
        noneSlotNames = []
        for slotName in self.getExportSlotNames(parentSlot = parentSlot):
            if requiredSlotNames and not slotName in requiredSlotNames:
                continue
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if not kind.isPublicForXmlRpc:
                continue
            # Key must be a string for xmlrpclib.
            # A slotName must always be a valid Python Indentifier, so no
            # encoding is needed
            # slotName = tools.utf8(slotName)
            assert type(slotName) == types.StringType
            assert not slotName in [
                '__noneSlotNames__', '__thingCategory__', '__thingName__']
            if not slot.hasLocalValue():
                continue
            value = slot.getLocalValue()
            exportedValue = kind.exportValueToXmlRpc(slot, value)
            if exportedValue is None:
                noneSlotNames.append(slotName)
            else:
                result[slotName] = exportedValue
        if noneSlotNames:
            result['__noneSlotNames__'] = noneSlotNames
        return result

    def getAllThingClasses(self):
        # Should be converted to a class method, when porting Glasnost to
        # Python 2.2.
        return allThingClasses

    def getC3ClassLinearization(self):
        import tools_new as commonTools
        return commonTools.getC3ClassLinearization(self.__class__)

    def getExportClassName(self):
        return self.__class__.__name__

    def getExportSlotNames(self, parentSlot = None):
        return self.getSlotNames(parentSlot = parentSlot)

    def getImportSlotNames(self, parentSlot = None):
        return self.getSlotNames(parentSlot = parentSlot)

    def getModifySlotNames(self, parentSlot = None):
        return self.getSlotNames(parentSlot = parentSlot)

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        return ['thingCategory', 'thingName', 'isEffective']

    def getPython22ClassLinearization(self):
        import tools_new as commonTools
        baseClasses = []
        commonTools.buildPython22ClassLinearization(
            self.__class__, baseClasses)
        return baseClasses

    def getSlot(self, attributeName, parentSlot = None):
        import slots
        self.initClass()
        if parentSlot is None:
            container = self
        else:
            container = None
        return slots.Attribute(
            attributeName, container = container, parent = parentSlot)

    def getSlotByPath(self, path, parentSlot = None):
        if path.startswith('self'):
            path = path[len('self'):]
        if not path:
            return parentSlot
        if path[0] != '.':
            raise faults.NonExistentSlotPath(path)
        path = path[1:]
        foundSlotName = None
        for slotName in self.getSlotNames(parentSlot = parentSlot):
            if path.startswith(slotName) and (
                foundSlotName is None or len(slotName) > len(foundSlotName)):
                foundSlotName = slotName
        if foundSlotName is None:
            raise faults.NonExistentSlotPath(path)
        slot = self.getSlot(foundSlotName, parentSlot = parentSlot)
        return slot.getKind().getModelSlotByPath(
            slot, path[len(foundSlotName):])

    def getSlotNames(self, parentSlot = None):
        self.initClass()
        baseClasses = self.getC3ClassLinearization()
        slotNames = [slotName
                     for slotName in self.__dict__.keys()
                     if slotName.endswith('_kind')]
        for baseClass in baseClasses:
            for slotName in baseClass.__dict__.keys():
                if slotName.endswith('_kind') and slotName not in slotNames:
                    slotNames.append(slotName)
        slotNames = [slotName[:-5]
                     for slotName in slotNames
                     if getattr(self, slotName) is not None \
                        or hasattr(self, slotName + 'Getter') \
                        and getattr(self, slotName + 'Getter') is not None]
        slotNames = [slotName
                     for slotName in slotNames
                     if not '_' in slotName \
                        or not slotName.split('_', 1)[0] in slotNames]
        return slotNames

    def getSlotObject(self, parentSlot = None):
        if parentSlot is None:
            if self.thingCategory == 'object':
                return self
            else:
                return None
        else:
            return parentSlot.getObject()

    def getThingName(cls):
        if type(cls) == types.InstanceType:
            cls = cls.__class__
        if cls.thingName:
            thingName = cls.thingName
        else:
            thingName = cls.__name__
        return thingName

    def hasThingClass(self, thingCategory, thingName):
        allThingClasses = self.getAllThingClasses()
        if allThingClasses is None:
            return 0
        if not allThingClasses.has_key(thingCategory):
            return 0
        return allThingClasses[thingCategory].has_key(thingName)

    def importFromXmlRpc(self, dataImport, parentSlot = None):
        if dataImport.has_key('__noneSlotNames__'):
            noneSlotNames = dataImport['__noneSlotNames__']
        else:
            noneSlotNames = []
        for slotName in self.getImportSlotNames(parentSlot = parentSlot):
            if slotName in [
                '__noneSlotNames__', '__thingCategory__', '__thingName__']:
                continue
            # A slotName must always be a valid Python Indentifier, so no
            # encoding is needed to for the dataImport keys.
            if dataImport.has_key(slotName):
                exportedValue = dataImport[slotName]
            elif slotName in noneSlotNames:
                exportedValue = None
            else:
                continue
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if not kind.isPublicForXmlRpc:
                continue
            slot.setValue(kind.importValueFromXmlRpc(slot, exportedValue))

    def inherited(self, cls, attributeName):
        baseClasses = self.getC3ClassLinearization()
        for i in range(baseClasses.index(cls) + 1, len(baseClasses)):
            baseClass = baseClasses[i]
            if baseClass.__dict__.has_key(attributeName):
                return getattr(baseClass, attributeName)
        raise AttributeError(
            'class %s has no inherited attribute \'%s\' in %s' % (
            cls.__name__, attributeName, baseClasses))

    def initClass(self):
        # Should be converted to a class method, when porting Glasnost to
        # Python 2.2.

        if not self.__class__.__dict__.has_key('classIsInited') \
           or not self.__class__.__dict__['classIsInited']:
            # classIsInited should be set prior to calling buildKinds()
            self.__class__.__dict__['classIsInited'] = 1
            self.buildKinds()

    def isEmpty(self, parentSlot = None):
        return not self.__dict__

    def isSlotName(self, attributeName, parentSlot = None):
        return attributeName in self.getSlotNames(parentSlot = parentSlot)

    def newKind(self, parentSlot = None):
        kind = self.newThing('kind', 'Thing')
        kind.valueThingCategory = self.thingCategory
        kind.valueThingName = self.getThingName()
        return kind

    def newThing(self, thingCategory, thingName, *arguments, **keywords):
        try:
	    category = self.getAllThingClasses()[thingCategory]
	except KeyError:
            raise Exception(
                'Unknown category (= %s) for new thing in object (= %s)' % (
                thingCategory, str(self)))
						      
        try:
            thingClass = category[thingName]
        except KeyError:
            raise Exception(
                'Unknown in category (= %s), name (= %s) for new thing in'
                'object (= %s). Possible values are %s' % (
                thingCategory, thingName, str(self), repr(category)))
        return thingClass(*arguments, **keywords)

    def removeIds(self, rolesToKeep, parentSlot = None):
        slotNames = self.getSlotNames(parentSlot = parentSlot)
        for slotName, value in self.__dict__.items():
            if not slotName in slotNames:
                continue
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            newValue = slot.getKind().removeValueIds(slot, value, rolesToKeep)
            if newValue != value:
                slot.setValue(newValue)

    def repair(self, toVersion, parentSlot = None):
        changed = 0
        slotNames = self.getSlotNames(parentSlot = parentSlot)
        for slotName, value in self.__dict__.items():
            if not slotName in slotNames:
                continue
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            newValue = slot.getKind().repairValue(slot, value, toVersion)
            if newValue is not None:
                changed = 1
                slot.setValue(newValue)
        return changed
