# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Common Tools"""

__version__ = '$Revision: 1.28 $'[11:-2]


import ConfigParser
import md5
import re
import smtplib
import string
import sys
import types

import glasnost

import faults
from gpg import Gpg
from parsers import makeDocBookFromSpip, makeHtmlFromDocBook, makeHtmlFromHtml, makeHtmlFromSpip, makeHtmlFromReStructuredText, makeHtmlFromText, makeLatexFromSpip
import context


config = None
configFile = '/etc/glasnost/config'
mainModule = sys.modules['__main__']
is8bit = re.compile("[\x80-\xff]").search


def extractApplicationHostName(id):
    try:
        return extractApplicationHostNameAndPort(id).split(':')[0]
    except IndexError:
        raise Exception('Malformed id or application id = %s' % str(id))


def extractApplicationHostNameAndPort(id):
    try:
        assert id.startswith('glasnost://')
        splittedApplicationId = id[11:].split('/')
        # applicationHostNameAndPort, applicationRole = splittedApplicationId
        return splittedApplicationId[0]
    except IndexError:
        raise Exception('Malformed id or application id = %s' % str(id))


def extractApplicationPort(id):
    infos = id[11:].split(':', 1)
    if len(infos) < 2:
        return str(eval(getConfig('Dispatcher', 'ServerPort'), {}))
    return infos[1]


def extractLocalId(id):
    applicationHostNameAndPort, applicationRole, localId = splitId(id)
    return localId


def extractServerId(id):
    assert id.startswith('glasnost://')
    splittedId = id[11:].split('/')
    return 'glasnost://%s/%s' % (splittedId[0], splittedId[1])


def extractServerRole(id):
    assert id.startswith('glasnost://')
    splittedId = id[11:].split('/')
    return splittedId[1]


def getConfig(section, value, default = None, vars = None, raw = 0):
    global config
    if config is None:
        config = ConfigParser.ConfigParser()
        config.readfp(open(configFile))
        return getConfig(section, value, default = default, vars = vars, raw = raw)
    else:
        try:
            return config.get(section, value, vars = vars, raw = raw)
        except ConfigParser.NoOptionError:
            return default
        except ConfigParser.NoSectionError:
            return default


def getConfigNoCache(section, value, default = None, vars = None, raw = 0):
    conf = ConfigParser.ConfigParser()
    conf.readfp(open(configFile))
    try:
        return conf.get(section, value, vars = None, raw = raw)
    except ConfigParser.NoOptionError:
        return default
    except ConfigParser.NoSectionError:
        return default


# FIXME: Remove getDefaultDispatcherId ASAP. The contextVar dispatcherId
# should become mandatory.
def getDefaultDispatcherId():
    dispatcherId = context.getVar('dispatcherId')
    if dispatcherId is not None:
        return dispatcherId
    if not hasattr(mainModule, 'defaultDispatcherIds') \
       or mainModule.defaultDispatcherIds is None:
        defaultDispatcherIds = getConfig(
            'Misc', 'DispatcherURLs', default = 'glasnost://localhost')
        defaultDispatcherIds = defaultDispatcherIds.replace(',', ' ')
        defaultDispatcherIds = defaultDispatcherIds.strip().split()
        for i in range(len(defaultDispatcherIds)):
            if defaultDispatcherIds[i][-1] == '/':
                defaultDispatcherIds[i] = defaultDispatcherIds[i][:-1]
        mainModule.defaultDispatcherIds = defaultDispatcherIds
    return mainModule.defaultDispatcherIds[0]


### FIXME: To remove ASAP.
def getDefaultDispatcherHostNameAndPort():
    return getDefaultDispatcherId()[11:]


def getStringFromDigest(object, digest):
    objectType = type(object)
    if objectType in [types.StringType, types.UnicodeType]:
        if md5.new(object.replace('\r\n', '\n')).hexdigest() == digest:
            return object
        else:
            return None
    elif objectType in [types.ListType, types.TupleType]:
        for item in object:
            result = getStringFromDigest(item, digest)
            if result is not None:
                return result
    elif objectType == types.DictionaryType:
        for key, value in object.items():
            result = getStringFromDigest(key, digest)
            if result is not None:
                return result
            result = getStringFromDigest(value, digest)
            if result is not None:
                return result
        return None
    elif objectType == types.InstanceType:
        for key, value in object.__dict__.items():
            result = getStringFromDigest(key, digest)
            if result is not None:
                return result
            result = getStringFromDigest(value, digest)
            if result is not None:
                return result
        return None
    else:
        return None


def iso8859_15(s):
    """Convert a string from utf-8 to iso-8859-15."""
    # FIXME: due to yet untraced errors I switched from latin-0 to latin-1
    if type(s) == types.UnicodeType:
        return s.encode('latin-1')
    elif is8bit(s):
        try:
            return unicode(s, 'utf-8').encode('latin-1')
        except UnicodeError:
            return s
    else:
        return s


def repairId(id):
    if not id or id.startswith('glasnost://'):
        return  None
    try:
        serverId, localId = id.split('/')
        assert serverId.startswith('ee')
        assert serverId.endswith('Server')
        serverRole = serverId[2:-6].lower()
    except IndexError:
        raise Exception('Malformed old id = %s' % str(id))
    newId = '%s/%s/%s' % (getDefaultDispatcherId(), serverRole, localId)
    return newId


def sendMail(mailFrom, mailTo, mailSubject, mailMessage, mailPerson = None,
             mailPersonId = None, moreHeaders = None):
    if type(mailTo) in [types.StringType, types.UnicodeType]:
        mailToStr = mailTo
    else:
        mailToStr = string.join(mailTo, ', ')
    mailHeader = "From: %s\nTo: %s\nSubject: %s\nMime-Type: 1.0\n" % (
        mailFrom, mailToStr, mailSubject)
    if moreHeaders:
        mailHeader += '\n'.join([ '%s: %s' % x for x in moreHeaders.items()])

    mailMessage = "\n" + mailMessage
    gpgEncrypted = 0
    if mailPersonId:
        from glasnost.proxy.tools import getProxyForServerRole
        preferencesProxy = getProxyForServerRole('preferences')
        cryptEmails = preferencesProxy.getPreferenceByUserId(
                mailPersonId).cryptEmails
        if cryptEmails:
            try:
                gpg = Gpg(email = mailPerson.email,
                          fingerprint = mailPerson.fingerprint)
                gpg.deleteKey()
                gpg.addKey()
                mailMessage = gpg.encrypt(mailMessage)
                gpgEncrypted = gpg.isEncrypted(mailMessage)
            except:
                pass
            if gpgEncrypted:
                mailHeader += """\
Content-Type: multipart/encrypted; boundary=bound1;
  protocol=\"application/pgp-encrypted\"; 

--bound1
Content-Type: application/pgp-encrypted

Version: 1

--bound1
Content-Type: application/octet-stream

"""
    mailMessage = "%s\n%s" % (mailHeader, mailMessage)
    if gpgEncrypted:
        mailMessage += "\n--bound1--\n"
    mailMessage = mailMessage.replace('\n', '\r\n')

    smtpServerName = getConfig('Mail', 'ServerHostName', default = 'localhost')
    smtpPort = getConfig('Mail', 'ServerPort', default = '25')
    smtpPort = int(smtpPort)

    try:
        smtpServer = smtplib.SMTP()
        smtpServer.connect(smtpServerName, smtpPort)
        smtpServer.sendmail(mailFrom, mailTo, mailMessage)
        smtpServer.quit()
    except:
        raise faults.SmtpError()


def splitApplicationId(applicationId):
    try:
        assert applicationId.startswith('glasnost://')
        splittedApplicationId = applicationId[11:].split('/')
        assert len(splittedApplicationId) == 2
        # applicationHostNameAndPort, applicationRole = splittedApplicationId
    except IndexError:
        raise Exception('Malformed application id = %s' % str(applicationId))
    except AssertionError:
        raise Exception('Malformed application id = %s' % str(applicationId))
    return splittedApplicationId


def splitId(id):
    try:
        assert id.startswith('glasnost://')
        splittedId = id[11:].split('/')
        assert len(splittedId) == 3
        # applicationHostNameAndPort, applicationRole, localId = splittedId
    except IndexError:
        raise Exception('Malformed id = %s' % str(id))
    except AssertionError:
        raise Exception('Malformed id = %s' % str(id))
    return splittedId


def utf8(s):
    """Convert a string from iso-8859-15 to utf-8."""
    if type(s) == types.UnicodeType:
        return s.encode('utf-8')
    elif is8bit(s):
        return unicode(s, 'latin-1').encode('utf-8')
    else:
        return s
