# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Common Tools"""

__version__ = '$Revision: 1.21 $'[11:-2]


import random


def buildPython22ClassLinearization(aClass, baseClasses):
    if aClass in baseClasses:
        baseClasses.remove(aClass)
    baseClasses.append(aClass)
    for baseClass in aClass.__bases__:
        buildPython22ClassLinearization(baseClass, baseClasses)


def extractDispatcherId(id):
    try:
        assert id.startswith('glasnost://')
        splittedApplicationId = id[11:].split('/')
        return 'glasnost://%s' % splittedApplicationId[0]
    except IndexError:
        raise Exception('Malformed id or application id = %s' % str(id))


def extractRole(id):
    try:
        assert id.startswith('glasnost://')
        splittedApplicationId = id[11:].split('/')
        if len(splittedApplicationId) == 1:
            return ''
        else:
            return splittedApplicationId[1]
    except IndexError:
        raise Exception('Malformed id or application id = %s' % str(id))


def getC3ClassLinearization(c):
    # An implementation of the C3 class hierarchy Linearization.
    # Cf. http://www.webcom.com/haahr/dylan/linearization-oopsla96.html
    linearization = [c]
    if c.__bases__:
        basesByPrecedenceOrder = list(c.__bases__)
        basesByPrecedenceOrder.reverse()
        remainingInputs = [getC3ClassLinearization(base)
                           for base in c.__bases__] + [basesByPrecedenceOrder]
        while remainingInputs:
            # The first class which is not in the tail of each remaining input
            # is the next class to append to the linearization.
            for remainingInput in remainingInputs:
                candidate = remainingInput[0]
                for remainingInput2 in remainingInputs:
                    if candidate in remainingInput2[1:]:
                        break
                else:
                    break
            else:
                raise Exception(
                    'Inconsistent class hierarchy for class %s' % c)
            # A good candidate has been found.
            # Add it  to the partial linearization and remove it from the
            # remaining inputs.
            linearization.append(candidate)
            newRemainingInputs = []
            for remainingInput in remainingInputs:
                if remainingInput[0] == candidate:
                    del remainingInput[0]
                    if not remainingInput:
                        continue
                newRemainingInputs.append(remainingInput)
            remainingInputs = newRemainingInputs
    return linearization


def getSlotPath(slot):
    if slot is None:
        return 'self'
    else:
        return slot.getPath()


def makeApplicationId(id, role):
    return '%s/%s' % (extractDispatcherId(id), role)


def makeHttpHostNameAndPort(httpHostName, httpPort):
    if httpHostName is None and httpPort is None:
        return None
    elif httpPort is None:
        return httpHostName
    elif httpHostName is None:
        return 'localhost:%s' % httpPort
    else:
        return '%s:%s' % (httpHostName, httpPort)


def makeHttpPathAndQuery(httpPath, httpQuery):
    if httpPath is None and httpQuery is None:
        return None
    elif httpQuery is None:
        return httpPath
    elif httpPath is None:
        return '/?%s' % httpQuery
    else:
        return '%s?%s' % (httpPath, httpQuery)


def makeHttpScriptDirectoryPath(httpScriptPath):
    if httpScriptPath is None:
        return '/'
    splittedPath = httpScriptPath.split('/')
    del splittedPath[-1]
    path = '/'.join(splittedPath)
    if not path or path[0] != '/':
        path = '/' + path
    return path


def makepassword(
    length = 8,
    allowChar = 'ABCDEFGHIJKLMNPQRSTUVWXYabcdefghijmnopqrstuvwxyz0123456789'):
    """Generate a random password.

    @param length: length of the result string
    @type length: int
    @param CARAC: allowed charactere for the result string
    @type allowChar: String
    """
    g = random.Random()
    result = ''
    for i in range(length):
        result += allowChar[g.randrange(len(allowChar))]
    return result
