# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Common Widgets"""

__version__ = '$Revision: 1.25 $'[11:-2]


import properties
import things
from tools import *


register = things.register


class EmptyWidget(things.BaseThing):
    name = None
    name_kind_isProperty = 0
    name_kind_isPublicForXmlRpc = 0
    name_kind_label = N_('Name')
    name_kindName = 'String'

    thingCategory = 'widget'

    def __init__(self, **attributes):
        things.BaseThing.__init__(self)
        for name, value in attributes.items():
            # Ensure that each argument has been declared as an attribute.
            if not hasattr(self.__class__, name):
                raise Exception('Undeclared attribute %s for %s' % (
                    name, self.__class__.__name__))
            if getattr(self, name) == value:
                continue
            setattr(self, name, value)

    def __str__(self):
        return self.getName()

    def buildOptions(self, options):
        for name, value in options.items():
            # Ensure that each option has been declared as an attribute.
            nameBase = name.split('_')[0]
            if not hasattr(self.__class__, nameBase):
                raise Exception('Undeclared attribute %s for %s' % (
                    name, self.__class__.__name__))
            if hasattr(self, name) and getattr(self, name) == value:
                continue
            setattr(self, name, value)

    def getName(cls):
        if type(cls) == types.InstanceType:
            cls = cls.__class__
        if cls.name:
            name = cls.name
        else:
            name = cls.getThingName.im_func(cls)
        return name


class BaseWidget(EmptyWidget):
    apply = 0
    apply_kind_isPublicForXmlRpc = 1
    apply_kindName = 'Boolean'

    colSpan = None
    colSpan_kind_isPublicForXmlRpc = 1
    colSpan_kindName = 'Integer'

    fieldLabel = None
    fieldLabel_kind_isPublicForXmlRpc = 1
    fieldLabel_kind_label = N_('Label')
    fieldLabel_kindName = 'String'

    helpAlias = None
    helpAlias_kind_isPublicForXmlRpc = 1
    helpAlias_kind_label = N_('Help Alias')
    helpAlias_kindName = 'Alias'

    inForm = 0
    inForm_kind_isPublicForXmlRpc = 1
    inForm_kindName = 'Boolean'

    readOnly = 0
    readOnly_kind_isPublicForXmlRpc = 1
    readOnly_kindName = 'Boolean'

    def getHelpAlias(self, slot, fields):
        if self.helpAlias:
            return self.helpAlias
        return slot.getKind().getHelpAlias(slot, fields)

    def getModelLabel(self, slot):
        if self.fieldLabel:
            return self.fieldLabel
        return slot.getKind().getModelLabel(slot)
    
    def isInForm(self):
        return self.inForm

    def isReadOnly(self, slot):
        return self.readOnly or slot.getKind().isReadOnly

    def setInForm(self, value = 1):
        self.inForm = value

    def setReadOnly(self, value = 1):
        self.readOnly = value
register(BaseWidget)


class Amount(BaseWidget):
    sideLabels = None
    sideLabels_kind_isPublicForXmlRpc = 1
    sideLabels_kind_keyKind_valueName = 'String'
    sideLabels_kind_valueKind_valueName = 'String'
    sideLabels_kindName = 'Mapping'
register(Amount)


class Date(BaseWidget):
    yearMinRel = None
    yearMinRel_kind_defaultValue = -1
    yearMinRel_kind_isPublicForXmlRpc = 1
    yearMinRel_kind_balloonHelp = N_('Minimum Year Value Difference to Now')
    yearMinRel_kind_label = 'Year Minimum'
    yearMinRel_kindName = 'Integer'

    yearMaxRel = None
    yearMinRel_kind_defaultValue = 3
    yearMaxRel_kind_isPublicForXmlRpc = 1
    yearMaxRel_kind_balloonHelp = N_('Maximum Year Value Difference to Now')
    yearMaxRel_kind_label = 'Year Maximum'
    yearMaxRel_kindName = 'Integer'
register(Date)

class Duration(BaseWidget):
    pass
register(Duration)

class Email(BaseWidget):
    maxLength = None
    maxLength_kind_isPublicForXmlRpc = 1
    maxLength_kindName = 'Integer'

    size = None
    size_kind_isPublicForXmlRpc = 1
    size_kindName = 'Integer'
register(Email)

class InputCheckBox(BaseWidget):
    labels = None
    labels_kind_isPublicForXmlRpc = 1
    labels_kind_keyKind_valueName = 'String'
    labels_kind_valueKind_valueName = 'String'
    labels_kindName = 'Mapping'

    titles = None
    titles_kind_isPublicForXmlRpc = 1
    titles_kind_keyKind_valueName = 'String'
    titles_kind_valueKind_valueName = 'String'
    titles_kindName = 'Mapping'

    def getLabels(self, slot, fields):
        if self.labels:
            return self.labels
        return slot.getKind().getLabels(slot, fields)
register(InputCheckBox)


class InputPassword(BaseWidget):
    maxLength = None
    maxLength_kind_isPublicForXmlRpc = 1
    maxLength_kindName = 'Integer'

    size = None
    size_kind_isPublicForXmlRpc = 1
    size_kindName = 'Integer'
register(InputPassword)


class InputText(BaseWidget):
    maxLength = None
    maxLength_kind_isPublicForXmlRpc = 1
    maxLength_kindName = 'Integer'

    size = None
    size_kind_isPublicForXmlRpc = 1
    size_kindName = 'Integer'
register(InputText)


class Link(BaseWidget):
    pass
register(Link)


class Mapping(BaseWidget):
    fieldLabelPlural = None
    fieldLabelPlural_kind_isPublicForXmlRpc = 1
    fieldLabelPlural_kindName = 'String'
register(Mapping)


class Multi(BaseWidget):
    fieldFind = 0
    fieldFind_kind_isPublicForXmlRpc = 1
    fieldFind_kindName = 'Boolean'

    fieldLabelPlural = None
    fieldLabelPlural_kind_isPublicForXmlRpc = 1
    fieldLabelPlural_kindName = 'String'
register(Multi)


class Mutation(properties.PropertiesContainerMixin, EmptyWidget):
##     defaultValue_kindName = 'Mutation'

    isEffective = 0
register(Mutation)


class Path(BaseWidget):
    pass
register(Path)


class Select(BaseWidget):
    labels = None
    labels_kind_isPublicForXmlRpc = 1
    labels_kind_keyKind_valueName = 'String'
    labels_kind_valueKind_valueName = 'String'
    labels_kindName = 'Mapping'

    order = None
    order_kind_isPublicForXmlRpc = 1
    order_kind_itemKind_valueName = 'String'
    order_kindName = 'Sequence'

    titles = None
    titles_kind_isPublicForXmlRpc = 1
    titles_kind_keyKind_valueName = 'String'
    titles_kind_valueKind_valueName = 'String'
    titles_kindName = 'Mapping'

    def getLabels(self, slot, fields):
        if self.labels:
            return self.labels
        return slot.getKind().getLabels(slot, fields)
register(Select)


class SelectId(BaseWidget):
    fieldFind = 0
    fieldFind_kind_isPublicForXmlRpc = 1
    fieldFind_kindName = 'Boolean'

    noneLabel = N_('None')
    noneLabel_kind_isPublicForXmlRpc = 1
    noneLabel_kindName = 'String'
register(SelectId)


class TextArea(BaseWidget):
    buttonAddImage = 0
    buttonAddImage_kind_isPublicForXmlRpc = 1
    buttonAddImage_kindName = 'Boolean'

    preview = 0
    preview_kind_isPublicForXmlRpc = 1
    preview_kindName = 'Boolean'

    cols = None
    cols_kind_isPublicForXmlRpc = 1
    cols_kindName = 'Integer'

    format = 'spip'
    format_kind_isPublicForXmlRpc = 1
    format_kind_isRequired = 1
    format_kind_values = [
        'docbook',
        'html',
        'spip',
        'text',
        ]
    format_kindName = 'Choice'

    rows = None
    rows_kind_isPublicForXmlRpc = 1
    rows_kindName = 'Integer'

    viewInTextArea = 0
    viewInTextArea_kind_isPublicForXmlRpc = 1
    viewInTextArea_kindName = 'Boolean'

    def getHelpAlias(self, slot, fields):
        if self.format == 'spip':
            return 'help-spip-syntax'
        else:
            return BaseWidget.getHelpAlias(self, slot, fields)
register(TextArea)


class Thing(BaseWidget):
    pass
register(Thing)


class Time(InputText):
    pass
register(Time)


class Token(BaseWidget):
    pass
register(Token)

        
class UploadFile(BaseWidget):
    fileName = None
    fileName_kind_isPublicForXmlRpc = 1
    fileName_kindName = 'String'
register(UploadFile)

class Url(BaseWidget):
    maxLength = None
    maxLength_kind_isPublicForXmlRpc = 1
    maxLength_kindName = 'Integer'

    size = None
    size_kind_isPublicForXmlRpc = 1
    size_kindName = 'Integer'
register(Url)

class XSelect(Select):
    maxLength = None
    maxLength_kind_isPublicForXmlRpc = 1
    maxLength_kindName = 'Integer'

    otherFieldLabel = None
    otherFieldLabel_kind_isPublicForXmlRpc = 1
    otherFieldLabel_kindName = 'String'

    size = None
    size_kind_isPublicForXmlRpc = 1
    size_kindName = 'Integer'
register(XSelect)


