# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Accounts Proxy"""

__version__ = '$Revision: 1.20 $'[11:-2]


from glasnost.common.AccountsCommon import *
import glasnost.common.context as context

from DispatcherProxy import MultiCall, callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
from tools import *


class AdminAccounts(AdminMixin, AdminAccountsCommon):
    def getProxy(self):
        return accountsProxy
register(AdminAccounts)


class Account(ObjectProxyMixin, AccountCommon):
    def getProxy(self):
        return accountsProxy
register(Account)


class AccountsProxy(AccountsCommon, ObjectsProxy):
    def computeObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        callServer(
            extractServerId(objectId),
            'computeObject',
            [ extractServerId(objectId), getApplicationToken(), userToken,
             objectId ])
        return None

    def getBalance(
        self, objectId, dates = None, certaintyLevel = 5, multiCall = None):
        cache = context.getVar('cache')
        if not dates:
            import time
            dates = time.time()
        if type(dates) == type('1.0'):
            dates = [ dates ]
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getBalance',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, dates, certaintyLevel],
            resultHandler = self.getObject_handleResult,
            multiCall = multiCall)

    def searchObjectIds(
        self, accountClassSet, serverId = None):
        serverId = self.getServerId(serverId = serverId)
        if not accountClassSet: return []
        if type(accountClassSet) == type(''):
            accountClassSet = [ accountClassSet ]
        objectIds = []
        for accountClass in accountClassSet:
            minValue = accountClass
            maxValue = str(string.atoi(accountClass) + 1)
            objectIds += ObjectsProxy.searchObjectIds(
                self, serverId = serverId,
                number_GE = minValue,
                number_LT = maxValue)
        objectIds = ObjectsProxy.sortObjectIds(
            self, objectIds, serverId = serverId)
        return objectIds

