# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Articles Proxy"""

__version__ = '$Revision: 1.16 $'[11:-2]


from glasnost.common.ArticlesCommon import *
import glasnost.common.context as context

from DispatcherProxy import MultiCall, callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
from tools import *


class AdminArticles(AdminMixin, AdminArticlesCommon):
    def getProxy(self):
        return articlesProxy
register(AdminArticles)


class Article(ObjectProxyMixin, ArticleCommon):
    def getProxy(self):
        return articlesProxy
register(Article)


class ArticlesProxy(ArticlesCommon, ObjectsProxy):
    def canGetObjectHistory(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canGetObjectHistory',
            [extractServerId(objectId), getApplicationToken(),
             userToken, objectId])

    def getLastObjectIds(
        self, objectsCount, possibleAuthorsSet, possibleReadersSet,
        possibleWritersSet, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        if possibleAuthorsSet is None:
            possibleAuthorsSet = []
        if possibleReadersSet is None:
            possibleReadersSet = []
        if possibleWritersSet is None:
            possibleWritersSet = []
        return callServer(
            serverId,
            'getLastObjectIds',
            [serverId, getApplicationToken(), userToken,
             objectsCount, possibleAuthorsSet, possibleReadersSet,
             possibleWritersSet])

    def getLastObjects(
        self, objectsCount, possibleAuthorsSet, possibleReadersSet,
        possibleWritersSet, requiredSlotNames = None, serverId = None):
        objectIds = self.getLastObjectIds(
            objectsCount, possibleAuthorsSet, possibleReadersSet,
            possibleWritersSet, serverId = serverId)
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getPartialObject(
                objectId, requiredSlotNames, multiCall = multiCall)
        return [lazyObject() for lazyObject in multiCall.call()]

    def getObjectDiff(self, objectId, editionTime):
        userToken = context.getVar('userToken', default = '')
        result = callServer(
            extractServerId(objectId),
            'getObjectDiff',
            [extractServerId(objectId), getApplicationToken(),
             userToken, objectId,
             editionTime])
        result['diff'] = [ iso8859_15(line) for line in result['diff']]
        return result

    def getObjectDocBookChapter(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return iso8859_15(callServer(
            extractServerId(objectId),
            'getObjectDocBookChapter',
            [extractServerId(objectId), getApplicationToken(),
             userToken, objectId]))

    def getObjectHistory(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getObjectHistory',
            [extractServerId(objectId), getApplicationToken(),
             userToken, objectId])

    def getObjectLatexChapter(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return iso8859_15(callServer(
            extractServerId(objectId),
            'getObjectLatexChapter',
            [extractServerId(objectId), getApplicationToken(),
             userToken, objectId]))

    def getObjectVersion(self, objectId, editionTime):
        userToken = context.getVar('userToken', default = '')
        result = callServer(
            extractServerId(objectId),
            'getObjectVersion',
            [extractServerId(objectId), getApplicationToken(),
             userToken, objectId,
             editionTime])
        result['body'] = iso8859_15(result['body'])
        return result

    def search(self, searchTerms, scope, language, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        result = callServer(
            serverId,
            'search',
            [serverId, getApplicationToken(), userToken, utf8(searchTerms),
             scope, language])
        return result

    def searchObjectIds(
        self, searchTerms, serverId = None):
        serverId = self.getServerId(serverId = serverId)
        if not searchTerms: return []
        searchTerms = [utf8(x) for x in searchTerms]
        objectIds = ObjectsProxy.searchObjectIds(
            self,
            serverId = serverId,
            body_PT = searchTerms)
        objectIds = ObjectsProxy.sortObjectIds(
            self, objectIds, serverId = serverId)
        objectIds = objectIds[:20]
        return objectIds

