# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Authentication PAM Proxy"""

__version__ = '$Revision: 1.13 $'[11:-2]


import glasnost.common.context as context

from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import Proxy
from tools import *


class AuthenticationPamProxy(Proxy):
    serverRole = 'authenticationPam'

    def addUser(self, userName, login, password, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        callServer(
            serverId,
            'addUser',
            [serverId, getApplicationToken(), userToken, utf8(userName),
             utf8(login), utf8(password)])

    def canEmailPassword(self, userName, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canEmailPassword',
            [serverId, getApplicationToken(), userToken, utf8(userName)])

    def changePassword(self, userName, oldPassword, newPassword,
                       serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        callServer(
            serverId,
            'changePassword',
            [serverId, getApplicationToken(), userToken, utf8(userName),
             utf8(oldPassword), utf8(newPassword)])

    def checkAuthentication(self, userName, userPassword, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'checkAuthentication',
            [serverId, getApplicationToken(), userToken, utf8(userName),
             utf8(userPassword)])

    def deleteUser(self, userName, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        callServer(
            serverId,
            'deleteUser',
            [serverId, getApplicationToken(), userToken, utf8(userName)])

    def getPassword(self, userName, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'getPassword',
            [serverId, getApplicationToken(), userToken, utf8(userName)])

