# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Books Proxy"""

__version__ = '$Revision: 1.14 $'[11:-2]


from glasnost.common.BooksCommon import *
import glasnost.common.context as context

from DispatcherProxy import MultiCall, callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
from tools import *


class AdminBooks(AdminMixin, AdminBooksCommon):
    def getProxy(self):
        return booksProxy
register(AdminBooks)


class Book(ObjectProxyMixin, BookCommon):
    def getProxy(self):
        return booksProxy
register(Book)


class BooksProxy(BooksCommon, ObjectsProxy):
    def getLastObjectIds(
        self, objectsCount, possibleAuthorsSet, possibleReadersSet,
        possibleWritersSet, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        if possibleAuthorsSet is None:
            possibleAuthorsSet = []
        if possibleReadersSet is None:
            possibleReadersSet = []
        if possibleWritersSet is None:
            possibleWritersSet = []
        return callServer(
            serverId,
            'getLastObjectIds',
            [serverId, getApplicationToken(), userToken,
             objectsCount, possibleAuthorsSet, possibleReadersSet,
             possibleWritersSet])

    def getLastObjects(
        self, objectsCount, possibleAuthorsSet, possibleReadersSet,
        possibleWritersSet, requiredSlotNames = None, serverId = None):
        objectIds = self.getLastObjectIds(
            objectsCount, possibleAuthorsSet, possibleReadersSet,
            possibleWritersSet, serverId = serverId)
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getPartialObject(
                objectId, requiredSlotNames, multiCall = multiCall)
        return [lazyObject() for lazyObject in multiCall.call()]

    def makeObjectBody(self, objectId):
        userToken = context.getVar('userToken', default = '')
        callServer(
            extractServerId(objectId),
            'makeObjectBody',
            [extractServerId(objectId), getApplicationToken(),
             userToken, objectId])

