# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Cards Proxy"""

__version__ = '$Revision: 1.29 $'[11:-2]


import glasnost.common.context as context
from glasnost.common.CardsCommon import *
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools

from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
import properties
from tools import *


class AdminCards(AdminMixin, AdminCardsCommon):
    def getProxy(self):
        return cardsProxy
register(AdminCards)


class Card(ObjectProxyMixin, properties.PropertiesContainerMixin, CardCommon):
    def getProxy(self):
        return cardsProxy
register(Card)


class CardsProxy(CardsCommon, ObjectsProxy):
    def getC3PrototypeLinearization(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getC3PrototypeLinearization',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def getImplementationPropertyNames(self, objectId, parentSlot = None):
        userToken = context.getVar('userToken', default = '')
        parentPath = commonTools.getSlotPath(parentSlot)
        return callServer(
            extractServerId(objectId),
            'getImplementationPropertyNames',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, parentPath])

    def getImplementationPropertyValueKind(
        self, objectId, propertyName, implementation,  parentSlot = None):
        userToken = context.getVar('userToken', default = '')
        parentPath = commonTools.getSlotPath(parentSlot)
        try:
            propertyKindImport = callServer(
                extractServerId(objectId),
                'getImplementationPropertyValueKind',
                [extractServerId(objectId), getApplicationToken(), userToken,
                 objectId, propertyName, parentPath])
        except faults.NoneValue:
            return None
        propertySlot = implementation.getPropertySlot(
            propertyName, parentSlot = parentSlot)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)
        propertyKind = self.newThing(
            'kind', propertyKindImport['__thingName__'])
        propertyKindSlot.value = propertyKind
        propertyKind.importFromXmlRpc(
            propertyKindImport, parentSlot = propertyKindSlot)
        return propertyKind

    def getPropertyValue(
        self, objectId, propertyName, propertyKind, implementation,
        parentSlot = None):
        userToken = context.getVar('userToken', default = '')
        parentPath = commonTools.getSlotPath(parentSlot)

        propertySlot = implementation.getPropertySlot(
            propertyName, parentSlot = parentSlot)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)
        propertyKindSlot.value = propertyKind
        propertyKindExport = propertyKind.exportToXmlRpc()
        try:
            valueImport = callServer(
                extractServerId(objectId),
                'getPropertyValue',
                [extractServerId(objectId), getApplicationToken(), userToken,
                 objectId, propertyName, propertyKindExport, parentPath])
        except faults.NoneValue:
            return None
        slot = implementation.getPropertyValueSlot(
            propertyName, parentSlot = parentSlot)
        return propertyKind.importValueFromXmlRpc(slot, valueImport)

    def setImplementationPropertyValueKind(
        self, objectId, propertyName, propertyKind, implementation,
        parentSlot = None):
        userToken = context.getVar('userToken', default = '')
        parentPath = commonTools.getSlotPath(parentSlot)

        propertySlot = implementation.getPropertySlot(
            propertyName, parentSlot = parentSlot)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)
        propertyKindSlot.value = propertyKind
        propertyKindExport = propertyKind.exportToXmlRpc(
            parentSlot = propertyKindSlot)

        callServer(
            extractServerId(objectId),
            'setImplementationPropertyValueKind',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, propertyName, propertyKindExport, parentPath])

