# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Directories Proxy"""

__version__ = '$Revision: 1.21 $'[11:-2]


import base64

from glasnost.common.DirectoriesCommon import *
import glasnost.common.context as context

from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
import things
from tools import *


class AdminDirectories(AdminMixin, AdminDirectoriesCommon):
    def getProxy(self):
        return directoriesProxy
register(AdminDirectories)


class Directory(ObjectProxyMixin, DirectoryCommon):
    def getProxy(self):
        return directoriesProxy
register(Directory)

class DirectoryFile(things.ThingMixin, DirectoryFileCommon):
    def __init__(self):
        pass
    
    def getProxy(self):
        return directoriesProxy
register(DirectoryFile)


class DirectoriesProxy(DirectoriesCommon, ObjectsProxy):
    def getObject(self, objectId, path = '', withData = 0, multiCall = None):
        userToken = context.getVar('userToken', default = '')
        serverId = extractServerId(objectId)
        objectImport = callServer(
            serverId,
            'getObject',
            [serverId, getApplicationToken(), userToken, objectId, utf8(path),
             withData],
            resultHandler = self.getObject_handleResult,
            multiCall = multiCall)
        ## FIXME: find for what sometime objectImport is dictionnary and
        #         sometime is instance of Directories. Where is bug ???
        if type(objectImport) == type({}):
            result = self.newThing('object', objectImport['__thingName__'])
            result.importFromXmlRpc(objectImport)
        else:
            result = objectImport
        return result
        
    def deletePath(self, objectId, path):
        userToken = context.getVar('userToken', default = '')
        serverId = extractServerId(objectId)
        callServer(
            serverId,
            'deletePath',
            [serverId, getApplicationToken(), userToken, objectId, utf8(path)])


    ### redefined methods
    def canAddObject(self, objectId = '', path = '/', serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canAddObject',
            [serverId, getApplicationToken(), userToken, objectId, utf8(path)])

    def canDeleteObject(self, objectId, path = '/'):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canDeleteObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, utf8(path)])

    def canGetObject(self, objectId, path = '/'):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canGetObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, utf8(path)])

    def canModifyObject(self, objectId, path = '/'):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canModifyObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, utf8(path)])

    def hasObject(self, objectId, path = '/'):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'hasObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, utf8(path)])

    def modifyObject(self, object):
        userToken = context.getVar('userToken', default = '')
        objectExport = object.exportToXmlRpc()
        object.version = callServer(
            extractServerId(object.id),
            'modifyObject',
            [extractServerId(object.id), getApplicationToken(), userToken,
             objectExport])
        return None

    def search(self, objectId, name, description):
        userToken = context.getVar('userToken', default = '')
        serverId = extractServerId(objectId)
        resulttmp = callServer(
            serverId,
            'search',
            [serverId, getApplicationToken(), userToken, objectId, utf8(name),
             utf8(description)])

        result = []
        for objectImport in resulttmp:
            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)
            result.append(object)
       
        return result

