# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Elections Proxy"""

__version__ = '$Revision: 1.15 $'[11:-2]


from glasnost.common.ElectionsCommon import *
import glasnost.common.context as context

from DispatcherProxy import MultiCall, callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
from tools import *


class AdminElections(AdminMixin, AdminElectionsCommon):
    def getProxy(self):
        return electionsProxy
register(AdminElections)


class ElectionMixin(ObjectProxyMixin):
    def getProxy(self):
        return electionsProxy


class AbstractElection(ElectionMixin, AbstractElectionCommon):
    pass
register(AbstractElection)


class ElectionAverage(ElectionMixin, ElectionAverageCommon):
    pass
register(ElectionAverage)


class ElectionCondorcet(ElectionMixin, ElectionCondorcetCommon):
    pass
register(ElectionCondorcet)


class ElectionsProxy(ElectionsCommon, ObjectsProxy):
    def abstainForVote(self, objectId, previousVoteToken):
        callServer(
            extractServerId(objectId),
            'abstainForVote',
            [extractServerId(objectId), getApplicationToken(), objectId,
             previousVoteToken])

    def canVote(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canVote',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def computeObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        callServer(
            extractServerId(objectId),
            'computeObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def getBallotKind(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getBallotKind',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def getLastObjectIds(
        self, objectsCount, possibleAuthorsSet, possibleReadersSet,
        possibleWritersSet, possibleStates, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        if possibleAuthorsSet is None:
            possibleAuthorsSet = []
        if possibleReadersSet is None:
            possibleReadersSet = []
        if possibleWritersSet is None:
            possibleWritersSet = []
        if possibleStates is None:
            possibleStates = []
        return callServer(
            serverId,
            'getLastObjectIds',
            [serverId, getApplicationToken(), userToken, objectsCount,
             possibleAuthorsSet, possibleReadersSet, possibleWritersSet,
             possibleStates])

    def getLastObjectIdsWithVoter(self, objectsCount, voterId, possibleStates,
                                  serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        if possibleStates is None:
            possibleStates = []
        return callServer(
            serverId,
            'getLastObjectIdsWithVoter',
            [serverId, getApplicationToken(), userToken, objectsCount, voterId,
             possibleStates])

    def getLastObjects(
        self, objectsCount, possibleAuthorsSet, possibleReadersSet,
        possibleWritersSet, possibleStates, requiredSlotNames = None,
        serverId = None):
        objectIds = self.getLastObjectIds(
            objectsCount, possibleAuthorsSet, possibleReadersSet,
            possibleWritersSet, possibleStates, serverId = serverId)
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getPartialObject(
                objectId, requiredSlotNames, multiCall = multiCall)
        return [lazyObject() for lazyObject in multiCall.call()]

    def getLastObjectsWithVoter(
        self, objectsCount, voterId, possibleStates,
        requiredSlotNames = None,
        serverId = None):
        objectIds = self.getLastObjectIdsWithVoter(
            objectsCount, voterId, possibleStates,
            serverId = serverId)
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getPartialObject(
                objectId, requiredSlotNames, multiCall = multiCall)
        return [lazyObject() for lazyObject in multiCall.call()]

    def getVote(self, objectId, voteToken):
        userToken = context.getVar('userToken', default = '')
        voteImport = callServer(
            extractServerId(objectId),
            'getVote',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, voteToken])
        vote = self.getVotesProxy().newThing(
            'object', voteImport['__thingName__'])
        vote.importFromXmlRpc(voteImport)
        return vote

    def getVotesProxy(self):
        from VotesProxy import votesProxy
        return votesProxy

    def pesterAbstentionnists(self, objectId):
        userToken = context.getVar('userToken', default = '')
        httpHostName = context.getVar('httpHostName')
        httpPort = context.getVar('httpPort')
        httpScriptDirectoryPath = context.getVar(
            'httpScriptDirectoryPath')
        if httpHostName is None:
            httpHostName = ''
        else:
            httpHostName = utf8(httpHostName)
        if httpPort is None:
            httpPort = ''
        else:
            httpPort = utf8(httpPort)
        if httpScriptDirectoryPath is None:
            httpScriptDirectoryPath = ''
        else:
            httpScriptDirectoryPath = utf8(httpScriptDirectoryPath)
        return callServer(
            extractServerId(objectId),
            'pesterAbstentionnists',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, httpHostName, httpPort, httpScriptDirectoryPath])

    def vote(self, objectId, voteToken, previousVoteToken):
        return callServer(
            extractServerId(objectId),
            'vote',
            [extractServerId(objectId), getApplicationToken(),
             objectId, voteToken,
             previousVoteToken])

