# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Groups Proxy"""

__version__ = '$Revision: 1.16 $'[11:-2]


from glasnost.common.GroupsCommon import *
import glasnost.common.context as context

from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
from tools import *


class AdminGroups(AdminMixin, AdminGroupsCommon):
    def getProxy(self):
        return getProxyForServerRole('groups')
register(AdminGroups)


class Group(ObjectProxyMixin, GroupCommon):
    def getProxy(self):
        return getProxyForServerRole('groups')
register(Group)


class GroupsProxy(GroupsCommon, ObjectsProxy):
    def addObjectMember(self, objectId, memberId):
        userToken = context.getVar('userToken', default = '')
        callServer(
            extractServerId(objectId),
            'addObjectMember',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, memberId])

    def getObjectContainedIds(self, objectId, serverRoles = None):
        userToken = context.getVar('userToken', default = '')
        if serverRoles is None:
            serverRoles = []
        return callServer(
            extractServerId(objectId),
            'getObjectContainedIds',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, serverRoles])

    def getObjectContainedMemberIds(self, objectId, serverRoles,
                                    encounteredForMember,
                                    encounteredForNonMember):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getObjectContainedMemberIds',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, serverRoles, encounteredForMember,
             encounteredForNonMember])

    def getObjectContainedNonMemberIds(self, objectId, serverRoles,
                                       encounteredForMember,
                                       encounteredForNonMember):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getObjectContainedNonMemberIds',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, serverRoles, encounteredForMember,
             encounteredForNonMember])

    def objectContains(self, objectId, containedId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'objectContains',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, containedId])

    def objectContainsAsMember(self, objectId, containedId,
                               encounteredForMember, encounteredForNonMember):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'objectContainsAsMember',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, containedId, encounteredForMember,
             encounteredForNonMember])

    def objectContainsAsNonMember(self, objectId, containedId,
                                  encounteredForMember,
                                  encounteredForNonMember):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'objectContainsAsNonMember',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, containedId, encounteredForMember,
             encounteredForNonMember])


def getSetContainedIds(set, serverRoles = None, serverId = None):
    userToken = context.getVar('userToken', default = '')
    serverId = getProxyForServerRole('groups').getServerId(serverId = serverId)
    if not set:
        return []
    if serverRoles is None:
        serverRoles = []
    return callServer(
        serverId,
        'getSetContainedIds',
        [serverId, getApplicationToken(), userToken, set, serverRoles])


def getSetDeltaContainedIds(membersSet, nonMembersSet, serverRoles = None,
                            serverId = None):
    userToken = context.getVar('userToken', default = '')
    serverId = getProxyForServerRole('groups').getServerId(serverId = serverId)
    if not membersSet:
        return []
    if nonMembersSet is None:
        nonMembersSet = []
    if serverRoles is None:
        serverRoles = []
    return callServer(
        serverId,
        'getSetDeltaContainedIds',
        [serverId, getApplicationToken(), userToken, membersSet, nonMembersSet,
         serverRoles])


def setContains(set, objectId, serverId = None):
    userToken = context.getVar('userToken', default = '')
    serverId = getProxyForServerRole('groups').getServerId(serverId = serverId)
    if not set:
        return 0
    return callServer(
        serverId,
        'setContains',
        [serverId, getApplicationToken(), userToken, set, objectId])
