# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Objects Proxy"""

__version__ = '$Revision: 1.36 $'[11:-2]


import glasnost.common.context as context
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools
from glasnost.common import XmlRpcTools

from DispatcherProxy import MultiCall, callServer, getApplicationToken
import kinds # Do not remove!
import things
from tools import *

register = things.register


class ObjectProxyMixin(things.ThingMixin):
    def getProxy(self):
        #return getProxyForServerRole(self.serverRole)
        raise NotImplementedError


class AdminWithoutWritersMixin(ObjectProxyMixin):
    def newDefaultObject(self):
        proxy = self.getProxy()
        return proxy.newThing('object', proxy.objectClassName)


class AdminMixin(AdminWithoutWritersMixin):
    pass


class Proxy(things.BaseThing):
    ### FIXME: Remove serverId ASAP.
    serverId = None
    serverRole = None
    thingCategory = 'proxy'
    notXmlRpcFunctions = None

    def __init__(self):
        dispatcherId = getDefaultDispatcherId()
        self.notAnXmlRpcFunction = []
        ### FIXME: Remove serverId ASAP.
        self.serverId = '%s/%s' % (dispatcherId, self.serverRole)

    def canGetAdmin(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canGetAdmin',
            [serverId, getApplicationToken(), userToken])

    def getHandledRoles(self):
        return None

    def getServerId(self, serverId = None):
        ### FIXME: serverId is a misnamer. It should be replaced by
        # dispatcherId.
        if serverId is not None:
            return commonTools.makeApplicationId(serverId, self.serverRole)
        dispatcherId = context.getVar('dispatcherId')
        if dispatcherId is not None:
            return commonTools.makeApplicationId(dispatcherId, self.serverRole)
        ### FIXME: Remove serverId ASAP. A global context is mandatory.
        return self.serverId

    def __getattr__(self, name):
        if name[:2] == '__' or name in self.notAnXmlRpcFunction:
            raise AttributeError
        class ProxyMethod:
            functionName = None
            masterProxy = None
            def __call__(self, *params, **kparams):
                if kparams.has_key('serverId'):
                    serverId = kparams['serverId']
                else:
                    if hasattr(self.masterProxy, 'serverRole'):
                        serverId = commonTools.makeApplicationId(
                            context.getVar(
                            'dispatcherId'), self.masterProxy.serverRole)
                    else:
                        raise 'hum' # FIXME: what should happen here ?
                params = XmlRpcTools.convertToXmlRpc(params)
                userToken = context.getVar('userToken', default = '')
                try:
                    response = callServer(
                            serverId,
                            self.functionName,
                            [serverId, getApplicationToken(), userToken] +
                            params)
                except faults.Fault, fault:
                    if fault.faultCode == -506:
                        delattr(self.masterProxy, self.functionName)
                        self.masterProxy.notAnXmlRpcFunction.append(
                            self.functionName)
                    raise
                except:
                    raise
                response = XmlRpcTools.convertFromXmlRpc(response)
                return response

        pm = ProxyMethod()
        pm.functionName = name
        pm.masterProxy = self
        setattr(self, name, pm)
        return pm


class AdministrableProxyMixin:
    def canModifyAdmin(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canModifyAdmin',
            [serverId, getApplicationToken(), userToken])

    def getAdmin(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        adminImport = callServer(
            serverId,
            'getAdmin',
            [serverId, getApplicationToken(), userToken])
        admin = self.newThing('object', adminImport['__thingName__'])
        admin.importFromXmlRpc(adminImport)
        return admin

    def isAdmin(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'isAdmin',
            [serverId, getApplicationToken(), userToken])

    def modifyAdmin(self, admin, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        adminExport = admin.exportToXmlRpc()
        admin.version = callServer(
            serverId,
            'modifyAdmin',
            [serverId, getApplicationToken(), userToken, adminExport])
        return None

    def newAdmin(self, fields):
        return self.newThing('object', self.adminClassName)


class ObjectsProxy(AdministrableProxyMixin, Proxy):
    adminClassName = None # To override.
    newObjectNameCapitalized = N_('New Object')
    objectClassName = None # To override.
    objectName = N_('object')
    objectNameCapitalized = N_('Object')
    objectsName = N_('objects')
    objectsNameCapitalized = N_('Objects')

    def addObject(self, object, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        objectExport = object.exportToXmlRpc()
        objectId =  callServer(
            serverId,
            'addObject',
            [serverId, getApplicationToken(), userToken, objectExport])
        return objectId

    def canAddObject(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canAddObject',
            [serverId, getApplicationToken(), userToken])

    def canCloneObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        serverId = extractServerId(objectId)
        return userToken and self.canGetObject(objectId) \
               and self.canAddObject(serverId = serverId)

    def canDeleteObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canDeleteObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def canGetObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canGetObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def canGetObjects(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canGetObjects',
            [serverId, getApplicationToken(), userToken])

    def canModifyObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canModifyObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def deleteObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        callServer(
            extractServerId(objectId),
            'deleteObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])
    
    def getLastObjectIds(self, objectsCount, possibleReadersSet,
                         possibleWritersSet, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        if possibleReadersSet is None:
            possibleReadersSet = []
        if possibleWritersSet is None:
            possibleWritersSet = []
        return callServer(
            serverId,
            'getLastObjectIds',
            [serverId, getApplicationToken(), userToken, objectsCount,
             possibleReadersSet, possibleWritersSet])

    def getLastObjects(
        self, objectsCount, possibleReadersSet, possibleWritersSet,
        requiredSlotNames = None, serverId = None):
        objectIds = self.getLastObjectIds(
            objectsCount, possibleReadersSet, possibleWritersSet,
            serverId = serverId)
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getPartialObject(
                objectId, requiredSlotNames,
                multiCall = multiCall)
        return [lazyObject() for lazyObject in multiCall.call()]

    def getObject(self, objectId, multiCall = None):
        cache = context.getVar('cache')
        if cache is not None:
            try:
                result = self.getObject_handleResult(cache[objectId])
                if multiCall:
                    multiCall.addEarlyResult(result)
                    return None
                else:
                    return result
            except KeyError:
                pass
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId],
            resultHandler = self.getObject_handleResult,
            multiCall = multiCall)

    def getObject_handleResult(self, lazyObject):
        objectImport = lazyObject()
        object = self.newThing('object', objectImport['__thingName__'])
        object.importFromXmlRpc(objectImport)
        cache = context.getVar('cache')
        if cache is not None and object.canCache():
            cache[object.id] = lazyObject
        return object

    def getObjectIds(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'getObjectIds',
            [serverId, getApplicationToken(), userToken])

    def getObjectLabelAndLanguage(self, objectId, multiCall = None):
        userToken = context.getVar('userToken', default = '')
        if not objectId:
            if multiCall is None:
                return '', None
            else:
                return multiCall.addEarlyResult(('', None))
        def handleResult(lazyLabelAndLanguage):
            label, language = lazyLabelAndLanguage()
            if not language:
                language = None
            return iso8859_15(label), language
        return callServer(
            extractServerId(objectId),
            'getObjectLabelAndLanguage',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId],
            resultHandler = handleResult,
            multiCall = multiCall)

    def getObjectLabelsTranslated(
        self, objectIds, destinationLanguages,
        serverId = None):
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getObjectLabelAndLanguage(
                objectId, multiCall = multiCall)
        lazyLabelsAndLanguages = multiCall.call()
        labelsAndLanguages = {}
        for i in range(len(objectIds)):
            try:
                labelsAndLanguages[objectIds[i]] = lazyLabelsAndLanguages[i]()
            except faults.MissingItem:
                pass
        translationsProxy = getProxyForServerRole('translations')
        labels = {}
        if translationsProxy:
            multiCall = MultiCall()
            for objectId, labelAndLanguage in labelsAndLanguages.items():
                label, language = labelAndLanguage
                translationsProxy.getTranslation(
                        label, objectId, 'self.getLabel()', language,
                        destinationLanguages, multiCall = multiCall)
            lazyTranslations = multiCall.call()
            objectIds = labelsAndLanguages.keys()
            for i in range(len(objectIds)):
                labels[objectIds[i]] = lazyTranslations[i]()
        else:
            for objectId, labelAndLanguage in labelsAndLanguages.items():
                labels[objectId] = labelAndLanguage[0]
        if commonTools.extractDispatcherId(objectId) \
               != context.getVar('dispatcherId'):
            for id, label in labels.items():
                label += _(' (at <%s>)') \
                         % extractApplicationHostNameAndPort(objectId)
                labels[id] = label
        return labels

    def getObjectLabelTranslated(
        self, objectId, destinationLanguages):
        if not objectId:
            return ''
        label, language = self.getObjectLabelAndLanguage(objectId)
        translationsProxy = getProxyForServerRole('translations')
        if translationsProxy:
            labelTranslated = translationsProxy.getTranslation(
                label, objectId, 'self.getLabel()', language,
                destinationLanguages )
        else:
            labelTranslated = label
        if commonTools.extractDispatcherId(objectId) \
               != context.getVar('dispatcherId'):
            labelTranslated += _(' (at <%s>)') \
                               % extractApplicationHostNameAndPort(objectId)
        return labelTranslated

    def getObjects(self, requiredSlotNames = None, serverId = None):
        objectIds = self.getObjectIds(serverId = serverId)
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getPartialObject(
                objectId, requiredSlotNames, multiCall = multiCall)
        lazyObjects = multiCall.call()
        objects = {}
        for i in range(len(objectIds)):
            objects[objectIds[i]] = lazyObjects[i]()
        return objects

    def getObjectStringFromDigest(self, objectId, path, digest):
        userToken = context.getVar('userToken', default = '')
        return iso8859_15(callServer(
            extractServerId(objectId),
            'getObjectStringFromDigest',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, utf8(path), digest]))

    def getPartialObject(self, objectId, requiredSlotNames, multiCall = None):
        if not requiredSlotNames:
            return self.getObject(objectId, multiCall = multiCall)
        cacheKey = requiredSlotNames[:]
        cacheKey.sort()
        cacheKey = '%s_%s' % (objectId, str(cacheKey)) 
        def handleResult(lazyObject, self = self, cacheKey = cacheKey):
#        def handleResult(lazyObject, self = self):
            objectImport = lazyObject()
            object = self.newThing('object', objectImport['__thingName__'])
            object.importFromXmlRpc(objectImport)
            cache = context.getVar('cache')
            if cache is not None and object.canCache():
                cache[cacheKey] = lazyObject
            return object
        cache = context.getVar('cache')
        if cache is not None:
            try:
                result = handleResult(cache[cacheKey])
                if multiCall:
                    multiCall.addEarlyResult(result)
                    return None
                else:
                    return result
            except KeyError:
                pass
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'getPartialObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, requiredSlotNames],
            resultHandler = handleResult,
            multiCall = multiCall)

    def hasObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'hasObject',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def isObjectId(self, id):
        return id.find(self.serverId + '/') == 0

    def modifyObject(self, object):
        userToken = context.getVar('userToken', default = '')
        objectExport = object.exportToXmlRpc()
        object.version = callServer(
            extractServerId(object.id),
            'modifyObject',
            [extractServerId(object.id), getApplicationToken(), userToken,
             objectExport])
        return None

    def modifyPartialObject(self, object, givenSlotNames):
        if not givenSlotNames:
            return self.modifyObject(object)
        givenSlotNames = givenSlotNames[:]
        if not 'id' in givenSlotNames \
           and object.isSlotName('id') \
           and object.getSlot('id').getKind().isPublicForXmlRpc:
            assert hasattr(object, 'id') and object.id is not None
            givenSlotNames.append('id')
        if not 'version' in givenSlotNames \
           and object.isSlotName('version') \
           and object.getSlot('version').getKind().isPublicForXmlRpc:
            assert hasattr(object, 'version') and object.version is not None
            givenSlotNames.append('version')
        userToken = context.getVar('userToken', default = '')
        objectExport = object.exportToXmlRpc(
            requiredSlotNames = givenSlotNames)
        object.version = callServer(
            extractServerId(object.id),
            'modifyPartialObject',
            [extractServerId(object.id), getApplicationToken(), userToken,
             objectExport, givenSlotNames])
        return None

    def newObject(self, fields):
        return self.newThing('object', self.objectClassName)

    def searchObjectIds(
        self, serverId = None, multiCall = None, **searchKeywords):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        searchDict = {}
        for sk in searchKeywords.keys():
            searchDict[sk] = searchKeywords[sk]
        if not searchDict:
            return []
        return callServer(
            serverId,
            'searchObjectIds',
            [serverId, getApplicationToken(), userToken, searchDict],
             multiCall = multiCall)

    def sortObjectIds(
        self, objectIds, serverId = None, sortRule = None, forceIds = 0):
        """ sortRule in ['modificationTime', 'id', 'date', '']
        """
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        if not objectIds and not forceIds:
            return []
        if sortRule is None:
            sortRule = ""
        return callServer(
            serverId,
            'sortObjectIds',
            [serverId, getApplicationToken(), userToken,
             objectIds, sortRule])

    def subscribe(self, method, when, callbackName, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'subscribe',
            [serverId, getApplicationToken(), userToken, method, when,
             callbackName])

    def unsubscribe(self, method = '_ALL_', when= 'any ', callbackName = 'any',
                    serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'unsubscribe',
            [serverId, getApplicationToken(), userToken, method, when,
             callbackName])
