# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Preferences Proxy"""

__version__ = '$Revision: 1.16 $'[11:-2]


from glasnost.common.PreferencesCommon import *
import glasnost.common.context as context

from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import register, ObjectProxyMixin, Proxy


class Preference(ObjectProxyMixin, PreferenceCommon):
    def getProxy(self):
        return preferencesProxy
register(Preference)


class PreferencesProxy(PreferencesCommon, Proxy):
    def deletePreference(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        callServer(
            serverId,
            'deleteObject',
            [serverId, getApplicationToken(), userToken])

    def getPreference(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        objectImport = callServer(
            serverId,
            'getObject',
            [serverId, getApplicationToken(), userToken])
        object = self.newThing('object', objectImport['__thingName__'])
        object.importFromXmlRpc(objectImport)
        return object

    def getPreferenceByUserId(self, userId, serverId = None):
        serverId = self.getServerId(serverId = serverId)
        objectImport = callServer(
            serverId,
            'getObjectByUserId',
            [serverId, getApplicationToken(), userId])
        object = self.newThing('object', objectImport['__thingName__'])
        object.importFromXmlRpc(objectImport)
        return object

    def rememberId(self, id, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        callServer(
            serverId,
            'rememberId',
            [serverId, getApplicationToken(), userToken, id])
        return None
    
    def setPreference(self, object, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        objectExport = object.exportToXmlRpc()
        object.version = callServer(
            serverId,
            'setObject',
            [serverId, getApplicationToken(), userToken, objectExport])
        return None

