# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.org>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@easter-eggs.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Records Proxy"""

__version__ = '$Revision: 1.20 $'[11:-2]


from glasnost.common.RecordsCommon import *
import glasnost.common.context as context

from DispatcherProxy import callServer, getApplicationToken
from EventsProxy import Event
from ObjectsProxy import register, AdminMixin, ObjectProxyMixin, ObjectsProxy
from tools import *


class AdminRecords(AdminMixin, AdminRecordsCommon):
    def getProxy(self):
        return recordsProxy
register(AdminRecords)


class Record(ObjectProxyMixin, RecordCommon):
    def getProxy(self):
        return recordsProxy
register(Record)


class RecordsProxy(RecordsCommon, ObjectsProxy):
    def computeObject(self, objectId):
        userToken = context.getVar('userToken', default = '')
        callServer(
            extractServerId(objectId),
            'computeObject',
            [ extractServerId(objectId), getApplicationToken(), userToken,
             objectId ])
        return None

    def searchObjectIds(
        self, accountsSet, startDate = None, endDate = None,
        certaintySet = [], serverId = None):
        serverId = self.getServerId(serverId = serverId)
        if not accountsSet: return []
        if type(accountsSet) == type(''):
            accountsSet = [ accountsSet ]
        if not startDate: startDate = ''
        if not endDate: endDate = ''
        if startDate > endDate:
            return []
        if not certaintySet:
            from glasnost.common.accounting import certaintyValues
            certaintySet = certaintyValues
        elif type(certaintySet) == type(''):
            certaintySet = [ certaintySet ]
        searchKeywords = {}
        searchKeywords["accountId_IN"] = accountsSet
        if startDate:
            searchKeywords["date_GE"] = startDate
        if endDate:
            searchKeywords["date_LE"] = endDate
        if certaintySet:
            searchKeywords["certainty_IN"] = certaintySet 
        objectIds = ObjectsProxy.searchObjectIds(
            self,
            serverId = serverId,
            **searchKeywords)
        objectIds = ObjectsProxy.sortObjectIds(
            self, objectIds, serverId = serverId,
            sortRule = "date")
        objectIds = objectIds[:50]
        return objectIds

