# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Sessions Proxy"""

__version__ = '$Revision: 1.9 $'[11:-2]


from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import Proxy


class SessionsProxy(Proxy):
    serverRole = 'sessions'

    def deleteSession(self, objectToken, serverId = None):
        serverId = self.getServerId(serverId = serverId)
        callServer(
            serverId,
            'deleteObject',
            [serverId, getApplicationToken(), objectToken])

    def getSession(self, objectToken, serverId = None):
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'getObject',
            [serverId, getApplicationToken(), objectToken])

    def newSession(self, serverId = None):
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'newObject',
            [serverId, getApplicationToken()])

    def setSession(self, object, serverId = None):
        # The isDirty flag is used to remove conflicts when a page embeds
        # another object (like an image).
        # Whithout isDirty flag, when editing, this can produce something like:
        # s1 = getSession <- beginning of page handling
        # s2 = getSession <- beginning of image handling
        # modification of s1
        # setSession(s1) <- end of page handling
        # setSession(s2) <- end of image handling => destroys s1 changes!
        if not object.has_key('isDirty'):
            return
        del object['isDirty']
        serverId = self.getServerId(serverId = serverId)
        object['version'] = callServer(
            serverId,
            'setObject',
            [serverId, getApplicationToken(), object])
        return None

