# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Translations Proxy"""

__version__ = '$Revision: 1.18 $'[11:-2]


import md5

import glasnost.common.context as context
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools
from glasnost.common.TranslationsCommon import *

from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import register, AdminWithoutWritersMixin, AdministrableProxyMixin, ObjectProxyMixin, Proxy
from tools import *


class AdminTranslations(AdminWithoutWritersMixin, AdminTranslationsCommon):
    def getProxy(self):
        return translationsProxy
register(AdminTranslations)


class Localization(ObjectProxyMixin, LocalizationCommon):
    def getProxy(self):
        return translationsProxy
register(Localization)


class TranslationsProxy(TranslationsCommon, AdministrableProxyMixin, Proxy):
    def canDeleteLocalization(self, localizationKey, sourceStringDigest,
                              serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canDeleteLocalization',
            [serverId, getApplicationToken(), userToken, localizationKey,
             sourceStringDigest])

    def canModifyLocalization(self, localizationKey, sourceStringDigest,
                              serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'canModifyLocalization',
            [serverId, getApplicationToken(), userToken, localizationKey,
             sourceStringDigest])

    def getLastDigestsAndLabels(self, localizationKey, digestsAndLabelsCount,
                                possibleStates, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        if possibleStates is None:
            possibleStates = []
        digestsAndLabels = callServer(
            serverId,
            'getLastDigestsAndLabels',
            [serverId, getApplicationToken(), userToken,
             localizationKey, digestsAndLabelsCount, possibleStates])
        for digestAndLabel in digestsAndLabels:
            digestAndLabel[1] = iso8859_15(digestAndLabel[1])
        return digestsAndLabels

    def getLocalization(self, localizationKey, sourceStringDigest,
                        serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        localizationImport = callServer(
            serverId,
            'getLocalization',
            [serverId, getApplicationToken(), userToken,
             localizationKey, sourceStringDigest])
        localization = self.newThing(
            'object', localizationImport['__thingName__'])
        localization.importFromXmlRpc(localizationImport)
        return localization

    def getTranslation(self, sourceString, sourceId, sourcePath,
                       sourceLanguage, destinationLanguages, multiCall = None):
        if not sourceString:
            if multiCall:
                multiCall.addEarlyResult('')
                return None
            else:
                return ''
        if not sourceId or not sourceLanguage:
            if multiCall:
                multiCall.addEarlyResult(sourceString)
                return None
            else:
                return sourceString
        sourceStringDigest = md5.new(sourceString.replace(
            '\r\n', '\n')).hexdigest()
        cacheKey = '_' .join([sourceStringDigest,
                              sourceLanguage,
                              str(destinationLanguages)])
        def handleResult(lazyTranslation, sourceString = sourceString,
                         cacheKey = cacheKey):
#        def handleResult(lazyTranslation, sourceString = sourceString):
            try:
                result = iso8859_15(lazyTranslation())
                cache = context.getVar('cache')
                if cache is not None:
                    cache[cacheKey] = lazyTranslation
                return result
            except faults.UnknownStringDigest:
                raise faults.UnknownStringDigest(sourceString)
        cache = context.getVar('cache')
        if cache is not None:
            try:
                result = handleResult(cache[cacheKey])
                if multiCall:
                    multiCall.addEarlyResult(result)
                    return None
                else:
                    return result
            except:
                pass
        userToken = context.getVar('userToken', default = '')
        serverId = commonTools.makeApplicationId(sourceId, self.serverRole)
        return callServer(
            serverId,
            'getTranslation',
            [serverId, getApplicationToken(), userToken, sourceStringDigest,
             sourceId, sourcePath, sourceLanguage, destinationLanguages],
            resultHandler = handleResult,
            multiCall = multiCall)

    def getTranslationInfos(self, sourceString, sourceId, sourcePath,
                            sourceLanguage, destinationLanguages,
                            ignoreNew = 0, multiCall = None):
        assert sourceId
        assert sourceLanguage
        sourceStringDigest = md5.new(sourceString.replace(
            '\r\n', '\n')).hexdigest()
        def handleResult(lazyInfos, sourceString = sourceString):
            try:
                infos = lazyInfos()
            except faults.UnknownStringDigest:
                raise faults.UnknownStringDigest(sourceString)
            infos[0] = iso8859_15(infos[0])
            return infos
        userToken = context.getVar('userToken', default = '')
        serverId = commonTools.makeApplicationId(sourceId, self.serverRole)
        return callServer(
            serverId,
            'getTranslationInfos',
            [serverId, getApplicationToken(), userToken, sourceStringDigest,
             sourceId, sourcePath, sourceLanguage, destinationLanguages,
             ignoreNew],
            resultHandler = handleResult,
            multiCall = multiCall)

    def getTranslatorLocalizationKeys(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'getTranslatorLocalizationKeys',
            [serverId, getApplicationToken(), userToken])

    def hasLocalization(self, localizationKey, sourceStringDigest,
                        serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'hasLocalization',
            [serverId, getApplicationToken(), userToken, localizationKey,
             sourceStringDigest])

    def hasSourceString(self, localizationKey, sourceStringDigest,
                        serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'hasSourceString',
            [serverId, getApplicationToken(), userToken, localizationKey,
             sourceStringDigest])

    def modifyLocalization(self, localization, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        localizationExport = localization.exportToXmlRpc()
        localization.version = callServer(
            serverId,
            'modifyLocalization',
            [serverId, getApplicationToken(), userToken,
             localizationExport])
        return None

