# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Votes Proxy"""

__version__ = '$Revision: 1.15 $'[11:-2]


from glasnost.common.VotesCommon import *

from DispatcherProxy import callServer, getApplicationToken
from ObjectsProxy import register, AdminWithoutWritersMixin, ObjectProxyMixin, ObjectsProxy
from tools import *


class AdminVotes(AdminWithoutWritersMixin, AdminVotesCommon):
    def getProxy(self):
        return votesProxy
register(AdminVotes)


class VoteMixin(ObjectProxyMixin):
    def getProxy(self):
        return votesProxy


class AbstractVote(VoteMixin, AbstractVoteCommon):
    pass
register(AbstractVote)


class VoteApproval(VoteMixin, VoteApprovalCommon):
    pass
register(VoteApproval)


class VoteDistribution(VoteMixin, VoteDistributionCommon):
    pass
register(VoteDistribution)


class VoteRanking(VoteMixin, VoteRankingCommon):
    pass
register(VoteRanking)


class VoteRating(VoteMixin, VoteRatingCommon):
    pass
register(VoteRating)


class VotesProxy(VotesCommon, ObjectsProxy):
    def addObject(self, object, serverId = None):
        serverId = self.getServerId(serverId = serverId)
        objectExport = object.exportToXmlRpc()
        objectId =  callServer(
            serverId,
            'addObject',
            [serverId, getApplicationToken(), objectExport])
        return objectId

    def deleteObject(self, objectId):
        callServer(
            extractServerId(objectId),
            'deleteObject',
            [extractServerId(objectId), getApplicationToken(),
             objectId])

    def getObject(self, objectId):
        objectImport = callServer(
            extractServerId(objectId),
            'getObject',
            [extractServerId(objectId), getApplicationToken(),
             objectId])
        object = self.newThing('object', objectImport['__thingName__'])
        object.importFromXmlRpc(objectImport)
        return object

    def getPartialObject(self, objectId, requiredSlotNames):
        if not requiredSlotNames:
            return self.getObject(objectId)
        objectImport = callServer(
            extractServerId(objectId),
            'getPartialObject',
            [extractServerId(objectId), getApplicationToken(),
             objectId, requiredSlotNames])
        object = self.newThing('object', objectImport['__thingName__'])
        object.importFromXmlRpc(objectImport)
        return object

    def hasObject(self, objectId):
        return callServer(
            extractServerId(objectId),
            'hasObject',
            [extractServerId(objectId), getApplicationToken(),
             objectId])

    def modifyObject(self, object):
        objectExport = object.exportToXmlRpc()
        object.version = callServer(
            extractServerId(object.id),
            'modifyObject',
            [extractServerId(object.id),
             getApplicationToken(), objectExport])
        return None

