# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Workflows Proxy"""

__version__ = '$Revision: 1.18 $'[11:-2]


from glasnost.common.WorkflowsCommon import *
import glasnost.common.context as context

from DispatcherProxy import MultiCall, callServer, getApplicationToken
from ObjectsProxy import register, AdminMixin, ObjectsProxy
from tools import *
import kinds


class AdminWorkflows(AdminMixin, AdminWorkflowsCommon):
    def getProxy(self):
        return workflowsProxy
register(AdminWorkflows)


class WorkflowAction(WorkflowActionCommon):
    pass
register(WorkflowAction)

class WorkflowActionShowMessage(WorkflowActionShowMessageCommon):
    pass
register(WorkflowActionShowMessage)

class WorkflowActionUserChoice(WorkflowActionUserChoiceCommon):
    pass
register(WorkflowActionUserChoice)

class WorkflowActionAddObject(WorkflowActionAddObjectCommon):
    pass
register(WorkflowActionAddObject)

class WorkflowActionEditObject(WorkflowActionEditObjectCommon):
    pass
register(WorkflowActionEditObject)

class WorkflowActionMail(WorkflowActionMailCommon):
    pass
register(WorkflowActionMail)

class WorkflowActionEnd(WorkflowActionEndCommon):
    pass
register(WorkflowActionEnd)

class WorkflowActionUserData(WorkflowActionUserDataCommon):
    pass
register(WorkflowActionUserData)

class WorkflowActionBranch(WorkflowActionBranchCommon):
    pass
register(WorkflowActionBranch)

class WorkflowActionGoto(WorkflowActionGotoCommon):
    pass
register(WorkflowActionGoto)

class Workflow(WorkflowCommon):
    pass
register(Workflow)

class RunningWorkflow(RunningWorkflowCommon):
    pass
register(RunningWorkflow)


class WorkflowsProxy(WorkflowsCommon, ObjectsProxy):
    def canRunWorkflow(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'canRunWorkflow',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def initWorkflow(self, objectId, dataObjectId = ''):
        userToken = context.getVar('userToken', default = '')
        callServer(
            extractServerId(objectId),
            'initWorkflow',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, dataObjectId])

    def getRunningWorkflow(self, objectId):
        userToken = context.getVar('userToken', default = '')
        objectImport = callServer(
            extractServerId(objectId),
            'getRunningWorkflow',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])
        object = self.newThing('object', objectImport['__thingName__'])
        object.importFromXmlRpc(objectImport)
        return object

    def advanceRunningWorkflow(self, objectId, data = None):
        userToken = context.getVar('userToken', default = '')
        if not data:
            data = ''
        dataXmlRpc = data.exportToXmlRpc()
        return callServer(
            extractServerId(objectId),
            'advanceRunningWorkflow',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId, dataXmlRpc])

    def isFinished(self, objectId):
        userToken = context.getVar('userToken', default = '')
        return callServer(
            extractServerId(objectId),
            'isFinished',
            [extractServerId(objectId), getApplicationToken(), userToken,
             objectId])

    def getRunningWorkflowsIdsForUser(self, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'getRunningWorkflowsIdsForUser',
            [serverId, getApplicationToken(), userToken])

    def getWorkflowsIdsForServerRole(self, serverRole, serverId = None):
        userToken = context.getVar('userToken', default = '')
        serverId = self.getServerId(serverId = serverId)
        return callServer(
            serverId,
            'getWorkflowsIdsForServerRole',
            [serverId, getApplicationToken(), userToken, serverRole])

    def getWorkflowsForServerRole(self, serverRole, serverId = None):
        objectIds = self.getWorkflowsIdsForServerRole(
            serverRole, serverId = serverId)
        multiCall = MultiCall()
        for objectId in objectIds:
            self.getPartialObject(
                objectId, None, multiCall = multiCall)
        lazyObjects = multiCall.call()
        objects = {}
        for i in range(len(objectIds)):
            objects[objectIds[i]] = lazyObjects[i]()
        return objects

