# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Proxy Tools"""

__version__ = '$Revision: 1.19 $'[11:-2]


import glasnost.proxy

from glasnost.common.tools import *
import glasnost.common.context as context

import imp
import sys
import fnmatch
import os

_cachedProxiesByServerRole = {}


def getObjectLabelsTranslated(ids, destinationLanguages):
    handledRoles = context.getVar('knownRoles')
    labels = {}
    remainingIds = ids[:]
    while remainingIds:
        id = remainingIds[0]
        del remainingIds[0]
        serverHostNameAndPort, serverRole, localId = splitId(id)
        proxyIds = [id]
        remainingIds2 = remainingIds[:]
        for id2 in remainingIds2:
            serverHostNameAndPort2, serverRole2, localId2 = splitId(id2)
            if serverHostNameAndPort2 == serverHostNameAndPort \
               and serverRole2 == serverRole:
                proxyIds.append(id2)
                del remainingIds[remainingIds.index(id2)]
        proxy = getProxyForServerRole(serverRole)
        if proxy is None:
            continue
        serverId = extractServerId(id)
        labels.update(proxy.getObjectLabelsTranslated(
            proxyIds, destinationLanguages, serverId = serverId))
    return labels


def getObjectLabelTranslated(objectId, destinationLanguages):
    return getProxy(objectId).getObjectLabelTranslated(
        objectId, destinationLanguages)


def getObject(objectId):
    proxy = getProxy(objectId)
    return proxy.getObject(objectId)


def getProxy(id):
    return getProxyForServerRole(extractServerRole(id))


def getProxyForServerRole(serverRole):
    if context.getVar('knownRoles') and \
            serverRole not in context.getVar('knownRoles'):
        return None
    if not _cachedProxiesByServerRole.has_key(serverRole):
        import tools_new as proxyTools
        baseProxies = proxyTools.getAllBaseProxies()
        for baseProxy in baseProxies.values():
            if serverRole == baseProxy.serverRole:
                proxy = baseProxy
                break
        else:
            # Two passes so we don't call filesObjectProxy if we don't need to.
            for baseProxy in baseProxies.values():
                handledRoles = baseProxy.getHandledRoles()
                if handledRoles is not None and serverRole in handledRoles:
                    proxy = baseProxy.duplicateForRole(serverRole)
                    break
            else:
                proxy = None
                try:
                    proxyName = '%s%sProxy' % (serverRole[0].upper(), serverRole[1:])
                    if sys.modules.has_key('glasnost.proxy.%s' % proxyName):
                        mod = sys.modules['glasnost.proxy.%s' % proxyName]
                    else:
                        try:
                            mod = imp.find_module(proxyName,
                                                glasnost.proxy.__path__)
                        except ImportError:
                            # probably caused by case mismatch between filename
                            # and serverRole
                            files = [x for x in os.listdir(glasnost.proxy.__path__[0]) \
                                     if fnmatch.fnmatch(x.lower(),
                                                '%s.py' % proxyName.lower())]
                            if len(files) != 1:
                                raise
                            proxyName = files[0][:-3]
                            mod = imp.find_module(proxyName,
                                                glasnost.proxy.__path__)
                            
                        mod = imp.load_module('glasnost.proxy.%s' % proxyName,
                                            mod[0], mod[1], mod[2])
                    proxy = mod.__dict__[proxyName]()
                except ImportError:
                    pass
                except KeyError:
                    pass

        _cachedProxiesByServerRole[serverRole] = proxy
    return _cachedProxiesByServerRole[serverRole]


def sortIds(ids):
    if ids is None:
        return []
    idsByServer = {}
    for id in ids:
        serverHostNameAndPort, serverRole, localId = splitId(id)
        serverId = extractServerId(id)
        if not idsByServer.has_key(serverId):
            proxy = getProxyForServerRole(serverRole)
            if not proxy:
                raise Exception('Unknown server role = %s for id = %s' % (
                    serverRole, id))
            idsByServer[serverId] = []
        idsByServer[serverId].append(id)
    serverIds = idsByServer.keys()
    # FIXME: Change the sort key.
    serverIds.sort()
    result = []
    for serverId in serverIds:
        serverHostNameAndPort, serverRole = splitApplicationId(serverId)
        proxy = getProxyForServerRole(serverRole)
        ids = idsByServer[serverId]
        sortedIds = proxy.sortObjectIds(ids, serverId = serverId, forceIds = 1)
        result += sortedIds
    return result
