# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Server Properties"""

__version__ = '$Revision: 1.8 $'[11:-2]


import glasnost.common.properties as commonProperties

import things


class PropertiesContainerMixin:
    def getModifySlotNames(self, parentSlot = None):
        names = self.inherited(PropertiesContainerMixin, 'getModifySlotNames')(
            self, parentSlot = parentSlot)
        names = names[:]
        if 'properties' in names:
            # The slot 'properties' is already mofified by the method
            # modify.
            names.remove('properties')
        return names

    def modify(self, changes, givenSlotNames = None):
        # The slot 'properties' must be imported first, because it is needed to
        # compute the slotNames list.
        for slotName in ['properties']:
            slot = self.getSlot(slotName)
            kind = slot.getKind()
            if kind.isAutomaticalyModified:
                kind.setAutomaticalValue(slot)
                continue
            if not kind.hasToModify:
                continue
            if givenSlotNames and not slotName in givenSlotNames:
                continue
            changesSlot = changes.getSlot(slotName)
            slot.setValue(changesSlot.getValue())
        self.inherited(PropertiesContainerMixin, 'modify')(
            self, changes, givenSlotNames = givenSlotNames)

class Property(things.ThingMixin, commonProperties.Property):
    pass
things.register(Property)
