# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Server Things"""

__version__ = '$Revision: 1.9 $'[11:-2]


import glasnost.common.things as commonThings


allThingClasses = {}


def register(thingClass):
    thingName = thingClass.getThingName.im_func(thingClass)
    if not allThingClasses.has_key(thingClass.thingCategory):
        allThingClasses[thingClass.thingCategory] = {}
    assert not allThingClasses[thingClass.thingCategory].has_key(thingName)
    allThingClasses[thingClass.thingCategory][thingName] = thingClass


class ThingMixin:
    def acquireNonCore(self, objectDirectoryPath, parentSlot = None):
        for slotName in self.getSlotNames(parentSlot = parentSlot):
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.useFileStorage:
                kind.loadValueFromFile(slot, objectDirectoryPath)
            else:
                kind.acquireNonCoreValue(slot, objectDirectoryPath)

    def getAllThingClasses(self):
        # Should be converted to a class method, when porting Glasnost to
        # Python 2.2.
        return allThingClasses
        
    def releaseNonCore(self, parentSlot = None):
        for slotName in self.getSlotNames(parentSlot = parentSlot):
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.useFileStorage:
                slot.delValue()
            else:
                kind.releaseNonCoreValue(slot)

    def saveNonCore(self, objectDirectoryPath, parentSlot = None):
        for slotName in self.getSlotNames(parentSlot = parentSlot):
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.useFileStorage:
                kind.saveValueToFile(slot, objectDirectoryPath)
            else:
                kind.saveNonCoreValue(slot, objectDirectoryPath)


class BaseThing(ThingMixin, commonThings.BaseThing):
    pass
