# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Accounts Web"""

__version__ = '$Revision: 1.31 $'[11:-2]


import glasnost.common.faults as faults
from glasnost.common.tools import extractLocalId
import glasnost.common.context as context

from glasnost.proxy.AccountsProxy import *

import widgets
from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import accounting


class AccountRecordsWidget(widgets.Multi):
    thingName = 'AccountRecords'

    def getHtmlField(self, slot, fields, **keywords):
        assert not self.isInForm()
        assert fields is not None
        count = 10
        certaintyLevel = 4
        displayedItems = []
        for member in slot.getValue():
            if member[2] >= certaintyLevel:
                displayedItems.append(member)
            if len(displayedItems) == 10:
                break
        count = len(displayedItems)
        if count == 0:
            return ''
        serverId = extractServerId(fields['id'])
        serverId = serverId.replace('accounts', 'records')
        recordsWeb = getWebForServerRole('records')
        for item in displayedItems:
            itemId = serverId + '/' + str(item[0])
            item.append(itemId)
            itemObject = recordsWeb.getPartialObject(
                itemId, ['eventId', 'label'])
            item.append(itemObject.eventId)
            item.append(itemObject.label)

        layout = tr = X.tr(_class = 'field-value')
        td = X.td(colspan = '2')
        td += X.h3()(_('Last Records'))
        layout += td
        table = X.table(_class = 'objects-table')
        td += table
        td += X.br()

        thead = X.thead()
        table += thead
        tr = X.tr()
        thead += tr
        tr += X.th(scope = 'col')(_('Record'))
        tr += X.th(scope = 'col')(_('Date'))
        tr += X.th(scope = 'col')(_('Label'))
        tr += X.th(scope = 'col')(_('Debit'))
        tr += X.th(scope = 'col')(_('Credit'))
        tbody = X.tbody()
        table += tbody
        for item in displayedItems:
            itemId = item[4]
            itemLocalId = item[0]
            itemDate = item[1]
            itemEvent = item[5]
            itemLabel = item[6]
            itemAmount = item[3]
            tr = X.tr()
            tbody += tr
            tr += X.td(_class = 'column-label')(
                X.a(href = X.url(itemId))(
                itemLocalId))
            tr += X.td(_class = 'column-label')(
                X.a(href = X.url(itemEvent))(
                accounting.dateDisplay(itemDate)))
            tr += X.td(_class = 'column-label')(
                X.a(href = X.url(itemEvent))(
                itemLabel))
            if itemAmount < 0:
                tr += X.td(_class='column-value', align='right')(
                    X.a(href = X.url(itemId))(
                    accounting.amountDisplay(- itemAmount, withSide = 0)))
                tr += X.td(_class='column-value')(X.nbsp)
            else:
                tr += X.td(_class='column-value')(X.nbsp)   
                tr += X.td(_class='column-value', align='right')(
                    X.a(href = X.url(itemId))(
                    accounting.amountDisplay(itemAmount, withSide = 0)))
        return layout
widgets.register(AccountRecordsWidget)        
        
        
class AdminAccounts(AdminMixin, AdminAccounts):
    def getWeb(self):
        return getWebForServerRole('accounts')
register(AdminAccounts)


class Account(ObjectWebMixin, Account):
    balance_kind_hasToSubmitField = 0
    balance_kind_isReadOnly = 1
    balance_kind_widget_fieldLabel = N_('Balance')
    balance_kind_widget_sideLabels = accounting.sideLabels
    balance_kind_widgetName = 'Amount'

    currency_kind_hasToSubmitField = 0
    currency_kind_isReadOnly = 1
    currency_kind_preference = 'currency'
    currency_kind_widget_fieldLabel = N_('Currency')
    currency_kind_widget_labels = accounting.currencyLabels
    currency_kind_widgetName = 'Select'

    label_kind_widget_fieldLabel = N_('Label')
    label_kind_widget_size = 40
    label_kind_widgetName = 'InputText'

    ledgerId_kind_widget_fieldLabel = N_('Ledger')
    ledgerId_kind_widgetName = 'SelectId'

    members_kind_hasToSubmitField = 0
    members_kind_isReadOnly = 1
    members_kind_widget_fieldLabel = N_('Last Records')
    members_kind_widgetName = 'AccountRecords'

    number_kind_widget_fieldLabel = N_('Number')
    number_kind_widget_size = 40
    number_kind_widgetName = 'InputText'

    startDate_kind_isReadOnly = 1
    startDate_kind_hasToSubmitField = 0
    startDate_kind_widget_fieldLabel = N_('Starting')
    startDate_kind_widgetName = 'Date'


    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in (
            'balance', 'members', 'startDate'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        for slotName in ('startDate'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            slotNames = slotNames[:]
            for slotName in ('readersSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('accounts')
register(Account)


class AccountsWeb(ObjectsWebMixin, AccountsProxy):
    def viewAll(self):
        if not self.canGetObjects():
            return accessForbidden()
        partialObjects = self.getObjects(['label', 'number'])
        layout = X.array()
        ids = self.getSortedIds(partialObjects)
        layout += self.getObjectsLayout(partialObjects, ids)
        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _(self.objectsNameCapitalized))
    viewAll.isPublicForWeb = 1

