# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Books Web"""

__version__ = '$Revision: 1.31 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.context as context

from glasnost.proxy.BooksProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *


class AdminBooks(AdminMixin, AdminBooks):
    def getWeb(self):
        return getWebForServerRole('books')
register(AdminBooks)


class Book(ObjectWebMixin, Book):
    appendixIds_kind_itemKind_value_widgetName = 'SelectId'
    appendixIds_kind_widget_fieldLabel = N_('Appendices')
    appendixIds_kind_widgetName = 'Multi'

    body_kind_isReadOnly = 1
    body_kind_widget_fieldLabel = N_('Text')
    body_kind_widget_cols = 75
    body_kind_widget_colSpan = 2
    body_kind_widget_format = 'docbook'
    body_kind_widget_rows = 20
    body_kind_widgetName = 'TextArea'

    chapterIds_kind_itemKind_value_widgetName = 'SelectId'
    chapterIds_kind_widget_fieldLabel = N_('Chapters')
    chapterIds_kind_widgetName = 'Multi'

    itemIds_kind_itemKind_value_widgetName = 'SelectId'
    itemIds_kind_widget_fieldLabel = N_('Items')
    itemIds_kind_widgetName = 'Multi'

    nonItemIds_kind_itemKind_value_widgetName = 'SelectId'
    nonItemIds_kind_widget_fieldLabel = N_('Non Items')
    nonItemIds_kind_widgetName = 'Multi'

    title_kind_widget_fieldLabel = N_('Title')
    #title_kind_widget_maxLength = 40
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        if 'body' in slotNames:
            slotNames.remove('body')
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'appendixIds', 'authorsSet', 'chapterIds', 'creationTime',
                'editionTime', 'itemIds', 'modificationTime', 'nonItemIds',
                'readersSet', 'title', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('books')
register(Book)


class BooksWeb(ObjectsWebMixin, BooksProxy):
    def getViewNavigationButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        layout += ObjectsWebMixin.getViewNavigationButtonsBarLayout(
            self, object, fields)
        userToken = context.getVar('userToken', default = '')
        if userToken:
            layout += X.buttonStandalone(
                'source',
                X.url('/%s/source' % urlId))
        return layout

    def source(self, id):
        if not self.hasObject(id):
            return pageNotFound()
        book = self.getObject(id)
        body = book.getBody()

        req = context.getVar('req')
        req.content_type = 'application/x-latex'
        setHttpCookie()
        req.send_http_header()
        if req.method == 'HEAD':
            return OK
        req.write(body)

        return OK
    source.isPublicForWeb = 1

    def viewAll(self):
        if not self.canGetObjects():
            return accessForbidden()
        isAdmin = self.isAdmin()
        userId = context.getVar('userId', default = '')
        if userId:
            userSet = [userId]
        else:
            userSet = None

        layout = X.array()
        requiredSlotNames = ['modificationTime', 'title']
        displayedSlotNames = ['modificationTime']

        if userSet:
            lastBooks = self.getLastObjects(
                10, None, None, userSet, requiredSlotNames)
            layout += self.getObjectsSectionLayout(
                lastBooks,
                _("""Your last books"""),
                displayedSlotNames)

        lastBooks = self.getLastObjects(
            10, None, userSet, None, requiredSlotNames)
        layout += self.getObjectsSectionLayout(
            lastBooks,
            _("""The last books"""),
            displayedSlotNames)

        if isAdmin:
            lastBooks = self.getLastObjects(
                50, None, None, None, requiredSlotNames)
            layout += self.getObjectsSectionLayout(
                lastBooks,
                _("""The last books to administer"""),
                displayedSlotNames)

        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _('Books'))
    viewAll.isPublicForWeb = 1

    def viewSource(self, id):
        if not self.hasObject(id):
            return pageNotFound()
        book = self.getObject(id)
        body = getProxyForServerRole('translations').getTranslation(
            book.getBody(), book.getId(), 'self.body',
            book.getLanguage(), context.getVar('readLanguages'))

        label = book.getLabelTranslated(
            context.getVar('readLanguages'))

        layout = X.array()
        layout += X.pre(_class = 'spip')(body)
        return writePageLayout(layout, _('Source - %s') % label)
    viewSource.isPublicForWeb = 1

