# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Breves Web"""

__version__ = '$Revision: 1.26 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.context as context

from glasnost.proxy.BrevesProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *


class AdminBreves(AdminMixin, AdminBreves):
    def getWeb(self):
        return getWebForServerRole('breves')
register(AdminBreves)


class BreveMixin(ObjectWebMixin):
    body_kind_widget_fieldLabel = N_('Text')
    body_kind_widget_cols = 75
    body_kind_widget_colSpan = 2
    body_kind_widget_rows = 20
    body_kind_widgetName = 'TextArea'

    links_kind_itemKind_value_widgetName = 'Link'
    links_kind_widgetName = 'Multi'
    links_kind_widget_fieldLabel = N_('Links')

    title_kind_widget_fieldLabel = N_('Title')
    #title_kind_widget_maxLength = 40
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'authorsSet', 'creationTime', 'modificationTime',
                'readersSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('breves')


class Breve(BreveMixin, Breve):
    pass
register(Breve)


class BreveElectionMixin(BreveMixin):
    def getLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ('electionId', 'electionSubject', 'electionTitle'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames


class BreveElectionCandidateAdded(BreveElectionMixin,
                                  BreveElectionCandidateAdded):
    pass
register(BreveElectionCandidateAdded)


class BreveElectionClosed(BreveElectionMixin,
                          BreveElectionClosed):
    pass
register(BreveElectionClosed)


class BreveElectionRunning(BreveElectionMixin,
                           BreveElectionRunning):
    pass
register(BreveElectionRunning)


class BrevesWeb(ObjectsWebMixin, BrevesProxy):
    def viewAll(self):
        if not self.canGetObjects():
            return accessForbidden()
        isAdmin = self.isAdmin()
        userId = context.getVar('userId', default = '')
        if userId:
            userSet = [userId]
        else:
            userSet = None

        layout = X.array()
        requiredSlotNames = ['electionTitle', 'modificationTime', 'title']
        displayedSlotNames = ['modificationTime']

        if userSet:
            lastBreves = self.getLastObjects(
                10, None, None, userSet, requiredSlotNames)
            layout += self.getObjectsSectionLayout(
                lastBreves,
                _("""Your last short news"""),
                displayedSlotNames)

        lastBreves = self.getLastObjects(
            10, None, userSet, None, requiredSlotNames)
        layout += self.getObjectsSectionLayout(
            lastBreves,
            _("""The last short news"""),
            displayedSlotNames)

        if isAdmin:
            lastBreves = self.getLastObjects(
                50, None, None, None, requiredSlotNames)
            layout += self.getObjectsSectionLayout(
                lastBreves,
                _("""The last short news to administer"""),
                displayedSlotNames)

        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _('Short News'))
    viewAll.isPublicForWeb = 1

