# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Cards Web"""

__version__ = '$Revision: 1.35 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots

from glasnost.proxy.CardsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
import properties
from tools import *


class AdminCards(AdminMixin, AdminCards):
    pass
register(AdminCards)


class AbstractCard(ObjectWebMixin, properties.PropertiesContainerMixin, Card):
    prototypeIds_kind_isConstant = 1
    prototypeIds_kind_isHidden = 1
    prototypeIds_kind_widget_fieldLabel = N_('Prototypes')

    def getWeb(self):
        return getWebForServerRole('cards')
        

class Card(AbstractCard):
    def getSlotToModifyNames(self, parentSlot = None):
        return self.getPropertyNames()
register(Card)


class CardAsDefinition(AbstractCard):
    properties_kind_isConstant = 0
    properties_kind_isHidden = 0

    prototypeIds_kind_isConstant = 0
    prototypeIds_kind_isHidden = 0

    def getExportClassName(self):
        return self.getWeb().objectClassName

    def getSlotNames(self, parentSlot = None):
        from glasnost.common.ObjectsCommon import ObjectCommon
        return ObjectCommon.getSlotNames(self, parentSlot = parentSlot)

    def getSlotToModifyNames(self, parentSlot = None):
        return self.getSlotNames(parentSlot = parentSlot)


class CardsWeb(ObjectsWebMixin, CardsProxy):
    def definition(self, id):
        urlId = getUrlId(id)
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canGetObject(id):
            return accessForbidden()
        object = self.getObject(id)
        object.__class__ = CardAsDefinition # Important!
        rememberObject(id)

        keywords = {}
        object.makeFieldsFromInstance(keywords)
        object.repairFields(keywords)

        label = object.getLabelTranslated(
            context.getVar('readLanguages'))

        layout = X.array()
        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(0)
        layout += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        layout += buttonsBar
        buttonsBar += X.span(_class = 'navigation-buttons-bar')(
            X.buttonStandalone( 'view-utilization', X.url('/%s' % urlId)))
        if self.canModifyObject(object.id):
            buttonsBar += X.span(_class = 'action-buttons-bar')(
                X.buttonStandalone(
                    'edit', X.url('/%s/editDefinition' % urlId)))

        context.push(
            currentObject = WebAPI.GlasnostObject(object = object),
            )
        resultLayout = writePageLayout(
            layout, _('Definition of Card "%s"') % object.getLabel())
        context.pull()
        return resultLayout
    definition.isPublicForWeb = 1

    def editDefinition(self, id, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canModifyObject(id):
            return accessForbidden()
        object = self.getObject(id)
        object.__class__ = CardAsDefinition # Important!
        rememberObject(id)

        if not again:
            object.makeFieldsFromInstance(keywords)
        object.repairFields(keywords)

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submitDefinition' % self.serverRole),
                      enctype = 'multipart/form-data', method = 'post')
        layout += form

        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
        form += X.div(_class = 'buttons-bar')(
            X.span(_class = 'action-buttons-bar')(
                X.buttonInForm('modify', 'modifyButton')))
        return writePageLayout(
            layout, _('Editing Definition of Card "%s"') % object.getLabel())
    editDefinition.isPublicForWeb = 1

    def editType(self, id, propertyName, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        keywords['propertyName'] = propertyName
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canModifyObject(id):
            return accessForbidden()
        object = self.getObject(id)
        rememberObject(id)

        if not propertyName in object.getPropertyNames():
            return pageNotFound()
        propertySlot = object.getPropertySlot(propertyName)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)

        # We have to generate the fields for the whole property and not only
        # property.kind, because some fields of property are used by its kind.
        property.makeFieldsFromInstance(keywords, parentSlot = propertySlot)
        property.repairFields(keywords, parentSlot = propertySlot)

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submitType' % self.serverRole),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form
        form += X.input(name = 'id', type = 'hidden', value = id)
        form += X.input(
            name = 'propertyName', type = 'hidden', value = propertyName)

        context.push(viewName = 'default')
        propertyKindWidget = propertyKindSlot.getWidget()
        propertyKindWidget.setInForm(1)
        propertyKindWidget.setReadOnly(0)
        form += propertyKindWidget.getModelPageBodyLayout(
            propertyKindSlot, keywords)
        context.pull()
        form += X.div(_class = 'buttons-bar')(
            X.span(_class = 'action-buttons-bar')(
                X.buttonInForm('modify', 'modifyButton')))
        return writePageLayout(
            layout,
            _('Editing Type of Property "%s" of Card "%s"') % (
                property.name, object.getLabel()))
    editType.isPublicForWeb = 1

    def editWidget(self, id, propertyName, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        keywords['propertyName'] = propertyName
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canModifyObject(id):
            return accessForbidden()
        object = self.getObject(id)
        rememberObject(id)

        if not propertyName in object.getPropertyNames():
            return pageNotFound()
        propertySlot = object.getPropertySlot(propertyName)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)
        propertyKind = propertyKindSlot.getValue()
        propertyWidgetSlot = propertyKind.getSlot(
            'widget', parentSlot = propertyKindSlot)

        # We have to generate the fields for the whole property and not only
        # property.kind, because some fields of property are used by its kind.
        property.makeFieldsFromInstance(keywords, parentSlot = propertySlot)
        property.repairFields(keywords, parentSlot = propertySlot)

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submitWidget' % self.serverRole),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form
        form += X.input(name = 'id', type = 'hidden', value = id)
        form += X.input(
            name = 'propertyName', type = 'hidden', value = propertyName)

        context.push(viewName = 'default')
        propertyWidgetWidget = propertyWidgetSlot.getWidget()
        propertyWidgetWidget.setInForm(1)
        propertyWidgetWidget.setReadOnly(0)
        form += propertyWidgetWidget.getModelPageBodyLayout(
            propertyWidgetSlot, keywords)
        context.pull()
        form += X.div(_class = 'buttons-bar')(
            X.span(_class = 'action-buttons-bar')(
                X.buttonInForm('modify', 'modifyButton')))
        return writePageLayout(
            layout,
            _('Editing Widget of Property "%s" of Card "%s"') % (
                property.name, object.getLabel()))
    editWidget.isPublicForWeb = 1

    def getViewNavigationButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        layout += ObjectsWebMixin.getViewNavigationButtonsBarLayout(
            self, object, fields)
        userToken = context.getVar('userToken', default = '')
        if userToken:
            layout += X.buttonStandalone(
                'view-definition',
                X.url('/%s/definition' % urlId))
        return layout

    def getViewOtherActionButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        layout += ObjectsWebMixin.getViewOtherActionButtonsBarLayout(
            self, object, fields)
        if self.canAddObject():
            layout += X.buttonStandalone(
                'implement',
                X.url('/%s/implement' % urlId))
        return layout

    def implement(self, id, again = '', error = '', **keywords):
        prototypeId = id
        id = ''
        if keywords is None:
            keywords = {}
        keywords['id'] = ''
        if not self.hasObject(prototypeId):
            return pageNotFound()
        if not self.canAddObject():
            return accessForbidden()
        object = self.newObject(keywords)
        object.prototypeIds = [prototypeId]

        if not again:
            object.makeFieldsFromInstance(keywords)
        object.repairFields(keywords)

        headerTitle = _('New %s') % _(self.objectNameCapitalized)

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(
            action = X.url('/%s/submitImplementation' % self.serverRole),
            enctype = 'multipart/form-data', method = 'post')
        layout += form
        form += X.input(
            name = 'prototypeId', type = 'hidden', value = prototypeId)

        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
##         form += X.table(_class = 'page-body')(object.getEditLayout(
##             keywords))
        form += X.div(_class = 'buttons-bar')(
            X.span(_class = 'action-buttons-bar')(
                X.buttonInForm('create', 'createButton')))
        return writePageLayout(layout, headerTitle)
    implement.isPublicForWeb = 1

    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id):
                return confirmDelete(
                    X.url('/%s/delete' % urlId),
                    objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        if id:
            # Important!
            oldObject = self.getPartialObject(
                id, ['prototypeIds', 'properties'])
            object.prototypeIds = oldObject.prototypeIds
            object.properties = oldObject.properties
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        if not id:
            try:
                id = self.addObject(object)
            except faults.Fault:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['versionError'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        return redirect(X.url(id))
    submit.isPublicForWeb = 1

    def submitDefinition(self, id, **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.getObject(id)
        object.__class__ = CardAsDefinition # Important!
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/editDefinition' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            self.modifyPartialObject(object, object.getSlotToModifyNames())
        except faults.WrongVersion:
            keywords['again'] = '1'
            keywords['error'] = '1'
            keywords['versionError'] = '1'
            uri = X.url('/%s/editDefinition' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        except:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(X.url('/%s/definition' % urlId))
    submitDefinition.isPublicForWeb = 1

    def submitImplementation(self, prototypeId, **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = ''

        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        object.prototypeIds = [prototypeId]
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/implement' % getUrlId(prototypeId))
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            id = self.addObject(object)
        except faults.Fault:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(X.url(id))
    submitImplementation.isPublicForWeb = 1

    def submitType(self, id, propertyName, **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        keywords['propertyName'] = propertyName
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.getObject(id)
        rememberObject(id)

        if not propertyName in object.getPropertyNames():
            return pageNotFound()
        propertySlot = object.getPropertySlot(propertyName)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)
        propertyKind = propertyKindSlot.getValue()

        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            propertyKind.submitFields(keywords, parentSlot = propertyKindSlot)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/editType/%s' % (urlId, propertyName))
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            self.setImplementationPropertyValueKind(
                object.id, propertyName, propertyKind, object)
        except faults.WrongVersion:
            keywords['again'] = '1'
            keywords['error'] = '1'
            keywords['versionError'] = '1'
            uri = X.url('/%s/editType/%s' % (urlId, propertyName))
            uri.addKeywords(keywords)
            return redirect(uri)
        except:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(X.url('/%s/type/%s' % (urlId, propertyName)))
    submitType.isPublicForWeb = 1

    def submitWidget(self, id, propertyName, **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        keywords['propertyName'] = propertyName
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.getObject(id)
        rememberObject(id)

        if not propertyName in object.getPropertyNames():
            return pageNotFound()
        propertySlot = object.getPropertySlot(propertyName)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)
        propertyKind = propertyKindSlot.getValue()
        propertyWidgetSlot = propertyKind.getSlot(
            'widget', parentSlot = propertyKindSlot)
        propertyWidget = propertyWidgetSlot.getValue()

        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            propertyWidget.submitFields(
                keywords, parentSlot = propertyWidgetSlot)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/editWidget/%s' % (urlId, propertyName))
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            self.setImplementationPropertyValueKind(
                object.id, propertyName, propertyKind, object)
        except faults.WrongVersion:
            keywords['again'] = '1'
            keywords['error'] = '1'
            keywords['versionError'] = '1'
            uri = X.url('/%s/editWidget/%s' % (urlId, propertyName))
            uri.addKeywords(keywords)
            return redirect(uri)
        except:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(X.url('/%s/widget/%s' % (urlId, propertyName)))
    submitWidget.isPublicForWeb = 1

    def type(self, id, propertyName):
        urlId = getUrlId(id)
        if not self.hasObject(id):
            return pageNotFound()
        object = self.getObject(id)
        rememberObject(id)
        if not propertyName in object.getPropertyNames():
            return pageNotFound()
        propertySlot = object.getPropertySlot(propertyName)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)

        keywords = {}

        # We have to generate the fields for the whole property and not only
        # property.kind, because some fields of property are used by its kind.
        property.makeFieldsFromInstance(keywords, parentSlot = propertySlot)
        property.repairFields(keywords, parentSlot = propertySlot)

        layout = X.array()

        context.push(viewName = 'default')
        propertyKindWidget = propertyKindSlot.getWidget()
        propertyKindWidget.setInForm(0)
        layout += propertyKindWidget.getModelPageBodyLayout(
            propertyKindSlot, keywords)
        context.pull()
##         layout += X.table(_class = 'page-body')(propertyKind.getViewLayout(
##             keywords, parentSlot = propertyKindSlot))

        buttonsBar = X.div(_class = 'buttons-bar')
        layout += buttonsBar
        buttonsBar += X.span(_class = 'navigation-buttons-bar')(
            X.buttonStandalone(
                'view-utilization', X.url('/%s' % urlId)),
            X.buttonStandalone(
                'view-definition', X.url('/%s/definition' % urlId)))
        if self.canModifyObject(object.id):
            buttonsBar += X.span(_class = 'action-buttons-bar')(
                X.buttonStandalone(
                    'edit',
                    X.url('/%s/editType/%s' % (urlId, propertyName))))

        return writePageLayout(
            layout, _('Type of Property "%s" of File "%s"') % (
                property.name, object.getLabel()))
    type.isPublicForWeb = 1

    def widget(self, id, propertyName):
        urlId = getUrlId(id)
        if not self.hasObject(id):
            return pageNotFound()
        object = self.getObject(id)
        rememberObject(id)
        if not propertyName in object.getPropertyNames():
            return pageNotFound()
        propertySlot = object.getPropertySlot(propertyName)
        property = propertySlot.getValue()
        propertyKindSlot = property.getSlot('kind', parentSlot = propertySlot)
        propertyKind = propertyKindSlot.getValue()
        propertyWidgetSlot = propertyKind.getSlot(
            'widget', parentSlot = propertyKindSlot)

        keywords = {}

        # We have to generate the fields for the whole property and not only
        # property.kind.widget, because some fields of property are used by its
        # kind.
        property.makeFieldsFromInstance(keywords, parentSlot = propertySlot)
        property.repairFields(keywords, parentSlot = propertySlot)

        layout = X.array()

        context.push(viewName = 'default')
        propertyWidgetWidget = propertyWidgetSlot.getWidget()
        propertyWidgetWidget.setInForm(0)
        layout += propertyWidgetWidget.getModelPageBodyLayout(
            propertyWidgetSlot, keywords)
        context.pull()
##         layout += X.table(_class = 'page-body')(
##             propertyWidget.getViewLayout(
##                 keywords, parentSlot = propertyWidgetSlot))

        buttonsBar = X.div(_class = 'buttons-bar')
        layout += buttonsBar
        buttonsBar += X.span(_class = 'navigation-buttons-bar')(
            X.buttonStandalone(
                'view-utilization', X.url('/%s' % urlId)),
            X.buttonStandalone(
                'view-definition', X.url('/%s/definition' % urlId)),
            X.buttonStandalone(
                'view-type',
                X.url('/%s/type/%s' % (urlId, propertyName))))
        if self.canModifyObject(object.id):
            buttonsBar += X.span(_class = 'action-buttons-bar')(
                X.buttonStandalone(
                    'edit',
                    X.url('/%s/editWidget/%s' % (urlId, propertyName))))

        return writePageLayout(
            layout, _('Widget of Property "%s" of File "%s"') % (
                property.name, object.getLabel()))
    widget.isPublicForWeb = 1

