# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Directories Web"""

__version__ = '$Revision: 1.39 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.DirectoriesProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
import things
from tools import *


class AdminDirectories(AdminMixin, AdminDirectories):
    def getWeb(self):
        return getWebForServerRole('directories')
register(AdminDirectories)


class Directory(ObjectWebMixin, Directory):
    adminsSet_kind_itemKind_value_defaultValue = 'user'
    adminsSet_kind_itemKind_value_widget_noneLabel = N_('Everybody')
    adminsSet_kind_itemKind_value_widgetName = 'SelectId'
    adminsSet_kind_widget_fieldLabel = N_('Administrators')
    adminsSet_kind_widgetName = 'Multi'

    name_kind_widget_fieldLabel = N_('Name')
    name_kind_widgetName = 'InputText'

    rootPath_kind_widget_fieldLabel = N_('Path')
    rootPath_kind_widgetName = 'Path'

#    readersSet_kind_isHidden = 1
#    writersSet_kind_isHidden = 1

    def getWeb(self):
        return getWebForServerRole('directories')
register(Directory)


class DirectoryFile(ObjectWebMixin, DirectoryFile): ##things.ThingMixin
#    adminsSet_kind_itemKind_value_defaultValue = 'user'
    adminsSet_kind_itemKind_value_widget_noneLabel = N_('Parent control')
    adminsSet_kind_itemKind_value_widgetName = 'SelectId'
    adminsSet_kind_widget_fieldLabel = N_('Administrators')
    adminsSet_kind_widgetName = 'Multi'

    data_kind_widget_fieldLabel = N_('File')
    data_kind_widgetName = 'UploadFile'

    description_kind_widget_fieldLabel = N_('Description')
    description_kind_widgetName = 'InputText'    

    directory_kind_isHidden = 1
    directory_kind_widget_fieldLabel = N_('Directory')
    directory_kind_widgetName = 'InputText'    

    fileChildren_kind_itemKind_value_widgetName = 'Path'
    fileChildren_kind_itemKind_value_checkFilePath = 0
    fileChildren_kind_isHidden = 1
    fileChildren_kind_widget_fieldLabel = N_('Children')
    fileChildren_kind_widgetName = 'Multi'

    fileType_kind_widget_fieldLabel = N_('File type')
    fileType_kind_isReadOnly = 1
    fileType_kind_widgetName = 'InputText'

    mimeType_kind_widget_fieldLabel = N_('Mime type')
    mimeType_kind_isHidden = 1
    mimeType_kind_widgetName = 'InputText'

    owner_kind_defaultValue = 'user'
    owner_kind_widget_fieldLabel = N_('Owner')
    owner_kind_widget_noneLabel = N_('Everybody')
    owner_kind_widgetName = 'SelectId'

    path_kind_widget_fieldLabel = N_('Path')
    path_kind_isReadOnly = 1
    path_kind_widgetName = 'Path'    
    path_kind_checkFilePath = 0

    readersSet_kind_itemKind_value_defaultValue = None
    readersSet_kind_itemKind_value_widget_noneLabel = N_('Parent control')

    repertoryChildren_kind_itemKind_value_widgetName = 'Path'
    repertoryChildren_kind_itemKind_value_checkFilePath = 0
    repertoryChildren_kind_isHidden = 1
    repertoryChildren_kind_widget_fieldLabel = N_('Children')
    repertoryChildren_kind_widgetName = 'Multi'

    size_kind_widget_fieldLabel = N_('Size')
    size_kind_isHidden = 1
    size_kind_widgetName = 'InputText'    

    writersSet_kind_itemKind_value_defaultValue = None
    writersSet_kind_itemKind_value_widget_noneLabel = N_('Parent control')

    def __init__(self):
        pass

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = self.getLayoutSlotNames(fields, parentSlot = parentSlot)
        if self.isRepertory():
            slotNames.remove('data')
        return slotNames

    def getWeb(self):
        return directoriesWeb
register(DirectoryFile)


class DirectoriesWeb(ObjectsWebMixin, DirectoriesProxy):
    def edit(self, id = '', again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        
        if keywords.has_key('path'):
            path = keywords['path']
        else:
            path = ''
            
        isNewRepertory = isButtonSelected('directory_addRepertory', keywords)
        isNewFile =  isButtonSelected('directory_addFile', keywords)
        isNew = isNewRepertory or isNewFile
        
        if id and not self.hasObject(id, path):
            return pageNotFound()
        if id and not isNew:
            if not self.canModifyObject(id, path):
                return accessForbidden()
            object = self.getObject(id, path)
            rememberObject(id)
        else:
            if not self.canAddObject(id, path):
                return accessForbidden()
            if id:
                object = self.newThing('object', 'DirectoryFile')
                object.id = id
                object.directory = id
                if keywords.has_key('directory_name') and \
                       keywords['directory_name']:
                    object.setPath(path + file_separator +
                                   keywords['directory_name'])
                else:
                    object.setPath(path + file_separator +  '+')
                if isNewFile:
                    object.fileType = 'file'
                else:
                    object.fileType = 'repertory'
            else:
                object = self.newThing('object', 'Directory')

        if id and not again:
            object.makeFieldsFromInstance(keywords)
        if not id and not again or isNew:
            object.initFields(keywords)
        object.repairFields(keywords)

        if not id or isNew:
            headerTitle = _('New %s') % _(self.objectNameCapitalized)
        else:
            headerTitle = _('Editing %s - %s') % (
                _(self.objectNameCapitalized), object.getLabel())

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submit' % self.serverRole),
                      enctype = 'multipart/form-data', method = 'post')
        layout += form
        if keywords.has_key('cloned') and keywords['cloned']:
            form += X.input(name = 'cloned', type = 'hidden', value = '1')

        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        if not id or isNew:
            actionButtonsBar += X.buttonInForm('create', 'createButton')
        else:
            if self.canDeleteObject(id):
                actionButtonsBar += X.buttonInForm('delete', 'deleteButton')
            actionButtonsBar += X.buttonInForm('modify', 'modifyButton')

        return writePageLayout(layout, headerTitle)
    edit.isPublicForWeb = 1

    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        if keywords.has_key('path'):
            path = keywords['path']
        else:
            path = ''
            
        keywords['id'] = id
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id, path):
                url = X.url('/%s/delete' % urlId)
                url.add('path', path)
                return confirmDelete(url, objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
            
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        if path:
            object = self.newThing('object', 'DirectoryFile')
        else:
            object = self.newThing('object', 'Directory')
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)

        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            if path:
                uri.add('path', path)
            return redirect(uri)

        isNew = isButtonSelected('createButton', keywords)
        if not id or isNew:
            try:
                if path and object.path[-2:] == file_separator + '+':
                    object.path = object.path[:-1] + keywords['dataFileName']
                id = self.addObject(object)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['versionError'] = '1'
                uri = X.url('/%s/edit' %  urlId)
                uri.addKeywords(keywords)
                if path:
                    uri.add('path', path)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        url = X.url(id)
        if path:
            url.add('path', object.path)
        return redirect(url)
    submit.isPublicForWeb = 1

    def confirmDelete(self, id, path = file_separator):
        urlId = getUrlId(id)
        if id and self.hasObject(id, path):
            url = X.url('/%s/delete' % urlId)
            url.add('path', path)
            return confirmDelete(url, objectName = self.objectName)
        else:
            return pageNotFound()
    confirmDelete.isPublicForWeb = 1

    def delete(self, id, path = None):
        if path is None or path == file_separator:
            return ObjectsWebMixin.delete(self, id)
        else:
            DirectoriesProxy.deletePath(self, id, path)
            newPath = file_separator.join(path.split(file_separator)[:-1])
            if not newPath:
                newPath = file_separator
            uri = X.url(id)
            uri.add('path', newPath)
            return redirect(uri)
    delete.isPublicForWeb = 1

    def download(self, id, path = ''):
        if path:
            object = DirectoriesProxy.getObject(self, id, path, 1)
            name = object.getName()
            req = context.getVar('req')
            if name:
                req.headers_out['Content-disposition'] = \
                              'attachment; filename="%s"' % name
            req.headers_out['Content-length'] = str(object.size)
            req.content_type = object.mimeType
            setHttpCookie()
            req.send_http_header()
            if req.method == 'HEAD':
                return OK
            req.write(object.data)
            return OK
    download.isPublicForWeb = 1
    
##    def applyAddRepertory(self, id,
##                          path = file_separator, name = '', description = ''):
##        if name:
##            object = DirectoryFile()
##            object.id = id
##            object.path = path + file_separator + name
##            object.description = description
##            DirectoriesProxy.addObject(self, object)
##            path = path + file_separator + name
            
##        uri = X.url(id)
##        uri.add('path', path)
##        return redirect(uri)
##    applyAddRepertory.isPublicForWeb = 1

##    def applyAddFile(self, id,
##                     path = '/', name = '', description = '', data=''):
##        if data:
##            if not name:
##                name = data.filename
##            DirectoriesProxy.addFile(self, id,
##                                     path, name, description, data.value)
##            path = path + '/' + name
##        uri = X.url(id)
##        uri.add('path', path)
##        return redirect(uri)
##    applyAddFile.isPublicForWeb = 1

##    def applyModify(self, id, path = None):
##        pass

##    def view(self, id, path = None):
##        urlId = getUrlId(id)
##        if not self.hasObject(id):
##            return pageNotFound()
##        if not self.canGetObject(id):
##            return accessForbidden()
##        object = self.getObject(id)
##        rememberObject(id)

##        if path is None:
##            path = '/'

##        current = self.getObjectForPath(id, path)
##        current.id = object.id

##        keywords = {}
##        current.makeFieldsFromInstance(keywords)
##        current.repairFields(keywords)

##        label = current.getLabel()

##        layout = X.array()
##        layout += getHeaderLayout('%s - %s' % (
##            _(self.objectNameCapitalized), label))
##        layout += X.table(current.getViewLayout(keywords))
##        layout += self.getViewAboveButtonsBarLayout(object, keywords)
##        layout += self.getViewButtonsBarLayout(object, keywords)
##        layout += self.getViewBelowButtonsBarLayout(object, keywords)
##        layout += getFooterLayout()
##        return writePageLayout(layout)
##    view.isPublicForWeb = 1

