# Glasnost 
# By: Odile Bnassy <obenassy@entrouvert.org>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@easter-eggs.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Events Web"""

__version__ = '$Revision: 1.40 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
from glasnost.common.tools import extractLocalId
import glasnost.common.context as context

from glasnost.proxy.EventsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import widgets
import accounting


class EventRecordsWidget(widgets.Multi):
    #fieldLabelPlural = None
    #fieldLabelPlural_kind_isPublicForXmlRpc = 1
    #fieldLabelPlural_kindName = 'String'

    thingName = 'EventRecords'

    def getHtmlField(self, slot, fields, **keywords):
        if self.isInForm() or not slot.getValue():
            return widgets.Multi.getHtmlField(
                self, slot, fields, **keywords)
        assert fields is not None
        count = slot.getKind().fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass

        requiredSlotNames = [
            'accountId', 'accountLabel', 'accountNumber', 'amount']
        displayedItems = []
        for i in range(count):
            itemSlot = slot.getKind().getItemSlot(slot, i)
            itemFieldName = itemSlot.getFieldName()
            itemFieldValue = itemSlot.getField(fields, default = '')
            if not itemFieldValue or not getWebForServerRole('records').hasObject(
                itemFieldValue):
                continue
            displayedItems.append(
                (itemFieldValue, (getWebForServerRole('records').getPartialObject(
                itemFieldValue, requiredSlotNames))))

        layout = tr = X.tr(_class = 'field-value')
        td = X.td(colspan = '2')
        td += X.h3()(_('Records'))
        layout += td
        table = X.table(_class = 'objects-table')
        td += table
        td += X.br()

        thead = X.thead()
        table += thead
        tr = X.tr()
        thead += tr
        tr += X.th(scope = 'col')(_('Record'))
        tr += X.th(scope = 'col')(_('Account'))
        tr += X.th(scope = 'col')(_('Debit'))
        tr += X.th(scope = 'col')(_('Credit'))
        tbody = X.tbody()
        table += tbody
        for i in range(len(displayedItems)):
            objectId = displayedItems[i][0]
            object = displayedItems[i][1]
            tr = X.tr()
            tbody += tr
            tr += X.td(_class = 'column-label')(
                X.a(href = X.url(objectId))(extractLocalId(objectId)))
            tr += X.td(_class = 'column-label')(
                X.a(href = X.url(object.accountId))(
                accounting.accountLabel(object.accountNumber,
                                          object.accountLabel)))
            if object.amount < 0:
                tr += X.td(_class='column-value', align='right')(
                    accounting.amountDisplay(- object.amount, withSide = 0))
                tr += X.td(_class='column-value')(X.nbsp)
            else:
                tr += X.td(_class='column-value')(X.nbsp)   
                tr += X.td(_class='column-value', align='right')(
                    accounting.amountDisplay(object.amount, withSide = 0))
        return layout
widgets.register(EventRecordsWidget)


class AdminEvents(AdminMixin, AdminEvents):
    def getWeb(self):
        return getWebForServerRole('events')
register(AdminEvents)


class Event(ObjectWebMixin, Event):
    certainty_kind_defaultValue = 1
    certainty_kind_widget_fieldLabel = N_('Certainty')
    certainty_kind_widget_labels = accounting.certaintyLabels
    certainty_kind_widgetName = 'Select'

    comment_kind_widget_fieldLabel = N_('Comment')
    comment_kind_widget_cols = 40
    comment_kind_widget_rows = 5
    comment_kind_widgetName = 'TextArea'

    currency_kind_hasToSubmitField = 0
    currency_kind_isReadOnly = 1
    currency_kind_preference = 'currency'
    currency_kind_widget_fieldLabel = N_('Currency')
    currency_kind_widget_labels = accounting.currencyLabels
    currency_kind_widgetName = 'Select'

    date_kind_defaultValue = 'now'
    date_kind_widget_fieldLabel = N_('Date')
    date_kind_widget_yearMinRel = -1
    date_kind_widget_yearMaxRel = 3
    date_kind_widgetName = 'Date'

    isBalanced_kind_hasToSubmitField = 0
    isBalanced_kind_isReadOnly = 1
    isBalanced_kind_defaultValue = 1
    isBalanced_kind_widget_labels = accounting.balancedLabels
    isBalanced_kind_widget_fieldLabel = N_('Balanced')
    isBalanced_kind_widgetName = 'InputCheckBox'

    label_kind_widget_fieldLabel = N_('Label')
    label_kind_widget_size = 60
    label_kind_widgetName = 'InputText'

    membersSet_kind_itemKind_value_widgetName = 'SelectId'
    membersSet_kind_widget_apply = 1
    membersSet_kind_widget_fieldLabel = N_('Records')
    membersSet_kind_widgetName = 'EventRecords'

    operationId_kind_widget_fieldLabel = N_('Operation')
    operationId_kind_widgetName = 'SelectId'

    partnerId_kind_hasToSubmitField = 0
    partnerId_kind_isReadOnly = 1
    partnerId_kind_widget_fieldLabel = N_('Partner')
    partnerId_kind_widgetName = 'SelectId'

    voucherId_kind_widget_fieldLabel = N_('Voucher')
    voucherId_kind_widgetName = 'SelectId'

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            slotNames = slotNames[:]
            for slotName in ('readersSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('events')
register(Event)



class EventsWeb(ObjectsWebMixin, EventsProxy):
    def addRecord(self, id, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if not self.hasObject(id):
            return pageNotFound()
        membersWeb = getWebForServerRole('records')
        if not membersWeb.canAddObject() \
           or not self.canModifyObject(id):
            return accessForbidden()

        member = membersWeb.newThing('object', 'Record')
        if not again:
            member.initFields(keywords)
        keywords["eventId"] = id
        member.repairFields(keywords)

        layout = X.array()
        layout += member.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/events/submitRecord'),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form

        slot = slots.Root(member)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonInForm('create', 'createButton')

        return writePageLayout(layout, _('New Record'))
    addRecord.isPublicForWeb = 1

    def getViewOtherActionButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        layout += ObjectsWebMixin.getViewOtherActionButtonsBarLayout(
            self, object, fields)
        membersWeb = getWebForServerRole('records')
        if membersWeb.canAddObject() and self.canModifyObject(object.id):
            layout += X.buttonStandalone(
                'Add Record', X.url('/%s/addRecord' % urlId))
        return layout

    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id):
                return confirmDelete(
                    X.url('/%s/delete' % urlId),
                    objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            if keywords.has_key('cloned'):
                parentsWeb = getWebForServerRole('operations')
                parentId = keywords['operationId']
                parent = parentsWeb.getPartialObject(parentId, ['membersSet'])
                brotherIds = parent.getSlot('membersSet').getValue()
                membersWeb = getWebForServerRole('records')
                memberKeys = [
                    x
                    for x in keywords.keys()
                    if x.startswith('membersSet') and x != 'membersSet_count']
                memberIdsToDuplicate = filter(None, [
                    keywords[key] for key in memberKeys])
                if not memberIdsToDuplicate:
                    newMembers = []
                else:
                    membersToDuplicate = [
                        membersWeb.getPartialObject(memberId, [
                        'accountId', 'amount', 'eventId'])
                        for memberId in memberIdsToDuplicate]
                    newMembers = membersToDuplicate[:]
                    newMemberIds = []
                    try:
                        for member in newMembers:
                            member.getSlot('id').setValue(None)
                            memberId = membersWeb.addObject(member)
                            newMemberIds.append(memberId)
                            member.getSlot('id').setValue(memberId)
                    except:
                        if context.getVar('debug'):
                            raise
                        return accessForbidden()
                    for counter in range(len(newMemberIds)):
                        keywords["membersSet_%s" % counter] = newMemberIds[
                            counter]
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        if not id:
            try:
                id = self.addObject(object)
                if keywords.has_key('cloned'):
                    brotherIds.append(id)
                    parent.getSlot('membersSet').setValue(brotherIds)
                    parentsWeb.modifyPartialObject(parent, ['membersSet'])
                    for member in newMembers:
                        member.getSlot('eventId').setValue(id)
                        membersWeb.modifyPartialObject(member, ['eventId'])
            except faults.Fault:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['versionError'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        return redirect(X.url(id))
    submit.isPublicForWeb = 1


    def submitRecord(self, id, **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id

        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        membersWeb = getWebForServerRole('records')
        member = membersWeb.newThing('object', 'Record')
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            del keywords['id'] # Not the article id.
            member.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/addRecord' % getUrlId(id))
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            memberId = membersWeb.addObject(member)
        except faults.Fault:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        object = self.getObject(id)
        if object.membersSet is None:
            object.membersSet = []
        object.membersSet.append(memberId)
        try:
            self.modifyPartialObject(
                object, object.getSlotToModifyNames())
        except faults.Fault:
            try:
                membersWeb.deleteObject(memberId)
            except:
                pass
            return accessForbidden()
        return redirect(X.url(id))
    submitRecord.isPublicForWeb = 1
    
