# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Function Calls Web"""

__version__ = '$Revision: 1.26 $'[11:-2]


import glasnost.common.context as context
import glasnost.common.tools_new as commonTools

from glasnost.proxy.FunctionCallsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *


class AdminFunctionCalls(AdminMixin, AdminFunctionCalls):
    def getWeb(self):
        return getWebForServerRole('functioncalls')
register(AdminFunctionCalls)
        
          
class FunctionCall(ObjectWebMixin, FunctionCall):
    arguments_kind_isConstant = 1
    arguments_kind_widget_fieldLabel = N_('Arguments')

    functionName_kind_widget_apply = 1
    functionName_kind_widget_fieldLabel = N_('Function Name')

    serverId_kind_widget_apply = 1
    serverId_kind_widget_fieldLabel = N_('Server ID')
##     serverId_kind_widgetName = 'SelectServerId'


    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ['arguments', 'result']:
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getFunctionDeclaration(self):
        if self.functionDeclaration is not None:
            return self.functionDeclaration
        if not self.serverId:
            return None
        if not self.functionName:
            return None
        import glasnost.common.context as context
        try:
            serverRole = commonTools.extractRole(self.serverId)
        except:
            return None
        try:
            web = getWebForServerRole(serverRole)
        except:
            return None
        if web is None:
            return None
        try:
            functionDeclarations = web.getFunctionDeclarations() 
        except:
            return None
        if not functionDeclarations.has_key(self.functionName):
            return None
        self.functionDeclaration = functionDeclarations[self.functionName]
        return self.functionDeclaration

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'creationTime', 'modificationTime', 'readersSet',
                'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('functioncalls')
register(FunctionCall)


class FunctionCallsWeb(ObjectsWebMixin, FunctionCallsProxy):
    def editArguments(self, id, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canModifyObject(id):
            return accessForbidden()
        object = self.getObject(id)
        rememberObject(id)

        slot = object.getSlot('arguments')
        kind = slot.getKind()
        widget = kind.getModelWidget(slot)

        if not again:
            kind.makeFieldFromValue(
                slot, keywords, slot.getValue())
        kind.repairField(slot, keywords)

        headerTitle = _('Editing Arguments of Function Call "%s"') % \
                      object.getLabel()

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submitArguments' % self.serverRole),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form
        form += X.input(name = 'id', type = 'hidden', value = id)
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getHtmlValue(slot, keywords)

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonInForm('modify', 'modifyButton')

        return writePageLayout(layout, headerTitle)
    editArguments.isPublicForWeb = 1
        
    def evaluate(self, id):
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canModifyObject(id):
            return accessForbidden()

        self.evaluateObject(id)
        return redirect(X.url(id))
    evaluate.isPublicForWeb = 1

    def getViewOtherActionButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        layout += ObjectsWebMixin.getViewOtherActionButtonsBarLayout(
            self, object, fields)
        layout += X.buttonStandalone(
            'evaluate', X.url('/%s/evaluate' % urlId))
        layout += X.buttonStandalone(
            'edit-arguments', X.url('/%s/editArguments' % urlId))
        return layout

    def submitArguments(self, id, **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.getObject(id)
        rememberObject(id)

        slot = object.getSlot('arguments')
        kind = slot.getKind()
        widget = kind.getModelWidget(slot)

        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            kind.submitField(slot, keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/editArguments' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            self.modifyPartialObject(object, object.getSlotToModifyNames())
        except faults.WrongVersion:
            keywords['again'] = '1'
            keywords['error'] = '1'
            keywords['versionError'] = '1'
            uri = X.url('/%s/editArguments' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        except:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        self.evaluateObject(id)
        return redirect(X.url(id))
    submitArguments.isPublicForWeb = 1

