# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost TAL driver (used for included files)"""

__version__ = '$Revision: 1.11 $'[11:-2]

import os
import os.path
import time

from glasnost.common.tools import getConfig
import glasnost.common.context as context


compiledFiles = {}

def compilefile(file, mode=None):
    assert mode in ("html", "xml", None)
    t1 = time.time()
    templateDirectoryName = context.getVar('templateDirectoryName')
    file = '%s/%s/%s' % (context.getVar('templatesDirectoryPath'), \
                            templateDirectoryName, file)
    if not os.path.exists(file):
        fallbackPath = context.getVar('fallbackTemplatesDirectoryPath')
        filename = file.split('/')[-1]
        if os.path.exists('%s/default/%s' % (fallbackPath, filename)):
            file = '%s/default/%s' % (fallbackPath, filename)
    if compiledFiles.has_key(file):
        #open('/tmp/tal.cfile', 'a').write('%f (%s) (faster)\n' % (time.time()-t1, file))
        return compiledFiles[file]
    if mode is None:
        ext = os.path.splitext(file)[1]
        if ext.lower() in (".html", ".htm", ".tal"):
            mode = "html"
        else:
            mode = "xml"
    from glasnost.web.GlasnostTALGenerator import GlasnostTALGenerator
    if mode == "html":
        from TAL.HTMLTALParser import HTMLTALParser
        p = HTMLTALParser(gen = GlasnostTALGenerator(xml=0))
    else:
        from TAL.TALParser import TALParser
        p = TALParser(gen = GlasnostTALGenerator(xml=1))
    p.parseFile(file)
    code = p.getCode()
    compiledFiles[file] = code
    #open('/tmp/tal.cfile', 'a').write('%f (%s)\n' % (time.time()-t1, file))
    return code

