# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Groups Web"""

__version__ = '$Revision: 1.20 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.context as context

from glasnost.proxy.GroupsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *


class AdminGroups(AdminMixin, AdminGroups):
    def getWeb(self):
        return getWebForServerRole('groups')
register(AdminGroups)


class Group(ObjectWebMixin, Group):
    # itemIdsGetter is defined below
    itemIds_kind_isReadOnly = 1
    itemIds_kind_itemKind_value_widgetName = 'SelectId'
    itemIds_kind_widget_fieldLabel = N_('Items')
    itemIds_kind_widgetName = 'Multi'

    membersSet_kind_itemKind_value_widgetName = 'SelectId'
    membersSet_kind_widget_fieldLabel = N_('Members')
    membersSet_kind_widgetName = 'Multi'

    name_kind_widget_fieldLabel = N_('Name')
    #name_kind_widget_maxLength = 40
    name_kind_widget_size = 40
    name_kind_widgetName = 'InputText'

    nonMembersSet_kind_itemKind_value_widgetName = 'SelectId'
    nonMembersSet_kind_widget_fieldLabel = N_('Non Members')
    nonMembersSet_kind_widgetName = 'Multi'

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ['itemIds']:
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in ('membersSet', 'nonMembersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('groups')

    def itemIdsGetter(self):
        itemIds = getSetDeltaContainedIds(self.membersSet, self.nonMembersSet)
        itemIds = sortIds(itemIds)
        return itemIds
register(Group)


class GroupsWeb(ObjectsWebMixin, GroupsProxy):
    pass

