# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Initiatives Web"""

__version__ = '$Revision: 1.25 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.InitiativesProxy import *
from glasnost.proxy.TranslationsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import geography


class AdminInitiatives(AdminMixin, AdminInitiatives):
    def getWeb(self):
        return getWebForServerRole('initiatives')
register(AdminInitiatives)


class Initiative(ObjectWebMixin, Initiative):
    abstract_kind_widget_fieldLabel = N_('Abstract')
    abstract_kind_widget_cols = 75
    abstract_kind_widget_colSpan = 2
    abstract_kind_widget_rows = 4
    abstract_kind_widgetName = 'TextArea'

    benefits_kind_widget_fieldLabel = N_('Social Benefits')
    benefits_kind_widget_cols = 75
    benefits_kind_widget_colSpan = 2
    benefits_kind_widget_rows = 4
    benefits_kind_widgetName = 'TextArea'

    contactIds_kind_itemKind_value_defaultValue = 'user'
    contactIds_kind_itemKind_value_widgetName = 'SelectId'
    contactIds_kind_widget_fieldLabel = N_('Contacts')
    contactIds_kind_widgetName = 'Multi'

    creationPartnerIds_kind_itemKind_value_widgetName = 'SelectId'
    creationPartnerIds_kind_widget_fieldLabel = N_('Partners')
    creationPartnerIds_kind_widgetName = 'Multi'

    description_kind_widget_fieldLabel = N_('Description')
    description_kind_widget_cols = 75
    description_kind_widget_colSpan = 2
    description_kind_widget_rows = 8
    description_kind_widgetName = 'TextArea'

    difficulties_kind_widget_fieldLabel = N_('Difficulties')
    difficulties_kind_widget_cols = 75
    difficulties_kind_widget_colSpan = 2
    difficulties_kind_widget_rows = 4
    difficulties_kind_widgetName = 'TextArea'

    editionTime_kind_defaultValue = 'now'
    editionTime_kind_isReadOnly = 1
    editionTime_kind_widget_fieldLabel = N_('Edition Time')
    editionTime_kind_widget_maxLength = 19
    editionTime_kind_widget_size = 19
    editionTime_kind_widgetName = 'InputText'

    evaluation_kind_widget_fieldLabel = N_('Assessment Process')
    evaluation_kind_widget_cols = 75
    evaluation_kind_widget_colSpan = 2
    evaluation_kind_widget_rows = 4
    evaluation_kind_widgetName = 'TextArea'

    financing_kind_widget_fieldLabel = N_('Financing')
    financing_kind_widget_cols = 75
    financing_kind_widget_colSpan = 2
    financing_kind_widget_rows = 4
    financing_kind_widgetName = 'TextArea'

    goals_kind_widget_fieldLabel = N_('Goals')
    goals_kind_widget_cols = 75
    goals_kind_widget_colSpan = 2
    goals_kind_widget_rows = 4
    goals_kind_widgetName = 'TextArea'

    initiativeCreationDate_kind_defaultValue = 'now'
    initiativeCreationDate_kind_widget_yearMinRel = -5
    initiativeCreationDate_kind_widget_yearMaxRel = 0
    initiativeCreationDate_kind_widget_fieldLabel = N_(
        'Initiative Creation Date')
    #initiativeCreationDate_kind_widget_maxLength = 19
    #initiativeCreationDate_kind_widget_size = 19
    initiativeCreationDate_kind_widgetName = 'Date'

    networkIds_kind_itemKind_value_widgetName = 'SelectId'
    networkIds_kind_widget_fieldLabel = N_('Networks')
    networkIds_kind_widgetName = 'Multi'

    ownerId_kind_defaultValue = 'user'
    ownerId_kind_widget_fieldLabel = N_('Project Owner')
    ownerId_kind_widgetName = 'SelectId'

    projectsUnderConsideration_kind_widget_fieldLabel = N_(
        'Projets Under Consideration')
    projectsUnderConsideration_kind_widget_cols = 75
    projectsUnderConsideration_kind_widget_colSpan = 2
    projectsUnderConsideration_kind_widget_rows = 4
    projectsUnderConsideration_kind_widgetName = 'TextArea'

    realUsers_kind_widget_fieldLabel = N_('Actual Users')
    realUsers_kind_widget_cols = 75
    realUsers_kind_widget_colSpan = 2
    realUsers_kind_widget_rows = 4
    realUsers_kind_widgetName = 'TextArea'

    source_kind_widget_fieldLabel = N_('Source')
    #source_kind_widget_maxLength = 40
    source_kind_widget_size = 40
    source_kind_widgetName = 'InputText'

    strongPoints_kind_widget_fieldLabel = N_('Strong points')
    strongPoints_kind_widget_cols = 75
    strongPoints_kind_widget_colSpan = 2
    strongPoints_kind_widget_rows = 4
    strongPoints_kind_widgetName = 'TextArea'

    territory_kind_itemKind_value_defaultValue = None
    territory_kind_itemKind_value_widgetName = 'XSelect'
    territory_kind_itemKind_value_labels = {}
    territory_kind_widget_fieldLabel = N_('Affected Territory/ies')
    territory_kind_widgetName = 'Multi'

    ticAppropriation_kind_widget_fieldLabel = N_('Web Tools Appropriation')
    ticAppropriation_kind_widget_cols = 75
    ticAppropriation_kind_widget_colSpan = 2
    ticAppropriation_kind_widget_rows = 4
    ticAppropriation_kind_widgetName = 'TextArea'

    ticRole_kind_defaultValue = None
    ticRole_kind_widget_fieldLabel = N_('Web Tools Role')
    ticRole_kind_widget_labels = {
        'central': 'Objet central de l\'initiative',
        'lever': 'Levier',
        'accelerator': 'Accelerateur',
        'hinder': 'Obstacle',
        'change': 'Changement d\'organisation',
        'neutral': 'Neutre',
        }
    ticRole_kind_widgetName = 'XSelect'

    ticUsages_kind_widget_fieldLabel = N_('Web Tools Usages')
    ticUsages_kind_widget_cols = 75
    ticUsages_kind_widget_colSpan = 2
    ticUsages_kind_widget_rows = 4
    ticUsages_kind_widgetName = 'TextArea'

    ticUsed_kind_widget_fieldLabel = N_('Used Tools')
    ticUsed_kind_widget_cols = 75
    ticUsed_kind_widget_colSpan = 2
    ticUsed_kind_widget_rows = 4
    ticUsed_kind_widgetName = 'TextArea'

    ticUsers_kind_widget_fieldLabel = N_('Web Tools Users')
    ticUsers_kind_widget_cols = 75
    ticUsers_kind_widget_colSpan = 2
    ticUsers_kind_widget_rows = 4
    ticUsers_kind_widgetName = 'TextArea'

    title_kind_widget_fieldLabel = N_('Title')
    #title_kind_widget_maxLength = 40
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    trigger_kind_widget_fieldLabel = N_('Initiative Trigger')
    trigger_kind_widget_cols = 75
    trigger_kind_widget_colSpan = 2
    trigger_kind_widget_rows = 4
    trigger_kind_widgetName = 'TextArea'

    webSite_kind_defaultValue = 'http://'
    webSite_kind_widget_fieldLabel = N_('Initiative Web Site')
    #webSite_kind_widget_maxLength = 40
    webSite_kind_widget_size = 40
    webSite_kind_widgetName = 'InputText'

    def __init__(self):
        labels = {}
        for key in self.territory_kind_itemKind_value_groupedValues[
            'French Regions']:
            labels[key] = geography.frenchRegionLabels[key]
        for key in self.territory_kind_itemKind_value_groupedValues[
            'European Countries']:
            labels[key] = geography.countryLabels[key]
        labels['fra'] = N_('French-speaking countries')
        self.territory_kind_itemKind_value_labels = labels

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ('creationTime', 'modificationTime', 'readersSet'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in ('writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'creationTime', 'modificationTime', 'readersSet',
                'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('initiatives')
register(Initiative)


class InitiativesWeb(ObjectsWebMixin, InitiativesProxy):
    pass

