# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Ledgers Web"""

__version__ = '$Revision: 1.26 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.LedgersProxy import *

import kinds
from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import accounting


class AdminLedgers(AdminMixin, AdminLedgers):
    def getWeb(self):
        return getWebForServerRole('ledgers')
register(AdminLedgers)


class Ledger(ObjectWebMixin, Ledger):
    currency_kind_preference = 'currency'
    currency_kind_widget_fieldLabel = N_('Currency')
    currency_kind_widget_labels = accounting.currencyLabels
    currency_kind_widgetName = 'Select'

    lastEditorId_kind_isReadOnly = 1
    lastEditorId_kind_widget_fieldLabel = N_('Last Editor')
    lastEditorId_kind_widgetName = 'SelectId'

    startDate_kind_defaultValue = 'now'
    startDate_kind_widget_fieldLabel = N_('Starting')
    startDate_kind_widget_yearMinRel = -1
    startDate_kind_widget_yearMaxRel = 3
    startDate_kind_widgetName = 'Date'

    title_kind_widget_fieldLabel = N_('Title')
    #title_kind_widget_maxLength = 40
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ('membersSet'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayout(self, fields, parentSlot = None):
        layout = X.array()
        td = X.td(_class = 'field-value', colspan = 2)
        layout += X.tr(td)
        if fields.has_key('title'):
            title = getProxyForServerRole('translations').getTranslation(
                fields['title'], self.getId(), 'self.title',
                self.getLanguage(), context.getVar('readLanguages'))
        else:
            title = ''
        titlePage = X.div(_class = 'titlepage')
        titlePage += X.h2(_class = 'title')(title)
        td += titlePage

        td = X.td(_class = 'field-value', colspan = 2)
        layout += X.tr(td)
        requiredSlotNames = ['modificationTime', 'partnerId', 'title']
        displayedSlotNames = ['partnerId', 'modificationTime']
        operationsWeb = getWebForServerRole('operations')
        lastOperationIds = operationsWeb.searchObjectIds(self.id)
        lastOperationItems = operationsWeb.getObjects(
            requiredSlotNames = requiredSlotNames)
        lastOperations = [ lastOperationItems[id] for id in lastOperationIds ]
        td += operationsWeb.getObjectsSectionLayout(
            lastOperations,
            _("""Last Operations"""),
            displayedSlotNames)

        layout += ObjectWebMixin.getViewLayout(
            self, fields, parentSlot = parentSlot)
        return layout

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            slotNames = slotNames[:]
            for slotName in (
                'creationTime', 'lastEditorId', 'modificationTime',
                'readersSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('ledgers')
register(Ledger)


class LedgersWeb(ObjectsWebMixin, LedgersProxy):
    def addOperation(self, id, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if not self.hasObject(id):
            return pageNotFound()
        membersWeb = getWebForServerRole('operations')
        if not membersWeb.canAddObject() \
           or not self.canModifyObject(id):
            return accessForbidden()

        member = membersWeb.newThing('object', 'Operation')
        if not again:
            member.initFields(keywords)
        keywords["ledgerId"] = id
        member.repairFields(keywords)

        layout = X.array()
        layout += member.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/ledgers/submitOperation'),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form

        slot = slots.Root(member)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonInForm('create', 'createButton')

        return writePageLayout(layout, _('New Operation'))
    addOperation.isPublicForWeb = 1

    def getViewOtherActionButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        layout += ObjectsWebMixin.getViewOtherActionButtonsBarLayout(
            self, object, fields)
        membersWeb = getWebForServerRole('operations')
        if membersWeb.canAddObject() and self.canModifyObject(object.id):
            layout += X.buttonStandalone(
                'Add Operation',
                X.url('/%s/addOperation' % urlId))
        return layout

    def submitOperation(self, id, **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id

        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        membersWeb = getWebForServerRole('operations')
        member = membersWeb.newThing('object', 'Operation')
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            del keywords['id'] # Not the article id.
            member.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/addOperation' % getUrlId(id))
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            memberId = membersWeb.addObject(member)
        except faults.Fault:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(X.url(memberId))
    submitOperation.isPublicForWeb = 1

