# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Objects Web"""

__version__ = '$Revision: 1.76 $'[11:-2]


import md5
import time

import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context
import glasnost.common.translation as translation
import glasnost.common.xhtmlgenerator as X

from glasnost.proxy.DispatcherProxy import MultiCall

from GlasnostTALEngine import GlasnostTALEngine
import kinds # Do not remove!
import things
from tools import *
import WebAPI


register = things.register


class ObjectWebMixin(things.ThingMixin):
    id_kind_isHidden = 1
    id_kind_widget_fieldLabel = N_('ID')
    id_kind_widgetName = 'InputText'

    language_kind_defaultValue = 'en'
    language_kind_widget_fieldLabel = N_('Language')
    language_kind_widget_labels = translation.languageLabels
    language_kind_widgetName = 'Select'

    version_kind_defaultValue = 0
    version_kind_isHidden = 1
    version_kind_widget_fieldLabel = N_('Version Number')
    version_kind_widgetName = 'InputText'

    def getEmbeddedViewLayout(self):
        fields = {}
        self.makeFieldsFromInstance(fields)
        self.repairFields(fields)
        # FIXME: Replace ASAP.
        # return self.getCompactViewLayout(fields, parentSlot = None)
        return self.getFieldsViewLayout(fields, bodyOnly = 1)

    def getFieldsViewLayout(self, fields, bodyOnly = 0):
        # FIXME: Remove each getFieldsViewLayout method (including this one)
        # ASAP.
        if bodyOnly:
            return self.getCompactViewLayout(fields, parentSlot = None)
        else:
            return self.getViewLayout(fields, parentSlot = None)

    def getLabelTranslated(self, destinationLanguages, multiCall = None):
        translationsProxy = getProxyForServerRole('translations')
        if translationsProxy:
            return translationsProxy.getTranslation(
                        self.getLabel(), self.getId(), 'self.getLabel()',
                        self.getLabelLanguage(), destinationLanguages,
                        multiCall = multiCall)
        else:
            return self.getLabel()

    def getSlotToModifyNames(self, parentSlot = None):
        return None # Modify all slots.

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = things.ThingMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            slotNames = slotNames[:]
            for slotName in ('language',):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        raise NotImplementedError

    def newWidget(self, parentSlot = None):
        admin = self.getWeb().getAdmin()
        viewName = context.getVar('viewName')
        if admin.views is not None and admin.hasView(viewName):
            return admin.getView(viewName)
        else:
            return self.newThing('widget', 'Thing')


class AdminWithoutWritersMixin(ObjectWebMixin):
    adminsSet_kind_itemKind_value_widget_noneLabel = N_('Everybody')
    adminsSet_kind_itemKind_value_widgetName = 'SelectId'
    adminsSet_kind_widget_fieldLabel = N_('Administrators')
    adminsSet_kind_widgetName = 'Multi'

    views_kind_widget_fieldLabel = N_('Views')

    def newWidget(self, parentSlot = None):
        return self.newThing('widget', 'Thing')


class AdminMixin(AdminWithoutWritersMixin):
    readersSet_kind_itemKind_value_defaultValue = None

    writersSet_kind_itemKind_value_defaultValue = None


class WebMixin(things.ThingMixin):
    thingCategory = 'web'


class ObjectsWebMixin(WebMixin):
    def admin(self):
        if not self.canGetAdmin():
            return accessForbidden()
        admin = self.getAdmin()

        keywords = {}
        admin.makeFieldsFromInstance(keywords)
        admin.repairFields(keywords)

        layout = X.array()
        if CssJunkie:
            layout += admin.getViewLayout( keywords)
        else:
            layout += X.table(_class = 'page-body')(admin.getViewLayout(
                keywords))

        buttonsBar = X.div(_class = 'buttons-bar')
        layout += buttonsBar
        navigationButtonsBar = X.span(_class = 'navigation-buttons-bar')
        buttonsBar += navigationButtonsBar
        navigationButtonsBar += X.buttonStandalone(
            'view-list',
            X.url('/%s' % self.serverRole))
        if self.canModifyAdmin():
            actionButtonsBar = X.span(_class = 'action-buttons-bar')
            buttonsBar += actionButtonsBar
            actionButtonsBar += X.buttonStandalone(
                'edit',
                X.url('/%s/%s' % (self.serverRole, 'adminEdit')))

        return writePageLayout(
            layout,_('%s Settings') % _(self.objectsNameCapitalized))
    admin.isPublicForWeb = 1

    def adminEdit(self, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        admin = self.getAdmin()
        userToken = context.getVar('userToken', default = '')
        isAdmin = userToken and (
            not admin.adminsSet \
            or getProxyForServerRole('authentication').setContainsUser(admin.adminsSet))
        if not isAdmin:
            return accessForbidden()

        if not again:
            admin.makeFieldsFromInstance(keywords)
        admin.repairFields(keywords)

        layout = X.array()
        layout += admin.getErrorLayout(error, keywords)
        form = X.form(
            action = X.url('/%s/%s' % (self.serverRole, 'adminSubmit')),
            enctype= 'multipart/form-data', method = 'post')
        layout += form
        if CssJunkie:
            form += admin.getEditLayout(keywords)
        else:
            form += X.table(_class = 'page-body')(
                admin.getEditLayout(keywords))

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonInForm('modify', 'modifyButton')

        return writePageLayout(
            layout, _('Editing %s Settings') % _(self.objectsNameCapitalized))
    adminEdit.isPublicForWeb = 1

    def adminSubmit(self, **keywords):
        if keywords is None:
            keywords = {}
        admin = self.getAdmin()
        userToken = context.getVar('userToken', default = '')
        isAdmin = userToken and (
            not admin.adminsSet
            or getProxyForServerRole('authentication').setContainsUser(
                admin.adminsSet))
        if not isAdmin:
            return accessForbidden()

        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        admin = self.newAdmin(keywords)
        admin.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/%s' % (self.serverRole, 'adminEdit'))
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            self.modifyAdmin(admin)
        except faults.WrongVersion:
            keywords['again'] = '1'
            keywords['error'] = '1'
            keywords['versionError'] = '1'
            uri = X.url('/%s/%s' % (self.serverRole, 'adminEdit'))
            uri.addKeywords(keywords)
            return redirect(uri)
        except:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(X.url('/%s/%s' % (self.serverRole, 'admin')))
    adminSubmit.isPublicForWeb = 1

    def clone(self, id):
        if not id or not self.hasObject(id):
            return pageNotFound()
        if not self.canCloneObject(id):
            return accessForbidden()
        object = self.getObject(id)
        keywords = {}
        object.makeFieldsFromInstance(keywords)
        del keywords['id']
        keywords['again'] = 1
        keywords['cloned'] = 1
        uri = X.url('/%s/edit' % self.serverRole)
        uri.addKeywords(keywords)
        return redirect(uri)
    clone.isPublicForWeb = 1

    def delete(self, id):
        if id and self.hasObject(id):
            try:
                self.deleteObject(id)
            except faults.Fault:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
            uri = X.url('/%s' % self.serverRole)
            return redirect(uri)
        else:
            # TODO: error page when user tries to delete an object he can't
            return pageNotFound()
    delete.isPublicForWeb = 1

    def download(self, id, path):
        localId = extractLocalId(id)
        if localId == '__admin__':
            if not self.canGetAdmin():
                return accessForbidden()
            object = self.getAdmin()
        else:
            if not self.hasObject(id):
                return pageNotFound()
            if not self.canGetObject(id):
                return accessForbidden()
            object = self.getObject(id)
            rememberObject(id)

        req = context.getVar('req')

        ### Commented out since this check is now done generically in
        ### webhandler.
        ### if hasattr(object, 'modificationTime'):
        ###     lastModTime = time.gmtime(object.modificationTime)
        ###     req.headers_out['Last-Modified'] = time.strftime(
        ###         '%a, %d %b %Y %H:%M:%S %Z', lastModTime)
        ###     if req.headers_in.has_key('If-Modified-Since'):
        ###         try: 
        ###             t = time.strptime(
        ###                 req.headers_in['If-Modified-Since'][:25],
        ###                 '%a, %d %b %Y %H:%M:%S')
        ###             if lastModTime <= t:
        ###                 return HTTP_NOT_MODIFIED
        ###         except:
        ###             pass 

        uploadSlot = object.getSlotByPath(path)
        upload = uploadSlot.getValue()
        data = upload.getSlot('data', parentSlot = uploadSlot).getValue()
        dataFileName = upload.getSlot(
            'dataFileName', parentSlot = uploadSlot).getValue()
        if dataFileName:
            req.headers_out['Content-Disposition'] = \
                'inline; filename="%s"' % dataFileName
        req.headers_out['Content-Length'] = str(len(data))
        req.content_type = upload.getSlot(
            'dataType', parentSlot = uploadSlot).getValue()
        setHttpCookie()
        req.send_http_header()
        if req.method == 'HEAD':
            return OK
        req.write(data)
        return OK
    download.isPublicForWeb = 1

    def edit(self, id = '', again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if id and not self.hasObject(id):
            return pageNotFound()
        if id:
            if not self.canModifyObject(id):
                return accessForbidden()
            object = self.getObject(id)
            rememberObject(id)
        else:
            if not self.canAddObject():
                return accessForbidden()
            object = self.newObject(keywords)

        if id and not again:
            object.makeFieldsFromInstance(keywords)
        if not id and not again:
            object.initFields(keywords)
        object.repairFields(keywords)

        if not id:
            headerTitle = _(self.newObjectNameCapitalized)
        else:
            headerTitle = _('Editing %s - %s') % (
                _(self.objectNameCapitalized), object.getLabel())

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submit' % self.serverRole),
                      enctype = 'multipart/form-data', method = 'post')
        layout += form
        if keywords.has_key('cloned') and keywords['cloned']:
            form += X.input(name = 'cloned', type = 'hidden', value = '1')

        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
##         form += X.table(_class = 'page-body')(object.getEditLayout(
##             keywords))

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        if not id:
            actionButtonsBar += X.buttonInForm('create', 'createButton')
        else:
            if self.canDeleteObject(id):
                actionButtonsBar += X.buttonInForm('delete', 'deleteButton')
            actionButtonsBar += X.buttonInForm('modify', 'modifyButton')

        return writePageLayout(layout, headerTitle)
    edit.isPublicForWeb = 1

    def getObjectsLayout(self, partialObjects, objectIds,
                         slotNames = None):
        layout = None
        translationsProxy = getProxyForServerRole('translations')
        if len(partialObjects) > 0:
            if translationsProxy:
                labelsMultiCall = MultiCall()
                for objectId in objectIds:
                    partialObjects[objectId].getLabelTranslated(
                        context.getVar('readLanguages'),
                        multiCall = labelsMultiCall)

            layout = X.array()
            table = X.table(_class = 'objects-table')
            layout += table
            partialObject = partialObjects.values()[0]
            if slotNames:
                thead = X.thead()
                table += thead
                tr = X.tr()
                thead += tr
                tr += X.th()(X.nbsp)
                for slotName in slotNames:
                    slot = partialObject.getSlot(slotName)
                    tr += slot.getWidget().getHtmlColumnLabel(slot)
            tbody = X.tbody()
            table += tbody
            even = 0
            if translationsProxy:
                lazyLabels = labelsMultiCall.call()
            else:
                lazyLabels = objectIds # fastest way to get a list of same len
            for objectId, lazyLabel in zip(objectIds, lazyLabels):
                if translationsProxy:
                    label = lazyLabel()
                else:
                    label = partialObjects[objectId].getLabel()
                partialObject = partialObjects[objectId]
                fields = {}
                if slotNames:
                    for slotName in slotNames:
                        slot = partialObject.getSlot(slotName)
                        kind = slot.getKind()
                        if kind.hasToMakeFieldFromValue:
                            kind.makeFieldFromValue(
                                slot, fields, slot.getValue())
                    for slotName in slotNames:
                        slot = partialObject.getSlot(slotName)
                        kind = slot.getKind()
                        if kind.hasToRepairField:
                            kind.repairField(slot, fields)
                tr = X.tr(_class = ((even and 'even') or 'odd'))
                even = not even
                tbody += tr
                tr += X.td()(
                    X.a(href = X.url(partialObject.id))(label))
                if slotNames:
                    for slotName in slotNames:
                        slot = partialObject.getSlot(slotName)
                        widget = slot.getWidget()
                        widget.setInForm(0)
                        widget.setReadOnly(0)
                        tr += widget.getHtmlColumnValue(slot, fields)
            layout += X.br()
        return layout

    def getObjectsSectionLayout(self, partialObjects, intertitle,
                                slotNames = None):
        layout = None
        translationsProxy = getProxyForServerRole('translations')
        if len(partialObjects) > 0:
            if translationsProxy:
                labelsMultiCall = MultiCall()
                for partialObject in partialObjects:
                    partialObject.getLabelTranslated(
                        context.getVar('readLanguages'),
                        multiCall = labelsMultiCall)

            layout = X.array(X.h3()(intertitle))
            table = X.table(_class = 'objects-table')
            layout += table
            partialObject = partialObjects[0]
            thead = X.thead()
            table += thead
            tr = X.tr()
            thead += tr
            if slotNames:
                tr += X.th()(X.nbsp)
                for slotName in slotNames:
                    slot = partialObject.getSlot(slotName)
                    tr += slot.getWidget().getHtmlColumnLabel(slot)
            tbody = X.tbody()
            table += tbody
            even = 0
            if translationsProxy:
                lazyLabels = labelsMultiCall.call()
            else:
                lazyLabels = partialObjects # fastest way to get a list of same len
            for partialObject, lazyLabel in zip(partialObjects, lazyLabels):
                if translationsProxy:
                    label = lazyLabel()
                else:
                    label = partialObject.getLabel()
                fields = {}
                if slotNames:
                    for slotName in slotNames:
                        slot = partialObject.getSlot(slotName)
                        kind = slot.getKind()
                        if kind.hasToMakeFieldFromValue:
                            kind.makeFieldFromValue(
                                slot, fields, slot.getValue())
                    for slotName in slotNames:
                        slot = partialObject.getSlot(slotName)
                        kind = slot.getKind()
                        if kind.hasToRepairField:
                            kind.repairField(slot, fields)
                tr = X.tr(_class = ((even and 'even') or 'odd'))
                even = not even
                tbody += tr
                tr += X.td()(
                    X.a(href = X.url(partialObject.id))(label))
                if slotNames:
                    for slotName in slotNames:
                        slot = partialObject.getSlot(slotName)
                        widget = slot.getWidget()
                        widget.setInForm(0)
                        widget.setReadOnly(0)
                        tr += widget.getHtmlColumnValue(slot, fields)
            layout += X.br()
        return layout

    def getSortedIds(self, objects):
        translationsProxy = getProxyForServerRole('translations')
        labels = {}
        ids = objects.keys()
        if not translationsProxy:
            for k, v in objects.items():
                labels[k] = v.getLabel()
        else:
            labelsMultiCall = MultiCall()
            for object in objects.values():
                object.getLabelTranslated(
                    context.getVar('readLanguages'),
                    multiCall = labelsMultiCall)
            for id, lazyLabel in zip(ids, labelsMultiCall.call()):
                labels[id] = lazyLabel()
        ids.sort(
            lambda id1, id2, labels = labels:
            locale.strcoll(labels[id1], labels[id2]))
        return ids

    def getViewAboveButtonsBarLayout(self, object, fields):
        return None

    def getViewActionButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        if self.canDeleteObject(object.id):
            layout += X.buttonStandalone(
                'delete', X.url('/%s/submit' % urlId).add(
                'deleteButton', _('Delete')))
        if self.canCloneObject(object.id):
            layout += X.buttonStandalone(
                'clone', X.url('/%s/clone' % urlId))
        if self.canModifyObject(object.id):
            layout += X.buttonStandalone(
                'edit', X.url('/%s/edit' % urlId))
        return layout

    def getViewAllActionButtonsBarLayout(self):
        layout = X.array()
        if self.canAddObject():
            layout += X.buttonStandalone(
                'new', X.url('/%s/edit' % self.serverRole))
        return layout

    def getViewAllButtonsBarLayout(self):
        layout = X.div(_class = 'buttons-bar')
        layout += X.span(_class = 'navigation-buttons-bar')(
            self.getViewAllNavigationButtonsBarLayout())
        layout += X.span(_class = 'other-action-buttons-bar')(
            self.getViewAllOtherActionButtonsBarLayout())
        layout += X.span(_class = 'action-buttons-bar')(
            self.getViewAllActionButtonsBarLayout())
        return layout

    def getViewAllNavigationButtonsBarLayout(self):
        layout = X.array()
        userToken = context.getVar('userToken')
        if self.canGetAdmin() and userToken:
            layout += X.buttonStandalone(
                'settings',
                X.url('/%s/admin' % self.serverRole))
        return layout

    def getViewAllOtherActionButtonsBarLayout(self):
        return None

    def getViewBelowButtonsBarLayout(self, object, fields):
        return None

    def getViewButtonsBarLayout(self, object, fields):
        layout = X.div(_class = 'buttons-bar')
        layout += X.span(_class = 'navigation-buttons-bar')(
            self.getViewNavigationButtonsBarLayout(object, fields))
        layout += X.span(_class = 'other-action-buttons-bar')(
            self.getViewOtherActionButtonsBarLayout(object, fields))
        layout += X.span(_class = 'action-buttons-bar')(
            self.getViewActionButtonsBarLayout(object, fields))
        return layout

    def getViewNavigationButtonsBarLayout(self, object, fields):
        layout = X.array()
        userToken = context.getVar('userToken', default = '')
        if userToken:
            layout += X.buttonStandalone(
                'view-list',
                X.url('/%s' % self.serverRole))
        knownRoles = context.getVar('knownRoles')
        if 'workflows' in knownRoles:
            workflowWeb = getWebForServerRole('workflows')
            wfs = workflowWeb.getWorkflowsForServerRole(self.serverRole)
            kw = { 'dataObjectId': object.id }
            for w in wfs.values():
                urlId = getUrlId(w.id)
                label = getProxyForServerRole('translations').getTranslation(
                    w.getLabel(), w.id, 'self.getLabel()', w.language,
                    context.getVar('readLanguages'))
                layout += X.buttonStandalone(
                    label,
                    X.url('/%s/start' % urlId).addKeywords(kw))
        return layout

    def getViewOtherActionButtonsBarLayout(self, object, fields):
        return None

    image = download
    image.isPublicForWeb = 1

    def imageEdit(self, id = '', path = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if id:
            localId = extractLocalId(id)
            if localId == '__admin__':
                if not self.canModifyAdmin():
                    return accessForbidden()
                object = self.getAdmin()
            else:
                if not self.hasObject(id):
                    return pageNotFound()
                if not self.canModifyObject(id):
                    return accessForbidden()
                object = self.getObject(id)
                rememberObject(id)
        else:
            if not self.canAddObject():
                return accessForbidden()
            object = self.newObject(keywords)

        uploadSlot = object.getSlotByPath(path)
        upload = uploadSlot.getValue()
        data = upload.getSlot('data', parentSlot = uploadSlot).getField(
            keywords)
        req = context.getVar('req')
        dataFileName = upload.getSlot(
            'dataFileName', parentSlot = uploadSlot).getField(keywords)
        if dataFileName:
            req.headers_out['Content-disposition'] = \
                'attachment; filename="%s"' % dataFileName
        req.headers_out['Content-length'] = str(len(data))
        req.content_type = upload.getSlot(
            'dataType', parentSlot = uploadSlot).getField(keywords)
        setHttpCookie()
        req.send_http_header()
        if req.method == 'HEAD':
            return OK
        req.write(data)
        return OK
    imageEdit.isPublicForWeb = 1

    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id):
                return confirmDelete(
                    X.url('/%s/delete' % urlId),
                    objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        if not id:
            try:
                id = self.addObject(object)
            except faults.Fault:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['versionError'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        return redirect(X.url(id))
    submit.isPublicForWeb = 1

    def thumbnail(self, id, path, width = '', height = ''):
        localId = extractLocalId(id)
        if localId == '__admin__':
            if not self.canGetAdmin():
                return accessForbidden()
            object = self.getAdmin()
        else:
            if not self.hasObject(id):
                return pageNotFound()
            if not self.canGetObject(id):
                return accessForbidden()
##            object = self.getObjectThumbnail(id, width, height)
            object = self.getObject(id)
            rememberObject(id)
        try:
            width = int(width)
        except ValueError:
            width = 128
        try:
            height = int(height)
        except ValueError:
            height = 128

        uploadSlot = object.getSlotByPath(path)
        upload = uploadSlot.getValue()
        data = upload.getSlot('data', parentSlot = uploadSlot).getValue()

        if data and upload.isType('image'):
            import cStringIO
            # From Python Imaging Library.
            try:
                import Image as PILImage
            except ImportError:
                PILImage = None
            
            if PILImage:
                imageFile = cStringIO.StringIO(data)
                imageObject = PILImage.open(imageFile)
                imageObject.thumbnail((width, height))
                thumbnailFile = cStringIO.StringIO()
                imageObject.save(thumbnailFile, imageObject.format)
                data = thumbnailFile.getvalue()
                width, height = imageObject.size

        req = context.getVar('req')
        dataFileName = upload.getSlot(
            'dataFileName', parentSlot = uploadSlot).getValue()
        if dataFileName:
            req.headers_out['Content-disposition'] = \
                'attachment; filename="%s"' % dataFileName
        req.headers_out['Content-length'] = str(len(data))
        req.content_type = upload.getSlot(
            'dataType', parentSlot = uploadSlot).getValue()
        setHttpCookie()
        req.send_http_header()
        if req.method == 'HEAD':
            return OK
        req.write(data)
        return OK
    thumbnail.isPublicForWeb = 1

    def view(self, id):
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canGetObject(id):
            return accessForbidden()
        object = self.getObject(id)
        rememberObject(id)

        keywords = {}
        
        object.makeFieldsFromInstance(keywords)
        object.repairFields(keywords)

        label = object.getLabelTranslated(
            context.getVar('readLanguages'))

        layout = X.array()
        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(0)
        layout += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
##         layout += X.table(_class = 'page-body')(object.getViewLayout(
##             keywords))
        layout += self.getViewAboveButtonsBarLayout(object, keywords)
        layout += self.getViewButtonsBarLayout(object, keywords)
        layout += self.getViewBelowButtonsBarLayout(object, keywords)

        context.push(
            currentObject = WebAPI.GlasnostObject(object = object),
            )
        layout = writePageLayout(
            layout, '%s - %s' % (_(self.objectNameCapitalized), label))
        context.pull()
        return layout
    view.isPublicForWeb = 1

    def viewAll(self):
        if not self.canGetObjects():
            return accessForbidden()
        objects = self.getObjects()
        layout = X.array()
        ids = self.getSortedIds(objects)
        layout += self.getObjectsLayout(objects, ids)
        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _(self.objectsNameCapitalized))
    viewAll.isPublicForWeb = 1

    def viewAllPy(self, id, **keywords):
        req = context.getVar('req')
        keywords['objects'] = [WebAPI.GlasnostObject(id)
                               for id in self.getObjectIds()]
##        keywords['urlargs'] = WebAPI.UrlArgs(keywords)
        keywords['req'] = req
        stdO = sys.stdout
        sys.stdout = req
        try:
            execfile(context.getVar('PyFile'), keywords)
            sys.stdout = stdO
        except:
            sys.stdout = stdO
            raise apache.SERVER_RETURN, HTTP_INTERNAL_SERVER_ERROR
        return OK

    def viewAllTal(self, **keywords):
        file = context.getVar('TALFile')
        keywords['objects'] = [WebAPI.GlasnostObject(id)
                               for id in self.getObjectIds()]
##        keywords['urlargs'] = WebAPI.UrlArgs(keywords)
        return processTALFile(
            '',
            xtal = context.getVar('TALFileIsXTAL'),
            file = file,
            **keywords)
    #viewAllTal.isPublicForWeb = 1

    def viewTal(self, id, **keywords):
        file = context.getVar('TALFile')
        keywords['currentObject'] = WebAPI.GlasnostObject(id)
##        keywords['urlargs'] = WebAPI.UrlArgs(keywords)
        rememberObject(id)
        return processTALFile(
            '',
            xtal = context.getVar('TALFileIsXTAL'),
            file = file,
            **keywords)
    #viewTal.isPublicForWeb = 1
        
    def viewPy(self, id, **keywords):
        req = context.getVar('req')
        keywords['currentObject'] = WebAPI.GlasnostObject(id)
##        keywords['urlargs'] = WebAPI.UrlArgs(keywords)
        keywords['req'] = context.req
        for k, v in getTemplateVars().items():
            keywords[k] = v

        rememberObject(id)
        stdO = sys.stdout
        sys.stdout = req
        try:
            execfile(context.getVar('PyFile'), keywords)
            sys.stdout = stdO
        except:
            sys.stdout = stdO
            raise apache.SERVER_RETURN, HTTP_INTERNAL_SERVER_ERROR
        return OK
