# Glasnost 
# By: Odile Bnassy <obenassy@entrouvert.org>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@easter-eggs.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Operations Web"""

__version__ = '$Revision: 1.26 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.OperationsProxy import *

import kinds
from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import accounting


class AdminOperations(AdminMixin, AdminOperations):
    def getWeb(self):
        return getWebForServerRole('operations')
register(AdminOperations)


class Operation(ObjectWebMixin, Operation):
    comment_kind_widget_fieldLabel = N_('Comment')
    comment_kind_widget_cols = 40
    comment_kind_widget_rows = 5
    comment_kind_widgetName = 'TextArea'

    currency_kind_hasToSubmitField = 0
    currency_kind_isReadOnly = 1
    currency_kind_preference = 'currency'
    currency_kind_widget_fieldLabel = N_('Currency')
    currency_kind_widget_labels = accounting.currencyLabels
    currency_kind_widgetName = 'Select'

    lastEditorId_kind_hasToSubmitField = 0
    lastEditorId_kind_isReadOnly = 1
    lastEditorId_kind_widget_fieldLabel = N_('Last Editor')
    lastEditorId_kind_widgetName = 'SelectId'

    ledgerId_kind_widget_fieldLabel = N_('Ledger')
    ledgerId_kind_widgetName = 'SelectId'

    membersSet_kind_itemKind_value_widgetName = 'SelectId'
    membersSet_kind_widget_apply = 1
    membersSet_kind_widget_fieldLabel = N_('Events')
    membersSet_kind_widgetName = 'Multi'

    partnerId_kind_widget_fieldLabel = N_('Partner')
    partnerId_kind_widgetName = 'SelectId'

    title_kind_widget_fieldLabel = N_('Title')
    #title_kind_widget_maxLength = 40
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'creationTime' 'lastEditorId', 'modificationTime',
                'readersSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('operations')
register(Operation)


class OperationsWeb(ObjectsWebMixin, OperationsProxy):
    def addEvent(self, id, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if not self.hasObject(id):
            return pageNotFound()
        eventsWeb = getWebForServerRole('events')
        if not membersWeb.canAddObject() \
           or not self.canModifyObject(id):
            return accessForbidden()

        member = membersWeb.newThing('object', 'Event')
        if not again:
            member.initFields(keywords)
        keywords["operationId"] = id
        member.repairFields(keywords)

        layout = X.array()
        layout += member.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/operations/submitEvent'),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form

        slot = slots.Root(member)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonInForm('create', 'createButton')

        return writePageLayout(layout, _('New Event'))
    addEvent.isPublicForWeb = 1

    def clone(self, id):
        if not id or not self.hasObject(id):
            return pageNotFound()
        if not self.canCloneObject(id):
            return accessForbidden()
        object = self.getObject(id)
        keywords = {}
        object.makeFieldsFromInstance(keywords)
        del keywords['id']
        for key in keywords.keys():
            if key.startswith('membersSet'):
                del keywords[key]
        keywords['again'] = 1
        keywords['cloned'] = 1
        uri = X.url('/%s/edit' % self.serverRole)
        uri.addKeywords(keywords)
        return redirect(uri)
    clone.isPublicForWeb = 1

    def getViewOtherActionButtonsBarLayout(self, object, fields):
        urlId = getUrlId(object.id)
        layout = X.array()
        layout += ObjectsWebMixin.getViewOtherActionButtonsBarLayout(
            self, object, fields)
        membersWeb = getWebForServerRole('events')
        if membersWeb.canAddObject() and self.canModifyObject(object.id):
            layout += X.buttonStandalone(
                'Add Event',
                X.url('/%s/addEvent' % urlId))
        return layout

    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id):
                return confirmDelete(
                    X.url('/%s/delete' % urlId),
                    objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            if keywords.has_key('cloned'):
                membersWeb = getWebForServerRole('events')
                memberKeys = [
                    x
                    for x in keywords.keys()
                    if x.startswith('membersSet') and x != 'membersSet_count']
                memberIdsToDuplicate = filter(None, [
                    keywords[key] for key in memberKeys])
                if memberIdsToDuplicate:
                    membersToDuplicate = [membersWeb.getObject(memberId)
                                          for memberId in memberIdsToDuplicate]
                    newMembers = membersToDuplicate[:]
                    for member in newMembers:
                        member.getSlot('id').setValue(None)
                        member.getSlot('membersSet').setValue([])
                    try:
                        newMemberIds = [membersWeb.addObject(member)
                                        for member in newMembers]
                    except:
                        if context.getVar('debug'):
                            raise
                        return accessForbidden()
                    for counter in range(len(newMemberIds)):
                        keywords["membersSet_%s" % counter] = newMemberIds[
                            counter]
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        if not id:
            try:
                id = self.addObject(object)
            except faults.Fault:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['versionError'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        return redirect(X.url(id))
    submit.isPublicForWeb = 1

    def submitEvent(self, id, **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id

        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        membersWeb = getWebForServerRole('members')
        member = membersWeb.newThing('object', 'Event')
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            del keywords['id'] # Not the event id.
            member.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/addEvent' % getUrlId(id))
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            memberId = membersWeb.addObject(member)
        except faults.Fault:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        object = self.getObject(id)
        if object.membersSet is None:
            object.membersSet = []
        object.membersSet.append(memberId)
        try:
            self.modifyPartialObject(
                object, object.getSlotToModifyNames())
        except faults.Fault:
            try:
                membersWeb.deleteObject(eventId)
            except:
                pass
            return accessForbidden()
        return redirect(X.url(memberId))
    submitEvent.isPublicForWeb = 1

