# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Organizations Web"""

__version__ = '$Revision: 1.15 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.OrganizationsProxy import *
from glasnost.proxy.TranslationsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import geography


class AdminOrganizations(AdminMixin, AdminOrganizations):
    def getWeb(self):
        return getWebForServerRole('organizations')
register(AdminOrganizations)


class Organization(ObjectWebMixin, Organization):
    address_kind_widget_fieldLabel = N_('Postal Address')
    address_kind_widget_cols = 40
    address_kind_widget_rows = 3
    address_kind_widgetName = 'TextArea'

    city_kind_widget_fieldLabel = N_('City')
    #city_kind_widget_maxLength = 100
    city_kind_widget_size = 40
    city_kind_widgetName = 'InputText'

    contactIds_kind_itemKind_value_defaultValue = 'user'
    contactIds_kind_itemKind_value_widgetName = 'SelectId'
    contactIds_kind_widget_fieldLabel = N_('Contacts')
    contactIds_kind_widgetName = 'Multi'

    country_kind_defaultValue = 'fr'
    country_kind_widget_fieldLabel = N_('Country')
    country_kind_widget_labels = geography.countryLabels
    country_kind_widgetName = 'Select'

    description_kind_widget_fieldLabel = N_('Description')
    description_kind_widget_cols = 75
    description_kind_widget_colSpan = 2
    description_kind_widget_rows = 8
    description_kind_widgetName = 'TextArea'

    email_kind_widget_fieldLabel = N_('Email')
    #email_kind_widget_maxlength = 100
    email_kind_widget_size = 40
    email_kind_widgetName = 'InputText'

    name_kind_widget_fieldLabel = N_('Name')
    #name_kind_widget_maxLength = 100
    name_kind_widget_size = 40
    name_kind_widgetName = 'InputText'

    official_kind_defaultValue = None
    official_kind_widget_fieldLabel = N_('Official Status')
    official_kind_widget_labels = {
        'nonprofit': N_('Non Profit Organization'),
        'cooperative': N_('Cooperative'),
        'company': N_('Company'),
        'mutual': N_('Mutual Benefit Society'),
        'local': N_('Local Authority'),
        }
    official_kind_widgetName = 'XSelect'

    phone_kind_defaultValue = '+33(0)'
    phone_kind_widget_fieldLabel = N_('Phone')
    phone_kind_widget_size = 20
    phone_kind_widgetName = 'InputText'

    webSite_kind_defaultValue = 'http://'
    webSite_kind_widget_fieldLabel = N_('Organization Web Site')
    webSite_kind_widget_maxLength = 100
    webSite_kind_widget_size = 40
    webSite_kind_widgetName = 'InputText'

    zip_kind_widget_fieldLabel = N_('Zip Code')
    #zip_kind_widget_maxLength = 100
    zip_kind_widget_size = 8
    zip_kind_widgetName = 'InputText'

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'creationTime', 'modificationTime', 'readersSet',
                'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('organizations')
register(Organization)


class OrganizationsWeb(ObjectsWebMixin, OrganizationsProxy):
    pass

