# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost PELActions Web"""

__version__ = '$Revision: 1.7 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.PELActionsProxy import *
from glasnost.proxy.TranslationsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import widgets


class AdminPELActions(AdminMixin, AdminPELActions):
    def getWeb(self):
        return getWebForServerRole('pelactions')
register(AdminPELActions)


class FinancialTable(widgets.BaseWidget):
    thingName = 'FinancialTable'

    def getHtmlField(self, parentSlot, fields, **keywords):
        assert fields is not None
        assert 'inForm' in keywords.keys()
        inForm = keywords['inForm']
        layout = tr = X.tr(_class = 'field-value')
        td = X.td(colspan = '2')
        td += X.h3()('Budget')
        layout += td
        table = X.table(_class = 'financial-table')
        td += table
        td += X.br()

        thead = X.thead()
        table += thead
        tr = X.tr()
        thead += tr
        tr += X.th(scope = 'col', colspan = 2)('Charges')
        tr += X.th(scope = 'col', colspan = 2)('Produits')
        tbody = X.tbody()
        tbody += X.tr((
            X.td(width=10)(X.nbsp), X.td()(X.nbsp),
            X.td(width=10)(X.nbsp), X.td()(X.nbsp)))
        table += tbody
        expenseSlotNames = ['purchases', 'salaries', 'organizationCosts', '']
        resourceSlotNames = ['contributions', 'ordinaryFundings',
                             'otherFundings', 'requiredSubsidy']
        total1, total2 = 0, 0
        for i in range(4):
            slotName1 = expenseSlotNames[i]
            if not slotName1:
                pass
            else:
                slot1 = parentSlot.getSlot(slotName1)
                kind1 = slot1.getKind()
                widget1 = kind1.getModelWidget(slot1)
                widget1.setInForm(inForm)
                if kind1.name == 'Integer Number':
                    fieldValue1 = slot1.getField(fields, default = '')
                    if fieldValue1:
                        try:
                            total1 += string.atoi(fieldValue1)
                        except:
                            pass
                elif kind1.name == 'Mapping':
                    for fieldName in fields.keys():
                        if fieldName.startswith(slotName1):
                            value = fields[fieldName]
                            try:
                                total1 += string.atoi(value)
                            except:
                                pass
                else:
                    raise Exception('Value error in %s' % slotName1)
            slotName2 = resourceSlotNames[i]
            if not slotName2:
                pass
            else:
                slot2 = parentSlot.getSlot(slotName2)
                kind2 = slot2.getKind()
                widget2 = kind2.getModelWidget(slot2)
                widget2.setInForm(inForm)
                if kind2.name == 'Integer Number':
                    fieldValue2 = slot2.getField(fields, default = '')
                    if fieldValue2:
                        try:
                            total2 += string.atoi(fieldValue2)
                        except:
                            pass
                elif kind2.name == 'Mapping':
                    for fieldName in fields.keys():
                        if fieldName.startswith(slotName2):
                            value = fields[fieldName]
                            try:
                                total2 += string.atoi(value)
                            except:
                                pass
                else:
                    raise Exception('Value error in %s' % slotName2)
            tr = X.tr()
            tbody += tr
            if not slotName1:
                tr += X.td()(X.nbsp)
                tr += X.td()(X.nbsp)
            else:
                tr += widget1.getHtmlFieldLabel(slot1)
                tr += widget1.getHtmlFieldValue(slot1, fields, **keywords)
            if not slotName2:
                tr += X.td()(X.nbsp)
                tr += X.td()(X.nbsp)
            else:                
                tr += widget2.getHtmlFieldLabel(slot2)
                tr += widget2.getHtmlFieldValue(slot2, fields, **keywords)
        if inForm:
            tbody += X.tr(X.td(
                _class = 'field-value-buttons-bar', colspan = '4')(
                X.buttonInForm('compute', 'applyButton')))
        if total1:
            total1 = str(total1)
        else:
            total1 = '0'
        if total2:
            total2 = str(total2)
        else:
            total2 = '0'
        totalSlot1 = parentSlot.getSlot('totalExpenses')
        totalSlot1.setField(fields, total1)
        kind1 = totalSlot1.getKind()
        widget1 = kind1.getModelWidget(totalSlot1)
        totalSlot2 = parentSlot.getSlot('totalResources')
        totalSlot2.setField(fields, total2)
        kind2 = totalSlot2.getKind()
        widget2 = kind2.getModelWidget(totalSlot2)
        tr = X.tr(_class='total')
        tbody += tr
        tr += widget1.getHtmlFieldLabel(totalSlot1)
        tr += widget1.getHtmlFieldValue(totalSlot1, fields, **keywords)
        tr += widget2.getHtmlFieldLabel(totalSlot2)
        tr += widget2.getHtmlFieldValue(totalSlot2, fields, **keywords)  
        return layout
register(FinancialTable)

    
class PELAction(ObjectWebMixin, PELAction):
    abstract_kind_widget_fieldLabel = 'Orientation'
    abstract_kind_widget_cols = 75
    abstract_kind_widget_colSpan = 2
    abstract_kind_widget_rows = 4
    abstract_kind_widgetName = 'TextArea'

    actionStatus_kind_defaultValue = 'new'
    actionStatus_kind_widget_fieldLabel = 'Stade'
    actionStatus_kind_widget_labels = {
        'new' : 'Activite nouvelle',
        'renewed' : 'Reconduction',
        'continued' : 'Convention pluri-annuelle',
        }
    actionStatus_kind_widgetName = 'Select'
    
    ages_kind_fieldsCountMin = 2
    ages_kind_itemKind_value_widgetName = 'Select'
    ages_kind_widget_fieldLabel = N_('Age')
    ages_kind_widgetName = 'Multi'
    
    area_kind_fieldsCountMin = 2
    area_kind_itemKind_value_widgetName = 'Select'
    area_kind_itemKind_value_widget_labels = {
        'insidearea' : N_('Inside Area'),
        'outsidearea' : N_('Outside Area'),
        }
    area_kind_widget_fieldLabel = 'Venant de'
    area_kind_widgetName = 'Multi'

    contactIds_kind_itemKind_value_defaultValue = 'user'
    contactIds_kind_itemKind_value_widgetName = 'SelectId'
    contactIds_kind_widget_fieldLabel = N_('Contacts')
    contactIds_kind_widgetName = 'Multi'

    contributions_kind_fieldsCountMin = 3
    contributions_kind_keyKind_value_defaultValues = [
        'Contribution structure',
        'Contribution usagers',
        ]
    contributions_kind_keyKind_value_widget_size = 20
    contributions_kind_keyKind_value_widgetName = 'InputText'
    contributions_kind_valueKind_value_defaultValue = 0
    contributions_kind_valueKind_value_widget_size = 5
    contributions_kind_valueKind_value_widgetName = 'InputText'
    contributions_kind_widget_fieldLabel = 'Contributions'
    contributions_kind_widgetName = 'Mapping'

    description_kind_widget_fieldLabel = 'Descriptif de l\'action'
    description_kind_widget_cols = 75
    description_kind_widget_colSpan = 2
    description_kind_widget_rows = 4
    description_kind_widgetName = 'TextArea'

    designPartners_kind_widget_fieldLabel = 'Partenaires dans l\'elaboration'
    designPartners_kind_widget_cols = 75
    designPartners_kind_widget_colSpan = 2
    designPartners_kind_widget_rows = 4
    designPartners_kind_widgetName = 'TextArea'

    duration_kind_widget_fieldLabel = 'Duree d\'une seance'
    duration_kind_widgetName = 'Duration'

    endDate_kind_widget_fieldLabel = N_('Ending')
    endDate_kind_widgetName = 'Time'

    evaluationPartners_kind_widget_fieldLabel = 'Partenaires dans l\'evaluation'
    evaluationPartners_kind_widget_cols = 75
    evaluationPartners_kind_widget_colSpan = 2
    evaluationPartners_kind_widget_rows = 4
    evaluationPartners_kind_widgetName = 'TextArea'

    expectedResults_kind_widget_fieldLabel = 'Resultats attendus'
    expectedResults_kind_widget_cols = 75
    expectedResults_kind_widget_colSpan = 2
    expectedResults_kind_widget_rows = 4
    expectedResults_kind_widgetName = 'TextArea'

    goals_kind_fieldsCountMin = 2
    goals_kind_itemKind_value_widget_labels = {
        'freetime' : 'Occupation educative du temps libre',
        'autonomy' : 'Apprentissage de l\'autonomie',
        'remedial' : 'Rattrapage scolaire',
        'socialisation' : 'Socialisation',
        'appropriation' : 'Appropriation des savoirs',
        'physical' : 'Developpements des capacites physiques',
        'effort' : 'Apprentissage du sens de l\'effort',
        'localenv' : 'Ouverture sur l\'environnement proche',
        'remoteenv' : 'Ouverture sur l\'environnement eloigne',
        'citizenship' : 'Comportement citoyen',
        'violence' : 'Diminution de la violence',
        'parents' : 'Aide aux parents',
        'generationmix' : 'Melange des generations',
        'expression' : 'Developper les capacites d\'expression',
        'selfrespect' : 'Restaurer l\'estime de soi',
        'locallife' : 'Developper la vie sociale du quartier',
        'localbonds' : 'Nouer des liens avec le quartier',
        'none' : 'Aucun',
        }
    goals_kind_itemKind_value_widgetName = 'XSelect'
    goals_kind_widget_fieldLabel = 'Objectifs'
    goals_kind_widgetName = 'Multi'

    implementationPartners_kind_widget_fieldLabel = 'Partenaires dans la mise en oeuvre'
    implementationPartners_kind_widget_cols = 75
    implementationPartners_kind_widget_colSpan = 2
    implementationPartners_kind_widget_rows = 4
    implementationPartners_kind_widgetName = 'TextArea'

    means_kind_widget_fieldLabel = 'Moyens'
    means_kind_widget_cols = 75
    means_kind_widget_colSpan = 2
    means_kind_widget_rows = 4
    means_kind_widgetName = 'TextArea'

    name_kind_widget_fieldLabel = 'Nom de l\'action'
    name_kind_widget_size = 40
    name_kind_widgetName = 'InputText'

    ordinaryFundings_kind_fieldsCountMin = 4
    ordinaryFundings_kind_keyKind_value_defaultValues = [
        'Ville',
        'CAF',
        'Conseil General',
        'Etat',
        ]
    ordinaryFundings_kind_keyKind_value_widget_size = 20
    ordinaryFundings_kind_keyKind_value_widgetName = 'InputText'
    ordinaryFundings_kind_valueKind_value_defaultValue = 0
    ordinaryFundings_kind_valueKind_value_widget_size = 5
    ordinaryFundings_kind_valueKind_value_widgetName = 'InputText'
    ordinaryFundings_kind_widget_fieldLabel = 'Financements ordinaires'
    ordinaryFundings_kind_widgetName = 'Mapping'

    organizationCosts_kind_defaultValue = 0
    organizationCosts_kind_widget_fieldLabel = 'Frais de structure (lies au projet)'
    organizationCosts_kind_widget_size = 5
    organizationCosts_kind_widgetName = 'InputText'

    otherExpenses_kind_fieldsCountMin = 2
    otherExpenses_kind_keyKind_value_widget_size = 20
    otherExpenses_kind_keyKind_value_widgetName = 'InputText'
    otherExpenses_kind_valueKind_value_widget_size = 5
    otherExpenses_kind_valueKind_value_widgetName = 'InputText'
    otherExpenses_kind_widget_fieldLabel = N_('Other')
    otherExpenses_kind_widgetName = 'Mapping'

    otherFundings_kind_fieldsCountMin = 3
    otherFundings_kind_keyKind_value_defaultValues = [
        'FAS',
        'Region',
        'Sponsors',
        ]
    otherFundings_kind_keyKind_value_widget_size = 20
    otherFundings_kind_keyKind_value_widgetName = 'InputText'
    otherFundings_kind_valueKind_value_defaultValue = 0
    otherFundings_kind_valueKind_value_widget_size = 5
    otherFundings_kind_valueKind_value_widgetName = 'InputText'
    otherFundings_kind_widget_fieldLabel = 'Autres recettes'
    otherFundings_kind_widgetName = 'Mapping'

    owner_kind_widget_fieldLabel = N_('Organization')
    owner_kind_widget_size = 40
    owner_kind_widgetName = 'InputText'

    participantCount_kind_widget_fieldLabel = 'Nombre de participants'
    participantCount_kind_widget_size = 10
    participantCount_kind_widgetName = 'InputText'

    periodicity_kind_widget_fieldLabel = N_('Periodicity')
    periodicity_kind_widget_labels = {
        'lunch' : N_('Lunch Time'),
        'wednesday' : N_('Wednesday'),
        'saturday' : N_('Saturday'),
        'schooldays' : N_('School Days'),
        'holiday' : N_('Holidays'),
        'vacation' : N_('Summer Vacation'),
        }
    periodicity_kind_widgetName = 'Select'

    presentationDate_kind_widget_fieldLabel = 'Presente au groupe de quartier le'
    presentationDate_kind_widgetName = 'Time'

    purchases_kind_fieldsCountMin = 3
    purchases_kind_keyKind_value_defaultValues = [
        'Achats lies au projet',
        'Prestations externes',
        ]
    purchases_kind_keyKind_value_widget_size = 20
    purchases_kind_keyKind_value_widgetName = 'InputText'
    purchases_kind_valueKind_value_defaultValue = 0
    purchases_kind_valueKind_value_widget_size = 5
    purchases_kind_valueKind_value_widgetName = 'InputText'    
    purchases_kind_widget_fieldLabel = 'Achats (lies au projet, prestations externes...)'
    purchases_kind_widgetName = 'Mapping'

    requiredSubsidy_kind_defaultValue = 0
    requiredSubsidy_kind_widget_fieldLabel = 'Subvention demandee'
    requiredSubsidy_kind_widget_size = 5
    requiredSubsidy_kind_widgetName = 'InputText'

    salaries_kind_fieldsCountMin = 3
    salaries_kind_keyKind_value_defaultValues = [
        'Salaires',
        'Charges sociales',
        ]
    salaries_kind_keyKind_value_widget_size = 20
    salaries_kind_keyKind_value_widgetName = 'InputText'
    salaries_kind_valueKind_value_defaultValue = 0
    salaries_kind_valueKind_value_widget_size = 5
    salaries_kind_valueKind_value_widgetName = 'InputText'
    salaries_kind_widget_fieldLabel = 'Frais de personnel'
    salaries_kind_widgetName = 'Mapping'

    startDate_kind_widget_fieldLabel = N_('Starting')
    startDate_kind_widgetName = 'Time'

    targetAudience_kind_widget_fieldLabel = 'Public vise'
    targetAudience_kind_widget_cols = 75
    targetAudience_kind_widget_colSpan = 2
    targetAudience_kind_widget_rows = 4
    targetAudience_kind_widgetName = 'TextArea'

    territory_kind_widget_fieldLabel = N_('Territory')
    territory_kind_widget_labels = {
        'basrivedroite' : 'Bas Rive Droite',
        'hautrivedroite': 'Haut Rive Droite',
        'bellevue' : 'Bellevue',
        'saintmarc': 'Saint-Marc',
        'centreville' : 'Centre Ville',
        'europe' : 'Europe',
        'lambezellec' : 'Lambezellec',
        }
    territory_kind_widgetName = 'Select'

    title_kind_widget_fieldLabel = N_('Title')
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    totalExpenses_kind_defaultValue = 0
    totalExpenses_kind_widget_fieldLabel = 'Total'
    totalExpenses_kind_widget_size = 6
    totalExpenses_kind_widgetName = 'InputText'

    totalResources_kind_defaultValue = 0
    totalResources_kind_widget_fieldLabel = 'Total'
    totalResources_kind_widget_size = 6
    totalResources_kind_widgetName = 'InputText'

    def getEditLayout(self, fields, parentSlot = None):
        layout = X.array()
        slotNames = self.getEditLayoutSlotNames(
            fields, parentSlot = parentSlot)
        hiddenSlotNames = self.getEditLayoutHiddenSlotNames(
            fields, parentSlot = parentSlot)
        readOnlySlotNames = self.getEditLayoutReadOnlySlotNames(
            fields, parentSlot = parentSlot)
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if slotName == 'purchases':
                widget = self.newThing('widget', 'FinancialTable')
            else:
                widget = kind.getModelWidget(slot)
            if slotName in hiddenSlotNames:
                if not kind.isConstant:
                    hidden = widget.getModelHiddenLayout(slot, fields)
                    layout += X.tr(X.td(hidden))
            else:
                if kind.isConstant:
                    widget.setInForm(0)
                else:
                    widget.setInForm(1)
                    widget.setReadOnly(slotName in readOnlySlotNames)
                if slotName == 'purchases':
                    layout += widget.getHtmlField(self, fields, inForm = 1)
                else:
                    layout += widget.getHtmlField(slot, fields)
        return layout

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in (
                'salaries', 'organizationCosts', 'totalExpenses',
                'contributions', 'ordinaryFundings', 'otherFundings',
                'requiredSubsidy', 'totalResources'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        for slotName in (
            'creationTime', 'modificationTime', 'readersSet'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in ('authorsSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getViewLayout(self, fields, parentSlot = None):
        layout = X.array()
        slotNames = self.getViewLayoutSlotNames(
            fields, parentSlot = parentSlot)
        hiddenSlotNames = self.getViewLayoutHiddenSlotNames(
            fields, parentSlot = parentSlot)
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if slotName in hiddenSlotNames:
                continue
            if slotName == 'purchases':
                widget = self.newThing('widget', 'FinancialTable')
            else:
                widget = kind.getModelWidget(slot)
            widget.setInForm(0)
            if slotName == 'purchases':
                layout += widget.getHtmlField(self, fields, inForm = 0)
            else:
                layout += widget.getHtmlField(slot, fields)
        return layout

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in (
                'salaries', 'organizationCosts', 'totalExpenses',
                'contributions', 'ordinaryFundings', 'otherFundings',
                'requiredSubsidy', 'totalResources'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        for slotName in ('readersSet'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'authorsSet', 'creationTime', 'modificationTime', 'readersSet',
                'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('pelactions')
register(PELAction)


class PELActionsWeb(ObjectsWebMixin, PELActionsProxy):
    pass

