# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost PageName Web"""

__version__ = '$Revision: 1.20 $'[11:-2]


import glasnost.common.faults as faults

from glasnost.proxy.PageNamesProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *


class AdminPageNames(AdminMixin, AdminPageNames):
    def getWeb(self):
        return getWebForServerRole('pagenames')
register(AdminPageNames)


class PageName(ObjectWebMixin, PageName):
    mappedId_kind_widget_fieldLabel = N_('Mapped Id')
    mappedId_kind_widgetName = 'SelectId'

    name_kind_widget_fieldLabel = N_('Name')
    #name_kind_widget_maxLength = 40
    name_kind_widget_size = 40
    name_kind_widgetName = 'InputText'

    def getWeb(self):
        return getWebForServerRole('pagenames')
register(PageName)


class PageNamesWeb(ObjectsWebMixin, PageNamesProxy):
    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id):
                return confirmDelete(
                    X.url('/%s/delete' % urlId),
                    objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        if not id:
            try:
                id = self.addObject(object)
            except faults.DuplicateValue:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['name_error'] = 'valueAlreadyUsed'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['versionError'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except faults.DuplicateValue:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['name_error'] = 'valueAlreadyUsed'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        return redirect(X.url(id))
    submit.isPublicForWeb = 1
