# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost People Web"""

__version__ = '$Revision: 1.44 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.context as context

from glasnost.proxy.PeopleProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import widgets


class AdminPeople(AdminMixin, AdminPeople):
    authenticationServerId_kind_widget_fieldLabel = N_(
        'Authentication Server ID')

    defaultGroupId_kind_widget_fieldLabel = N_('Default Group')
    defaultGroupId_kind_widgetName = 'SelectId'

    userCanChoosePassword_kind_defaultValue = 0
    userCanChoosePassword_kind_widget_fieldLabel = N_('Password')
    userCanChoosePassword_kind_widget_labels = {
        '0': N_('Automatically Generated'),        
        '1': N_('User Choice'),
        }
    userCanChoosePassword_kind_widgetName = 'InputCheckBox'

    writersSet_kind_itemKind_value_widget_noneLabel = N_('Everybody')

    def getWeb(self):
        return getWebForServerRole('people')
register(AdminPeople)


class InputPasswordWidget(widgets.InputPassword):
    def getHtmlValue(self, slot, fields, **keywords):
        layout = None
        peopleWeb = getWebForServerRole('people')
        if not self.isInForm():
            object = slot.getObject()
            if peopleWeb.canEmailPassword(object.id):
                layout = X.buttonStandalone(
                    'send-by-email',
                    X.url('/%s/confirmEmailPassword' % getUrlId(object.id)))
        elif self.isReadOnly(slot) \
                 or not peopleWeb.getAdmin().userCanChoosePassword:
            layout = X.array()
            layout += self.getModelHiddenLayout(slot, fields)
            changeButtonName = slot.getFieldOptionName('changeButton')
            layout += X.buttonInForm('change-password', changeButtonName)
        else:
            fieldName = slot.getFieldName()
            againFieldName = slot.getFieldOptionName('again')
            fieldValue = slot.getField(fields, default = '')
            againFieldValue = slot.getFieldOption(
                fields, 'again', default = '')
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            inputAttributes = {}
            if self.maxLength:
                inputAttributes['maxlength'] = self.maxLength
            if self.size:
                inputAttributes['size'] = self.size
            layout += X.input(name = fieldName, type = 'password',
                              value = fieldValue, **inputAttributes)
            layout += X.input(name = againFieldName, type = 'password',
                              value = againFieldValue, **inputAttributes)
        return layout

    def getModelHiddenLayout(self, slot, fields):
        fieldName = slot.getFieldName()
        againFieldName = slot.getFieldOptionName('again')
        fieldValue = slot.getField(fields, default = '')
        againFieldValue = slot.getFieldOption(fields, 'again', default = '')
        return X.array(
            X.input(name = fieldName, type = 'hidden', value = fieldValue),
            X.input(name = againFieldName, type = 'hidden',
                    value = againFieldValue),
            )
widgets.register(InputPasswordWidget)


class Person(ObjectWebMixin, Person):
    email_kind_widget_fieldLabel = N_('Email')
    #email_kind_widget_maxLength = 40
    email_kind_widget_size = 40
    email_kind_widgetName = 'Email' #'InputText'

    fingerprint_kind_widget_fieldLabel = N_('GnuPG Fingerprint')
    #fingerprint_kind_widget_maxLength = 40
    fingerprint_kind_widget_size = 40
    fingerprint_kind_widgetName = 'InputText'

    firstName_kind_widget_fieldLabel = N_('First Name')
    #firstName_kind_widget_maxLength = 40
    firstName_kind_widget_size = 40
    firstName_kind_widgetName = 'InputText'

    lastName_kind_widget_fieldLabel = N_('Last Name')
    #lastName_kind_widget_maxLength = 40
    lastName_kind_widget_size = 40
    lastName_kind_widgetName = 'InputText'

    login_kind_widget_fieldLabel = N_('Username')
    login_kind_widget_maxLength = 15
    login_kind_widget_size = 15
    login_kind_widgetName = 'InputText'

    loginTime_kind_isReadOnly = 1
    loginTime_kind_widget_fieldLabel = N_('Last Login Time')
    loginTime_kind_widget_maxLength = 19
    loginTime_kind_widget_size = 19
    loginTime_kind_widgetName = 'InputText'

    nickname_kind_widget_fieldLabel = N_('Nickname')
    #nickname_kind_widget_maxLength = 40
    nickname_kind_widget_size = 40
    nickname_kind_widgetName = 'InputText'

    password_kind_hasToSubmitField = 0
    password_kind_isNewPassword = 1
    password_kind_widget_fieldLabel = N_('Password')
    password_kind_widget_maxLength = 15
    password_kind_widget_size = 15
    password_kind_widgetName = 'InputPasswordWidget'

    voteTokens_kind_isConstant = 1
    voteTokens_kind_isHidden = 1

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userId = context.getVar('userId', default = '')
        if self.id is None or userId != self.id:
            slotName = 'password'
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userId = context.getVar('userId', default = '')
        userToken = context.getVar('userToken', default = '')
        if not userToken:
            slotName = 'email'
            if slotName in slotNames:
                slotNames.remove(slotName)
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in ('creationTime', 'modificationTime', 'loginTime'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('people')

    def submitFields(self, fields, parentSlot = None):
        ObjectWebMixin.submitFields(self, fields, parentSlot = parentSlot)
        userId = context.getVar('userId', default = '')
        if self.id is not None and userId == self.id:
            slot = self.getSlot('password', parentSlot = parentSlot)
            slot.getKind().submitField(slot, fields)
register(Person)


class PeopleWeb(ObjectsWebMixin, PeopleProxy):
    _emailPassword = PeopleProxy.emailPassword

    def confirmEmailPassword(self, id):
        if not self.hasObject(id):
            return pageNotFound()
        person = self.getObject(id)
        if not self.canEmailPassword(person.id):
            # TODO: error page when user tries to email a password he can't
            return pageNotFound()
        layout = X.array()
        layout += X.div(_class = 'alert')(
            _('Are you sure you want to send the password by email to %s?') \
            % person.getFullName())
        layout += X.br()
        form = X.form(
            action = X.url('/%s/emailPassword' % getUrlId(id)),
            enctype = 'multipart/form-data', method = 'post')
        form += X.buttonInForm('send-by-email', 'sendByEmail')
        layout += form
        return writePageLayout(layout,
                                _('Confirm Sending of Password by Email'),
                                canCache = 0)
    confirmEmailPassword.isPublicForWeb = 1

    def emailPassword(self, id):
        if not self.hasObject(id):
            return pageNotFound()
        person = self.getObject(id)
        try:
            self._emailPassword(id)
        except faults.Fault:
            return failure(
                _('The sending of the password by email to %s has failed!') \
                % person.getFullName(),
                X.url(id))
        return success(
            _('The password has been sent by email to %s!') \
            % person.getFullName(),
            X.url(id))
    emailPassword.isPublicForWeb = 1

    def getSortedIds(self, objects):
        labels = {}
        for object in objects.values():
            labels[object.id] = object.getSortString()
        ids = objects.keys()
        ids.sort(
            lambda id1, id2, labels = labels:
            cmp(labels[id1], labels[id2]))
        return ids

    def getViewAboveButtonsBarLayout(self, object, fields):
        ballotsWeb = getWebForServerRole('ballots')
        electionsWeb = getWebForServerRole('elections')
        layout = X.array()
        layout += ObjectsWebMixin.getViewAboveButtonsBarLayout(
            self, object, fields)

        def getVotesSectionLayout(person, votes, elections, intertitle):
            layout = None
            if len(elections) > 0:
                layout = X.array( X.h3()(intertitle) )
                layout += X.br()
                table = X.table()
                layout += table
                for election in elections:
                    if not person.id in election.getVoterIds():
                        continue
                    tr = X.tr(X.td(X.a(href = X.url(election.id))(
                        election.getLabelTranslated(
                        context.getVar('readLanguages')))))
                    table += tr
                    if not votes.has_key(election.id):
                        tr += X.td(_('Abstention'))
                    else:
                        vote = votes[election.id]
                        if vote == 'secret':
                            tr += X.td(_('Secret Ballot'))
                        else:
                            tr += X.td(X.a(href = X.url(vote.id))(
                                X.asIs(vote.getLabelLine(election))))
            return layout

        userId = context.getVar('userId', default = '')
        knownRoles = context.getVar('knownRoles')
        if userId and 'ballots' in knownRoles and 'elections' in knownRoles:
            # Only an identified user can see someone's votes.
            ballotsWeb = getWebForServerRole('ballots')
            electionsWeb = getWebForServerRole('elections')
            votes = ballotsWeb.getVotesFromTokens(object.voteTokens)
            try:
                lastElections = electionsWeb.getLastObjectsWithVoter(
                    50, object.id, ['running'])
            except faults.UserAccessDenied:
                lastElections = []
            layout += getVotesSectionLayout(
                object, votes, lastElections,
                _("""The votes for the elections in progress"""))

            try:
                lastElections = electionsWeb.getLastObjectsWithVoter(
                    10, object.id, ['closed'])
            except faults.UserAccessDenied:
                lastElections = []
            layout += getVotesSectionLayout(
                object, votes, lastElections,
                _("""The votes for the closed elections"""))
        return layout

    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id):
                return confirmDelete(
                    X.url('/%s/delete' % urlId), objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)

            slot = object.getSlot('password')
            changeButtonName = slot.getFieldOptionName('changeButton')
            if isButtonSelected(changeButtonName, keywords):
                keywords['again'] = '1'
                peopleWeb = getWebForServerRole('people')
                try:
                    password = peopleWeb.changePassword(object.id)
                except faults.Fault:
                    slot.setFieldOption(keywords, 'error', 'can\'tChangeValue')
                else:
                    object.password = password
                    slot.setField(keywords, password)
                    keywords['hideErrors'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return success(
                    _('The new password has been sent by email to %s!') \
                    % object.getFullName(),
                    uri)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        if not id:
            try:
                id = self.addObject(object)
            except faults.DuplicateLogin:
                error = 1
                keywords['login_error'] = 'valueAlreadyUsed'
            except faults.DuplicateFullName:
                error = 1
                keywords['lastName_error'] = 'valueAlreadyUsed'
            except faults.DuplicateEmail:
                error = 1
                keywords['email_error'] = 'valueAlreadyUsed'
            except faults.DuplicateFingerprint:
                error = 1
                keywords['email_fingerprint'] = 'valueAlreadyUsed'
            except faults.SmtpError:
                return failure(_("""\
The account has successfully been created, but a SMTP error has occurred.
Maybe the Mail Transport Agent on the Glasnost server is badly configured.\
"""),
                               X.url('/'))
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
            else:
                error = 0
            if error:
                keywords['again'] = '1'
                keywords['error'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            if object.login and object.email:
                return success(
                    _("""\
The account has successfully been created.
An email containing the password has been sent to %s.\
""") % object.email,
                    X.url(id))
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                error = 1
                keywords['versionError'] = '1'
            except faults.DuplicateLogin:
                error = 1
                keywords['login_error'] = 'valueAlreadyUsed'
            except faults.DuplicateFullName:
                error = 1
                keywords['lastName_error'] = 'valueAlreadyUsed'
            except faults.DuplicateEmail:
                error = 1
                keywords['email_error'] = 'valueAlreadyUsed'
            except faults.DuplicateFingerprint:
                error = 1
                keywords['email_fingerprint'] = 'valueAlreadyUsed'
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
            else:
                error = 0
            if error:
                keywords['again'] = '1'
                keywords['error'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
        return redirect(X.url(id))
    submit.isPublicForWeb = 1

    def viewAll(self):
        if not self.canGetObjects():
            return accessForbidden()
        partialObjects = self.getObjects(
            ['firstName', 'lastName', 'nickname', 'login'])
        layout = X.array()
        ids = self.getSortedIds(partialObjects)
        layout += self.getObjectsLayout(partialObjects, ids, ['login'])
        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _(self.objectsNameCapitalized))
    viewAll.isPublicForWeb = 1

