# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Preferences Web"""

__version__ = '$Revision: 1.27 $'[11:-2]


from glasnost.common.ObjectsCommon import *
import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context
import glasnost.common.translation as translation

from glasnost.proxy.PreferencesProxy import *

import accounting
from ObjectsWeb import register, ObjectWebMixin, WebMixin
from tools import *


class Preference(ObjectWebMixin, Preference):
    cryptEmails_kind_defaultValue = 1
    cryptEmails_kind_widget_fieldLabel = N_('Crypt Emails')
    cryptEmails_kind_widget_labels = {
        '0': N_('Never'),        
        '1': N_('When Possible'),
        }
    cryptEmails_kind_widgetName = 'InputCheckBox'

    currency_kind_defaultValue = 'eur'
    currency_kind_widget_fieldLabel = N_('Currency')
    currency_kind_widget_labels = accounting.currencyLabels
    currency_kind_widgetName = 'Select'

    language_kind_widget_fieldLabel = N_('Language')
    language_kind_widget_labels = {
        'C': N_('English'),
        'None': N_('Use Browser Settings'),
        'de': N_('German'),
        'fi': N_('Finnish'),
        'fr': N_('French'),
        'sv': N_('Swedish'),
        }
    language_kind_widgetName = 'Select'

    objectsMemory_kind_isConstant = 1
    objectsMemory_kind_isHidden = 1

    def getWeb(self):
        return getWebForServerRole('preferences')

    def newWidget(self, parentSlot = None):
        return self.newThing('widget', 'Thing')
register(Preference)


class PreferencesWeb(WebMixin, PreferencesProxy):
    def edit(self, again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        userToken = context.getVar('userToken', default = '')
        if not userToken:
            return accessForbidden()

        preference = self.getPreference()
        if not again:
            preference.makeFieldsFromInstance(keywords)
        preference.repairFields(keywords)

        headerTitle = _('Editing Preferences')

        layout = X.array()
        layout += preference.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/preferences/submit'),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form

        slot = slots.Root(preference)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonInForm('modify', 'modifyButton')

        return writePageLayout(layout, headerTitle)
    edit.isPublicForWeb = 1

    def submit(self, **keywords):
        if keywords is None:
            keywords = {}
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        preference = Preference()
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            preference.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/preferences/edit')
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            self.setPreference(preference)
        except faults.WrongVersion:
            keywords['again'] = '1'
            keywords['error'] = '1'
            keywords['versionError'] = '1'
            uri = X.url('/preferences/edit')
            uri.addKeywords(keywords)
            return redirect(uri)
        except:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(X.url('/preferences'))
    submit.isPublicForWeb = 1

    def viewAll(self):
        userToken = context.getVar('userToken', default = '')
        if not userToken:
            return accessForbidden()

        preference = self.getPreference()

        keywords = {}
        preference.makeFieldsFromInstance(keywords)
        preference.repairFields(keywords)

        layout = X.array()
        slot = slots.Root(preference)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(0)
        layout += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        layout += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonStandalone(
            'edit',
            X.url('/preferences/edit'))

        return writePageLayout(layout, _('Preferences'))
    viewAll.isPublicForWeb = 1

