# Glasnost 
# By: Odile Bnassy <obenassy@entrouvert.org>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@easter-eggs.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Records Web"""

__version__ = '$Revision: 1.28 $'[11:-2]


import glasnost.common.faults as faults
from glasnost.common.tools import extractLocalId
import glasnost.common.context as context

from glasnost.proxy.RecordsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import accounting


class AdminRecords(AdminMixin, AdminRecords):
    def getWeb(self):
        return getWebForServerRole('records')
register(AdminRecords)


class Record(ObjectWebMixin, Record):
    accountId_kind_widget_fieldLabel = N_('Account')
    accountId_kind_widgetName = 'SelectId'

    accountLabel_kind_hasToSubmitField = 0
    accountLabel_kind_isReadOnly = 1
    accountLabel_kind_widget_fieldLabel = N_('Account Label')
    accountLabel_kind_widgetName = 'InputText'
    
    accountNumber_kind_hasToSubmitField = 0
    accountNumber_kind_isReadOnly = 1
    accountNumber_kind_widget_fieldLabel = N_('Account Number')
    accountNumber_kind_widgetName = 'InputText'
    
    amount_kind_widget_fieldLabel = N_('Amount')
    amount_kind_widget_sideLabels = accounting.sideLabels
    amount_kind_widgetName = 'Amount'

    certainty_kind_hasToSubmitField = 0
    certainty_kind_isReadOnly = 1
    certainty_kind_defaultValue = 1
    certainty_kind_widget_fieldLabel = N_('Certainty')
    certainty_kind_widget_labels = accounting.certaintyLabels
    certainty_kind_widgetName = 'Select'

    currency_kind_hasToSubmitField = 0
    currency_kind_isReadOnly = 1
    currency_kind_preference = 'currency'
    currency_kind_widget_fieldLabel = N_('Currency')
    currency_kind_widget_labels = accounting.currencyLabels
    currency_kind_widgetName = 'Select'

    date_kind_hasToSubmitField = 0
    date_kind_isReadOnly = 1
    date_kind_widget_fieldLabel = N_('Date')
    date_kind_widgetName = 'Date'
    
    eventId_kind_widget_fieldLabel = N_('Event')
    eventId_kind_widgetName = 'SelectId'

    label_kind_hasToSubmitField = 0
    label_kind_isReadOnly = 1
    label_kind_widget_fieldLabel = N_('Label')
    label_kind_widget_size = 40
    label_kind_widgetName = 'InputText'

    partnerId_kind_hasToSubmitField = 0
    partnerId_kind_isReadOnly = 1
    partnerId_kind_widget_fieldLabel = N_('Partner')
    partnerId_kind_widgetName = 'SelectId'

    readersSet_kind_hasToSubmitField = 0
    readersSet_kind_isReadOnly = 1
 
    voucherId_kind_hasToSubmitField = 0
    voucherId_kind_isReadOnly = 1
    voucherId_kind_widget_fieldLabel = N_('Voucher')
    voucherId_kind_widgetName = 'SelectId'

    writersSet_kind_hasToSubmitField = 0
    writersSet_kind_isReadOnly = 1

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in (
            'certainty', 'accountLabel', 'accountNumber', 'date', 'label',
            'partnerId', 'voucherId'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ('accountLabel', 'accountNumber'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in ('readersSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('records')

    def objectDisplay(self):
        amount = accounting.amountDisplay(self.amount)
        localId = extractLocalId(self.id)
        return '%s - %s -- %s' % (localId, self.accountNumber, amount)    
register(Record)


class RecordsWeb(ObjectsWebMixin, RecordsProxy):
    def getObjectsLayout(self, partialObjects, objectIds,
                         slotNames = None):
        layout = None
        if len(partialObjects) > 0:
            layout = X.array()
            table = X.table(_class = 'column')
            layout += table
            for objectId in objectIds:
                partialObject = partialObjects[objectId]
                objectDisplay = partialObject.objectDisplay()
                table += X.tr(X.td(X.a(href = X.url(objectId))(
                    objectDisplay)))
            layout += X.br()
        return layout

    def submit(self, id = '', **keywords):
        urlId = getUrlId(id)
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        accountsWeb = getWebForServerRole('accounts')
        parentsWeb = getWebForServerRole('events')
        accountId = keywords['accountId']
        account = accountsWeb.getPartialObject(accountId, ['id'])
        parentId = keywords['eventId']
        parent = parentsWeb.getPartialObject(parentId, ['id'])
        if isButtonSelected('deleteButton', keywords):
            if id and self.hasObject(id):
                return confirmDelete(
                    X.url('/%s/delete' % urlId),
                    objectName = self.objectName)
            else:
                # TODO: error page when user tries to delete an object he can't
                return pageNotFound()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        object = self.newObject(keywords)
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            if keywords.has_key('cloned'):
                brotherIds = parent.getSlot('membersSet').getValue()
            object.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % urlId)
            uri.addKeywords(keywords)
            return redirect(uri)
        if not id:
            try:
                id = self.addObject(object)
                if keywords.has_key('cloned'):
                    brotherIds.append(id)
                    parent.getSlot('membersSet').setValue(brotherIds)
                    parentsWeb.modifyPartialObject(parent, ['membersSet'])
            except faults.Fault:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        else:
            try:
                self.modifyPartialObject(object, object.getSlotToModifyNames())
            except faults.WrongVersion:
                keywords['again'] = '1'
                keywords['error'] = '1'
                keywords['versionError'] = '1'
                uri = X.url('/%s/edit' % urlId)
                uri.addKeywords(keywords)
                return redirect(uri)
            except:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
        accountsWeb.computeObject(accountId)
        parentsWeb.computeObject(parentId)    # compute new balance         
        return redirect(X.url(parentId))
    submit.isPublicForWeb = 1

    def viewAll(self):
        if not self.canGetObjects():
            return accessForbidden()
        partialObjects = self.getObjects(
            ['accountLabel', 'accountNumber', 'amount', 'side'])
        layout = X.array()
        ids = self.getSortedIds(partialObjects)
        layout += self.getObjectsLayout(partialObjects, ids)
        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _(self.objectsNameCapitalized))
    viewAll.isPublicForWeb = 1

