# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Skill Offers Web"""

__version__ = '$Revision: 1.5 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.SkillOffersProxy import *
from glasnost.proxy.TranslationsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import geography


class AdminSkillOffers(AdminMixin, AdminSkillOffers):
    def getWeb(self):
        return getWebForServerRole('skilloffers')
register(AdminSkillOffers)


class SkillOffer(ObjectWebMixin, SkillOffer):
    contactIds_kind_itemKind_value_defaultValue = 'user'
    contactIds_kind_itemKind_value_widgetName = 'SelectId'
    contactIds_kind_widget_fieldLabel = N_('Contacts')
    contactIds_kind_widgetName = 'Multi'

    deal_kind_widget_fieldLabel = N_('Expected Compensation')
    deal_kind_widget_labels = {
        'nocharge' : N_('No Charge'),
        'exchanging' : N_('Skills Exchange'),
        'paying' : N_('With Charge'),
        }
    deal_kind_widgetName = 'XSelect'

    offerDescription_kind_widget_fieldLabel = N_('Offer Description')
    offerDescription_kind_widget_cols = 75
    offerDescription_kind_widget_colSpan = 2
    offerDescription_kind_widget_rows = 4
    offerDescription_kind_widgetName = 'TextArea'

    offerKeywords_kind_itemKind_value_labels = {}
    offerKeywords_kind_itemKind_value_widgetName = 'Select'
    offerKeywords_kind_widget_fieldLabel = N_('Offer Keywords')
    offerKeywords_kind_widgetName = 'Multi'

    ownerIds_kind_itemKind_value_widgetName = 'SelectId'
    ownerIds_kind_widget_fieldLabel = N_('Offer Owners')
    ownerIds_kind_widgetName = 'Multi'

    skillDescription_kind_widget_fieldLabel = N_('Skills Description')
    skillDescription_kind_widget_cols = 75
    skillDescription_kind_widget_colSpan = 2
    skillDescription_kind_widget_rows = 4
    skillDescription_kind_widgetName = 'TextArea'

    skillList_kind_itemKind_value_labels = {}
    skillList_kind_itemKind_value_widgetName = 'Select'
    skillList_kind_widget_fieldLabel = N_('List of Skills')
    skillList_kind_widgetName = 'Multi'

    territory_kind_itemKind_value_labels = {}
    territory_kind_itemKind_value_widgetName = 'XSelect'
    territory_kind_widget_fieldLabel = N_('Affected Territory/ies')
    territory_kind_widgetName = 'Multi'

    title_kind_widget_fieldLabel = N_('Title')
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    webSite_kind_widget_fieldLabel = N_('More info at this url')
    webSite_kind_widget_size = 40
    webSite_kind_widgetName = 'InputText'


    def __init__(self):
        labels = {}
        for key in self.territory_kind_itemKind_value_groupedValues[
            'French Regions']:
            labels[key] = geography.frenchRegionLabels[key]
        for key in self.territory_kind_itemKind_value_groupedValues[
            'European Countries']:
            labels[key] = geography.countryLabels[key]
        labels['fra'] = N_('French-speaking countries')
        self.territory_kind_itemKind_value_labels = labels

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ('creationTime', 'modificationTime', 'readersSet'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in ('writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'creationTime', 'modificationTime', 'readersSet',
                'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('skilloffers')
register(SkillOffer)


class SkillOffersWeb(ObjectsWebMixin, SkillOffersProxy):
    pass

