# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Tasks Web"""

__version__ = '$Revision: 1.8 $'[11:-2]

from glasnost.proxy.TasksProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin

from tools import *

class AdminTasks(AdminMixin, AdminTasks):
    def getWeb(self):
        return getWebForServerRole('tasks')
register(AdminTasks)


class Task(ObjectWebMixin, Task):
    description_kind_widget_cols = 40
    description_kind_widget_colSpan = 2
    description_kind_widget_rows = 10
    description_kind_widgetName = 'TextArea'

    ownersSet_kind_itemKind_value_defaultValue = 'user'
    ownersSet_kind_balloonHelp = N_(
         'Select the people and groups who are assigned to this task.')
    ownersSet_kind_label = N_('Owners')

    state_kind_defaultValue = 'not started'
    
    def getWeb(self):
        return getWebForServerRole('tasks')
register(Task)


class TasksWeb(ObjectsWebMixin, TasksProxy):
    def viewAll(self, owner = None, sortedOrder = 'None', showFinished = 0):
        if not self.canGetObjects():
            return accessForbidden()

        if owner is None:
            owner = getProxyForServerRole('authentication').getUserId()
            
        if owner:
            tasks = self.getTask([owner], sortedOrder, showFinished)
        else:
            tasks = self.getTask(owner, sortedOrder, showFinished)
        layout = X.array()


        form = X.form(action = '', enctype= 'multipart/form-data',
                      method = 'post')
        select = X.select(name = 'owner')
        optionAttributes = {}
        if not owner:
            optionAttributes['selected'] = 'selected'
        select += X.option(value = '', **optionAttributes)(_('Everybody'))
        select += X.optgroup(label = _('People'))
        ## FIXME: change getObjects in getLastObjects,
        ## if someone know how to do that ?
        for personId, person in getProxyForServerRole('people').getObjects().items():
            optionAttributes = {}
            if owner == personId:
                optionAttributes['selected'] = 'selected'
            select += X.option(value = personId,
                               **optionAttributes)(person.getLabel())
        select += X.optgroup(label = _('Groups'))
        ## FIXME: change getObjects in getLastObjects
        for groupId, group in getProxyForServerRole('groups').getObjects().items():
            optionAttributes = {}
            if owner == groupId:
                optionAttributes['selected'] = 'selected'
            select += X.option( value = groupId,
                                **optionAttributes)(group.getLabel())
        form += select
        
        select = X.select(name = 'sortedOrder')
        for value in [N_('None'), N_('ownersSet'), N_('category'),
                      N_('priority'), N_('state'), N_('endingTime')]:
            optionAttributes = {}
            if sortedOrder == value:
                optionAttributes['selected'] = 'selected'
            select += X.option( value = value,
                                **optionAttributes)(_(value))
        form += X.span(_class = 'label')(_('sorted by'))
        form += select

        form += X.span(_class = 'label')(_('show finished'))
        if showFinished:
            optionAttributes['checked'] = 'checked'
        form += X.input(name = 'showFinished', type = 'checkbox', value = 1,
                        **optionAttributes)
      
        form += X.buttonInForm('apply', 'applyButton')
        layout += form
        
        for owner, objects in tasks.items():
            partialObjects = self.getObjects()
            layout += self.getObjectsLayout(partialObjects,
                                            [o.getId() for o in objects],
                                            ['ownersSet', 'category',
                                             'priority','state', 'endingTime'])
        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _(self.objectsNameCapitalized))
    viewAll.isPublicForWeb = 1

