# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Translations Web"""

__version__ = '$Revision: 1.45 $'[11:-2]


import copy
import difflib
import md5

import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context
import glasnost.common.translation as translation

from glasnost.proxy.TranslationsProxy import *

from ObjectsWeb import register, AdminWithoutWritersMixin, ObjectWebMixin, ObjectsWebMixin
import kinds
from tools import *
import widgets


class SimilarStringWidget(widgets.TextArea):
    thingName = 'SimilarString'

    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        similarString = slot.getField(fields, default = '')
        assert self.isInForm()
        assert self.isReadOnly(slot)
        sourceStringSlot = slot.getContainer().getSlot('sourceString')
        sourceString = sourceStringSlot.getField(fields, default = '')

        layout = X.array(
            self.getModelHiddenLayout(slot, fields),
            )
        tt = X.tt(_class = 'diff')
        layout += tt
        sequenceMatcher = difflib.SequenceMatcher(
            None, similarString, sourceString)
        for tag, i1, i2, j1, j2 in sequenceMatcher.get_opcodes():
            if tag == 'delete':
                tt += X.span(_class = 'diff-old')(
                    similarString[i1:i2])
            elif tag == 'equal':
                tt += X.span(_class = 'diff-equal')(
                    similarString[i1:i2])
            elif tag == 'insert':
                tt += X.span(_class = 'diff-new')(
                    sourceString[j1:j2])
            elif tag == 'replace':
                tt += X.span(_class = 'diff-old')(
                    similarString[i1:i2])
                tt += X.span(_class = 'diff-new')(
                    sourceString[j1:j2])
        return layout
widgets.register(SimilarStringWidget)


class TranslationToAddWidget(widgets.BaseWidget):
    thingName = 'TranslationToAdd'

    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        fromFieldName = slot.getFieldOptionName('from')
        toFieldName = slot.getFieldOptionName('to')
        fromFieldValue = slot.getFieldOption(fields, 'from', default = '')
        toFieldValue = slot.getFieldOption(fields, 'to', default = '')
        assert self.isInForm()
        assert not self.isReadOnly(slot)
        values = translation.languageKeys
        labels = translation.languageLabels
        layout = X.array()
        layout += self.getModelErrorLayout(slot, fields)
        layout += self.getModelHelpLayout(slot, fields)
        fromSelect = X.select(name = fromFieldName)
        if not fromFieldValue in values:
            fromSelect += X.option(
                selected = 'selected', value = '')(_('None'))
        for value in values:
            optionAttributes = {}
            if value == fromFieldValue:
                optionAttributes['selected'] = 'selected'
            fromSelect += X.option(value = value, **optionAttributes)(
                _(labels[value]))
        toSelect = X.select(name = toFieldName)
        if not toFieldValue in values:
            toSelect += X.option(
                selected = 'selected', value = '')(_('None'))
        for value in values:
            optionAttributes = {}
            if value == toFieldValue:
                optionAttributes['selected'] = 'selected'
            toSelect += X.option(value = value, **optionAttributes)(
                _(labels[value]))
        layout += _('From ')
        layout += fromSelect
        layout += _(' to ')
        layout += toSelect
        layout += ' '
        layout += X.buttonInForm('add', slot.getFieldOptionName('addButton'))
        return layout
widgets.register(TranslationToAddWidget)


class TranslatorsSetWidget(widgets.Multi):
    thingName = 'TranslatorsSet'

    def getHtmlFieldLabel(self, slot):
        fieldName = slot.getFieldName()
        assert fieldName.endswith('TranslatorsSet')
        languages = fieldName[:-len('TranslatorsSet')][-4:]
        fromLanguage = languages[:2]
        toLanguage = languages[2:4]
        fieldLabelLocalized = _('%(from)s to %(to)s Translators') % {
            'from': _(translation.languageLabels[fromLanguage]),
            'to': _(translation.languageLabels[toLanguage]),
            }
        labelAttributes = {'for': fieldName}
        layout = X.td(_class = 'field-label')(
            X.label(**labelAttributes)(fieldLabelLocalized, X.asIs(_(':'))))
        # FIXME: Is this line required?
        layout += X.input(name = fieldName, type = 'hidden', value = '1')
        return layout

    def getHtmlCssFieldLabel(self, slot):
        fieldName = slot.getFieldName()
        assert fieldName.endswith('TranslatorsSet')
        languages = fieldName[:-len('TranslatorsSet')][-4:]
        fromLanguage = languages[:2]
        toLanguage = languages[2:4]
        fieldLabelLocalized = _('%(from)s to %(to)s Translators') % {
            'from': _(translation.languageLabels[fromLanguage]),
            'to': _(translation.languageLabels[toLanguage]),
            }
        labelAttributes = {'for': fieldName}
        if self.isInForm():
            layout = X.label(**labelAttributes)(
                fieldLabelLocalized, X.asIs(_(':')))
            # FIXME: Is this line required?
            layout += X.input(name = fieldName, type = 'hidden', value = '1')
        else:
            return X.span(_class = 'label')(
                fieldLabelLocalized, X.asIs(_(':')))
        return layout
widgets.register(TranslatorsSetWidget)


class AdminTranslations(AdminWithoutWritersMixin, AdminTranslations):
    translationToAdd = None
    translationToAdd_kindName = 'TranslationToAdd'
    translationToAdd_kind_widget_fieldLabel = N_('Translation')
    translationToAdd_kind_widgetName = 'TranslationToAdd'

    translatorsSets_kind_isConstant = 1
    translatorsSets_kind_isHidden = 1
    translatorsSets_kind_isPublicForXmlRpc = 0
    translatorsSets_kind_isRequired = 0

    def addTranslatorsSetsFields(self, localizationKeys):
        for localizationKey in localizationKeys:
            attributeName = localizationKey + 'TranslatorsSet'
            if not hasattr(self, attributeName):
                setattr(self, attributeName, None)
            setattr(self, attributeName + '_kind', kinds.Sequence(
                isPublicForXmlRpc = 1,
                itemKind = kinds.Id(
                    serverRoles = ['groups', 'people'],
                    widget = widgets.SelectId()),
                widget = TranslatorsSetWidget()))

    def delTranslatorsSetsFields(self, localizationKeys):
        for localizationKey in localizationKeys:
            attributeName = localizationKey + 'TranslatorsSet'
            delattr(self, attributeName + '_kind')

    def exportToXmlRpc(self, requiredSlotNames = None, parentSlot = None):
        # It would have been better not having been forced to write this
        # here... Something like a better field_exporter support...
        result = AdminWithoutWritersCommon.exportToXmlRpc(
            self, requiredSlotNames = requiredSlotNames,
            parentSlot = parentSlot)
        translatorsSets = {}
        for translatorsLabel, translatorsSet in self.__dict__.items():
            if not translatorsLabel.endswith('TranslatorsSet'):
                continue
            if translatorsSet is None:
                continue
            localizationKey = translatorsLabel[:-len('TranslatorsSet')][-4:]
            translatorsSets[localizationKey] = translatorsSet
        if translatorsSets:
            result['translatorsSets'] = translatorsSets
        else:
            if not result.has_key('__noneSlotNames__'):
                result['__noneSlotNames__'] = []
            result['__noneSlotNames__'].append('translatorsSets')
        return result

    def getEditLayout(self, fields, parentSlot = None):
        localizationKeys = []
        for translatorsLabel in fields.keys():
            if not translatorsLabel.endswith('TranslatorsSet_count'):
                continue
            localizationKey = translatorsLabel[:-len('TranslatorsSet_count')][
                -4:]
            localizationKeys.append(localizationKey)
        self.addTranslatorsSetsFields(localizationKeys)

        context.push(fields = fields)
        layout = AdminWithoutWritersMixin.getEditLayout(
            self, fields, parentSlot = parentSlot)
        context.pull()

        self.delTranslatorsSetsFields(localizationKeys)
        return layout

    def getOrderedLayoutSlotNames(self, parentSlot = None):
        slotNames = AdminWithoutWritersCommon.getOrderedLayoutSlotNames(
            self, parentSlot = parentSlot)
        slotNames = slotNames[:]
        fields = context.getVar('fields')
        for translatorsLabel in fields.keys():
            if not translatorsLabel.endswith('TranslatorsSet_count'):
                continue
            localizationKey = translatorsLabel[:-len('TranslatorsSet_count')][
                -4:]
            slotNames.append(localizationKey + 'TranslatorsSet')
        slotNames += ['translationToAdd']
        return slotNames

    def getViewLayout(self, fields, parentSlot = None):
        localizationKeys = []
        for translatorsLabel in fields.keys():
            if not translatorsLabel.endswith('TranslatorsSet_count'):
                continue
            localizationKey = translatorsLabel[:-len('TranslatorsSet_count')][
                -4:]
            localizationKeys.append(localizationKey)
        self.addTranslatorsSetsFields(localizationKeys)

        context.push(fields = fields)
        layout = AdminWithoutWritersMixin.getViewLayout(
            self, fields, parentSlot = parentSlot)
        context.pull()

        self.delTranslatorsSetsFields(localizationKeys)
        return layout

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = AdminWithoutWritersMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        if 'translationToAdd' in slotNames:
            slotNames.remove('translationToAdd')
        return slotNames

    def getWeb(self):
        return getWebForServerRole('translations')

    def importFromXmlRpc(self, dataImport, parentSlot = None):
        if dataImport.has_key('__noneSlotNames__'):
            noneSlotNames = dataImport['__noneSlotNames__']
        else:
            noneSlotNames = []
        for slotName in self.getSlotNames(parentSlot = parentSlot):
            if slotName in [
                '__noneSlotNames__', '__thingCategory__', '__thingName__']:
                continue
            # A slotName must always be a valid Python Indentifier, so no
            # encoding is needed to for the dataImport keys.
            if dataImport.has_key(slotName):
                exportedValue = dataImport[slotName]
            elif slotName in noneSlotNames:
                exportedValue = None
            else:
                continue
            if slotName == 'translatorsSets':
                if exportedValue is None:
                    continue
                for localizationKey, translatorsSet in exportedValue.items():
                    slotName = localizationKey + 'TranslatorsSet'
                    setattr(self, slotName, translatorsSet)
                continue
            slot = self.getSlot(slotName)
            kind = slot.getKind()
            if not kind.isPublicForXmlRpc:
                continue
            slot.setValue(kind.importValueFromXmlRpc(slot, exportedValue))

    def makeFieldsFromInstance(self, fields, parentSlot = None):
        localizationKeys = []
        for translatorsLabel in self.__dict__.keys():
            if not translatorsLabel.endswith('TranslatorsSet'):
                continue
            localizationKey = translatorsLabel[:-len('TranslatorsSet')][-4:]
            localizationKeys.append(localizationKey)
        self.addTranslatorsSetsFields(localizationKeys)
        AdminWithoutWritersMixin.makeFieldsFromInstance(
            self, fields, parentSlot = parentSlot)
        self.delTranslatorsSetsFields(localizationKeys)

    def repairFields(self, fields, parentSlot = None):
        localizationKeys = []
        for translatorsLabel in fields.keys():
            if not translatorsLabel.endswith('TranslatorsSet'):
                continue
            localizationKey = translatorsLabel[:-len('TranslatorsSet')][-4:]
            localizationKeys.append(localizationKey)
        self.addTranslatorsSetsFields(localizationKeys)
        AdminWithoutWritersMixin.repairFields(
            self, fields, parentSlot = parentSlot)
        self.delTranslatorsSetsFields(localizationKeys)

    def submitFields(self, fields, parentSlot = None):
        localizationKeys = []
        for translatorsLabel in fields.keys():
            if not translatorsLabel.endswith('TranslatorsSet'):
                continue
            localizationKey = translatorsLabel[:-len('TranslatorsSet')][-4:]
            localizationKeys.append(localizationKey)
        self.addTranslatorsSetsFields(localizationKeys)
        AdminWithoutWritersMixin.submitFields(
            self, fields, parentSlot = parentSlot)
        self.delTranslatorsSetsFields(localizationKeys)
register(AdminTranslations)


class Localization(ObjectWebMixin, Localization):
    destinationLanguage_kind_defaultValue = 'VoteRanking'
    destinationLanguage_kind_isReadOnly = 1
    destinationLanguage_kind_widget_fieldLabel = N_('Translation Language')
    destinationLanguage_kind_widget_labels = translation.languageLabels
    destinationLanguage_kind_widgetName = 'Select'

    destinationString_kind_widget_fieldLabel = N_('Translated String')
    destinationString_kind_widget_cols = 75
    destinationString_kind_widget_colSpan = 2
    destinationString_kind_widget_rows = 20
    destinationString_kind_widgetName = 'TextArea'

    isFuzzy_kind_defaultValue = 0
    isFuzzy_kind_widget_fieldLabel = N_('Translation State')
    isFuzzy_kind_widget_labels = {
        '0': N_('Good'),
        '1': N_('Fuzzy'),
        }
    isFuzzy_kind_widgetName = 'InputCheckBox'

    isTranslatable_kind_defaultValue = 1
    isTranslatable_kind_widget_apply = 1
    isTranslatable_kind_widget_fieldLabel = N_('Translation')
    isTranslatable_kind_widget_labels = {
        '0': N_('Unneeded'),
        '1': N_('Needed'),
        }
    isTranslatable_kind_widgetName = 'InputCheckBox'

    similarString_kind_isReadOnly = 1
    similarString_kind_widget_fieldLabel = N_('Similar String')
    similarString_kind_widget_cols = 75
    similarString_kind_widget_colSpan = 2
    similarString_kind_widget_rows = 20
    similarString_kind_widget_viewInTextArea = 1
    similarString_kind_widgetName = 'SimilarString'

    sourceIds_kind_hasToSubmitField = 0
    sourceIds_kind_isReadOnly = 1
    sourceIds_kind_itemKind_value_widgetName = 'SelectId'
    sourceIds_kind_widget_fieldLabel = N_('Sources')
    sourceIds_kind_widgetName = 'Multi'

    sourceLanguage_kind_defaultValue = 'VoteRanking'
    sourceLanguage_kind_isReadOnly = 1
    sourceLanguage_kind_widget_fieldLabel = N_('Original Language')
    sourceLanguage_kind_widget_labels = translation.languageLabels
    sourceLanguage_kind_widgetName = 'Select'

    sourceString_kind_isReadOnly = 1
    sourceString_kind_widgetName = 'TextArea'
    sourceString_kind_widget_fieldLabel = N_('Original String')
    sourceString_kind_widget_cols = 75
    sourceString_kind_widget_colSpan = 2
    sourceString_kind_widget_rows = 20
    sourceString_kind_widget_viewInTextArea = 1

    sourceStringDigest_kind_isHidden = 1

    def getEditLayout(self, fields, parentSlot = None):
        layout = X.array()
        layout += ObjectWebMixin.getEditLayout(
            self, fields, parentSlot = parentSlot)
        nbLines = len([
            x
            for x in self.getSlot(
                'sourceString', parentSlot = parentSlot).getField(
                fields, default = '')
            if x == '\n']) + 2
        if nbLines > 20:
            nbLines = 20
        self.getSlot('sourceString', parentSlot = parentSlot).getWidget(
            ).rows = nbLines
        self.getSlot('destinationString', parentSlot = parentSlot).getWidget(
            ).rows = nbLines
        return layout

    def getEditLayoutHiddenSlotNames(self, fields, parentSlot = None):
        hiddenSlotNames = ObjectWebMixin.getEditLayoutHiddenSlotNames(
            self, fields, parentSlot = parentSlot)
        hiddenSlotNames = hiddenSlotNames[:]
        if self.getSlot('isTranslatable', parentSlot = parentSlot).getField(
            fields):
            isTranslatable = 1
        else:
            isTranslatable = 0
        if not isTranslatable:
            for slotName in ['destinationString', 'isFuzzy']:
                if not slotName in hiddenSlotNames:
                    hiddenSlotNames.append(slotName)
        return hiddenSlotNames

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        if self.getSlot('isTranslatable', parentSlot = parentSlot).getField(
            fields):
            isTranslatable = 1
        else:
            isTranslatable = 0
        slotName = 'similarString'
        slot = self.getSlot(slotName, parentSlot = parentSlot)
        if not isTranslatable or not slot.getField(fields):
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ['similarString']:
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('translations')

    def submitFields(self, fields, parentSlot = None):
        if self.getSlot('isTranslatable', parentSlot = parentSlot).getField(
            fields):
            isTranslatable = 1
        else:
            isTranslatable = 0
        for slotName in ('similarString', 'destinationString', 'isFuzzy'):
            # Duplicate each kind.
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kindName = slotName + '_kind'
            setattr(self, kindName, copy.copy(getattr(self, kindName)))
            getattr(self, kindName).hasToSubmitField = isTranslatable
        ObjectWebMixin.submitFields(self, fields, parentSlot = parentSlot)
        for slotName in ('similarString', 'destinationString', 'isFuzzy'):
            # Remove kind duplicate.
            kindName = slotName + '_kind'
            delattr(self, kindName)
register(Localization)


class TranslationsWeb(ObjectsWebMixin, TranslationsProxy):
    def delete(self, localizationKey, sourceStringDigest):
        localization = Localization()
        localization.destinationLanguage = localizationKey[2:]
        localization.sourceLanguage = localizationKey[:2]
        localization.sourceStringDigest = sourceStringDigest
        try:
            self.modifyLocalization(localization)
        except faults.Fault:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(
            X.url('/%s' % self.serverRole).add(
                'localizationKey', localizationKey))
    delete.isPublicForWeb = 1

    def edit(self, localizationKey, sourceStringDigest, again = '',
             error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['localizationKey'] = localizationKey
        keywords['sourceStringDigest'] = sourceStringDigest
        if not self.hasSourceString(
            localizationKey, sourceStringDigest):
            return pageNotFound()
        if not self.canModifyLocalization(
            localizationKey, sourceStringDigest):
            return accessForbidden()
        localization = self.getLocalization(
            localizationKey, sourceStringDigest)

        if not again:
            localization.makeFieldsFromInstance(keywords)
        localization.repairFields(keywords)

        headerTitle = _('Editing Translation')

        layout = X.array()
        layout += localization.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submit' % self.serverRole),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form
        if keywords.has_key('cloned') and keywords['cloned']:
            form += X.input(name = 'cloned', type = 'hidden', value = '1')

        slot = slots.Root(localization)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
        form += X.input(name = 'localizationKey', type = 'hidden',
                        value = localizationKey)

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        if self.canDeleteLocalization(localizationKey, sourceStringDigest):
            actionButtonsBar += X.buttonInForm('delete', 'deleteButton')
        actionButtonsBar += X.buttonInForm('modify', 'modifyButton')

        return writePageLayout(layout, headerTitle)
    edit.isPublicForWeb = 1

    def getObjectsSectionLayout(self, key, digestsAndLabels, intertitle):
        layout = None
        if len(digestsAndLabels) > 0:
            layout = X.array( X.h3()(intertitle) )
            table = X.table()
            layout += table
            for digest, label in digestsAndLabels:
                table += X.tr(X.td(X.a(
                    href = X.url('/translations/edit').add(
                    'localizationKey', key).add('sourceStringDigest', digest))(
                    label)))
            layout += X.br()
        return layout

    def submit(self, localizationKey, sourceStringDigest, **keywords):
        if keywords is None:
            keywords = {}
        keywords['localizationKey'] = localizationKey
        keywords['sourceStringDigest'] = sourceStringDigest
        if isButtonSelected('deleteButton', keywords):
            return confirmDelete(
                X.url('/translations/delete').add(
                'localizationKey', localizationKey).add(
                'sourceStringDigest', sourceStringDigest),
                objectName = self.objectName)
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        localization = Localization()
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            localization.submitFields(keywords)
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/translations/edit')
            uri.addKeywords(keywords)
            return redirect(uri)
        try:
            self.modifyLocalization(localization)
        except faults.WrongVersion:
            keywords['again'] = '1'
            keywords['error'] = '1'
            keywords['versionError'] = '1'
            uri = X.url('/translations/edit')
            uri.addKeywords(keywords)
            return redirect(uri)
        except:
            if context.getVar('debug'):
                raise
            return accessForbidden()
        return redirect(
            X.url('/translations').add('localizationKey', localizationKey))
    submit.isPublicForWeb = 1

    def viewAll(self, localizationKey = ''):
        userToken = context.getVar('userToken', default = '')
        if not userToken:
            return accessForbidden()
        isAdmin = self.isAdmin()
        localizationKeys = self.getTranslatorLocalizationKeys()
        if not isAdmin:
            if not localizationKeys:
                return accessForbidden()
            if localizationKey and not localizationKey in localizationKeys:
                return accessForbidden()
        if localizationKeys:
           if not localizationKey or not localizationKey in localizationKeys:
               localizationKey = localizationKeys[0]
        else:
            localizationKey = ''
        if localizationKey:
            fromLanguage = localizationKey[:2]
            toLanguage = localizationKey[2:4]
            localizationLabelLocalized = _('%(from)s to %(to)s') % {
                'from': _(translation.languageLabels[fromLanguage]),
                'to': _(translation.languageLabels[toLanguage]),
                }
        else:
            localizationLabelLocalized = ''

        otherLocalizations = []
        for otherLocalizationKey in localizationKeys:
            if otherLocalizationKey == localizationKey:
                continue
            fromLanguage = otherLocalizationKey[:2]
            toLanguage = otherLocalizationKey[2:4]
            labelLocalized = _('%(from)s to %(to)s') % {
                'from': _(translation.languageLabels[fromLanguage]),
                'to': _(translation.languageLabels[toLanguage]),
                }
            otherLocalizations.append({
                'label': labelLocalized,
                'link': X.url('/translations').add(
                'localizationKey', otherLocalizationKey),
                })

        layout = X.array()

        if localizationKey:
            digestsAndLabels = self.getLastDigestsAndLabels(
                localizationKey, 50, ['untranslated'])
            layout += self.getObjectsSectionLayout(
                localizationKey, digestsAndLabels,
                _("""The last strings to translate"""))

            digestsAndLabels = self.getLastDigestsAndLabels(
                localizationKey, 50, ['fuzzy'])
            layout += self.getObjectsSectionLayout(
                localizationKey, digestsAndLabels,
                _("""The last fuzzy translations"""))

            digestsAndLabels = self.getLastDigestsAndLabels(
                localizationKey, 50, ['translated'])
            layout += self.getObjectsSectionLayout(
                localizationKey, digestsAndLabels,
                _("""The last translations"""))

            digestsAndLabels = self.getLastDigestsAndLabels(
                localizationKey, 50, ['untranslatable'])
            layout += self.getObjectsSectionLayout(
                localizationKey, digestsAndLabels,
                _("""The last strings to leave untranslated"""))

            digestsAndLabels = self.getLastDigestsAndLabels(
                localizationKey, 50, ['obsolete'])
            layout += self.getObjectsSectionLayout(
                localizationKey, digestsAndLabels,
                _("""The last obsolete translations"""))

        if otherLocalizations:
            div = X.div()
            layout += div
            div += X.i(_('Other Translations'))
            div += X.asIs(_(':'))
            div += X.nbsp
            for otherLocalization in otherLocalizations:
                div += X.a(href = otherLocalization['link'])(
                    otherLocalization['label'])
            layout += X.br()

        if self.canGetAdmin():
            layout += X.buttonStandalone(
                'settings',
                X.url('/translations/admin'))
        return writePageLayout(
            layout, _('Translations') + ':' + ' ' + localizationLabelLocalized)
    viewAll.isPublicForWeb = 1

