# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Upload Files Web"""

__version__ = '$Revision: 1.49 $'[11:-2]


import cStringIO
import copy

# From Python Imaging Library.
import Image as PILImage

import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.UploadFilesProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *


class AdminUploadFiles(AdminMixin, AdminUploadFiles):
    def getWeb(self):
        return getWebForServerRole('uploadfiles')
register(AdminUploadFiles)


class UploadFile(ObjectWebMixin, UploadFile):
    comment_kind_widget_fieldLabel = N_('Comment')
    comment_kind_widget_cols = 40
    comment_kind_widget_rows = 5
    comment_kind_widgetName = 'TextArea'

    data_kindName = 'UploadFile'
    data_kind_widget_fieldLabel = N_('File')
    data_kind_widget_fileName = None
    data_kind_widgetName = 'UploadFile'

    dataFileName_kind_isReadOnly = 1
    dataFileName_kind_widget_fieldLabel = N_('File Name')
    #dataFileName_kind_widget_maxLength = 40
    dataFileName_kind_widget_size = 40
    dataFileName_kind_widgetName = 'InputText'

    dataType_kind_isReadOnly = 1
    dataType_kind_widget_fieldLabel = N_('Mime Type')
    dataType_kind_widgetName = 'InputText'

    height_kind_hasToRepairField = 0
    height_kind_hasToSubmitField = 0
    height_kind_isReadOnly = 1
    height_kind_min = 0
    height_kind_widget_fieldLabel = N_('Height')
    height_kind_widget_maxLength = 4
    height_kind_widget_size = 4
    height_kind_widgetName = 'InputText'

    size_kind_hasToSubmitField = 0
    size_kind_isReadOnly = 1
    size_kind_min = 0
    size_kind_widget_fieldLabel = N_('Size')
    size_kind_widget_maxLength = 6
    size_kind_widget_size = 6
    size_kind_widgetName = 'InputText'

    title_kind_widget_fieldLabel = N_('Title')
    #title_kind_widget_maxLength = 40
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    width_kind_hasToRepairField = 0
    width_kind_hasToSubmitField = 0
    width_kind_isReadOnly = 1
    width_kind_min = 0
    width_kind_widget_fieldLabel = N_('Width')
    width_kind_widget_maxLength = 4
    width_kind_widget_size = 4
    width_kind_widgetName = 'InputText'

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ('height', 'size', 'width'):
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        if not isTypeOfMimeType(fields['dataType'], 'image'):
            if 'height' in slotNames:
                slotNames.remove('height')
            if 'width' in slotNames:
                slotNames.remove('width')
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in [
                'creationTime', 'dataFileName', 'dataType', 'height', 'size',
                'modificationTime', 'readersSet', 'width', 'writersSet']:
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getViewLayout(self, fields, parentSlot = None):
        if self.dataFileName:
            slot = self.getSlot('data', parentSlot = parentSlot)
            self.data_kind = copy.copy(self.data_kind) # Deleted below.
            self.data_kind.widget = copy.copy(slot.getWidget())
            self.data_kind.widget.fileName = self.dataFileName
        layout = ObjectWebMixin.getViewLayout(
            self, fields, parentSlot = parentSlot)
        if self.dataFileName:
            del self.data_kind
        return layout
        
    def getWeb(self):
        return getWebForServerRole('uploadfiles')

    def makeFieldsFromInstance(self, fields, parentSlot = None):
        for slotName in ('height', 'width'):
            self.getSlot(slotName, parentSlot = parentSlot).getKind(
                ).hasToMakeFieldFromValue = self.isType('image')
        ObjectWebMixin.makeFieldsFromInstance(
            self, fields, parentSlot = parentSlot)
        for slotName in ('height', 'width'):
            del self.getSlot(slotName, parentSlot = parentSlot).getKind(
                ).hasToMakeFieldFromValue
register(UploadFile)


class UploadFilesWeb(ObjectsWebMixin, UploadFilesProxy):
    def edit(self, id = '', again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if id and not self.hasObject(id):
            return pageNotFound()
        if id:
            if not self.canModifyObject(id):
                return accessForbidden()
            object = self.getObject(id)
            rememberObject(id)
        else:
            if not self.canAddObject():
                return accessForbidden()
            object = self.newObject(keywords)

        if id and not again:
            object.makeFieldsFromInstance(keywords)
        if not id and not again:
            object.initFields(keywords)
        object.repairFields(keywords)
        if again:
            if keywords.has_key('data') and keywords['data']:
                data = keywords['data']
                if keywords.has_key('dataType') \
                   and isTypeOfMimeType(keywords['dataType'], 'image'):
                    uploadFileFile = cStringIO.StringIO(data)
                    uploadFileObject = PILImage.open(uploadFileFile)
                    width, height = uploadFileObject.size
                    keywords['width'] = str(width)
                    keywords['height'] = str(height)
                keywords['size'] = str(len(data))
        if keywords.has_key('data') and keywords['data']:
            uri = X.url('/%s/imageEdit' % getUrlId(id))
            uri.add('again', again)
            uri.add('error', error)
            uri.addKeywords(keywords)
        else:
            uri = None
        keywords['dataUri'] = uri

        if not id:
            headerTitle = _('New %s') % _(self.objectNameCapitalized)
        else:
            headerTitle = _('Editing %s - %s') % (
                _(self.objectNameCapitalized), object.getLabel())

        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/submit' % self.serverRole),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form
        if keywords.has_key('cloned') and keywords['cloned']:
            form += X.input(name = 'cloned', type = 'hidden', value = '1')

        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        if not id:
            actionButtonsBar += X.buttonInForm('create', 'createButton')
        else:
            if self.canDeleteObject(id):
                actionButtonsBar += X.buttonInForm('delete', 'deleteButton')
            actionButtonsBar += X.buttonInForm('modify', 'modifyButton')

        return writePageLayout(layout, headerTitle)
    edit.isPublicForWeb = 1

    def image(self, id, fileName = None):
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canGetObject(id):
            return accessForbidden()
        object = self.getPartialObject(
            id, ['modificationTime', 'dataFileName'])
        rememberObject(id)

        if fileName and object.dataFileName != fileName:
            pageNotFound()

        req = context.getVar('req')
        req.depends.append(id)
        ### Commented out since this check is now done generically in
        ### webhandler.
        ### lastModTime = time.gmtime(object.modificationTime)
        ### req.headers_out['Last-Modified'] = time.strftime(
        ###     '%a, %d %b %Y %H:%M:%S %Z', lastModTime)
        ### if req.headers_in.has_key('If-Modified-Since'):
        ###     try: 
        ###         t = time.strptime(req.headers_in['If-Modified-Since'][:25],
        ###                             '%a, %d %b %Y %H:%M:%S')
        ###         if lastModTime <= t:
        ###             return HTTP_NOT_MODIFIED
        ###     except:
        ###         pass 
        object = self.getObject(id)
        if object.dataFileName:
            req.headers_out['Content-Disposition'] = \
                'inline; filename="%s"' % object.dataFileName
        req.headers_out['Content-Length'] = str(object.size)
        req.content_type = object.dataType
        setHttpCookie()
        req.send_http_header()
        if req.method == 'HEAD':
            return OK
        if req.caching:
            req.openCachePage()
        req.write(object.data)
        if req.caching:
            req.closeCachePage()
        return OK
    image.isPublicForWeb = 1

    download = image
    download.isPublicForWeb = 1

    def imageEdit(self, id = '', again = '', error = '', **keywords):
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        if id and not self.hasObject(id):
            return pageNotFound()
        if id:
            if not self.canModifyObject(id):
                return accessForbidden()
            object = self.getObject(id)
            rememberObject(id)
        else:
            if not self.canAddObject():
                return accessForbidden()
            object = self.newObject(keywords)

        if id and not again:
            object.makeFieldsFromInstance(keywords)
        if not id and not again:
            object.initFields(keywords)
        object.repairFields(keywords)

        req = context.getVar('req')
        req.content_type = keywords['dataType']
        setHttpCookie()
        req.send_http_header()
        if req.method == 'HEAD':
            return OK
        req.write(keywords['data'])
        return OK
    imageEdit.isPublicForWeb = 1

    def thumbnail(self, id, width = '', height = ''):
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canGetObject(id):
            return accessForbidden()
        try:
            width = int(width)
        except ValueError:
            width = 128
        try:
            height = int(height)
        except ValueError:
            height = 128
        object = self.getObjectThumbnail(
            id, width, height)
        rememberObject(id)

        req = context.getVar('req')
        req.content_type = object.dataType
        setHttpCookie()
        req.send_http_header()
        if req.method == 'HEAD':
            return OK
        req.write(object.data)
        return OK
    thumbnail.isPublicForWeb = 1

    def view(self, id):
        urlId = getUrlId(id)
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canGetObject(id):
            return accessForbidden()
        
        object = UploadFile()
        slotNames = [x for x in object.getSlotNames() if x != 'data']
        object = self.getPartialObject(id, slotNames)
        rememberObject(id)

        keywords = {}
        object.makeFieldsFromInstance(keywords)
        object.repairFields(keywords)
        
        if isTypeOfMimeType(object.dataType, 'image'):
            fullUri = X.url('/%s/image' % urlId)
        else:
            fullUri = X.url('/%s/download' % urlId)
        keywords['dataFullUri'] = fullUri
        uri = X.url('/%s/thumbnail' % urlId)
        uri.add('width', 256)
        uri.add('height', 256)
        keywords['dataUri'] = uri
        
        label = object.getLabelTranslated(
            context.getVar('readLanguages'))

        layout = X.array()
        slot = slots.Root(object)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(0)
        layout += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
        layout += self.getViewAboveButtonsBarLayout(object, keywords)
        layout += self.getViewButtonsBarLayout(object, keywords)
        layout += self.getViewBelowButtonsBarLayout(object, keywords)
        return writePageLayout(
            layout, '%s - %s' % (_(self.objectNameCapitalized), label))
    view.isPublicForWeb = 1

    def viewAll(self):
        userId = context.getVar('userId', default = '')
        if not self.canGetObjects():
            return accessForbidden()
        isAdmin = self.isAdmin()
        if userId:
            userSet = [userId]
        else:
            userSet = None

        layout = X.array()
        requiredSlotNames = ['dataType', 'size', 'title']
        displayedSlotNames = ['dataType', 'size']

        if userSet:
            lastUploadFiles = self.getLastObjects(
                10, None, userSet, requiredSlotNames)
            layout += self.getObjectsSectionLayout(
                lastUploadFiles,
                _("""Your last files"""),
                displayedSlotNames)

        lastUploadFiles = self.getLastObjects(
            10, userSet, None, requiredSlotNames)
        layout += self.getObjectsSectionLayout(
            lastUploadFiles,
            _("""The last files"""),
            displayedSlotNames)

        if isAdmin:
            lastUploadFiles = self.getLastObjects(
                50, None, None, requiredSlotNames)
            layout += self.getObjectsSectionLayout(
                lastUploadFiles,
                _("""The last files to administer"""),
                displayedSlotNames)

        layout += self.getViewAllButtonsBarLayout()
        return writePageLayout(layout, _('Files'))
    viewAll.isPublicForWeb = 1

