# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Virtual Hosts Web"""

__version__ = '$Revision: 1.28 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.context as context

from glasnost.proxy.VirtualHostsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *


class AdminVirtualHosts(AdminMixin, AdminVirtualHosts):
    def getWeb(self):
        return getWebForServerRole('virtualhosts')
register(AdminVirtualHosts)


_VirtualHost = VirtualHost
class VirtualHost(ObjectWebMixin, VirtualHost):
    defaultDispatcherId_kind_defaultValue = 'glasnost://localhost/'
    defaultDispatcherId_kind_widget_fieldLabel = N_('Glasnost Dispatcher ID')
    defaultDispatcherId_kind_widget_size = 40
    defaultDispatcherId_kind_widgetName = 'InputText'

    hostName_kind_widget_fieldLabel = N_('Web Host Name')
    #hostName_kind_widget_maxLength = 40
    hostName_kind_widget_size = 40
    hostName_kind_widgetName = 'InputText'

    templateDirectoryName_kind_defaultValue = 'glasnost2'
    templateDirectoryName_kind_widget_fieldLabel = N_('Template')
    templateDirectoryName_kind_widgetName = 'Select'

    title_kind_widget_fieldLabel = N_('Title')
    #title_kind_widget_maxLength = 40
    title_kind_widget_size = 40
    title_kind_widgetName = 'InputText'

    def initTemplateList(self):
        templatesDirectoryPath = context.getVar(
            'templatesDirectoryPath')
        dirs = os.listdir(templatesDirectoryPath)
        slot = self.getSlot('templateDirectoryName')
        kind = slot.getKind()
        widget = slot.getWidget()
        kind.values = []
        widget.labels = {}
        widget.titles = {}
        for d in dirs:
            try:
                label = open(os.path.join(templatesDirectoryPath, d, 'infos')
                             ).readline()[:-1]
            except IOError:
                continue
            kind.values.append(d)
            widget.labels[d] = label
            widget.titles[d] = label
        kind.values.sort()
        
    def getViewLayout(self, fields, parentSlot = None):
        self.initTemplateList()
        return ObjectWebMixin.getViewLayout(
            self, fields, parentSlot = parentSlot)
        
    def getEditLayout(self, fields, parentSlot = None):
        self.initTemplateList()
        return ObjectWebMixin.getEditLayout(
            self, fields, parentSlot = parentSlot)
        
    def submitFields(self, fields, parentSlot = None):
        self.initTemplateList()
        return ObjectWebMixin.submitFields(
            self, fields, parentSlot = parentSlot)

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'creationTime', 'modificationTime', 'readersSet',
                'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return getWebForServerRole('virtualhosts')
register(VirtualHost)


class VirtualHostsWeb(ObjectsWebMixin, VirtualHostsProxy):
    def getObject(self, objectId, multiCall = None):
        object = VirtualHostsProxy.getObject(
            self, objectId, multiCall = multiCall)
        if object is not None: # not a multiCall
            object.initTemplateList()
        return object

