# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Votes Web Proxy"""

__version__ = '$Revision: 1.42 $'[11:-2]


import copy

import glasnost.common.faults as faults
import glasnost.common.slots as slots
import glasnost.common.context as context

from glasnost.proxy.VotesProxy import *

from ObjectsWeb import register, AdminWithoutWritersMixin, ObjectWebMixin, \
     ObjectsWebMixin
from tools import *
import widgets


class AdminVotes(AdminWithoutWritersMixin, AdminVotes):
    def getWeb(self):
        return getWebForServerRole('votes')
register(AdminVotes)


class MarksWidget(widgets.BaseWidget):
    def getModelHiddenLayout(self, slot, fields):
        container = slot.getContainer()
        idsGetterName = slot.getKind().idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        layout = X.array()
        if objectIds:
            for objectId in objectIds:
                fieldName = slot.getFieldOptionName(objectId)
                mark = slot.getFieldOption(fields, objectId, default = '')
                layout += X.input(name = fieldName, type = 'hidden',
                                  value = mark),
        return layout
    def getHtmlCssFieldValue(self, slot, fields, **keywords):
        layout = self.getHtmlFieldValue(slot, fields, **keywords)
        return X.div(_class = 'cell')(layout)


class VoteApprovalMarksWidget(MarksWidget):
    def getHtmlFieldValue(self, slot, fields, **keywords):
        container = slot.getContainer()
        idsGetterName = slot.getKind().idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        if not self.isInForm():
            if container.isAbstention():
                return X.td(_class = 'field-value')(X.i(_('Abstention')))
            elif container.isBlank(objectIds):
                return X.td(_class = 'field-value')(X.i(_('Blank Vote')))
        tdAttributes = {}
        if self.colSpan:
            tdAttributes['colspan'] = self.colSpan
        layout = X.td(_class = 'field-value', **tdAttributes)
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if objectIds:
            table = X.table()
            layout += table
            for objectId in objectIds:
                fieldName = slot.getFieldOptionName(objectId)
                mark = slot.getFieldOption(fields, objectId, default = '')
                if mark:
                    markSymbol = 'x'
                else:
                    markSymbol = X.nbsp
                tr = X.tr()
                table += tr
                if not self.isInForm() or self.isReadOnly(slot):
                    tr += X.td(_class = 'field-value')(
                        '[',
                        markSymbol,
                        ']',
                        )
                else:
                    td = X.td(_class = 'field-value')
                    tr += td
                    errorCode = slot.getFieldOption(
                        fields, objectId + '_error')
                    if errorCode and not fields.has_key('hideErrors'):
                        td += X.asIs(makeErrorMessage(errorCode))
                    inputAttributes = {}
                    if mark:
                        inputAttributes['checked'] = 'checked'
                    td += X.input(name = fieldName, type = 'checkbox',
                                  value = 1, **inputAttributes)
                tr += X.td(_class = 'field-label')(
                    X.objectHypertextLabel(objectId))
        return layout
widgets.register(VoteApprovalMarksWidget)


class VoteDistributionMarksWidget(MarksWidget):
    def getHtmlFieldValue(self, slot, fields, **keywords):
        container = slot.getContainer()
        idsGetterName = slot.getKind().idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        if not self.isInForm():
            if container.isAbstention():
                return X.td(_class = 'field-value')(X.i(_('Abstention')))
            elif container.isBlank(objectIds):
                return X.td(_class = 'field-value')(X.i(_('Blank Vote')))
        tdAttributes = {}
        if self.colSpan:
            tdAttributes['colspan'] = self.colSpan
        layout = X.td(_class = 'field-value', **tdAttributes)
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if objectIds:
            table = X.table()
            layout += table
            for objectId in objectIds:
                fieldName = slot.getFieldOptionName(objectId)
                mark = slot.getFieldOption(fields, objectId, default = '')
                tr = X.tr()
                table += tr
                tr += X.td(_class = 'field-label')(
                    X.objectHypertextLabel(objectId),
                    X.asIs(_(':')),
                    )
                if not self.isInForm() or self.isReadOnly(slot):
                    tr += X.td(_class = 'field-value')(
                        mark,
                        X.nbsp,
                        '%',
                        )
                else:
                    td = X.td(_class = 'field-value')
                    tr += td
                    errorCode = slot.getFieldOption(
                        fields, objectId + '_error')
                    if errorCode and not fields.has_key('hideErrors'):
                        td += X.asIs(makeErrorMessage(errorCode))
                    td += X.input(maxlength = 6, name = fieldName, size = 6,
                                  type = 'text', value = mark)
                    td += X.nbsp
                    td += '%'
        return layout
widgets.register(VoteDistributionMarksWidget)


class VoteRankingMarksWidget(MarksWidget):
    def getHtmlFieldValue(self, slot, fields, **keywords):
        container = slot.getContainer()
        idsGetterName = slot.getKind().idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        if not self.isInForm():
            if container.isAbstention():
                return X.td(_class = 'field-value')(X.i(_('Abstention')))
            elif container.isBlank(objectIds):
                return X.td(_class = 'field-value')(X.i(_('Blank Vote')))
        tdAttributes = {}
        if self.colSpan:
            tdAttributes['colspan'] = self.colSpan
        layout = X.td(_class = 'field-value', **tdAttributes)
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if objectIds:
            table = X.table()
            layout += table
            for objectId in objectIds:
                fieldName = slot.getFieldOptionName(objectId)
                mark = slot.getFieldOption(fields, objectId, default = '')
                tr = X.tr()
                table += tr
                if not self.isInForm() or self.isReadOnly(slot):
                    tr += X.td(_class = 'field-value')(
                        '[',
                        X.nbsp,
                        mark,
                        X.nbsp,
                        ']',
                        )
                else:
                    td = X.td(_class = 'field-value')
                    tr += td
                    errorCode = slot.getFieldOption(
                        fields, objectId + '_error')
                    if errorCode and not fields.has_key('hideErrors'):
                        td += X.asIs(makeErrorMessage(errorCode))
                    maxLength = len(str(len(objectIds)))
                    td += X.input(
                        maxlength = maxLength, name = fieldName,
                        size = maxLength, type = 'text', value = mark)
                tr += X.td(_class = 'field-label')(
                    X.objectHypertextLabel(objectId))
        return layout
widgets.register(VoteRankingMarksWidget)


class VoteRatingMarksWidget(MarksWidget):
    def getHtmlFieldValue(self, slot, fields, **keywords):
        container = slot.getContainer()
        idsGetterName = slot.getKind().idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        if not self.isInForm():
            if container.isAbstention():
                return X.td(_class = 'field-value')(X.i(_('Abstention')))
            elif container.isBlank(objectIds):
                return X.td(_class = 'field-value')(X.i(_('Blank Vote')))
        tdAttributes = {}
        if self.colSpan:
            tdAttributes['colspan'] = self.colSpan
        layout = X.td(_class = 'field-value', **tdAttributes)
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if objectIds:
            table = X.table()
            layout += table
            for objectId in objectIds:
                fieldName = slot.getFieldOptionName(objectId)
                mark = slot.getFieldOption(fields, objectId, default = '')
                tr = X.tr()
                table += tr
                tr += X.td(_class = 'field-label')(
                    X.objectHypertextLabel(objectId),
                    X.asIs(_(':')),
                    )
                if not self.isInForm() or self.isReadOnly(slot):
                    tr += X.td(_class = 'field-value')(mark)
                else:
                    td = X.td(_class = 'field-value')
                    tr += td
                    errorCode = slot.getFieldOption(
                        fields, objectId + '_error')
                    if errorCode and not fields.has_key('hideErrors'):
                        td += X.asIs(makeErrorMessage(errorCode))
                    td += X.input(maxlength = 6, name = fieldName, size = 6,
                                  type = 'text', value = mark)
        return layout
widgets.register(VoteRatingMarksWidget)


class VoteMixin(ObjectWebMixin):
    ballotKind_kind_widget_fieldLabel = N_('Kind of Ballot')
    ballotKind_kind_widget_labels = {
        'public': N_('Public Ballot'),
        'secret': N_('Secret Ballot'),
        }
    ballotKind_kind_widgetName = 'Select'

    comment_kind_widget_fieldLabel = N_('Comment')
    comment_kind_widget_cols = 75
    comment_kind_widget_colSpan = 2
    comment_kind_widget_rows = 4
    comment_kind_widgetName = 'TextArea'

    electionId_kind_hasToInitField = 0
    electionId_kind_hasToMakeFieldFromValue = 0
    electionId_kind_hasToRepairField = 0
    electionId_kind_hasToSubmitField = 0
    electionId_kind_isReadOnly = 1
    electionId_kindName = 'Id'
    electionId_kind_serverRoles = ['elections']
    electionId_kind_widget_fieldLabel = N_('Election')
    electionId_kind_widgetName = 'SelectId'

    electionToken_kind_isHidden = 1
    electionToken_kind_widget_fieldLabel = N_('Election Token')
    electionToken_kind_widgetName = 'Token'

    token_kind_isReadOnly = 1
    token_kind_widget_fieldLabel = N_('Vote Token')
    token_kind_widgetName = 'Token'

    voterId_kind_hasToInitField = 0
    voterId_kind_hasToMakeFieldFromValue = 0
    voterId_kind_hasToRepairField = 0
    voterId_kind_hasToSubmitField = 0
    voterId_kind_isReadOnly = 1
    voterId_kindName = 'Id'
    voterId_kind_serverRoles = ['people']
    voterId_kind_widget_fieldLabel = N_('Voter')
    voterId_kind_widgetName = 'SelectId'

    voterToken_kind_isReadOnly = 1
    voterToken_kind_widget_fieldLabel = N_('Voter Token')
    voterToken_kind_widgetName = 'Token'

    def getElectionCandidateIds(self, fields):
        return fields['election'].candidateIds

    def getEditLayout(self, fields, parentSlot = None):
        election = fields['election']
        slot = election.getSlot('subject')
        subject, destinationLanguage, state = \
                 getProxyForServerRole('translations').getTranslationInfos(
            election.subject, election.getId(), slot.getPath(),
            election.getLanguage(), context.getVar('readLanguages'))
        self.subject = subject
        self.subject_kind = copy.copy(election.subject_kind)
        self.subject_kind.isReadOnly = 1
        self.subject_kind.isTranslatable = 0
        self.subject_kind.widget = widgets.TextArea(
            fieldLabel = N_('Subject'),
            cols = 75,
            colSpan = 2,
            rows = 4)

        layout = ObjectWebMixin.getEditLayout(
            self, fields, parentSlot = parentSlot)

        del self.subject
        del self.subject_kind

        return layout

    def getEditLayoutHiddenSlotNames(self, fields, parentSlot = None):
        hiddenSlotNames = ObjectWebMixin.getEditLayoutHiddenSlotNames(
            self, fields, parentSlot = parentSlot)
        hiddenSlotNames = hiddenSlotNames[:]
        slot = self.getSlot('voterId')
        if slot.getField(fields):
            hiddenSlotNames.append('voterToken')
        else:
            hiddenSlotNames.append('voterId')
        election = fields['election']
        if election.ballotKind != 'voterChoice':
            hiddenSlotNames.append('ballotKind')
        return hiddenSlotNames

    def getViewLayout(self, fields, parentSlot = None):
        election = fields['election']
        slot = election.getSlot('subject')
        subject, destinationLanguage, state = \
                 getProxyForServerRole('translations').getTranslationInfos(
            election.subject, election.getId(), slot.getPath(),
            election.getLanguage(), context.getVar('readLanguages'))
        self.subject = subject
        self.subject_kind = copy.copy(election.subject_kind)
        self.subject_kind.isReadOnly = 1
        self.subject_kind.isTranslatable = 0
        self.subject_kind.widget = widgets.TextArea(
            fieldLabel = N_('Subject'),
            cols = 75,
            colSpan = 2,
            rows = 4)

        layout = ObjectWebMixin.getViewLayout(
            self, fields, parentSlot = parentSlot)

        del self.subject
        del self.subject_kind

        return layout

    def getViewLayoutHiddenSlotNames(self, fields, parentSlot = None):
        hiddenSlotNames = ObjectWebMixin.getViewLayoutHiddenSlotNames(
            self, fields, parentSlot = parentSlot)
        hiddenSlotNames = hiddenSlotNames[:]
        slot = self.getSlot('voterId')
        if slot.getField(fields):
            hiddenSlotNames.append('voterToken')
        else:
            hiddenSlotNames.append('voterId')
        election = fields['election']
        if election.ballotKind != 'voterChoice':
            hiddenSlotNames.append('ballotKind')
        return hiddenSlotNames

    def getWeb(self):
        return getWebForServerRole('votes')

    def makeFieldsFromInstance(self, fields, parentSlot = None):
        if self.isBlank(self.getElectionCandidateIds(fields)):
            # Duplicate the marks kind.
            slot = self.getSlot('marks', parentSlot = parentSlot)
            self.marks_kind = copy.copy(self.marks_kind)
            self.marks_kind.hasToMakeFieldFromValue = 0
            ObjectWebMixin.makeFieldsFromInstance(
                self, fields, parentSlot = parentSlot)
            slot.getKind().makeFieldFromValue(slot, fields, {})
            # Delete the duplicated marks kind.
            del self.marks_kind
        else:
            ObjectWebMixin.makeFieldsFromInstance(
                self, fields, parentSlot = parentSlot)

    def submitFields(self, fields, parentSlot = None):
        election = fields['election']
        # Duplicate the ballotKind kind.
        slot = self.getSlot('ballotKind', parentSlot = parentSlot)
        self.ballotKind_kind = copy.copy(self.ballotKind_kind)
        self.ballotKind_kind.hasToSubmitField = \
                                           election.ballotKind == 'voterChoice'
        ObjectWebMixin.submitFields(self, fields, parentSlot = parentSlot)
        # Delete the duplicated ballotKind kind.
        del self.ballotKind_kind

class AbstractVote(VoteMixin, AbstractVote):
    pass
register(AbstractVote)


class VoteApproval(VoteMixin, VoteApproval):
    marks_kind_idsGetterName = 'getElectionCandidateIds'
    marks_kindName = 'ApprovalMarks'
    marks_kind_widget_fieldLabel = N_('Choice')
    marks_kind_widgetName = 'VoteApprovalMarksWidget'

    def getLabelLine(self, election):
        if self.isAbstention():
            line = '<i>%s</i>' % _('Abstention')
        elif self.isBlank(election.candidateIds):
            line = '<i>%s</i>' % _('Blank Vote')
        else:
            marks = self.getMarks(election.candidateIds)
            candidates = [
                getObjectLabelTranslated(
                    candidateId,
                    context.getVar('readLanguages'))
                for candidateId in marks.keys()
                if marks[candidateId]]
            return ', '.join(candidates)

    def getMarksRowLayout(self, candidateIds):
        marks = self.getMarks(candidateIds)
        layout = X.array()
        for candidateId in candidateIds:
            mark = marks[candidateId]
            tdAttributes = {}
            if mark:
                red = 0
                green = 255
                cell = 'x'
            else:
                red = 255
                green = 0
                cell = X.nbsp
            tdAttributes['style'] = 'background-color: #%(red)02x%(green)02x00' % {
                'green': green,
                'red': red,
                }
            layout += X.td(**tdAttributes)(cell)
        return layout
register(VoteApproval)


class VoteDistribution(VoteMixin, VoteDistribution):
    marks_kind_idsGetterName = 'getElectionCandidateIds'
    marks_kindName = 'DistributionMarks'
    marks_kind_widget_fieldLabel = N_('Choice')
    marks_kind_widgetName = 'VoteDistributionMarksWidget'

    def getLabelLine(self, election):
        if self.isAbstention():
            line = '<i>%s</i>' % _('Abstention')
        elif self.isBlank(election.candidateIds):
            line = '<i>%s</i>' % _('Blank Vote')
        else:
            line = ''
            marks = self.getMarks(election.candidateIds)
            candidateIds = marks.keys()[:]
            def candidatesSorter(id1, id2, marks = marks):
                return cmp(marks[id2], marks[id1])
            candidateIds.sort(candidatesSorter)
            for candidateId in candidateIds:
                if line:
                    line = line + ', '
                line = line + '%(candidate)s (%(mark)s)' % {
                    'candidate': getObjectLabelTranslated(
                        candidateId,
                        context.getVar('readLanguages')),
                    'mark': marks[candidateId],
                    }
        return line

    def getMarksRowLayout(self, candidateIds):
        marks = self.getMarks(candidateIds)
        minMark = min(marks.values())
        maxMark = max(marks.values())
        if minMark < maxMark:
            averageMark = (maxMark + minMark) / 2.0
            coef = 255 / float(averageMark - minMark)
        layout = X.array()
        for candidateId in candidateIds:
            mark = marks[candidateId]
            tdAttributes = {}
            if minMark < maxMark:
                if mark <= averageMark:
                    red = 255
                    green = coef * (mark - minMark)
                else:
                    red = coef * (maxMark - mark)
                    green = 255
                tdAttributes['style'] = 'background-color: #%(red)02x%(green)02x00' % {
                    'green': green,
                    'red': red,
                    }
            layout += X.td(**tdAttributes)(
                '%.2f' % (mark * 100),
                X.nbsp,
                '%',
                )
        return layout
register(VoteDistribution)


class VoteRanking(VoteMixin, VoteRanking):
    marks_kind_idsGetterName = 'getElectionCandidateIds'
    marks_kindName = 'RankingMarks'
    marks_kind_widget_fieldLabel = N_('Choice')
    marks_kind_widgetName = 'VoteRankingMarksWidget'

    def getLabelLine(self, election):
        if self.isAbstention():
            line = '<i>%s</i>' % _('Abstention')
        elif self.isBlank(election.candidateIds):
            line = '<i>%s</i>' % _('Blank Vote')
        else:
            line = ''
            marks = self.getMarks(election.candidateIds)
            candidateIds = marks.keys()[:]
            def candidatesSorter(id1, id2, marks = marks):
                return cmp(marks[id1], marks[id2])
            candidateIds.sort(candidatesSorter)
            for candidateId in candidateIds:
                if line:
                    line = line + ', '
                line = line + '(%(mark)s) %(candidate)s' % {
                    'candidate': getObjectLabelTranslated(
                        candidateId,
                        context.getVar('readLanguages')),
                    'mark': marks[candidateId],
                    }
        return line

    def getMarksRowLayout(self, candidateIds):
        marks = self.getMarks(candidateIds)
        scores = self.getScores(candidateIds)
        minScore = 0
        maxScore = 1
        if minScore < maxScore:
            averageScore = (maxScore + minScore) / 2.0
            coef = 255 / float(averageScore - minScore)
        layout = X.array()
        for candidateId in candidateIds:
            mark = marks[candidateId]
            tdAttributes = {}
            if minScore < maxScore:
                score = scores[candidateId]
                if score <= averageScore:
                    red = 255
                    green = coef * (score - minScore)
                else:
                    red = coef * (maxScore - score)
                    green = 255
                tdAttributes['style'] = 'background-color: #%(red)02x%(green)02x00' % {
                    'green': green,
                    'red': red,
                    }
            layout += X.td(**tdAttributes)(mark)
        return layout
register(VoteRanking)


class VoteRating(VoteMixin, VoteRating):
    marks_kind_idsGetterName = 'getElectionCandidateIds'
    marks_kindName = 'RatingMarks'
    marks_kind_widget_fieldLabel = N_('Choice')
    marks_kind_widgetName = 'VoteRatingMarksWidget'

    def getLabelLine(self, election):
        if self.isAbstention():
            line = '<i>%s</i>' % _('Abstention')
        elif self.isBlank(election.candidateIds):
            line = '<i>%s</i>' % _('Blank Vote')
        else:
            line = ''
            marks = self.getMarks(election.candidateIds)
            candidateIds = marks.keys()[:]
            def candidatesSorter(id1, id2, marks = marks):
                return cmp(marks[id2], marks[id1])
            candidateIds.sort(candidatesSorter)
            for candidateId in candidateIds:
                if line:
                    line = line + ', '
                line = line + '%(candidate)s (%(mark)s)' % {
                    'candidate': getObjectLabelTranslated(
                        candidateId,
                        context.getVar('readLanguages')),
                    'mark': marks[candidateId],
                    }
        return line

    def getMarksRowLayout(self, candidateIds):
        marks = self.getMarks(candidateIds)
        minMark = 0
        maxMark = 1
        if minMark < maxMark:
            averageMark = (maxMark + minMark) / 2.0
            coef = 255 / float(averageMark - minMark)
        layout = X.array()
        for candidateId in candidateIds:
            mark = marks[candidateId]
            tdAttributes = {}
            if minMark < maxMark:
                if mark <= averageMark:
                    red = 255
                    green = coef * (mark - minMark)
                else:
                    red = coef * (maxMark - mark)
                    green = 255
                tdAttributes['style'] = 'background-color: #%(red)02x%(green)02x00' % {
                    'green': green,
                    'red': red,
                    }
            layout += X.td(**tdAttributes)(
                '%.2f' % (mark * 100))
        return layout
register(VoteRating)


class VotesWeb(ObjectsWebMixin, VotesProxy):
    def canAddObject(self, serverId = None):
        return 0

    def edit(self, id = '', electionId = '', again = '', error = '',
             **keywords):
        ballotsWeb = getWebForServerRole('ballots')
        electionsWeb = getWebForServerRole('elections')
        peopleWeb = getWebForServerRole('people')
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        keywords['electionId'] = electionId
        userId = context.getVar('userId', default = '')
        userToken = context.getVar('userToken', default = '')
        if not userToken:
            return accessForbidden()
        if id:
            vote = ballotsWeb.getVote(id)
            if vote == 'secret':
                return accessForbidden()
            rememberObject(id)
            electionId = ballotsWeb.getElectionIdFromToken(
                vote.electionToken, serverId = extractServerId(id))
            if not electionId \
               or not electionsWeb.hasObject(electionId):
                return pageNotFound()
            if not electionsWeb.canVote(electionId):
                return accessForbidden()
            election = electionsWeb.getObject(electionId)
            rememberObject(electionId)
        else:
            if not electionId \
               or not electionsWeb.hasObject(electionId):
                return pageNotFound()
            if not electionsWeb.canVote(electionId):
                return accessForbidden()
            election = electionsWeb.getObject(electionId)
            rememberObject(electionId)
            voter = peopleWeb.getObject(userId)
            if voter.voteTokens is not None \
               and voter.voteTokens.has_key(electionId):
                voteToken = voter.voteTokens[electionId]
                if voteToken == 'secret':
                    return accessForbidden()
                try:
                    vote = ballotsWeb.getVoteFromToken(voteToken)
                except faults.UnknownVoteToken:
                    vote = election.newVote()
                else:
                    id = vote.id
                    rememberObject(id)
            else:
                vote = election.newVote()

        keywords['election'] = election
        if id and not again:
            vote.makeFieldsFromInstance(keywords)
        if not id and not again:
            vote.initFields(keywords)
        vote.repairFields(keywords)

        slot = vote.getSlot('electionId')
        slot.getKind().makeFieldFromValue(slot, keywords, electionId)
        slot = election.getSlot('subject')
        slot.getKind().makeFieldFromValue(
            slot, keywords, election.subject)
        slot = vote.getSlot('voterId')
        slot.getKind().makeFieldFromValue(slot, keywords, userId)

        if not id:
            headerTitle = _('New %s') % _(self.objectNameCapitalized)
        else:
            headerTitle = _('Editing %s - %s') % (
                _(self.objectNameCapitalized), vote.getLabel())

        layout = X.array()
        layout += vote.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/votes/submit'),
                      enctype= 'multipart/form-data', method = 'post')
        layout += form

        slot = slots.Root(vote)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(1)
        widget.setReadOnly(0)
        form += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
        del keywords['election']

        buttonsBar = X.div(_class = 'buttons-bar')
        form += buttonsBar
        actionButtonsBar = X.span(_class = 'action-buttons-bar')
        buttonsBar += actionButtonsBar
        actionButtonsBar += X.buttonInForm('abstain', 'abstainButton')
        actionButtonsBar += X.buttonInForm(
            'vote-blank',
            vote.getSlot('marks').getFieldOptionName('blankButton'))
        actionButtonsBar += X.buttonInForm('vote', 'voteButton')

        return writePageLayout(layout, headerTitle)
    edit.isPublicForWeb = 1

    def submit(self, id = '', electionId = '', **keywords):
        ballotsWeb = getWebForServerRole('ballots')
        electionsWeb = getWebForServerRole('elections')
        if keywords is None:
            keywords = {}
        keywords['id'] = id
        keywords['electionId'] = electionId
        userId = context.getVar('userId', default = '')
        userToken = context.getVar('userToken', default = '')
        error = 0
        if not userToken:
            return accessForbidden()
        if not electionId \
           or not electionsWeb.hasObject(electionId):
            return pageNotFound()
        if not electionsWeb.canVote(electionId):
            return accessForbidden()
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        elif isButtonSelected('abstainButton', keywords):
            try:
                ballotsWeb.abstainForVote(electionId)
            except faults.Fault:
                if context.getVar('debug'):
                    raise
                return accessForbidden()
            return redirect(X.url('/votes').add(
                'electionId', electionId).add('voterId', userId))
        election = electionsWeb.getObject(electionId)
        vote = election.newVote()
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            keywords['election'] = election
            vote.submitFields(keywords)
            del keywords['election']
        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/edit' % getUrlId(id))
            uri.add('electionId', electionId)
            uri.addKeywords(keywords)
            return redirect(uri)
        #try:
        id = ballotsWeb.vote(electionId, vote)
        #except faults.Fault:
        #    return accessForbidden()
        return redirect(X.url(id))
    submit.isPublicForWeb = 1

    def view(self, id = '', electionId = '', voterId = ''):
        ballotsWeb = getWebForServerRole('ballots')
        electionsWeb = getWebForServerRole('elections')
        peopleWeb = getWebForServerRole('people')
        userId = context.getVar('userId', default = '')
        if id:
            vote = ballotsWeb.getVote(id)
            if vote == 'secret':
                return accessForbidden()
            rememberObject(id)
            electionId = ballotsWeb.getElectionIdFromToken(
                vote.electionToken, serverId = extractServerId(id))
            if ballotsWeb.canGetVoteVoterId(id):
                voterId = ballotsWeb.getVoteVoterId(id)
            else:
                voterId = ''
            if not electionId \
               or not electionsWeb.hasObject(electionId):
                return pageNotFound()
            if not electionsWeb.canGetObject(electionId):
                return accessForbidden()
            election = electionsWeb.getObject(electionId)
            rememberObject(electionId)
        else:
            if not electionId \
               or not electionsWeb.hasObject(electionId):
                return pageNotFound()
            if not voterId \
               or not peopleWeb.hasObject(voterId):
                return pageNotFound()
            if not electionsWeb.canGetObject(electionId):
                return accessForbidden()
            if not peopleWeb.canGetObject(voterId):
                return accessForbidden()
            election = electionsWeb.getObject(electionId)
            rememberObject(electionId)
            voter = peopleWeb.getObject(voterId)
            if voter.voteTokens is not None \
               and voter.voteTokens.has_key(electionId):
                voteToken = voter.voteTokens[electionId]
                if voteToken == 'secret':
                    return accessForbidden()
                try:
                    vote = ballotsWeb.getVoteFromToken(voteToken)
                except faults.UnknownVoteToken:
                    vote = election.newVote()
                else:
                    id = vote.id
                    rememberObject(id)
            else:
                vote = election.newVote()

        keywords = {}
        keywords['election'] = election
        vote.makeFieldsFromInstance(keywords)
        vote.repairFields(keywords)

        slot = vote.getSlot('electionId')
        slot.getKind().makeFieldFromValue(slot, keywords, electionId)
        slot = election.getSlot('subject')
        slot.getKind().makeFieldFromValue(
            slot, keywords, election.subject)
        slot = vote.getSlot('voterId')
        slot.getKind().makeFieldFromValue(slot, keywords, voterId)

        label = election.getLabelTranslated(
            context.getVar('readLanguages'))

        layout = X.array()
        slot = slots.Root(vote)
        context.push(viewName = 'default')
        widget = slot.getWidget()
        widget.setInForm(0)
        layout += widget.getModelPageBodyLayout(slot, keywords)
        context.pull()
        del keywords['election']
        layout += X.br()
        if voterId and voterId == userId \
           and electionsWeb.canVote(electionId):
            layout += X.buttonStandalone(
                'edit',
                X.url('/%s/edit' % getUrlId(id)).add(
                'electionId', electionId))
        return writePageLayout(
            layout, '%s - %s' % (_(self.objectNameCapitalized), label))
    view.isPublicForWeb = 1

    def viewAll(self):
        if not self.canGetObjects():
            return accessForbidden()
        layout = X.array()

        if self.canGetAdmin():
            layout += X.buttonStandalone(
                'settings',
                X.url('/votes/admin'))
        return writePageLayout(layout, _('Votes'))
    viewAll.isPublicForWeb = 1

