# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import nested_scopes

__doc__ = """Glasnost Workflows Web"""

__version__ = '$Revision: 1.26 $'[11:-2]


import glasnost.common.faults as faults
import glasnost.common.context as context

from glasnost.proxy.WorkflowsProxy import *

from ObjectsWeb import register, AdminMixin, ObjectWebMixin, ObjectsWebMixin
from tools import *
import md5
import widgets

class WorkflowActionWidget(widgets.Thing):
    def getHtmlValue(self, slot, fields, **keywords):
        value = slot.getValue()
        layout = X.div(
            X.h4(_class = 'workflowActionName')(value.__class__.__name__),
            widgets.Thing.getHtmlValue(self, slot, fields, **keywords) )
        return layout
register(WorkflowActionWidget)


class AdminWorkflows(AdminMixin, AdminWorkflows):
    def getWeb(self):
        return getWebForServerRole('workflows')
register(AdminWorkflows)


class WorkflowAction(ObjectWebMixin, WorkflowAction):
    def getLayout(self, workflowId, **keywords):
        return None

    def getButtons(self, workflowId):
        return []

    def getWeb(self):
        return getWebForServerRole('workflows')
register(WorkflowAction)

class WorkflowActionShowMessage(WorkflowAction, WorkflowActionShowMessage):
    def getLayout(self, workflowId, **keywords):
        return None

    def getButtons(self, workflowId):
        workflowUrlId = getUrlId(workflowId)
        return [ X.buttonStandalone(
                    'next',
                    X.url('/%s/%snext' % (
            workflowUrlId,
            context.getVar('templatePrefix'))).add(
            'nextButton', _('Next'))) ]
register(WorkflowActionShowMessage)

class WorkflowActionEnd(WorkflowAction, WorkflowActionEnd):
    def getLayout(self, workflowId, **keywords):
        if self.message:
            return None
        return X.div()(N_('End of workflow'))
register(WorkflowActionEnd)

class WorkflowActionUserChoice(WorkflowAction, WorkflowActionUserChoice):
    def getLayout(self, workflowId, **keywords):
        layout = X.div()(self.message)
        return layout
    def getButtons(self, workflowId):
        workflowUrlId = getUrlId(workflowId)
        return [ X.buttonStandalone( # FIXME: shouldn't use X.buttonStandalone
                    x,
                    X.url('/%s/%snext' % (
            workflowUrlId,
            context.getVar('templatePrefix'))).add(
            'choice', md5.new(x).hexdigest())) for x in self.choices.keys() ]
register(WorkflowActionUserChoice)

class WorkflowActionAddObject(WorkflowAction, WorkflowActionAddObject):
    def getLayout(self, workflowId, **keywords):
        workflowUrlId = getUrlId(workflowId)
        if keywords is None:
            keywords = {}
        error = ''
        if keywords.has_key('error'):
            error = keywords['error']
        web = getWebForServerRole(self.objectType)
        object = web.newThing('object', web.objectClassName)
        if not keywords.has_key('again'):
            object.initFields(keywords)
        object.repairFields(keywords)
        layout = X.array()
        layout += object.getErrorLayout(error, keywords)
        form = X.form(action = X.url('/%s/%ssubmitAction' % (
            workflowUrlId, context.getVar('templatePrefix'))),
                      enctype= 'multipart/form-data', method = 'post')
        oldGetLayoutSlotNames = object.getLayoutSlotNames
        slotNames = object.getLayoutSlotNames(keywords)
        if self.ignoreFields:
            slotNames = [x for x in slotNames if x not in self.ignoreFields]
        def make_getSlotNames(l):
            def getSlotNames(self, **keywords): return l
            return getSlotNames
        object.getLayoutSlotNames = make_getSlotNames(slotNames)
        form += X.table(object.getEditLayout(keywords))
        object.getLayoutSlotNames = oldGetLayoutSlotNames
        form += X.buttonInForm('create', 'createButton')
        layout += form
        return layout

    def submitAction(self, workflowId, keywords):
        workflowUrlId = getUrlId(workflowId)
        if keywords is None:
            keywords = {}
        if isButtonSelected('applyButton', keywords):
            keywords['again'] = '1'
            keywords['hideErrors'] = '1'
        error = 0
        web = getWebForServerRole(self.objectType)
        object = web.newThing('object', web.objectClassName)
        if error:
            keywords['again'] = '1'
            keywords['error'] = '1'
        else:
            object.submitFields(keywords)
        open('/tmp/wf.errors', 'a+').write('%s\n' % repr(keywords))
        if self.ignoreFields:
            removed = 0
            for k in keywords.keys():
                p = k.split('_')
                p1, p2 = p[0], p[-1]
                if p[-1] == 'error' and p[0] in self.ignoreFields:
                    del keywords[k]
                    removed = 1
            l = [ x for x in keywords.keys() if '_' in x and x.split('_')[-1] == 'error' ]
            if len(l) == 0 and removed:
                del keywords['again']

        if keywords.has_key('again') and keywords['again']:
            uri = X.url('/%s/%srunning' % (
                workflowUrlId, context.getVar('templatePrefix')))
            uri.addKeywords(keywords)
            raise redirect(uri)

        return object
register(WorkflowActionAddObject)

class WorkflowActionEditObject(WorkflowAction, WorkflowActionEditObject):
    def getLayout(self, workflowId, **keywords):
        return X.div()('EditObjectAction')
register(WorkflowActionEditObject)

class WorkflowActionMail(WorkflowAction, WorkflowActionMail):
    def getLayout(self, workflowId, **keywords):
        return X.div()('MailAction')
register(WorkflowActionMail)

class WorkflowActionUserData(WorkflowAction, WorkflowActionUserData):
    userData = None
    userData_kind_isRequired = 1
    userData_kind_isTranslatable = 0
    userData_kindName = 'String'
    userData_kind_widget_fieldLabel = ''
    userData_kind_widgetName = 'InputText'

    def getLayout(self, workflowId, **keywords):
        workflowUrlId = getUrlId(workflowId)

        self.userData_kindName = self.dataKind.name
        slot = self.getSlot('userData')
        slot.getWidget().setInForm()
        slot.getWidget().setReadOnly(0)

        form = X.form(action = X.url('/%s/%ssubmitAction' % (
            workflowUrlId, context.getVar('templatePrefix'))),
                      enctype= 'multipart/form-data', method = 'post')
        form += slot.getWidget().getHtmlValue(slot, {})
        form += X.buttonInForm('ok', 'okButton')
        return form

    def submitAction(self, workflowId, keywords):
        workflowUrlId = getUrlId(workflowId)
        if keywords is None:
            keywords = {}
        if not keywords.has_key('userData'):
            uri = X.url('/%s/%srunning' % (
                workflowUrlId, context.getVar('templatePrefix')))
            raise redirect(uri)
        # hum: is this the right way to export a string ?
        var = workflowsWeb.newThing('kind', 'String')
        var.value_kind = var
        var.isPublicForXmlRpc = 1
        slot = var.getSlot('value')
        slot.setValue(keywords['userData'])
        return var
register(WorkflowActionUserData)

class WorkflowActionBranch(WorkflowAction, WorkflowActionBranch):
    pass
register(WorkflowActionBranch)

class WorkflowActionGoto(WorkflowAction, WorkflowActionGoto):
    pass
register(WorkflowActionGoto)

class Workflow(ObjectWebMixin, Workflow):
    name_kind_widget_fieldLabel = N_('Name')
    #name_kind_widget_maxLength = 40
    name_kind_widget_size = 40
    name_kind_widgetName = 'InputText'

    actions_kind_widgetName = 'Multi'
    actions_kind_widget_fieldLabel = N_('Actions')
    actions_kind_itemKind_value_widgetName = 'WorkflowActionWidget'

    startupObjectType_kind_widgetName = 'InputText'
    startupObjectType_kind_widget_fieldLabel = N_('Startup Object')

    startupEvents_kind_widgetName = 'Multi'
    startupEvents_kind_widget_fieldLabel = N_('Startup Events')
    startupEvents_kind_itemKind_value_widgetName = 'InputText'

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getEditLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        for slotName in ['actions']:
            if slotName in slotNames:
                slotNames.remove(slotName)
        return slotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        slotNames = ObjectWebMixin.getViewLayoutSlotNames(
            self, fields, parentSlot = parentSlot)
        slotNames = slotNames[:]
        userToken = context.getVar('userToken', default = '')
        if not userToken or (
            fields.has_key('useCompactLayout') and fields['useCompactLayout']):
            for slotName in (
                'readersSet', 'writersSet'):
                if slotName in slotNames:
                    slotNames.remove(slotName)
        return slotNames

    def getWeb(self):
        return workflowsWeb
register(Workflow)


class RunningWorkflow(ObjectWebMixin, RunningWorkflow):
    def getWeb(self):
        return workflowsWeb
register(RunningWorkflow)


class WorkflowsWeb(ObjectsWebMixin, WorkflowsProxy):
    def next(self, id, **keywords):
        workflowUrlId = getUrlId(id)
        if not self.hasObject(id):
            return pageNotFound()
#        try:
        self.advanceRunningWorkflow(id, keywords)
#        except:
#            # TODO: catch the right exception
#            layout = X.array()
#            layout += getHeaderLayout(_('Workflow ended'))
#            layout += 'Workflow ended'
#            layout += getFooterLayout()
#            return writePageLayout(layout)
        return redirect(X.url('/%s/%srunning' % (
            workflowUrlId, context.getVar('templatePrefix'))))
    next.isPublicForWeb = 1

    def running(self, id, **keywords):
        if not self.hasObject(id):
            return pageNotFound()

        rW = self.getRunningWorkflow(id)
        workflow = self.getObject(id)
        layout = X.array()

        if rW.message:
            layout += X.div()(rW.message)
        layout += rW.getLayout(id, **keywords)
        buttons = rW.getButtons(id)
        if buttons:
            div = X.div(_class = 'buttons')
            for b in buttons:
                div += b
            layout += div
        return writePageLayout(layout, workflow.getLabel())
    running.isPublicForWeb = 1

    def start(self, id, dataObjectId = '', **keywords):
        workflowUrlId = getUrlId(id)
        if not self.hasObject(id):
            return pageNotFound()
        if not self.canRunWorkflow(id):
            return accessForbidden()
        try:
            self.initWorkflow(id, dataObjectId)
        except:
            raise
        return redirect(X.url('/%s/%srunning' % (
            workflowUrlId, context.getVar('templatePrefix'))))
    start.isPublicForWeb = 1
    
    def submitAction(self, id, **keywords):
        workflowUrlId = getUrlId(id)
        if not self.hasObject(id):
            return pageNotFound()
        rW = self.getRunningWorkflow(id)
        data = rW.submitAction(id, keywords)
        try:
            self.advanceRunningWorkflow(id, data)
        except:
            pass
        return redirect(X.url('/%s/%srunning' % (
            workflowUrlId, context.getVar('templatePrefix'))))
    submitAction.isPublicForWeb = 1

