# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Web Kinds"""

__version__ = '$Revision: 1.71 $'[11:-2]


import re
import time

import glasnost.common.faults as faults
import glasnost.common.context as context
from glasnost.common import translation

import glasnost.proxy.kinds as proxyKinds

import things
from tools import *
import WebAPI

import functions # Do not remove!
import properties # Do not remove!
import uploads # Do not remove!
import views # Do not remove!
import widgets # Do not remove!


register = things.register


class EmptyKindMixin(things.ThingMixin):
    category_kind_isConstant = 1

    name_kind_isConstant = 1
    
##     def buildOptions(self, options):
##         self.inherited(EmptyKindMixin, 'buildOptions')(self, options)


class KindMixin(EmptyKindMixin):
    defaultValue_kind_isConstant = 1
    defaultValue_kind_isHidden = 1

    hasToInitField = 1
    hasToInitField_kind_isConstant = 1
    hasToInitField_kind_isHidden = 1
    hasToInitField_kind_isPublicForXmlRpc = 1
    hasToInitField_kindName = 'Boolean'

    hasToMakeFieldFromValue = 1
    hasToMakeFieldFromValue_kind_isConstant = 1
    hasToMakeFieldFromValue_kind_isHidden = 1
    hasToMakeFieldFromValue_kind_isPublicForXmlRpc = 1
    hasToMakeFieldFromValue_kindName = 'Boolean'

    hasToModify_kind_isConstant = 1
    hasToModify_kind_isHidden = 1

    hasToRepairField = 1
    hasToRepairField_kind_isConstant = 1
    hasToRepairField_kind_isHidden = 1
    hasToRepairField_kind_isPublicForXmlRpc = 1
    hasToRepairField_kindName = 'Boolean'

    hasToSubmitField = 1
    hasToSubmitField_kind_isConstant = 1
    hasToSubmitField_kind_isHidden = 1
    hasToSubmitField_kind_isPublicForXmlRpc = 1
    hasToSubmitField_kindName = 'Boolean'

    isAutomaticalyModified_kind_isConstant = 1
    isAutomaticalyModified_kind_isHidden = 1

    isConstant = 0
    isConstant_kind_isConstant = 1
    isConstant_kind_isHidden = 1
    isConstant_kind_isPublicForXmlRpc = 1
    isConstant_kindName = 'Boolean'

    isHidden = 0
    isHidden_kind_isConstant = 1
    isHidden_kind_isHidden = 1
    isHidden_kind_isPublicForXmlRpc = 1
    isHidden_kindName = 'Boolean'

    isProperty_kind_isConstant = 1
    isProperty_kind_isHidden = 1

    isPublicForXmlRpc_kind_isConstant = 1
    isPublicForXmlRpc_kind_isHidden = 1

    isTranslatable_kind_isConstant = 1
    isTranslatable_kind_isHidden = 1

    labelPlural_kind_isConstant = 1
    labelPlural_kind_isHidden = 1

    preference = None
    preference_kind_isConstant = 1
    preference_kind_isHidden = 1
    preference_kind_isPublicForXmlRpc = 1
    preference_kindName = 'String'

    possibleWidgetNames_kind_isConstant = 1
    possibleWidgetNames_kind_isHidden = 1

    useCustomStorage_kind_isConstant = 1
    useCustomStorage_kind_isHidden = 1

    useFileStorage_kind_isConstant = 1
    useFileStorage_kind_isHidden = 1

    widget_kind_isConstant = 1
    widget_kind_isHidden = 1

    widgetName_kind_titleFromValueGetterName = 'makeWidgetNameTitleFromValue'
    widgetName_kind_widgetSlotName = 'widget'

    def getHelpAlias(self, slot, fields):
        return self.helpAlias

    def getModelWidget(self, slot):
        if self.widget is None:
            widgetOptionHeader = 'widget_'
            widgetOptionHeaderLen = len(widgetOptionHeader)
            widgetOptionNames = [
                name
                for name in self.__dict__.keys()
                if name.startswith(widgetOptionHeader) \
                and not name.startswith(widgetOptionHeader + 'kind')]
            baseClasses = self.getC3ClassLinearization()
            for baseClass in baseClasses:
                for name in baseClass.__dict__.keys():
                    if name.startswith(widgetOptionHeader) \
                       and not name.startswith(widgetOptionHeader + 'kind') \
                       and name not in widgetOptionNames:
                        widgetOptionNames.append(name)
            widgetOptions = {}
            for widgetOptionName in widgetOptionNames:
                widgetOptions[widgetOptionName[widgetOptionHeaderLen:]
                              ] = getattr(self, widgetOptionName)
            widget = self.newModelWidget(slot)
            widget.buildOptions(widgetOptions)
            self.widget = widget
        return self.widget

    def getOrderedFieldSlotNames(self, fields, parentSlot = None):
        slotNames = EmptyKindMixin.getOrderedFieldSlotNames(
            self, fields, parentSlot = parentSlot)
        # The field widgetName must be set before the field widget.
        slotNames += ['widgetName', 'widget']
        return slotNames

    def initField(self, slot, fields):
        hasDefault = 0
        if self.preference:
            preferences = context.getVar('preferences')
            if preferences is not None:
                hasDefault = 1
                value = getattr(preferences, self.preference)
        elif self.defaultValue is not None:
            hasDefault = 1
            value = self.defaultValue
        if hasDefault:
            fieldValue = str(value)
            slot.setField(fields, fieldValue)

    def isEmptyModelValue(self, slot, value):
        return value is None

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        if value is None:
            fieldValue = ''
        else:
            fieldValue = value
        slot.setField(fields, fieldValue)

    def makeWidgetNameTitleFromValue(self, fields, value, parentSlot = None):
        container = parentSlot.getContainer()
        return container.makeWidgetNameTitleFromValue(
            fields, value, parentSlot = parentSlot.parent)

    def newModelWidget(self, slot):
        return self.newThing('widget', self.widgetName)
        
    def repairField(self, slot, fields, **keywords):
        pass

    def submitField(self, slot, fields, **keywords):
        assert fields is not None
        error, errorFields, value = self.submitField2(slot, fields, **keywords)
        if error:
            fields['again'] = '1'
            fields['error'] = str(error)
            fields.update(errorFields)
        elif not slot.hasValue() or value != slot.getValue():
            slot.setValue(value)

    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            value = fieldValue
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)


class AbstractSequenceMixin(KindMixin):
    def canAddItem(self, slot):
        return 1

    def getItemsCount(self, slot, fields):
        raise NotImplementedError

    def isLabelImportant(self, slot):
        raise NotImplementedError


class ChoiceMixin(KindMixin):
    titles_kind_isConstant = 1
    titles_kind_isHidden = 1

    def makeModelTitleFromValue(self, slot, fields, value):
        valueAsString = str(value)
        if self.titles and self.titles.has_key(valueAsString):
            return _(self.titles[valueAsString])
        return None

    def repairField(self, slot, fields, **keywords):
        values = self.getValues(slot, fields)
        # Values can be None when field is read-only or something like that.
        if values is not None and not slot.getField(fields) in values:
            value = self.defaultValue
            fieldValue = value
            slot.setField(fields, fieldValue)

    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        values = self.getValues(slot, fields)
        valuesAsStrings = [ str(val) for val in values ]
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        elif not fieldValue in valuesAsStrings:
            error = 2
            slot.setFieldOption(errorFields, 'error', 'wrongValue')
        else:
            if fieldValue in values:
                value = fieldValue
            else:
                i = valuesAsStrings.index(fieldValue)
                value = values[i]
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)


class IdMixin(KindMixin):
    pass


class SequenceMixin(AbstractSequenceMixin):
    fieldsCountMin = 1 # FIXME: To rename?
    fieldsCountMin_kind_isPublicForXmlRpc = 1
    fieldsCountMin_kindName = 'Integer'

##     item_widget = None
##     item_widget_kind_isPublicForXmlRpc = 1
##     item_widget_kindName = 'Widget'

    def getItemsCount(self, slot, fields):
        count = self.fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        return count

    def initField(self, slot, fields):
        if self.itemKind.preference \
           or self.itemKind.defaultValue:
            itemSlot = self.getItemSlot(slot, 0)
            self.itemKind.initField(itemSlot, fields)

    def isEmptyModelValue(self, slot, value):
        return not value

    def isLabelImportant(self, slot):
        return 0

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        count = self.fieldsCountMin
        if value is not None:
            count = max(count, len(value))
            for i in range(len(value)):
                itemSlot = self.getItemSlot(slot, i)
                self.itemKind.makeFieldFromValue(
                    itemSlot, fields, value[i], **keywords)
        slot.setFieldOption(fields, 'count', str(count))
    
    def repairField(self, slot, fields, **keywords):
        count = self.fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        slot.setFieldOption(fields, 'count', count)
        for i in range(count):
            itemSlot = self.getItemSlot(slot, i)
            self.itemKind.repairField(itemSlot, fields, **keywords)

    def submitField2(self, slot, fields, **keywords):
        error = 0
        errorFields = {}
        count = self.fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        addButtonName = slot.getFieldOptionName('addButton')
        if isButtonSelected(addButtonName, fields):
            count = count + 1
            slot.setFieldOption(fields, 'count', str(count))
            fields['again'] = '1'
            fields['hideErrors'] = '1'
            return (error, errorFields, None)
        findButtonName = slot.getFieldOptionName('findButton')
        if isButtonSelected(findButtonName, fields):
            foundsCount = 0
            try:
                foundsCount = max(foundsCount, int(slot.getFieldOption(
                    fields, 'foundsCount', default = '0')))
            except ValueError:
                pass
            for i in range(foundsCount):
                optionName = 'foundIds_%d' % i
                if slot.hasFieldOption(fields, optionName):
                    slot.delFieldOption(fields, optionName)
            serverRoles = self.serverRoles # FIXME: not sure at all
            if not serverRoles:
                serverRoles = context.getVar('knownRoles')
            serverRoles = serverRoles[:]
            serverRoles.sort()
            searchText = slot.getFieldOption(
                fields, 'searchText', default = '')
            foundIds = []
            for serverRole in serverRoles:
                web = getWebForServerRole(serverRole)
                if not web:
                    continue
                webFoundIds = web.findObjectIds(searchText)
                foundIds = foundIds + webFoundIds
            foundsCount = len(foundIds)
            slot.setFieldOption(fields, 'foundsCount', str(foundsCount))
            for i in range(foundsCount):
                slot.setFieldOption(fields, 'foundIds_%d' % i, foundIds[i])
            fields['again'] = '1'
            fields['hideErrors'] = '1'
            return (error, errorFields, None)
        value = []
        for i in range(count):
            itemSlot = self.getItemSlot(slot, i)
            itemError, itemErrorFields, itemValue = self.itemKind.submitField2(
                itemSlot, fields, **keywords)
            if itemError > 1 and itemError > error:
                error = itemError
                errorFields.update(itemErrorFields)
            if not itemError and not self.itemKind.isEmptyModelValue(
                itemSlot, itemValue):
                value.append(itemValue)
        if self.requiredCount:
            if len(value) < self.requiredCount:
                if not error:
                    error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            #    itemSlot = self.getItemSlot(slot, 0)
            #    if not itemSlot.hasFieldOption(errorFields, 'error'):
            #        itemSlot.setFieldOption(
            #            errorFields, 'error', 'missingValue')
        if self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)


class StringMixin(KindMixin):
    def initField(self, slot, fields):
        hasDefault = 0
        if self.preference:
            preferences = context.getVar('preferences')
            if preferences is not None:
                hasDefault = 1
                value = getattr(preferences, self.preference)
        elif self.defaultValue:
            hasDefault = 1
            value = self.defaultValue
        if hasDefault:
            fieldValue = value
            slot.setField(fields, fieldValue)

    def isEmptyModelValue(self, slot, value):
        return not value

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        if value is None:
            fieldValue = ''
        else:
            fieldValue = value
        slot.setField(fields, fieldValue)


class ThingMixin(KindMixin):
    def initField(self, slot, fields):
        value = None
        if slot.hasValue():
            value = slot.getValue()
        valueThingName = self.getValueThingNameFromFields(slot, fields)
        if value is None:
            value = self.newThing(self.valueThingCategory, valueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != self.valueThingCategory \
             or value.getThingName() != valueThingName:
            value = self.newThing(self.valueThingCategory, valueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        value.initFields(fields, parentSlot = parentSlot)

    def isEmptyModelValue(self, slot, value):
        if value is None:
            return 1
        parentSlot = slot.detach(value)
        return value.isEmpty(parentSlot = parentSlot)

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        valueThingName = self.getValueThingNameFromFields(slot, fields)
        if value is None:
            value = self.newThing(self.valueThingCategory, valueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != self.valueThingCategory \
             or value.getThingName() != valueThingName:
            value = self.newThing(self.valueThingCategory, valueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        value.makeFieldsFromInstance(fields, parentSlot = parentSlot)

    def newModelWidget(self, slot):
        value = None
        if slot.hasValue():
            value = slot.getValue()
        valueThingName = self.getValueThingName(slot)
        if value is None:
            value = self.newThing(self.valueThingCategory, valueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != self.valueThingCategory \
             or value.getThingName() != valueThingName:
            value = self.newThing(self.valueThingCategory, valueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        return value.newWidget(parentSlot = parentSlot)

    def repairField(self, slot, fields, **keywords):
        value = None
        if slot.hasValue():
            value = slot.getValue()
        valueThingName = self.getValueThingNameFromFields(slot, fields)
        if value is None:
            value = self.newThing(self.valueThingCategory, valueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != self.valueThingCategory \
             or value.getThingName() != valueThingName:
            value = self.newThing(self.valueThingCategory, valueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        value.repairFields(fields, parentSlot = parentSlot)

    def submitField2(self, slot, fields, **keywords):
        value = None
        if slot.hasValue():
            value = slot.getValue()
        valueThingName = self.getValueThingName(slot)
        if value is None:
            value = self.newThing(self.valueThingCategory, valueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != self.valueThingCategory \
             or value.getThingName() != valueThingName:
            value = self.newThing(self.valueThingCategory, valueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        # FIXME: Horrible hack, because I don't want to change the behavior of
        # submitFields now.
        if fields.has_key('again'):
            previousAgain = fields['again']
            del fields['again']
        else:
            previousAgain = None
        if fields.has_key('error'):
            previousError = fields['error']
            del fields['error']
        else:
            previousError = None
        value.submitFields(fields, parentSlot = parentSlot)
        error = 0
        errorFields = {}
        if fields.has_key('error') and fields['error']:
            try:
                valueError = int(fields['error'])
            except ValueError:
                valueError = 2
            except KeyError:
                valueError = 2
            if valueError > error:
                error = valueError
        if error == 1 and not self.isRequired:
            error = 0
            errorFields = {}
            value = None
            if fields.has_key('error'):
                del fields['error']
            if fields.has_key('again'):
                del fields['again']
            slotFieldName = slot.getFieldName()
            if slotFieldName:
                slotFieldName += '_'
            for fieldName in fields.keys():
                if fieldName.startswith(slotFieldName) \
                   and fieldName.endswith('_error'):
                    del fields[fieldName]
        if previousAgain is not None:
            fields['again'] = previousAgain
        if previousError is not None:
            fields['error'] = previousError
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)


class TimeMixin(KindMixin):
    widget_maxLength = 19
    widget_size = 19

    def initField(self, slot, fields):
        hasDefault = 0
        if self.preference:
            preferences = context.getVar('preferences')
            if preferences is not None:
                hasDefault = 1
                value = getattr(preferences, self.preference)
        elif self.defaultValue:
            hasDefault = 1
            value = self.defaultValue
        if hasDefault:
            if value == 'now':
                value = time.time()
                fieldValue = time.strftime('%Y-%m-%d %H:%M:%S',
                                           time.localtime(value))
            else:
                fieldValue = ''
            slot.setField(fields, fieldValue)

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        try:
            localtime = time.localtime(value)
            if tuple(localtime[3:6]) == (0, 0, 0):
                fieldValue = time.strftime('%Y-%m-%d',
                                           time.localtime(value))
            else:
                fieldValue = time.strftime('%Y-%m-%d %H:%M:%S',
                                           time.localtime(value))
        except:
            if self.defaultValue == 'now':
                fieldValue = time.strftime('%Y-%m-%d %H:%M:%S',
                                           time.localtime(time.time()))
            else:
                fieldValue = ''
        slot.setField(fields, fieldValue)

    def repairField(self, slot, fields, **keywords):
        pass

    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            formatDate = ( '%Y-%m-%d', '%y-%m-%d', '%d/%m/%Y', '%d/%m/%y' )
            formatTime = ( '%H:%M:%S', '%H:%M', '' )
            try:
                for format in [x+' '+y for x in formatDate for y in formatTime]:
                    try:
                        t = time.strptime(fieldValue, format)
                    except ValueError:
                        continue
                    break
                else:
                    raise ValueError
                # Set Daylight Saving Time to -1, so that it is handled
                # correctly by mktime.
                t = tuple(list(t[0:-1]) + [-1])
                value = time.mktime(t)
            except ValueError:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'wrongValue')
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)


class TokenMixin(KindMixin):
    pass


class BaseKind(KindMixin, proxyKinds.BaseKind):
    pass
register(BaseKind)


class Alias(StringMixin, proxyKinds.Alias):
    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            if ' ' in fieldValue or '/' in fieldValue:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'wrongValue')
            if not error:
                value = fieldValue
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Alias)


class Amount(KindMixin, proxyKinds.Amount):
    def initField(self, slot, fields):
        hasDefault = 0
        if self.preference:
            preferences = context.getVar('preferences')
            if preferences is not None:
                hasDefault = 1
                value = getattr(preferences, self.preference)
        elif self.defaultValue:
            hasDefault = 1
            value = getattr(self, self.defaultValue)
        if hasDefault:
            if value == 0:
                mainFieldValue = '0'
                centsFieldValue = '00'
                sideFieldValue = 'credit'
            else:
                if value < 0:
                    sideFieldValue = 'debit'
                    absValue = - value
                else:
                    sideFieldValue = 'credit'
                    absValue = value
                absFieldValue = str(absValue)
                mainFieldValue = absFieldValue[:-2]
                centsFieldValue = absFieldValue[-2:]
            slot.setFieldOption(fields, 'main', mainFieldValue)
            slot.setFieldOption(fields, 'cents', centsFieldValue)
            slot.setFieldOption(fields, 'side', sideFieldValue)

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        assert type(value) == type(1)
        if value < 0:
            sideFieldValue = 'debit'
            absValue = - value
        else:
            sideFieldValue = 'credit'
            absValue = value
        amountFieldValue = str(absValue)
        if len(amountFieldValue) == 1:
            centsFieldValue = '0' + amountFieldValue
        else:
            centsFieldValue = amountFieldValue[-2:]
        if len(amountFieldValue) > 2:
            mainFieldValue = amountFieldValue[:-2]
        else:
            mainFieldValue = '0'
        slot.setFieldOption(fields, 'main', mainFieldValue)
        slot.setFieldOption(fields, 'cents', centsFieldValue)
        slot.setFieldOption(fields, 'side', sideFieldValue)
    
    def submitField2(self, slot, fields, **keywords):
        assert fields is not None
        mainFieldValue = slot.getFieldOption(fields, 'main', default = '0')
        centsFieldValue = slot.getFieldOption(fields, 'cents', default = '00')
        sideFieldValue = slot.getFieldOption(
            fields, 'side', default = 'credit')
        error = 0
        errorFields = {}
        value = 0
        try:
            mainValue = string.atoi(mainFieldValue) 
            centsValue = string.atoi(centsFieldValue) 
            value = mainValue * 100 + centsValue
            assert sideFieldValue in ['credit', 'debit']
            if sideFieldValue == 'debit':
                value = - value
        except:
            error = 2
            slot.setFieldOption(errorFields, 'error', 'wrongValue')
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Amount)


class ApplicationToken(TokenMixin, proxyKinds.ApplicationToken):
    pass
register(ApplicationToken)


class AuthorsSet(SequenceMixin, proxyKinds.AuthorsSet):
    itemKind_value_defaultValue = 'user'
register(AuthorsSet)


class Boolean(KindMixin, proxyKinds.Boolean):
    values_kind_isConstant = 1
    values_kind_isHidden = 1

    titles_kind_isConstant = 1
    titles_kind_isHidden = 1

    def initField(self, slot, fields):
        hasDefault = 0
        if self.preference:
            preferences = context.getVar('preferences')
            if preferences is not None:
                hasDefault = 1
                value = getattr(preferences, self.preference)
        elif self.defaultValue:
            hasDefault = 1
            value = self.defaultValue
        if hasDefault:
            if value:
                fieldValue = '1'
            else:
                fieldValue = ''
            slot.setField(fields, fieldValue)

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        if value:
            fieldValue = '1'
        else:
            fieldValue = ''
        slot.setField(fields, fieldValue)

    def makeModelTitleFromValue(self, slot, fields, value):
        valueAsString = str(value)
        if self.titles and self.titles.has_key(valueAsString):
            return _(self.titles[valueAsString])
        return None
    
    def repairField(self, slot, fields, **keywords):
        if not slot.getField(fields, default = '') in ['', '1']:
            if self.defaultValue:
                value = self.defaultValue
            else:
                value = 0
            if value:
                fieldValue = '1'
            else:
                fieldValue = ''
            slot.setField(fields, fieldValue)

    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        if fieldValue:
            value = 1
        else:
            value = 0
        return (error, errorFields, value)
register(Boolean)


class Choice(ChoiceMixin, proxyKinds.Choice):
    pass
register(Choice)


class IntegerChoice(ChoiceMixin, proxyKinds.IntegerChoice):
    pass
register(IntegerChoice)


class CreationTime(TimeMixin, proxyKinds.CreationTime):
    pass
register(CreationTime)


class Data(KindMixin, proxyKinds.Data):
    pass
register(Data)


class Date(KindMixin, proxyKinds.Date):
    def initField(self, slot, fields):
        hasDefault = 0
        value = ''
        if self.preference:
            preferences = context.getVar('preferences')
            if preferences is not None:
                hasDefault = 1
                value = getattr(preferences, self.preference)
        elif self.defaultValue:
            hasDefault = 1
            value = self.defaultValue
        if hasDefault:
            if value == 'now':
                valueAsString = time.strftime(
                    '%Y%m%d', time.localtime(time.time()))
                yearFieldValue = valueAsString[:4]
                monthFieldValue = valueAsString[4:6]
                dayFieldValue = valueAsString[6:8]
        else:
            yearFieldValue = ''
            monthFieldValue = ''
            dayFieldValue = ''
        if 'year' in self.requiredParts:
            slot.setFieldOption(fields, 'year', yearFieldValue)
        if 'month' in self.requiredParts:
            slot.setFieldOption(fields, 'month', monthFieldValue)
        if 'day' in self.requiredParts:
            slot.setFieldOption(fields, 'day', dayFieldValue)

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        try:
            valueAsString = str(time.strftime('%Y%m%d', time.localtime(value)))
            yearFieldValue = valueAsString[:4]
            monthFieldValue = valueAsString[4:6]
            dayFieldValue = valueAsString[6:8]
        except:
            if self.defaultValue == 'now':
                valueAsString = time.strftime(
                    '%Y%m%d', time.localtime(time.time()))
                yearFieldValue = valueAsString[:4]
                monthFieldValue = valueAsString[4:6]
                dayFieldValue = valueAsString[6:8]
            else:
                yearFieldValue = ''
                monthFieldValue = ''
                dayFieldValue = ''
        if 'year' in self.requiredParts:
            slot.setFieldOption(fields, 'year', yearFieldValue)
        if 'month' in self.requiredParts:
            slot.setFieldOption(fields, 'month', monthFieldValue)
        if 'day' in self.requiredParts:
            slot.setFieldOption(fields, 'day', dayFieldValue)
    
    def submitField2(self, slot, fields, **keywords):
        assert fields is not None
        error = 0
        errorFields = {}
        valueAsString = ''
        for partName in ('year', 'month', 'day'):
            partFieldValue = slot.getFieldOption(
                fields, partName, default = '')
            if partFieldValue:
                try:
                    string.atoi(partFieldValue)
                except ValueError:
                    slot.setFieldOption(errorFields, 'error', 'wrongValue')
                valueAsString += partFieldValue
            else:        
                if self.isRequired and partName in self.requiredParts:
                    error = 1
                    slot.setFieldOption(errorFields, 'error', 'missingValue')
                    return (error, {}, None)
                else:
                    valueAsString += '00'
        if valueAsString and len(valueAsString) != 8:
            error = 2
            valueAsString = None
            slot.setFieldOption(errorFields, 'error', 'wrongValue')
        if not error and self.isEmptyModelValue(slot, valueAsString):
            valueAsString = None
        if valueAsString:
            value = time.mktime(time.strptime(valueAsString, '%Y%m%d'))
        else:
            value = None
        return (error, errorFields, value)
register(Date)

class Duration(KindMixin, proxyKinds.Duration):
    def initField(self, slot, fields):
        self.makeFieldFromValue(slot, fields, 0)

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        secondFieldValue = value % 60
        minuteFieldValue = value / 60 % 60
        hourFieldValue = value / 60 / 60 % 24
        dayFieldValue = value / 60 / 60 / 24

        if 'second' in self.requiredParts:
            slot.setFieldOption(fields, 'second', secondFieldValue)
        if 'minute' in self.requiredParts:
            slot.setFieldOption(fields, 'minute', minuteFieldValue)
        if 'hour' in self.requiredParts:
            slot.setFieldOption(fields, 'hour', hourFieldValue)
        if 'day' in self.requiredParts:
            slot.setFieldOption(fields, 'day', dayFieldValue)

    def submitField2(self, slot, fields, **keywords):
        assert fields is not None
        error = 0
        errorFields = {}
        value = 0

        factor = {
            'second' : 1, 'minute' : 60, 'hour' : 60*60, 'day' : 24*60*60}
        for partName in ('second', 'minute', 'hour', 'day'):
            partFieldValue = slot.getFieldOption(fields, partName, default = 0)
            if partFieldValue:
                try:
                    partFieldValue = string.atoi(partFieldValue)
                except ValueError:
                    slot.setFieldOption(errorFields, 'error', 'wrongValue')
                value += partFieldValue * factor[partName]
            else:        
                if self.isRequired and partName in self.requiredParts:
                    error = 1
                    slot.setFieldOption(errorFields, 'error', 'missingValue')
                    return (error, errorFields, None)
                else:
                    value += 0

        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Duration)


class Email(StringMixin, proxyKinds.Email):
    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            if fieldValue.count('@') != 1:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'wrongValue')
            if not error:
                value = fieldValue
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Email)


class FilePath(StringMixin, proxyKinds.FilePath):
    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            value = fieldValue
            value = value.replace(' ', '')
            if self.checkFilePath:
                systemFilesWeb = getWebForServerRole('systemfiles')
                try:
                    systemFilesWeb.checkFilePath(value)
                except faults.NonExistentFilePath:
                    error = 2
                    slot.setFieldOption(
                        errorFields, 'error', 'nonExistentFilePath')
                except faults.NotAFile:
                    error = 2
                    slot.setFieldOption(errorFields, 'error', 'notAFile')
                except faults.FileAccessDenied:
                    error = 2
                    slot.setFieldOption(
                        errorFields, 'error', 'fileAccessDenied')
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(FilePath)


class Fingerprint(StringMixin, proxyKinds.Fingerprint):
    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            value = fieldValue
            value = value.replace(' ', '')
            if len(value) < 40:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'tooSmallValue')
            elif len(value) > 40:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'tooBigValue')
            if not error:
                value = value.upper()
                for char in value:
                    if char not in '0123456789ABCDEF':
                        error = 2
                        slot.setFieldOption(errorFields, 'error', 'wrongValue')
                        break
            if not error:
                value = '%s %s %s %s %s  %s %s %s %s %s' % (
                    value[0:4],
                    value[4:8],
                    value[8:12],
                    value[12:16],
                    value[16:20],
                    value[20:24],
                    value[24:28],
                    value[28:32],
                    value[32:36],
                    value[36:40]
                    )
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Fingerprint)


class Float(KindMixin, proxyKinds.Float):
    pass
register(Float)


class FunctionName(ChoiceMixin, proxyKinds.FunctionName):
    pass
register(FunctionName)


class Id(IdMixin, proxyKinds.Id):
    def initField(self, slot, fields):
        hasDefault = 0
        if self.preference:
            preferences = context.getVar('preferences')
            if preferences is not None:
                hasDefault = 1
                value = getattr(preferences, self.preference)
        elif self.defaultValue:
            hasDefault = 1
            value = self.defaultValue
        if hasDefault:
            if value == 'user':
                userId = context.getVar('userId', default = '')
                fieldValue = userId
            else:
                fieldValue = str(value)
            slot.setField(fields, fieldValue)
register(Id)


class Integer(KindMixin, proxyKinds.Integer):
    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            try:
                value = int(fieldValue)
            except ValueError:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'wrongValue')
            if not error and self.min is not None and value < self.min:
                error = 2
                value = None
                slot.setFieldOption(errorFields, 'error', 'tooSmallValue')
            if not error and self.max is not None and value > self.max:
                error = 2
                value = None
                slot.setFieldOption(errorFields, 'error', 'tooBigValue')
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Integer)


class Kind(ThingMixin, proxyKinds.Kind):
    pass
register(Kind)


class KindName(ChoiceMixin, proxyKinds.KindName):
    kindSlotName = None
    kindSlotName_kind_isPublicForXmlRpc = 1
    kindSlotName_kind_isRequired = 1
    kindSlotName_kindName = 'SlotName'

    titleFromValueGetterName = None
    titleFromValueGetterName_kind_isPublicForXmlRpc = 1
    titleFromValueGetterName_kind_isRequired = 1
    titleFromValueGetterName_kindName = 'PythonIdentifier'

    def __init__(self, **attributes):
        proxyKinds.KindName.__init__(self, **attributes)
        self.labels = {}
        for kindClass in self.getAllThingClasses()['kind'].values():
            value = kindClass.getThingName.im_func(kindClass)
            self.labels[value] = kindClass.getName.im_func(kindClass)

    def makeModelTitleFromValue(self, slot, fields, value):
        container = slot.getContainer()
        titleFromValueGetterName = self.titleFromValueGetterName
        if titleFromValueGetterName is None:
            return _(self.getLabels(slot, fields)[str(value)])
        titleFromValueGetter = getattr(container, titleFromValueGetterName)
        return titleFromValueGetter(fields, value, parentSlot = slot.parent)

    def submitField(self, slot, fields, **keywords):
        kindSlotName = self.kindSlotName
        if kindSlotName is None:
            return ChoiceMixin.submitField(self, slot, fields, **keywords)
        assert fields is not None
        error, errorFields, value = self.submitField2(slot, fields, **keywords)
        if error:
            fields['again'] = '1'
            fields['error'] = str(error)
            fields.update(errorFields)
        else:
            if not slot.hasValue() or value != slot.getValue():
                slot.setValue(value)
            assert value
            kindValue = self.getAllThingClasses()['kind'][value]()
            kindValue.isPublicForXmlRpc = 1
            kindSlot = slot.getContainer().getSlot(
                kindSlotName, parentSlot = slot.parent)
            oldKindValue = kindSlot.getValue()
            if oldKindValue is None \
                   or oldKindValue.__class__ != kindValue.__class__:
                kindSlot.setValue(kindValue)
register(KindName)


class LanguageChoice(ChoiceMixin, proxyKinds.LanguageChoice):
    def initField(self, slot, fields):
        slot.setField(fields, context.getVar('readLanguages')[0])
register(LanguageChoice)


class Link(KindMixin, proxyKinds.Link):
    def isEmptyModelValue(self, slot, value):
        return value is None or not value[0] and not value[1]

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        if value:
            nameFieldValue, urlFieldValue = value
            if not urlFieldValue:
                urlFieldValue = 'http://'
        else:
            nameFieldValue = ''
            urlFieldValue = 'http://'
        slot.setFieldOption(fields, 'name', nameFieldValue)
        slot.setFieldOption(fields, 'url', urlFieldValue)
    
    def repairField(self, slot, fields, **keywords):
        if not slot.getFieldOption(fields, 'url'):
            slot.setFieldOption(fields, 'url', 'http://')

    def submitField2(self, slot, fields, **keywords):
        nameFieldValue = slot.getFieldOption(fields, 'name')
        urlFieldValue = slot.getFieldOption(fields, 'url')
        error = 0
        errorFields = {}
        value = None
        if urlFieldValue == 'http://':
            value = (nameFieldValue, '')
        else:
            value = (nameFieldValue, urlFieldValue)
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Link)


class Mapping(KindMixin, proxyKinds.Mapping):
    fieldsCountMin = 1 # FIXME: To rename?
    fieldsCountMin_kind_isPublicForXmlRpc = 1
    fieldsCountMin_kindName = 'Integer'

##     key_widget = None
##     key_widget_kind_isPublicForXmlRpc = 1
##     key_widget_kindName = 'Widget'

##     value_widget = None
##     value_widget_kind_isPublicForXmlRpc = 1
##     value_widget_kindName = 'Widget'

    def initField(self, slot, fields):
        hasDefault = 0
        value = {}
        if self.defaultValue:
            hasDefault = 1
            value = self.defaultValue
        if value or hasattr(self.valueKind, 'defaultValue') \
               or hasattr(self.keyKind, 'defaultValues'):
            self.makeFieldFromValue(slot, fields, value)
            
    def getItemsCount(self, slot, fields):
        count = self.fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        return count

    def canAddItem(self, slot):
        return not self.keyKind.isReadOnly

    def isEmptyModelValue(self, slot, value):
        return not value

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        count = self.fieldsCountMin
        valueKeys = []
        if hasattr(self.keyKind, 'values') and self.keyKind.values \
               and self.keyKind.isReadOnly:
            valueKeys = self.keyKind.values
        elif value:
            valueKeys = value.keys()
        elif hasattr(self.keyKind, 'defaultValues') \
                 and self.keyKind.defaultValues:
            valueKeys = self.keyKind.defaultValues
        #else:
        #    keys = []
            #for i in range(max(count, 1)):
            #    keys.append('')
        count = max(count, len(valueKeys))
        if hasattr(self.valueKind, 'defaultValue'):
            defaultValue = self.valueKind.defaultValue
        else:
            defaultValue = ''
        if count and value is None:
            value = {}
        if value is not None:
            for i in range(count):
                keySlot = self.getItemKeySlot(slot, i)
                if i < len(valueKeys):
                    valueKey = valueKeys[i]
                else:
                    valueKey = ''
                if not valueKey in value.keys() or not value[valueKey]:
                    value[valueKey] = defaultValue
                self.keyKind.makeFieldFromValue(
                    keySlot, fields, valueKey, **keywords)
                valueSlot = self.getItemKeyValueSlot(keySlot)
                self.valueKind.makeFieldFromValue(
                    valueSlot, fields, value[valueKey], **keywords)
            slot.setFieldOption(fields, 'count', str(count))
    
    def repairField(self, slot, fields, **keywords):
        count = self.fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        slot.setFieldOption(fields, 'count', count)
        for i in range(count):
            keySlot = self.getItemKeySlot(slot, i)
            self.keyKind.repairField(keySlot, fields, **keywords)
            valueSlot = self.getItemKeyValueSlot(keySlot)
            self.valueKind.repairField(valueSlot, fields, **keywords)

    def submitField2(self, slot, fields, **keywords):
        error = 0
        errorFields = {}
        count = self.fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        addButtonName = slot.getFieldOptionName('addButton')
        if isButtonSelected(addButtonName, fields):
            count = count + 1
            slot.setFieldOption(fields, 'count', str(count))
            fields['again'] = '1'
            fields['hideErrors'] = '1'
            return (error, errorFields, None)
        value = {}
        for i in range(count):
            keySlot = self.getItemKeySlot(slot, i)
            keyError, keyErrorFields, itemKey = self.keyKind.submitField2(
                keySlot, fields, **keywords)
            if keyError > 1 and keyError > error:
                error = keyError
                errorFields.update(keyErrorFields)
            valueSlot = self.getItemKeyValueSlot(keySlot)
            valueError, valueErrorFields, itemValue = \
                self.valueKind.submitField2(valueSlot, fields, **keywords)
            if valueError > 1 and valueError > error:
                error = valueError
                errorFields.update(valueErrorFields)
            if keyError or valueError \
                   or self.keyKind.isEmptyModelValue(keySlot, itemKey):
                continue
            if self.deleteNullValues and not itemValue:
                continue
            value[itemKey] = itemValue
        if self.requiredCount:
            if len(value) < self.requiredCount:
                if not error:
                    error = 1
                keySlot = self.getItemKeySlot(slot, 0)
                if not keySlot.hasFieldOption(errorFields, 'error'):
                    keySlot.setFieldOption(
                        errorFields, 'error', 'missingValue')
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Mapping)


class Marks(KindMixin, proxyKinds.Marks):
    idsGetterName = None
    idsGetterName_kind_isPublicForXmlRpc = 1
    idsGetterName_kindName = 'String'

    def makeFieldFromValue(self, slot, fields, value):
        if value is not None:
            for objectId, mark in value.items():
                if mark is None:
                    fieldValue = ''
                else:
                    fieldValue = str(mark)
                slot.setFieldOption(fields, objectId, fieldValue)
register(Marks)


class ApprovalMarks(Marks):
    defaultValue_kindName = 'ApprovalMarks'

    def makeFieldFromValue(self, slot, fields, value):
        if value is not None:
            for objectId, mark in value.items():
                if mark:
                    fieldValue = '1'
                else:
                    fieldValue = ''
                slot.setFieldOption(fields, objectId, fieldValue)

    def submitField2(self, slot, fields):
        error = 0
        errorFields = {}
        container = slot.getContainer()
        idsGetterName = self.idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        isBlank = isButtonSelected(
            slot.getFieldOptionName('blankButton'), fields)
        marks = {}
        for objectId in objectIds:
            if isBlank or not slot.getFieldOption(fields, objectId):
                mark = 0
            else:
                mark = 1
            marks[objectId] = mark
        if not error:
            marks = container.repairMarks(marks, objectIds)
            if self.isEmptyModelValue(slot, marks):
                marks = None
        return (error, errorFields, marks)
register(ApprovalMarks)


class DistributionMarks(Marks):
    defaultValue_kindName = 'DistributionMarks'

    def makeFieldFromValue(self, slot, fields, value):
        if value is not None:
            for objectId, mark in value.items():
                if mark is None:
                    fieldValue = ''
                else:
                    fieldValue = '%.2f' % (mark * 100)
                slot.setFieldOption(fields, objectId, fieldValue)

    def submitField2(self, slot, fields):
        error = 0
        errorFields = {}
        container = slot.getContainer()
        idsGetterName = self.idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        isBlank = isButtonSelected(
            slot.getFieldOptionName('blankButton'), fields)
        marks = {}
        for objectId in objectIds:
            if isBlank or not slot.hasFieldOption(fields, objectId):
                mark = 0
            else:
                mark = slot.getFieldOption(fields, objectId)
                if mark and mark.strip():
                    try:
                        mark = float(mark) / 100.0
                    except ValueError:
                        error = 2
                        slot.setFieldOption(
                            errorFields, objectId + '_error', 'wrongValue')
                        mark = 0
                else:
                    mark = 0
            marks[objectId] = mark
        if not error:
            marks = container.repairMarks(marks, objectIds)
            if self.isEmptyModelValue(slot, marks):
                marks = None
        return (error, errorFields, marks)
register(DistributionMarks)


class RankingMarks(Marks):
    defaultValue_kindName = 'RankingMarks'

    def submitField2(self, slot, fields):
        error = 0
        errorFields = {}
        container = slot.getContainer()
        idsGetterName = self.idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        isBlank = isButtonSelected(
            slot.getFieldOptionName('blankButton'), fields)
        marks = {}
        for objectId in objectIds:
            if isBlank or not slot.hasFieldOption(fields, objectId):
                mark = None
            else:
                mark = slot.getFieldOption(fields, objectId)
                if mark and mark.strip():
                    try:
                        mark = int(mark)
                    except ValueError:
                        error = 2
                        slot.setFieldOption(
                            errorFields, objectId + '_error', 'wrongValue')
                        mark = None
                else:
                    mark = None
            marks[objectId] = mark
        if not error:
            marks = container.repairMarks(marks, objectIds)
            if self.isEmptyModelValue(slot, marks):
                marks = None
        return (error, errorFields, marks)
register(RankingMarks)


class RatingMarks(Marks):
    defaultValue_kindName = 'RatingMarks'

    def submitField2(self, slot, fields):
        error = 0
        errorFields = {}
        container = slot.getContainer()
        idsGetterName = self.idsGetterName
        idsGetter = getattr(container, idsGetterName)
        objectIds = idsGetter(fields)
        isBlank = isButtonSelected(
            slot.getFieldOptionName('blankButton'), fields)
        marks = {}
        for objectId in objectIds:
            if isBlank or not slot.hasFieldOption(fields, objectId):
                mark = 0
            else:
                mark = slot.getFieldOption(fields, objectId)
                if mark and mark.strip():
                    try:
                        mark = float(mark)
                    except ValueError:
                        error = 2
                        slot.setFieldOption(
                            errorFields, objectId + '_error', 'wrongValue')
                        mark = 0
                else:
                    mark = 0
            marks[objectId] = mark
        if not error:
            marks = container.repairMarks(marks, objectIds)
            if self.isEmptyModelValue(slot, marks):
                marks = None
        return (error, errorFields, marks)
register(RatingMarks)


class Memory(KindMixin, proxyKinds.Memory):
    pass
register(Memory)


class Mutation(EmptyKindMixin, properties.PropertiesContainerMixin,
               proxyKinds.Mutation):
    pass
register(Mutation)


class ModificationTime(TimeMixin, proxyKinds.ModificationTime):
    pass
register(ModificationTime)


class PairwiseMatrix(KindMixin, proxyKinds.PairwiseMatrix):
    pass
register(PairwiseMatrix)


class Password(StringMixin, proxyKinds.Password):
    isNewPassword = 0
    isNewPassword_kind_isPublicForXmlRpc = 1
    isNewPassword_kindName = 'Boolean'

    def initField(self, slot, fields):
        StringMixin.initField(self, slot, fields)
        if self.isNewPassword:
            slot.setFieldOption(fields, 'again', slot.getField(fields))

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        StringMixin.makeFieldFromValue(
            self, slot, fields, value, **keywords)
        if self.isNewPassword:
            slot.setFieldOption(fields, 'again', slot.getField(fields))

    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if self.isNewPassword:
            againFieldValue = slot.getFieldOption(
                fields, 'again', default = '')
            if fieldValue != againFieldValue:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'wrongValue')
                return (error, errorFields, value)
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            value = fieldValue
            if self.isEmptyModelValue(slot, value):
                value = None
        return (error, errorFields, value)
register(Password)


class Path(StringMixin, proxyKinds.Path):
    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        else:
            value = fieldValue
            value = value.replace(' ', '')
            if self.checkFilePath:
                systemFilesWeb = getWebForServerRole('systemfiles')
                try:
                    systemFilesWeb.checkFilePath(value)
                except faults.NonExistentFilePath:
                    error = 2
                    slot.setFieldOption(
                        errorFields, 'error', 'nonExistentFilePath')
                except faults.NotAFile:
                    pass
                except faults.FileAccessDenied:
                    error = 2
                    slot.setFieldOption(
                        errorFields, 'error', 'fileAccessDenied')
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Path)


class Properties(SequenceMixin, proxyKinds.Properties):
    fieldsCountMin = 0

    isConstant = 1

    isHidden = 1
register(Properties)


class PythonIdentifier(StringMixin, proxyKinds.PythonIdentifier):
    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
        else:
            if re.match('[a-zA-Z_][a-zA-Z0-9_]*$', fieldValue) is None:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'wrongValue')
            if not error:
                value = fieldValue
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(PythonIdentifier)


class Rating(KindMixin, proxyKinds.Rating):
    pass
register(Rating)


class ReadersSet(SequenceMixin, proxyKinds.ReadersSet):
    itemKind_value_defaultValue = 'user'
    itemKind_value_widget_noneLabel = N_('Everybody')
register(ReadersSet)


class Sequence(SequenceMixin, proxyKinds.Sequence):
    pass
register(Sequence)


class ServerId(IdMixin, proxyKinds.ServerId):
    pass
register(ServerId)


class ServerRole(ChoiceMixin, proxyKinds.ServerRole):
    pass
register(ServerRole)


class SlotName(ChoiceMixin, proxyKinds.SlotName):
    pass
register(SlotName)


class String(StringMixin, proxyKinds.String):
    useFileStorage_kind_isConstant = 0
    useFileStorage_kind_isHidden = 0
register(String)


class Structure(AbstractSequenceMixin, proxyKinds.Structure):
    def canAddItem(self, slot):
        return 0

    def getItemsCount(self, slot, fields):
        if self.parameters is None:
            return 0
        else:
            return len(self.parameters)

    def initField(self, slot, fields):
        if self.parameters is not None:
            for i in range(len(self.parameters)):
                itemSlot = self.getItemSlot(slot, i)
                itemSlot.getKind().initField(itemSlot, fields)

    def isEmptyModelValue(self, slot, value):
        if value is None:
            return 1
        for i in range(len(self.parameters)):
            itemSlot = self.getItemSlot(slot, i)
            if not itemSlot.getKind().isEmptyModelValue(itemSlot, value[i]):
                return 0
        return 1

    def isLabelImportant(self, slot):
        return 1

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        if value is not None:
            for i in range(len(value)):
                itemSlot = self.getItemSlot(slot, i)
                itemSlot.getKind().makeFieldFromValue(
                    itemSlot, fields, value[i], **keywords)
    
    def repairField(self, slot, fields, **keywords):
        if self.parameters is not None:
            for i in range(len(self.parameters)):
                itemSlot = self.getItemSlot(slot, i)
                itemSlot.getKind().repairField(
                    itemSlot, fields, **keywords)

    def submitField2(self, slot, fields, **keywords):
        error = 0
        errorFields = {}
        value = []
        if self.parameters is not None:
            for i in range(len(self.parameters)):
                itemSlot = self.getItemSlot(slot, i)
                itemError, itemErrorFields, itemValue \
                    = itemSlot.getKind().submitField2(
                    itemSlot, fields, **keywords)
                if itemError > error:
                    error = itemError
                    errorFields.update(itemErrorFields)
                value.append(itemValue)
        if error == 1 and not self.isRequired:
            error = 0
            errorFields = {}
            value = None
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(Structure)


class Thing(ThingMixin, proxyKinds.Thing):
    pass
register(Thing)


class Time(TimeMixin, proxyKinds.Time):
    pass
register(Time)


class Token(TokenMixin, proxyKinds.Token):
    pass
register(Token)


class TranslationToAdd(KindMixin, proxyKinds.TranslationToAdd):
    def makeFieldFromValue(self, slot, fields, value, **keywords):
        pass

    def repairField(self, slot, fields, **keywords):
        pass

    def submitField2(self, slot, fields, **keywords):
        fromFieldValue = slot.getFieldOption(fields, 'from', default = '')
        toFieldValue = slot.getFieldOption(fields, 'to', default = '')
        error = 0
        errorFields = {}
        value = None
        if isButtonSelected(slot.getFieldOptionName('addButton'), fields):
            if not fromFieldValue or not toFieldValue:
                error = 1
                errorFields[fieldName + '_error'] = 'missingValue'
            elif not fromFieldValue in translation.languageKeys \
                 or not toFieldValue in translation.languageKeys \
                 or fromFieldValue == toFieldValue:
                error = 2
                slot.setFieldOption(errorFields, 'error', 'wrongValue')
            else:
                localizationKey = fromFieldValue + toFieldValue
                translatorsCountName = localizationKey + 'TranslatorsSet_count'
                if not fields.has_key(translatorsCountName) \
                   or fields[translatorsCountName] == '0':
                    fields[translatorsCountName] = '1'
                fields['again'] = '1'
                fields['hideErrors'] = '1'
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(TranslationToAdd)


class TranslatorsSets(KindMixin, proxyKinds.TranslatorsSets):
    pass
register(TranslatorsSets)


class Upload(ThingMixin, proxyKinds.Upload):
    pass
register(Upload)


class UploadFile(Data):
    defaultValue_kindName = 'UploadFile'

    def submitField2(self, slot, fields, **keywords):
        if slot.hasFieldOption(fields, 'field'):
            slot.delFieldOption(fields, 'field')
        if not slot.hasField(fields):
            return (0, {}, None)
        else:
            return KindMixin.submitField2(self, slot, fields, **keywords)
register(UploadFile)


class UserToken(TokenMixin, proxyKinds.UserToken):
    pass
register(UserToken)


class Widget(ThingMixin, proxyKinds.Widget):
    pass
register(Widget)


class WidgetName(ChoiceMixin, proxyKinds.WidgetName):
    titleFromValueGetterName = None
    titleFromValueGetterName_kind_isPublicForXmlRpc = 1
    titleFromValueGetterName_kind_isRequired = 1
    titleFromValueGetterName_kindName = 'PythonIdentifier'

    widgetSlotName = None
    widgetSlotName_kind_isPublicForXmlRpc = 1
    widgetSlotName_kind_isRequired = 1
    widgetSlotName_kindName = 'SlotName'

##     def __init__(self, **attributes):
##         proxyKinds.WidgetName.__init__(self, **attributes)
##         self.labels = {}
##         for widgetClass in self.getAllThingClasses()['widget'].values():
##             value = widgetClass.getThingName.im_func(widgetClass)
##             self.labels[value] = widgetClass.getName.im_func(widgetClass)

    def makeModelTitleFromValue(self, slot, fields, value):
        container = slot.getContainer()
        titleFromValueGetterName = self.titleFromValueGetterName
        if titleFromValueGetterName is None:
            return _(self.getLabels(slot, fields)[str(value)])
        titleFromValueGetter = getattr(container, titleFromValueGetterName)
        return titleFromValueGetter(fields, value, parentSlot = slot.parent)

    def submitField(self, slot, fields, **keywords):
        widgetSlotName = self.widgetSlotName
        if widgetSlotName is None:
            return ChoiceMixin.submitField(self, slot, fields, **keywords)
        assert fields is not None
        error, errorFields, value = self.submitField2(slot, fields, **keywords)
        if error:
            fields['again'] = '1'
            fields['error'] = str(error)
            fields.update(errorFields)
        else:
            if not slot.hasValue() or value != slot.getValue():
                slot.setValue(value)
            assert value
            widgetValue = self.getAllThingClasses()['widget'][value]()
            widgetValue.isPublicForXmlRpc = 1
            widgetSlot = slot.getContainer().getSlot(
                widgetSlotName, parentSlot = slot.parent)
            oldWidgetValue = widgetSlot.getValue()
            if oldWidgetValue is None \
                   or oldWidgetValue.__class__ != widgetValue.__class__:
                widgetSlot.setValue(widgetValue)
register(WidgetName)


class WritersSet(SequenceMixin, proxyKinds.WritersSet):
    itemKind_value_defaultValue = 'user'
register(WritersSet)


class XChoice(ChoiceMixin, proxyKinds.XChoice):
    otherFieldLabel = None
    otherFieldLabel_kind_isPublicForXmlRpc = 1
##     otherFieldLabel_kind_label = N_('Input Text Label')
    otherFieldLabel_kindName = 'String'

    titles_kind_isConstant = 1
    titles_kind_isHidden = 1

    def makeFieldFromValue(self, slot, fields, value, **keywords):
        if value is None:
            fieldValue = ''
            otherFieldValue = ''
        elif value in self.getValues(slot, fields):
            fieldValue = str(value)
            otherFieldValue = ''
        else:
            fieldValue = ''
            otherFieldValue = str(value)
        slot.setField(fields, fieldValue)
        slot.setFieldOption(fields, 'other', otherFieldValue)

    def makeModelTitleFromValue(self, slot, fields, value):
        valueAsString = str(value)
        if self.titles and self.titles.has_key(valueAsString):
            return _(self.titles[valueAsString])
        return valueAsString

    def submitField2(self, slot, fields, **keywords):
        fieldValue = slot.getField(fields, default = '')
        otherFieldValue = slot.getFieldOption(fields, 'other', default = '')
        error = 0
        errorFields = {}
        value = None
        if not fieldValue and not otherFieldValue:
            if self.isRequired:
                error = 1
                slot.setFieldOption(errorFields, 'error', 'missingValue')
            else:
                value = None
        elif otherFieldValue: 
            value = otherFieldValue
        elif fieldValue in self.getValues(slot, fields):
            value = fieldValue
        else:
            error = 2
            slot.setFieldOption(errorFields, 'error', 'wrongValue')
        if not error and self.isEmptyModelValue(slot, value):
            value = None
        return (error, errorFields, value)
register(XChoice)


