# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Web Properties"""

__version__ = '$Revision: 1.14 $'[11:-2]


import glasnost.common.xhtmlgenerator as X

import glasnost.proxy.properties as proxyProperties

import things
from tools import getUrlId


class PropertiesContainerMixin:
    def getProxyForId(self, id):
        from glasnost.web.tools import getWeb
        return getWeb(id)

    def initFields(self, fields, parentSlot = None):
        self.makeFieldsFromInstance(fields, parentSlot = parentSlot)


class Property(things.ThingMixin, proxyProperties.Property):
    kind_kind_isConstant = 1
    kind_kind_isHidden = 1

    kindName_kind_kindSlotName = 'kind'
    kindName_kind_titleFromValueGetterName = 'makeKindNameTitleFromValue'
    kindName_kind_widget_fieldLabel = N_('Type')
    kindName_kind_widgetName = 'Select'

    name_kind_widget_fieldLabel = N_('Name')
    #name_kind_widget_maxLength = 40
    name_kind_widget_size = 40
    name_kind_widgetName = 'InputText'

    def getOrderedFieldSlotNames(self, fields, parentSlot = None):
        slotNames = things.ThingMixin.getOrderedFieldSlotNames(
            self, fields, parentSlot = parentSlot)
        # The field kindName must be set before the field kind.
        slotNames += ['kindName', 'kind']
        return slotNames

    def makeKindNameTitleFromValue(self, fields, value, parentSlot = None):
        valueAsString = str(value)
        object = parentSlot.getObject()
        urlId = getUrlId(object.id)
        serverRole = object.getWeb().serverRole
        kindNameSlot = self.getSlot('kindName', parentSlot = parentSlot)
        return X.a(
            href = X.url('/%s/type/%s' % (urlId, self.name)))(
            _(kindNameSlot.getKind().getLabels(kindNameSlot, fields)[
                valueAsString]))

    def makeWidgetNameTitleFromValue(self, fields, value, parentSlot = None):
        valueAsString = str(value)
        object = parentSlot.getObject()
        urlId = getUrlId(object.id)
        serverRole = object.getWeb().serverRole
        kindSlot = self.getSlot('kind', parentSlot = parentSlot)
        widgetNameSlot = kindSlot.getValue().getSlot(
            'widgetName', parentSlot = kindSlot)
        return X.a(
            href = X.url('/%s/widget/%s' % (urlId, self.name)))(
            _(widgetNameSlot.getKind().getLabels(widgetNameSlot, fields)[
                valueAsString]))
things.register(Property)
