# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Web Things"""

__version__ = '$Revision: 1.25 $'[11:-2]


import glasnost.common.xhtmlgenerator as X

import glasnost.proxy.things as proxyThings
from tools import CssJunkie


allThingClasses = {}


def register(thingClass):
    thingName = thingClass.getThingName.im_func(thingClass)
    if not allThingClasses.has_key(thingClass.thingCategory):
        allThingClasses[thingClass.thingCategory] = {}
    if not allThingClasses[thingClass.thingCategory].has_key(thingName):
        # FIXME: this asserts on thingName == 'FunctionDeclaration' after
        # getWebForServerRole generalization (night from 16 to 17/2/2003)
        pass
    #assert not allThingClasses[thingClass.thingCategory].has_key(thingName)
    allThingClasses[thingClass.thingCategory][thingName] = thingClass


class ThingMixin:
    isEffective_kind_isConstant = 1
    isEffective_kind_isHidden = 1

    thingCategory_kind_isConstant = 1
    thingCategory_kind_isHidden = 1
    thingCategory_kind_widget_fieldLabel = N_('Category')

    thingName_kind_isConstant = 1
    thingName_kind_isHidden = 1
    thingName_kind_widget_fieldLabel = N_('Name')

    def getAllThingClasses(self):
        # Should be converted to a class method, when porting Glasnost to
        # Python 2.2.
        return allThingClasses

    def getCompactViewLayout(self, fields, parentSlot = None):
        fields['useCompactLayout'] = 1
        return self.getViewLayout(fields, parentSlot = parentSlot)

    def getEditLayout(self, fields, parentSlot = None):
        layout = X.array()
        slotNames = self.getEditLayoutSlotNames(
            fields, parentSlot = parentSlot)
        hiddenSlotNames = self.getEditLayoutHiddenSlotNames(
            fields, parentSlot = parentSlot)
        readOnlySlotNames = self.getEditLayoutReadOnlySlotNames(
            fields, parentSlot = parentSlot)
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            widget = kind.getModelWidget(slot)
            if slotName in hiddenSlotNames:
                if not kind.isConstant:
                    hidden = widget.getModelHiddenLayout(slot, fields)
                    if CssJunkie:
                        layout += hidden
                    else:
                        layout += X.tr(X.td(hidden))
            else:
                if kind.isConstant:
                    widget.setInForm(0)
                else:
                    widget.setInForm(1)
                    widget.setReadOnly(slotName in readOnlySlotNames)
                if CssJunkie:
                    layout += widget.getHtmlCssField(slot, fields)
                else:
                    layout += widget.getHtmlField(slot, fields)
        if CssJunkie:
            #layout += X.div(_class = 'clear')
            pass
        return layout

    def getEditLayoutHiddenSlotNames(self, fields, parentSlot = None):
        slotNames = self.getEditLayoutSlotNames(
            fields, parentSlot = parentSlot)
        hiddenSlotNames = []
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.isHidden:
                hiddenSlotNames.append(slotName)
        return hiddenSlotNames

    def getEditLayoutReadOnlySlotNames(self, fields, parentSlot = None):
        slotNames = self.getEditLayoutSlotNames(
            fields, parentSlot = parentSlot)
        readOnlySlotNames = []
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.isReadOnly:
                readOnlySlotNames.append(slotName)
        return readOnlySlotNames

    def getEditLayoutSlotNames(self, fields, parentSlot = None):
        return self.getLayoutSlotNames(fields, parentSlot = parentSlot)

    def getErrorLayout(self, error, keywords):
        layout = X.array()
        if error and not keywords.has_key('hideErrors'):
            div = X.div(_class = 'error-message')
            div += X.h1(_('Error!'))
            layout += div
            if keywords.has_key('versionError') and keywords['versionError']:
                div += X.p()(
                    _("""\
The informations have been changed while you were editing them.
Please backup your changes and redo the edition.\
"""))
        return layout

    def getFieldSlotNames(self, fields, parentSlot = None):
        allSlotNames = self.getSlotNames(parentSlot = parentSlot)
        orderedSlotNames = self.getOrderedFieldSlotNames(
            fields, parentSlot = parentSlot)
        slotNames = []
        for slotName in orderedSlotNames:
            if not slotName in allSlotNames:
                continue
            slotNames.append(slotName)
        for slotName in allSlotNames:
            if slotName in slotNames:
                continue
            slotNames.append(slotName)
        return slotNames

    def getHiddenLayout(self, fields, parentSlot = None):
        layout = X.array()
        slotNames = self.getHiddenLayoutSlotNames(
            fields, parentSlot = parentSlot)
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.isConstant:
                continue
            widget = kind.getModelWidget(slot)
            layout += widget.getModelHiddenLayout(slot, fields)
        return layout

    def getHiddenLayoutSlotNames(self, fields, parentSlot = None):
        return self.getSlotNames(parentSlot = parentSlot)

    def getLayoutSlotNames(self, fields, parentSlot = None):
        allSlotNames = self.getSlotNames(parentSlot = parentSlot)
        orderedSlotNames = self.getOrderedLayoutSlotNames(
            parentSlot = parentSlot)
        slotNames = []
        for slotName in orderedSlotNames:
            if not slotName in allSlotNames:
                continue
            slotNames.append(slotName)
        for slotName in allSlotNames:
            if slotName in slotNames:
                continue
            slotNames.append(slotName)
        return slotNames

    def getLineEditLayout(self, fields, parentSlot = None):
        return self.getEditLayout(fields, parentSlot = parentSlot)

    def getLineViewLayout(self, fields, parentSlot = None):
        return self.getCompactViewLayout(fields, parentSlot = parentSlot)

    def getOrderedFieldSlotNames(self, fields, parentSlot = None):
        return []

    def getViewLayout(self, fields, parentSlot = None):
        layout = X.array()
        slotNames = self.getViewLayoutSlotNames(
            fields, parentSlot = parentSlot)
        hiddenSlotNames = self.getViewLayoutHiddenSlotNames(
            fields, parentSlot = parentSlot)
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if slotName in hiddenSlotNames:
                continue
            widget = kind.getModelWidget(slot)
            widget.setInForm(0)
            if CssJunkie:
                layout += widget.getHtmlCssField(slot, fields)
            else:
                layout += widget.getHtmlField(slot, fields)
        if CssJunkie:
            #layout += X.div(_class = 'clear')
            pass
        return layout

    def getViewLayoutHiddenSlotNames(self, fields, parentSlot = None):
        slotNames = self.getViewLayoutSlotNames(
            fields, parentSlot = parentSlot)
        hiddenSlotNames = []
        for slotName in slotNames:
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.isHidden:
                hiddenSlotNames.append(slotName)
        return hiddenSlotNames

    def getViewLayoutSlotNames(self, fields, parentSlot = None):
        return self.getLayoutSlotNames(fields, parentSlot = parentSlot)

    def initFields(self, fields, parentSlot = None):
        for slotName in self.getFieldSlotNames(
            fields, parentSlot = parentSlot):
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.hasToInitField:
                kind.initField(slot, fields)

    def makeFieldsFromInstance(self, fields, parentSlot = None):
        for slotName in self.getFieldSlotNames(
            fields, parentSlot = parentSlot):
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.hasToMakeFieldFromValue:
                kind.makeFieldFromValue(
                    slot, fields, slot.getValue())

    def newWidget(self, parentSlot = None):
        return self.newThing('widget', 'Thing')

    def repairFields(self, fields, parentSlot = None):
        for slotName in self.getFieldSlotNames(
            fields, parentSlot = parentSlot):
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if kind.hasToRepairField:
                kind.repairField(slot, fields)

    def submitFields(self, fields, parentSlot = None):
        for slotName in self.getFieldSlotNames(
            fields, parentSlot = parentSlot):
            slot = self.getSlot(slotName, parentSlot = parentSlot)
            kind = slot.getKind()
            if not kind.isConstant and kind.hasToSubmitField:
                kind.submitField(slot, fields)


class BaseThing(ThingMixin, proxyThings.BaseThing):
    pass
