# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


from __future__ import nested_scopes
    # lambdas are so much fun with nested scopes...


__doc__ = """Glasnost Web Tools"""

__version__ = '$Revision: 1.80 $'[11:-2]


import base64
import Cookie
from cStringIO import StringIO
import os.path
import re
from stat import *
import sys
import time
import types
import urllib
import imp
import locale

# zLOG is used by TAL that considers Zope is always there.  How wrong.
sys.modules['zLOG'] = imp.new_module('zLOG')

import glasnost.common.context as context
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools
import glasnost.common.xhtmlgenerator as X

from glasnost.proxy.tools import *

CssJunkie = (getConfig('Misc', 'UseCSSEveryWhere', default='false') == 'true')

_cachedWebsByServerRole = {}
mainModule = sys.modules['__main__']
currentTemplates = {}

HTTP_CONTINUE                     = 100
HTTP_SWITCHING_PROTOCOLS          = 101
HTTP_PROCESSING                   = 102
HTTP_OK                           = 200
HTTP_CREATED                      = 201
HTTP_ACCEPTED                     = 202
HTTP_NON_AUTHORITATIVE            = 203
HTTP_NO_CONTENT                   = 204
HTTP_RESET_CONTENT                = 205
HTTP_PARTIAL_CONTENT              = 206
HTTP_MULTI_STATUS                 = 207
HTTP_MULTIPLE_CHOICES             = 300
HTTP_MOVED_PERMANENTLY            = 301
HTTP_MOVED_TEMPORARILY            = 302
HTTP_SEE_OTHER                    = 303
HTTP_NOT_MODIFIED                 = 304
HTTP_USE_PROXY                    = 305
HTTP_TEMPORARY_REDIRECT           = 307
HTTP_BAD_REQUEST                  = 400
HTTP_UNAUTHORIZED                 = 401
HTTP_PAYMENT_REQUIRED             = 402
HTTP_FORBIDDEN                    = 403
HTTP_NOT_FOUND                    = 404
HTTP_METHOD_NOT_ALLOWED           = 405
HTTP_NOT_ACCEPTABLE               = 406
HTTP_PROXY_AUTHENTICATION_REQUIRED= 407
HTTP_REQUEST_TIME_OUT             = 408
HTTP_CONFLICT                     = 409
HTTP_GONE                         = 410
HTTP_LENGTH_REQUIRED              = 411
HTTP_PRECONDITION_FAILED          = 412
HTTP_REQUEST_ENTITY_TOO_LARGE     = 413
HTTP_REQUEST_URI_TOO_LARGE        = 414
HTTP_UNSUPPORTED_MEDIA_TYPE       = 415
HTTP_RANGE_NOT_SATISFIABLE        = 416
HTTP_EXPECTATION_FAILED           = 417
HTTP_UNPROCESSABLE_ENTITY         = 422
HTTP_LOCKED                       = 423
HTTP_FAILED_DEPENDENCY            = 424
HTTP_INTERNAL_SERVER_ERROR        = 500
HTTP_NOT_IMPLEMENTED              = 501
HTTP_BAD_GATEWAY                  = 502
HTTP_SERVICE_UNAVAILABLE          = 503
HTTP_GATEWAY_TIME_OUT             = 504
HTTP_VERSION_NOT_SUPPORTED        = 505
HTTP_VARIANT_ALSO_VARIES          = 506
HTTP_INSUFFICIENT_STORAGE         = 507
HTTP_NOT_EXTENDED                 = 510

OK = 0


def accessForbidden():
    userToken = context.getVar('userToken', default = '')
    if userToken:
        layout = X.array()
        # TODO: nice text telling the user he should move away...
        return writePageLayout(layout, _('Access Forbidden!'), canCache = 0)

    loginUrl = X.url('/login').add(
        'nextUri', cleanUpUnparsedUri([], 'http')).add('access', 'forbidden')
    hostNameAndPort = commonTools.makeHttpHostNameAndPort(
        context.getVar('httpHostName'),
        context.getVar('httpPort'))
    if getConfig('Misc', 'UseHTTPS') == 'true':
        loginUrl = 'https://%(hostNameAndPort)s%(nameAndQuery)s' % {
                'hostNameAndPort': hostNameAndPort,
                'nameAndQuery': loginUrl,
                }
    return redirect(loginUrl)

def appendToUri(uri, item):
    if '?' in uri:
        separator = '&'
    else:
        separator = '?'
    return uri + separator + item


def cleanUpUnparsedUri(trashList, protocol = None):
    hostNameAndPort = commonTools.makeHttpHostNameAndPort(
        context.getVar('httpHostName'),
        context.getVar('httpPort'))
    pathAndQuery = commonTools.makeHttpPathAndQuery(
        context.getVar('httpPath'),
        context.getVar('httpQuery'))
    if pathAndQuery is None:
        pathAndQuery = ''
    if not hostNameAndPort:
        uri = pathAndQuery
    elif protocol == 'http':
        uri = 'http://%(hostNameAndPort)s%(pathAndQuery)s' % {
            'hostNameAndPort': hostNameAndPort,
            'pathAndQuery': pathAndQuery,
            }
    elif protocol == 'https':
        uri = 'https://%(hostNameAndPort)s%(pathAndQuery)s' % {
            'hostNameAndPort': hostNameAndPort,
            'pathAndQuery': pathAndQuery,
            }
    else:
        uri = pathAndQuery
    return cleanUpUri(uri, trashList)


def cleanUpUri(uri, trashList):
    for s in trashList:
        s = s + '='
        i = uri.find(s)
        if i >= 0:
            j = uri.find('&', i + len(s))
            if j < 0:
                s = uri[i:]
                if i > 0:
                    c = uri[i - 1]
                    if c in ['?', '&']:
                        uri = uri[:i - 1]
            else:
                uri = uri[:i] + uri[j + 1:]
    return uri


def confirmDelete(deleteUrl, objectName = None):
    backUrl = X.url(deleteUrl.path[:-7])
    if objectName is None:
        objectName = N_('entry')
    layout = X.array()
    layout += X.div(_class = 'alert')(
        _('Are you sure you want to delete the %s?') % _(objectName))
    layout += X.br()
    layout += X.buttonStandalone('delete', deleteUrl)
    layout += X.buttonStandalone('cancel', backUrl)
    return writePageLayout(layout, _('Confirm Deletion'), canCache = 0)


def failure(message, url):
    layout = X.array()
    layout += X.div(_class = 'alert')(message)
    layout += X.br()
    layout += X.buttonStandalone('ok', url)
    return writePageLayout(layout, _('Failure'), canCache = 0)


def getWeb(id):
    return getWebForServerRole(extractServerRole(id))


def getWebForServerRole(serverRole):
    # FIXME: We should clear _cachedWebsByServerRole when userToken value
    # changes. Because different users can use the same process, or a user can
    # login or logout.
    if not _cachedWebsByServerRole.has_key(serverRole):
        import tools_new as webTools
        baseWebs = webTools.getAllBaseWebs()
        for baseWeb in baseWebs.values():
            if serverRole == baseWeb.serverRole:
                web = baseWeb
                break
        else:
            # Two passes so we don't call filesObjectWeb if we don't need to.
            for baseWeb in baseWebs.values():
                handledRoles = baseWeb.getHandledRoles()
                if handledRoles is not None and serverRole in handledRoles:
                    web = baseWeb.duplicateForRole(serverRole)
                    break
            else:
                web = None
                try:
                    webName = '%s%sWeb' % (serverRole[0].upper(), serverRole[1:])
                    if sys.modules.has_key('glasnost.web.%s' % webName):
                        mod = sys.modules['glasnost.web.%s' % webName]
                    else:
                        try:
                            mod = imp.find_module(webName,
                                                glasnost.web.__path__)
                        except ImportError:
                            # probably caused by case mismatch between filename
                            # and serverRole
                            files = [x for x in os.listdir(glasnost.web.__path__[0]) \
                                     if fnmatch.fnmatch(x.lower(),
                                                '%s.py' % webName.lower())]
                            if len(files) != 1:
                                raise
                            webName = files[0][:-3]
                            mod = imp.find_module(webName,
                                                glasnost.web.__path__)
                            
                        mod = imp.load_module('glasnost.web.%s' % webName,
                                            mod[0], mod[1], mod[2])
                    web = mod.__dict__[webName]()
                except ImportError:
                    pass
                except AttributeError:
                    pass
                if not web:
                    web = getProxyForServerRole(serverRole)
        _cachedWebsByServerRole[serverRole] = web
    return _cachedWebsByServerRole[serverRole]


def isButtonSelected(buttonName, keywords):
    selected = 0
    if keywords.has_key('button') and keywords['button'] == buttonName \
       or keywords.has_key(buttonName) and keywords[buttonName] \
       or keywords.has_key(buttonName + '.x') and keywords[buttonName + '.x']:
        selected = 1
    if keywords.has_key('button') and keywords['button'] == buttonName:
        del keywords['button']
    if keywords.has_key(buttonName):
        del keywords[buttonName]
    if keywords.has_key(buttonName + '.x'):
        del keywords[buttonName + '.x']
    if keywords.has_key(buttonName + '.y'):
        del keywords[buttonName + '.y']
    return selected


def isTypeOfMimeType(mimeType, type):
    if mimeType is None or len(mimeType) < len(type):
        return 0
    return mimeType.startswith(type)


def makeErrorMessage(error):
    if error == 'can\'tChangeValue':
        message = N_('Can\'t change value!')
    elif error == 'duplicateValue':
        message = N_('Duplicate value!')
    elif error == 'fileAccessDenied':
        message = N_('Access to file is denied!')
    elif error == 'missingValue':
        message = N_('Missing value!')
    elif error == 'nonExistentFilePath':
        message = N_('Specified file path does not exist!')
    elif error == 'notAFile':
        message = N_('Specified path is not a file!')
    elif error == 'outOfRangeValue':
        message = N_('Value out of range!')
    elif error == 'tooBigValue':
        message = N_('Value too big!')
    elif error == 'tooSmallValue':
        message = N_('Value too small!')
    elif error == 'valueAlreadyUsed':
        message = N_('Value already used!')
    elif error == 'wrongValue':
        message = N_('Wrong value!')
    else:
        message = _('Error!')
    return '<span class="error-message">%s</span><br />' % _(message)


def pageNotFound():
    from mod_python import apache
    raise apache.SERVER_RETURN, HTTP_NOT_FOUND


def redirect(url):
    if type(url) == types.InstanceType and hasattr(url, 'getAsUrl'):
        url = url.getAsUrl()
    else:
        url = str(url)
    req = context.getVar('req')
    req.err_headers_out['location'] = url
    setHttpCookie()
    req.status = HTTP_MOVED_TEMPORARILY
    req.send_http_header()
    return OK


def redirectPermanently(url):
    if type(url) == types.InstanceType and hasattr(url, 'getAsUrl'):
        url = url.getAsUrl()
    else:
        url = str(url)
    req = context.getVar('req')
    req.err_headers_out['location'] = url
    req.status = HTTP_MOVED_PERMANENTLY
    setHttpCookie()
    req.send_http_header()
    return OK


def rememberObject(id):
    preferencesWeb = getWebForServerRole('preferences')
    preferencesWeb.rememberId(id)


def repairMimeType(fieldValue, fileName):
    # FIXME: Integrate this function in uploadFiles object
    # to correct mime type sent by the web browser.
    def getFileExt(fileName):
        return fileName[string.rfind(fileName, '.') + 1:]

    mimeTypesByExt = {
        'avi' : 'video/x-msvideo',
        'bz2' : 'application/bzip2',
        'doc' : 'application/msword',
        'mpeg' : 'video/mpeg ',
        'mpg' : 'video/mpeg ',
        'mov' : 'video/quicktime',
        'pdf' : 'application/pdf',
        'ppt' : 'application/vnd.ms-powerpoint',
        'ps' : 'application/postscript',
        'qt' : 'video/quicktime',
        'tar' : 'application/x-tar',
        'tex' : 'application/tex',
        'xls' : 'application/vnd.ms-excel',
        'xml' : 'text/xml ',
        'xsl' : 'text/xml',
        'Z' : 'application/zoo',
        }

    if len(fieldValue) < 12:
        return
    if fieldValue[12:] == 'octet-stream':
        ext = getFileExt()
        raise str(ext) + ' - ' + fieldValue + ' - ' + fileName
        if mimeTypesByExt.has_key(ext):
            fieldValue = mimeTypesByExt[getFileExt()]
            if fieldValue is not None:
                return fielValue
        return fieldValue


def replaceSpecialTags(text):
    text = text.replace('[{glasnost:defaultdispatchernameandport}]',
                        getDefaultDispatcherHostNameAndPort())

    text = text.replace(
        '[{glasnost:localuri}]/',
        context.getVar('httpScriptDirectoryPath'))
    text = text.replace(
        '[{glasnost:localuri}]',
        context.getVar('httpScriptDirectoryPath'))

    def aliasLabelReplacer(matchObject):
        alias = matchObject.group('alias')
        name = ''
        if 'pagenames' in context.getVar('knownRoles'):
            id = getProxyForServerRole('pagenames').getIdByName(alias)
            if id:
                try:
                    name = getObjectLabelTranslated(
                        id,
                        context.getVar('readLanguages'))
                except:
                    pass
        name = name.replace('&', '&amp;')
        name = name.replace('<', '&lt;')
        name = name.replace('"', '&quot;')
        return name

    text = re.sub(
        '\[\{glasnost\:aliaslabel\:(?P<alias>.*?)\}\]',
        aliasLabelReplacer, text)

    def aliasReplacer(matchObject):
        alias = matchObject.group('alias')
        url = '/%s' % alias
        return X.url(alias).getAsUrl()

    text = re.sub(
        '\[\{glasnost\:alias\:(?P<alias>.*?)\}\]',
        aliasReplacer, text)

    def imageReplacer(matchObject):
        serverRole = matchObject.group('serverRole')
        localId = matchObject.group('localId')
        objectId = 'glasnost://%s/%s/%s' % (
                        getDefaultDispatcherHostNameAndPort(),
                        serverRole, localId)
        web = getWebForServerRole(serverRole)
        objectUrlId = getUrlId(objectId)
        fileName = ''
        try:
            object = web.getPartialObject(objectId, ['dataFileName'])
            if hasattr(object, 'dataFileName') and object.dataFileName:
                fileName = '/%s' % object.dataFileName
        except faults.Fault:
            pass
        return X.url(
            '/%s/image%s' % (objectUrlId, fileName)
            ).getAsUrl()

    text = re.sub(
        '\[\{glasnost\:image\:(?P<serverRole>.*?)\:(?P<localId>.*?)\}\]',
        imageReplacer, text)

    def labelReplacer(matchObject):
        serverRole = matchObject.group('serverRole')
        localId = matchObject.group('localId')
        try:
            name = getObjectLabelTranslated(
                'glasnost://%s/%s/%s' % (
                getDefaultDispatcherHostNameAndPort(), serverRole, localId),
                context.getVar('readLanguages'))
        except faults.MissingItem:
            name = ''
        name = name.replace('&', '&amp;')
        name = name.replace('<', '&lt;')
        name = name.replace('"', '&quot;')
        return name

    text = re.sub(
        '\[\{glasnost\:label\:(?P<serverRole>.*?)\:(?P<localId>.*?)\}\]',
        labelReplacer, text)

    def partialIdReplacer(matchObject):
        serverRole = matchObject.group('serverRole')
        localId = matchObject.group('localId')
        computedId =  'glasnost://%s/%s/%s' % (
            getDefaultDispatcherHostNameAndPort(), serverRole, localId)
        if 'pagenames' in context.getVar('knownRoles'):
            newComputedId = '/%s' % getProxyForServerRole('pagenames').getNameByMappedId(
                computedId)
            if newComputedId[1:]:
                computedId = newComputedId
        return X.url(computedId).getAsUrl()

    text = re.sub(
        '\[\{glasnost\:partialid\:(?P<serverRole>.*?)\:(?P<localId>.*?)\}\]',
        partialIdReplacer, text)

    def thumbnailReplacer(matchObject):
        serverRole = matchObject.group('serverRole')
        localId = matchObject.group('localId')
        width = matchObject.group('width')
        height = matchObject.group('height')
        web = getWebForServerRole(serverRole)
        return X.url('/%s/thumbnail' % web.serverRole).add(
            'id',
            'glasnost://%s/%s/%s' % (
            getDefaultDispatcherHostNameAndPort(), serverRole, localId)).add(
            'width', width).add('height', height).getAsUrl()
    text = re.sub(
        '\[\{glasnost\:thumbnail\:(?P<serverRole>.*?)\:(?P<localId>.*?)\:width=(?P<width>.*?)\:height=(?P<height>.*?)\}\]',
        thumbnailReplacer, text)

    canUseCookie = context.getVar('canUseCookie', default = 0)

    tag = urllib.quote('[{glasnost:sessiontoken}]')
    sessionToken = context.getVar('sessionToken')
    if not canUseCookie and sessionToken is not None:
        text = text.replace(tag, sessionToken)
    else:
        text = text.replace('?sessionToken=' + tag + '&', '?')
        text = text.replace('?sessionToken=' + tag, '')
        text = text.replace('&sessionToken=' + tag, '')

    return text


def setHttpCookie():
    if not context.getVar('canUseCookie', default = 0):
        return
    req = context.getVar('req')
    oldCookie = context.getVar('cookie')
    cookie = Cookie.SmartCookie()
    cookieContent = {}
    sessionToken = context.getVar('sessionToken')
    if sessionToken is not None:
        cookieContent['sessionToken'] = sessionToken
    cookie['glasnost'] = cookieContent
    cookie['glasnost']['path'] = '/'
    if not cookieContent:
        # Remove the cookie on the user browser.
        cookie['glasnost']['max-age'] = 0
    cookieOutput = cookie.output(header = '')[1:]
    expurgedCookieOutput = cookieOutput.replace(' Path=/;', '')
    if oldCookie is None and cookieContent \
           or oldCookie is not None \
           and expurgedCookieOutput != oldCookie.output(header = '')[1:]:
        req.headers_out['Set-Cookie'] = cookieOutput


def success(message, url):
    layout = X.array()
    layout += X.div(_class = 'alert')(message)
    layout += X.br()
    layout += X.buttonStandalone('ok', url)
    return writePageLayout(layout, _('Success'), canCache = 0)


def getUrlId(id):
    if not id:
        return ''
    # Convert the id so that it can be used in url such as
    # '/articles/%s/edit' % id
    if 'pagenames' in context.getVar('knownRoles'):
        newId = getProxyForServerRole('pagenames').getNameByMappedId(id)
        if newId:
            return newId

    serverHostNameAndPort, serverRole, localId = splitId(id)
    # FIXME: it currently doesn't handle remote ids. (is this true ?)
    if serverHostNameAndPort == '[{glasnost:defaultdispatchernameandport}]' \
       or serverHostNameAndPort == getDefaultDispatcherHostNameAndPort():
        return '%s/%s' % (serverRole, localId)
    else:
        return '%s/glasnost/%s/%s' % (serverRole, serverHostNameAndPort, localId)


from UserDict import UserDict
import WebAPI

class TemplateDict(UserDict):
    pass

def getTemplateVars():
    userId = context.getVar('userId', default = '')
    if userId:
        userSet = [userId]
    else:
        userSet = None

    loginUrl = X.url('/login').add(
        'nextUri', cleanUpUnparsedUri([], 'http'))
    hostNameAndPort = commonTools.makeHttpHostNameAndPort(
        context.getVar('httpHostName'),
        context.getVar('httpPort'))
    if getConfig('Misc', 'UseHTTPS') == 'true':
        loginUrl = 'https://%(hostNameAndPort)s%(nameAndQuery)s' % {
                'hostNameAndPort': hostNameAndPort,
                'nameAndQuery': loginUrl,
                }

    logoutUrl = X.url('/login/logout').add(
        'nextUri', cleanUpUnparsedUri([], 'http'))
    if getConfig('Misc', 'UseHTTPS') == 'true':
        logoutUrl = 'http://%(hostNameAndPort)s%(nameAndQuery)s' % {
                'hostNameAndPort': hostNameAndPort,
                'nameAndQuery': logoutUrl,
                }

    httpScriptDirectoryPath = context.getVar(
        'httpScriptDirectoryPath')

    imagesUrl = os.path.join(httpScriptDirectoryPath, 'images')
    imageUrl = os.path.join(httpScriptDirectoryPath, 'images')
            
    aboutButton = X.buttonStandalone('about', X.url('/about')).getAsXml()
    aboutUrl = X.url('/about').getAsUrl()
    
    logoutButton = X.buttonStandalone('logout', logoutUrl).getAsXml()
    
    prefsButton = X.buttonStandalone('prefs', X.url('/preferences')).getAsXml()
    prefsUrl = X.url('/preferences').getAsUrl()
    
    loginButton = X.buttonStandalone('login', loginUrl).getAsXml()
    
    newAccountButton = X.buttonStandalone('new-account', X.url('/people/edit')).getAsXml()
    newAccountUrl = X.url('/people/edit').getAsUrl()

    shortNewsLabel = _('Short News')
    
    indexUrl = X.url('/')
    
    session = context.getVar('session')
    if session and session.has_key('userName'):
        userName = session['userName']
    else:
        userName = ''

    return locals()


def writePageLayout(layout, title, canCache = 1, showTitle = 1):
    dict = getTemplateVars()
    dict['X'] = X
    dict['title'] = title
    dict['showTitle'] = showTitle
    
    req = context.getVar('req')
    req.content_type = 'text/html'

    setHttpCookie()
    req.send_http_header()

    if req.method == 'HEAD':
        return OK

    if req.caching:
        req.openCachePage()

    # we need to chdir for included files
    #os.chdir(context.getVar('webDirectoryPath'))
    os.chdir( context.getVar('templatesDirectoryPath') )
    
    dict['body'] = layout.getAsXml()

    fileName = context.getVar('templateFileName')
    if not currentTemplates.has_key(fileName):
        # caching this in currentTemplates improves rendering time by about
        # 1.2 seconds on my (fpeters) computer.  Not bad
        ### (benchmark on 2002-11-16)
        templateDirectoryName = context.getVar('templateDirectoryName')
        path = context.getVar('templatesDirectoryPath')
        fallbackPath = context.getVar('fallbackTemplatesDirectoryPath')

        try:
            file = open(os.path.join(path, templateDirectoryName, fileName))
        except IOError:
            file = open(os.path.join(fallbackPath, 'default', fileName))
            # should raise 404 if it fails
        file = file.read()

        from TAL.HTMLTALParser import HTMLTALParser
        from glasnost.web.GlasnostTALGenerator import GlasnostTALGenerator
        t = HTMLTALParser(gen = GlasnostTALGenerator(xml=0))
        t.parseString(file)

        currentTemplates[fileName] = t

    t = currentTemplates[fileName]
    program, macros = t.getCode()
    # this takes roughly 0.01 seconds, not worth caching
    
    from TAL.TALInterpreter import TALInterpreter
    from GlasnostTALEngine import GlasnostTALEngine
    engine = GlasnostTALEngine(macros)
    context.setVar('talEngine', engine)
    engine.locals = WebAPI.getAPIDict()
    engine.locals['currentObject'] = context.getVar('currentObject')

    for k, v in dict.items():
        engine.locals[k] = v

    interp = TALInterpreter(program, macros, engine, stream=req)
    interp()

    if req.caching:
        req.closeCachePage()
    return OK


def getAppropriateTalFile(action, idValue, serverRole = None):
    if idValue:
        id = splitId(idValue)
        serverRole = id[1]
        fileNames = (
            '%s-%s.%s.%s.tal' % (serverRole, id[0], id[2], action),
                # articles-glasnost.theridion.lan.211.print.tal
            '%s-%s.%s.tal' % (serverRole, id[0], action),
                # articles-glasnost.theridion.lan.print.tal
            '%s-%s.%s.tal' % (serverRole, id[2], action),
                # articles-211.print.tal
            '%s.%s.tal' % (serverRole, action),
                # articles.print.tal
            '%s-%s.%s.%s.xtal' % (serverRole, id[0], id[2], action),
                # articles-glasnost.theridion.lan.211.print.xtal
            '%s-%s.%s.xtal' % (serverRole, id[0], action),
                # articles-glasnost.theridion.lan.print.xtal
            '%s-%s.%s.xtal' % (serverRole, id[2], action),
                # articles-211.print.xtal
            '%s.%s.xtal' % (serverRole, action),
                # articles.print.xtal
            )
    else:
        id = ('',)
        fileNames = (
            '%s-%s.all.%s.tal' % (serverRole, id[0], action),
                # articles-glasnost.theridion.lan.211.print.tal
            '%s-all.%s.tal' % (serverRole, action),
                # articles-all.print.tal
            '%s-%s.all.%s.xtal' % (serverRole, id[0], action),
                # articles-glasnost.theridion.lan.211.print.xtal
            '%s-all.%s.xtal' % (serverRole, action),
                # articles-all.print.xtal
            )

    directories = [
        context.getVar('webDirectoryPath'),
        context.getVar('templatesDirectoryPath'),
        getConfig('Misc', 'AlternRootPath'),
        ]
    for directory in directories:
        if not directory:
            continue
        for fileName in fileNames:
            fileName = directory + '/' + fileName
            if os.path.exists(fileName):
                return fileName

    return None

def getAppropriatePyFile(action, idValue, serverRole = None):
    if idValue:
        id = splitId(idValue)
        serverRole = id[1]
        fileNames = (
            '%s-%s.%s.%s.py' % (serverRole, id[0], id[2], action),
                # articles-glasnost.theridion.lan.211.print.py
            '%s-%s.%s.py' % (serverRole, id[0], action),
                # articles-glasnost.theridion.lan.print.py
            '%s-%s.%s.py' % (serverRole, id[2], action),
                # articles-211.print.py
            '%s.%s.py' % (serverRole, action),
                # articles.print.py
            )
    else:
        id = ('',)
        fileNames = (
            '%s-%s.all.%s.py' % (serverRole, id[0], action),
                # articles-glasnost.theridion.lan.211.print.py
            '%s-all.%s.py' % (serverRole, action),
                # articles-all.print.py
            )

    directories = [
        getConfig(
            context.getVar('configSection'), 'ScriptDirectoryPath',
                default = context.getVar('templatesDirectoryPath')),
            context.getVar('templatesDirectoryPath'),
            getConfig('Misc', 'AlternRootPath'),
        ]
    for directory in directories:
        if not directory:
            continue
        for fileName in fileNames:
            fileName = directory + '/' + fileName
            if os.path.exists(fileName):
                return fileName

    return None


def getAppropriateFunction(action, serverRole, idValue):
    if not action:
        action = 'default'

    web = getWebForServerRole(serverRole)
    talFileName = getAppropriateTalFile(action, idValue, serverRole)

    if talFileName:
        context.setVar('TALFile', open(talFileName).read())
        context.setVar(
            'TALFileIsXTAL', talFileName.endswith('.xtal'))
        if idValue:
            return web.viewTal
        else:
            return web.viewAllTal

    pyFileName = getAppropriatePyFile(action, idValue, serverRole)

    if pyFileName:
        context.setVar('PyFile', pyFileName)
        if idValue:
            return web.viewPy
        else:
            return web.viewAllPy
    
    if action == 'default':
        if idValue:
            action = 'view'
        else:
            action = 'viewAll'

    try:
        if hasattr(web, action):
            function = getattr(web, action)
            if hasattr(function, 'isPublicForWeb') \
               and function.isPublicForWeb:
                return function
    except:
        pass
    return None
 
 
def processTALFile(fileName, file = None, xtal = None, **keywords):
    req = context.getVar('req')
    if xtal:
        req.content_type = 'text/xml'
    else:
        req.content_type = 'text/html'
    setHttpCookie()
    req.send_http_header()
    if req.method == 'HEAD':
        return OK

    if req.caching:
        req.openCachePage()

    os.chdir(context.getVar('webDirectoryPath'))
    if not file:
        file = open(fileName).read()
    #os.chdir(context.getVar('templatesDirectoryPath'))
    
    from TAL.TALParser import TALParser
    from TAL.HTMLTALParser import HTMLTALParser
    from glasnost.web.GlasnostTALGenerator import GlasnostTALGenerator
    if xtal:
        t = TALParser(gen = GlasnostTALGenerator(xml=1))
    else:
        t = HTMLTALParser(gen = GlasnostTALGenerator(xml=0))
    t.parseString(file)
    program, macros = t.getCode()

    from TAL.TALInterpreter import TALInterpreter
    from GlasnostTALEngine import GlasnostTALEngine
    engine = GlasnostTALEngine(macros)
    context.setVar('talEngine', engine)
    engine.locals = WebAPI.getAPIDict()
    engine.locals['currentObject'] = None
    for k, v in keywords.items():
        engine.locals[k] = v
    dict = getTemplateVars()
    for k, v in dict.items():
        engine.locals[k] = v

    t1 = context.getVar('t1')
    interp = TALInterpreter(program, macros, engine, stream=req)
    open('/tmp/webbench', 'a+').write('   %f  (before interp())\n' % (time.time()-t1))
    interp()
    open('/tmp/webbench', 'a+').write('   %f  (after interp())\n' % (time.time()-t1))

    if req.caching:
        req.closeCachePage()
    return OK


def processXTALFile(fileName, file = None, **keywords):
    return processTALFile(fileName, file = file, xtal = 1, **keywords)
