# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Web Views"""

__version__ = '$Revision: 1.9 $'[11:-2]


import glasnost.common.xhtmlgenerator as X

import glasnost.proxy.views as proxyViews

import things
import widgets


class View(widgets.ThingMixin, proxyViews.View):
    apply_kind_isConstant = 1
    apply_kind_isHidden = 1

    colSpan_kind_isConstant = 1
    colSpan_kind_isHidden = 1

    fieldLabel_kind_isConstant = 1
    fieldLabel_kind_isHidden = 1

    helpAlias_kind_isConstant = 1
    helpAlias_kind_isHidden = 1

    fields_kind_widget_fieldLabel = N_('Fields')

    name_kind_widget_fieldLabel = N_('Name')

    template_kind_widget_fieldLabel = N_('Template')

    def getModelPageBodyLayout(self, slot, fields):
        if self.isInForm():
            return widgets.ThingMixin.getModelPageBodyLayout(
                self, slot, fields)
        value = None
        if slot.hasValue():
            value = slot.getValue()
        kind = slot.getKind()
        kindValueThingName = kind.getValueThingNameFromFields(slot, fields)
        if value is None:
            value = self.newThing(kind.valueThingCategory, kindValueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != kind.valueThingCategory \
             or value.getThingName() != kindValueThingName:
            value = self.newThing(kind.valueThingCategory, kindValueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        if self.fields is None:
            layout = None
        else:
            layout = X.table(_class = 'page-body')
            hiddenSlotNames = value.getViewLayoutHiddenSlotNames(
                fields, parentSlot = parentSlot)
            for field in self.fields:
                slotName = field.slotName
                slot = value.getSlot(slotName, parentSlot = parentSlot)
                if slotName in hiddenSlotNames:
                    continue
                widget = field.widget
                widget.setInForm(0)
                layout += widget.getHtmlField(slot, fields)
        return layout
things.register(View)


class ViewField(things.ThingMixin, proxyViews.ViewField):
    slotName_kind_widget_apply = 1
    slotName_kind_widget_fieldLabel = N_('Name')

    widget_kind_widget_fieldLabel = N_('Graphical Component')
##     widget_kind_isConstant = 1
##     widget_kind_isHidden = 1

    widgetName_kind_widget_apply = 1
    widgetName_kind_widget_fieldLabel = N_('Graphical Component Name')
    widgetName_kind_widgetSlotName = 'widget'

    def getOrderedFieldSlotNames(self, fields, parentSlot = None):
        slotNames = things.ThingMixin.getOrderedFieldSlotNames(
            self, fields, parentSlot = parentSlot)
        # The field widgetName must be set before the field widget.
        slotNames += ['widgetName', 'widget']
        return slotNames
things.register(ViewField)
