# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Web Widgets"""

__version__ = '$Revision: 1.102 $'[11:-2]


import time

import glasnost.common.faults as faults
import glasnost.common.context as context
import glasnost.common.translation as translation
import glasnost.common.xhtmlgenerator as X

import glasnost.proxy.widgets as proxyWidgets

import properties
import things
from tools import *


register = things.register


class EmptyWidgetMixin(things.ThingMixin):
    pass
##     def buildOptions(self, options):
##         self.inherited(EmptyWidgetMixin, 'buildOptions')(self, options)


class WidgetMixin(EmptyWidgetMixin):
    inForm_kind_isConstant = 1
    inForm_kind_isHidden = 1

    readOnly_kind_isConstant = 1
    readOnly_kind_isHidden = 1

    def getHtmlColumnLabel(self, slot):
        label = self.getModelLabel(slot)
        return X.th(scope = 'col')(_(label))

    def getHtmlColumnValue(self, slot, fields, **keywords):
        tdAttributes = {}
        return X.td(**tdAttributes)(
            self.getHtmlValue(slot, fields, **keywords))

##     def getHtmlField(self, slot, fields, **keywords):
##         assert fields is not None
##         useCssEverywhere = context.getVar(
##             'useCssEverywhere', default = 0)
##         if useCssEverywhere:
##             layout = X.div(_class = 'row')
##         else:
##             layout = X.tr()
##         layout += self.getHtmlFieldLabel(slot)
##         fieldValueContainer = layout
##         if self.colSpan:
##             if useCssEverywhere:
##                 layout += X.div(_class = 'clear')
##             else:
##                 layout = X.array(layout)
##                 fieldValueContainer = X.tr(_class = 'colspan')
##                 layout += fieldValueContainer
##         fieldValueContainer = self.getHtmlFieldValue(slot, fields, **keywords)
##         return layout

    def getHtmlField(self, slot, fields, **keywords):
        assert fields is not None
        layout = tr = X.tr(_class = 'row')
        tr += self.getHtmlFieldLabel(slot)
        if self.colSpan:
            layout = X.array(tr)
            tr = X.tr(_class = 'colspan')
            layout += tr
        balloonHelp = slot.getKind().balloonHelp
        if balloonHelp and self.isInForm():
            tr += X.td(_class = 'field-description')(_(balloonHelp))
        else:
            tr += X.td()()
        td = self.getHtmlFieldValue(
            slot, fields, **keywords)
        if slot.getFieldOption(fields, 'error') \
               and not fields.has_key('hideErrors'):
            td.setAttribute('class', 'field-value error')
        tr += td
        return layout

    def getHtmlCssField(self, slot, fields, **keywords):
        assert fields is not None
        layout = X.div(_class = 'row')
        layout += self.getHtmlCssFieldLabel(slot)
        balloonHelp = slot.getKind().balloonHelp
        if balloonHelp and self.isInForm():
            layout += X.span(_class = 'field-description')(_(balloonHelp))
        #if self.colSpan:
        #    layout += X.div(_class = 'clear')
        cell = self.getHtmlCssFieldValue(slot, fields, **keywords)
        if slot.getFieldOption(fields, 'error') \
               and not fields.has_key('hideErrors'):
            cell.setAttribute('class', ' '.join(
                [cell.getAttribute('class'), 'error']))
        layout += cell
        return layout

##     def getHtmlFieldLabel(self, slot):
##         label = self.getModelLabel(slot)
##         labelAttributes = {'for': slot.getFieldName()}
##         if context.getVar('useCssEverywhere', default = 0):
##             if 0 and self.isInForm():
##                 # Mozilla 1.0 doesn't display labels in this case.
##                 # Mozilla 1.1 works.
##                 return X.label(**labelAttributes)(
##                     _(label), X.asIs(_(':') + ' '))
##             else:
##                 return X.span(_class = 'label')(_(label), X.asIs(_(':') + ' '))
##         else:
##             return X.td(_class = 'field-label')(
##                 X.label(**labelAttributes)(_(label), X.asIs(_(':') + ' ')))

    def getHtmlFieldLabel(self, slot):
        label = self.getModelLabel(slot)
        fieldName = slot.getFieldName()
        labelAttributes = {'for': fieldName}
        return X.td(_class = 'field-label')(
            X.label(**labelAttributes)(_(label), X.asIs(_(':') + ' ')))

    def getHtmlCssFieldLabel(self, slot):
        label = self.getModelLabel(slot)
        labelAttributes = {'for': slot.getFieldName()}
        if 0 and self.isInForm():
            if slot.getKind().isRequired and self.isInForm() \
                   and not self.isReadOnly(slot):
                labelAttributes['class'] = 'required'
            return X.label(**labelAttributes)(_(label), X.asIs(_(':') + ' '))
        else:
            t = ''
            if slot.getKind().isRequired and self.isInForm() \
                   and not self.isReadOnly(slot):
                t = ' required'
            return X.span(_class = 'label%s'%t)(_(label), X.asIs(_(':') + ' '))

    def getHtmlFieldValue(self, slot, fields, **keywords):
        tdAttributes = {}
        if self.colSpan:
            tdAttributes['colspan'] = self.colSpan
        return X.td(_class = 'field-value', **tdAttributes)(
            self.getHtmlValue(slot, fields, **keywords))

    def getHtmlCssFieldValue(self, slot, fields, **keywords):
        # FIXME: Big mess to rewrite ASAP.
        v = self.getHtmlValue(slot, fields, **keywords)
        className = 'cell'
        #if self.colSpan:
        #    className = None
        if isinstance(v, X.array) and v.children:
            realC = [x for x in v.children if x]
            # FIXME (properly) realC should always be an instance
            if hasattr(realC, '__class__') and \
                   realC[0].__class__ in (
                       X.input, X.select, X.asIs, X.span, X.button):
                return X.span(_class = className)(v)
            elif hasattr(realC, '__class__') and \
                   realC[0].__class__ is X.fieldset:
                realC[0].setAttribute('class', 'cell')
                return v
            else:
                return X.div(_class = className)(v)

        if isinstance(v, X.asIs):
            if className:
                return X.span(_class = className)(v)
            else:
                return v
        if className:
            return X.div(_class = className)(v)
        else:
            return X.div()(v)

    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        if not self.isInForm():
            layout = X.asIs(replaceSpecialTags(
                makeHtmlFromSpip(fieldValue)))
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                X.asIs(replaceSpecialTags(makeHtmlFromSpip(fieldValue))),
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            inputAttributes = {}
            layout += X.input(name = fieldName, type = 'text',
                              value = fieldValue, **inputAttributes)
        return layout

    def getModelErrorLayout(self, slot, fields):
        errorCode = slot.getFieldOption(fields, 'error')
        if errorCode and not fields.has_key('hideErrors'):
            return X.asIs(makeErrorMessage(errorCode))
        return None

    def getModelHelpLayout(self, slot, fields):
        helpAlias = self.getHelpAlias(slot, fields)
        if helpAlias and 'pagenames' in context.getVar('knownRoles') and \
                getWebForServerRole('pagenames').getIdByName(helpAlias):
            return X.a(_class = 'online-help',
                       href = X.url('/%s' % helpAlias))(_('help'))
        return None

    def getModelHiddenLayout(self, slot, fields):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        return X.input(name = fieldName, type = 'hidden', value = fieldValue)


class ThingMixin(WidgetMixin):
    def getHtmlValue(self, slot, fields, **keywords):
        value = None
        if slot.hasValue():
            value = slot.getValue()
        kind = slot.getKind()
        kindValueThingName = kind.getValueThingNameFromFields(slot, fields)
        if value is None:
            value = self.newThing(kind.valueThingCategory, kindValueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != kind.valueThingCategory \
             or value.getThingName() != kindValueThingName:
            value = self.newThing(kind.valueThingCategory, kindValueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        if not self.isInForm():
            layout = X.table(_class = 'field-value')(value.getLineViewLayout(
                fields, parentSlot = parentSlot))
        elif self.isReadOnly(slot):
            layout = X.array(
                value.getHiddenLayout(fields, parentSlot = parentSlot),
                X.table(_class = 'field-value')(value.getLineViewLayout(
                    fields, parentSlot = parentSlot)),
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            layout += X.table(_class = 'field-value')(value.getLineEditLayout(
                fields, parentSlot = parentSlot))
        return layout

    def getModelPageBodyLayout(self, slot, fields):
        value = None
        if slot.hasValue():
            value = slot.getValue()
        kind = slot.getKind()
        kindValueThingName = kind.getValueThingNameFromFields(slot, fields)
        if value is None:
            value = self.newThing(kind.valueThingCategory, kindValueThingName)
            parentSlot = slot.detach(value)
        elif value.thingCategory != kind.valueThingCategory \
             or value.getThingName() != kindValueThingName:
            value = self.newThing(kind.valueThingCategory, kindValueThingName)
            # FIXME: Do we need to copy the __dict__ to the new value? 
            parentSlot = slot.detach(value)
        else:
            parentSlot = slot
        if not self.isInForm():
            layout = value.getViewLayout(fields, parentSlot = parentSlot)
        else:
            layout = value.getEditLayout(fields, parentSlot = parentSlot)
##         if not context.getVar('useCssEverywhere', default = 0):
        if not CssJunkie:
            layout = X.table(_class = 'page-body')(layout)
        return layout


class BaseWidget(WidgetMixin, proxyWidgets.BaseWidget):
    pass
register(BaseWidget)


class Amount(WidgetMixin, proxyWidgets.Amount):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        # headers
        mainSize = 7
        centsSize = 2
        centsSep = '.'
        # FIXME we have to localize that
        mainFieldName = slot.getFieldOptionName('main')
        centsFieldName = slot.getFieldOptionName('cents')
        sideFieldName = slot.getFieldOptionName('side')
        mainFieldValue = slot.getFieldOption(fields, 'main', default = '0')
        centsFieldValue = slot.getFieldOption(fields, 'cents', default = '00')
        sideFieldValue = slot.getFieldOption(
            fields, 'side', default = 'credit')
        sideLabels = self.sideLabels
        sideFieldLabel = sideLabels[sideFieldValue]
        if not self.isInForm():
            layout = X.array(
                X.asIs(mainFieldValue),
                centsSep,
                X.asIs(centsFieldValue),
                X.nbsp,
                X.asIs(_(sideFieldLabel)),
                )
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                X.asIs(mainFieldValue),
                centsSep,
                X.asIs(centsFieldValue),
                X.nbsp,
                X.asIs(_(sideFieldLabel)),
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            layout += X.input(name = mainFieldName, value = mainFieldValue,
                               size = mainSize, maxlength = mainSize),
            layout += X.nbsp
            layout += centsSep
            layout += X.nbsp
            layout += X.input(name = centsFieldName, value = centsFieldValue,
                               size = centsSize, maxlength = centsSize)
            layout += X.nbsp
            select = X.select(name = sideFieldName)
            if not sideFieldValue:
                select += X.option(selected = 'selected', value = '')(
                    _('None'))
            else:
                for sideValue in sideLabels.keys():
                    optionAttributes = {}
                    if sideValue == sideFieldValue:
                        optionAttributes['selected'] = 'selected'
                    select += X.option(value = sideValue, **optionAttributes)(
                        _(sideLabels[sideValue]))                    
            layout += select
        return layout

    def getModelHiddenLayout(self, slot, fields):
        mainFieldName = slot.getFieldOptionName('main')
        centsFieldName = slot.getFieldOptionName('cents')
        sideFieldName = slot.getFieldOptionName('side')
        mainFieldValue = slot.getFieldOption(fields, 'main', default = '')
        centsFieldValue = slot.getFieldOption(fields, 'cents', default = '')
        sideFieldValue = slot.getFieldOption(fields, 'side', default = '')
        return X.array(
            X.input(name = mainFieldName, type = 'hidden',
                    value = mainFieldValue),
            X.input(name = centsFieldName, type = 'hidden',
                    value = centsFieldValue),
            X.input(name = sideFieldName, type = 'hidden',
                    value = sideFieldValue),
            )
register(Amount)


class Date(WidgetMixin, proxyWidgets.Date):
    def getHtmlValue(self, slot, fields, **keywords):
        sep = '-'
        fieldName = slot.getFieldName()
        monthTitles = {
            '01':N_('January'),
            '02':N_('February'),
            '03':N_('March'),
            '04':N_('April'),
            '05':N_('May'),
            '06':N_('June'),
            '07':N_('July'),
            '08':N_('August'),
            '09':N_('September'),
            '10':N_('October'),
            '11':N_('November'),
            '12':N_('December')}
        monthLabels = monthTitles

        yearFieldName = slot.getFieldOptionName('year')
        monthFieldName = slot.getFieldOptionName('month')
        dayFieldName = slot.getFieldOptionName('day')
        yearFieldValue = slot.getFieldOption(fields, 'year', default = '')
        monthFieldValue = slot.getFieldOption(fields, 'month', default = '')
        dayFieldValue = slot.getFieldOption(fields, 'day', default = '')

        partsToDisplay = slot.getKind().requiredParts
        layout = X.array()
        if monthFieldValue:
            monthTitle = monthTitles[monthFieldValue]
        else:
            monthTitle = ''
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if not self.isInForm() or self.isReadOnly(slot): 
            for partName in partsToDisplay:
                if partName == 'month' and not 'day' in partsToDisplay:
                    layout += X.asIs(monthTitle)
                else:
                    layout += X.asIs(locals()["%sFieldValue" % partName])
                if partName != partsToDisplay[-1]:
                    if partName == 'month' and not 'day' in partsToDisplay:
                        layout += X.nbsp
                    else:
                        layout += sep
        else:
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            # set select specifications
            yearValues = [ '%d'%(
                time.localtime(time.time())[0]+x) for x in range(
                self.yearMinRel, self.yearMaxRel + 1) ]
            monthValues = [ '%02d'%x for x in range(1, 13) ]
            dayValues = [ '%02d'%x for x in range(1, 32) ]
            # year
            if 'year' in partsToDisplay:
                yearSelect = X.select(name = yearFieldName)
                if not yearFieldValue in yearValues:
                    yearSelect += X.option(
                        selected = 'selected', value = '')(_('Year'))
                for value in yearValues:
                    optionAttributes = {}
                    if value == yearFieldValue:
                        optionAttributes['selected'] = 'selected'
                    yearSelect += X.option(
                        value = value, **optionAttributes)(value)
                layout += yearSelect
                layout += X.nbsp
            # month
            if 'month' in partsToDisplay:
                monthSelect = X.select(name = monthFieldName)
                if not monthFieldValue in monthValues:
                    monthSelect += X.option(
                        selected = 'selected', value = '')(_('Month'))
                for value in monthValues:
                    optionAttributes = {}
                    if value == monthFieldValue:
                        optionAttributes['selected'] = 'selected'
                    monthSelect += X.option(
                        value = value, **optionAttributes)(
                        _(monthLabels[value]))
                layout += monthSelect
                layout += X.nbsp
            # day
            if 'day' in partsToDisplay:
                daySelect = X.select(name = dayFieldName)
                if not dayFieldValue in dayValues:
                    daySelect += X.option(
                        selected = 'selected', value = '')(_('Day'))
                for value in dayValues:
                    optionAttributes = {}
                    if value == dayFieldValue:
                        optionAttributes['selected'] = 'selected'
                    daySelect += X.option(
                        value = value, **optionAttributes)(value)
                layout += daySelect
        return layout

    def getModelHiddenLayout(self, slot, fields):
        yearFieldName = slot.getFieldOptionName('year')
        monthFieldName = slot.getFieldOptionName('month')
        dayFieldName = slot.getFieldOptionName('day')
        yearFieldValue = slot.getFieldOption(fields, 'year', default = '')
        monthFieldValue = slot.getFieldOption(fields, 'month', default = '')
        dayFieldValue = slot.getFieldOption(fields, 'day', default = '')
        return X.array(
            X.input(name = yearFieldName, type = 'hidden',
                    value = yearFieldValue),
            X.input(name = monthFieldName, type = 'hidden',
                    value = monthFieldValue),
            X.input(name = dayFieldName, type = 'hidden',
                    value = dayFieldValue),
            )
register(Date)

class Duration(WidgetMixin, proxyWidgets.Duration):
    def getHtmlValue(self, slot, fields, **keywords):
        partLabels = {
            'day' : N_('day'),
            'hour' : N_('hour'),
            'minute' : N_('minute'),
            'second' : N_('second'),
            }
        fieldName = slot.getFieldName()

        secondFieldName = slot.getFieldOptionName('second')
        minuteFieldName = slot.getFieldOptionName('minute')
        hourFieldName = slot.getFieldOptionName('hour')
        dayFieldName = slot.getFieldOptionName('day')

        secondFieldValue = slot.getFieldOption(fields, 'second', default = 0)
        minuteFieldValue = slot.getFieldOption(fields, 'minute', default = 0)
        hourFieldValue = slot.getFieldOption(fields, 'hour', default = 0)
        dayFieldValue = slot.getFieldOption(fields, 'day', default = 0)

        partsToDisplay = slot.getKind().requiredParts
        layout = X.array()
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if not self.isInForm() or self.isReadOnly(slot):
            for partName in partsToDisplay:
                partValue = locals()['%sFieldValue' % partName]
                if partValue:
                    layout += X.asIs(partValue)
                    layout += X.nbsp
                    if partValue == 1:
                        layout += X.asIs('%s' % _(partLabels[partName]))
                    else:
                        layout += X.asIs('%ss' % _(partLabels[partName]))
                    layout += X.nbsp
        else:
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)

            for partName in partsToDisplay:
                layout += X.input(maxlength = 3,
                                  name = locals()['%sFieldName' % partName],
                                  size = 3, type = 'text',
                                  value = locals()['%sFieldValue' % partName])
                layout += X.nbsp
                layout += X.asIs('%ss' % _(partLabels[partName]))
                layout += X.nbsp
        return layout

    def getModelHiddenLayout(self, slot, fields):
        secondFieldName = slot.getFieldOptionName('second')
        minuteFieldName = slot.getFieldOptionName('minute')
        hourFieldName = slot.getFieldOptionName('hour')
        dayFieldName = slot.getFieldOptionName('day')

        secondFieldValue = slot.getFieldOption(fields, 'second', default = 0)
        minuteFieldValue = slot.getFieldOption(fields, 'minute', default = 0)
        hourFieldValue = slot.getFieldOption(fields, 'hour', default = 0)
        dayFieldValue = slot.getFieldOption(fields, 'day', default = 0)

        return X.array(
            X.input(name = secondFieldName, type = 'hidden',
                    value = secondFieldValue),
            X.input(name = minuteFieldName, type = 'hidden',
                    value = minuteFieldValue),
            X.input(name = hourFieldName, type = 'hidden',
                    value = hourFieldValue),
            X.input(name = dayFieldName, type = 'hidden',
                    value = dayFieldValue),
            )
register(Duration)

class Email(WidgetMixin, proxyWidgets.Email):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        layout = X.array()
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if not self.isInForm() or self.isReadOnly(slot):
            if fieldValue:
                layout +=  X.a(href = 'mailto:%s' % fieldValue)(fieldValue)
        else:
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)

            inputAttributes = {}
            if self.maxLength:
                inputAttributes['maxlength'] = self.maxLength
            if self.size:
                inputAttributes['size'] = self.size
            layout += X.input(name = fieldName, type = 'text',
                              value = fieldValue, **inputAttributes)
        return layout

    def getModelHiddenLayout(self, slot, fields):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')

        return X.array(
            X.input(name = fieldName, type = 'hidden',
                    value = fieldValue),
            )
register(Email)


class InputCheckBox(WidgetMixin, proxyWidgets.InputCheckBox):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        if fieldValue:
            value = 1
        else:
            value = 0
        if not value in slot.getKind().values:
            value = slot.getKind().defaultValue
        labels = self.getLabels(slot, fields)
        if not self.isInForm():
            layout = self.makeModelTitleFromValue(slot, fields, value)
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                self.makeModelTitleFromValue(slot, fields, value),
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            inputAttributes = {}
            if value:
                inputAttributes['checked'] = 'checked'
            layout += X.input(name = fieldName, type = 'checkbox',
                              value = 1, **inputAttributes)
            layout += X.nbsp
            label = labels['1']
            layout += _(label)
            if self.apply:
                layout += ' '
                layout += X.buttonInForm('apply', 'applyButton')
        return layout

    def makeModelTitleFromValue(self, slot, fields, value):
        valueAsString = str(value)
        if self.titles and self.titles.has_key(valueAsString):
            return _(self.titles[valueAsString])
        kind = slot.getKind()
        title = kind.makeModelTitleFromValue(slot, fields, value)
        if title:
            return title
        labels = self.getLabels(slot, fields)
        if labels and labels.has_key(valueAsString):
            return _(labels[valueAsString])
        return None
register(InputCheckBox)


class InputPassword(WidgetMixin, proxyWidgets.InputPassword):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        cryptedValue = '*' * len(fieldValue)
        if not self.isInForm():
            layout = cryptedValue
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                cryptedValue,
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            inputAttributes = {}
            if self.maxLength:
                inputAttributes['maxlength'] = self.maxLength
            if self.size:
                inputAttributes['size'] = self.size
            layout += X.input(name = fieldName, type = 'password',
                              value = fieldValue, **inputAttributes)
        return layout
register(InputPassword)


class InputText(WidgetMixin, proxyWidgets.InputText):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = str(slot.getField(fields, default = ''))
        object = slot.getObject()
        translationsProxy = getProxyForServerRole('translations')
        if not self.isInForm():
            if slot.getKind().isTranslatable:
                sourceId = object.getId()
                sourceLanguage = object.getLanguage()
                sourceStringDigest = md5.new(fieldValue.replace(
                                    '\r\n', '\n')).hexdigest()
                if translationsProxy and fieldValue and sourceId and sourceLanguage:
                    fieldValue, destinationLanguage, state = \
                                translationsProxy.getTranslationInfos(
                                        fieldValue, sourceId, slot.getPath(),
                                        sourceLanguage,
                                        context.getVar('readLanguages'))
                    language = context.getVar('readLanguages')[0]
                else:
                    sourceStringDigest = '' # Not used.
                    language = '' # Not used.
                    state = 'invariant'
            else:
                sourceLanguage = '' # Not used.
                sourceStringDigest = '' # Not used.
                language = '' # Not used.
                state = 'invariant'
            layout = X.asIs(replaceSpecialTags(
                    makeHtmlFromSpip(fieldValue)))
            if state in ['fuzzy', 'untranslated']:
                buttonKey = 'translate'
            elif state in ['translated', 'untranslatable']:
                buttonKey = 'change-translation'
            elif state == 'obsolete':
                buttonKey = 'update-translation'
            else:
                buttonKey = None
            if buttonKey is not None and translationsProxy and \
                        translationsProxy.canModifyLocalization(
                            sourceLanguage + language, sourceStringDigest):
                layout = X.array(
                    layout,
                    X.div(_class = 'buttonBar')(X.buttonStandalone(
                        buttonKey,
                        X.url('/translations/edit').add(
                            'localizationKey', sourceLanguage + language).add(
                            'sourceStringDigest', sourceStringDigest))),
                    )
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                X.asIs(replaceSpecialTags(makeHtmlFromSpip(fieldValue))),
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            inputAttributes = {}
            if self.maxLength:
                inputAttributes['maxlength'] = self.maxLength
            if self.size:
                inputAttributes['size'] = self.size
            layout += X.input(name = fieldName, type = 'text',
                              value = fieldValue, **inputAttributes)
        return layout
register(InputText)


class Link(WidgetMixin, proxyWidgets.Link):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        object = slot.getObject()
        nameFieldName = slot.getFieldOptionName('name')
        urlFieldName = slot.getFieldOptionName('url')
        nameFieldValue = slot.getFieldOption(fields, 'name', default = '')
        urlFieldValue = slot.getFieldOption(fields, 'url', default = '')
        translationsProxy = getProxyForServerRole('translations')
        if not self.isInForm():
            sourceId = object.getId()
            sourceLanguage = object.getLanguage()
            if translationsProxy and nameFieldValue and sourceId and sourceLanguage:
                nameFieldValue, destinationLanguage, state = \
                                translationsProxy.getTranslationInfos(
                                    nameFieldValue, sourceId, slot.getPath(),
                                    sourceLanguage, context.getVar('readLanguages'))
                language = context.getVar('readLanguages')[0]
            else:
                sourceStringDigest = '' # Not used.
                language = '' # Not used.
                state = 'invariant'
            if urlFieldValue.startswith('glasnost://'):
                url = X.url(urlFieldValue)
            else:
                url = urlFieldValue
            layout = X.a(href = url)(nameFieldValue)
            if state in ['fuzzy', 'untranslated']:
                buttonKey = 'translate'
            elif state in ['translated', 'untranslatable']:
                buttonKey = 'change-translation'
            elif state == 'obsolete':
                buttonKey = 'update-translation'
            else:
                buttonKey = None
            if buttonKey is not None and translationsProxy and \
                        translationsProxy.canModifyLocalization(
                            sourceLanguage + language, sourceStringDigest):
                layout = X.array(
                    layout,
                    X.div(_class = 'buttonBar')(X.buttonStandalone(
                        buttonKey,
                        X.url('/translations/edit').add(
                            'localizationKey', sourceLanguage + language).add(
                            'sourceStringDigest', sourceStringDigest))),
                    )
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                X.a(href = X.url(urlFieldValue))(nameFieldValue),
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            layout += X.table(
                X.tr(
                    X.td(_class = 'field-label')(_('Title'), X.asIs(_(':') + \
                                                                    ' ')),
                    X.td(X.input(
                        maxlength = 80, name = nameFieldName, size = 80,
                        type = 'text', value = nameFieldValue))),
                X.tr(
                    X.td(_class = 'field-label')(_('URL'), X.asIs(_(':') + \
                                                                  ' ')),
                    X.td(X.input(
                        maxlength = 80, name = urlFieldName, size = 80,
                        type = 'text', value = urlFieldValue))),
                )
        return layout

    def getModelHiddenLayout(self, slot, fields):
        nameFieldName = slot.getFieldOptionName('name')
        urlFieldName = slot.getFieldOptionName('url')
        nameFieldValue = slot.getFieldOption(fields, 'name', default = '')
        urlFieldValue = slot.getFieldOption(fields, 'url', default = '')
        return X.array(
            X.input(name = nameFieldName, type = 'hidden',
                    value = nameFieldValue),
            X.input(name = urlFieldName, type = 'hidden',
                    value = urlFieldValue),
            )
register(Link)


class Mapping(WidgetMixin, proxyWidgets.Mapping):
    def getHtmlValue(self, slot, fields, **keywords):
        kind = slot.getKind()
        countName = slot.getFieldOptionName('count')
        count = kind.getItemsCount(slot, fields)
        layout = X.array()
        if CssJunkie:
            table = X.fieldset()
        else:
            table = X.table(_class = 'field-value')
        layout += table
        if self.isInForm():
            layout += X.input(name = countName, type = 'hidden', value = count)
        for i in range(count):
            keySlot = kind.getItemKeySlot(slot, i)
            keyWidget = keySlot.getWidget()
            if keyWidget is None:
                raise Exception('Missing key widget for %s in %s' % (
                    slot.getPath(), str(keySlot.getKind().__dict__)))
            keyWidget.setInForm(self.isInForm())
            keyWidget.setReadOnly(self.isReadOnly(slot))
##             error, itemKey = keySlot.getKind().submitField2(
##                 keySlot, fields, **keywords)
            valueSlot = kind.getItemKeyValueSlot(keySlot)
            valueWidget = kind.valueKind.getModelWidget(valueSlot)
            if valueWidget is None:
                raise Exception('Missing value widget for %s in %s' % (
                    slot.getPath(), str(valueSlot.getKind().__dict__)))
            valueWidget.setInForm(self.isInForm())
            valueWidget.setReadOnly(self.isReadOnly(slot))
            if CssJunkie:
                table += X.div(_class = 'row')(
                    keyWidget.getHtmlFieldValue(keySlot, fields),
                    valueWidget.getHtmlFieldValue(valueSlot, fields))
            else:
                table += X.tr(_class = 'field-value')(
                    keyWidget.getHtmlFieldValue(keySlot, fields),
                    valueWidget.getHtmlFieldValue(valueSlot, fields))
        if self.isInForm() and not self.isReadOnly(slot):
            if kind.canAddItem(slot):
                addButtonName = slot.getFieldOptionName('addButton')
                layout += X.buttonInForm('add', addButtonName)
            if self.apply:
                layout += X.buttonInForm('apply', 'applyButton')
        return layout

    def getModelHiddenLayout(self, slot, fields):
        countName = slot.getFieldOptionName('count')
        count = slot.getKind().fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        layout = X.array()
        for i in range(count):
            keySlot = slot.getKind().getItemKeySlot(slot, i)
            keyWidget = keySlot.getWidget()
            if keyWidget is None:
                raise Exception('Missing key widget for %s in %s' % (
                    slot.getPath(), str(keySlot.getKind().__dict__)))
            layout += keyWidget.getModelHiddenLayout(keySlot, fields)
##             error, itemKey = keySlot.getKind().submitField2(
##                 keySlot, fields)
            valueSlot = slot.getKind().getItemKeyValueSlot(keySlot)
            valueWidget = valueSlot.getWidget()
            if valueWidget is None:
                raise Exception('Missing value widget for %s in %s' % (
                    slot.getPath(), str(valueSlot.getKind().__dict__)))
            layout += valueWidget.getModelHiddenLayout(valueSlot, fields)
        return layout
register(Mapping)


class Multi(WidgetMixin, proxyWidgets.Multi):
    def getHtmlValue(self, slot, fields, **keywords):
        kind = slot.getKind()
        countName = slot.getFieldOptionName('count')
        count = kind.getItemsCount(slot, fields)
        layout = X.array()
        if self.isInForm():
            layout += self.getModelErrorLayout(slot, fields)
        if CssJunkie:
            fieldset = X.fieldset()
            layout += fieldset
        else:
            table = X.table(_class = 'field-value')
            layout += table
        if self.isInForm():
            layout += X.input(name = countName, type = 'hidden', value = count)
            if self.fieldFind:
                findButtonName = slot.getFieldOptionName('findButton')
                foundsCountName = slot.getFieldOptionName('foundsCount')
                foundsCount = 0
                try:
                    foundsCount = max(foundsCount, int(slot.getFieldOption(
                        fields, 'foundsCount', default = '0')))
                except ValueError:
                    pass
                searchTextName = slot.getFieldOptionName('searchText')
                searchText = slot.getFieldOption(
                    fields, 'searchText', default = '')
                layout += X.input(name = foundsCountName, type = 'hidden',
                              value = foundsCount)
                # FIXME: never tested (and will fail) with useCSSEverywhere
                table += X.tr(_class = 'field-value')(
                    X.td(_class = 'field-value')(
                        X.input(name = searchTextName, type = 'text',
                                value = searchText),
                        X.nbsp,
                        X.buttonInForm('find', findButtonName)))
                foundIds = []
                for i in range(foundsCount):
                    optionName = 'foundIds_%d' % i
                    foundIdName = slot.getFieldOptionName(optionName)
                    foundId = slot.getFieldOption(
                        fields, optionName, default = '')
                    layout += X.input(name = foundIdName, type = 'hidden',
                                  value = foundId)
                    if foundId:
                        foundIds.append(foundId)
                if foundIds:
                    foundItemsMenuName = slot.getFieldOptionName(
                        'foundItemsMenu')
                    slot.setFieldOption(
                        fields, 'foundItemsMenu',
                        getObjectLabelsTranslated(
                        foundIds,
                        context.getVar('readLanguages')))
        for i in range(count):
            itemSlot = kind.getItemSlot(slot, i)
            itemWidget = itemSlot.getWidget()
            if itemWidget is None:
                raise Exception('Missing item widget for %s in %s' % (
                    slot.getPath(), str(itemSlot.getKind().__dict__)))
            itemWidget.setInForm(self.isInForm())
            itemWidget.setReadOnly(self.isReadOnly(slot))
            if kind.isLabelImportant(slot):
                if CssJunkie:
                    fieldset += itemWidget.getHtmlCssField(itemSlot, fields)
                else:
                    table += itemWidget.getHtmlField(itemSlot, fields)
            else:
                if CssJunkie:
                    fieldset += itemWidget.getHtmlCssFieldValue(
                        itemSlot, fields)
                else:
                    table += X.tr(_class = 'field-value')(
                        itemWidget.getHtmlFieldValue(itemSlot, fields))
        if self.isInForm() and not self.isReadOnly(slot):
            if kind.canAddItem(slot):
                addButtonName = slot.getFieldOptionName('addButton')
                if CssJunkie:
                    fieldset += X.buttonInForm('add', addButtonName)
                else:
                    layout += X.buttonInForm('add', addButtonName)
            if self.apply:
                if CssJunkie:
                    fieldset += X.buttonInForm('apply', 'applyButton')
                else:
                    layout += X.buttonInForm('apply', 'applyButton')
        return layout

    def getModelHiddenLayout(self, slot, fields):
        countName = slot.getFieldOptionName('count')
        count = slot.getKind().fieldsCountMin
        try:
            count = max(count, int(slot.getFieldOption(
                fields, 'count', default = '0')))
        except ValueError:
            pass
        layout = X.array()
        for i in range(count):
            itemSlot = slot.getKind().getItemSlot(slot, i)
            itemWidget = itemSlot.getWidget()
            if itemWidget is None:
                raise Exception('Missing item widget for %s in %s' % (
                    slot.getPath(), str(itemSlot.getKind().__dict__)))
            layout += itemWidget.getModelHiddenLayout(itemSlot, fields)
        return layout
register(Multi)


class Mutation(EmptyWidgetMixin, properties.PropertiesContainerMixin,
               proxyWidgets.Mutation):
    pass
register(Mutation)


class Path(WidgetMixin, proxyWidgets.Path):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        object = slot.getObject()
        if not self.isInForm():
            layout = X.a(href = X.url(object.id).add('path', fieldValue))(
                fieldValue)
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                X.a(href = X.url(object.id).add('path', fieldValue))(
                    fieldValue),
                )
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            layout += X.input(
                maxlength = 80, name = fieldName, size = 80,
                type = 'text', value = fieldValue)
        return layout
register(Path)


class Select(WidgetMixin, proxyWidgets.Select):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        kind = slot.getKind()
        values = kind.getValues(slot, fields)
        if values is None:
            raise Exception('Kind "%s" of slot "%s" has no values' % (
                kind, slot))
        if not fieldValue in values:
            fieldValue = kind.defaultValue
        labels = self.getLabels(slot, fields)
        if not self.isInForm():
            if labels.has_key(str(fieldValue)):
                layout = self.makeModelTitleFromValue(slot, fields, fieldValue)
            else:
                layout = None
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                )
            if labels.has_key(str(fieldValue)):
                layout += self.makeModelTitleFromValue(
                    slot, fields, fieldValue)
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            selectAttributes = {}
            selectAttributes['name'] = fieldName
            select = X.select(**selectAttributes)
            if not fieldValue in values:
                if labels.has_key(str(fieldValue)):
                    noneLabel = _(labels[str(fieldValue)])
                else:
                    noneLabel = _('None')
                select += X.option(selected = 'selected', value = '')(
                    noneLabel)
            groupNames, groupedValues = kind.getGroupedValues(slot, fields)
            if groupNames is not None:
                for groupName in groupNames:
                    groupValues = groupedValues[groupName]
                    optgroup = X.optgroup(label = _(groupName))
                    select += optgroup
                    for value in groupValues:
                        optionAttributes = {}
                        if value == fieldValue:
                            optionAttributes['selected'] = 'selected'
                        optgroup += X.option(
                            value = value, **optionAttributes)(
                                _(labels[str(value)]))
            else:
                if hasattr(self, 'order') and self.order is not None:
                    assert len(self.order) == len(values)
                    values = self.order
                for value in values:
                    optionAttributes = {}
                    if value == fieldValue:
                        optionAttributes['selected'] = 'selected'
                    select += X.option(value = value, **optionAttributes)(
                        _(labels[str(value)]))
            layout += select
            if self.apply:
                layout += ' '
                layout += X.buttonInForm('apply', 'applyButton')
        return layout

    def makeModelTitleFromValue(self, slot, fields, value):
        valueAsString = str(value)
        if self.titles and self.titles.has_key(valueAsString):
            return _(self.titles[valueAsString])
        kind = slot.getKind()
        title = kind.makeModelTitleFromValue(slot, fields, value)
        if title:
            return title
        labels = self.getLabels(slot, fields)
        if labels and labels.has_key(valueAsString):
            return _(labels[valueAsString])
        return None
register(Select)


class SelectId(WidgetMixin, proxyWidgets.SelectId):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        if not self.isInForm():
            if not fieldValue:
                layout = _(self.noneLabel)
            else:
                layout = X.objectHypertextLabel(fieldValue)
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                )
            if not fieldValue:
                layout += _(self.noneLabel)
            else:
                layout += X.objectHypertextLabel(fieldValue)
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            serverRoles = slot.getKind().getServerRoles()
            if not serverRoles:
                serverRoles = context.getVar('knownRoles')
            serverRoles = serverRoles[:]
            serverRoles.sort()
            menus = {}
            if self.fieldFind:
                foundItemsMenuName = slot.getFieldOptionName(
                    'foundItemsMenu')
                menus = slot.getFieldOption(
                    fields, 'foundItemsMenu', default = {})
            if not menus:
                preferences = context.getVar('preferences')
                if preferences is None:
                    menus = {}
                else:
                    objectsMemory = preferences.objectsMemory
                    if objectsMemory is None:
                        menus = {}
                    else:
                        for serverRole in serverRoles:
                            if not objectsMemory.has_key(serverRole):
                                continue
                            serverMemory = objectsMemory[serverRole]
                            if not serverMemory:
                                continue
                            menus[serverRole] = getObjectLabelsTranslated(
                                serverMemory,
                                context.getVar('readLanguages'))
            selectAttributes = {}
            selectAttributes['name'] = fieldName
            select = X.select(**selectAttributes)
            optionAttributes = {}
            if not fieldValue:
                optionAttributes['selected'] = 'selected'
            select += X.option(value = '', **optionAttributes)(
                _(self.noneLabel))
            if fieldValue:
                objectId = fieldValue
                try:
                    objectLabel = getObjectLabelTranslated(
                        objectId,
                        context.getVar('readLanguages'))
                except:
                    pass
                else:
                    select += X.option(
                        selected = 'selected', value = objectId)(objectLabel)
            for serverRole in serverRoles:
                web = getWebForServerRole(serverRole)
                if not web:
                    continue
                if not menus.has_key(serverRole):
                    continue
                menu = menus[serverRole]
                if not menu or len(menu) == 1 and menu.has_key(fieldValue):
                    continue
                if len(menus) > 1:
                    optgroup = X.optgroup(
                        label = _(web.objectsNameCapitalized))
                    select += optgroup
                else:
                    optgroup = select
                for objectId, objectLabel in menu.items():
                    if objectId == fieldValue:
                        continue
                    if len(objectLabel) > 80:
                        objectLabel = objectLabel[
                            : 70+objectLabel[70:].find(' ')] + ' (...)'
                    optgroup += X.option(value = objectId)(objectLabel)
            layout += select
            layout += X.input(
                _class = 'button', type = 'button', value = _('Others'),
                onClick = 'selectOthers(this.parentNode, "%s")' % ','.join(
                menus.keys()))
        return layout
register(SelectId)


class TextArea(WidgetMixin, proxyWidgets.TextArea):
    def getHtmlFieldValue(self, slot, fields, **keywords):
        tdAttributes = {}
        if self.colSpan:
            tdAttributes['colspan'] = self.colSpan
        return X.td(_class = 'colspan', **tdAttributes)(
            self.getHtmlValue(slot, fields, **keywords))

    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        object = slot.getObject()
        textareaAttributes = {}
        if self.cols:
            textareaAttributes['cols'] = self.cols
        else:
            textareaAttributes['cols'] = 80
        if self.rows:
            textareaAttributes['rows'] = self.rows
        else:
            textareaAttributes['rows'] = 10
        translationsProxy = getProxyForServerRole('translations')
        if not self.isInForm():
            if slot.getKind().isTranslatable:
                sourceId = object.getId()
                sourceLanguage = object.getLanguage()
                if translationsProxy and fieldValue and sourceId and sourceLanguage:
                    sourceStringDigest = md5.new(fieldValue.replace(
                                        '\r\n', '\n')).hexdigest()
                    fieldValue, destinationLanguage, state = \
                                translationsProxy.getTranslationInfos(
                                        fieldValue, sourceId,
                                        slot.getPath(), sourceLanguage,
                                        context.getVar('readLanguages'))
                    language = context.getVar('readLanguages')[0]
                else:
                    sourceStringDigest = '' # Not used.
                    language = '' # Not used.
                    state = 'invariant'
            else:
                sourceLanguage = '' # Not used.
                sourceStringDigest = '' # Not used.
                language = '' # Not used.
                state = 'invariant'
            if self.viewInTextArea:
                layout = X.textarea(readonly = 'readonly',
                                    **textareaAttributes)(fieldValue)
            else:
                if self.format == 'docbook':
                    formattedText = makeHtmlFromDocBook(fieldValue)
                elif self.format == 'html':
                    formattedText = makeHtmlFromHtml(fieldValue)
                elif self.format == 'text':
                    formattedText = makeHtmlFromText(fieldValue)
                elif self.format == 'rst':
                    formattedText = makeHtmlFromReStructuredText(fieldValue)
                else:
                    formattedText = makeHtmlFromSpip(fieldValue)
                formattedText = replaceSpecialTags(formattedText)
                layout = X.asIs(formattedText)
            if state in ['fuzzy', 'untranslated']:
                buttonKey = 'translate'
            elif state in ['translated', 'untranslatable']:
                buttonKey = 'change-translation'
            elif state == 'obsolete':
                buttonKey = 'update-translation'
            else:
                buttonKey = None
            if buttonKey is not None and translationsProxy and \
                        translationsProxy.canModifyLocalization(
                            sourceLanguage + language, sourceStringDigest):
                layout = X.array(
                    layout,
                    X.div(_class = 'buttonBar')(X.buttonStandalone(
                        buttonKey,
                        X.url('/translations/edit').add(
                            'localizationKey', sourceLanguage + language).add(
                            'sourceStringDigest', sourceStringDigest))),
                    )
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                )
            if self.viewInTextArea:
                layout += X.textarea(readonly = 'readonly',
                                     **textareaAttributes)(fieldValue)
            else:
                if self.format == 'docbook':
                    formattedText = makeHtmlFromDocBook(fieldValue)
                elif self.format == 'html':
                    formattedText = makeHtmlFromHtml(fieldValue)
                elif self.format == 'text':
                    formattedText = makeHtmlFromText(fieldValue)
                elif self.format == 'rst':
                    formattedText = makeHtmlFromReStructuredText(fieldValue)
                else:
                    formattedText = makeHtmlFromSpip(fieldValue)
                formattedText = replaceSpecialTags(formattedText)
                layout += X.asIs(formattedText)
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            layout += X.textarea(
                name = fieldName, **textareaAttributes)(
                fieldValue)
            if self.buttonAddImage \
                   and 'pagenames' in context.getVar('knownRoles'):
                pageNamesProxy = getProxyForServerRole('pagenames')
                idValue = pageNamesProxy.getIdByName('addImage')
                if idValue:
                    layout += X.buttonPopup(
                        'addImage', X.url('/addImage/popup-start'))
            if self.preview:
                layout += X.div(_class = 'field-value-buttons-bar')(
                    X.buttonInForm('preview', 'applyButton'))
                if self.format == 'docbook':
                    formattedText = makeHtmlFromDocBook(fieldValue)
                elif self.format == 'html':
                    formattedText = makeHtmlFromHtml(fieldValue)
                elif self.format == 'text':
                    formattedText = makeHtmlFromText(fieldValue)
                elif self.format == 'rst':
                    formattedText = makeHtmlFromReStructuredText(fieldValue)
                else:
                    formattedText = makeHtmlFromSpip(fieldValue)
                formattedText = replaceSpecialTags(formattedText)
                if formattedText:
                    layout += X.div(_class = 'preview')(X.asIs(formattedText))
        return layout
register(TextArea)


class Thing(ThingMixin, proxyWidgets.Thing):
    pass
register(Thing)


class Time(InputText, proxyWidgets.Time):
    def getHtmlValue(self, slot, fields, **keywords):
        layout = InputText.getHtmlValue(self, slot, fields, **keywords)
        if not self.isInForm() or self.isReadOnly(slot):
            return layout
        layout += X.input(
            _class = 'button', type = 'button', value = _('Calendar'),
            onClick = "this.previousSibling.id = 'editedTime'; " \
            "window.open('%s', 'cal'," \
            "'dependent=yes,width=180,height=180," \
            "screenX=200,screenY=300,titlebar=no'); " \
            "return false" %  X.url('/calendarPopup').getAsUrl())
        return layout
register(Time)


class Token(WidgetMixin, proxyWidgets.Token):
    pass
register(Token)


class UploadFile(WidgetMixin, proxyWidgets.UploadFile):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        if self.fileName:
            addedToUrl = '/%s' % self.fileName
        else:
            addedToUrl = ''
        fieldValue = slot.getField(fields, default = '')
        typeSlot = slot.getContainer().getSlot(
            slot.name + 'Type', parentSlot = slot.parent)
##         fileNameSlot = slot.getContainer().getSlot(
##             slot.name + 'FileName', parentSlot = slot.parent)
        object = slot.getObject()
        image = '' # _('download')
        if object.id is not None and isTypeOfMimeType(
            typeSlot.getField(fields, default = ''), 'image'):
            uri = X.url('/%s/thumbnail' % (getUrlId(object.id)))
            if slot.parent is not None:
                uri.add('path', slot.parent.getPath())
            uri.add('width', 256)
            uri.add('height', 256)
            image = X.img(src = uri)
##             uriSlot = slot.getContainer().getSlot(
##                 slot.name + 'Uri', parentSlot = slot.parent)
##             image = X.img(src = uriSlot.getField(fields))
##         elif fileNameSlot.hasField(fields):
##             image = fileNameSlot.getField(fields)
##         fullUriSlot = slot.getContainer().getSlot(
##             slot.name + 'FullUri', parentSlot = slot.parent)
##         if fullUriSlot.hasField(fields):
##             image = X.a(_class = 'image', href = fullUriSlot.getField(fields))(
##                 image)
        if not self.isInForm():
            if object.id is not None:
                if isTypeOfMimeType(
                    typeSlot.getField(fields, default = ''), 'image'):
                    uri = X.url('/%s/thumbnail' % getUrlId(object.id))
                    if slot.parent is not None:
                        uri.add('path', slot.parent.getPath())
                    uri.add('width', 256)
                    uri.add('height', 256)
                    image = X.img(src = uri)
                    uri = X.url('/%s/image%s' % ( getUrlId(object.id), addedToUrl))
                    if slot.parent is not None \
                           and slot.parent.getPath() != 'self':
                        uri.add('path', slot.parent.getPath())
                else:
                    uri = X.url('/%s/download' % (
                        object.getWeb().serverRole, getUrlId(object.id)))
                    if slot.parent is not None:
                        uri.add('path', slot.parent.getPath())
                if not image:
                    image = _('download')
                image = X.a(_class = 'image', href = uri)(image)
            layout = image
        else:
            if isTypeOfMimeType(
                typeSlot.getField(fields, default = ''), 'image'):
                if object.id is not None:
                    uri = X.url('/%s/imageEdit' % getUrlId(object.id))
                else:
                    uri = X.url('/%s/imageEdit' % (object.getWeb().serverRole))
                if slot.parent is not None:
                    uri.add('path', slot.parent.getPath())
                uri.add('%s|session' % fieldName, 1)
                uri.addKeywords(keywords)
                image = X.img(src = uri)
            if self.isReadOnly(slot):
                session = context.getVar('session')
                session['field_' + fieldName] = base64.encodestring(fieldValue)
                session['isDirty'] = 1
                layout = X.array(
                    self.getModelHiddenLayout(slot, fields),
                    image,
                    )
            else:
                layout = X.array()
                layout += self.getModelErrorLayout(slot, fields)
                layout += self.getModelHelpLayout(slot, fields)
                session = context.getVar('session')
                session['field_' + fieldName] = base64.encodestring(fieldValue)
                session['isDirty'] = 1
                layout += X.input(name = '%s|session' % fieldName,
                                  type = 'hidden', value = 1)
                if image:
                    layout += image
                    layout += X.br()
                layout += X.input(name = '%s_field' % fieldName, type = 'file')
        return layout
register(UploadFile)

class Url(WidgetMixin, proxyWidgets.Url):
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        layout = X.array()
        if self.isReadOnly(slot):
            layout += self.getModelHiddenLayout(slot, fields)
        if not self.isInForm() or self.isReadOnly(slot):
            if fieldValue:
                if fieldValue.startswith('http://'):
                    href = fieldValue
                else:
                    href = 'http://%s' % fieldValue
                layout +=  X.a(href = href)(fieldValue)
            else:
                layout += X.nbsp
        else:
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)

            inputAttributes = {}
            if self.maxLength:
                inputAttributes['maxlength'] = self.maxLength
            if self.size:
                inputAttributes['size'] = self.size
            layout += X.input(name = fieldName, type = 'text',
                              value = fieldValue, **inputAttributes)
        return layout

    def getModelHiddenLayout(self, slot, fields):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')

        return X.array(
            X.input(name = fieldName, type = 'hidden',
                    value = fieldValue),
            )
register(Url)


class XSelect(Select, proxyWidgets.XSelect):
    otherFieldLabel_defaultValue = N_('other:')
    otherFieldLabel_kind_widget_fieldLabel = N_('Label')
    
    def getHtmlValue(self, slot, fields, **keywords):
        fieldName = slot.getFieldName()
        fieldValue = slot.getField(fields, default = '')
        otherFieldLabel = self.otherFieldLabel or \
                          self.otherFieldLabel_defaultValue 
        otherFieldValue = slot.getFieldOption(fields, 'other', default = '')
        kind = slot.getKind()
        values = kind.getValues(slot, fields)
        groupNames, groupedValues = kind.getGroupedValues(slot, fields)
        if values is None:
            raise Exception('Kind "%s" of slot "%s" has no values' % (
                slot.getKind(), slot))
        if not fieldValue in values:
            fieldValue = slot.getKind().defaultValue
        labels = self.getLabels(slot, fields)
        if not self.isInForm():
            if labels.has_key(str(fieldValue)):
                layout = self.makeModelTitleFromValue(slot, fields, fieldValue)
            elif otherFieldValue:
                layout = X.asIs(_(str(otherFieldValue)))
            else:
                layout = None
        elif self.isReadOnly(slot):
            layout = X.array(
                self.getModelHiddenLayout(slot, fields),
                )
            if labels.has_key(str(fieldValue)):
                displayFieldValue = self.makeModelTitleFromValue(
                    slot, fields, fieldValue)
            else:
                displayFieldValue = _(str(fieldValue))
            layout += displayFieldValue
        else:
            layout = X.array()
            layout += self.getModelErrorLayout(slot, fields)
            layout += self.getModelHelpLayout(slot, fields)
            table = X.table(_class = 'field-value')
            layout += table
            selectAttributes = {}
            selectAttributes['name'] = fieldName
            select = X.select(**selectAttributes)
            if not fieldValue in values:
                if labels.has_key(str(fieldValue)):
                    noneLabel = _(labels[str(fieldValue)])
                else:
                    noneLabel = _('None')
                select += X.option(selected = 'selected', value = '')(
                    noneLabel)
            if groupNames is not None:
                for groupName in groupNames:
                    groupValues = groupedValues[groupName]
                    optgroup = X.optgroup(label = _(groupName))
                    select += optgroup
                    for value in groupValues:
                        optionAttributes = {}
                        if value == fieldValue:
                            optionAttributes['selected'] = 'selected'
                        optgroup += X.option(
                            value = value, **optionAttributes)(
                                _(labels[str(value)]))
            else:
                if hasattr(self, 'order') and self.order is not None:
                    assert len(self.order) == len(values)
                    values = self.order
                for value in values:
                    optionAttributes = {}
                    if value == fieldValue:
                        optionAttributes['selected'] = 'selected'
                    select += X.option(value = value, **optionAttributes)(
                        _(labels[str(value)]))
            table += X.tr(_class = 'field-value')(
                X.td(_class = 'field-value', colspan="2")(select))
            inputAttributes = {}
            if self.maxLength:
                inputAttributes['maxlength'] = self.maxLength
            if self.size:
                inputAttributes['size'] = self.size
            table += X.tr(_class = 'field-value')(
                X.td(_class = 'field-label')(X.asIs(_(otherFieldLabel))),
                X.td(_class = 'field-value')(
                X.input(
                name = '%s_other' % fieldName, type = 'text',
                value = otherFieldValue, **inputAttributes)))
            if self.apply:
                layout += ' '
                layout += X.buttonInForm('apply', 'applyButton')
        return layout

    def makeModelTitleFromValue(self, slot, fields, value):
        valueAsString = str(value)
        if self.titles and self.titles.has_key(valueAsString):
            return _(self.titles[valueAsString])
        labels = self.getLabels(slot, fields)        
        if labels and labels.has_key(valueAsString):
            return _(labels[valueAsString])
        kind = slot.getKind()
        title = kind.makeModelTitleFromValue(slot, fields, value)
        if title:
            return title
        return valueAsString
register(XSelect)


