import calendar
import time

minimumDayForFirstWeek = 1

months = [N_('January'), N_('February'), N_('March'), N_('April'),
          N_('May'), N_('June'), N_('July'), N_('August'), N_('September'),
          N_('October'), N_('November'), N_('December') ]

days = [ N_('Monday'), N_('Tuesday'), N_('Wednesday'), N_('Thursday'),
         N_('Friday'), N_('Saturday'), N_('Sunday') ]


def castDate(*datePart):
    result = []
    for d in datePart:
        if type(d) == type(''):
            if d != '':
                result.append(int(d))
            else:
                result.append(0)
        elif type(d) == type(0):
            result.append(d)
        else:
            raise "Bad type for date element: %s (%s)" % (d, type(d))
    return result

def getDayLabels():
    return days

def getMonthLabels():
    return months

def generateOptionTag(content, value, selectedValue=None, condition=1):
    if condition:
        return """<option %s value="%s">%s</option>""" % (['','selected=""'][value==selectedValue], value, content)
    else:
        return ""
        
def monthTuple(year, month):
    year, month = castDate(year, month)
    cal = calendar.monthcalendar(year, month)
    for i in range(len(cal)):
        w = cal[i]
        for j in range(len(w)):
            d = cal[i][j]
            cal[i][j] = Date(year, month, cal[i][j])
    return cal

class Date:    
    def __init__(self, year, month, day):
        self.year, self.month, self.day = castDate(year, month, day)

    def __repr__(self):
        return self.getAsString()

    def clone(self):
        return Date(self.year, self.month, self.day)

    def dayOfYear(self):
        """
        return the number of day between the 1 january and the current day
        """
        if self.day == 0:
            day = 1
        else:
            day = self.day
        return time.localtime(time.mktime((self.year, self.month, day,
                                           12,0,0, 0,0, -1)))[7]
    
    def firstDayOfMonth(self):
        """
        return the first day of month where the current day is
        """
        result = self.clone()
        result.day = 1
        return result

    def firstDayOfWeek(self):
        """
        return the first day of week where the current day is
        """
        result = self.clone()
        result.year, result.month, result.day = \
                     time.localtime(time.mktime((self.year, self.month,
                     self.day - self.weekday() + calendar.firstweekday(),
                                                 12,0,0, 0,0, -1)))[0:3]
        return result

    def getAsString(self):
        return str(self.year)+'-'+str(self.month)+'-'+str(self.day)

    def getClassDay(self):
        if self.day == 0:
            return 'noday'

        currentTime = time.mktime(time.localtime())
        if time.mktime((self.year, self.month, self.day+1,
                        0, 0, 0, 0, 0, -1)) <currentTime:
            return 'previewDay'
        elif currentTime < time.mktime((self.year, self.month, self.day,
                                        0, 0, 0, 0, 0, -1)):
            return 'nextDay'
        else:
            return 'today'

    def getDayAsString(self):
        if self.day == 0:
            return ''
        else:
            return str(self.day)

    def getMonthAsString(self):
        return _(getMonthLabels()[self.month-1])

    def getSaint(self):
        return saint[self.month-1][self.day-1]

    def getYearAsString(self):
        return str(self.year)

    def haveSaint(self):
        return self.isValid()

    def isValid(self):
        return self.month != 0 and self.day != 0

    def lastDayOfMonth(self):
        """
        return the last day of month where the current day is
        """
        result = self.clone()
        result.day = calendar.monthrange(result.year, result.month)[1]
        return result
 
    def lastDayOfWeek(self):
        """
        return the last day of week where the current day is
        """
        result = self.clone()
        result.year, result.month, result.day = \
                     time.localtime(time.mktime((self.year, self.month,
                     self.day - self.weekday() + calendar.firstweekday() + 6,
                                                 12,0,0, 0,0, -1)))[0:3]
        return result

    def mktime(self):
        result = time.mktime((self.year, self.month, self.day,
                              0, 0, 0, 0, 0, -1))
        return result
 
    def monthTuple(self):
        cal = calendar.monthcalendar(self.year, self.month)
        for i in range(len(cal)):
            w = cal[i]
            for j in range(len(w)):
                d = cal[i][j]
                tmp = self.clone()
                tmp.day = cal[i][j]
                cal[i][j] = tmp
        return cal

    def nextDay(self):
        result = self.clone()
        result.year, result.month, result.day = \
               time.localtime(time.mktime((self.year, self.month, self.day + 1,
                                           0, 0, 0, 0, 0, -1)))[0:3]
        return result

    def nextMonth(self):
        result = self.clone()
        result.year, result.month, result.day = \
               time.localtime(time.mktime((self.year, self.month + 1, self.day,
                                  0, 0, 0, 0, 0, -1)))[0:3]
        return result

    def previewMonth(self):
        result = self.clone()
        result.year, result.month, result.day = \
               time.localtime(time.mktime((self.year, self.month - 1, self.day,
                                           0, 0, 0, 0, 0, -1)))[0:3]
        return result

    def weekOfYear(self):
        dowJan1 = calendar.weekday(self.year, 1, 1)
        _dayOfYear = self.dayOfYear()
        week = (_dayOfYear - 1 + dowJan1) / 7 # 0..53
        if (7 - dowJan1) >= minimumDayForFirstWeek:
            week = week + 1
        return week

    def weekday(self):
        return calendar.weekday(self.year, self.month, self.day)
       
    def weekTuple(self):
        """
        @return : list of day those represent week where the current date is.
        """
        first = self.firstDayOfWeek()
        result = [first]
        for i in range(6):
            result.append(result[-1].nextDay())
        return result

class AppointmentDate(Date):
    def __init__(self, year, month, day, appointmentsList):
        Date.__init__(self, year, month, day)
        self.appointmentsList = appointmentsList

    def clone(self):
        return AppointmentDate(self.year, self.month, self.day,
                               self.appointmentsList)

    def haveAppointments(self):
        return len(self.getAppointments()) > 0

    def getAppointments(self, hour = None):        
        if not hour:
            dayBegin = time.mktime((self.year, self.month, self.day,
                                    0, 0, 0, 0, 0, -1))
            dayEnd = time.mktime((self.year, self.month, self.day,
                                  23, 59, 59, 0, 0, -1))
        else:
            dayBegin = time.mktime((self.year, self.month, self.day,
                                    hour, 0, 0, 0, 0, -1))
            dayEnd = time.mktime((self.year, self.month, self.day,
                                  hour, 59, 59, 0, 0, -1))
        
        result = []
        for o in self.appointmentsList:
            if o.dateFor(dayBegin, dayEnd):
                result.append(o)
        return result




if __name__ == '__main__':
    d = Date(2003, 1, 21)
    print 'd.getAsString()', d.getAsString()
    print 'd.clone()', d.clone()
    print 'd.dayOfYear()', d.dayOfYear()
    print 'd.firstDayOfMonth()', d.firstDayOfMonth()
    print 'd.firstDayOfWeek()', d.firstDayOfWeek()
    print 'd.getAsString()', d.getAsString()
    print 'd.getClassDay()', d.getClassDay()
    print 'd.getDayAsString()', d.getDayAsString()
    print 'd.getMonthAsString()', d.getMonthAsString()
    print 'd.getSaint()', d.getSaint()
    print 'd.isValid()', d.isValid()
    print 'd.lastDayOfMonth()', d.lastDayOfMonth()
    print 'd.lastDayOfWeek()', d.lastDayOfWeek()
    print 'd.mktime()', d.mktime()
    print 'd.monthTuple()', d.monthTuple()
    print 'd.nextDay()', d.nextDay()
    print 'd.nextMonth()', d.nextMonth()
    print 'd.previewMonth()', d.previewMonth()
    print 'd.weekOfYear()', d.weekOfYear()
    print 'd.weekday()', d.weekday()
    print 'd.weekTuple()', d.weekTuple()
    print
    print d.firstDayOfMonth().mktime()

