# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Web Handler"""

__version__ = '$Revision: 1.83 $'[11:-2]


import __builtin__
import base64
import binascii
import Cookie
import fcntl
import gettext
import imp
import locale
import md5
import os
import socket
import sys
import time
import types

from mod_python import apache, util

__builtin__.__dict__['N_'] = lambda x: x

import glasnost as glasnost # Do not edit or remove this line!

import glasnost.common.context as context
from glasnost.common.cache import cache
import glasnost.common.faults as faults
import glasnost.common.tools_new as commonTools
import glasnost.common.xhtmlgenerator as X

# Shall be located before the proxy imports.
context.clear()

from glasnost.proxy.DispatcherProxy import getRegisteredRoles

from glasnost.web.tools import *


localePath = getConfig('Misc', 'LocaleDir', '/usr/share/locale')
mainModule = sys.modules['__main__']
mainModule.applicationName = 'glasnost-web'
mainModule.applicationRole = 'glasnost-web'


class BufferedRequest(apache.Request):
    buffered = 1

    def __init__(self, req):
        self._req = req
        self.buffer = StringIO()
    
    def write(self, str):
        self.buffer.write(str)

    def finish(self):
        self.buffer.seek(0)
        self._req.write(self.buffer.read())


class RequestCache(apache.Request):
    buffered = 0
    cacheFile = None
    cacheDirectoryPath = os.path.join(
        getConfig('Misc', 'vardir', '/var/lib/glasnost'), 'webcache')
    cacheTime = 15 * 60 # Default cache time: 15 minutes.
    caching = 1
    depends = None

    def __init__(self, req, language):
        for key, value in req.__dict__.items():
            self.__dict__[key] = value
        self.cacheFilePath = os.path.join(
            self.cacheDirectoryPath,
            self.getCacheFileName(self.unparsed_uri, language))
        self.cacheFilePathDepends = os.path.join(
            self.cacheDirectoryPath,
            self.getCacheFileName(self.unparsed_uri, language) + '.depends')
        self.cacheFilePathMimeType = os.path.join(
            self.cacheDirectoryPath,
            self.getCacheFileName(self.unparsed_uri, language) + '.mimetype')
        self.depends = []

    def cancel(self):
        if self.cacheFile is not None:
            self.cacheFile.close()
            del self.cacheFile
        if os.path.exists(self.cacheFilePath):
            os.unlink(self.cacheFilePath)

    def checkDepends(self):
        if not os.path.exists(self.cacheFilePathDepends):
            return
        if not os.path.exists(self.cacheFilePath):
            return
        ids = open(self.cacheFilePathDepends).readlines()
        modificationTime = 0
        for id in ids:
            web = getWeb(id)
            object = web.getPartialObject(id, ['modificationTime'])
            if object.modificationTime > os.stat(self.cacheFilePath)[ST_CTIME]:
                os.unlink(self.cacheFilePath)
                break
            if object.modificationTime > modificationTime:
                modificationTime = object.modificationTime
        else:
            self.caching = 1
        return time.gmtime(modificationTime)

    def isPageCached(self):
        return os.path.exists(self.cacheFilePath) \
               and time.time() - \
               os.stat(self.cacheFilePath)[ST_CTIME] <= self.cacheTime

    def closeCachePage(self):
        self.cacheFile.close()
        del self.cacheFile

        if self.caching == -1:
            if self.depends:
                open(self.cacheFilePathDepends, 'w').write('\n'.join(
                    self.depends))
                open(self.cacheFilePathMimeType, 'w').write(self.content_type)
            else:
                try:
                    os.unlink(self.cacheFilePath)
                except OSError:
                    pass
        elif self.caching == 1:
            open(self.cacheFilePathMimeType, 'w').write(self.content_type)


    def getCachePage(self):
        self.cacheFile = open(self.cacheFilePath, 'r')
        fcntl.lockf(self.cacheFile, fcntl.LOCK_SH)
        cachePage = self.cacheFile.read()
        fcntl.lockf(self.cacheFile, fcntl.LOCK_UN)
        self.cacheFile.close()
        if os.path.exists(self.cacheFilePathMimeType):
            mimeType = open(self.cacheFilePathMimeType).read()
        else:
            mimeType = 'text/html'
        return cachePage, mimeType

    def getCacheFileName(self, uri, language):
        return binascii.hexlify(md5.new(uri + language).digest())

    def getExpires(self):
        try:
            t = os.stat(self.cacheFilePath)[ST_CTIME] + self.cacheTime
        except IOError:
            t = time.time() + self.cacheTime
        return time.strftime('%a, %d %b %Y %H:%M:%S %Z', time.gmtime(t))

    def openCachePage(self):
        try:
            self.cacheFile = open(self.cacheFilePath, 'w')
        except IOError:
            self.cacheFile = open('/dev/null', 'w')

    def write(self, str):
        self._req.write(str)
        if self.cacheFile is not None:
            fcntl.lockf(self.cacheFile, fcntl.LOCK_EX)
            self.cacheFile.write(str)
            fcntl.lockf(self.cacheFile, fcntl.LOCK_UN)


def callFunction():
    function = context.getVar('function')
    positionalArguments = context.getVar('positionalArguments')
    keywordsArguments = context.getVar('keywordsArguments')
    objectId = context.getVar('objectId')

    if positionalArguments is None:
        arguments = []
    else:
        arguments = positionalArguments[:]
    if keywordsArguments is None:
        keywords = {}
    else:
        keywords = keywordsArguments.copy()
    if objectId is not None:
        keywords['id'] = objectId

    functionCode = function.func_code
    expectedArguments = functionCode.co_varnames[:functionCode.co_argcount]
    if function.func_defaults is None:
        requiredArguments = expectedArguments        
    else:
        requiredArguments = expectedArguments[:-len(function.func_defaults)]
    acceptsArbitraryKeywordsArguments = functionCode.co_flags & 8 != 0
    acceptsArbitraryPositionalArguments = functionCode.co_flags & 4 != 0

    # Remove unexpected arguments.
    if not acceptsArbitraryKeywordsArguments:
        for argumentName in keywords.keys():
            if argumentName not in expectedArguments:
                del keywords[argumentName]

    # Insert the needed keywords arguments inside the positional arguments and
    # check that there is no missing argument.
    newArguments = []
    hasSelf = 0
    for argumentName in requiredArguments:
        if argumentName == 'self':
            hasSelf = 1
        elif argumentName in keywords.keys():
            newArguments.append(keywords[argumentName])
            del keywords[argumentName]
        elif not arguments:
            if context.getVar('debug'):
                raise Exception('missing argument = %s' % argumentName)
            return HTTP_NOT_FOUND
        else:
            newArguments.append(arguments[0])
            del arguments[0]
    arguments = newArguments + arguments
 
    # Check that there is not too much positional arguments.
    if not acceptsArbitraryPositionalArguments \
       and hasSelf + len(arguments) > len(expectedArguments):
        if context.getVar('debug'):
            raise Exception('Too much positional arguments = %s' % arguments)
        return HTTP_NOT_FOUND

    req = context.getVar('req')
    try:
        try:
            result = function(*arguments, **keywords)
        finally:
            if req.buffered:
                req.finish()
            session = context.getVar('session')
            if session is not None:
                getProxyForServerRole('sessions').setSession(session)
    except IOError:
        # Write failed, client closed connection.
        if req.caching:
            req.cancel()
        return HTTP_INTERNAL_SERVER_ERROR
    except:
        if req.caching:
            req.cancel()
        raise

    t1 = context.getVar('t1')
    open('/tmp/webbench', 'a+').write('%f  %s\n' % (
        time.time() - t1, req.unparsed_uri))
    return result


def handler(req):
    t1 = time.time()
    debug = getConfig('Misc', 'Debug') == 'true'
    # Use direct access to _req for speed.
    _req = req._req

    context.push(
        applicationId = None,
        applicationToken = None,
        cache = None,
        canUseCookie = 0, # Are the cookies accepted by the client browser?
        configSection = None,
        cookie = None,
        debug = debug,
        dispatcherId = None,
        fallbackTemplatesDirectoryPath = None,
        function = None,
        httpHostName = None,
        httpLocalPath = None,
            # The HTTP path, without the httpScriptDirectoryPath prefix.
        httpPath = None,
            # The full HTTP path, beginning with httpScriptDirectoryPath.
        httpPort = None,
        httpProtocol = None, # http or https
        httpQuery = None,
        httpScriptDirectoryPath = None,
            # Used when glasnost is inside another web site.
        keywordsArguments = None,
        knownRoles = None,
        language = None,
        objectId = None,
        positionalArguments = None,
        preferences = None,
        readLanguages = None,
        req = req,
        serverRole = None,
        session = None,
        sessionToken = None,
        sessionTokenInCookie = 0, # Was the sessionToken stored in the url?
        t1 = t1,
        talEngine = None,
        templateDirectoryName = None,
        templatesDirectoryPath = None,
        templateFileName = None,
        templatePrefix = None,
        useCssEverywhere = CssJunkie,
        userId = '',
        userToken = '',
        virtualHost = None,
        webFileName = None,
        )

    if req.method in ('OPTIONS', 'PUT', 'DELETE', 'TRACE', 'CONNECT'):
        return HTTP_NOT_IMPLEMENTED

    httpPath = _req.uri
    while len(httpPath) > 1 and httpPath[-1] == '/':
        httpPath = httpPath[:-1]
    if not httpPath:
        httpPath = '/'
    context.setVar('httpPath', httpPath)
    context.setVar('httpQuery', req.args)
    if _req.path_info:
        httpScriptPath = _req.uri[: - len(_req.path_info)]
    else:
        httpScriptPath = _req.uri
    while len(httpScriptPath) > 1 and httpScriptPath[-1] == '/':
        httpScriptPath = httpScriptPath[:-1]
    if not httpScriptPath:
        httpScriptPath = '/'
    httpScriptDirectoryPath = commonTools.makeHttpScriptDirectoryPath(
        httpScriptPath)
    context.setVar(
        'httpScriptDirectoryPath', httpScriptDirectoryPath)
    if httpScriptDirectoryPath == '/':
        httpLocalPath = httpPath
    else:
        httpLocalPath = httpPath[len(httpScriptDirectoryPath):]
    context.setVar('httpLocalPath', httpLocalPath)
    import glasnost.web.tools_new as webTools

    webFilePath = _req.filename
    while len(webFilePath) > 1 and webFilePath[-2] == '/':
        webFilePath = webFilePath[:-1]
    if not webFilePath:
        webFilePath = '/'
    if os.path.isdir(webFilePath):
        webFilePath += '/'
    webDirectoryPath, webFileName = os.path.split(webFilePath)
    if not webFileName:
        choices = ['index.tal', 'index.py', 'index.html']
        for c in choices:
            if os.path.exists( os.path.join(webDirectoryPath, c) ):
                webFileName = c
                webFilePath += '/' + webFileName
                break
        else:
            return HTTP_NOT_FOUND

    context.setVar('webDirectoryPath', webDirectoryPath)
    context.setVar('webFileName', webFileName)

    # If the file at webFilePath exists and is static (images, stylesheets...)
    # then return its content directly.
    webFileExtension = ''
    splittedWebFileName = webFileName.split('.')
    if len(splittedWebFileName) > 1:
        webFileExtension = splittedWebFileName[-1]
    if not webFileExtension in ['tal', 'py', 'xtal'] \
           and webFileName != 'index':
        # The above test ensures that the user is not able to see the source of
        # executable files.
        staticFilePath = None
        if os.path.exists(webFilePath):
            staticFilePath = webFilePath
        else:
            alternRootPath = getConfig('Misc', 'AlternRootPath')
            if alternRootPath:
                alternWebFilePath = os.path.join(
                    alternRootPath, httpLocalPath[1:])
                if os.path.exists(alternWebFilePath):
                    staticFilePath = alternWebFilePath
        if staticFilePath:
            try:
                staticFile = open(staticFilePath)
            except IOError, error:
                if error.errno == 13:
                    return HTTP_FORBIDDEN
                return HTTP_NOT_FOUND
            lastModTime = time.gmtime(os.stat(staticFilePath)[-2])
            return outputStaticFile(req, staticFile.read(), lastModTime)

    if req.headers_in.has_key('Cookie'):
        cookie = Cookie.SmartCookie(req.headers_in['Cookie'])
        try:
            cookieContent = cookie['glasnost'].value
        except KeyError:
            cookie = None
            cookieContent = {}
    else:
        cookie = None
        cookieContent = {}
    context.setVar('cookie', cookie)

    # There seems to be no way to know if we are currently using the http or
    # https protocol!
    # context.setVar('httpProtocol', None)
    httpHostName = None
    httpPort = None
    if req.headers_in.has_key('Host'):
        hostNameAndPort = req.headers_in['Host']
        if ':' in hostNameAndPort:
            httpHostName, httpPort = hostNameAndPort.split(':')
        else:
            httpHostName = hostNameAndPort
    context.setVar('httpHostName', httpHostName)
    context.setVar('httpPort', httpPort)
    dispatcherIds = getConfig(
       'Misc', 'DispatcherURLs', default = 'glasnost://localhost')
    dispatcherIds = dispatcherIds.replace(',', ' ')
    dispatcherIds = dispatcherIds.strip().split()
    for i in range(len(dispatcherIds)):
        if dispatcherIds[i][-1] == '/':
            dispatcherIds[i] = dispatcherIds[i][:-1]
    dispatcherId = dispatcherIds[0]
    context.setVar('dispatcherId', dispatcherId)
    context.setVar(
        'applicationId',
        commonTools.makeApplicationId(dispatcherId,
                                      mainModule.applicationRole))

    # we have to do it before asking for the VirtualHost object
    context.setVar('cache', cache)
    try:
        cache.checkCachedValues()
    except socket.error, e:
        if e[0] != 111:
            raise
        return HTTP_SERVICE_UNAVAILABLE

    virtualHost = None
    try:
        virtualHostsWeb = getWebForServerRole('virtualhosts')
        virtualHost = virtualHostsWeb.getObjectByHostName(httpHostName)
    except faults.MissingItem:
        try:
            virtualHost = virtualHostsWeb.getObjectByDispatcherId(dispatcherId)
        except faults.MissingItem:
            pass
    except socket.error, e:
        if e[0] != 111: # Connection refused
            raise
        return HTTP_SERVICE_UNAVAILABLE
    context.setVar('virtualHost', virtualHost)
    if virtualHost is not None \
       and virtualHost.defaultDispatcherId:
        dispatcherId = virtualHost.defaultDispatcherId
        if dispatcherId[-1] == '/':
            dispatcherId = dispatcherId[:-1]
    context.setVar('dispatcherId', dispatcherId)
    context.setVar(
        'applicationId',
        commonTools.makeApplicationId(
            dispatcherId, mainModule.applicationRole))

    configSection = extractApplicationHostName(dispatcherId)
    context.setVar('configSection', configSection)

    templatesDirectoryPath = getConfig(
        configSection, 'TemplatesDirectoryPath',
        default = getConfig('Misc', 'TemplatesDirectoryPath'))
    fallbackTemplatesDirectoryPath = getConfig(
        'Misc', 'TemplatesDirectoryPath', default=None)
    if not templatesDirectoryPath:
        # configuration file changed between 0.5.4 and 0.5.5.  This fallbacks
        # to the old configuration.  Will be removed someday.
        templatesDirectoryPath = getConfig('Templates', 'Directory') + '/../'
        fallbackTemplatesDirectoryPath = templatesDirectoryPath
    context.setVar('templatesDirectoryPath', templatesDirectoryPath)
    context.setVar(
        'fallbackTemplatesDirectoryPath', fallbackTemplatesDirectoryPath)

    templateDirectoryName = getConfig('Misc', 'DefaultTemplate')
    if virtualHost is not None \
       and virtualHost.templateDirectoryName is not None:
        templateDirectoryName = virtualHost.templateDirectoryName
    context.setVar('templateDirectoryName', templateDirectoryName)
 
    args = {}
    context.setVar('keywordsArguments', args)

    # Process input, if any.
    fieldStorage = util.FieldStorage(req, keep_blank_values = 1)

    fields = {}
    for field in fieldStorage.list:
        if field.name is not None and not fields.has_key(field.name):
            fields[field.name] = fieldStorage[field.name]
        elif field.name is not None:
            if not type(fields[field.name]) is type([]):
                fields[field.name] = [ fields[field.name] ]
            fields[field.name].append(fieldStorage[field.name])

    fieldNamesToDelete = []
    for fieldName, fieldValue  in fields.items():
        if fieldName[-7:] == '|base64' \
           and type(fieldValue) in [types.StringType, types.UnicodeType]:
            fieldNamesToDelete.append(fieldName)
            name = fieldName[:-7]
            fields[name] = base64.decodestring(fieldValue)
    for fieldName, fieldValue  in fields.items():
        if fieldName[-6:] == '_field':
            if type(fieldValue) in [types.StringType, types.UnicodeType]:
                raise Exception('Wrong field value for %s = "%s"' % (
                    fieldName, fieldValue))
            else:
                fileContent = fieldValue.file.read()
                if fileContent:
                    fieldNamesToDelete.append(fieldName)
                    name = fieldName[:-6]
                    fields[name] = fileContent
                    fields[name + 'FileName'] = fieldValue.filename
                    fields[name + 'Type'] = fieldValue.type

    for fieldName in fieldNamesToDelete:
        del fields[fieldName]
    args.update(fields)

    # Handle optional session.
    canUseCookie = 1
    sessionToken = None
    sessionTokenInCookie = 0
    sessionTokenInUrl = 0
    if args.has_key('sessionToken'):
        sessionToken = args['sessionToken']
        sessionTokenInUrl = 1
    if cookieContent.has_key('sessionToken'):
        if sessionToken is None:
            sessionTokenInCookie = 1
            sessionToken = cookieContent['sessionToken']
        elif sessionToken == cookieContent['sessionToken']:
            sessionTokenInCookie = 1
    session = None
    if sessionToken is not None:
        try:
            session = getProxyForServerRole('sessions').getSession(
                sessionToken)
        except faults.UnknownSessionToken:
            if sessionTokenInUrl:
                uri = cleanUpUnparsedUri(['sessionToken'], 'http')
                return redirect(uri)
            else:
                sessionToken = None
                sessionTokenInCookie = 0
        if sessionToken is not None and not sessionTokenInCookie:
            # The sessionToken is valid but is not stored in the cookie.
            # So, don't try to use the cookie.
            canUseCookie = 0
    context.setVar('canUseCookie', canUseCookie)
    context.setVar('session', session)
    context.setVar('sessionToken', sessionToken)
    context.setVar('sessionTokenInCookie', sessionTokenInCookie)
    userId = ''
    if session and session.has_key('userId'):
        userId = session['userId']
    context.setVar('userId', userId)
    userToken = ''
    if session and session.has_key('userToken'):
        userToken = session['userToken']
    context.setVar('userToken', userToken)

    if args.has_key('keywords'):
        del args['keywords']
        if session and session.has_key('keywords'):
            for name, value in session['keywords'].items():
                if not args.has_key(name):
                    if type(value) in [types.StringType, types.UnicodeType] \
                       and value:
                        args[name] = base64.decodestring(value)
                    else:
                        args[name] = value

    if session:
        for name, value in session.items():
            if name[:6] == 'field_' and args.has_key(name[6:] + '|session'):
                nameBase = name[6:]
                del args[nameBase + '|session']
                if not args.has_key(nameBase):
                    if type(value) in [types.StringType, types.UnicodeType] \
                       and value:
                        args[nameBase] = base64.decodestring(value)
                    else:
                        args[nameBase] = value

    # Handle preferences
    if userToken:
        preferences = getWebForServerRole('preferences').getPreference()
    else:
        preferences = None
    context.setVar('preferences', preferences)

    # Handle languages.
    lang = None
    languages = ['C']
    if session and session.has_key('lang'):
        lang = session['lang']
        languages = [lang]
    elif preferences is not None and preferences.language \
       and preferences.language != 'None':
        languages = [preferences.language]
    elif req.headers_in.has_key('Accept-Language'):
        try:
            languages = req.headers_in['Accept-Language']
            languages = languages.split(',')
            languages = [x.strip()[:2] for x in languages]
            if len(languages) == 0:
                languages = ['C']
        except:
            return HTTP_BAD_REQUEST
    elif virtualHost is not None:
        languages = [virtualHost.language]
    if lang:
        lang == 'en' and 'C' or lang
    context.setVar('language', lang)
    languages = [
        language == 'en' and 'C' or language
        for language in languages]
    readLanguages = [
        language == 'C' and 'en' or language
        for language in languages]
    context.setVar('readLanguages', readLanguages)

    try:
        locale.setlocale(locale.LC_COLLATE, (languages[0], None))
    except locale.Error:
        locale.setlocale(locale.LC_COLLATE, 'C')

    try:
        translation = gettext.translation(
            '%s-web' % glasnost.applicationName, localePath,
            languages = languages)
    except IOError:
        translation = gettext.NullTranslations()
    __builtin__.__dict__['_'] = translation.gettext

    # Handle web cache.
    if sessionToken is None and not args.has_key('keywords'):
        reqCache = RequestCache(req, languages[0])
        if getConfig(configSection, 'CacheFiles', default='false') == 'false':
            reqCache.caching = -1
            modTime = reqCache.checkDepends()
        else:
            req.headers_out['Expires'] = reqCache.getExpires()
            modTime = None
        if reqCache.isPageCached():
#            and \
#                not (req.headers_in.has_key('Cache-Control') and  \
#                     req.headers_in['Cache-Control'] == 'max-age=0') and \
#                not (req.headers_in.has_key('Cache-Control') and \
#                     req.headers_in['Cache-Control'] == 'no-cache') and \
#                not (req.headers_in.has_key('Pragma') and \
#                     req.headers_in['Pragma'] == 'no-cache'):
            cachedPage, contentType = reqCache.getCachePage()
            setHttpCookie()
            return outputStaticFile(req, cachedPage, modTime, contentType)
    elif getConfig('Misc', 'UseBufferedRequests', default = 'false') == 'true':
        reqCache = BufferedRequest(req)
        reqCache.caching = 0
        reqCache.depends = []
    else:
        # The system caching is not activated when a session is open.
        reqCache = req
        reqCache.caching = 0
        reqCache.depends = []
        reqCache.buffered = 0
    context.setVar('req', reqCache)


    # Handle the httpLocalPath in URL and call appropriate function.
    knownRoles = getRegisteredRoles(dispatcherId)
    context.setVar('knownRoles', knownRoles)
    context.setVar('templateFileName', 'template.html')
    context.setVar('templatePrefix', '')
    result = parseHttpPath([x for x in httpLocalPath.split('/') if x])
    if result is not None:
        return result
    return HTTP_NOT_FOUND


def outputStaticFile(req, data, lastModTime = None, mimeType = None):
    t1 = context.getVar('t1')
    if lastModTime:
        req.headers_out['Last-Modified'] = time.strftime(
            '%a, %d %b %Y %H:%M:%S %Z', lastModTime)
    req.headers_out['Content-Length'] = '%d' % len(data)
    # TODO: could also output Content-MD5
    if lastModTime and req.headers_in.has_key('If-Modified-Since'):
        # we don't want to use bandwith if the file was not modified
        try:
            t = time.strptime(req.headers_in['If-Modified-Since'][:25],
                                '%a, %d %b %Y %H:%M:%S')
            if lastModTime <= t:
                open('/tmp/webbench', 'a+').write(
                    '%f  %s (static, not mod)\n' % (
                    time.time() - t1, req.unparsed_uri))
                return HTTP_NOT_MODIFIED
        except KeyError:
            pass
    if mimeType:
        req.content_type = mimeType
    req.send_http_header()
    if req.method != 'HEAD':
        req.write(data)
    open('/tmp/webbench', 'a+').write('%f  %s (static)\n' % (
        time.time() - t1, req.unparsed_uri))
    return OK


def parseHttpPath(remaining):
    context.push()
    try:
        result = parseHttpPathWorkspace(remaining)
        if result is not None:
            return result
        result = parseHttpPathAlias(remaining)
        if result is not None:
            return result
        result = parseHttpPathServerRole(remaining)
        if result is not None:
            return result
        result = parseHttpPathTalFile(remaining)
        if result is not None:
            return result
        result = parseHttpPathModule(remaining)
        if result is not None:
            return result
        return None
    finally:
        context.pull()


def parseHttpPathAction(remaining):
    context.push()
    try:
        if not remaining:
            action = None
        else:
            action = remaining[0]
            remaining = remaining[1:]
        changeTemplate = 0
        if action is not None and '-' in action:
            splittedAction = [x for x in action.split('-') if x]
            if len(splittedAction) >= 2:
                changeTemplate = 1
                template = '-'.join(splittedAction[:-1])
                action = splittedAction[-1]
        function = getAppropriateFunction(
            action,
            context.getVar('serverRole'),
            context.getVar('objectId'))
        if function is None:
            return None
        if changeTemplate:
            context.setVar(
                'templateFileName', 'template-%s.html' % template)
            context.setVar('templatePrefix', '%s-' % template)
        context.setVar('function', function)

        result = parseHttpPathArguments(remaining)
        if result is not None:
            return result
        return None
    finally:
        context.pull()


def parseHttpPathAlias(remaining):
    context.push()
    try:
        assert context.getVar('objectId') is None
        assert context.getVar('serverRole') is None
        if not remaining:
            return None
        if not 'pagenames' in context.getVar('knownRoles'):
            return None
        objectId = getWebForServerRole('pageNames').getIdByName(remaining[0])
        if not objectId:
            return None
        context.setVar('objectId', objectId)
        context.setVar(
            'serverRole', commonTools.extractRole(objectId))
        remaining = remaining[1:]

        result = parseHttpPathAction(remaining)
        if result is not None:
            return result
        return None
    finally:
        context.pull()


def parseHttpPathArguments(remaining):
    context.push()
    try:
        if remaining:
            context.setVar('positionalArguments', remaining)
            remaining = []
        return callFunction()
    finally:
        context.pull()


def parseHttpPathId(remaining):
    context.push()
    try:
        assert context.getVar('objectId') is None
        assert context.getVar('serverRole') is not None
        if not remaining:
            return None
        if remaining[0] == 'glasnost':
            if len(remaining) < 3:
                return None
            objectId = 'glasnost://%s/%s/%s' % (
                remaining[1],
                context.getVar('serverRole'),
                remaining[2])
            remaining = remaining[3:]
        else:
            objectId = '%s/%s/%s' % (
                context.getVar('dispatcherId'),
                context.getVar('serverRole'),
                remaining[0])
            remaining = remaining[1:]
        context.setVar('objectId', objectId)
        result = parseHttpPathAction(remaining)
        if result is not None:
            return result
        return None
    finally:
        context.pull()


def parseHttpPathModule(remaining):
    context.push()
    try:
        if remaining:
            moduleName = remaining[0]
            if moduleName.endswith('.py'):
                moduleName = moduleName[:-len('.py')]
            if len(remaining) >= 2:
                functionName = remaining[1]
                if functionName[0] == '_':
                    return None
                remaining = remaining[2:]
            else:
                functionName = 'index'
                remaining = remaining[1:]
        else:
            moduleName = 'index'
            functionName = 'index'
        try:
            module = apache.import_module(
                moduleName,
                context.getVar('req')._req,
                [context.getVar('webDirectoryPath'),
                 getConfig('Misc', 'AlternRootPath', '')])
        except ImportError:
            if context.getVar('debug'):
                raise
            return None
        try:
            function = getattr(module, functionName)
        except AttributeError:
            if context.getVar('debug'):
                raise
            return None
        if type(function) != types.FunctionType:
            return None
        context.setVar('function', function)

        result = parseHttpPathArguments(remaining)
        if result is not None:
            return result
        return None
    finally:
        context.pull()


def parseHttpPathServerRole(remaining):
    context.push()
    try:
        assert context.getVar('serverRole') is None
        if not remaining:
            return None
        if not remaining[0] in context.getVar('knownRoles'):
            return None
        context.setVar('serverRole', remaining[0])
        remaining = remaining[1:]

        result = parseHttpPathAction(remaining)
        if result is not None:
            return result
        result = parseHttpPathId(remaining)
        if result is not None:
            return result
        return None
    finally:
        context.pull()


def parseHttpPathTalFile(remaining):
    context.push()
    try:
        if remaining:
            fileNameCore = remaining[0]
            if fileNameCore.endswith('.tal'):
                fileNameCore = fileNameCore[:-len('.tal')]
            elif fileNameCore.endswith('.xtal'):
                fileNameCore = fileNameCore[:-len('.xtal')]
            remaining = remaining[1:]
        else:
            fileNameCore = 'index'
        webDirectoryPath = context.getVar('webDirectoryPath')
        talFilePathCore = os.path.join(webDirectoryPath, fileNameCore)
        talFilePath = '%s.tal' % talFilePathCore
        xtalFilePath = '%s.xtal' % talFilePathCore
        if os.path.exists(talFilePath):
            function = processTALFile
            remaining.insert(0, talFilePath)
        elif os.path.exists(xtalFilePath):
            function = processXTALFile
            remaining.insert(0, xtalFilePath)
        else:
            alternRootPath = getConfig('Misc', 'AlternRootPath')
            if alternRootPath:
                talFilePathCore = os.path.join(alternRootPath, fileNameCore)
                talFilePath = '%s.tal' % talFilePathCore
                xtalFilePath = '%s.xtal' % talFilePathCore
                if os.path.exists(talFilePath):
                    function = processTALFile
                    remaining.insert(0, talFilePath)
                elif os.path.exists(xtalFilePath):
                    function = processXTALFile
                    remaining.insert(0, talFilePath)
                else:
                    return None
            else:
                return None
        context.setVar('function', function)

        result = parseHttpPathArguments(remaining)
        if result is not None:
            return result
        return None
    finally:
        context.pull()


def parseHttpPathWorkspace(remaining):
    context.push()
    try:
        assert context.getVar('objectId') is None
        assert context.getVar('serverRole') is None
        if not remaining:
            return None
        if not 'workspace' in context.getVar('knownRoles'):
            return None
        workspaceName = remaining[0]
        if not 0: # test of workgroup existance
            return None
        # alter dispatcher ?
        remaining = remaining[1:]
        return parseHttpPath(remaining)
    finally:
        context.pull()
