# Glasnost
# By: Odile Bnassy <obenassy@entrouvert.com>
#     Thierry Dulieu <tdulieu@easter-eggs.com>
#     Frdric Pters <fpeters@theridion.com>
#     Benjamin Poussin <poussin@codelutin.com>
#     Emmanuel Raviart <eraviart@entrouvert.com>
#     Emmanuel Saracco <esaracco@easter-eggs.com>
#
# Copyright (C) 2000, 2001 Easter-eggs & Emmanuel Raviart
# Copyright (C) 2002 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Frdric Pters, Benjamin Poussin, Emmanuel Raviart,
#     Emmanuel Saracco & Thridion
# Copyright (C) 2003 Odile Bnassy, Code Lutin, Thierry Dulieu, Easter-eggs,
#     Entr'ouvert, Ouvaton, Frdric Pters, Benjamin Poussin, Rodolphe
#     Quideville, Emmanuel Raviart, Emmanuel Saracco, Thridion & Vecam
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


__doc__ = """Glasnost Index Web Page"""

__version__ = '$Revision: 1.22 $'[11:-2]


import glasnost.common.context as context
import glasnost.common.xhtmlgenerator as X

from glasnost.web.tools import *


def index():
    knownRoles = context.getVar('knownRoles')
    articlesWeb = getWebForServerRole('articles')
    rubricsWeb = getWebForServerRole('rubrics')
    isArticlesAdmin = articlesWeb.isAdmin()

    if 'shortnews' in knownRoles:
        brevesWeb = getWebForServerRole('breves')
        isBrevesAdmin = brevesWeb.isAdmin()
    else:
        brevesWeb = None
        isBrevesAdmin = 0

    if 'elections' in knownRoles:
        electionsWeb = getWebForServerRole('elections')
        isElectionsAdmin = electionsWeb.isAdmin()
    else:
        electionsWeb = None
        isElectionsAdmin = 0

    userId = context.getVar('userId', default = '')
    if userId:
        userSet = [userId]
    else:
        userSet = None

    layout = X.array()

    try:
        mainRubric = rubricsWeb.getMainObject()
    except (faults.MissingMainRubric, faults.UserAccessDenied):
        pass
    else:
        if mainRubric.contentId:
            web = getWeb(mainRubric.contentId)
            if web.canGetObject(mainRubric.contentId):
                layout += X.table(web.getObject(
                    mainRubric.contentId).getEmbeddedViewLayout())
                layout += X.br()

    requiredSlotNames = ['editionTime', 'format', 'title']
    displayedSlotNames = ['format', 'editionTime']
    if userSet:
        lastArticles = articlesWeb.getLastObjects(
            10, None, None, userSet, requiredSlotNames)
        layout += articlesWeb.getObjectsSectionLayout(
            lastArticles,
            _("""Your last articles"""),
            displayedSlotNames)

    lastArticles = articlesWeb.getLastObjects(
        10, None, userSet, None, requiredSlotNames)
    layout += articlesWeb.getObjectsSectionLayout(
        lastArticles,
        _("""The last articles"""),
        displayedSlotNames)

    if isArticlesAdmin:
        lastArticles = articlesWeb.getLastObjects(
            50, None, None, None, requiredSlotNames)
        layout += articlesWeb.getObjectsSectionLayout(
            lastArticles,
            _("""The last articles to administer"""),
            displayedSlotNames)

    requiredSlotNames = ['electionTitle', 'modificationTime', 'title']
    displayedSlotNames = ['modificationTime']

    if brevesWeb and brevesWeb.canGetObjects():
        lastBreves = brevesWeb.getLastObjects(
            10, None, userSet, None, requiredSlotNames)
        layout += brevesWeb.getObjectsSectionLayout(
            lastBreves,
            _("""The last short news"""),
            displayedSlotNames)

    if isBrevesAdmin:
        lastBreves = brevesWeb.getLastObjects(
            50, None, None, None, requiredSlotNames)
        layout += brevesWeb.getObjectsSectionLayout(
            lastBreves,
            _("""The last short news to administer"""),
            displayedSlotNames)

    requiredSlotNames = ['endTime', 'isAlwaysRunning', 'state', 'title']
    displayedSlotNames = ['state', 'isAlwaysRunning', 'endTime']

    if userSet and electionsWeb and electionsWeb.canGetObjects():
        lastElections = electionsWeb.getLastObjects(
            10, None, None, userSet, ['draft'], requiredSlotNames)
        layout += electionsWeb.getObjectsSectionLayout(
            lastElections,
            _("""Your elections being written"""),
            displayedSlotNames)

    if isElectionsAdmin:
        lastElections = electionsWeb.getLastObjects(
            10, None, None, None, ['proposed'], requiredSlotNames)
        layout += electionsWeb.getObjectsSectionLayout(
            lastElections,
            _("""The elections being submitted for evaluation"""),
            displayedSlotNames)

    if electionsWeb and electionsWeb.canGetObjects():
        lastElections = electionsWeb.getLastObjects(
            10, None, None, None, ['running'], requiredSlotNames)
        layout += electionsWeb.getObjectsSectionLayout(
            lastElections,
            _("""The elections in progress"""),
            displayedSlotNames)

        lastElections = electionsWeb.getLastObjects(
            10, None, None, None, ['closed'], requiredSlotNames)
        layout += electionsWeb.getObjectsSectionLayout(
            lastElections,
            _("""The last closed elections"""),
            displayedSlotNames)

    return writePageLayout(layout, _('Home'))

